//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/map-coordinates.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, Math,
  JScriptObjects, gmAPI, gmMap, gmBase, gmOverlaysInfoWindow, gmMapTypes,
  gmEvents;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FChicago: TLatLng;
    procedure InitMap(Sender: TObject);
    procedure MapZoomChanged(Sender: TObject);
    function CreateInfoWindowContent: String;
  public
    property Chicago: TLatLng read FChicago write FChicago;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------
//TMercatorProjection

const
  TileSize = 255;

type
  TMercatorProjection = class(TProjection)
  private
    FPixelOrigin: TPoint;
    FPixelsPerLonDegree: Double;
    FPixelsPerLonRadian: Double;
    function Bound(Value, OptMin, OptMax: Double): Double;
  public
    destructor Destroy; override;
    procedure AfterConstruction; override;
    function FromLatLngToPoint(LatLng: TLatLng; OptPoint: TPoint = nil): TPoint; override;
    function FromPointToLatLng(Point: TPoint; NoWrap: Boolean): TLatLng; override;
  end;

procedure TMercatorProjection.AfterConstruction;
begin
  inherited;
  FPixelOrigin:=gmAPI.New(TScript(Script).Google.Maps.Point(TileSize/2,TileSize/2));
  FPixelsPerLonDegree:=TileSize/360;
  FPixelsPerLonRadian:=TileSize/(2*Pi);
end;

destructor TMercatorProjection.Destroy;
begin
  FPixelOrigin.Free;
  inherited;
end;

function TMercatorProjection.Bound(Value, OptMin, OptMax: Double): Double;
begin
  Value:=Max(Value,OptMin);
  Value:=Min(Value,OptMax);
  Result:=Value;
end;

function TMercatorProjection.FromLatLngToPoint(LatLng: TLatLng; OptPoint: TPoint): TPoint;
var
  SinY: Double;
begin
  if not assigned(OptPoint)
    then OptPoint:=gmAPI.New(TScript(Script).Google.Maps.Point(0,0));
  OptPoint.X:=FPixelOrigin.X+LatLng.Lng*FPixelsPerLonDegree;
  //NOTE(appleton): Truncating to 0.9999 effectively limits latitude to
  //89.189.  This is about a third of a tile past the edge of the world
  //tile.
  SinY:=Bound(Sin(DegToRad(LatLng.Lat)),-0.9999,0.9999);
  OptPoint.Y:=FPixelOrigin.Y+0.5*Ln((1+SinY)/(1-SinY))*-FPixelsPerLonRadian;
  Result:=OptPoint;
end;

function TMercatorProjection.FromPointToLatLng(Point: TPoint; NoWrap: Boolean): TLatLng;
var
  Lat, Lng: Double;
  LatRadians: Double;
begin
  Lng:=(Point.X-FPixelOrigin.X)/FPixelsPerLonDegree;
  LatRadians:=(Point.Y-FPixelOrigin.Y)/-FPixelsPerLonRadian;
  Lat:=RadToDeg(2*ArcTan(Exp(LatRadians))-Pi/2);
  Result:=gmAPI.New(TScript(Script).Google.Maps.LatLng(Lat,Lng));
end;

//------------------------------------------------------------------------------
//TForm1

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  CoordInfoWindow: TInfoWindow;
  Map: TMap;
  MapOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    Chicago:=New(Google.Maps.LatLng(41.850033,-87.6500523));
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=0;
      Center:=Chicago;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    Map:=New(Google.Maps.Map(MapOptions));
    CoordInfoWindow:=New(Google.Maps.InfoWindow);
    CoordInfoWindow.SetContent(CreateInfoWindowContent);
    CoordInfoWindow.SetPosition(Chicago);
    CoordInfoWindow.Open(Map);
    Map.OnZoomChanged:=MapZoomChanged;
  end;
end;

function TForm1.CreateInfoWindowContent: String;
var
  NumTiles: Integer;
  Projection: TMercatorProjection;
  PixelCoordinate, TileCoordinate, WorldCoordinate: TPoint;
begin
  with Script do
  begin
    NumTiles:=1 shl Maps[0].GetZoom;
    Projection:=TMercatorProjection.Create;
    WorldCoordinate:=Projection.FromLatLngToPoint(Chicago);
    PixelCoordinate:=New(Google.Maps.Point(WorldCoordinate.X*NumTiles,
                                           WorldCoordinate.Y*NumTiles));
    TileCoordinate:=New(Google.Maps.Point(Floor(PixelCoordinate.X/TileSize),
                                          Floor(PixelCoordinate.Y/TileSize)));
    Result:=TJScriptArray.Create([
      'Chicago, IL',
      'LatLng: '+SysUtils.FloatToStr(Chicago.Lat)+', '+SysUtils.FloatToStr(Chicago.Lng),
      'World Coordinate: '+SysUtils.FloatToStr(WorldCoordinate.X)+', '+SysUtils.FloatToStr(WorldCoordinate.Y),
      'Pixel Coordinate: '+IntToStr(Floor(PixelCoordinate.X))+', '+IntToStr(Floor(PixelCoordinate.Y)),
      'Tile Coordinate: ' +SysUtils.FloatToStr(TileCoordinate.X)+', '+SysUtils.FloatToStr(TileCoordinate.Y)+
                                                                 ' at Zoom Level: '+IntToStr(Maps[0].GetZoom)
      ]).Join('<br>');
  end;
end;

procedure TForm1.MapZoomChanged(Sender: TObject);
begin
  with Script do
  begin
    InfoWindows[0].SetContent(CreateInfoWindowContent);
    InfoWindows[0].Open(Maps[0]);
  end;
end;

end.
