//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/map-projection-simple.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, Math, gmAPI,
  gmMap, gmBase, gmMapTypes, gmOverlaysMarker, gmEvents;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure InitMap(Sender: TObject);
    {$IFNDEF USE_ANONYMOUS_METHODS}
    procedure MapClick(Sender: TObject; Event: TMouseEvent);
    function GetTileURL(Sender: TObject; Coord: TPoint; Zoom: Integer): String;
    {$ENDIF}
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------
//TGallPetersProjection

const
  //Note: this value is exact as the map projects a full 360 degrees of longitude
  GallPetersRangeX = 800;
  //Note: this value is inexact as the map is cut off at ~ +/- 83 degrees.
  //However, the polar regions produce very little increase in Y range, so
  //we will use the tile size.
  GallPetersRangeY = 510;

type
  TGallPetersProjection = class(TProjection)
  private
    FWorldCoordinateLatRange: Double;
    FWorldCoordinatePerLonDegree: Double;
    FWorldOrigin: TPoint;
  public
    destructor Destroy; override;
    procedure AfterConstruction; override;
    function FromLatLngToPoint(LatLng: TLatLng; Point: TPoint): TPoint; override;
    function FromPointToLatLng(Point: TPoint; NoWrap: Boolean): TLatLng; override;
  end;

procedure TGallPetersProjection.AfterConstruction;
begin
  inherited;
  //Using the base map tile, denote the lat/lon of the equatorial origin.
  FWorldOrigin:=gmAPI.New(TScript(Script).Google.Maps.Point(GallPetersRangeX*400/800,
                                                            GallPetersRangeY/2));
  //This projection has equidistant meridians, so each longitude degree is a
  //linear mapping.
  FWorldCoordinatePerLonDegree:=GallPetersRangeX/360;
  //This constant merely reflects that latitudes vary from +90 to -90 degrees.
  FWorldCoordinateLatRange:=GallPetersRangeY/2;
end;

destructor TGallPetersProjection.Destroy;
begin
  FWorldOrigin.Free;
  inherited;
end;

function TGallPetersProjection.FromLatLngToPoint(LatLng: TLatLng; Point: TPoint): TPoint;
var
  X, Y, LatRadians: Double;
begin
  X:=FWorldOrigin.X+FWorldCoordinatePerLonDegree*LatLng.Lng;
  //Note that latitude is measured from the world coordinate origin at the top
  //left of the map.
  LatRadians:=DegToRad(LatLng.Lat);
  Y:=FWorldOrigin.Y-FWorldCoordinateLatRange*Sin(LatRadians);
  Result:=gmAPI.New(TScript(Script).Google.Maps.Point(X,Y));
end;

function TGallPetersProjection.FromPointToLatLng(Point: TPoint; NoWrap: Boolean): TLatLng;
var
  X, Y, Lat, Lng, LatRadians: Double;
begin
  X:=Point.X;
  Y:=Point.Y;
  if Y<0
    then Y:=0;
  if Y>=GallPetersRangeY
    then Y:=GallPetersRangeY;
  Lng:=(X-FWorldOrigin.X)/FWorldCoordinatePerLonDegree;
  LatRadians:=ArcSin((FWorldOrigin.Y-Y)/FWorldCoordinateLatRange);
  Lat:=RadToDeg(LatRadians);
  Result:=gmAPI.New(TScript(Script).Google.Maps.LatLng(Lat,Lng,NoWrap));
end;

//------------------------------------------------------------------------------
//TForm1

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
  procedure AddMarker(Map: TMap; Position: TLatLng; Title: String);
  var
    MarkerOptions: TMarkerOptions;
  begin
    MarkerOptions:=TMarkerOptions.Create;
    MarkerOptions.Map:=Map;
    MarkerOptions.Position:=Position;
    MarkerOptions.Title:=Title;
    New(Script.Google.Maps.Marker(MarkerOptions));
  end;
var
  Chicago, Anchorage, Mexico, Equator, London,
  Johannesburg, Kinshasa, Sydney: TLatLng;
  MapTypeOptions: TImageMapTypeOptions;
  GallPetersMap: TMap;
  GallPetersMapType: TImageMapType;
  MapOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    Chicago:=New(Google.Maps.LatLng(41.850033,-87.6500523));
    Anchorage:=New(Google.Maps.LatLng(61.2180556,-149.9002778));
    Mexico:=New(Google.Maps.LatLng(19.4270499,-99.1275711));
    Equator:=New(Google.Maps.LatLng(0,0));
    London:=New(Google.Maps.LatLng(51.5001524,-0.1262362));
    Johannesburg:=New(Google.Maps.LatLng(-26.201452,28.045488));
    Kinshasa:=New(Google.Maps.LatLng(-4.325,15.322222));
    Sydney:=New(Google.Maps.LatLng( -33.867139,151.207114));
    MapTypeOptions:=TImageMapTypeOptions.Create;
    with MapTypeOptions do
    begin
      {$IFDEF USE_ANONYMOUS_METHODS}
      GetTileUrl:=
        function(Sender: TObject; Coord: TPoint; Zoom: Integer): String
        var
          X: Integer;
          NumTiles: Integer;
        begin
          Result:='';
          NumTiles:=1 shl Zoom;
          //Don't wrap tiles vertically.
          if (Coord.Y<0) or (Coord.Y>=NumTiles)
            then Exit;
          //Wrap tiles horizontally.
          X:=((Round(Coord.X)  mod NumTiles)+NumTiles) mod NumTiles;
          //For simplicity, we use a tileset consisting of 1 tile at zoom
          //level 0 and 4 tiles at zoom level 1. Note that we set the base URL
          //to a relative directory.
          Result:='http://code.google.com/apis/maps/documentation/javascript/examples/images/';
          Result:=Result+'gall-peters_'+IntToStr(Zoom)+'_'+IntToStr(X)+'_'+IntToStr(Round(Coord.Y))+'.png';
        end;
      {$ELSE}
      GetTileUrl:=Self.GetTileURL;
      {$ENDIF}
      TileSize:=New(Google.Maps.Size(800,512));
      IsPng:=true;
      MinZoom:=0;
      MaxZoom:=1;
      Name:='Gall-Peters';
    end;
    GallPetersMapType:=New(Google.Maps.ImageMapType(MapTypeOptions));
    GallPetersMapType.Projection:=TGallPetersProjection.Create;
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=0;
      Center:=New(Google.Maps.LatLng(0,0));
      MapTypeControlOptions.MapTypeIDs.Add([Google.Maps.MapTypeID.Roadmap,'gallPetersMap'])
    end;
    GallPetersMap:=New(Google.Maps.Map(MapOptions));
    GallPetersMap.MapTypes.{$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}('gallPetersMap',GallPetersMapType);
    GallPetersMap.SetMapTypeID('gallPetersMap');
    {$IFDEF USE_ANONYMOUS_METHODS}
    GallPetersMap.OnClick:=
      procedure(Sender: TObject; Event: TMouseEvent)
      begin
        ShowMessage('Point.X.Y: '+Event.LatLng.ToUrlValue(2));
      end;
    {$ELSE}
    GallPetersMap.OnClick:=MapClick;
    {$ENDIF}
    AddMarker(GallPetersMap,Chicago,'Chicago');
    AddMarker(GallPetersMap,Anchorage,'Anchorage');
    AddMarker(GallPetersMap,Mexico,'Mexico');
    AddMarker(GallPetersMap,Equator,'Equator');
    AddMarker(GallPetersMap,London,'London');
    AddMarker(GallPetersMap,Johannesburg,'Johannesburg');
    AddMarker(GallPetersMap,Kinshasa,'Kinshasa');
    AddMarker(GallPetersMap,Sydney,'Sydney');
  end;
end;

{$IFNDEF USE_ANONYMOUS_METHODS}
function TForm1.GetTileURL(Sender: TObject; Coord: TPoint; Zoom: Integer): String;
var
  X: Integer;
  NumTiles: Integer;
begin
  Result:='';
  NumTiles:=1 shl Zoom;
  //Don't wrap tiles vertically.
  if (Coord.Y<0) or (Coord.Y>=NumTiles)
    then Exit;
  //Wrap tiles horizontally.
  X:=((Round(Coord.X)  mod NumTiles)+NumTiles) mod NumTiles;
  //For simplicity, we use a tileset consisting of 1 tile at zoom
  //level 0 and 4 tiles at zoom level 1. Note that we set the base URL
  //to a relative directory.
  Result:='http://code.google.com/apis/maps/documentation/javascript/examples/images/';
  Result:=Result+'gall-peters_'+IntToStr(Zoom)+'_'+IntToStr(X)+'_'+IntToStr(Round(Coord.Y))+'.png';
end;

procedure TForm1.MapClick(Sender: TObject; Event: TMouseEvent);
begin
  ShowMessage('Point.X.Y: '+Event.LatLng.ToUrlValue(2));
end;
{$ENDIF}

end.
