//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/control-custom-state.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, HTMLObjects,
  JScriptObjects, gmAPI, gmMap, gmBase;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure InitMap(Sender: TObject);
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------
//THomeControl

//The HomeControl adds a control to the map that returns the user to the
//control's defined home.
 type
  THomeControl = class(TJScriptObject)
  private
    FHome: TLatLng;
    {$IFNDEF USE_ANONYMOUS_METHODS}
    FMap: TMap;
    procedure GoHome;
    procedure SetHome;
    {$ENDIF}
  public
    constructor Create(ControlDiv: TNode; Map: TMap; Home: TLatLng);
    property Home: TLatLng read FHome write FHome;
  end;

//The HomeControl adds a control to the map that simply returns the user to
//Chicago. This constructor takes the control DIV as an argument.
constructor THomeControl.Create(ControlDiv: TNode; Map: TMap; Home: TLatLng);
var
  GoHomeUI, SetHomeUI, GoHomeText, SetHomeText: TNode;
begin
  inherited Create;
  with TScript(Script) do
  begin
    FHome:=Home;
    //Set CSS styles for the DIV containing the control
    //Setting padding to 5 px will offset the control from the edge of the map
    ControlDiv.Style.Padding:='5px';
    //Set CSS for the control border
    GoHomeUI:=Document.CreateDivNode;
    with GoHomeUI do
    begin
      with Style do
      begin
        BackgroundColor:=clWhite;
        BorderStyle:='solid';
        BorderWidth:='2px';
        Cursor:='pointer';
        TextAlign:='center';
      end;
      Title:='Click to set the map to Home';
    end;
    ControlDiv.AppendChild(GoHomeUI);
    //Set CSS for the control interior
    GoHomeText:=Document.CreateDivNode;
    with GoHomeText do
    begin
      with Style do
      begin
        FontFamily:='Arial,sans-serif';
        FontSize:='12px';
        PaddingLeft:='4px';
        PaddingRight:='4px';
      end;
      InnerHTML:='<b>Home</b>';
    end;
    GoHomeUI.AppendChild(GoHomeText);
    //Set CSS for the setHome control border
    SetHomeUI:=Document.CreateDivNode;
    with SetHomeUI do
    begin
      with Style do
      begin
        BackgroundColor:=clWhite;
        BorderStyle:='solid';
        BorderWidth:='2px';
        Cursor:='pointer';
        TextAlign:='center';
      end;
      Title:='Click to set Home to the current center';
    end;
    ControlDiv.AppendChild(SetHomeUI);
    //Set CSS for the control interior
    SetHomeText:=Document.CreateDivNode;
    with SetHomeText do
    begin
      with Style do
      begin
        FontFamily:='Arial,sans-serif';
        FontSize:='12px';
        PaddingLeft:='4px';
        PaddingRight:='4px';
      end;
      InnerHTML:='<b>Set Home</b>';
    end;
    SetHomeUI.AppendChild(SetHomeText);
    //Setup the click event listener for Home:
    //Simply set the map to the control's current home property.
    {$IFDEF USE_ANONYMOUS_METHODS}
    Google.Maps.Event.AddDomListener(GoHomeUI,'click',NewFunction(
      procedure
      begin
        Map.SetCenter(FHome);
      end));
    //Setup the click event listener for Set Home:
    //Set the control's home to the current Map center.
    Google.Maps.Event.AddDomListener(SetHomeUI,'click',NewFunction(
      procedure
      begin
        FHome:=Map.GetCenter;
      end));
    {$ELSE}
    FMap:=Map;
    Google.Maps.Event.AddDomListener(GoHomeUI,'click',NewFunction(GoHome));
    Google.Maps.Event.AddDomListener(SetHomeUI,'click',NewFunction(SetHome));
    {$ENDIF}
  end;
end;

{$IFNDEF USE_ANONYMOUS_METHODS}
procedure THomeControl.GoHome;
begin
  FMap.SetCenter(FHome);
end;

procedure THomeControl.SetHome;
begin
  FHome:=FMap.GetCenter;
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TForm1

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Chicago: TLatLng;
  HomeControlDiv: TNode;
  Map: TMap;
  MyOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    Chicago:=New(Google.Maps.LatLng(41.850033,-87.6500523));
    MyOptions:=TMapOptions.Create;
    with MyOptions do
    begin
      Zoom:=12;
      Center:=Chicago;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    Map:=New(Google.Maps.Map(MyOptions));
    //Create the DIV to hold the control and call the HomeControl() constructor
    //passing in this DIV.
    HomeControlDiv:=Document.CreateDivNode;
    THomeControl.Create(HomeControlDiv,Map,Chicago);
    Map.Controls[Google.Maps.ControlPosition.TopRight].Push(HomeControlDiv);
  end;
end;

end.
