//http://code.google.com/apis/maps/documentation/javascript/geometry.html
//http://code.google.com/apis/maps/documentation/javascript/geometry.html#Navigation
//http://code.google.com/apis/maps/documentation/javascript/examples/geometry-headings.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, ExtCtrls,
  gmAPI, gmMap, gmBase, gmOverlaysPolylinePolygon, gmEvents, gmMVC;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    Panel1: TPanel;
    Label1: TLabel;
    Heading: TLabel;
    Label2: TLabel;
    Origin: TLabel;
    Label3: TLabel;
    Destination: TLabel;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FPoly, FGeodesic: TPolyline;
    FClickcount: Integer;
    procedure AddOrigin(Event: TMouseEvent);
    procedure AddDestination(Event: TMouseEvent);
    procedure AdjustHeading;
    procedure ClearPaths;
    procedure InitMap(Sender: TObject);
    procedure MapClick(Sender: TObject; MouseEvent: TMouseEvent);
  public
    property Clickcount: Integer read FClickcount write FClickcount;
    property Geodesic: TPolyline read FGeodesic write FGeodesic;
    property Poly: TPolyline read FPoly write FPoly;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap,[libGeometry]);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Map: TMap;
  MapOptions: TMapOptions;
  Atlantic: TLatLng;
  PolyOptions, GeodesicOptions: TPolylineOptions;
begin
  Heading.Caption:='';
  Origin.Caption:='';
  Destination.Caption:='';
  with TScript(Sender) do
  begin
    Atlantic:=New(Google.Maps.LatLng(34,-40.605));
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=4;
      Center:=Atlantic;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    Map:=New(Google.Maps.Map(MapOptions));
    Map.OnClick:=MapClick;
    PolyOptions:=TPolylineOptions.Create;
    with PolyOptions do
    begin
      StrokeColor:=$FF0000;
      StrokeOpacity:=1.0;
      StrokeWeight:=3;
    end;
    Poly:=New(Google.Maps.Polyline(PolyOptions));
    Poly.SetMap(Map);
    GeodesicOptions:=TPolylineOptions.Create;
    with GeodesicOptions do
    begin
      StrokeColor:=$CC0099;
      StrokeOpacity:=1.0;
      StrokeWeight:=3;
      Geodesic:=true;
    end;
    Geodesic:=New(Google.Maps.Polyline(GeodesicOptions));
    Geodesic.SetMap(Map);
  end;
end;

procedure TForm1.ClearPaths;
var
  Path, GPath: TLatLngMVCArray;
begin
  Path:=Poly.GetPath;
  while Path.GetLength>0 do
    Path.Pop;
  GPath:=Geodesic.GetPath;
  while GPath.GetLength>0 do
    GPath.Pop;
end;

procedure TForm1.AddOrigin(Event: TMouseEvent);
var
  Path, GPath: TLatLngMVCArray;
begin
  ClearPaths;
  Path:=Poly.GetPath;
  Path.Push(Event.LatLng);
  GPath:=Geodesic.GetPath;
  GPath.Push(Event.LatLng);
end;

procedure TForm1.AddDestination(Event: TMouseEvent);
var
  Path, GPath: TLatLngMVCArray;
begin
  Path:=Poly.GetPath;
  Path.Push(Event.LatLng);
  GPath:=Geodesic.GetPath;
  GPath.Push(Event.LatLng);
  AdjustHeading;
  Clickcount:=0;
end;

procedure TForm1.AdjustHeading;
var
  Path: TLatLngMVCArray;
  PathSize: Integer;
  Heading: Double;
begin
  Path:=Poly.GetPath;
  PathSize:=Path.GetLength;
  Heading:=Script.Google.Maps.Geometry.Spherical.ComputeHeading(Path.GetAt(0),Path.GetAt(PathSize-1));
  Self.Heading.Caption:=FloatToStr(Heading,'.');
  Origin.Caption:=FloatToStr(Path.GetAt(0).Lat,'.')+', '+FloatToStr(Path.GetAt(0).Lng,'.');
  Destination.Caption:=FloatToStr(Path.GetAt(PathSize-1).Lat,'.')+', '+FloatToStr(Path.GetAt(PathSize-1).Lng,'.');
end;

procedure TForm1.MapClick(Sender: TObject; MouseEvent: TMouseEvent);
begin
  Clickcount:=Clickcount+1;
  if Clickcount=1
    then AddOrigin(MouseEvent);
  if Clickcount=2
    then AddDestination(MouseEvent);
end;

end.
