//http://code.google.com/apis/maps/documentation/javascript/places.html#place_searches
//http://code.google.com/apis/maps/documentation/javascript/examples/place-search.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, gmAPI, gmMap,
  gmBase, gmOverlaysMarker, gmOverlaysInfoWindow, gmEvents, gmLibrariesPlaces,
  HTMLObjects;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure InitMap(Sender: TObject);
    {$IFNDEF USE_ANONYMOUS_METHODS}
    procedure Callback(Sender: TObject; SearchResult: TPlaceResultArray; Status: String);
    procedure CreateMarker(Place: TPlaceResult);
    procedure MarkerClick(Sender: TObject; Event: TEvent);
    {$ENDIF}
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap,[libPlaces]);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  MapOptions: TMapOptions;
  Pyrmont: TLatLng;
  Request: TPlaceSearchRequest;
  Service: TPlacesService;
begin
  with TScript(Sender) do
  begin
    Pyrmont:=New(Google.Maps.LatLng(-33.8665433,151.1956316));
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=15;
      Center:=Pyrmont;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    New(Google.Maps.Map(MapOptions));
    Request:=TPlaceSearchRequest.Create;
    with Request do
    begin
      Location:=Pyrmont;
      Radius:=500;
      Types.Add('store');
    end;
    Service:=New(Google.Maps.Places.PlacesService(Maps[0]));
    New(Google.Maps.InfoWindow);
  end;
  {$IFDEF USE_ANONYMOUS_METHODS}
  Service.Search(Request,
    procedure(Sender: TObject; SearchResult: TPlaceResultArray; Status: String)
      function CreateEvent(Marker: TMarker; PlaceName: String): TMapEventEvent;
      begin
        Result:=
          procedure(Sender: TObject; Event: TEvent)
          begin
            with Script do
            begin
              Infowindows[0].SetContent(PlaceName);
              Infowindows[0].Open(Maps[0],Marker);
            end;
          end;
      end;
    var
      n: Integer;
      Marker: TMarker;
      MarkerOptions: TMarkerOptions;
    begin
      with Script do
      begin
        if Status=Google.Maps.Places.PlacesServiceStatus.OK then
        begin
          for n:=0 to SearchResult.Count-1 do
          begin
            MarkerOptions:=TMarkerOptions.Create;
            with MarkerOptions do
            begin
              Map:=Maps[0];
              Position:=SearchResult[n].Geometry.Location;
            end;
            Marker:=New(Google.Maps.Marker(MarkerOptions));
            Marker.OnClick:=CreateEvent(Marker,SearchResult[n].Name);
          end;
        end;
      end;
    end);
  {$ELSE}
  Service.Search(Request,Callback);
  {$ENDIF}
end;

{$IFNDEF USE_ANONYMOUS_METHODS}
procedure TForm1.CreateMarker(Place: TPlaceResult);
var
  Marker: TMarker;
  MarkerOptions: TMarkerOptions;
begin
  with Script do
  begin
    MarkerOptions:=TMarkerOptions.Create;
    with MarkerOptions do
    begin
      Map:=Maps[0];
      Position:=Place.Geometry.Location;
    end;
    Marker:=New(Google.Maps.Marker(MarkerOptions));
    Marker.OnClick:=MarkerClick;
    Marker.Properties['PlaceName']:=Place.Name;          //additional property
  end;
end;

procedure TForm1.Callback(Sender: TObject; SearchResult: TPlaceResultArray; Status: String);
var
  n: Integer;
begin
  if Status=Script.Google.Maps.Places.PlacesServiceStatus.OK then
  begin
    for n:=0 to SearchResult.Count-1 do
      CreateMarker(SearchResult[n]);
  end;
end;

procedure TForm1.MarkerClick(Sender: TObject; Event: TEvent);
var
  Marker: TMarker;
  Text: String;
begin
  with Script do
  begin
    Marker:=TMarker(Sender);
    Text:=Marker.Properties['PlaceName'];
    Infowindows[0].SetContent(Text);
    Infowindows[0].Open(Maps[0],Marker);
  end;
end;
{$ENDIF}

end.
