//http://code.google.com/apis/maps/documentation/javascript/places.html#places_autocomplete
//http://code.google.com/apis/maps/documentation/javascript/examples/places-autocomplete.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, ExtCtrls,
  gmAPI, gmMap, gmBase, gmOverlaysMarker, gmOverlaysInfoWindow,
  gmLibrariesPlaces, HTMLObjects;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FAutocomplete: TAutocomplete;
    FEdit: THTMLEdit;
    FInfoWindow: TInfoWindow;
    FMap: TMap;
    FMarker: TMarker;
    procedure InitMap(Sender: TObject);
    procedure PlaceChanged(Sender: TObject);
  public
    property Autocomplete: TAutocomplete read FAutocomplete write FAutocomplete;
    property Edit: THTMLEdit read FEdit write FEdit;
    property InfoWindow: TInfoWindow read FInfoWindow write FInfoWindow;
    property Map: TMap read FMap write FMap;
    property Marker: TMarker read FMarker write FMarker;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Edit.Free;
  inherited;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap,[libPlaces]);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  MapOptions: TMapOptions;
  MarkerOptions: TMarkerOptions;
  Pyrmont: TLatLng;
begin
  with TScript(Sender) do
  begin
    Pyrmont:=New(Google.Maps.LatLng(-33.8665433,151.1956316));
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=15;
      Center:=Pyrmont;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    Map:= New(Google.Maps.Map(MapOptions));
    Infowindow:=New(Google.Maps.InfoWindow);
    MarkerOptions:=TMarkerOptions.Create;
    MarkerOptions.Map:=Map;
    Marker:=New(Google.Maps.Marker(MarkerOptions));
    Edit.Free;
    Edit:=THTMLEdit.Create(Map,Google.Maps.ControlPosition.TopLeft);
    Edit.Top:=5;
    Edit.Width:=50;
    Autocomplete:=New(Google.Maps.Places.Autocomplete(Edit));
    Autocomplete.OnPlaceChanged:=PlaceChanged;
    //Autocomplete.BindTo('bounds',Map);
  end;
end;

procedure TForm1.PlaceChanged(Sender: TObject);
var
  Place: TPlaceResult;
  Image: TMarkerImage;
  Text: String;
begin
  with Script do
  begin
    InfoWindow.Close;
    Place:=Autocomplete.GetPlace;
    if assigned(Place.Geometry.Viewport)
      then Map.FitBounds(Place.Geometry.Viewport)
      else begin
        Map.Center:=Place.Geometry.Location;
        Map.Zoom:=17;
      end;
    Image:=New(Google.Maps.MarkerImage(Place.Icon,
                                       New(Google.Maps.Size(71,71)),
                                       New(Google.Maps.Point(0,0)),
                                       New(Google.Maps.Point(17,34)),
                                       New(Google.Maps.Size(35,35))));
    Marker.IconImage:=Image;
    Marker.Position:=Place.Geometry.Location;
    if assigned(Place.AddressComponents) then
    begin
      Text:='';
      if assigned(Place.AddressComponents[0])
        then Text:=Place.AddressComponents[0].ShortName;
      if assigned(Place.AddressComponents[1]) then
      begin
        if Length(Text)>0
          then Text:=Text+' ';
        Text:=Text+Place.AddressComponents[1].ShortName
      end;
      if assigned(Place.AddressComponents[2]) then
      begin
        if Length(Text)>0
          then Text:=Text+' ';
        Text:=Text+Place.AddressComponents[2].ShortName;
      end;
      InfoWindow.Content:='<div><strong>'+Place.Name+'</strong><br>'+Text;
      InfoWindow.Open(Map,Marker);
    end;
  end;
end;

end.
