unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, Math, OleCtrls, SHDocVw, HTMLObjects,
  gmAPI, gmBase, gmMap, gmMapTypes, gmOverlaysBase;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FScaleBar: TDivNode;
    procedure InitMap(Sender: TObject);
    procedure MapTypeIDChanged(Sender: TObject);
    function GetTileUrl(Sender: TObject; Point: TPoint; Zoom: Integer): String;
    function CreateScaleBar(Script: TScript; Limits: array of Integer; Steps: array of Integer; Units: String): TDivNode;
  public
    property ScaleBar: TDivNode read FScaleBar write FScaleBar;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Map: TMap;
  MapOptions: TMapOptions;
  MarsMapTypeOptions: TImageMapTypeOptions;
  MarsElevationMapType, MarsVisibleMapType, MarsInfraredMapType: TImageMapType;
begin
  with TScript(Sender) do
  begin
    //Hhenkarte mit Falschfarbendarstellung:
    MarsMapTypeOptions:=TImageMapTypeOptions.Create;            //Options-Objekt anlegen
    with MarsMapTypeOptions do
    begin
      GetTileUrl:=self.GetTileUrl;                              //Callback-Funktion zur Lieferung des Kartenausschnittes
      TileSize:=Google.Maps.Size(256,256);                      //Gre der einzelnen Bildkacheln
      IsPng:=false;                                             //kein PNG-Bild
      MinZoom:=0;
      MaxZoom:=9;
      Radius:=3397200;                                          //Marsradius
      Name:='Elevation';                                        //Layername
    end;
    MarsElevationMapType:=Google.Maps.ImageMapType(MarsMapTypeOptions);   //Layer-Objekt anlegen
    //Schwarz-Wei-Karte im sichtbaren Spektrum:
    MarsMapTypeOptions:=TImageMapTypeOptions.Create;            //Options-Objekt anlegen
    with MarsMapTypeOptions do
    begin
      GetTileUrl:=self.GetTileUrl;                              //Callback-Funktion zur Lieferung des Kartenausschnittes
      TileSize:=Google.Maps.Size(256,256);                      //Gre der einzelnen Bildkacheln
      IsPng:=false;                                             //kein PNG-Bild
      MinZoom:=0;
      MaxZoom:=9;
      Radius:=3397200;                                          //Marsradius
      Name:='Visible';                                          //Layername
    end;
    MarsVisibleMapType:=Google.Maps.ImageMapType(MarsMapTypeOptions);     //Layer-Objekt anlegen
    //Schwarz-Wei-Karte im infraroten Spektrum:
    MarsMapTypeOptions:=TImageMapTypeOptions.Create;            //Options-Objekt anlegen
    with MarsMapTypeOptions do
    begin
      GetTileUrl:=Self.GetTileUrl;                              //Callback-Funktion zur Lieferung des Kartenausschnittes
      TileSize:=Google.Maps.Size(256,256);                      //Gre der einzelnen Bildkacheln
      IsPng:=false;                                             //kein PNG-Bild
      MinZoom:=0;
      MaxZoom:=9;
      Radius:=3397200;                                          //Marsradius
      Name:='Infrared';                                         //Layername
    end;
    MarsInfraredMapType:=Google.Maps.ImageMapType(MarsMapTypeOptions);    //Layer-Objekt anlegen
    //Karte anlegen:
    MapOptions:=TMapOptions.Create;                             //Options-Objekt anlegen
    with MapOptions do
    begin
      Zoom:=3;                                                  //Zoom
      Center:=Google.Maps.LatLng(0,-90);                        //Kartenzentrum
      MapTypeControlOptions.MapTypeIDs.Add(['Elevation','Visible','Infrared']);
      StreetViewControl:=false;                                 //auf dem Mars spazieren gehen - irgendwann einmal... ;-)
    end;
    Map:=Google.Maps.Map(MapOptions);                           //Karten-Objekt anlegen
    Map.MapTypes.{$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}('Elevation',MarsElevationMapType);
    Map.MapTypes.{$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}('Visible',MarsVisibleMapType);
    Map.MapTypes.{$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}('Infrared',MarsInfraredMapType);
    Map.SetMapTypeID('Elevation');                              //Layer anzeigen
    Map.OnMapTypeIDChanged:=MapTypeIDChanged;
    //Hhenmastab erstellen:
    ScaleBar:=CreateScaleBar(Script,[-9,21],[-9,-6,-3,0,3,6,9,12,15,18,21],'km');
    //Die Eingliederung ber das Controls-Array ist ungnstig, da die Ausrichtung
    //links-unten das Element RECHTS neben dem Google-Logo anordnen wrde:
    //Map.Controls[Google.Maps.ControlPosition.Left].Push(FScaleBar);
    //Besser ist eine direkte Eingliederung ber den Container, der die Karte
    //enthlt. Dabei ist ein hherer ZIndex wichtig, damit keine Verdeckung durch
    //die Karte oder ihre Steuerelemente erfolgt:
    ScaleBar.Style.ZIndex:=1000;
    Map.Container.AppendChild(ScaleBar);
  end;
end;

function TForm1.GetTileUrl(Sender: TObject; Point: TPoint; Zoom: Integer): String;
const
  TileURL = 'http://mw1.google.com/mw-planetary/mars/';
var
  z: Integer;
  x, y: Double;
  TileRange: Integer;
begin
  Result:='';
  x:=Point.x;
  y:=Point.y;
  TileRange:=1 shl Zoom;                                   //Zoom 0: 1 Teil, 1: 2 Teile, 2: 4 Teile, 3: 8 Teile usw.
  if (y<0) or (y>=TileRange)
    then Exit;                                             //keine Wiederholung entlag der y-Achse (verikal)
  if (x<0) or (x>=TileRange)                               //Wiederholung entlang der x-Achse (horizontal)
    then x:=(round(x) mod TileRange+TileRange) mod TileRange;
  Result:=TileURL+LowerCase(Script.Maps[0].MapTypeID)+'/t';//Der Server unterscheidet zwischen Gro- und Kleinschreibung (Linux?)!!!
  for z:=0 to Zoom-1 do
  begin
    TileRange:=TileRange shr 1;
    if y<TileRange then
    begin
      if x<TileRange
        then Result:=Result+'q'
        else begin
          Result:=Result+'r';
          x:=x-TileRange;
        end;
    end else
    begin
      if x<TileRange then
      begin
        Result:=Result+'t';
        y:=y-TileRange;
      end else
      begin
        Result:=Result+'s';
        x:=x-TileRange;
        y:=y-TileRange;
      end;
    end;
  end;
  Result:=Result+'.jpg';
end;

procedure TForm1.MapTypeIDChanged(Sender: TObject);
begin
  if TMap(Sender).MapTypeID='Elevation'
    then ScaleBar.Style.Visibility:='visible'              //Skale anzeigen
    else ScaleBar.Style.Visibility:='hidden';              //Skale aussblenden
end;

function TForm1.CreateScaleBar(Script: TScript; Limits, Steps: array of Integer; Units: String): TDivNode;
const
  ScalebarImageSrc = 'http://www.google.com/mars/images/scalebar.png';
  ScalebarHeight = 150;
var
  n: Integer;
  Container, Legend, StepDiv, LabelDiv, Tic, Caption: TDivNode;
  Image: TNode;
  Bottom, Top, Delta, Offset: Integer;
begin
  Container:=Script.Document.CreateDivNode;
  with Container.Style do
  begin
    Position:='absolute';
    Bottom:='50px';
    Left:='10px';
    Height:=IntToStr(ScalebarHeight)+'px';
    WhiteSpace:='nowrap';
    FontSize:='10px';
    FontFamily:='Arial, sans serif';
  end;
  Legend:=Script.Document.CreateDivNode;
  Bottom:=Limits[0];
  Top:=Limits[1];
  Delta:=Top-Bottom;
  for n:=0 to Length(Steps)-1 do
  begin
    Offset:=round(ScalebarHeight*((Top-Steps[n])/Delta));
    StepDiv:=Script.Document.CreateDivNode;
    with StepDiv.Style do
    begin
      Position:='absolute';
      Top:=IntToStr(Offset)+'px';
      Height:='1px';
    end;
    Tic:=Script.Document.CreateDivNode;
    with Tic.Style do
    begin
      Position:='absolute';
      Height:='1px';
      Width:='20px';
      Border:='#000 solid 0px';
      BorderTopWidth:='1px';
    end;
    LabelDiv:=Script.Document.CreateDivNode;
    with LabelDiv.Style do
    begin
      Position:='absolute';
      Top:='-0.6em';
      Left:='25px';
      LineHeight:='1.0em';
    end;
    LabelDiv.AppendChild(Script.Document.CreateTextNode(IntToStr(Steps[n])+' '+Units));
    StepDiv.AppendChild(Tic);
    StepDiv.AppendChild(LabelDiv);
    Legend.AppendChild(StepDiv);
  end;
  Caption:=Script.Document.CreateDivNode;
  with Caption.Style do
  begin
    Position:='absolute';
    Top:='-2.0em';
    Left:='0px';
  end;
  //Caption.AppendChild(Script.Document.CreateTextNode('Elevation'));
  Image:=Script.Document.CreateElement('img');
  Image.AsVariant.Src:=ScalebarImageSrc;
  Container.AppendChild(Image);
  Container.AppendChild(Legend);
  Container.AppendChild(Caption);
  Result:=Container;
end;

end.
