//http://code.google.com/apis/maps/documentation/javascript/examples/maptype-base.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, HTMLObjects,
  BrowserTools, gmAPI, gmMap, gmMapTypes, gmBase, gmEvents;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure InitMap(Sender: TObject);
    {$IFNDEF USE_ANONYMOUS_METHODS}
    procedure MapTypeIDChanged(Sender: TObject);
    {$ENDIF}
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

type
  TCoordMapType = class(TMapType)
  public
    constructor Create;
    procedure ReleaseTile(Tile: TNode); override;
    function GetTile(TileCoord: TPoint; Zoom: Integer; OwnerDocument: TDocument): TNode; override;
  end;

constructor TCoordMapType.Create;
begin
  inherited Create;
  TileSize:=TScript(Script).Google.Maps.Size(256,256);
  MaxZoom:=19;
  Name:='Tile #s';
  Alt:='Tile Coordinate Map Type';
end;

function TCoordMapType.GetTile(TileCoord: TPoint; Zoom: Integer; OwnerDocument: TDocument): TNode;
begin
  Result:=OwnerDocument.CreateDivNode;
  with Result do
  begin
    InnerHTML:=TileCoord.ToLocaleString;
    with  Style do
    begin
      Width:=IntToStr(round(TileSize.Width))+'px';
      Height:=IntToStr(round(TileSize.Height))+'px';
      FontSize:='10';
      BorderStyle:='solid';
      BorderWidth:='1px';
      BorderColor:=WebColorStrToColor('#AAAAAA');
    end;
  end;
end;

procedure TCoordMapType.ReleaseTile(Tile: TNode);
begin
end;

//------------------------------------------------------------------------------
//TForm1

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Chicago: TLatLng;
  Map: TMap;
  MapOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    Chicago:=New(Google.Maps.LatLng(41.850033,-87.6500523));
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=10;
      Center:=Chicago;
      MapTypeID:='coordinate';
      with MapTypeControlOptions do
      begin
        MapTypeIDs.Add(['coordinate',Google.Maps.MapTypeID.Roadmap]);
        Style:=Google.Maps.MapTypeControlStyle.DropdownMenu;
      end;
      StreetViewControl:=false;
    end;
    Map:=New(Google.Maps.Map(MapOptions));
    {$IFDEF USE_ANONYMOUS_METHODS}
    Map.OnMapTypeIDChanged:=
      procedure(Sender: TObject)
      var
        ShowStreetViewControl: Boolean;
        MapOptions: TMapOptions;
      begin
        ShowStreetViewControl:=Map.GetMapTypeID<>'coordinate';
        MapOptions:=TMapOptions.Create;
        MapOptions.StreetViewControl:=ShowStreetViewControl;
        Map.SetOptions(MapOptions);
      end;
    {$ELSE}
    Map.OnMapTypeIDChanged:=MapTypeIDChanged;
    {$ENDIF}
    //Now attach the coordinate map type to the map's registry
    Map.MapTypes.{$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}('coordinate',TCoordMapType.Create);
  end;
end;

{$IFNDEF USE_ANONYMOUS_METHODS}
procedure TForm1.MapTypeIDChanged(Sender: TObject);
var
  Map: TMap;
  ShowStreetViewControl: Boolean;
  MapOptions: TMapOptions;
begin
  Map:=Script.Maps[0];
  ShowStreetViewControl:=Map.GetMapTypeID<>'coordinate';
  MapOptions:=TMapOptions.Create;
  MapOptions.StreetViewControl:=ShowStreetViewControl;
  Map.SetOptions(MapOptions);
end;
{$ENDIF}

end.
