//http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/overlays.html
//http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/overlays.html#ImageMapTypes
//http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/examples/maptype-image.html

unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, OleCtrls, SHDocVw, Math, gmBase, gmAPI,
  gmMap, gmMapTypes, gmOverlaysBase;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    procedure InitMap(Sender: TObject);
    function GetTileUrl(Sender: TObject; Point: TPoint; Zoom: Integer): String;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do                    //Skript-Objekt erstellen
      LoadAPIAsync(InitMap);                               //Google Maps API laden und Karte erstellen
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Map: TMap;
  MapOptions: TMapOptions;
  MoonMapTypeOptions: TImageMapTypeOptions;
  MoonMapType: TImageMapType;
begin
  with TScript(Sender) do
  begin
    //Layer anlegen:
    MoonMapTypeOptions:=TImageMapTypeOptions.Create;            //Options-Objekt anlegen
    MoonMapTypeOptions.GetTileUrl:=GetTileUrl;                  //Callback-Funktion zur Lieferung des Kartenausschnittes
    MoonMapTypeOptions.TileSize:=Google.Maps.Size(256,256);     //Gre der einzelnen Bildkacheln
    MoonMapTypeOptions.IsPng:=false;                            //kein PNG-Bild
    MoonMapTypeOptions.MaxZoom:=9;
    MoonMapTypeOptions.MinZoom:=0;
    MoonMapTypeOptions.Radius:=1738000;                         //Mondradius
    MoonMapTypeOptions.Name:='Moon';                            //Layername
    MoonMapType:=Google.Maps.ImageMapType(MoonMapTypeOptions);  //Layer-Objekt anlegen
    //Karte anlegen:
    MapOptions:=TMapOptions.Create;                             //Options-Objekt anlegen
    with MapOptions do
    begin
      Zoom:=1;                                                  //Zoom
      Center:=Google.Maps.LatLng(0,0);                          //Kartenzentrum
      MapTypeControlOptions.MapTypeIDs.Add('moon');             //Layer der Karte zuweisen
      StreetViewControl:=false;                                 //auf dem Mond spazieren gehen!? ;-)
    end;
    Map:=Google.Maps.Map(MapOptions);                           //Karten-Objekt anlegen
    Map.MapTypes.{$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}('moon',MoonMapType);
    Map.SetMapTypeID('moon');                                   //Layer anzeigen
  end;
end;

function TForm1.GetTileUrl(Sender: TObject; Point: TPoint; Zoom: Integer): String;
type
  TDoublePoint = record
    x: Double;
    y: Double;
  end;
  function DoublePoint(x, y: Double): TDoublePoint;
  begin
    Result.x:=x;
    Result.y:=y;
  end;
  //Normalizes the coords that tiles repeat across the x axis (horizontally)
  //like the standard Google map tiles.
  function GetNormalizedCoord(Coord: TPoint; Zoom: Integer): TDoublePoint;
  var
    x, y: Double;
    TileRange: Integer;
  begin
    x:=Coord.x;
    y:=Coord.y;
    //tile range in one direction range is dependent on zoom level
    //0 = 1 tile, 1 = 2 tiles, 2 = 4 tiles, 3 = 8 tiles, etc
    TileRange:=1 shl Zoom;
    //don't repeat across y-axis (vertically)
    if (y<0) or (y>=TileRange) then
    begin
      Result:=DoublePoint(-1,-1);
      Exit;
    end;
    //repeat across x-axis
    if (x<0) or (x>=TileRange)
      then x:=(round(x) mod TileRange + TileRange) mod TileRange;
    Result:=DoublePoint(x,y);
  end;
var
  NormalizedCoord: TDoublePoint;
  Bound: Extended;
begin
  Result:='';
  NormalizedCoord:=GetNormalizedCoord(Point,Zoom);
  if (NormalizedCoord.x=-1) and (NormalizedCoord.y=-1)
    then Exit;
  Bound:=Power(2,Zoom);
  Result:='http://mw1.google.com/mw-planetary/lunar/lunarmaps_v1/clem_bw'+
          '/'+IntToStr(Zoom)+'/'+FloatToStr(NormalizedCoord.x,'.')+'/'+
          FloatToStr(Bound-NormalizedCoord.y-1,'.')+'.jpg';
end;
(*
<!DOCTYPE html>
<html>
<head>
  <title>Google Maps JavaScript API v3 Example: Image MapTypes</title>
  <link href="http://code.google.com/apis/maps/documentation/javascript/examples/standard.css" rel="stylesheet" type="text/css" />
  <script type="text/javascript" src="http://maps.google.com/maps/api/js?sensor=false"></script>
  <script>
    var moonMapType = new google.maps.ImageMapType({
      getTileUrl: function(coord, zoom) {
        var normalizedCoord = getNormalizedCoord(coord, zoom);
        if (!normalizedCoord) {
          return null;
        }
        var bound = Math.pow(2, zoom);
        return "http://mw1.google.com/mw-planetary/lunar/lunarmaps_v1/clem_bw" +
            "/" + zoom + "/" + normalizedCoord.x + "/" +
            (bound - normalizedCoord.y - 1) + ".jpg";
      },
      tileSize: new google.maps.Size(256, 256),
      isPng: false,
      maxZoom: 9,
      minZoom: 0,
      radius: 1738000,
      name: "Moon"
    });

    // Normalizes the coords that tiles repeat across the x axis (horizontally)
    // like the standard Google map tiles.
    function getNormalizedCoord(coord, zoom) {
      var y = coord.y;
      var x = coord.x;

      // tile range in one direction range is dependent on zoom level
      // 0 = 1 tile, 1 = 2 tiles, 2 = 4 tiles, 3 = 8 tiles, etc
      var tileRange = 1 << zoom;

      // don't repeat across y-axis (vertically)
      if (y < 0 || y >= tileRange) {
        return null;
      }

      // repeat across x-axis
      if (x < 0 || x >= tileRange) {
        x = (x % tileRange + tileRange) % tileRange;
      }

      return {
        x: x,
        y: y
      };
    }

    function initialize() {
      var myLatlng = new google.maps.LatLng(0, 0);
      var myOptions = {
        center: myLatlng,
        zoom: 1,
        mapTypeControlOptions: {
          mapTypeIds: ["moon"]
        }
      };

      var map = new google.maps.Map(document.getElementById("map_canvas"), myOptions);
      map.mapTypes.set("moon", moonMapType);
      map.setMapTypeId("moon");
    }
  </script>
</head>
<body onload="initialize()">
  <div id="map_canvas" style="width: 640px; height: 480px;"></div>
</body>
</html>
*)
end.
