//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/overlay-hideshow.html
unit Unit1;

interface  

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, SHDocVw, OleCtrls, StdCtrls, ExtCtrls,
  ComCtrls, HTMLObjects, gmAPI, gmMap, gmBase, gmOverlaysBase;

type
  TUSGSOverlay = class(TOverlayView)
  private
    FBounds: TLatLngBounds;
    FDivNode: TDivNode;
    FImage: String;
    FImageNode: TNode;
    FMap: TMap;
  public
    constructor Create(Bounds: TLatLngBounds; Image: String; Map: TMap); reintroduce;
    procedure Draw; override;
    procedure Hide;
    procedure OnAdd; override;
    procedure OnRemove; override;
    procedure Show;
    procedure Toggle;
    procedure ToggleDOM;
  end;

  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    Panel1: TPanel;
    Toggle: TButton;
    ToggleDOM: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ToggleClick(Sender: TObject);
    procedure ToggleDOMClick(Sender: TObject);
  private
    FMap: TMap;
    FOverlay: TUSGSOverlay;
    FScript: TScript;
    procedure InitMap(Sender: TObject);
  public
    property Script: TScript read FScript write FScript;
    property Map: TMap read FMap write FMap;
    property Overlay: TUSGSOverlay read FOverlay write FOverlay;
  end;

  TDoublePoint = record
    x: Double;
    y: Double;
    {$IFDEF DELPHI2006_UP}
    class operator Implicit(a: TPoint): TDoublePoint;
    {$ENDIF}
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------
//TDoublePoint

{$IFDEF DELPHI2006_UP}
class operator TDoublePoint.Implicit(a: TPoint): TDoublePoint;
begin
  if assigned(a) then
  begin
    Result.x:=a.x;
    Result.y:=a.y;
  end else
  begin
    Result.x:=0;
    Result.y:=0;
  end;
end;
{$ENDIF}

//------------------------------------------------------------------------------
//USGSOverlay

constructor TUSGSOverlay.Create(Bounds: TLatLngBounds; Image: String; Map: TMap);
begin
  inherited Create;
  FBounds:=Bounds;
  FImage:=Image;
  FMap:=Map;
  SetMap(Map);                                             //Overlay der Karte zuordnen
end;

procedure TUSGSOverlay.OnAdd;
var
  MapPanes: TMapPanes;
begin
  FDivNode:=Script.Document.CreateDivNode;
  with FDivNode.Style do
  begin
    BorderStyle:='none';
    BorderWidth:='0px';
    Position:='absolute';
  end;
  FImageNode:=Script.Document.CreateElement('img');
  with FImageNode do
  begin
    with Style do
    begin
      Width:='100%';
      Height:='100%';
//      Filter:='Alpha(opacity=50)';
    end;
    AsVariant.Src:=FImage;
  end;
  FDivNode.AppendChild(FImageNode);
  MapPanes:=GetPanes;
  MapPanes.OverlayImage.AppendChild(FDivNode);
end;

procedure TUSGSOverlay.Draw;
var
  OverlayProjection: TMapCanvasProjection;
  //gmBase.TPoint bietet sich hier NICHT an, da bei jeder Abfrage von
  //TMapCanvasProjection.FromLatLngToDivPixel() das selbe interne
  //TMapCanvasProjection-Delphi-Objekt fr TPoint zurckgegeben wird. Damit
  //wrden die Variablen sw und ne die selben Werte liefern. Deshalb werden die
  //Koordinaten in einem eigenen Typ gespeichert, was auch dank berladener
  //Operatoren ohne Probleme mglich ist:
  SW, NE: TDoublePoint {TPoint};
begin
  OverlayProjection:=GetProjection;
  {$IFDEF DELPHI2006_UP}
  //einfach mit Klassen-Operator:
  SW:=OverlayProjection.FromLatLngToDivPixel(FBounds.GetSouthWest);
  NE:=OverlayProjection.FromLatLngToDivPixel(FBounds.GetNorthEast);
  {$ELSE}
  //herkmmlich mit einzelnen Zuweisungen (das, was der Klassen-Operator bernimmt):
  with OverlayProjection.FromLatLngToDivPixel(FBounds.GetSouthWest) do
  begin
    SW.x:=x;
    SW.y:=y;
  end;
  with OverlayProjection.FromLatLngToDivPixel(FBounds.GetNorthEast) do
  begin
    NE.x:=x;
    NE.y:=y;
  end;
  {$ENDIF}
  with FDivNode.Style do
  begin
    Left:=FloatToStr(SW.x,'.')+'px';
    Top:=FloatToStr(NE.y,'.')+'px';
    Width:=FloatToStr(NE.x-SW.x,'.')+'px';
    Height:=FloatToStr(SW.y-NE.y,'.')+'px';
  end;
end;

procedure TUSGSOverlay.Hide;
begin
   FDivNode.Style.Visibility:='hidden';
end;

procedure TUSGSOverlay.OnRemove;
begin
  Variant(FDivNode.ParentNode).RemoveChild(FDivNode.Disp);
  FreeAndNil(FDivNode);
  FreeAndNil(FImageNode);
end;

procedure TUSGSOverlay.Show;
begin
   FDivNode.Style.Visibility:='visible';
end;

procedure TUSGSOverlay.Toggle;
begin
  if assigned(Map) then
  begin
    if FDivNode.Style.Visibility='hidden'
      then Show
      else Hide;
  end;
end;

procedure TUSGSOverlay.ToggleDOM;
begin
  if assigned(Map)
    then Map:=nil
    else Map:=FMap;
end;

//------------------------------------------------------------------------------
//TForm1

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do                    //Skript-Objekt erstellen
      LoadAPIAsync(InitMap);
end;

const
  SrcImage = 'http://code.google.com/intl/de/apis/maps/'+
             'documentation/javascript/examples/images/talkeetna.png';

procedure TForm1.InitMap(Sender: TObject);
var
  SWBound, NEBound: TLatLng;
  Bounds: TLatLngBounds;
  MapOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Center:=Google.Maps.LatLng(62.343,-150.14);
      Zoom:=10;
      MapTypeID:=Google.Maps.MapTypeId.Satellite;
    end;
    Map:=Google.Maps.Map(MapOptions);
    SWBound:=Google.Maps.LatLng(62.281819,-150.287132);
    NEBound:=Google.Maps.LatLng(62.400471,-150.005608);
    Bounds:=Google.Maps.LatLngBounds(SWBound,NEBound);
    Overlay:=TUSGSOverlay.Create(Bounds,SrcImage,Map);
  end;
end;

procedure TForm1.ToggleClick(Sender: TObject);
begin
  Overlay.Toggle;
end;

procedure TForm1.ToggleDOMClick(Sender: TObject);
begin
  Overlay.ToggleDOM;
end;

end.
