//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/directions-complex.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, ExtCtrls,
  JScriptObjects, gmAPI, gmMap, gmBase, gmServicesDirections,
  gmServicesDirectionsRenderer, gmOverlaysInfoWindow, gmOverlaysMarker,
  gmEvents, HTMLObjects;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    StartPos: TComboBox;
    EndPos: TComboBox;
    Splitter1: TSplitter;
    Panel2: TPanel;
    WebBrowser2: TWebBrowser;
    Panel3: TPanel;
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FWarnings: TNode;
    {$IFNDEF USE_ANONYMOUS_METHODS}
    procedure AttachInstructionText(Marker: TMarker; Text: String);
    procedure CalcRoute(Sender: TObject);
    procedure DirectionsCallback(Sender: TObject; Response: TDirectionsResult; Status: String);
    procedure MarkerClick(Sender: TObject; Event: TEvent);
    procedure ShowSteps(DirectionResult: TDirectionsResult);
    {$ELSE}
    procedure AttachInstructionText(Map: TMap; InfoWindow: TInfoWindow; Marker: TMarker; Text: String);
    {$ENDIF}
    procedure InitMap(Sender: TObject);
  public
    property Warnings: TNode read FWarnings write FWarnings;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

type
  TLocation = record
    Value: String;
    Name: String;
  end;

const
  StartLocations: array[0..4] of TLocation = (
    (Value: 'penn station, new york, ny'                 ; Name: 'Penn Station'),
    (Value: 'grand central station, new york, ny'        ; Name: 'Grand Central Station'),
    (Value: '625 8th Avenue, New York, NY, 10018'        ; Name: 'Port Authority Bus Terminal'),
    (Value: 'staten island ferry terminal, new york, ny' ; Name: 'Staten Island Ferry Terminal'),
    (Value: '101 E 125th Street, New York, NY'           ; Name: 'Harlem - 125th St Station'));

  EndLocations: array[0..5] of TLocation = (
    (Value: '260 Broadway New York NY 10007'             ; Name: 'City Hall'),
    (Value: 'W 49th St & 5th Ave, New York, NY 10020'    ; Name: 'Rockefeller Center'),
    (Value: 'moma, New York, NY'                         ; Name: 'MOMA'),
    (Value: '350 5th Ave, New York, NY, 10118'           ; Name: 'Empire State Building'),
    (Value: '253 West 125th Street, New York, NY'        ; Name: 'Apollo Theater'),
    (Value: '1 Wall St, New York, NY'                    ; Name: 'Wall St'));

procedure TForm1.FormCreate(Sender: TObject);
  procedure FillComboBox(ComboBox: TComboBox; Locations: array of TLocation);
  var
    n: Integer;
  begin
    with ComboBox, Items do
    begin
      Clear;
      for n:=Low(Locations) to High(Locations) do
      Add(Locations[n].Name);
      ItemIndex:=0;
    end;
  end;
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
  FillComboBox(StartPos,StartLocations);
  FillComboBox(EndPos,EndLocations);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  {$IFDEF USE_ANONYMOUS_METHODS}
  ReleaseEvent(EndPos.OnClick);
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if not assigned(Script(WebBrowser2)) then
    with TCustomScript.Create(WebBrowser2) do
    begin
      LoadEmptyPage;
      with Document.Body.Style do
      begin
        BackgroundColor:=Self.Color;
        Margin:='4px';
      end;
      Warnings:=Document.CreateDivNode;
      Document.Body.AppendChild(Warnings);
    end;
  if not assigned(Script(WebBrowser1)) then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  {$IFDEF USE_ANONYMOUS_METHODS}
  DirectionsDisplay: TDirectionsRenderer;
  DirectionsService: TDirectionsService;
  StepDisplay: TInfoWindow;
  {$ENDIF}
  Manhattan: TLatLng;
  Map: TMap;
  MyOptions: TMapOptions;
  RendererOptions: TDirectionsRendererOptions;
  Script: TScript;
begin
  Script:=TScript(Sender);
  with Script do
  begin
    Warnings.InnerHTML:='';
    //Instantiate a directions service.
    {$IFDEF USE_ANONYMOUS_METHODS}DirectionsService:={$ENDIF}
    New(Google.Maps.DirectionsService);
    //Create a map and center it on Manhattan.
    Manhattan:=New(Google.Maps.LatLng(40.7711329,-73.9741874));
    MyOptions:=TMapOptions.Create(Script);
    with MyOptions do
    begin
      Zoom:=13;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
      Center:=Manhattan;
    end;
    Map:=New(Google.Maps.Map(MyOptions));
    //Create a renderer for directions and bind it to the map.
    RendererOptions:=TDirectionsRendererOptions.Create(Script);
    RendererOptions.Map:=Map;
    {$IFDEF USE_ANONYMOUS_METHODS}DirectionsDisplay:={$ENDIF}
    New(Google.Maps.DirectionsRenderer(RendererOptions));
    //Instantiate an info window to hold step text.
    {$IFDEF USE_ANONYMOUS_METHODS}StepDisplay:={$ENDIF}
    New(Google.Maps.InfoWindow);
    {$IFDEF USE_ANONYMOUS_METHODS}
    ReleaseEvent(EndPos.OnClick);
    EndPos.OnClick:=MakeEvent(
      procedure(Sender: TObject)
      var
        n: Integer;
        Request: TDirectionsRequest;
      begin
        //First, remove any existing markers from the map.
        for n:=0 to Markers.Count-1 do
          Markers[n].Map:=nil;
        //Now, clear the array itself.
        Markers.Clear;
        //Retrieve the start and end locations and create
        //a DirectionsRequest using Walking directions.
        Request:=TDirectionsRequest.Create(Script);
        with Request do
        begin
          OriginAsString:=StartLocations[StartPos.ItemIndex].Value;
          DestinationAsString:=EndLocations[EndPos.ItemIndex].Value;
          TravelMode:=Google.Maps.DirectionsTravelMode.Walking;
        end;
        //Route the directions and pass the response to a function to create
        //markers for each step.
        DirectionsService.Route(Request,
          procedure(Sender: TObject; Response: TDirectionsResult; Status: String)
            {Will not work with Delphi XE (Internal Error):
            procedure AttachInstructionText(Marker: TMarker; Text: String);
            begin
              Marker.OnClick:=MakeEvent(
                procedure(Sender: TObject; Event: TEvent)
                begin
                  StepDisplay.SetContent(Text);
                  StepDisplay.Open(Map,Marker);
                end);
            end;}
          var
            n: Integer;
            Marker: TMarker;
            MarkerOptions: TMarkerOptions;
            MyRoute: TDirectionsLeg;
          begin
            if Status=Google.Maps.DirectionsStatus.OK then
            begin
              Warnings.InnerHTML:='<b>'+Response.Routes[0].Warnings.ToString+'</b>';
              DirectionsDisplay.SetDirections(Response);
              //For each step, place a marker, and add the text to the marker's
              //info window. Also attach the marker to an array so we can keep
              //track of it and remove it when calculating new routes.
              MyRoute:=Response.Routes[0].Legs[0];
              for n:=0 to MyRoute.Steps.Count-1 do
              begin
                MarkerOptions:=TMarkerOptions.Create(Script);
                MarkerOptions.Position:=MyRoute.Steps[n].StartLocation;
                MarkerOptions.Map:=Map;
                Marker:=New(Google.Maps.Marker(MarkerOptions));
                //Will not work with Delphi XE (Internal Error):
                //AttachInstructionText(Marker,MyRoute.Steps[n].Instructions);
                AttachInstructionText(Map,StepDisplay,Marker,MyRoute.Steps[n].Instructions);
              end;
            end;
          end);
      end);
    {$ELSE}
    EndPos.OnClick:=CalcRoute;
    {$ENDIF}
  end;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TForm1.AttachInstructionText(Map: TMap; InfoWindow: TInfoWindow;
                                       Marker: TMarker; Text: String);
begin
  Marker.OnClick:=
    procedure(Sender: TObject; Event: TEvent)
    begin
      InfoWindow.SetContent(Text);
      InfoWindow.Open(Map,Marker);
    end;
end;
{$ELSE}
procedure TForm1.CalcRoute(Sender: TObject);
var
  n: Integer;
  Request: TDirectionsRequest;
begin
  with Script(WebBrowser1) do
  begin
    //First, remove any existing markers from the map.
    for n:=0 to Markers.Count-1 do
      Markers[n].Map:=nil;
    // Now, clear the array itself.
    Markers.Clear;
    //Retrieve the start and end locations and create
    //a DirectionsRequest using Walking directions.
    Request:=TDirectionsRequest.Create(Script(WebBrowser1));
    with Request do
    begin
      OriginAsString:=StartLocations[StartPos.ItemIndex].Value;
      DestinationAsString:=EndLocations[EndPos.ItemIndex].Value;
      TravelMode:=Google.Maps.DirectionsTravelMode.Walking;
    end;
    //Route the directions and pass the response to a
    //function to create markers for each step.
    DirectionsServices[0].Route(Request,DirectionsCallback);
  end;
end;

procedure TForm1.DirectionsCallback(Sender: TObject; Response: TDirectionsResult; Status: String);
begin
  with Script(WebBrowser1) do
  begin
    if Status=Google.Maps.DirectionsStatus.OK then
    begin
      Warnings.InnerHTML:='<b>'+Response.Routes[0].Warnings.ToString+'</b>';
      DirectionsRenderer[0].SetDirections(Response);
      ShowSteps(Response);
    end;
  end;
end;

procedure TForm1.ShowSteps(DirectionResult: TDirectionsResult);
var
  n: Integer;
  Marker: TMarker;
  MarkerOptions: TMarkerOptions;
  MyRoute: TDirectionsLeg;
  Script: TScript;
begin
  Script:=gmAPI.Script(WebBrowser1);
  with Script do
  begin
    //For each step, place a marker, and add the text to the marker's info window.
    //Also attach the marker to an array so we can keep track of it and remove it
    //when calculating new routes.
    MyRoute:=DirectionResult.Routes[0].Legs[0];
    for n:=0 to MyRoute.Steps.Count-1 do
    begin
      MarkerOptions:=TMarkerOptions.Create(Script);
      with MarkerOptions do
      begin
        Position:=MyRoute.Steps[n].StartLocation;
        Map:=Maps[0];
      end;
      Marker:=New(Google.Maps.Marker(MarkerOptions));
      AttachInstructionText(Marker,MyRoute.Steps[n].Instructions);
    end;
  end;
end;

procedure TForm1.AttachInstructionText(Marker: TMarker; Text: String);
begin
  Marker.OnClick:=MarkerClick;
  Marker.Properties['InstructionText']:=Text;
end;

procedure TForm1.MarkerClick(Sender: TObject; Event: TEvent);
var
  Text: String;
begin
  with Script(WebBrowser1) do
  begin
    //Open an info window when the marker is clicked on, containing the text of
    //the step.
    TMarker(Sender).Properties['InstructionText'];
    InfoWindows[0].SetContent(Text);
    InfoWindows[0].Open(Maps[0],TMarker(Sender));
  end;
end;
{$ENDIF}

end.
