//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/directions-draggable.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, HTMLObjects,
  gmAPI, gmMap, gmBase, gmServicesDirections, gmServicesDirectionsRenderer,
  gmEvents;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    {$IFNDEF USE_ANONYMOUS_METHODS}
    FTotalDiv: TNode;
    procedure DirectionsCallback(Sender: TObject; Response: TDirectionsResult; Status: String);
    procedure DirectionsChanged(Sender: TObject);
    {$ENDIF}
    procedure InitMap(Sender: TObject);
  public
    {$IFNDEF USE_ANONYMOUS_METHODS}
    property TotalDiv: TNode read FTotalDiv write FTotalDiv;
    {$ENDIF}
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
    begin
      BackgroundColor:=TColor($E4E4E4);
      LoadAPIAsync(InitMap);
    end;
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Australia: TLatLng;
  DirectionsDisplay: TDirectionsRenderer;
  DirectionsService: TDirectionsService;
  Map: TMap;
  MapAlignDiv, MapDiv, DirectionsPanel: TNode;
  {$IFDEF USE_ANONYMOUS_METHODS}
  TotalDiv: TNode;
  {$ENDIF}
  MyOptions: TMapOptions;
  RendererOptions: TDirectionsRendererOptions;
  Request: TDirectionsRequest;
  WayPoint: TDirectionsWaypoint;
begin
  with TScript(Sender) do
  begin
    //The alignment is not beautiful. But this is on the ToDo list for a
    //future release. ;-)
    MapAlignDiv:=Document.CreateDivNode;
    with MapAlignDiv.Style do
    begin
      Position:='absolute';
      Width:='100%';
      Height:='100%';
      PaddingRight:='300px';                               //Space for DirectionsPanel
    end;
    Document.Body.AppendChild(MapAlignDiv);
    MapDiv:=Document.CreateDivNode;
    with MapDiv.Style do
    begin
      Position:='absolute';
      Width:='100%';
      Height:='100%';
    end;
    MapAlignDiv.AppendChild(MapDiv);
    DirectionsPanel:=Document.CreateDivNode;
    with DirectionsPanel.Style do
    begin
      Position:='absolute';
      Padding:='4px';
      Width:='300px';
      Height:='100%';
      Right:='0px';
      Overflow:='auto';                                    //Vertical ScrollBar
    end;
    TotalDiv:=Document.CreateDivNode;
    DirectionsPanel.AppendChild(TotalDiv);
    Document.Body.AppendChild(DirectionsPanel);
    RendererOptions:=TDirectionsRendererOptions.Create;
    RendererOptions.Draggable:=true;
    DirectionsDisplay:=New(Google.Maps.DirectionsRenderer(RendererOptions));
    DirectionsService:=New(Google.Maps.DirectionsService);
    Australia:=New(Google.Maps.LatLng(-25.274398,133.775136));
    MyOptions:=TMapOptions.Create;
    with MyOptions do
    begin
      Zoom:=7;
      Center:=Australia;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    Map:=New(Google.Maps.Map(MapDiv,MyOptions));
    DirectionsDisplay.SetMap(Map);
    DirectionsDisplay.SetPanel(DirectionsPanel);
    {$IFDEF USE_ANONYMOUS_METHODS}
    DirectionsDisplay.OnDirectionsChanged:=
      procedure(Sender: TObject)
      var
        n: Integer;
        Total: Integer;
        MyRoute: TDirectionsRoute;
      begin
        Total:=0;
        MyRoute:=DirectionsDisplay.Directions.Routes[0];
        for n:=0 to MyRoute.Legs.Count-1 do
          Inc(Total,MyRoute.Legs[n].Distance.Value);
        TotalDiv.innerHTML:='<b>Total Distance: </b>'+FloatToStr(Total/1000)+' km';
      end;
    {$ELSE}
    DirectionsDisplay.OnDirectionsChanged:=DirectionsChanged;
    {$ENDIF}
    Request:=TDirectionsRequest.Create;
    with Request do
    begin
      OriginAsString:='Sydney, NSW';
      DestinationAsString:='Sydney, NSW';
      TravelMode:=Google.Maps.DirectionsTravelMode.Driving;
    end;
    WayPoint:=TDirectionsWayPoint.Create;
    WayPoint.LocationAsString:='Bourke, NSW';
    Request.WayPoints.Add(WayPoint);
    WayPoint:=TDirectionsWayPoint.Create;
    WayPoint.LocationAsString:='Broken Hill, NSW';
    Request.WayPoints.Add(WayPoint);
    {$IFDEF USE_ANONYMOUS_METHODS}
    DirectionsService.Route(Request,
      procedure(Sender: TObject; Response: TDirectionsResult; Status: String)
      begin
        if Status=Google.Maps.DirectionsStatus.OK
          then DirectionsDisplay.SetDirections(Response);
      end);
    {$ELSE}
    DirectionsService.Route(Request,DirectionsCallback);
    {$ENDIF}
  end;
end;

{$IFNDEF USE_ANONYMOUS_METHODS}
procedure TForm1.DirectionsCallback(Sender: TObject; Response: TDirectionsResult; Status: String);
begin
  with Script do
  begin
    if Status=Google.Maps.DirectionsStatus.OK
      then DirectionsRenderer[0].SetDirections(Response);
  end;
end;

procedure TForm1.DirectionsChanged(Sender: TObject);
var
  n: Integer;
  Total: Integer;
  MyRoute: TDirectionsRoute;
begin
  with Script do
  begin
    Total:=0;
    MyRoute:=DirectionsRenderer[0].Directions.Routes[0];
    for n:=0 to MyRoute.Legs.Count-1 do
      Inc(Total,MyRoute.Legs[n].Distance.Value);
    TotalDiv.innerHTML:='<b>Total Distance: </b>'+FloatToStr(Total/1000,'.')+' km';
  end;
end;
{$ENDIF}

end.
