//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/directions-panel.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, ExtCtrls,
  HTMLObjects, JScriptObjects, BrowserTools, gmAPI, gmMap, gmBase,
  gmServicesDirections, gmServicesDirectionsRenderer, gmEvents;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    StartPos: TComboBox;
    EndPos: TComboBox;
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    {$IFNDEF USE_ANONYMOUS_METHODS}
    FDirectionsPanel, FMapAlignDiv: TNode;
    procedure CalcRoute(Sender: TObject);
    procedure DirectionsCallback(Sender: TObject; Response: TDirectionsResult; Status: String);
    {$ENDIF}
    procedure InitMap(Sender: TObject);
  public
    {$IFNDEF USE_ANONYMOUS_METHODS}
    property DirectionsPanel: TNode read FDirectionsPanel write FDirectionsPanel;
    property MapAlignDiv: TNode read FMapAlignDiv write FMapAlignDiv;
    {$ENDIF}
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

type
  TLocation = record
    Value: String;
    Name: String;
  end;

const
  Locations: array[0..11] of TLocation = (
    (Value: 'chicag il'         ; Name: 'Chicago'),
    (Value: 'st louis, mo'      ; Name: 'St Louis'),
    (Value: 'joplin, mo'        ; Name: 'Joplin, MO'),
    (Value: 'oklahoma city, ok' ; Name: 'Oklahoma City'),
    (Value: 'amarillo, tx'      ; Name: 'Amarillo'),
    (Value: 'gallup, nm'        ; Name: 'Gallup, NM'),
    (Value: 'flagstaff, az'     ; Name: 'Flagstaff, AZ'),
    (Value: 'winona, az'        ; Name: 'Winona'),
    (Value: 'kingman, az'       ; Name: 'Kingman'),
    (Value: 'barstow, ca'       ; Name: 'Barstow'),
    (Value: 'san bernardino, ca'; Name: 'San Bernardino'),
    (Value: 'los angeles, ca'   ; Name: 'Los Angeles'));

procedure TForm1.FormCreate(Sender: TObject);
  procedure FillComboBox(ComboBox: TComboBox);
  var
    n: Integer;
  begin
    with ComboBox, Items do
    begin
      Clear;
      for n:=Low(Locations) to High(Locations) do
      Add(Locations[n].Name);
      ItemIndex:=0;
    end;
  end;
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
  FillComboBox(StartPos);
  FillComboBox(EndPos);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  {$IFDEF USE_ANONYMOUS_METHODS}
  ReleaseEvent(StartPos.OnClick);
  ReleaseEvent(EndPos.OnClick);
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if not assigned(Script(WebBrowser1)) then
    with TScript.Create(WebBrowser1) do
    begin
      BackgroundColor:=TColor($E4E4E4);
      LoadAPIAsync(InitMap);
    end;
end;

procedure TForm1.InitMap(Sender: TObject);
var
  {$IFDEF USE_ANONYMOUS_METHODS}
  DirectionsService: TDirectionsService;
  DirectionsPanel, MapAlignDiv: TNode;
  {$ENDIF}
  DirectionsDisplay: TDirectionsRenderer;
  Map: TMap;
  MapDiv: TNode;
  MapOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    //The alignment is not beautiful. But this is on the ToDo list for a
    //future release. ;-)
    MapAlignDiv:=Document.CreateDivNode;
    with MapAlignDiv.Style do
    begin
      Position:='absolute';
      Width:='100%';
      Height:='100%';
    end;
    Document.Body.AppendChild(MapAlignDiv);
    MapDiv:=Document.CreateDivNode;
    with MapDiv.Style do
    begin
      Position:='absolute';
      Width:='100%';
      Height:='100%';
    end;
    MapAlignDiv.AppendChild(MapDiv);
    DirectionsPanel:=Document.CreateDivNode;
    with DirectionsPanel.Style do
    begin
      Position:='absolute';
      Padding:='4px';
      Width:='300px';
      Height:='100%';
      Right:='0px';
      Overflow:='auto';                                    //Vertical ScrollBar
    end;
    Document.Body.AppendChild(DirectionsPanel);
    DirectionsDisplay:=New(Google.Maps.DirectionsRenderer);
    {$IFDEF USE_ANONYMOUS_METHODS}DirectionsService:={$ENDIF}New(Google.Maps.DirectionsService);
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=7;
      Center:=New(Google.Maps.LatLng(41.850033,-87.6500523));
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    Map:=New(Google.Maps.Map(MapDiv,MapOptions));
    DirectionsDisplay.SetMap(Map);
    DirectionsDisplay.SetPanel(DirectionsPanel);
    {$IFDEF USE_ANONYMOUS_METHODS}
    ReleaseEvent(StartPos.OnClick);
    ReleaseEvent(EndPos.OnClick);
    StartPos.OnClick:=MakeEvent(
      procedure(Sender: TObject)
      var
        Request: TDirectionsRequest;
      begin
        Request:=TDirectionsRequest.Create;
        with Request do
        begin
          OriginAsString:=Locations[StartPos.ItemIndex].Value;
          DestinationAsString:=Locations[EndPos.ItemIndex].Value;
          TravelMode:=Google.Maps.DirectionsTravelMode.Driving;
        end;
        DirectionsService.Route(Request,
          procedure(Sender: TObject; Response: TDirectionsResult; Status: String)
          begin
            if Status=Google.Maps.DirectionsStatus.OK then
            begin
              if (MapAlignDiv.Style.PaddingRight<>'300px') then
              begin
                MapAlignDiv.Style.PaddingRight:='300px';   //Space for DirectionsPanel
                Map.Resized;
              end;
              DirectionsDisplay.SetDirections(Response);
            end;
          end);
        end);
    EndPos.OnClick:=AddRefToEvent(StartPos.OnClick);
    {$ELSE}
    StartPos.OnClick:=CalcRoute;
    EndPos.OnClick:=CalcRoute;
    {$ENDIF}
  end;
end;

{$IFNDEF USE_ANONYMOUS_METHODS}
procedure TForm1.CalcRoute(Sender: TObject);
var
  Request: TDirectionsRequest;
begin
  with Script do
  begin
    Request:=TDirectionsRequest.Create;
    with Request do
    begin
      OriginAsString:=Locations[StartPos.ItemIndex].Value;
      DestinationAsString:=Locations[EndPos.ItemIndex].Value;
      TravelMode:=Google.Maps.DirectionsTravelMode.Driving;
    end;
    DirectionsServices[0].Route(Request,DirectionsCallback);
  end;
end;

procedure TForm1.DirectionsCallback(Sender: TObject; Response: TDirectionsResult; Status: String);
begin
  with Script do
  begin
    if Status=Google.Maps.DirectionsStatus.OK then
    begin
      if (MapAlignDiv.Style.PaddingRight<>'300px') then
      begin
        MapAlignDiv.Style.PaddingRight:='300px';   //Space for DirectionsPanel
        Maps[0].Resized;
      end;
      DirectionsRenderer[0].SetDirections(Response);
    end;
  end;
end;
{$ENDIF}

(*
      var directionDisplay;
      var directionsService = new google.maps.DirectionsService();

      function initialize() {
        directionsDisplay = new google.maps.DirectionsRenderer();
        var myOptions = {
          zoom: 7,
          mapTypeId: google.maps.MapTypeId.ROADMAP,
          center: new google.maps.LatLng(41.850033, -87.6500523)
        };
        var map = new google.maps.Map(document.getElementById('map_canvas'),
            myOptions);
        directionsDisplay.setMap(map);
        directionsDisplay.setPanel(document.getElementById('directions-panel'));

        var control = document.getElementById('control');
        control.style.display = 'block';
        map.controls[google.maps.ControlPosition.TOP].push(control);
      }

      function calcRoute() {
        var start = document.getElementById('start').value;
        var end = document.getElementById('end').value;
        var request = {
          origin: start,
          destination: end,
          travelMode: google.maps.DirectionsTravelMode.DRIVING
        };
        directionsService.route(request, function(response, status) {
          if (status == google.maps.DirectionsStatus.OK) {
            directionsDisplay.setDirections(response);
          }
        });
      }

      google.maps.event.addDomListener(window, 'load', initialize);
    </script>
    </div>
    <div id="directions-panel"></div>
    <div id="map_canvas"></div>
  </body>
</html>
*)

end.
