//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/directions-simple.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, ExtCtrls,
  gmAPI, gmMap, gmBase, gmServicesDirections, gmServicesDirectionsRenderer,
  gmEvents;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    Panel1: TPanel;
    Label1: TLabel;
    StartPos: TComboBox;
    Label2: TLabel;
    EndPos: TComboBox;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    {$IFNDEF USE_ANONYMOUS_METHODS}
    procedure CalcRoute(Sender: TObject);
    procedure DirectionsCallback(Sender: TObject; Response: TDirectionsResult; Status: String);
    {$ENDIF}
    procedure InitMap(Sender: TObject);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

type
  TLocation = record
    Value: String;
    Name: String;
  end;

const
  Locations: array[0..11] of TLocation = (
    (Value: 'chicag il'         ; Name: 'Chicago'),
    (Value: 'st louis, mo'      ; Name: 'St Louis'),
    (Value: 'joplin, mo'        ; Name: 'Joplin, MO'),
    (Value: 'oklahoma city, ok' ; Name: 'Oklahoma City'),
    (Value: 'amarillo, tx'      ; Name: 'Amarillo'),
    (Value: 'gallup, nm'        ; Name: 'Gallup, NM'),
    (Value: 'flagstaff, az'     ; Name: 'Flagstaff, AZ'),
    (Value: 'winona, az'        ; Name: 'Winona'),
    (Value: 'kingman, az'       ; Name: 'Kingman'),
    (Value: 'barstow, ca'       ; Name: 'Barstow'),
    (Value: 'san bernardino, ca'; Name: 'San Bernardino'),
    (Value: 'los angeles, ca'   ; Name: 'Los Angeles'));

procedure TForm1.FormCreate(Sender: TObject);
  procedure FillComboBox(ComboBox: TComboBox);
  var
    n: Integer;
  begin
    with ComboBox, Items do
    begin
      Clear;
      for n:=Low(Locations) to High(Locations) do
      Add(Locations[n].Name);
      ItemIndex:=0;
    end;
  end;
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
  FillComboBox(StartPos);
  FillComboBox(EndPos);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  {$IFDEF USE_ANONYMOUS_METHODS}
  ReleaseEvent(StartPos.OnClick);
  ReleaseEvent(EndPos.OnClick);
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Chicago: TLatLng;
  DirectionsDisplay: TDirectionsRenderer;
  {$IFDEF USE_ANONYMOUS_METHODS}
  DirectionsService: TDirectionsService;
  {$ENDIF}
  Map: TMap;
  MapOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    DirectionsDisplay:=New(Google.Maps.DirectionsRenderer);
    {$IFDEF USE_ANONYMOUS_METHODS}DirectionsService:={$ENDIF}New(Google.Maps.DirectionsService);
    Chicago:=New(Google.Maps.LatLng(41.850033,-87.6500523));
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=7;
      Center:=Chicago;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    Map:=New(Google.Maps.Map(MapOptions));
    DirectionsDisplay.SetMap(Map);
    {$IFDEF USE_ANONYMOUS_METHODS}
    ReleaseEvent(StartPos.OnClick);
    ReleaseEvent(EndPos.OnClick);
    StartPos.OnClick:=MakeEvent(
      procedure(Sender: TObject)
      var
        Request: TDirectionsRequest;
      begin
        Request:=TDirectionsRequest.Create;
        with Request do
        begin
          OriginAsString:=Locations[StartPos.ItemIndex].Value;
          DestinationAsString:=Locations[EndPos.ItemIndex].Value;
          TravelMode:=Google.Maps.DirectionsTravelMode.Driving;
        end;
        DirectionsService.Route(Request,
          procedure(Sender: TObject; Response: TDirectionsResult; Status: String)
          begin
            if Status=Google.Maps.DirectionsStatus.OK
              then DirectionsDisplay.SetDirections(Response);
          end);
        end);
    EndPos.OnClick:=AddRefToEvent(StartPos.OnClick);
    {$ELSE}
    StartPos.OnClick:=CalcRoute;
    EndPos.OnClick:=CalcRoute;
    {$ENDIF}
  end;
end;

{$IFNDEF USE_ANONYMOUS_METHODS}
procedure TForm1.CalcRoute(Sender: TObject);
var
  Request: TDirectionsRequest;
begin
  with Script do
  begin
    Request:=TDirectionsRequest.Create;
    with Request do
    begin
      OriginAsString:=Locations[StartPos.ItemIndex].Value;
      DestinationAsString:=Locations[EndPos.ItemIndex].Value;
      TravelMode:=Google.Maps.DirectionsTravelMode.Driving;
    end;
    DirectionsServices[0].Route(Request,DirectionsCallback);
  end;
end;

procedure TForm1.DirectionsCallback(Sender: TObject; Response: TDirectionsResult; Status: String);
begin
  with Script do
  begin
    if Status=Google.Maps.DirectionsStatus.OK
      then DirectionsRenderer[0].SetDirections(Response);
  end;
end;
{$ENDIF}

end.
