//http://code.google.com/apis/maps/documentation/javascript/services.html#distance_matrix
//http://code.google.com/apis/maps/documentation/javascript/examples/distance-matrix.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, ExtCtrls,
  DispObject, JScriptObjects, gmAPI, gmMap, gmBase, gmOverlaysMarker,
  gmServicesDistanceMatrix, gmServicesGeocoder;

const
  DestinationIcon = 'http://chart.apis.google.com/chart?chst=d_map_pin_letter&chld=D|FF0000|000000';
  OriginIcon      = 'http://chart.apis.google.com/chart?chst=d_map_pin_letter&chld=O|FFFF00|000000';

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    Panel1: TPanel;
    Panel2: TPanel;
    CalculateDistances: TButton;
    Memo1: TMemo;
    Splitter1: TSplitter;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CalculateDistancesClick(Sender: TObject);
  private
    FBounds: TLatLngBounds;
    FGeocoder: TGeocoder;
    FMap: TMap;
    procedure AddMarker(Results: TGeocoderResultArray; Status, MarkerIcon: String);
    procedure AddDestinationMarker(Sender: TObject; Results: TGeocoderResultArray; Status: String);
    procedure AddOriginMarker(Sender: TObject; Results: TGeocoderResultArray; Status: String);
    procedure Callback(Sender: TObject; Response: TDistanceMatrixResponse; Status: String);
    procedure InitMap(Sender: TObject);
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  MapOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Zoom:=10;
      Center:=New(Google.Maps.LatLng(55.53,9.4));
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    FMap:=New(Google.Maps.Map(MapOptions));
    FGeocoder:=New(Google.Maps.Geocoder);
    FBounds:=New(Google.Maps.LatLngBounds);
  end;
end;

procedure TForm1.CalculateDistancesClick(Sender: TObject);
var
  Origin1, Destination2: TLatLng;
  Origin2, Destination1: String;
  Request: TDistanceMatrixRequest;
  Service: TDistanceMatrixService;
begin
  with Script do
  begin
    Origin1:=New(Google.Maps.LatLng(55.930385,-3.118425));
    Origin2:='Greenwich, England';
    Destination1:='Stockholm, Sweden';
    Destination2:=New(Google.Maps.LatLng(50.087692,14.421150));
    Request:=TDistanceMatrixRequest.Create;
    with Request do
    begin
      Origins.Add(Origin1);
      Origins.Add(Origin2);
      Destinations.Add(Destination1);
      Destinations.Add(Destination2);
      TravelMode:=Google.Maps.TravelMode.Driving;
      UnitSystem:=Google.Maps.UnitSystem.Metric;
      AvoidHighways:=false;
      AvoidTolls:=false;
    end;
    Service:=New(Google.Maps.DistanceMatrixService);
    Service.GetDistanceMatrix(Request,Callback);
  end;
end;

procedure TForm1.Callback(Sender: TObject; Response: TDistanceMatrixResponse; Status: String);
  procedure AddMarker(Location: String; IsDestination: Boolean);
  var
    Callback: TGeocoderCallback;
    GeocoderRequest: TGeocoderRequest;
  begin
    if IsDestination
      then Callback:=AddDestinationMarker
      else Callback:=AddOriginMarker;
    GeocoderRequest:=TGeocoderRequest.Create(Location);
    FGeocoder.Geocode(GeocoderRequest,Callback);
  end;
  procedure DeleteOverlays;
  var
    n: Integer;
  begin
    with Script do
    begin
      for n:=0 to Markers.Count-1 do
        Markers[n].Map:=nil;
    end;
  end;
var
  i, j: Integer;
  Destinations, Origins: TStringArray;
  Results: TDistanceMatrixResponseElementArray;
begin
  with Script do
  begin
    Memo1.Lines.Clear;
    DeleteOverlays;
    if Status=Google.Maps.DistanceMatrixStatus.OK then
    begin
      Origins:=Response.OriginAddresses;
      Destinations:=Response.DestinationAddresses;
      for i:=0 to Origins.Count-1 do
      begin
        Results:=Response.Rows[i].Elements;
        AddMarker(Origins[i],false);
        for j:=0 to Results.Count-1 do
        begin
          AddMarker(Destinations[j],true);
          Memo1.Lines.Add(Origins[i]+' to '+Destinations[j]+
                          ': '+Results[j].Distance.Text+' in '+
                          Results[j].Duration.Text);
        end;
      end;
    end else Memo1.Lines.Add('Error was: '+Status);
  end;
end;

procedure TForm1.AddMarker(Results: TGeocoderResultArray; Status, MarkerIcon: String);
var
  Opts: TMarkerOptions;
begin
  with Script do
  begin
    if Status=Google.Maps.GeocoderStatus.Ok then
    begin
      FBounds.Extend(Results[0].Geometry.Location);
      FMap.FitBounds(FBounds);
      Opts:=TMarkerOptions.Create;
      with Opts do
      begin
        Map:=FMap;
        Position:=Results[0].Geometry.Location;
        Icon:=MarkerIcon;
        Title:=Results[0].AddressComponents[0].LongName;
      end;
      New(Google.Maps.Marker(Opts));
    end else Memo1.Lines.Add('Geocode was not successful for the following reason: '+Status);
  end;
end;

procedure TForm1.AddDestinationMarker(Sender: TObject; Results: TGeocoderResultArray; Status: String);
begin
  AddMarker(Results,Status,DestinationIcon);
end;

procedure TForm1.AddOriginMarker(Sender: TObject; Results: TGeocoderResultArray; Status: String);
begin
  AddMarker(Results,Status,OriginIcon);
end;

end.
