//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/elevation-paths.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, ExtCtrls,
  TeEngine, TeeProcs, Chart, BrowserTools, JScriptObjects, Series, gmAPI, gmMap,
  gmBase, gmServicesElevation, gmOverlaysInfoWindow, gmOverlaysPolylinePolygon,
  gmEvents;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    Chart: TChart;
    Splitter1: TSplitter;
    Series1: TBarSeries;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    {$IFNDEF USE_ANONYMOUS_METHODS}
    procedure ElevationCallback(Sender: TObject; Elevations: TElevationResultArray;
                                Status: String);
    {$ENDIF}
    procedure InitMap(Sender: TObject);
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  Chart.Series[0].Clear;
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);
end;

procedure TForm1.InitMap(Sender: TObject);
var
  {$IFDEF USE_ANONYMOUS_METHODS}
  Map: TMap;
  {$ENDIF}
  Elevator: TElevationService;
  MyOptions: TMapOptions;
  PathRequest: TPathElevationRequest;
  Whitney, Lonepine, Owenslake, Beattyjunction,
  Panamintsprings, Badwater: TLatLng;
begin
  with TScript(Sender) do
  begin
    //The following path marks a general path from Mt. Whitney, the highest
    //point in the continental United States to Badwater, Death Vallet, the
    //lowest point.
    Whitney:=New(Google.Maps.LatLng(36.578581,-118.291994));
    Lonepine:=New(Google.Maps.LatLng(36.606111,-118.062778));
    Owenslake:=New(Google.Maps.LatLng(36.433269,-117.950916));
    Beattyjunction:=New(Google.Maps.LatLng(36.588056,-116.943056));
    Panamintsprings:=New(Google.Maps.LatLng(36.339722,-117.467778));
    Badwater:=New(Google.Maps.LatLng(36.23998,-116.83171));
    MyOptions:=TMapOptions.Create;
    with MyOptions do
    begin
      Zoom:=8;
      Center:=Lonepine;
      MapTypeID:=Google.Maps.MapTypeID.Terrain;
    end;
    {$IFDEF USE_ANONYMOUS_METHODS}Map:={$ENDIF}New(Google.Maps.Map(MyOptions));
    //Create an ElevationService.
    Elevator:=New(Google.Maps.ElevationService);
    //Draw the path, using the Elevation service.
    Chart.Series[0].Clear;
    //Create a PathElevationRequest object using this array.
    //Ask for 256 samples along that path.
    PathRequest:=TPathElevationRequest.Create;
    with PathRequest do
    begin
      {$IFDEF CAN_OVERLOAD}
      Path.Push([Whitney,Lonepine,Owenslake,Panamintsprings,Beattyjunction,Badwater]);
      {$ELSE}
      Path.Push(Whitney);
      Path.Push(Lonepine);
      Path.Push(Owenslake);
      Path.Push(Panamintsprings);
      Path.Push(Beattyjunction);
      Path.Push(Badwater);
      {$ENDIF}
      Samples:=256;
    end;
    //Initiate the path request.
    {$IFDEF USE_ANONYMOUS_METHODS}
    Elevator.GetElevationAlongPath(PathRequest,
      procedure(Sender: TObject; Elevations: TElevationResultArray; Status: String)
      var
        n: Integer;
        PathOptions: TPolylineOptions;
      begin
        //Takes an array of ElevationResult objects, draws the path on the map
        //and plots the elevation profile on a Visualization API ColumnChart.
        if Status=Google.Maps.ElevationStatus.OK then
        begin
          //Extract the elevation samples from the returned results and store
          //them.
          PathOptions:=TPolylineOptions.Create;
          for n:=0 to Elevations.Count-1 do
            PathOptions.Path.Push(Elevations[n].Location);
          //Display a polyline of the elevation path.
          with PathOptions do
          begin
            StrokeColor:=WebColorStrToColor('#0000CC');
            StrokeOpacity:=0.4;
          end;
          PathOptions.Map:=Map;
          New(Google.Maps.Polyline(PathOptions));
          //Extract the data from which to populate the chart. Because the
          //samples are equidistant, the 'Sample' column here does double duty
          //as distance along the X axis.
          for n:=0 to Elevations.Count-1 do
            Chart.Series[0].Add(Elevations[n].Elevation);
        end;
      end);
    {$ELSE}
    Elevator.GetElevationAlongPath(PathRequest,ElevationCallback);
    {$ENDIF}
  end;
end;

{$IFNDEF USE_ANONYMOUS_METHODS}
procedure TForm1.ElevationCallback(Sender: TObject;
                                   Elevations: TElevationResultArray; Status: String);
var
  n: Integer;
  PathOptions: TPolylineOptions;
begin
  with Script do
  begin
    //Takes an array of ElevationResult objects, draws the path on the map
    //and plots the elevation profile on a Visualization API ColumnChart.
    if Status=Google.Maps.ElevationStatus.OK then
    begin
      //Extract the elevation samples from the returned results and store
      //them.
      PathOptions:=TPolylineOptions.Create;
      for n:=0 to Elevations.Count-1 do
        PathOptions.Path.Push(Elevations[n].Location);
      //Display a polyline of the elevation path.
      with PathOptions do
      begin
        StrokeColor:=WebColorStrToColor('#0000CC');
        StrokeOpacity:=0.4;
      end;
      PathOptions.Map:=Maps[0];
      New(Google.Maps.Polyline(PathOptions));
      //Extract the data from which to populate the chart. Because the
      //samples are equidistant, the 'Sample' column here does double duty
      //as distance along the X axis.
      for n:=0 to Elevations.Count-1 do
        Chart.Series[0].Add(Elevations[n].Elevation);
    end;
  end;
end;
{$ENDIF}

end.
