//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/streetview-custom-tiles.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw, gmAPI, gmMap,
  gmBase, gmStreetView;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    procedure CreateCustomLinks(EntryPanoID: String);
    procedure InitMap(Sender: TObject);
    function GetCustomPanorama(Sender: TObject; Pano: String): TStreetViewPanoramaData;
    function GetCustomPanoramaTileUrl(Sender: TObject; Pano: String; Zoom, TileX, TileY: Integer): String;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
      LoadAPIAsync(InitMap);                               //API load and create map
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Radius: Integer;
  SydneyOffice: TLatLng;
  Map: TMap;
  MapOptions: TMapOptions;
  PanoOptions: TStreetViewPanoramaOptions;
  Panorama: TStreetViewPanorama;
  StreetViewService: TStreetViewService;
begin
  with TScript(Sender) do
  begin
    // The Latlng of the entry point to the Google office on the road.
    SydneyOffice:=New(Google.Maps.LatLng(-33.867386,151.195767));
    //Set up the map and enable the Street View control.
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Center:=SydneyOffice;
      Zoom:=16;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
    end;
    Map:=New(Google.Maps.Map(MapOptions));
    Panorama:=Map.GetStreetView;
    //Set up Street View and initially set it visible. Register the
    //custom panorama provider function.
    PanoOptions:=TStreetViewPanoramaOptions.Create;
    with PanoOptions do
    begin
      Position:=SydneyOffice;
      Visible:=True;
      PanoProvider:=GetCustomPanorama;
    end;
    Panorama.SetOptions(PanoOptions);
    //Create a StreetViewService object.
    StreetViewService:=New(Script.Google.Maps.StreetViewService);
    //Compute the nearest panorama to the Google Sydney office
    //using the service and store that pano ID.
    Radius:=50;
    StreetViewService.GetPanoramaByLocation(SydneyOffice,Radius,
      procedure(Sender: TObject; Result: TStreetViewPanoramaData; Status: String)
      begin
        if Status=Google.Maps.StreetViewStatus.OK then
        begin
          //We'll monitor the links_changed event to check if the current
          //pano is either a custom pano or our entry pano.
          Panorama.OnLinksChanged:=
            procedure(Sender: TObject)
            begin
              CreateCustomLinks(Result.Location.Pano);
            end;
        end;
      end);
  end;
end;

function TForm1.GetCustomPanorama(Sender: TObject; Pano: String): TStreetViewPanoramaData;
begin
  Result:=nil;
  with Script do
  begin
    if Pano='reception' then
    begin
      Result:=TStreetViewPanoramaData.Create;
      with Result do
      begin
        with Location do
        begin
          Pano:='reception';
          Description:='Google Sydney - Reception';
          LatLng:=New(Google.Maps.LatLng(-33.86684,151.19583));
        end;
        // The text for the copyright control.
        Copyright:='Imagery (c) 2010 Google';
        // The definition of the tiles for this panorama.
        with Tiles do
        begin
          TileSize:=New(Google.Maps.Size(1024,512));
          WorldSize:=New(Google.Maps.Size(2048,1024));
          // The heading at the origin of the panorama tile set.
          CenterHeading:=105;
          GetTileUrl:=GetCustomPanoramaTileUrl;
        end;
      end;
    end;
  end;
end;

procedure TForm1.CreateCustomLinks(EntryPanoID: String);
var
  Panorama: TStreetViewPanorama;
  Link: TStreetViewLink;
  Links: TStreetViewLinkArray;
  PanoID: String;
begin
  Panorama:=Script.Maps[0].GetStreetView;
  Links:=Panorama.GetLinks;
  PanoID:=Panorama.GetPano;
  if PanoID=EntryPanoID then
  begin
    Link:=TStreetViewLink.Create;
    with Link do
    begin
      Heading:=25;
      Description:='Google Sydney';
      Pano:='reception';
    end;
   //Adding a link in the view from the entrance of the building to reception.
   Links.Push(Link);
  end;
  if PanoID='reception' then
  begin
    Link:=TStreetViewLink.Create;
    with Link do
    begin
      Heading:=195;
      Description:='Exit';
      Pano:=EntryPanoId;
    end;
    // Adding a link in the view from the entrance of the office
    // with an arrow pointing at 100 degrees, with a text of "Exit"
    // and loading the street entrance of the building pano on click.
   Links.Push(Link);
  end;
end;

function TForm1.GetCustomPanoramaTileUrl(Sender: TObject; Pano: String; Zoom,
                                         TileX, TileY: Integer): String;
begin
  //Return a pano image given the panoID.
  Result:='http://code.google.com/apis/maps/documentation/javascript/examples/'+
          'images/panoReception1024-'+IntToStr(Zoom)+'-'+IntToStr(TileX)+'-'+
          IntToStr(TileY)+'.jpg';
end;

end.
