//http://code.google.com/apis/maps/documentation/javascript/examples/index.html
//http://code.google.com/apis/maps/documentation/javascript/examples/streetview-service.html
unit Unit1;

interface

{$INCLUDE gmConfig.inc}

uses
  Windows, Messages, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Classes,
  Graphics, Controls, Forms, Dialogs, StdCtrls, OleCtrls, SHDocVw,
  gmAPI, gmBase, gmMap, gmStreetView, gmOverlaysMarker, gmEvents, HTMLObjects;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FPanorama: TStreetViewPanorama;
    FStreetViewService: TStreetViewService;
    procedure InitMap(Sender: TObject);
    procedure MapClick(Sender: TObject; MouseEvent: TMouseEvent);
    procedure MarkerClick(Sender: TObject; Event: TEvent);
    procedure ProcessSVData(Sender: TObject;
                            StreetViewPanoramaData: TStreetViewPanoramaData;
                            Status: String);
  public
    property StreetViewService: TStreetViewService read FStreetViewService
                                                   write FStreetViewService;
    property Panorama: TStreetViewPanorama read FPanorama write FPanorama;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  {$IFDEF DELPHI2006_UP}
  ReportMemoryLeaksOnShutdown:=true;
  {$ENDIF}
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if Script=nil then
    with TScript.Create(WebBrowser1) do
    begin
      BackgroundColor:=Form1.Color;
      LoadAPIAsync(InitMap);
    end;
end;

procedure TForm1.InitMap(Sender: TObject);
var
  Berkeley: TLatLng;
  Map: TMap;
  MapOptions: TMapOptions;
begin
  with TScript(Sender) do
  begin
    Berkeley:=New(Google.Maps.LatLng(37.869085,-122.254775));
    MapOptions:=TMapOptions.Create;
    with MapOptions do
    begin
      Width:=300;
      Height:=250;
      Align:=alNone;
      Center:=Berkeley;
      Zoom:=16;
      MapTypeID:=Google.Maps.MapTypeID.Roadmap;
      StreetViewControl:=false;
    end;
    Map:=New(Google.Maps.Map(MapOptions));
    Map.OnClick:=MapClick;
    StreetViewService:=New(Google.Maps.StreetViewService);
    Panorama:=New(Google.Maps.StreetViewPanorama);
    with Panorama do
    begin
      Left:=300;
      Width:=300;
      Height:=250;
      Align:=alNone;
      //Visible:=false;                //don't work with Visible=false -> API bug!?
    end;
  end;
end;

procedure TForm1.MapClick(Sender: TObject; MouseEvent: TMouseEvent);
begin
  //GetPanoramaByLocation will return the nearest pano when the given radius is
  //50 meters or less.
  StreetViewService.GetPanoramaByLocation(MouseEvent.LatLng,50,ProcessSVData);
end;

procedure TForm1.MarkerClick(Sender: TObject; Event: TEvent);
var
  Pov: TStreetViewPov;
begin
  //Set the Pano to use the passed panoID
  Panorama.SetPano(TMarker(Sender).Properties['MarkerPanoID']);
  Pov:=TStreetViewPov.Create;
  with Pov do
  begin
    Heading:=270;
    Pitch:=0;
    Zoom:=1;
  end;
  Panorama.SetPov(Pov);
  //Panorama.SetVisible(true);
end;

procedure TForm1.ProcessSVData(Sender: TObject;
                               StreetViewPanoramaData: TStreetViewPanoramaData;
                               Status: String);
var
  Pov: TStreetViewPov;
  Marker: TMarker;
  MarkerOptions: TMarkerOptions;
begin
  with Script do
  begin
    if Status=Google.Maps.StreetViewStatus.OK then
    begin
      MarkerOptions:=TMarkerOptions.Create;
      with MarkerOptions do
      begin
        Position:=StreetViewPanoramaData.Location.LatLng;
        Map:=Maps[0];
        Title:=StreetViewPanoramaData.Location.Description;
      end;
      Marker:=New(Google.Maps.Marker(MarkerOptions));
      Pov:=TStreetViewPov.Create;
      with Pov do
      begin
        Heading:=270;
        Pitch:=0;
        Zoom:=1;
      end;
      Panorama.SetPov(Pov);
      Panorama.SetPano(StreetViewPanoramaData.Location.Pano);
      //Panorama.SetVisible(true);
      Marker.Properties['MarkerPanoID']:=StreetViewPanoramaData.Location.Pano;
      Marker.OnClick:=MarkerClick;
    end else ShowMessage('Street View data not found for this location.');
  end;
end;

end.
