unit gmAPI;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, Graphics, SysUtils, Contnrs, Windows, Messages, DispObject, ShellAPI,
  OleCtrls, Forms, JScriptObjects, HTMLObjects, BrowserTools, gmBase, gmMVC,
  gmServicesDirections, gmServicesElevation, gmServicesDistanceMatrix,
  gmServicesDirectionsRenderer, gmServicesGeocoder, gmServicesMaxZoom,
  gmMap, gmEvents, gmLayers, gmStreetView, gmOverlaysBase, gmOverlaysMarker,
  gmOverlaysPolylinePolygon, gmOverlaysInfoWindow, gmControls, gmMapTypes,
  gmLibrariesPanoramio, gmLibrariesPlaces, gmLibrariesAdsense,
  gmLibrariesGeometry, gmAdditionalConstants, gmAdditionalMessages;

const
  DefaultLanguage = 'default';
  DefaultRegion   = 'default';
  DefaultVersion  = 0;

type
  TCircleCollection = class;
  TDirectionsRendererCollection = class;
  TDirectionsServiceCollection = class;
  TDispObjectCollection = class;
  TDistanceMatrixServiceCollection = class;
  TElevationServiceCollection = class;
  TGeocoderCollection = class;
  TGoogle = class;
  TGroundOverlayCollection = class;
  TInfoWindowCollection = class;
  TLayerCollection = class;
  TMapCollection = class;
  TMaps = class;
  TMapTypeCollection = class;
  TMarkerCollection = class;
  TMaxZoomServiceCollection = class;
  TPolygonCollection = class;
  TPolylineCollection = class;
  TRectangleCollection = class;
  TStreetViewPanoramaCollection = class;
  TScript = class;

  TGoogleLibrary = (libGeometry, libPanoramio, libPlaces, libAdsense);
  TGoogleLibraries = set of TGoogleLibrary;

  TReloadType = (rtNone, rtAuto, rtDialog);

  TGoogle = class(TCustomJScriptObject)
  private
    FMaps: TMaps;
    function GetMaps: TMaps;
  public
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    property Maps: TMaps read GetMaps;
  end;

  TMaps = class(TCustomJScriptObject)
  private
    FAdsense: TAdsense;
    FAnimation: TAnimation;
    FControlPosition: TControlPosition;
    FDirectionsStatus: TDirectionsStatus;
    FDirectionsTravelMode: TDirectionsTravelMode;
    FDirectionsUnitSystem: TDirectionsUnitSystem;
    FDistanceMatrixStatus: TDistanceMatrixStatus;
    FElevationStatus: TElevationStatus;
    FEvent: TEvent;
    FGeocoderLocationType: TGeocoderLocationType;
    FGeocoderStatus: TGeocoderStatus;
    FGeometry: TGeometry;
    FMapTypeControlStyle: TMapTypeControlStyle;
    FMapTypeID: TMapTypeID;
    FMaxZoomStatus: TMaxZoomStatus;
    FNavigationControlsStyle: TNavigationControlStyle;
    FPanoramio: TPanoramio;
    FPlaces: TPlaces;
    FScaleControlStyle: TScaleControlStyle;
    FStreetViewStatus: TStreetViewStatus;
    FTravelMode: TTravelMode;
    FUnitSystem: TUnitSystem;
    FZoomControlStyle: TZoomControlStyle;
    function GetAdsense: TAdsense;
    function GetAnimation: TAnimation;
    function GetControlPosition: TControlPosition;
    function GetDirectionsStatus: TDirectionsStatus;
    function GetDirectionsTravelMode: TDirectionsTravelMode;
    function GetDirectionsUnitSystem: TDirectionsUnitSystem;
    function GetDistanceMatrixStatus: TDistanceMatrixStatus;
    function GetElevationStatus: TElevationStatus;
    function GetEvent: TEvent;
    function GetGeocoderLocationType: TGeocoderLocationType;
    function GetGeocoderStatus: TGeocoderStatus;
    function GetGeometry: TGeometry;
    function GetMapTypeControlStyle: TMapTypeControlStyle;
    function GetMapTypeID: TMapTypeID;
    function GetMaxZoomStatus: TMaxZoomStatus;
    function GetNavigationControlStyle: TNavigationControlStyle;
    function GetPanoramio: TPanoramio;
    function GetPlaces: TPlaces;
    function GetScaleControlStyle: TScaleControlStyle;
    function GetStreetViewStatus: TStreetViewStatus;
    function GetTravelMode: TTravelMode;
    function GetUnitSystem: TUnitSystem;
    function GetVersion: String;
    function GetZoomControlStyle: TZoomControlStyle;
  public
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    function BicyclingLayer: TBicyclingLayer;
    function Circle(Opts: TCircleOptions = nil): TCircle;
    function DirectionsRenderer(Opts: TDirectionsRendererOptions = nil): TDirectionsRenderer;
    function DirectionsService: TDirectionsService;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DistanceMatrixService'}{$ENDIF}
    ///<summary>
    ///  Creates a new instance of a DistanceMatrixService that sends distance
    ///  matrix queries to Google servers.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixService"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function DistanceMatrixService: TDistanceMatrixService;
    function ElevationService: TElevationService;
    function FusionTablesLayer(Options: TFusionTablesLayerOptions): TFusionTablesLayer;
    function Geocoder: TGeocoder;
    function GroundOverlay(const URL: String; Bounds: TLatLngBounds; Opts: TGroundOverlayOptions = nil): TGroundOverlay;
    function ImageMapType(Opts: TImageMapTypeOptions): TImageMapType;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InfoWindow'}{$ENDIF}
    ///<summary>
    ///  Erstellt ein Info-Fenster mit den angegebenen Optionen. Ein Info-Fenster
    ///  kann auf einer Karte oder einem StreetViewPanorama - je nach angegebenen
    ///  Optionen - an einer bestimmten Position oder oberhalb einer Markierung
    ///  platziert werden. Dabei wird die Karte gescrollt, damit das Fenster beim
    ///  ffnen sichtbar wird - es sei denn, das automatische Schwenken wurde
    ///  deaktiviert. Nachdem ein Info- Fenster erstellt wurde, mu die Methode
    ///  <see cref="gmOverlays.InfoWindow|TInfoWindow.Open(TMap,TMVCObject)">Open</see>
    ///  aufgerufen werden, um es auf der Karte anzuzeigen. Der Benutzer kann auf
    ///  die Schaltflche <i>Schlieen</i> im InfoWindow klicken, um es aus der
    ///  Karte zu entfernen. Alternativ kann
    ///  <see cref="gmOverlays.InfoWindow|TInfoWindow.Close">Close</see>
    ///  aufrufen werden.
    ///</summary>
    ///<seealso cref="gmOverlays.InfoWindow|TInfoWindow"/>
    ///<seealso cref="gmOverlays.InfoWindow|TInfoWindowOptions"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function InfoWindow(Opts: TInfoWindowOptions = nil): TInfoWindow;
    function KmlLayer(const URL: String; Opts: TKmlLayerOptions = nil): TKmlLayer;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LatLng'}{$ENDIF}
    ///<summary>
    ///  Erstellt einen Punkt mit den angegebenen geografischen Koordinaten.
    ///  Obwohl bei Kartenprojektionen normalerweise die geografische Lnge der
    ///  x- und die Breite der y-Koordinate zugeordnet ist, steht die Breite
    ///  immer <i>vor</i> der Lnge: (Lat,Lng).
    ///</summary>
    ///<remarks>
    ///  Die Koordinaten eines Punktes knnen nach seiner Erstellung nicht mehr
    ///  gendert werden.
    ///  <para/>
    ///  Ist fr das <i>NoWrap</i>-Flag der Wert <i>true</i> eingestellt, werden
    ///  die Werte fr <i>Lat</i> und <i>Lng</i> so verwendet, wie sie bergeben
    ///  wurden. Anderenfalls muss die Breite zwischen -90 und +90 Grad und die
    ///  Lnge zwischen -180 Grad und +180 Grad liegen.
    ///</remarks>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function LatLng(Lat, Lng: Double; NoWrap: Boolean = true): TLatLng;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LatLngBounds'}{$ENDIF}
    ///<summary>
    ///  Erstellt ein Rechteck aus den Punkten der angegebenen sdwestlichen
    ///  <i>(SW)</i> und nordstlichen <i>(NE)</i> Ecke.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function LatLngBounds(SW: TLatLng = nil; NE: TLatLng = nil): TLatLngBounds; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LatLngBounds'}{$ENDIF}
    ///<summary>
    ///  Erstellt ein Rechteck mit dem angegebenen Mittelpunkt und der Abmessungen
    ///  <i>Lat</i> und <i>Lng</i>.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function LatLngBounds(Center: TLatLng; Lat, Lng: Double): TLatLngBounds; overload;
    function Map(Opts: TMapOptions): TMap; overload;
    function Map(MapDiv: TNode; Opts: TMapOptions): TMap; overload;
    //Steht zwar in der API-Dokumentation, ist aber im Maps-Namespace nicht vorhanden:
    //function MapCanvasProjection: TMapCanvasProjection;
    function MapTypeRegistry: TMapTypeRegistry;
    //Each MapTypeStyleElementType distinguishes between the different representations
    //of a feature.
    //Ist zwar im API dokumentiert, befindet sich aber nicht im Maps-Namepace.
    function MapTypeStyleElementType: TMapTypeStyleElementType;
    //Possible values for feature types. Specify these values as strings, i.e.
    //'administrative' or 'poi.park'. Stylers applied to a parent feature type
    //automatically apply to all child feature types. Note however that parent
    //features may include some additional features that are not included in one
    //of their child feature types.
    //Ist zwar im API dokumentiert, befindet sich aber nicht im Maps-Namepace.
    function MapTypeStyleFeatureType: TMapTypeStyleFeatureType;
    function Marker(Opts: TMarkerOptions = nil): TMarker;
    //Zitat aus Google Maps API:
    //http://code.google.com/intl/de/apis/maps/documentation/javascript/reference.html#MarkerImage
    //Definiert ein Bild, das als Symbol oder Schatten fr eine Markierung
    //verwendet werden soll. "origin" und "size" werden verwendet, um ein
    //Segment eines Schattenbildes auszuwhlen, "anchor" setzt die Position
    //des Ankerpunktes von seiner Standardposition unten in der Mitte auer
    //Kraft. Der Anker eines Bildes ist das Pixel, auf das sich das System
    //beim Tracking der Bildposition bezieht. Standardmig ist der Anker
    //auf die untere Mitte des Bildes festgelegt (Koordinaten Breite/2, Hhe).
    //Wird also eine Markierung bei einer bestimmten LatLng platziert,
    //befindet sich das als Anker definierte Pixel bei der angegebenen LatLng.
    //Wenn Sie das Bild skalieren mchten (egal ob mit oder ohne Sprite),
    //legen Sie als Wert fr scaledSize die Gre des gesamten Bildes fest
    //und geben Sie size, origin und anchor in skalierten Werten an. Sobald das
    //MarkerImage einmal konstruiert ist, kann es nicht gendert werden.
    function MarkerImage(const URL: String; Size: TSize = nil; Origin: TPoint = nil; Anchor: TPoint = nil; ScaledSize: TSize = nil): TMarkerImage;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MaxZoomService'}{$ENDIF}
    ///<summary>
    ///  Creates a new instance of a <c>TMaxZoomService</c> that can be used to
    ///  send queries about the maximum zoom level available for satellite
    ///  imagery.
    ///</summary>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomService"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function MaxZoomService: TMaxZoomService;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MVCArray'}{$ENDIF}
    ///<summary>
    ///  A mutable MVC Array.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function MVCArray: TMVCArray; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MVCArray'}{$ENDIF}
    ///<summary>
    ///  A mutable MVC Array.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function MVCArray(const Elements: array of OleVariant): TMVCArray; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MVCObject'}{$ENDIF}
    ///<summary>
    ///  Base class implementing KVO.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function MVCObject: TMVCObject;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OverlayView'}{$ENDIF}
    ///<summary>
    ///  Das TOverlayView-Objekt enthlt abstrakte Methoden und kann deshalb nicht auf
    ///  diese Art und Weise erstellt werden. Abgeleitede Objekte mssen die abstrakten
    ///  Methoden berschreiben und den Konstruktor von TOverlayView direkt aufrufen:
    ///
    ///  type
    ///  MyOverlay = class(TOverlayView)
    ///  protected
    ///    procedure Draw; override;
    ///    procedure OnAdd; override;
    ///    procedure OnRemove; override;
    ///  public
    ///    constructor Create(eigene Parameter); reintroduce;
    ///  end;
    ///
    ///  constructor MyOverlay.Create(eigene Parameter);
    ///  begin
    ///    inherited Create;
    ///    ...
    ///  end;
    ///
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    //function OverlayView: TOverlayView;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Point'}{$ENDIF}
    ///<summary>
    ///  Erstellt einen Punkt mit den Koordinaten <i>X</i> und <i>Y</i>.<br/><br/>
    ///  A point on a two-dimensional plane.
    ///</summary>
    ///<seealso cref="gmBase|TPoint"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Point(X, Y: Double): TPoint;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Polygon'}{$ENDIF}
    ///<summary>
    ///  Create a polygon using the passed <c>TPolygonOption</c>, which specify
    ///  the polygon's path, the stroke style for the polygon's edges, and the
    ///  fill style for the polygon's interior regions. A polygon may contain
    ///  one or more paths, where each path consists of an array of
    ///  <c>TLatLng</c>'s. You may pass either an array of <c>TLatLng</c>'s
    ///  or an <c>TMVCArray</c> of <c>TLatLng</c>'s when constructing these
    ///  paths. Arrays are converted to <c>TMVCArray</c>'s within the polygon
    ///  upon instantiation.
    ///</summary>
    ///<seealso cref="gmOverlays.PolylinePolygon|TPolygon"/>
    ///<seealso cref="gmOverlays.PolylinePolygon|TPolygonOptions"/>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmMVC|TMVCArray"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Polygon(Opts: TPolygonOptions = nil): TPolygon;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Polyline'}{$ENDIF}
    ///<summary>
    ///  Create a polyline using the passed <i>TPolylineOption</i>, which specify both the
    ///  path of the polyline and the stroke style to use when drawing the polyline.
    ///  You may pass either an array of <i>TLatLng</i>'s or an <i>TMVCArray</i> of
    ///  <i>LatLng</i>'s when constructing a polyline, though simple arrays are converted
    ///  to <i>TMVCArray</i>'s within the polyline upon instantiation.
    ///</summary>
    ///<seealso cref="gmOverlays.PolylinePolygon|TPolyline"/>
    ///<seealso cref="gmOverlays.PolylinePolygon|TPolygonOptions"/>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmMVC|TMVCArray"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Polyline(Opts: TPolylineOptions = nil): TPolyline;
    function Rectangle(Opts: TRectangleOptions = nil): TRectangle;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Size'}{$ENDIF}
    ///<summary>
    ///  Erstellt ein Objekt mit der angegebenen Breite und Hhe in den Einheiten
    ///  <i>WidthUnit</i> und <i>HeightUnit</i>.<br/><br/>
    ///  Two-dimensonal size, where <i>width</i> is the distance on the x-axis, and
    ///  <i>height</i> is the distance on the y-axis.
    ///</summary>
    ///<param name="WidthUnit">optional</param>
    ///<param name="HeightUnit">optional</param>
    ///<seealso cref="gmBase|TSize"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Size(Width, Height: Double; const WidthUnit: String = ''; const HeightUnit: String = ''): TSize;
    function StreetViewPanorama(Opts: TStreetViewPanoramaOptions = nil): TStreetViewPanorama; overload;
    function StreetViewPanorama(Container: TNode; Opts: TStreetViewPanoramaOptions = nil): TStreetViewPanorama; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StreetViewService'}{$ENDIF}
    ///<summary>
    ///  A <c>TStreetViewService</c> object performs searches for Street View
    ///  data.
    ///</summary>
    ///<seealso cref="gmStreetView|TStreetViewService"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function StreetViewService: TStreetViewService;
    function StyledMapType(Styles: TMapTypeStyleArray; Opts: TStyledMapTypeOptions = nil): TStyledMapType;
    function TrafficLayer: TTrafficLayer;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Adsense'}{$ENDIF}
    ///<summary>
    ///  Library.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdsense"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Adsense: TAdsense read GetAdsense;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Animation'}{$ENDIF}
    ///<summary>
    ///  Identifiers used to specify the placement of controls on the map.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Animation: TAnimation read GetAnimation;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ControlPosition'}{$ENDIF}
    ///<summary>
    ///  Identifiers used to specify the placement of controls on the map.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ControlPosition: TControlPosition read GetControlPosition;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DirectionsStatus'}{$ENDIF}
    ///<summary>
    ///  The status returned by the <c>TDirectionsService</c> on the completion
    ///  of a call to <c>Route()</c>.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsService"/>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DirectionsStatus: TDirectionsStatus read GetDirectionsStatus;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DirectionsTravelMode'}{$ENDIF}
    ///<summary>
    ///  The valid unit systems that can be specified in a
    ///  <c>TDirectionsRequest</c>.
    ///</summary>
    ///<remarks>
    ///  Deprecated: Use <c>TTravelMode</c>.
    ///</remarks>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    ///<seealso cref="gmServices.Directions|TDirectionsTravelMode"/>
    ///<seealso cref="gmServices.Directions|TTravelMode"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DirectionsTravelMode: TDirectionsTravelMode read GetDirectionsTravelMode; //deprecated;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DirectionsUnitSystem'}{$ENDIF}
    ///<summary>
    ///  The valid unit systems that can be specified in a
    ///  <c>TDirectionsRequest</c>.
    ///</summary>
    ///<remarks>
    ///  Deprecated: Use <c>TUnitMode</c>.
    ///</remarks>
    ///<seealso cref="gmServices.Directions|TDirectionsUnitSystem"/>
    ///<seealso cref="gmServices.Directions|TUnitSystem"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DirectionsUnitSystem: TDirectionsUnitSystem read GetDirectionsUnitSystem; //deprecated;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DistanceMatrixStatus'}{$ENDIF}
    ///<summary>
    ///  The top-level status about the request in general returned by the
    ///  DistanceMatrixService upon completion of a distance matrix request.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixService"/>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DistanceMatrixStatus: TDistanceMatrixStatus read GetDistanceMatrixStatus;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ElevationStatus'}{$ENDIF}
    ///<summary>
    ///  The status returned by the <c>TElevationService</c> upon completion of
    ///  an elevation request.
    ///</summary>
    ///<seealso cref="gmServices.Elevation|TElevationService"/>
    ///<seealso cref="gmServices.Elevation|TElevationStatus"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ElevationStatus: TElevationStatus read GetElevationStatus;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Event'}{$ENDIF}
    ///<summary>
    ///  Namespace.
    ///</summary>
    ///<seealso cref="gmEvents|TEvent"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Event: TEvent read GetEvent;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GeocoderLocationType'}{$ENDIF}
    ///<summary>
    ///  Describes the type of location returned from a geocode.
    ///</summary>
    ///<seealso cref="gmServices.Geocoder|TGeocoder"/>
    ///<seealso cref="gmServices.Geocoder|TGeocoderLocationType"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property GeocoderLocationType: TGeocoderLocationType read GetGeocoderLocationType;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GeocoderStatus'}{$ENDIF}
    ///<summary>
    ///  The status returned by the Geocoder on the completion of a call to
    ///  <c>Geocode()</c>.
    ///</summary>
    ///<seealso cref="gmServices.Geocoder|TGeocoder"/>
    ///<seealso cref="gmServices.Geocoder|TGeocoderStatus"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property GeocoderStatus: TGeocoderStatus read GetGeocoderStatus;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Geometry'}{$ENDIF}
    ///<summary>
    ///  Library.
    ///</summary>
    ///<seealso cref="gmLibraries.Geometry|TGeometry"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Geometry: TGeometry read GetGeometry;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MapTypeControlStyle'}{$ENDIF}
    ///<summary>
    ///  Identifiers for common MapTypesControls.
    ///</summary>
    ///<seealso cref="gmControls|TMapTypeControlStyle"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MapTypeControlStyle: TMapTypeControlStyle read GetMapTypeControlStyle;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MapTypeID'}{$ENDIF}
    ///<summary>
    ///  Identifiers for common MapTypes.
    ///</summary>
    ///<seealso cref="gmMap|TMapTypeID"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MapTypeID: TMapTypeID read GetMapTypeID;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation NavigationControlStyle'}{$ENDIF}
    ///<remarks>
    ///  <note type="important">
    ///    Deprecated since version v3.3.<br/>
    ///    Use <see cref="gmControls|TZoomControlStyle">TZoomControlStyle</see>.
    ///  </note>
    ///</remarks>
    ///<seealso cref="gmControls|TNavigationControlStyle"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property NavigationControlStyle: TNavigationControlStyle read GetNavigationControlStyle; //deprecated
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MaxZoomStatus'}{$ENDIF}
    ///<summary>
    ///  The status returned by the MaxZoomService on the completion of a call
    ///  to <c>GetMaxZoomAtLatLng</c>.
    ///</summary>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomStatus"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MaxZoomStatus: TMaxZoomStatus read GetMaxZoomStatus;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Panoramio'}{$ENDIF}
    ///<summary>
    ///  Library.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramio"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Panoramio: TPanoramio read GetPanoramio;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Places'}{$ENDIF}
    ///<summary>
    ///  Library.
    ///</summary>
    ///<seealso cref="gmLibraries.Places|TPlaces"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Places: TPlaces read GetPlaces;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ScaleControlStyle'}{$ENDIF}
    ///<summary>
    ///  Identifiers for scale control IDs.
    ///</summary>
    ///<seealso cref="gmControls|TScaleControlStyle"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ScaleControlStyle: TScaleControlStyle read GetScaleControlStyle;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StreetViewStatus'}{$ENDIF}
    ///<summary>
    ///  The status returned by the <c>TStreetViewService</c> on completion of a Street
    ///  View request.
    ///</summary>
    ///<seealso cref="gmStreetView|TStreetViewStatus"/>
    ///<seealso cref="gmStreetView|TStreetViewService"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property StreetViewStatus: TStreetViewStatus read GetStreetViewStatus;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation TravelMode'}{$ENDIF}
    ///<summary>
    ///  The valid travel modes that can be specified in a <c>TDirectionsRequest</c>
    ///  as well as the travel modes returned in a <c>TDirectionsStep</c>.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    ///<seealso cref="gmServices.Directions|TTravelMode"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property TravelMode: TTravelMode read GetTravelMode;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UnitSystem'}{$ENDIF}
    ///<summary>
    ///  The valid unit systems that can be specified in a <c>TDirectionsRequest</c>.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    ///<seealso cref="gmServices.Directions|TUnitSystem"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UnitSystem: TUnitSystem read GetUnitSystem;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Version'}{$ENDIF}
    ///<summary>
    ///  API version.
    ///</summary>
    ///<seealso href="http://code.google.com/p/gmaps-api-issues/wiki/JavascriptMapsAPIv3Changelog">
    ///  Javascript Maps API v3 Changelog
    ///</seealso>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Version: String read GetVersion;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZoomControlStyle'}{$ENDIF}
    ///<summary>
    ///  Identifiers for the zoom control.
    ///</summary>
    ///<seealso cref="gmControls|TZoomControlStyle"/>
    ///<seealso cref="gmAPI|TMaps"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZoomControlStyle: TZoomControlStyle read GetZoomControlStyle;
  end;

  TDispObjectEnumerator = class(TObject)
  private
    FIndex: Integer;
    FCollection: TDispObjectCollection;
  public
    constructor Create(Collection: TDispObjectCollection);
    function MoveNext: Boolean;
  end;

  TCircleEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TCircle; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TCircle read GetCurrent;
  end;

  TDirectionsRendererEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TDirectionsRenderer; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TDirectionsRenderer read GetCurrent;
  end;

  TDirectionsServiceEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TDirectionsService; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TDirectionsService read GetCurrent;
  end;

  TDistanceMatrixServiceEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TDistanceMatrixService; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TDistanceMatrixService read GetCurrent;
  end;

  TElevationServiceEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TElevationService; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TElevationService read GetCurrent;
  end;

  TGeocoderEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TGeocoder; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TGeocoder read GetCurrent;
  end;

  TGroundOverlayEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TGroundOverlay; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TGroundOverlay read GetCurrent;
  end;

  TInfoWindowEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TInfoWindow; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TInfoWindow read GetCurrent;
  end;

  TLayerEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TLayer; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TLayer read GetCurrent;
  end;

  TMapEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TMap; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TMap read GetCurrent;
  end;

  TMapTypeEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TMapType; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TMapType read GetCurrent;
  end;

  TMarkerEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TMarker; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TMarker read GetCurrent;
  end;

  TMaxZoomServiceEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TMaxZoomService; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TMaxZoomService read GetCurrent;
  end;

  TPolygonEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TPolygon; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TPolygon read GetCurrent;
  end;

  TPolylineEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TPolyline; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TPolyline read GetCurrent;
  end;

  TRectangleEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TRectangle; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TRectangle read GetCurrent;
  end;

  TStreetViewPanoramaEnumerator = class(TDispObjectEnumerator)
  public
    function GetCurrent: TStreetViewPanorama; {$IFDEF USE_INLINE}inline;{$ENDIF}
    property Current: TStreetViewPanorama read GetCurrent;
  end;

  TDispObjectCollection = class(TNotifyObject)
  private
    FItems: TObjectList;
  protected
    procedure FreeNotify(Sender: TObject); override;
    function GetItem(Index: Integer): TDispObject; virtual;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Delete(Index: Integer);
    function Add(Item: TDispObject): Integer; virtual;
    function Count: Integer;
    function IndexOf(Item: TObject): Integer;
    function Remove(Item: TDispObject): Integer;
  end;

  TCircleCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TCircle; reintroduce;
  public
    function Add(Item: TCircle): Integer; reintroduce;
    function GetEnumerator: TCircleEnumerator;
    property Items[Index: Integer]: TCircle read GetItem; default;
  end;

  TDirectionsRendererCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TDirectionsRenderer; reintroduce;
  public
    function Add(Item: TDirectionsRenderer): Integer; reintroduce;
    function GetEnumerator: TDirectionsRendererEnumerator;
    property Items[Index: Integer]: TDirectionsRenderer read GetItem; default;
  end;

  TDirectionsServiceCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TDirectionsService; reintroduce;
  public
    function Add(Item: TDirectionsService): Integer; reintroduce;
    function GetEnumerator: TDirectionsServiceEnumerator;
    property Items[Index: Integer]: TDirectionsService read GetItem; default;
  end;

  TDistanceMatrixServiceCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TDistanceMatrixService; reintroduce;
  public
    function Add(Item: TDistanceMatrixService): Integer; reintroduce;
    function GetEnumerator: TDistanceMatrixServiceEnumerator;
    property Items[Index: Integer]: TDistanceMatrixService read GetItem; default;
  end;

  TElevationServiceCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TElevationService; reintroduce;
  public
    function Add(Item: TElevationService): Integer; reintroduce;
    function GetEnumerator: TElevationServiceEnumerator;
    property Items[Index: Integer]: TElevationService read GetItem; default;
  end;

  TGeocoderCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TGeocoder; reintroduce;
  public
    function Add(Item: TGeocoder): Integer; reintroduce;
    function GetEnumerator: TGeocoderEnumerator;
    property Items[Index: Integer]: TGeocoder read GetItem; default;
  end;

  TGroundOverlayCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TGroundOverlay; reintroduce;
  public
    function Add(Item: TGroundOverlay): Integer; reintroduce;
    function GetEnumerator: TGroundOverlayEnumerator;
    property Items[Index: Integer]: TGroundOverlay read GetItem; default;
  end;

  TInfoWindowCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TInfoWindow; reintroduce;
  public
    function Add(Item: TInfoWindow): Integer; reintroduce;
    function GetEnumerator: TInfoWindowEnumerator;
    property Items[Index: Integer]: TInfoWindow read GetItem; default;
  end;

  TLayerCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TLayer; reintroduce;
  public
    function Add(Item: TLayer): Integer; reintroduce;
    function GetEnumerator: TLayerEnumerator;
    property Items[Index: Integer]: TLayer read GetItem; default;
  end;

  TMapCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TMap; reintroduce;
  public
    function Add(Item: TMap): Integer; reintroduce;
    function GetEnumerator: TMapEnumerator;
    property Items[Index: Integer]: TMap read GetItem; default;
  end;

  TMapTypeCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TMapType; reintroduce;
  public
    function Add(Item: TMapType): Integer; reintroduce;
    function GetEnumerator: TMapTypeEnumerator;
    property Items[Index: Integer]: TMapType read GetItem; default;
  end;

  TMarkerCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TMarker; reintroduce;
  public
    function Add(Item: TMarker): Integer; reintroduce;
    function GetEnumerator: TMarkerEnumerator;
    property Items[Index: Integer]: TMarker read GetItem; default;
  end;

  TMaxZoomServiceCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TMaxZoomService; reintroduce;
  public
    function Add(Item: TMaxZoomService): Integer; reintroduce;
    function GetEnumerator: TMaxZoomServiceEnumerator;
    property Items[Index: Integer]: TMaxZoomService read GetItem; default;
  end;

  TPolygonCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TPolygon; reintroduce;
  public
    function Add(Item: TPolygon): Integer; reintroduce;
    function GetEnumerator: TPolygonEnumerator;
    property Items[Index: Integer]: TPolygon read GetItem; default;
  end;

  TPolylineCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TPolyline; reintroduce;
  public
    function Add(Item: TPolyline): Integer; reintroduce;
    function GetEnumerator: TPolylineEnumerator;
    property Items[Index: Integer]: TPolyline read GetItem; default;
  end;

  TRectangleCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TRectangle; reintroduce;
  public
    function Add(Item: TRectangle): Integer; reintroduce;
    function GetEnumerator: TRectangleEnumerator;
    property Items[Index: Integer]: TRectangle read GetItem; default;
  end;

  TStreetViewPanoramaCollection = class(TDispObjectCollection)
  protected
    function GetItem(Index: Integer): TStreetViewPanorama; reintroduce;
  public
    function Add(Item: TStreetViewPanorama): Integer; reintroduce;
    function GetEnumerator: TStreetViewPanoramaEnumerator;
    property Items[Index: Integer]: TStreetViewPanorama read GetItem; default;
  end;

  {$IFDEF USE_ANONYMOUS_METHODS}
  TInitMapProc = reference to procedure(Script: TScript);
  {$ENDIF}

  TInitMapParams = record
    Async: Boolean;
    Version: Double;
    Language: String;
    Region: String;
    UseSensor: Boolean;
    Libraries: TGoogleLibraries;
    InitMapProc: Classes.TNotifyEvent;
    {$IFDEF USE_ANONYMOUS_METHODS}
    InitMapProcA: TInitMapProc;
    {$ENDIF}
  end;

  TScript = class(TCustomScript)
  private
    FCallbackHelper: TObject;
    FBackgroundColor: TColor;
    FCircles: TCircleCollection;
    FDirectionsRenderer: TDirectionsRendererCollection;
    FDirectionsServices: TDirectionsServiceCollection;
    FDistanceMatrixServices: TDistanceMatrixServiceCollection;
    FElevationServices: TElevationServiceCollection;
    FGeocoders: TGeocoderCollection;
    FGoogle: TGoogle;
    FGroundOverlays: TGroundOverlayCollection;
    FInfoWindows: TInfoWindowCollection;
    FInitMapParams: TInitMapParams;
    FLayers: TLayerCollection;
    FPolygones: TPolygonCollection;
    FPolylines: TPolylineCollection;
    FMaps: TMapCollection;
    FMapTypes: TMapTypeCollection;
    FMarkers: TMarkerCollection;
    FMaxZoomServices: TMaxZoomServiceCollection;
    FRectangles: TRectangleCollection;
    FReloadType: TReloadType;
    FStreetViewPanoramas: TStreetViewPanoramaCollection;
    FOnAPILoaded: TJScriptFunction;
    procedure HideLoadingPage;
    procedure LoadAPIAsyncInt(InitMapProc: Classes.TNotifyEvent;
                              {$IFDEF USE_ANONYMOUS_METHODS}InitMapProcA: TInitMapProc;{$ENDIF}
                              Language, Region: String; Version: Double; UseSensor: Boolean;
                              Libraries: TGoogleLibraries );
    procedure SetBackgroundColor(Value: TColor);
    procedure OnAPILoadedAsync;
    function GetBackgroundColor: TColor;
    function GetGoogle: TGoogle;
    function LoadAPIInt(InitMapProc: Classes.TNotifyEvent;
                        {$IFDEF USE_ANONYMOUS_METHODS}InitMapProcA: TInitMapProc;{$ENDIF}
                        Language, Region: String; Version: Double; UseSensor: Boolean;
                        Libraries: TGoogleLibraries): Boolean;
  protected
    function TranslateAccelerator(Sender: TObject; Accelerator: Integer): Boolean; override;
    //JavaScript event handler:
    procedure OnAPILoaded;
  public
    constructor Create(Browser: TOleControl); virtual;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    procedure BeforeNavigate(URL: String; var Cancel: WordBool);
    procedure LoadAPIAsync(InitMapProc: Classes.TNotifyEvent = nil;
                           Language: String = DefaultLanguage;
                           Region: String = DefaultRegion;
                           Version: Double = DefaultVersion;
                           UseSensor: Boolean = false;
                           Libraries: TGoogleLibraries = []); overload;
    procedure LoadAPIAsync(InitMapProc: Classes.TNotifyEvent;
                           Libraries: TGoogleLibraries); overload;
    procedure LoadAPIAsync(InitMapProc: Classes.TNotifyEvent;
                           UseSensor: Boolean); overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure LoadAPIAsync(InitMapProc: TInitMapProc;
                           Language: String = DefaultLanguage;
                           Region: String = DefaultRegion;
                           Version: Double = DefaultVersion;
                           UseSensor: Boolean = false;
                           Libraries: TGoogleLibraries = []); overload;
    procedure LoadAPIAsync(InitMapProc: TInitMapProc;
                           Libraries: TGoogleLibraries); overload;
    procedure LoadAPIAsync(InitMapProc: TInitMapProc;
                           UseSensor: Boolean); overload;
    {$ENDIF}
    procedure RegisterObject(ScriptObject: TScriptObject); override;
    procedure Reset; virtual;
    function LoadAPI(InitMapProc: Classes.TNotifyEvent = nil;
                     Language: String = DefaultLanguage;
                     Region: String = DefaultRegion;
                     Version: Double = DefaultVersion;
                     UseSensor: Boolean = false;
                     Libraries: TGoogleLibraries = []): Boolean; overload;
    function LoadAPI(InitMapProc: Classes.TNotifyEvent;
                     Libraries: TGoogleLibraries): Boolean; overload;
    function LoadAPI(InitMapProc: Classes.TNotifyEvent;
                     UseSensor: Boolean): Boolean; overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    function LoadAPI(InitMapProc: TInitMapProc;
                     Language: String = DefaultLanguage;
                     Region: String = DefaultRegion;
                     Version: Double = DefaultVersion;
                     UseSensor: Boolean = false;
                     Libraries: TGoogleLibraries = []): Boolean; overload;
    function LoadAPI(InitMapProc: TInitMapProc;
                     Libraries: TGoogleLibraries): Boolean; overload;
    function LoadAPI(InitMapProc: TInitMapProc;
                     UseSensor: Boolean): Boolean; overload;
    {$ENDIF}
    property Circles: TCircleCollection read FCircles;
    property BackgroundColor: TColor read GetBackgroundColor write SetBackgroundColor;
    property DirectionsRenderer: TDirectionsRendererCollection read FDirectionsRenderer;
    property DirectionsServices: TDirectionsServiceCollection read FDirectionsServices;
    property DistanceMatrixServices: TDistanceMatrixServiceCollection read FDistanceMatrixServices;
    property ElevationServices: TElevationServiceCollection read FElevationServices;
    property Geocoders: TGeocoderCollection read FGeocoders;
    property Google: TGoogle read GetGoogle;
    property GroundOverlays: TGroundOverlayCollection read FGroundOverlays;
    property InfoWindows: TInfoWindowCollection read FInfoWindows;
    property Layers: TLayerCollection read FLayers;
    property Maps: TMapCollection read FMaps;
    property MapTypes: TMapTypeCollection read FMapTypes;
    property Markers: TMarkerCollection read FMarkers;
    property MaxZoomServices: TMaxZoomServiceCollection read FMaxZoomServices;
    property Polygones: TPolygonCollection read FPolygones;
    property Polylines: TPolylineCollection read FPolylines;
    property Rectangles: TRectangleCollection read FRectangles;
    property ReloadType: TReloadType read FReloadType write FReloadType;
    property StreetViewPanoramas: TStreetViewPanoramaCollection read FStreetViewPanoramas;
  end;

procedure CheckMaps(Script: TCustomScript);
procedure CheckScript(Script: TCustomScript);
function HTMLPage(LoadAPI: Boolean; UseSensor: Boolean; DivName: String;
                  APIVersion: String = ''; APILanguage: String = ''; APIRegion: String = '';
                  Libraries: TGoogleLibraries = []): String; overload;
                  {$IFDEF USE_DEPRECATED}
                    deprecated
                    {$IFDEF USE_DEPRECATEDMESSAGE}
                      'Use TScript.LoadAPI or TScript.LoadAPIAsync instead LoadBrowserFromSource';
                    {$ENDIF}
                  {$ENDIF}
function StrToFloatDef(const S: String; Default: Extended; DecimalSeparator: Char): Extended; overload;
function FloatToStr(Value: Extended; DecimalSeparator: Char): String; overload;
function Distance_km(Lat1, Lng1, Lat2, Lng2: Double): Double; overload;
function Distance_km(LatLng1, LatLng2: TLatLng): Double; overload;
function Distance_m(Lat1, Lng1, Lat2, Lng2: Double): Double; overload;
function Distance_m(LatLng1, LatLng2: TLatLng): Double; overload;
function Script(Browser: TOleControl = nil): TScript;
//Syntax-Helper:
function New(Value: TBicyclingLayer): TBicyclingLayer; overload;
function New(Value: TCircle): TCircle; overload;
function New(Value: TDirectionsRenderer): TDirectionsRenderer; overload;
function New(Value: TDirectionsService): TDirectionsService; overload;
function New(Value: TDistanceMatrixService): TDistanceMatrixService; overload;
function New(Value: TElevationService): TElevationService; overload;
function New(Value: TFusionTablesLayer): TFusionTablesLayer; overload;
function New(Value: TGeocoder): TGeocoder; overload;
function New(Value: TGroundOverlay): TGroundOverlay; overload;
function New(Value: TImageMapType): TImageMapType; overload;
function New(Value: TInfoWindow): TInfoWindow; overload;
function New(Value: TKmlLayer): TKmlLayer; overload;
function New(Value: TLatLng): TLatLng; overload;
function New(Value: TLatLngBounds): TLatLngBounds; overload;
function New(Value: TMap): TMap; overload;
function New(Value: TMapTypeRegistry): TMapTypeRegistry; overload;
function New(Value: TMarker): TMarker; overload;
function New(Value: TMarkerImage): TMarkerImage; overload;
function New(Value: TMaxZoomService): TMaxZoomService; overload;
function New(Value: TMVCArray): TMVCArray; overload;
function New(Value: TMVCObject): TMVCObject; overload;
function New(Value: TPoint): TPoint; overload;
function New(Value: TPolygon): TPolygon; overload;
function New(Value: TPolyline): TPolyline; overload;
function New(Value: TRectangle): TRectangle; overload;
function New(Value: TSize): TSize; overload;
function New(Value: TStreetViewPanorama): TStreetViewPanorama; overload;
function New(Value: TStreetViewService): TStreetViewService; overload;
function New(Value: TStyledMapType): TStyledMapType; overload;
function New(Value: TTrafficLayer): TTrafficLayer; overload;

implementation

function Script(Browser: TOleControl): TScript;
var
  CustomScript: TCustomScript;
begin
  Result:=nil;
  CustomScript:=JScriptObjects.Script(Browser);
  if assigned(CustomScript) and (CustomScript is TScript)
    then Result:=TScript(CustomScript);
end;

procedure CheckScript(Script: TCustomScript);
begin
  if not assigned(Script) or not (Script is TScript)
    then raise Exception.Create(ExceptionNoScript);
end;

procedure CheckMaps(Script: TCustomScript);
begin
  CheckScript(Script);
  if not assigned(TScript(Script).Google)
    then raise Exception.Create(ExceptionNoGoogle);
  if not assigned(TScript(Script).Google.Maps)
    then raise Exception.Create(ExceptionNoMaps);
end;

function StrToFloatDef(const S: String; Default: Extended; DecimalSeparator: Char): Extended;
var
  OldDecimalSeparator: Char;
begin
  OldDecimalSeparator:={$IFDEF DELPHIXE_UP}FormatSettings.{$ELSE}SysUtils.{$ENDIF}DecimalSeparator;
  try
    {$IFDEF DELPHIXE_UP}FormatSettings.{$ELSE}SysUtils.{$ENDIF}DecimalSeparator:=DecimalSeparator;
    try
      Result:=SysUtils.StrToFloat(S);
    except
      Result:=Default;
    end;
  finally
    {$IFDEF DELPHIXE_UP}FormatSettings.{$ELSE}SysUtils.{$ENDIF}DecimalSeparator:=OldDecimalSeparator;
  end;
end;

function FloatToStr(Value: Extended; DecimalSeparator: Char): String;
var
  OldDecimalSeparator: Char;
begin
  OldDecimalSeparator:={$IFDEF DELPHIXE_UP}FormatSettings.{$ELSE}SysUtils.{$ENDIF}DecimalSeparator;
  try
    {$IFDEF DELPHIXE_UP}FormatSettings.{$ELSE}SysUtils.{$ENDIF}DecimalSeparator:=DecimalSeparator;
    Result:=SysUtils.FloatToStr(Value);
  finally
    {$IFDEF DELPHIXE_UP}FormatSettings.{$ELSE}SysUtils.{$ENDIF}DecimalSeparator:=OldDecimalSeparator;
  end;
end;

function New(Value: TBicyclingLayer): TBicyclingLayer;
begin
  Result:=Value;
end;

function New(Value: TCircle): TCircle;
begin
  Result:=Value;
end;

function New(Value: TDirectionsRenderer): TDirectionsRenderer;
begin
  Result:=Value;
end;

function New(Value: TDirectionsService): TDirectionsService;
begin
  Result:=Value;
end;

function New(Value: TDistanceMatrixService): TDistanceMatrixService;
begin
  Result:=Value;
end;

function New(Value: TElevationService): TElevationService;
begin
  Result:=Value;
end;

function New(Value: TFusionTablesLayer): TFusionTablesLayer;
begin
  Result:=Value;
end;

function New(Value: TGeocoder): TGeocoder;
begin
  Result:=Value;
end;

function New(Value: TGroundOverlay): TGroundOverlay;
begin
  Result:=Value;
end;

function New(Value: TImageMapType): TImageMapType;
begin
  Result:=Value;
end;

function New(Value: TInfoWindow): TInfoWindow;
begin
  Result:=Value;
end;

function New(Value: TKmlLayer): TKmlLayer;
begin
  Result:=Value;
end;

function New(Value: TLatLng): TLatLng;
begin
  Result:=Value;
end;

function New(Value: TLatLngBounds): TLatLngBounds;
begin
  Result:=Value;
end;

function New(Value: TMap): TMap;
begin
  Result:=Value;
end;

function New(Value: TMapTypeRegistry): TMapTypeRegistry;
begin
  Result:=Value;
end;

function New(Value: TMarker): TMarker;
begin
  Result:=Value;
end;

function New(Value: TMarkerImage): TMarkerImage;
begin
  Result:=Value;
end;

function New(Value: TMaxZoomService): TMaxZoomService;
begin
  Result:=Value;
end;

function New(Value: TMVCArray): TMVCArray;
begin
  Result:=Value;
end;

function New(Value: TMVCObject): TMVCObject;
begin
  Result:=Value;
end;

function New(Value: TPoint): TPoint;
begin
  Result:=Value;
end;

function New(Value: TPolygon): TPolygon;
begin
  Result:=Value;
end;

function New(Value: TPolyline): TPolyline;
begin
  Result:=Value;
end;

function New(Value: TRectangle): TRectangle;
begin
  Result:=Value;
end;

function New(Value: TSize): TSize;
begin
  Result:=Value;
end;

function New(Value: TStreetViewPanorama): TStreetViewPanorama;
begin
  Result:=Value;
end;

function New(Value: TStreetViewService): TStreetViewService;
begin
  Result:=Value;
end;

function New(Value: TStyledMapType): TStyledMapType;
begin
  Result:=Value;
end;

function New(Value: TTrafficLayer): TTrafficLayer;
begin
  Result:=Value;
end;

//Abstand zweier Punkte in km
function Distance_km(Lat1, Lng1, Lat2, Lng2: Double): Double;
var
  dLat, dLng, a, c, d: Double;
begin
  dLat:=(Lat2-Lat1)*Pi/180;
  dLng:=(Lng2-Lng1)*Pi/180;
  a:=sin(dLat/2)*sin(dLat/2)+cos(Lat1*Pi/180)*
     cos(Lat2*Pi/180)*sin(dLng/2)*sin(dLng/2);
  c:=2*arctan(sqrt(a)/sqrt(1-a));
  d:=EarthRadius*c/1000;
  Result:=d;
end;

function Distance_km(LatLng1, LatLng2: TLatLng): Double;
begin
  Result:=0;
  if not assigned(LatLng1) or not assigned(LatLng2)
    then Exit;
  Result:=Distance_km(LatLng1.Lat,LatLng1.Lng,LatLng2.Lat,LatLng2.Lng);
end;

function Distance_m(Lat1, Lng1, Lat2, Lng2: Double): Double;
begin
  Result:=Distance_km(Lat1,Lng1,Lat2,Lng2)*1000;
end;

function Distance_m(LatLng1, LatLng2: TLatLng): Double;
begin
  Result:=Distance_km(LatLng1,LatLng2)*1000;
end;

function GetAPIURL(Language, Region: String; UseSensor: Boolean; Version: Double;
                   Libraries: TGoogleLibraries): String;
var
  Libs: String;
  Options: String;
begin
  Result:='http://maps.google.com/maps/api/js?';
  Options:='';
  if Libraries<>[] then
  begin
    Libs:='';
    if libGeometry in Libraries then
    begin
      if Libs<>'' then
        Libs:=Libs+',';
      Libs:=Libs+'geometry';
    end;
    if libPanoramio in Libraries then
    begin
      if Libs<>'' then
        Libs:=Libs+',';
      Libs:=Libs+'panoramio';
    end;
    if libPlaces in Libraries then
    begin
      if Libs<>'' then
        Libs:=Libs+',';
      Libs:=Libs+'places';
    end;
    if libAdsense in Libraries then
    begin
      if Libs<>'' then
        Libs:=Libs+',';
      Libs:=Libs+'adsense';
    end;
    if Libs<>'' then
    begin
      if Length(Options)>0
        then Options:=Options+'&';
      Options:=Options+'libraries='+Libs;
    end;
  end;
  if Version<>DefaultVersion then
  begin
    if Length(Options)>0
      then Options:=Options+'&';
    Options:=Options+'v='+FloatToStr(Version,'.');
  end;
  if Length(Options)>0
    then Options:=Options+'&';
  Options:=Options+'sensor=';
  if UseSensor
    then Options:=Options+'true'
    else Options:=Options+'false';
  if (Language<>'') and (Language<>DefaultLanguage) then
  begin
    if Length(Options)>0
      then Options:=Options+'&';
    Options:=Options+'language='+Language;
  end;
  if (Region<>'') and (Region<>DefaultRegion) then
  begin
    if Length(Options)>0
      then Options:=Options+'&';
    Options:=Options+'region='+Region;
  end;
  Result:=Result+Options;
end;

const
  LoadingPageDivName = 'Loading';
  AniIndicator =
    'R0lGODlhEAAQAIQAAP///wAAAPDw8IqKiuDg4EZGRnp6egAAAFhYWCQkJKysrL6+vhQUFJycnAQE'+
    'BDY2NmhoaP///////////////////////////////////////////////////////////yH/C05F'+
    'VFNDQVBFMi4wAwEAAAAh/hxEZWxwaGkgR29vZ2xlIE1hcHMgRnJhbWV3b3JrACH5BAkKAB8ALAAA'+
    'AAAQABAAAAV34Cd+Agkh5ah+REIIx0GQq0gcieDIwqwSs8FhsAOmRILCEOZAIF62FEHpWPiQUKRA'+
    '8bgiVYOCs1EjCc7hQoFcO5/NC8MxqhIMGAVvzzoyJBp2A0AECiR8LDMEDwsKDQSCXyMCCAYfDWQE'+
    'gDUEBjOXIgtzNQsLZSEAIfkECQoAHwAsAAAAABAAEAAABXbgJ34CaUBlOY5EQQjJQRCMsorE8cAO'+
    'ES+r2WdwaCQYjUNhJUAgBYzH6XBIqQgInfCzeL1GAoHCdVOJFBCDAXgjhQUNBATCLr/dhIZZFGaO'+
    'EQRMAl8iAwUKcAozQoNXgQQQBAuIXXxMBg1cNpJ7fHqaOJ03WzchACH5BAkKAB8ALAAAAAAQABAA'+
    'AAV44Cd+AtkMZTmOBEQIRSIIj7KKhIPAMnIsq9encVDEFIfChwAUGGq0wmDwOBAGB+sSwkAQCDhi'+
    'FkgSLCAq0eIwGC2mDfBtRlcY7uQ5vUxQpEUzKwIKECiCAkIiDQYLZo17iCpCV18viX8CDUBfJImC'+
    'fktygTdBcjchACH5BAkKAB8ALAAAAAAQABAAAAV24Cd+Atk0ZTmOhCEISPEiyyoSjxsL0FGPr0+D'+
    'sYgtDhASgTSAEAQFw6lwEDQOjFIr97wtEgef6LVwrQjIp2DRUCiWNgiD4aCdGvBVL/wgCQgLKiwE'+
    'hCkEAygrL10ibE+FL4YqQWuRf5OKC0uUjIpdQX42NqA2IQAh+QQJCgAfACwAAAAAEAAQAAAFbOAn'+
    'fgJpGGU5jgQiCMfxHssqEocbCwy9vh8DLdY4MEgEEoKxgD1OO6Hs08oRVIRF7NAQvRauFe5xFRAa'+
    '6KTtsWWeDOpVb0syX1da8t2MVom0MWpnVwRlQHR1SC8vd2A/hYlmfiOSiZY2lJMjIQAh+QQJCgAf'+
    'ACwAAAAAEAAQAAAFfuAnfgK5LGU5joIiCIPxGsQqCoY7DHBSsyUChLAjMAakmmChIMAaJ8NjmSik'+
    'FgOUSkBAHB6/F8G1EhQaLzHBafs0EIjCYL1WrRqJBANC4r5WBA8NdX2EImMHBwVhTmkvDAddBwwp'+
    'KY0tBwgfTGVsfwkHC21pfR8EBw1tZVsrIQAh+QQJCgAfACwAAAAAEAAQAAAFeeAnfgJJEGU5jsIi'+
    'CE3zNsQqwqgiN0XNloSBQEcoKEwm1HBxakBahaeJ+boRIAyE7yVArQQQBXesWikgEIRwbPsoCo+H'+
    'gdStjggIcfV1KhMWBwdSJmMGEAkHYAcPKSkoDAcDB3N/X0qQBIg+X1wOBycOR203CIwkKyEAIfkE'+
    'CQoAHwAsAAAAABAAEAAABXfgJ34CSRBlOY4CKiyLICjEKr4o/EI1WxIxGAGyIKVastNJYWghBkeX'+
    'qmV4GHqy7EowiGW/ts/CQFZ8VasFBPE0urZEbbKFJSQOzZvrcBgMCglcBwVHHwN4CQkNfR9AKwR8'+
    'BAwMAg8HPSsKBzGTLQlFYTUCbCkrIQAh+QQJCgAfACwAAAAAEAAQAAAFeeAnfgIpnOSonidBnEup'+
    'kq/g2o0sorb9DgSTiUVcKGyDnKkmOzUgjeAQNRJEicTZhzDoHrMzgmGsMC0SixWQtS0cDtEdocAY'+
    'lASOw4NwSDQUEAVWCRAlDWkDBwMPDwpwW1IifAkCjAIIDpEjCpkCCQ82BWlad2N3KiEAIfkECQoA'+
    'HwAsAAAAABAAEAAABXngJ34CKZzkqJ4sS5Qq+bbEAouoMJ8LYZo714vQsAFZOIFiYDTpbqQeC8Ko'+
    'QmIfgmK7eBy+BSyxwdQRGb6RoLHLFr7NE+TRKAkSB8i9sB0gBAsIBiUKPgMHDQgFCwwKMioEBw8C'+
    'igIGD2kqCwkvBQU6EJkrJAMDdiohACH5BAkKAB8ALAAAAAAQABAAAAV14Cd+Aimc5KiebFuqJOEK'+
    'skoQcVujwnEMM5lgUVv4DjUTgfg6GQ6KlfAESVghsM9yQUQ4GAxDdquQyRQF3EhQZmkRvuRpAFGU'+
    'BI+DQVBAcBsQQwYNJUQfDVAGEAQPUTQ2DAUCigINaWNpAhAITmowdw2EKSMhACH5BAEKAB8ALAAA'+
    'AAAQABAAAAV54Cd+Aimc5KguDHG+qPotRwILrkoQH3IorxxKkDgMCIdD8OQqEYqHhkK0LJkajoXq'+
    'Jhg8CoWBjLTbGRJosQxXfi0QvJFg0SwJDIeW9aQYLOw+AwIQBjsKDThSTjwsCwOFEDwxVA8QXQYC'+
    'CphjBJGXJw1xayQKQCkjIQA7';

{.$DEFINE IE7_MODUS}
{$DEFINE USE_ONLOADCALLBACK}
{$DEFINE USE_SPLASHSCREEN}
{$DEFINE USE_ANIINDICATOR}                                 //don't work with IE6 (IE7?)

function HTMLPage(LoadAPI: Boolean = true; DivName: String = '';
                  Language: String = DefaultLanguage;
                  Region: String = DefaultRegion;
                  APIVersion: Double = DefaultVersion;
                  UseSensor: Boolean = false;
                  Libraries: TGoogleLibraries = []): String; overload;
begin
  Result:={$IFDEF IE7_MODUS}
          '<!DOCTYPE html>'+sLineBreak+
          {$ENDIF}
          '<html>'+sLineBreak+
          '  <head>'+sLineBreak;
  {$IFDEF IE7_MODUS}
    Result:=Result+
          '    <meta http-equiv="X-UA-Compatible" content="IE=EmulateIE7" />';
  {$ENDIF}
  if LoadAPI then
  begin
    Result:=Result+
          '    <script type="text/javascript" src="'+GetAPIURL(Language,Region,UseSensor,APIVersion,Libraries)+'"></script>'+sLineBreak;
  end else Result:=Result+                                 //sonst wird kein Script-Objekt im Document-Objekt erzeugt
          '    <script type="text/javascript"></script>';  //und das API kann spter nicht asynchron nachgeladen werden
  {$IFDEF USE_ONLOADCALLBACK}
  Result:=Result+
          '    <script type="text/javascript">'+sLineBreak+
          '      function OnLoad() {'+sLineBreak+
          '        if (external.OnLoad) {'+sLineBreak+
          '          external.OnLoad();'+sLineBreak+
          '        }'+sLineBreak+
          '      }'+sLineBreak+
          '    </script>'+sLineBreak;
  {$ENDIF}
  Result:=Result+
          '  </head>'+sLineBreak+
          '  <body style="margin: 0px; padding: 0px" '+{$IFDEF USE_LOADINGPAGE}'onload="OnLoad()"'+{$ENDIF}'>'+sLineBreak;
  {$IFDEF USE_SPLASHSCREEN}
  Result:=Result+
          '    <div id="'+LoadingPageDivName+'" style="width: 100%; height: 100% background-color:#FF0000">'+sLineBreak+
          '      <div style="position: absolute; right: 10px; bottom: 10px;'+
          '                  font-family: Arial; font-size: 15px; font-weight: bolder;'+
          '                  font-style: italic; color: #808080">'+sLineBreak+
          '        Loading...'+sLineBreak+
          '      </div>'+sLineBreak+
          {$IFDEF USE_ANIINDICATOR}
          '      <div style="position: absolute; right: 50%; bottom: 50%">'+sLineBreak+
          '        <img src="data:image/gif;base64,'+AniIndicator+'"/>'+sLineBreak+
          '      </div>'+sLineBreak+
          {$ENDIF}
          '    </div>'+sLineBreak;
  {$ENDIF}
  if DivName<>''
    then Result:=Result+
          '    <div id="'+DivName+'" style="width: 100%; height: 100%"></div>'+sLineBreak;
  Result:=Result+
    '  </body>'+sLineBreak+
    '</html>';
end;

function HTMLPage(LoadAPI: Boolean; UseSensor: Boolean; DivName: String;
                  APIVersion: String; APILanguage, APIRegion: String;
                  Libraries: TGoogleLibraries): String;
begin
  Result:=HTMLPage(LoadAPI,DivName,APILanguage,APIRegion,
                   StrToFloatDef(APIVersion,DefaultVersion,'.'),
                   UseSensor,Libraries);
end;

//------------------------------------------------------------------------------
//TGoogle

destructor TGoogle.Destroy;
begin
  FMaps.Free;
  inherited;
end;

class function TGoogle.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[GoogleProp_Maps]);
end;

function TGoogle.GetMaps: TMaps;
begin
  Result:=TMaps(GetPropAsObject(GoogleProp_Maps,TMaps,FMaps));
end;

//------------------------------------------------------------------------------
//TMaps

destructor TMaps.Destroy;
begin
  FAdsense.Free;
  FAnimation.Free;
  FControlPosition.Free;
  FDirectionsStatus.Free;
  FDirectionsTravelMode.Free;
  FDirectionsUnitSystem.Free;
  FDistanceMatrixStatus.Free;
  FElevationStatus.Free;
  FEvent.Free;
  FGeocoderLocationType.Free;
  FGeocoderStatus.Free;
  FGeometry.Free;
  FMapTypeControlStyle.Free;
  FMapTypeID.Free;
  FMaxZoomStatus.Free;
  FNavigationControlsStyle.Free;
  FPanoramio.Free;
  FPlaces.Free;
  FScaleControlStyle.Free;
  FStreetViewStatus.Free;
  FTravelMode.Free;
  FUnitSystem.Free;
  FZoomControlStyle.Free;
  inherited;
end;

class function TMaps.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MapsMethod_BicyclingLayer,
                                 MapsMethod_Circle,
                                 MapsProp_ControlPosition]);
end;

function TMaps.GetAdsense: TAdsense;
begin
  if not PropertyExists(MapsProp_Adsense)
    then raise Exception.Create(ExceptionNoAdsenseLibraryLoaded);
  Result:=TAdsense(GetPropAsObject(MapsProp_Adsense,TAdsense,FAdsense));
end;

function TMaps.GetAnimation: TAnimation;
begin
  Result:=TAnimation(GetPropAsObject(MapsProp_Animation,TAnimation,FAnimation));
end;

function TMaps.GetControlPosition: TControlPosition;
begin
  Result:=TControlPosition(GetPropAsObject(MapsProp_ControlPosition,TControlPosition,
                                           FControlPosition));
end;

function TMaps.GetDirectionsStatus: TDirectionsStatus;
begin
  Result:=TDirectionsStatus(GetPropAsObject(MapsProp_DirectionsStatus,TDirectionsStatus,
                                            FDirectionsStatus));
end;

function TMaps.GetDirectionsTravelMode: TDirectionsTravelMode;
begin
  Result:=TDirectionsTravelMode(GetPropAsObject(MapsProp_DirectionsTravelMode,
                                                TDirectionsTravelMode,FDirectionsTravelMode));
end;

function TMaps.GetDirectionsUnitSystem: TDirectionsUnitSystem;
begin
  Result:=TDirectionsUnitSystem(GetPropAsObject(MapsProp_DirectionsUnitSystem,
                                                TDirectionsUnitSystem,FDirectionsUnitSystem));
end;

function TMaps.GetDistanceMatrixStatus: TDistanceMatrixStatus;
begin
  Result:=TDistanceMatrixStatus(GetPropAsObject(MapsProp_DistanceMatrixStatus,
                                                TDistanceMatrixStatus,
                                                FDistanceMatrixStatus));
end;

function TMaps.GetElevationStatus: TElevationStatus;
begin
  Result:=TElevationStatus(GetPropAsObject(MapsProp_ElevationStatus,TElevationStatus,
                                           FElevationStatus));
end;

function TMaps.GetEvent: TEvent;
begin
  Result:=TEvent(GetPropAsObject(MapsProp_Event,TEvent,FEvent));
end;

function TMaps.GetGeocoderLocationType: TGeocoderLocationType;
begin
  Result:=TGeocoderLocationType(GetPropAsObject(MapsProp_GeocoderLocationType,
                                                TGeocoderLocationType,FGeocoderLocationType));
end;

function TMaps.GetGeocoderStatus: TGeocoderStatus;
begin
  Result:=TGeocoderStatus(GetPropAsObject(MapsProp_GeocoderStatus,TGeocoderStatus,
                                          FGeocoderStatus));
end;

function TMaps.GetGeometry: TGeometry;
begin
  if not PropertyExists(MapsProp_Geometry)
    then raise Exception.Create(ExceptionNoGeometryLibraryLoaded);
  Result:=TGeometry(GetPropAsObject(MapsProp_Geometry,TGeometry,FGeometry));
end;

function TMaps.GetMapTypeControlStyle: TMapTypeControlStyle;
begin
  Result:=TMapTypeControlStyle(GetPropAsObject(MapsProp_MapTypeControlStyle,
                                               TMapTypeControlStyle,FMapTypeControlStyle));
end;

function TMaps.GetMapTypeID: TMapTypeID;
begin
  Result:=TMapTypeID(GetPropAsObject(MapsProp_MapTypeID,TMapTypeID,FMapTypeID));
end;

function TMaps.GetMaxZoomStatus: TMaxZoomStatus;
begin
  Result:=TMaxZoomStatus(GetPropAsObject(MapsProp_MaxZoomStatus,
                                         TMaxZoomStatus,FMaxZoomStatus));
end;

function TMaps.GetNavigationControlStyle: TNavigationControlStyle;
begin
  Result:=TNavigationControlStyle(GetPropAsObject(MapsProp_NavigationControlStyle,
                                                  TNavigationControlStyle,
                                                  FNavigationControlsStyle));
end;

function TMaps.GetPanoramio: TPanoramio;
begin
  if not PropertyExists(MapsProp_Panoramio)
    then raise Exception.Create(ExceptionNoPanoramioLibraryLoaded);
  Result:=TPanoramio(GetPropAsObject(MapsProp_Panoramio,TPanoramio,FPanoramio));
end;

function TMaps.GetPlaces: TPlaces;
begin
  if not PropertyExists(MapsProp_Places)
    then raise Exception.Create(ExceptionNoPlacesLibraryLoaded);
  Result:=TPlaces(GetPropAsObject(MapsProp_Places,TPlaces,FPlaces));
end;

function TMaps.GetScaleControlStyle: TScaleControlStyle;
begin
  Result:=TScaleControlStyle(GetPropAsObject(MapsProp_ScaleControlStyle,
                                             TScaleControlStyle,FScaleControlStyle));
end;

function TMaps.GetStreetViewStatus: TStreetViewStatus;
begin
  Result:=TStreetViewStatus(GetPropAsObject(MapsProp_StreetViewStatus,
                                            TStreetViewStatus,FStreetViewStatus));
end;

function TMaps.GetTravelMode: TTravelMode;
begin
  Result:=TTravelMode(GetPropAsObject(MapsProp_TravelMode,TTravelMode,FTravelMode));
end;

function TMaps.GetUnitSystem: TUnitSystem;
begin
  Result:=TUnitSystem(GetPropAsObject(MapsProp_UnitSystem,TUnitSystem,FUnitSystem));
end;

function TMaps.GetVersion: String;
begin
  Result:=GetPropAsString(MapsProp_Version);
end;

function TMaps.GetZoomControlStyle: TZoomControlStyle;
begin
  Result:=TZoomControlStyle(GetPropAsObject(MapsProp_ZoomControlStyle,
                                            TZoomControlStyle,FZoomControlStyle));
end;

//Constructors:

function TMaps.BicyclingLayer: TBicyclingLayer;
begin
  Result:=TBicyclingLayer.Create(Script);
end;

function TMaps.Circle(Opts: TCircleOptions): TCircle;
begin
  Result:=TCircle.Create(Script,Opts);
end;

function TMaps.DirectionsRenderer(Opts: TDirectionsRendererOptions): TDirectionsRenderer;
begin
  Result:=TDirectionsRenderer.Create(Script,Opts);
end;

function TMaps.DirectionsService: TDirectionsService;
begin
  Result:=TDirectionsService.Create(Script);
end;

function TMaps.DistanceMatrixService: TDistanceMatrixService;
begin
  Result:=TDistanceMatrixService.Create(Script);
end;

function TMaps.ElevationService: TElevationService;
begin
  Result:=TElevationService.Create(Script);
end;

function TMaps.FusionTablesLayer(Options: TFusionTablesLayerOptions): TFusionTablesLayer;
begin
  Result:=TFusionTablesLayer.Create(Script,Options);
end;

function TMaps.Geocoder: TGeocoder;
begin
  Result:=TGeocoder.Create(Script);
end;

function TMaps.GroundOverlay(const URL: String; Bounds: TLatLngBounds;
                             Opts: TGroundOverlayOptions): TGroundOverlay;
begin
  Result:=TGroundOverlay.Create(Script,URL,Bounds,Opts);
end;

function TMaps.ImageMapType(Opts: TImageMapTypeOptions): TImageMapType;
begin
  Result:=TImageMapType.Create(Script,Opts);
end;

function TMaps.InfoWindow(Opts: TInfoWindowOptions): TInfoWindow;
begin
  Result:=TInfoWindow.Create(Script,Opts);
end;

function TMaps.KmlLayer(const URL: String; Opts: TKmlLayerOptions): TKmlLayer;
begin
  Result:=TKmlLayer.Create(Script,URL,Opts);
end;

function TMaps.LatLng(Lat, Lng: Double; NoWrap: Boolean = true): TLatLng;
begin
  Result:=TLatLng.Create(Script,Lat,Lng,NoWrap);
end;

function TMaps.LatLngBounds(SW, NE: TLatLng): TLatLngBounds;
begin
  Result:=TLatLngBounds.Create(Script,SW,NE);
end;

function TMaps.LatLngBounds(Center: TLatLng; Lat, Lng: Double): TLatLngBounds;
var
  SW, NE: TLatLng;
begin
  SW:=LatLng(Center.Lat-Lat/2,Center.Lng-Lng/2);
  try
    NE:=LatLng(Center.Lat+Lat/2,Center.Lng+Lng/2);
    try
      Result:=LatLngBounds(SW,NE);
    finally
      NE.Free;
    end;
  finally
    SW.Free;
  end;
end;

function TMaps.Map(Opts: TMapOptions): TMap;
begin
  Result:=TMap.Create(Script,Opts);
end;

function TMaps.Map(MapDiv: TNode; Opts: TMapOptions): TMap;
begin
  Result:=TMap.Create(Script,MapDiv,Opts);
end;
{
function TMaps.MapCanvasProjection: TMapCanvasProjection;
begin
//Steht zwar in der API-Dokumentation - ist aber im Maps-Namespace nicht vorhanden!   ToDo!!!
end;
}
function TMaps.MapTypeRegistry: TMapTypeRegistry;
begin
  Result:=TMapTypeRegistry.Create(Script);
end;

function TMaps.MapTypeStyleElementType: TMapTypeStyleElementType;
begin
  Result:=gmMapTypes.MapTypeStyleElementType;
end;

function TMaps.MapTypeStyleFeatureType: TMapTypeStyleFeatureType;
begin
  Result:=gmMapTypes.MapTypeStyleFeatureType;
end;

function TMaps.Marker(Opts: TMarkerOptions): TMarker;
begin
  Result:=TMarker.Create(Script,Opts);
end;

function TMaps.MarkerImage(const URL: String; Size: TSize; Origin: TPoint; Anchor: TPoint; ScaledSize: TSize): TMarkerImage;
begin
  Result:=TMarkerImage.Create(Script,URL,Size,Origin,Anchor,ScaledSize);
end;

function TMaps.MaxZoomService: TMaxZoomService;
begin
  Result:=TMaxZoomService.Create(Script);
end;

function TMaps.MVCArray: TMVCArray;
begin
  Result:=MVCArray.Create(Script);
end;

function TMaps.MVCArray(const Elements: array of OleVariant): TMVCArray;
begin
  Result:=TMVCArray.Create(Script,Elements);
end;

function TMaps.MVCObject: TMVCObject;
begin
  Result:=TMVCObject.Create(Script)
end;
{
function TMaps.OverlayView: TOverlayView;
begin
  Result:=CreateOverlayViewWrapper(Script,CreateOverlayView);
end;
}
function TMaps.Point(X, Y: Double): TPoint;
begin
  Result:=TPoint.Create(Script,X,Y);
end;

function TMaps.Polygon(Opts: TPolygonOptions): TPolygon;
begin
  Result:=TPolygon.Create(Script,Opts);
end;

function TMaps.Polyline(Opts: TPolylineOptions): TPolyline;
begin
  Result:=TPolyline.Create(Script,Opts);
end;

function TMaps.Rectangle(Opts: TRectangleOptions): TRectangle;
begin
  Result:=TRectangle.Create(Script,Opts);
end;

function TMaps.Size(Width, Height: Double; const WidthUnit, HeightUnit: String): TSize;
begin
  Result:=TSize.Create(Script,Width,Height,WidthUnit,HeightUnit);
end;

function TMaps.StreetViewPanorama(Opts: TStreetViewPanoramaOptions): TStreetViewPanorama;
begin
  Result:=TStreetViewPanorama.Create(Script,Opts);
end;

function TMaps.StreetViewPanorama(Container: TNode; Opts: TStreetViewPanoramaOptions): TStreetViewPanorama;
begin
  Result:=TStreetViewPanorama.Create(Script,Container,Opts);
end;

function TMaps.StreetViewService: TStreetViewService;
begin
  Result:=TStreetViewService.Create(Script);
end;

function TMaps.StyledMapType(Styles: TMapTypeStyleArray; Opts: TStyledMapTypeOptions): TStyledMapType;
begin
  Result:=TStyledMapType.Create(Script,Styles,Opts);
end;

function TMaps.TrafficLayer: TTrafficLayer;
begin
  Result:=TTrafficLayer.Create(Script);
end;

//------------------------------------------------------------------------------
//TDispObjectEnumerator

constructor TDispObjectEnumerator.Create(Collection: TDispObjectCollection);
begin
  inherited Create;
  FIndex:=-1;
  FCollection:=Collection;
end;

function TDispObjectEnumerator.MoveNext: Boolean;
begin
  Result:=FIndex<FCollection.Count-1;
  if Result
    then Inc(FIndex);
end;

//------------------------------------------------------------------------------
//TCircleEnumerator

function TCircleEnumerator.GetCurrent: TCircle;
begin
  Result:=TCircle(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TDirectionsRendererEnumerator

function TDirectionsRendererEnumerator.GetCurrent: TDirectionsRenderer;
begin
  Result:=TDirectionsRenderer(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TDirectionsServiceEnumerator

function TDirectionsServiceEnumerator.GetCurrent: TDirectionsService;
begin
  Result:=TDirectionsService(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TDistanceMatrixServiceEnumerator

function TDistanceMatrixServiceEnumerator.GetCurrent: TDistanceMatrixService;
begin
  Result:=TDistanceMatrixService(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TElevationServiceEnumerator

function TElevationServiceEnumerator.GetCurrent: TElevationService;
begin
  Result:=TElevationService(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TGeocoderEnumerator

function TGeocoderEnumerator.GetCurrent: TGeocoder;
begin
  Result:=TGeocoder(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TGroundOverlayEnumerator

function TGroundOverlayEnumerator.GetCurrent: TGroundOverlay;
begin
  Result:=TGroundOverlay(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TInfoWindowEnumerator

function TInfoWindowEnumerator.GetCurrent: TInfoWindow;
begin
  Result:=TInfoWindow(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TLayerEnumerator

function TLayerEnumerator.GetCurrent: TLayer;
begin
  Result:=TLayer(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TMapEnumerator

function TMapEnumerator.GetCurrent: TMap;
begin
  Result:=TMap(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TMapTypeEnumerator

function TMapTypeEnumerator.GetCurrent: TMapType;
begin
  Result:=TMapType(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TMarkerEnumerator

function TMarkerEnumerator.GetCurrent: TMarker;
begin
  Result:=TMarker(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TMaxZoomServiceEnumerator

function TMaxZoomServiceEnumerator.GetCurrent: TMaxZoomService;
begin
  Result:=TMaxZoomService(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TPolygonEnumerator

function TPolygonEnumerator.GetCurrent: TPolygon;
begin
  Result:=TPolygon(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TPolylineEnumerator

function TPolylineEnumerator.GetCurrent: TPolyline;
begin
  Result:=TPolyline(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TRectangleEnumerator

function TRectangleEnumerator.GetCurrent: TRectangle;
begin
  Result:=TRectangle(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TStreetViewPanoramaEnumerator

function TStreetViewPanoramaEnumerator.GetCurrent: TStreetViewPanorama;
begin
  Result:=TStreetViewPanorama(FCollection.GetItem(FIndex));
end;

//------------------------------------------------------------------------------
//TDispObjectCollection

constructor TDispObjectCollection.Create;
begin
  inherited;
  FItems:=TObjectList.Create(true);
end;

destructor TDispObjectCollection.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure TDispObjectCollection.FreeNotify(Sender: TObject);
begin
  FItems.Extract(Sender);
end;

function TDispObjectCollection.Add(Item: TDispObject): Integer;
begin
  Result:=-1;
  if assigned(Item) and (FItems.IndexOf(Item)<0) then
  begin
    Item.AddNotify(Self);
    Result:=FItems.Add(Item);
  end;
end;

procedure TDispObjectCollection.Delete(Index: Integer);
begin
  FItems.Delete(Index);
end;

function TDispObjectCollection.Remove(Item: TDispObject): Integer;
begin
  Result:=FItems.Remove(Item);
end;

procedure TDispObjectCollection.Clear;
begin
  FItems.Clear;
end;

function TDispObjectCollection.Count: Integer;
begin
  Result:=FItems.Count;
end;

function TDispObjectCollection.GetItem(Index: Integer): TDispObject;
begin
  Result:=nil;
  if (Index>=0) and (Index<Count)
    then Result:=TDispObject(FItems[Index]);
end;

function TDispObjectCollection.IndexOf(Item: TObject): Integer;
begin
  Result:=FItems.IndexOf(Item);
end;

//------------------------------------------------------------------------------
//TCircleCollection

function TCircleCollection.Add(Item: TCircle): Integer;
begin
  Result:=inherited Add(Item);
end;

function TCircleCollection.GetEnumerator: TCircleEnumerator;
begin
  Result:=TCircleEnumerator.Create(Self);
end;

function TCircleCollection.GetItem(Index: Integer): TCircle;
begin
  Result:=TCircle(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TDirectionsRendererCollection

function TDirectionsRendererCollection.Add(Item: TDirectionsRenderer): Integer;
begin
  Result:=inherited Add(Item);
end;

function TDirectionsRendererCollection.GetEnumerator: TDirectionsRendererEnumerator;
begin
  Result:=TDirectionsRendererEnumerator.Create(Self);
end;

function TDirectionsRendererCollection.GetItem(Index: Integer): TDirectionsRenderer;
begin
  Result:=TDirectionsRenderer(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TDirectionsServiceCollection

function TDirectionsServiceCollection.Add(Item: TDirectionsService): Integer;
begin
  Result:=inherited Add(Item);
end;

function TDirectionsServiceCollection.GetEnumerator: TDirectionsServiceEnumerator;
begin
  Result:=TDirectionsServiceEnumerator.Create(Self);
end;

function TDirectionsServiceCollection.GetItem(Index: Integer): TDirectionsService;
begin
  Result:=TDirectionsService(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TDistanceMatrixServiceCollection

function TDistanceMatrixServiceCollection.Add(Item: TDistanceMatrixService): Integer;
begin
  Result:=inherited Add(Item);
end;

function TDistanceMatrixServiceCollection.GetEnumerator: TDistanceMatrixServiceEnumerator;
begin
  Result:=TDistanceMatrixServiceEnumerator.Create(Self);
end;

function TDistanceMatrixServiceCollection.GetItem(Index: Integer): TDistanceMatrixService;
begin
  Result:=TDistanceMatrixService(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TElevationServiceCollection

function TElevationServiceCollection.Add(Item: TElevationService): Integer;
begin
  Result:=inherited Add(Item);
end;

function TElevationServiceCollection.GetEnumerator: TElevationServiceEnumerator;
begin
  Result:=TElevationServiceEnumerator.Create(Self);
end;

function TElevationServiceCollection.GetItem(Index: Integer): TElevationService;
begin
  Result:=TElevationService(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TGeocoderCollection

function TGeocoderCollection.Add(Item: TGeocoder): Integer;
begin
  Result:=inherited Add(Item);
end;

function TGeocoderCollection.GetEnumerator: TGeocoderEnumerator;
begin
  Result:=TGeocoderEnumerator.Create(Self);
end;

function TGeocoderCollection.GetItem(Index: Integer): TGeocoder;
begin
  Result:=TGeocoder(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TGroundOverlayCollection

function TGroundOverlayCollection.Add(Item: TGroundOverlay): Integer;
begin
  Result:=inherited Add(Item);
end;

function TGroundOverlayCollection.GetEnumerator: TGroundOverlayEnumerator;
begin
  Result:=TGroundOverlayEnumerator.Create(Self);
end;

function TGroundOverlayCollection.GetItem(Index: Integer): TGroundOverlay;
begin
  Result:=TGroundOverlay(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TLayerCollection

function TLayerCollection.Add(Item: TLayer): Integer;
begin
  Result:=inherited Add(Item);
end;

function TLayerCollection.GetEnumerator: TLayerEnumerator;
begin
  Result:=TLayerEnumerator.Create(Self);
end;

function TLayerCollection.GetItem(Index: Integer): TLayer;
begin
  Result:=TLayer(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TMapCollection

function TMapCollection.Add(Item: TMap): Integer;
begin
  Result:=inherited Add(Item);
end;

function TMapCollection.GetEnumerator: TMapEnumerator;
begin
  Result:=TMapEnumerator.Create(Self);
end;

function TMapCollection.GetItem(Index: Integer): TMap;
begin
  Result:=TMap(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TMapTypeCollection

function TMapTypeCollection.Add(Item: TMapType): Integer;
begin
  Result:=inherited Add(Item);
end;

function TMapTypeCollection.GetEnumerator: TMapTypeEnumerator;
begin
  Result:=TMapTypeEnumerator.Create(Self);
end;

function TMapTypeCollection.GetItem(Index: Integer): TMapType;
begin
  Result:=TMapType(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TMarkerCollection

function TMarkerCollection.Add(Item: TMarker): Integer;
begin
  Result:=inherited Add(Item);
end;

function TMarkerCollection.GetEnumerator: TMarkerEnumerator;
begin
  Result:=TMarkerEnumerator.Create(Self);
end;

function TMarkerCollection.GetItem(Index: Integer): TMarker;
begin
  Result:=TMarker(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TMaxZoomServiceCollection

function TMaxZoomServiceCollection.Add(Item: TMaxZoomService): Integer;
begin
  Result:=inherited Add(Item);
end;

function TMaxZoomServiceCollection.GetEnumerator: TMaxZoomServiceEnumerator;
begin
  Result:=TMaxZoomServiceEnumerator.Create(Self);
end;

function TMaxZoomServiceCollection.GetItem(Index: Integer): TMaxZoomService;
begin
  Result:=TMaxZoomService(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TInfoWindowCollection

function TInfoWindowCollection.Add(Item: TInfoWindow): Integer;
begin
  Result:=inherited Add(Item);
end;

function TInfoWindowCollection.GetEnumerator: TInfoWindowEnumerator;
begin
  Result:=TInfoWindowEnumerator.Create(Self);
end;

function TInfoWindowCollection.GetItem(Index: Integer): TInfoWindow;
begin
  Result:=TInfoWindow(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TPolygoneCollection

function TPolygonCollection.Add(Item: TPolygon): Integer;
begin
  Result:=inherited Add(Item);
end;

function TPolygonCollection.GetEnumerator: TPolygonEnumerator;
begin
  Result:=TPolygonEnumerator.Create(Self);
end;

function TPolygonCollection.GetItem(Index: Integer): TPolygon;
begin
  Result:=TPolygon(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TPolylineCollection

function TPolylineCollection.Add(Item: TPolyline): Integer;
begin
  Result:=inherited Add(Item);
end;

function TPolylineCollection.GetEnumerator: TPolylineEnumerator;
begin
  Result:=TPolylineEnumerator.Create(Self);
end;

function TPolylineCollection.GetItem(Index: Integer): TPolyline;
begin
  Result:=TPolyline(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TRectangleCollection

function TRectangleCollection.Add(Item: TRectangle): Integer;
begin
  Result:=inherited Add(Item);
end;

function TRectangleCollection.GetEnumerator: TRectangleEnumerator;
begin
  Result:=TRectangleEnumerator.Create(Self);
end;

function TRectangleCollection.GetItem(Index: Integer): TRectangle;
begin
  Result:=TRectangle(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TStreetViewPanoramaCollection

function TStreetViewPanoramaCollection.Add(Item: TStreetViewPanorama): Integer;
begin
  Result:=inherited Add(Item);
end;

function TStreetViewPanoramaCollection.GetEnumerator: TStreetViewPanoramaEnumerator;
begin
  Result:=TStreetViewPanoramaEnumerator.Create(Self);
end;

function TStreetViewPanoramaCollection.GetItem(Index: Integer): TStreetViewPanorama;
begin
  Result:=TStreetViewPanorama(inherited GetItem(Index));
end;

//------------------------------------------------------------------------------
//TScript

constructor TScript.Create(Browser: TOleControl);
begin
  inherited;
  FCircles:=TCircleCollection.Create;
  FDirectionsRenderer:=TDirectionsRendererCollection.Create;
  FDirectionsServices:=TDirectionsServiceCollection.Create;
  FDistanceMatrixServices:=TDistanceMatrixServiceCollection.Create;
  FElevationServices:=TElevationServiceCollection.Create;
  FGeocoders:=TGeocoderCollection.Create;
  FGroundOverlays:=TGroundOverlayCollection.Create;
  FInfoWindows:=TInfoWindowCollection.Create;
  FLayers:=TLayerCollection.Create;
  FMaps:=TMapCollection.Create;
  FMapTypes:=TMapTypeCollection.Create;
  FMarkers:=TMarkerCollection.Create;
  FMaxZoomServices:=TMaxZoomServiceCollection.Create;
  FPolygones:=TPolygonCollection.Create;
  FPolylines:=TPolylineCollection.Create;
  FRectangles:=TRectangleCollection.Create;
  FStreetViewPanoramas:=TStreetViewPanoramaCollection.Create;
  FReloadType:=rtDialog;
  FBackgroundColor:=clWhite;
end;

destructor TScript.Destroy;
begin
  FCallbackHelper.Free;
  FOnAPILoaded.Free;
  FCircles.Free;
  FDirectionsRenderer.Free;
  FDirectionsServices.Free;
  FDistanceMatrixServices.Free;
  FElevationServices.Free;
  FGeocoders.Free;
  FGroundOverlays.Free;
  FInfoWindows.Free;
  FLayers.Free;
  FMaps.Free;
  FMapTypes.Free;
  FMarkers.Free;
  FMaxZoomServices.Free;
  FPolygones.Free;
  FPolylines.Free;
  FRectangles.Free;
  FStreetViewPanoramas.Free;
  FGoogle.Free;
  inherited;
end;

class function TScript.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[ScriptProp_Google]);
end;

procedure TScript.BeforeNavigate(URL: String; var Cancel: WordBool);
begin
  if URL<>JScriptObjects.EmptyPage then
  begin
    ShellExecute(Application.MainForm.Handle,'open',
                 {$IFDEF DELPHI2009_UP}PWideChar{$ELSE}PChar{$ENDIF}(URL),
                 nil,nil,SW_SHOWNORMAL);
    Cancel:=true;
  end else Cancel:=false;
end;

function TScript.LoadAPIInt(InitMapProc: Classes.TNotifyEvent;
                            {$IFDEF USE_ANONYMOUS_METHODS}InitMapProcA: TInitMapProc;{$ENDIF}
                            Language, Region: String; Version: Double; UseSensor: Boolean;
                            Libraries: TGoogleLibraries): Boolean;
begin
  FInitMapParams.Async:=false;                             //save for refresh
  FInitMapParams.Version:=Version;
  FInitMapParams.Language:=Language;
  FInitMapParams.Region:=Region;
  FInitMapParams.UseSensor:=UseSensor;
  FInitMapParams.Libraries:=Libraries;
  FInitMapParams.InitMapProc:=InitMapProc;
  {$IFDEF USE_ANONYMOUS_METHODS}
  FInitMapParams.InitMapProcA:=InitMapProcA;
  {$ENDIF}
  Result:=LoadBrowserFromSource(DocHostUIHandler.Browser,  //empty page:
                                HTMLPage(true,             //-with Google Maps API
                                         '',               //-without DIV container
                                         Language,         //-API language
                                         Region,           //-API region
                                         Version,          //-API version
                                         UseSensor,        //-with/without GPS sensor
                                         Libraries),       //-Libraries
                                true);                     //wait until the entire page has loaded
  if Result then
  begin
    Document.Body.Style.BackgroundColor:=FBackgroundColor;
    HideLoadingPage;
    OnAPILoaded;
  end;
end;

function TScript.LoadAPI(InitMapProc: Classes.TNotifyEvent; Language, Region: String;
                         Version: Double; UseSensor: Boolean;
                         Libraries: TGoogleLibraries): Boolean;
begin
  Result:=LoadAPIInt(InitMapProc,{$IFDEF USE_ANONYMOUS_METHODS}nil,{$ENDIF}
                     Language,Region,Version,UseSensor,Libraries);
end;

function TScript.LoadAPI(InitMapProc: Classes.TNotifyEvent; Libraries: TGoogleLibraries): Boolean;
begin
  Result:=LoadAPIInt(InitMapProc,{$IFDEF USE_ANONYMOUS_METHODS}nil,{$ENDIF}
                     DefaultLanguage,DefaultRegion,DefaultVersion,false,Libraries);
end;

function TScript.LoadAPI(InitMapProc: Classes.TNotifyEvent; UseSensor: Boolean): Boolean;
begin
  Result:=LoadAPIInt(InitMapProc,{$IFDEF USE_ANONYMOUS_METHODS}nil,{$ENDIF}
                     DefaultLanguage,DefaultRegion,DefaultVersion,UseSensor,[]);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
function TScript.LoadAPI(InitMapProc: TInitMapProc; Language, Region: String;
                         Version: Double; UseSensor: Boolean;
                         Libraries: TGoogleLibraries): Boolean;
begin
  Result:=LoadAPIInt(nil,InitMapProc,Language,Region,Version,UseSensor,Libraries);
end;

function TScript.LoadAPI(InitMapProc: TInitMapProc; Libraries: TGoogleLibraries): Boolean;
begin
  Result:=LoadAPIInt(nil,InitMapProc,DefaultLanguage,DefaultRegion,DefaultVersion,
                     false,Libraries);
end;

function TScript.LoadAPI(InitMapProc: TInitMapProc; UseSensor: Boolean): Boolean;
begin
  Result:=LoadAPIInt(nil,InitMapProc,DefaultLanguage,DefaultRegion,DefaultVersion,
                     UseSensor,[]);
end;
{$ENDIF}

procedure TScript.LoadAPIAsyncInt(InitMapProc: Classes.TNotifyEvent;
                                  {$IFDEF USE_ANONYMOUS_METHODS}InitMapProcA: TInitMapProc;{$ENDIF}
                                  Language, Region: String; Version: Double; UseSensor: Boolean;
                                  Libraries: TGoogleLibraries );
var
  Node: TScriptNode;
  Source: String;
begin
  FInitMapParams.Async:=true;                              //save for refresh
  FInitMapParams.Version:=Version;
  FInitMapParams.Language:=Language;
  FInitMapParams.Region:=Region;
  FInitMapParams.UseSensor:=UseSensor;
  FInitMapParams.Libraries:=Libraries;
  FInitMapParams.InitMapProc:=InitMapProc;
  {$IFDEF USE_ANONYMOUS_METHODS}
  FInitMapParams.InitMapProcA:=InitMapProcA;
  {$ENDIF}
  if LoadBrowserFromSource(DocHostUIHandler.Browser,       //empty page:
                           HTMLPage(false),                //-without Google Maps API
                           true) then                      //wait until the entire page has loaded
  begin
    Document.Body.Style.BackgroundColor:=FBackgroundColor;
    Node:=Document.CreateScriptNode;
    try
      Node.Type_:='text/javascript';
      Source:=GetAPIURL(Language,Region,UseSensor,Version,Libraries)+'&callback=';
      if assigned(ExternalMethods) then
      begin
        if not assigned(FOnAPILoaded)
          then FOnAPILoaded:=NewFunction(OnAPILoaded,'OnAPILoaded');
        Source:=Source+'external.OnAPILoaded';             //Callback method
      end else Source:=Source+'eval';                      //Dummy funktion: Otherwise the other scripts are not loaded!
      Node.Src:=Source;                                    //JavaScript
      Document.Head.AppendChild(Node);
    finally
      Node.Free;
    end;
  end;
end;

procedure TScript.LoadAPIAsync(InitMapProc: Classes.TNotifyEvent; Language, Region: String;
                               Version: Double; UseSensor: Boolean;
                               Libraries: TGoogleLibraries);
begin
  LoadAPIAsyncInt(InitMapProc,{$IFDEF USE_ANONYMOUS_METHODS}nil,{$ENDIF}
                  Language,Region,Version,UseSensor,Libraries);
end;

procedure TScript.LoadAPIAsync(InitMapProc: Classes.TNotifyEvent; Libraries: TGoogleLibraries);
begin
  LoadAPIAsyncInt(InitMapProc,{$IFDEF USE_ANONYMOUS_METHODS}nil,{$ENDIF}
                  DefaultLanguage,DefaultRegion,DefaultVersion,false,Libraries);
end;

procedure TScript.LoadAPIAsync(InitMapProc: Classes.TNotifyEvent; UseSensor: Boolean);
begin
  LoadAPIAsyncInt(InitMapProc,{$IFDEF USE_ANONYMOUS_METHODS}nil,{$ENDIF}
                  DefaultLanguage,DefaultRegion,DefaultVersion,UseSensor,[]);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TScript.LoadAPIAsync(InitMapProc: TInitMapProc; Language, Region: String;
                               Version: Double; UseSensor: Boolean;
                               Libraries: TGoogleLibraries);
begin
  LoadAPIAsyncInt(nil,InitMapProc,Language,Region,Version,UseSensor,Libraries);
end;

procedure TScript.LoadAPIAsync(InitMapProc: TInitMapProc; Libraries: TGoogleLibraries);
begin
  LoadAPIAsyncInt(nil,InitMapProc,DefaultLanguage,DefaultRegion,DefaultVersion,
                  false,Libraries);
end;

procedure TScript.LoadAPIAsync(InitMapProc: TInitMapProc; UseSensor: Boolean);
begin
  LoadAPIAsyncInt(nil,InitMapProc,DefaultLanguage,DefaultRegion,DefaultVersion,
                  UseSensor,[]);
end;
{$ENDIF}

const
  WM_APILOADED = WM_USER+10;

type
  TCallbackHelper = class(TMessageHelper)
  private
    FScript: TScript;
    procedure WMAPILoaded(var Meg: TMessage); message WM_APILOADED;
  public
    constructor Create(Script: TScript);
  end;

constructor TCallbackHelper.Create(Script: TScript);
begin
  inherited Create;
  FScript:=Script;
end;

procedure TCallbackHelper.WMAPILoaded(var Meg: TMessage);
begin
  if assigned(FScript)
    then FScript.OnAPILoadedAsync;
end;

procedure TScript.HideLoadingPage;
var
  LoadingPageDiv: TNode;
begin
  LoadingPageDiv:=Document.GetElementByID(LoadingPageDivName);
  if assigned(LoadingPageDiv) then
    with LoadingPageDiv.Style do
    begin
      Visibility:='hidden';
      Width:='0px';
      Height:='0px';
    end;
end;

procedure TScript.OnAPILoaded;
begin
  HideLoadingPage;
  if not assigned(FCallbackHelper)
    then FCallbackHelper:=TCallbackHelper.Create(Self);
  PostMessage(TCallbackHelper(FCallbackHelper).Handle,
              WM_APILOADED,0,0);                           //separate from JavaScript-Thread
end;

procedure TScript.OnAPILoadedAsync;
begin
  {$IFDEF USE_ANONYMOUS_METHODS}
  if assigned(FInitMapParams.InitMapProcA)
    then FInitMapParams.InitMapProcA(Self)
    else
  {$ENDIF}
  if assigned(FInitMapParams.InitMapProc)
    then FInitMapParams.InitMapProc(Self);
end;

procedure TScript.RegisterObject(ScriptObject: TScriptObject);
begin
  if assigned(ScriptObject) then
  begin
    if ScriptObject is TCircle
      then FCircles.Add(TCircle(ScriptObject));
    if ScriptObject is TDirectionsRenderer
      then FDirectionsRenderer.Add(TDirectionsRenderer(ScriptObject));
    if ScriptObject is TDirectionsService
      then FDirectionsServices.Add(TDirectionsService(ScriptObject));
    if ScriptObject is TDistanceMatrixService
      then FDistanceMatrixServices.Add(TDistanceMatrixService(ScriptObject));
    if ScriptObject is TElevationService
      then FElevationServices.Add(TElevationService(ScriptObject));
    if ScriptObject is TGeocoder
      then FGeocoders.Add(TGeocoder(ScriptObject));
    if ScriptObject is TGroundOverlay
      then FGroundOverlays.Add(TGroundOverlay(ScriptObject));
    if ScriptObject is TInfoWindow
      then FInfoWindows.Add(TInfoWindow(ScriptObject));
    if ScriptObject is TLayer
      then FLayers.Add(TLayer(ScriptObject));
    if ScriptObject is TMap
      then FMaps.Add(TMap(ScriptObject));
    if ScriptObject is TMapType
      then FMapTypes.Add(TMapType(ScriptObject));
    if ScriptObject is TMarker
      then FMarkers.Add(TMarker(ScriptObject));
    if ScriptObject is TMaxZoomService
      then FMaxZoomServices.Add(TMaxZoomService(ScriptObject));
    if ScriptObject is TPolygon
      then FPolygones.Add(TPolygon(ScriptObject));
    if ScriptObject is TPolyline
      then FPolylines.Add(TPolyline(ScriptObject));
    if ScriptObject is TRectangle
      then FRectangles.Add(TRectangle(ScriptObject));
    if ScriptObject is TStreetViewPanorama
      then FStreetViewPanoramas.Add(TStreetViewPanorama(ScriptObject));
  end;
end;

procedure TScript.Reset;
begin
  //delete all objects:
  FCircles.Clear;
  FDirectionsRenderer.Clear;
  FDirectionsServices.Clear;
  FDistanceMatrixServices.Clear;
  FElevationServices.Clear;
  FGeocoders.Clear;
  FGroundOverlays.Clear;
  FInfoWindows.Clear;
  FLayers.Clear;
  FMaps.Clear;
  FMapTypes.Clear;
  FMarkers.Clear;
  FMaxZoomServices.Clear;
  FPolygones.Clear;
  FPolylines.Clear;
  FRectangles.Clear;
  FStreetViewPanoramas.Clear;
  CallbackManager.CleanUp;
  FreeAndNil(FGoogle);
  //load API and create map:
  with FInitMapParams do
  begin
    if not Async
      then LoadAPIInt(InitMapProc,{$IFDEF USE_ANONYMOUS_METHODS}InitMapProcA,{$ENDIF}
                      Language,Region,Version,UseSensor,Libraries)
      else LoadAPIAsyncInt(InitMapProc,{$IFDEF USE_ANONYMOUS_METHODS}InitMapProcA,{$ENDIF}
                           Language,Region,Version,UseSensor,Libraries);
  end;
end;

function TScript.TranslateAccelerator(Sender: TObject; Accelerator: Integer): Boolean;
begin
  case Accelerator of
    VK_F5:
    begin
      if assigned(FInitMapParams.InitMapProc)
         {$IFDEF USE_ANONYMOUS_METHODS}or assigned(FInitMapParams.InitMapProcA){$ENDIF} then
      begin
        case FReloadType of
          rtNone  : ;
          rtAuto  : Reset;
          rtDialog: if Application.MessageBox(PChar(ReloadMap),PChar(Application.MainForm.Caption),
                                              MB_YESNO or MB_ICONQUESTION)=IDYES
                      then Reset;
        end;
      end;
      Result:=true;
    end
    else Result:=false;
  end;
end;

function TScript.GetBackgroundColor: TColor;
begin
  if assigned(Document) and assigned(Document.Body)
    then FBackgroundColor:=Document.Body.Style.BackgroundColor;
  Result:=FBackgroundColor;
end;

procedure TScript.SetBackgroundColor(Value: TColor);
begin
  FBackgroundColor:=Value;
  if assigned(Document) and assigned(Document.Body)
    then Document.Body.Style.BackgroundColor:=FBackgroundColor;
end;

function TScript.GetGoogle: TGoogle;
var
  Google: OleVariant;
begin
  //Geht hier nicht, da TScript nur von TDispObject abgeleitet ist und nicht von
  //TScriptObject. Daher ist sie Methode GetPropAsObject nicht berschrieben und
  //beim neu erstellten Objekt TGoogle wird die Script-Eigenschaft nicht gesetzt:
  //Result:=TGoogle(GetPropAsObject(ScriptProp_Google,TGoogle,FGoogle));
  Result:=nil;
  Google:=GetPropAsVariant(ScriptProp_Google);
  if VarIsValidDispatch(Google) then
  begin
    if assigned(FGoogle)
      then FGoogle.Disp:=Google
      else FGoogle:=TGoogle.Create(self,Google);
    Result:=FGoogle;
  end;
end;

end.


