///<summary>
///  Diese Unit enthlt grundlegende Objekte des
///  <see href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html">
///  Google Maps API</see>'s.<br/><br/>
///  This unit contains basic objects of the
///  <see href="http://code.google.com/apis/maps/documentation/javascript/reference.html">
///  Google Maps API</see>.
///  <para/>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmBase|TLatLng"/></item>
///    <item><see cref="gmBase|TLatLng"/></item>
///    <item><see cref="gmBase|TLatLngBounds"/></item>
///    <item><see cref="gmBase|TPoint"/></item>
///    <item><see cref="gmBase|TSize"/></item>
///  </list>
///  <para/>
///  <para><b>Additional Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmBase|TLatLngArray"/></item>
///    <item><see cref="gmBase|TLatLngArrayArray"/></item>
///  </list>
///</summary>

//geogr. Breite: Latitude (Lat) -90(Sdpol)..0(quator)..90(Nordpol)
//geogr. Lnge: Longitude (Lng) -180..180 (+Ost, -West)

unit gmBase;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} BrowserTools,
  DispObject, JScriptObjects, gmAdditionalConstants, gmAdditionalMessages;

type
  TLatLng = class;
  TLatLngArray = class;
  TLatLngArrayArray = class;
  TLatLngBounds = class;
  TPoint = class;
  TSize = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TLatLng'}{$ENDIF}
  ///<summary>
  ///  <i>TLatLng</i> ist ein Punkt, der durch die geografische Breite und Lnge definiert ist.<br/><br/>
  ///  <i>TLatLng</i> is a point in geographical coordinates, latitude and longitude.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmBase"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmAPI|TMaps.LatLng(Double,Double,Boolean)">Google.Maps.LatLng</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TLatLng.Equals(TLatLng)">Equals</see></item>
  ///    <item><see cref="gmBase|TLatLng.Lat">Lat</see></item>
  ///    <item><see cref="gmBase|TLatLng.Lng">Lng</see></item>
  ///    <item><see cref="gmBase|TLatLng.ToUrlValue(Integer)">ToUrlValue</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  Die Koordinaten eines Punktes knnen nicht gendert werden.<br/><br/>
  ///  Notice that although usual map projections associate longitude with the x-coordinate
  ///  of the map, and latitude with the y-coordinate, the latitude coordinate is always
  ///  written first, followed by the longitude.<br/>
  ///  Notice also that you cannot modify the coordinates of a <i>TLatLng</i>. If you want
  ///  to compute another point, you have to create a new one.
  ///</remarks>
  ///<example>
  ///  <code lang="Delphi">
  ///  with Script do
  ///    Oxford:=New(Google.Maps.LatLng(34.3664951,-89.5192484));</code>
  ///</example>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#LatLng">
  ///  LatLng (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#LatLng">
  ///  LatLng (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TLatLng = class(TCustomJScriptObject)
  protected
    class function New(Script: TCustomScript; Lat, Lng: Double; NoWrap: Boolean): IDispatch; reintroduce;
  public
    constructor Create(Lat, Lng: Double; NoWrap: Boolean = true); reintroduce; overload;
    constructor Create(Script: TCustomScript; Lat, Lng: Double; NoWrap: Boolean = true); reintroduce; overload;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Equals'}{$ENDIF}
    ///<summary>
    ///  Vergleicht diesen Punkt mit einem anderen.<br/><br/>
    ///  Comparison function.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Equals(Other: TLatLng): Boolean; reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Lat'}{$ENDIF}
    ///<summary>
    ///  Gibt die geografische Breite in Grad zurck.<br/><br/>
    ///  Returns the latitude in degrees.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Lat: Double;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Lng'}{$ENDIF}
    ///<summary>
    ///  Gibt die geografische Lnge in Grad zurck.<br/><br/>
    ///  Returns the longitude in degrees.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Lng: Double;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ToString'}{$ENDIF}
    ///<summary>
    ///  Gibt eine Darstellung dieses Punktes als Zeichenfolge zurck.<br/><br/>
    ///  Converts to string representation..
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ToString: String; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ToUrlValue'}{$ENDIF}
    ///<summary>
    ///  Gibt einen String der Form 'lat,lng' fr dieses LatLng-Objekt
    ///  zurck. Die Werte der Lngen- und Breitengrade werden standardmig auf
    ///  sechs Dezimalstellen gerundet.<br/><br/>
    ///  Returns a string of the form 'lat,lng' for this LatLng. We round the
    ///  lat/lng values to 6 decimal places by default.
    ///</summary>
    ///<param name="Precision">optional, default 6</param>
    ///<seealso cref="gmBase|TLatLng"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ToUrlValue(Precision: Integer = 6): String;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TLatLngArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;LatLng&gt;<br/><br/>
  ///  Klasse zur Verwaltung von <see cref="gmBase|TLatLng"></see>-Objekten.<br/><br/>
  ///  Class to manage <see cref="gmBase|TLatLng"></see>-objects.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmBase"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TLatLngArray.Add(TLatLng)">Add(LatLng)</see></item>
  ///    <item><see cref="gmBase|TLatLngArray.Add(Double,Double)">Add(Lat,Lng)</see></item>
  ///    <item><see cref="gmBase|TLatLngArray.Insert(Integer,TLatLng)">Insert(Index,LatLng)</see></item>
  ///    <item><see cref="gmBase|TLatLngArray.Insert(Integer,Double,Double)">Insert(Index,Lat,Lng)</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TLatLngArray.Items(Integer)">Items</see></item>
  ///    <item><see cref="gmBase|TLatLngArray.MaxPoints">MaxPoints</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="note">
  ///    Im Google Maps API nicht vorhanden.<br/><br/>
  ///    Not available in the Google Maps API.
  ///  </note>
  ///</remarks>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TLatLngArray = class(TObjectArray)
  private
    FMaxPoints: Integer;
    procedure SetItem(Index: Integer; Value: TLatLng);
    function GetItem(Index: Integer): TLatLng;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Insert'}{$ENDIF}
    ///<summary>
    ///  Fgt der Liste an der Position <i>Index</i> ein mit den bergebenen
    ///  Werten <i>Lat</i> und <i>Lng</i> neu erstelltes <see cref="gmBase|TLatLng">
    ///  TLatLng </see>-Objekt hinzu.<br/><br/>
    ///  Add a new <see cref="gmBase|TLatLng">TLatLng</see> object at
    ///  position <i>Index</i> with the Values <i>Lat</i> and <i>Lng</i>.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Insert(Index: Integer; Lat, Lng: Double); reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Insert'}{$ENDIF}
    ///<summary>
    ///  Fgt der Liste an der Position <i>Index</i> ein <see cref="gmBase|TLatLng">
    ///  TLatLng</see>-Objekt hinzu.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Insert(Index: Integer; LatLng: TLatLng); reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<summary>
    ///  Fgt der Liste am Ende ein mit den bergebenen Werten <i>Lat</i> und
    ///  <i>Lng</i> neu erstelltes <see cref="gmBase|TLatLng">TLatLng</see>-
    ///  Objekt hinzu.
    ///</summary>
    ///<returns>
    ///  Index des neu hinzugefgten <see cref="gmBase|TLatLng">TLatLng</see>-
    ///  Objektes.
    ///</returns>
    ///<seealso cref="gmBase|TLatLngArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(Lat, Lng: Double): Integer; reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<summary>
    ///  Fgt der Liste am Ende ein <see cref="gmBase|TLatLng">TLatLng</see>-
    ///  Objekt hinzu.
    ///</summary>
    ///<returns>
    ///  Index des neu hinzugefgten <see cref="gmBase|TLatLng">TLatLng</see>-
    ///  Objektes.
    ///</returns>
    ///<seealso cref="gmBase|TLatLngArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(LatLng: TLatLng): Integer; reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Liste mit den enthaltenen <see cref="gmBase|TLatLng">TLatLng</see>-Objekten.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TLatLng read GetItem write SetItem; default;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MaxPoints'}{$ENDIF}
    ///<summary>
    ///  Maximale Anzahl an <see cref="gmBase|TLatLng">TLatLng</see>-Objekten,
    ///  die die Liste enthalten kann.
    ///</summary>
    ///<remarks>
    ///  Betrgt der Wert 0 (Standard), knnen beliebig viele Werte aufgenommen
    ///  werden.
    ///</remarks>
    ///<seealso cref="gmBase|TLatLngArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MaxPoints: Integer read FMaxPoints write FMaxPoints;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TLatLngArrayArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;Array.&lt;LatLng&gt;&gt;<br/><br/>
  ///  Klasse zur Verwaltung von <see cref="gmBase|TLatLngArray"></see>-Objekten.<br/><br/>
  ///  Class to manage <see cref="gmBase|TLatLngArray"></see>-objects.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmBase"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TLatLngArrayArray.Add(TLatLngArray)">Add</see></item>
  ///    <item><see cref="gmBase|TLatLngArrayArray.Insert(Integer,TLatLngArray)">Insert</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TLatLngArrayArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="note">
  ///    Im Google Maps API nicht vorhanden.<br/><br/>
  ///    Not available in the Google Maps API.
  ///  </note>
  ///</remarks>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TLatLngArrayArray = class(TObjectArray)
  private
    procedure SetItem(Index: Integer; Value: TLatLngArray);
    function GetItem(Index: Integer): TLatLngArray;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Insert'}{$ENDIF}
    ///<summary>
    ///  Fgt der Liste an der Position <i>Index</i> ein <see cref="gmBase|TLatLngArray">
    ///  TLatLngArray</see>-Objekt hinzu.<br/><br/>
    ///  Add a new <see cref="gmBase|TLatLngArray">TLatLngArray</see> object at
    ///  position <i>Index</i>.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArrayArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Insert(Index: Integer; LatLngArray: TLatLngArray); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<summary>
    ///  Fgt der Liste am Ende ein neu erstelltes
    ///  <see cref="gmBase|TLatLngArray">TLatLngArray</see>-Objekt hinzu.
    ///</summary>
    ///<returns>
    ///  Index des hinzugefgten <see cref="gmBase|TLatLngArray">TLatLngArray</see>-
    ///  Objektes.<br/><br/>
    ///  Index of added <see cref="gmBase|TLatLngArray">TLatLngArray</see>
    ///  object.<br/><br/>
    ///</returns>
    ///<seealso cref="gmBase|TLatLngArrayArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(LatLngArray: TLatLngArray): Integer; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Liste mit den enthaltenen <see cref="gmBase|TLatLngArray">TLatLngArray</see>
    /// -Objekten.<br/><br/>
    ///  List of all contained <see cref="gmBase|TLatLngArray">TLatLngArray</see> objects.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArrayArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TLatLngArray read GetItem write SetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TLatLngBounds'}{$ENDIF}
  ///<summary>
  ///  Eine <i>TLatLngBounds</i>-Objekt stellt ein Rechteck in geographischen Koordinaten dar,
  ///  einschlielich einer Koordinate, die jenseits des 180-Grad-Meridians liegt.<br/><br/>
  ///  A <i>TLatLngBounds</i> instance represents a rectangle in geographical coordinates,
  ///  including one that crosses the 180 degrees longitudinal meridian.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmBase"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmAPI|TMaps.LatLngBounds(TLatLng,TLatlng)">Google.Maps.LatLngBounds</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TLatLngBounds.Contains(TLatLng)">Contains</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.Equals(TLatLngBounds)">Equals</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.Extend(TLatLng)">Extend</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.GetCenter">GetCenter</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.GetNorthEast">GetNorthEast</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.GetSouthWest">GetSouthWest</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.Intersects(TLatLngBounds)">Intersects</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.IsEmpty">IsEmpty</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.ToSpan">ToSpan</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.ToString">ToString</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.ToUrlValue(Integer)">ToUrlValue</see></item>
  ///    <item><see cref="gmBase|TLatLngBounds.Union(TLatLngBounds)">Union</see></item>
  ///  </list>
  ///</summary>
  ///<example>
  ///  <code lang="Delphi">
  ///  with Script do
  ///  begin
  ///    Oxford:=New(Google.Maps.LatLng(51.7523,-1.2559));
  ///    Bedford:=New(Google.Maps.LatLng(52.1357,-0.468));
  ///    Bounds:=New(Google.Maps.LatLngBounds(Oxford,Bedford));
  ///  end;</code>
  ///</example>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#LatLngBounds">
  ///  LatLngBounds (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#LatLngBounds">
  ///  LatLngBounds (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TLatLngBounds = class(TCustomJScriptObject)
  private
    FCenter: TLatLng;
    FExtend: TLatLngBounds;
    FNorthEast: TLatLng;
    FSouthWest: TLatLng;
    FSpan: TLatLng;
    FUnion: TLatLngBounds;
  protected
    class function New(Script: TCustomScript; SW, NE: TLatLng): IDispatch; reintroduce;
  public
    constructor Create(SW: TLatLng = nil; NE: TLatLng = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; SW: TLatLng = nil; NE: TLatLng = nil); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Contains'}{$ENDIF}
    ///<summary>
    ///  Gibt <i>true</i> zurck, falls die angegebene geografische Breite/Lnge
    ///  innerhalb dieser Grenzen liegt.<br/><br/>
    ///  Returns <i>true</i> if the given lat/lng is in this bounds.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Contains(LatLng: TLatLng): Boolean;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Equals'}{$ENDIF}
    ///<summary>
    ///  Gibt <i>true</i> zurck, wenn dieser Auswahlbereich ungefhr den
    ///  angegebenen Grenzen entspricht.<br/><br/>
    ///  Returns <i>true</i> if this bounds approximately equals the given bounds.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Equals(Other: TLatLngBounds): Boolean; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Extend'}{$ENDIF}
    ///<summary>
    ///  Erweitert diese Grenzen so, dass sie den angegebenen Punkt einschlieen.<br/><br/>
    ///  Extends this bounds to contain the given point.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
   {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Extend(Point: TLatLng): TLatLngBounds;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetCenter'}{$ENDIF}
    ///<summary>
    ///  Berechnet den Mittelpunkt dieses <i>TLatLngBounds</i>-Objekts.<br/><br/>
    ///  Computes the center of this <i>TLatLngBounds</i>.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetCenter: TLatLng;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetNorthEast'}{$ENDIF}
    ///<summary>
    ///  Gibt die nordstliche Ecke dieser Grenzen zurck.<br/><br/>
    ///  Returns the north-east corner of this bounds.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetNorthEast: TLatLng;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetSouthWest'}{$ENDIF}
    ///<summary>
    ///  Gibt die sdwestliche Ecke dieser Grenzen zurck.<br/><br/>
    ///  Returns the south-west corner of this bounds.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetSouthWest: TLatLng;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Intersects'}{$ENDIF}
    ///<summary>
    ///  Gibt <i>true</i> zurck, wenn dieser Auswahlbereich Punkte mit diesen
    ///  Grenzen gemeinsam hat.<br/><br/>
    ///  Returns <i>true</i> if this bounds shares any points with this bounds.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Intersects(Other: TLatLngBounds): Boolean;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation IsEmpty'}{$ENDIF}
    ///<summary>
    ///  Gibt <i>true</i> zurckgegeben, wenn die Grenzen leer sind.<br/><br/>
    ///  Returns <i>true</i> if the bounds are empty.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function IsEmpty: Boolean;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ToSpan'}{$ENDIF}
    ///<summary>
    ///  Konvertiert die Breite und Hhe in einen <i>TLatLng</i>-Wert.<br/><br/>
    ///  Converts the given map bounds to a lat/lng span.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ToSpan: TLatLng;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ToString'}{$ENDIF}
    ///<summary>
    ///  Gibt eine Darstellung dieses Rechtecks als Zeichenfolge zurck.<br/><br/>
    ///  Converts to string.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ToString: String; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ToUrlValue'}{$ENDIF}
    ///<summary>
    ///  Gibt eine Zeichenkette der Form <i>lat_lo,lng_lo,lat_hi,lng_hi</i> fr die
    ///  Grenzen zurck, wobei <i>lo</i> der sdwestlichen und <i>hi</i> der
    ///  nordstlichen Ecke des Rechtecks entspricht.<br/><br/>
    ///  Returns a string of the form <i>lat_lo,lng_lo,lat_hi,lng_hi</i> for this
    ///  bounds, where <i>lo</i> corresponds to the southwest corner of the bounding
    ///  box, while <i>hi</i> corresponds to the northeast corner of that box.
    ///</summary>
    ///<param name="Precision">optional, default 6</param>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ToUrlValue(Precision: Integer = 6): String;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation'}{$ENDIF}
    ///<summary>
    ///  Erweitert das Rechteck, so dass es die Verbindung seiner bisherigen und
    ///  der angegebenen Grenzen enthlt.<br/><br/>
    ///  Extends this bounds to contain the union of this and the given bounds.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Union(Other: TLatLngBounds): TLatLngBounds;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPoint'}{$ENDIF}
  ///<summary>
  ///  <i>TPoint</i> ist ein Punkt auf einer Ebene.<br/><br/>
  ///  A point on a two-dimensional plane.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmBase"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmAPI|TMaps.Point(Double,Double)">Google.Maps.Point</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TPoint.Equals(TPoint)">Equals</see></item>
  ///    <item><see cref="gmBase|TPoint.ToString">ToString</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TPoint.X">X</see></item>
  ///    <item><see cref="gmBase|TPoint.Y">Y</see></item>
  ///  </list>
  ///</summary>
  ///<example>
  ///  <code lang="Delphi">
  ///  with Script do
  ///    MyPoint:=New(Google.Maps.Point(10,20));</code>
  ///</example>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#Point">
  ///  Point (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Point">
  ///  Point (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPoint = class(TCustomJScriptObject)
  private
    procedure SetX(Value: Double);
    procedure SetY(Value: Double);
    function GetX: Double;
    function GetY: Double;
  protected
    class function New(Script: TCustomScript; X, Y: Double): IDispatch; reintroduce;
  public
    constructor Create(X, Y: Double); reintroduce; overload;
    constructor Create(Script: TCustomScript; X, Y: Double); reintroduce; overload;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Equals'}{$ENDIF}
    ///<summary>
    ///  Vergleicht diesen Punkt mit einem anderen.<br/><br/>
    ///  Compares two Points.
    ///</summary>
    ///<seealso cref="gmBase|TPoint"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Equals(Other: TPoint): Boolean; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ToString'}{$ENDIF}
    ///<summary>
    ///  Gibt eine Darstellung dieses Punktes als Zeichenfolge zurck.<br/><br/>
    ///  Returns a string representation of this Point.
    ///</summary>
    ///<seealso cref="gmBase|TPoint"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ToString: String; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation x'}{$ENDIF}
    ///<summary>
    ///  X-Koordinate des Punktes.<br/><br/>
    ///  The X coordinate.
    ///</summary>
    ///<seealso cref="gmBase|TPoint"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property X: Double read GetX write SetX;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation y'}{$ENDIF}
    ///<summary>
    ///  Y-Koordinate des Punktes.<br/><br/>
    ///  The Y coordinate.
    ///</summary>
    ///<seealso cref="gmBase|TPoint"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Y: Double read GetY write SetY;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TSize'}{$ENDIF}
  ///<summary>
  ///  <i>TSize</i> ist die Gre eines Objekte.<br/><br/>
  ///  Two-dimensonal size, where <i>width</i> is the distance on the x-axis,
  ///  and <i>height</i> is the distance on the y-axis.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmBase"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmAPI|TMaps.Size(Double,Double,String,String)">Google.Maps.Size</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TSize.Equals(TSize)">Equals</see></item>
  ///    <item><see cref="gmBase|TSize.ToString">ToString</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmBase|TSize.Height">Height</see></item>
  ///    <item><see cref="gmBase|TSize.Width">Width</see></item>
  ///  </list>
  ///</summary>
  ///<example>
  ///  <code lang="Delphi">
  ///  with Script do
  ///    MySize:=New(Google.Maps.Size(100,70));</code>
  ///</example>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#Size">
  ///  Size (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Size">
  ///  Size (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TSize = class(TCustomJScriptObject)
  private
    procedure SetHeight(Value: Double);
    procedure SetWidth(Value: Double);
    function GetHeight: Double;
    function GetWidth: Double;
  protected
    class function New(Script: TCustomScript; Width, Height: Double; WidthUnit: String; HeightUnit: String): IDispatch; reintroduce;
  public
    constructor Create(Width, Height: Double; WidthUnit: String = ''; HeightUnit: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; Width, Height: Double; WidthUnit: String = ''; HeightUnit: String = ''); reintroduce; overload;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Equals'}{$ENDIF}
    ///<summary>
    ///  Vergleicht diese Gre mit einem anderen.<br/><br/>
    ///  Compares two Sizes.
    ///</summary>
    ///<seealso cref="gmBase|TSize"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Equals(Other: TSize): Boolean; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ToString'}{$ENDIF}
    ///<summary>
    ///  Gibt eine Darstellung dieser Gre als Zeichenfolge zurck.<br/><br/>
    ///  Returns a string representation of this Size.
    ///</summary>
    ///<seealso cref="gmBase|TSize"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ToString: String; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Height'}{$ENDIF}
    ///<summary>
    ///  Hhe entlang der Y-Achse in Pixeln.<br/><br/>
    ///  The height along the y-axis, in pixels.
    ///</summary>
    ///<seealso cref="gmBase|TSize"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Height: Double read GetHeight write SetHeight;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Width'}{$ENDIF}
    ///<summary>
    ///  Breite entlang der X-Achse in Pixeln.<br/><br/>
    ///  The width along the x-axis, in pixels.
    ///</summary>
    ///<seealso cref="gmBase|TSize"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Width: Double read GetWidth write SetWidth;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TLatLng

constructor TLatLng.Create(Lat, Lng: Double; NoWrap: Boolean = true);
begin
  Create(JScriptObjects.Script,Lat,Lng,NoWrap);
end;

constructor TLatLng.Create(Script: TCustomScript; Lat, Lng: Double; NoWrap: Boolean = true);
begin
  inherited Create(Script,New(Script,Lat,Lng,NoWrap));
end;

class function TLatLng.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[LatLngMethod_Equals,
                                 LatLngMethod_Lat,
                                 LatLngMethod_Lng]);
end;

class function TLatLng.New(Script: TCustomScript; Lat, Lng: Double; NoWrap: Boolean): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                              MapsMethod_LatLng,
                                              {$IFDEF CAN_OVERLOAD}
                                              [Lat,Lng,NoWrap]
                                              {$ELSE}
                                              Lat,Lng,NoWrap,false
                                              {$ENDIF});
end;

function TLatLng.Equals(Other: TLatLng): Boolean;
begin
  Result:=VarAsBoolean(Exec(LatLngMethod_Equals,Other));
end;

function TLatLng.Lat: Double;
begin
  Result:=VarAsDouble(Exec(LatLngMethod_Lat));
end;

function TLatLng.Lng: Double;
begin
  Result:=VarAsDouble(Exec(LatLngMethod_Lng));
end;

function TLatLng.ToString: String;
begin
  Result:=VarAsString(Exec(LatLngMethod_ToString));
end;

function TLatLng.ToUrlValue(Precision: Integer = 6): String;
begin
  Result:=VarAsString(Exec(LatLngMethod_ToUrlValue,[Precision]));
end;

//------------------------------------------------------------------------------
//TLatLngArray

function TLatLngArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TLatLng.Create(Script,Dispatch);
end;

function TLatLngArray.GetItem(Index: Integer): TLatLng;
begin
  Result:=TlatLng(inherited Items[Index]);
end;

procedure TLatLngArray.SetItem(Index: Integer; Value: TLatLng);
begin
  inherited Items[Index]:=Value;
end;

procedure TLatLngArray.Insert(Index: Integer; LatLng: TLatLng);
begin
  if (FMaxPoints>0) and (Count=FMaxPoints)
    then raise Exception.Create(ExceptionToManyPoints);
  inherited Insert(Index,LatLng);
end;

procedure TLatLngArray.Insert(Index: Integer; Lat, Lng: Double);
var
  LatLng: TLatLng;
begin
  LatLng:=TLatLng.Create(Lat,Lng);
  try
    Insert(Index,LatLng);
  finally
    LatLng.Free;
  end;
end;

function TLatLngArray.Add(LatLng: TLatLng): Integer;
begin
  if (FMaxPoints>0) and (Count=FMaxPoints)
    then raise Exception.Create(ExceptionToManyPoints);
  Result:=inherited Add(LatLng);
end;

function TLatLngArray.Add(Lat, Lng: Double): Integer;
begin
  Result:=Count;
  Insert(Result,Lat,Lng);
end;

//------------------------------------------------------------------------------
//TLatLngArrayArray

function TLatLngArrayArray.Add(LatLngArray: TLatLngArray): Integer;
begin
  Result:=inherited Add(LatLngArray);
end;

function TLatLngArrayArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TLatLngArray.Create(Script,Dispatch);
end;

function TLatLngArrayArray.GetItem(Index: Integer): TLatLngArray;
begin
  Result:=TlatLngArray(inherited Items[Index]);
end;

procedure TLatLngArrayArray.Insert(Index: Integer; LatLngArray: TLatLngArray);
begin
  inherited Insert(Index,LatLngArray);
end;

procedure TLatLngArrayArray.SetItem(Index: Integer; Value: TLatLngArray);
begin
    inherited Items[Index]:=Value;
end;

//------------------------------------------------------------------------------
//TLatLngBounds

constructor TLatLngBounds.Create(SW, NE: TLatLng);
begin
  Create(JScriptObjects.Script,SW,NE);
end;

constructor TLatLngBounds.Create(Script: TCustomScript; SW, NE: TLatLng);
begin
  inherited Create(Script,New(Script,SW,NE));
end;

destructor TLatLngBounds.Destroy;
begin
  FCenter.Free;
  FExtend.Free;
  FNorthEast.Free;
  FSouthWest.Free;
  FSpan.Free;
  FUnion.Free;
  inherited;
end;

class function TLatLngBounds.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[LatLngBoundsMethod_Contains,
                                 LatLngBoundsMethod_Equals,
                                 LatLngBoundsMethod_Extend,
                                 LatLngBoundsMethod_GetCenter]);
end;

class function TLatLngBounds.New(Script: TCustomScript; SW, NE: TLatLng): IDispatch;
var
  SWDisp, NEDisp: IDispatch;
begin
  CheckMaps(Script);
  if assigned(SW)
    then SWDisp:=SW.Disp
    else SWDisp:=nil;
  if assigned(NE)
    then NEDisp:=NE.Disp
    else NEDisp:=nil;
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_LatLngBounds,
                              {$IFDEF CAN_OVERLOAD}
                              [SWDisp,NEDisp]
                              {$ELSE}
                              SWDisp,NEDisp
                              {$ENDIF}
                              ,true);
end;

function TLatLngBounds.Contains(LatLng: TLatLng): Boolean;
begin
  Result:=VarAsBoolean(Exec(LatLngBoundsMethod_Contains,LatLng));
end;

function TLatLngBounds.Equals(Other: TLatLngBounds): Boolean;
begin
  Result:=VarAsBoolean(Exec(LatLngBoundsMethod_Equals,Other));
end;

function TLatLngBounds.Extend(Point: TLatLng): TLatLngBounds;
begin
  Result:=TLatLngBounds(GetObjectWithExec(LatLngBoundsMethod_Extend,
                                          TLatLngBounds,Point,FExtend));
end;

function TLatLngBounds.GetCenter: TLatLng;
begin
  Result:=TLatLng(GetObjectWithExec(LatLngBoundsMethod_GetCenter,TLatLng,FCenter));
end;

function TLatLngBounds.GetNorthEast: TLatLng;
begin
  Result:=TLatLng(GetObjectWithExec(LatLngBoundsMethod_GetNorthEast,TLatLng,
                                    FNorthEast));
end;

function TLatLngBounds.GetSouthWest: TLatLng;
begin
  Result:=TLatLng(GetObjectWithExec(LatLngBoundsMethod_GetSouthWest,TLatLng,
                                    FSouthWest));
end;

function TLatLngBounds.Intersects(Other: TLatLngBounds): Boolean;
begin
  Result:=VarAsBoolean(Exec(LatLngBoundsMethod_Intersects,Other));
end;

function TLatLngBounds.IsEmpty: Boolean;
begin
  Result:=VarAsBoolean(Exec(LatLngBoundsMethod_IsEmpty));
end;

function TLatLngBounds.ToSpan: TLatLng;
begin
  Result:=TLatLng(GetObjectWithExec(LatLngBoundsMethod_ToSpan,TLatLng,FSpan));
end;

function TLatLngBounds.ToString: String;
begin
  Result:=VarAsString(Exec(LatLngBoundsMethod_ToString));
end;

function TLatLngBounds.ToUrlValue(Precision: Integer): String;
begin
  Result:=VarAsString(Exec(LatLngBoundsMethod_ToUrlValue,[Precision]));
end;

function TLatLngBounds.Union(Other: TLatLngBounds): TLatLngBounds;
begin
  Result:=TLatLngBounds(GetObjectWithExec(LatLngBoundsMethod_Union,
                                          TLatLngBounds,FUnion));
end;

//------------------------------------------------------------------------------
//TPoint

constructor TPoint.Create(X, Y: Double);
begin
  Create(JScriptObjects.Script,X,Y);
end;

constructor TPoint.Create(Script: TCustomScript; X, Y: Double);
begin
  inherited Create(Script,New(Script,X,Y));
end;

class function TPoint.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[PointMethod_Equals,
                                 PointProp_X,
                                 PointProp_Y]);
end;

class function TPoint.New(Script: TCustomScript; X, Y: Double): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_Point,
                              {$IFDEF CAN_OVERLOAD}
                              [X,Y]
                              {$ELSE}
                              X,Y
                              {$ENDIF}
                              ,true);
end;

function TPoint.Equals(Other: TPoint): Boolean;
begin
  Result:=VarAsBoolean(Exec(PointMethod_Equals,Other));
end;

function TPoint.ToString: String;
begin
  Result:=VarAsString(Exec(PointMethod_ToString));
end;

function TPoint.GetX: Double;
begin
  Result:=GetPropAsDouble(PointProp_X);
end;

procedure TPoint.SetX(Value: Double);
begin
  SetProp(PointProp_X,Value);
end;

function TPoint.GetY: Double;
begin
  Result:=GetPropAsDouble(PointProp_Y);
end;

procedure TPoint.SetY(Value: Double);
begin
  SetProp(PointProp_Y,Value);
end;

//------------------------------------------------------------------------------
//TSize

constructor TSize.Create(Width, Height: Double; WidthUnit: String = ''; HeightUnit: String = '');
begin
  Create(JScriptObjects.Script,Width,Height,WidthUnit,HeightUnit);
end;

constructor TSize.Create(Script: TCustomScript; Width, Height: Double; WidthUnit: String = ''; HeightUnit: String = '');
begin
  inherited Create(Script,New(Script,Width,Height,WidthUnit,HeightUnit));
end;

class function TSize.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[SizeMethod_Equals,
                                 SizeProp_Height,
                                 SizeProp_Width]);
end;

class function TSize.New(Script: TCustomScript; Width, Height: Double;
                         WidthUnit, HeightUnit: String): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_Size,
                              {$IFDEF CAN_OVERLOAD}
                              [Width,Height,WidthUnit,HeightUnit]
                              {$ELSE}
                              Width,Height,WidthUnit,HeightUnit
                              {$ENDIF}
                              ,true);
end;

function TSize.Equals(Other: TSize): Boolean;
begin
  Result:=VarAsBoolean(Exec(SizeMethod_Equals,Other));
end;

function TSize.ToString: String;
begin
  Result:=VarAsString(Exec(SizeMethod_ToString));
end;

function TSize.GetHeight: Double;
begin
  Result:=GetPropAsDouble(SizeProp_Height);
end;

procedure TSize.SetHeight(Value: Double);
begin
  SetProp(SizeProp_Height,Value);
end;

function TSize.GetWidth: Double;
begin
  Result:=GetPropAsDouble(SizeProp_Width);
end;

procedure TSize.SetWidth(Value: Double);
begin
  SetProp(SizeProp_Width,Value);
end;

end.

