///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmControls|TControlPosition"/></item>
///    <item><see cref="gmControls|TMapTypeControlOptions"/></item>
///    <item><see cref="gmControls|TMapTypeControlStyle"/></item>
///    <item><see cref="gmControls|TOverviewMapControlOptions"/></item>
///    <item><see cref="gmControls|TPanControlOptions"/></item>
///    <item><see cref="gmControls|TRotateControlOptions"/></item>
///    <item><see cref="gmControls|TScaleControlOptions"/></item>
///    <item><see cref="gmControls|TScaleControlStyle"/></item>
///    <item><see cref="gmControls|TStreetViewControlOptions"/></item>
///    <item><see cref="gmControls|TZoomControlOptions"/></item>
///    <item><see cref="gmControls|TZoomControlStyle"/></item>
///  </list>
///  <para/>
///  <para><b>Deprecated Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmControls|TNavigationControlOptions"/></item>
///    <item><see cref="gmControls|TNavigationControlStyle"/></item>
///  </list>
///</summary>
unit gmControls;

interface

{$INCLUDE gmConfig.inc}

uses
  SysUtils, JScriptObjects, gmAdditionalConstants;

type
  TControlPosition = class;
  TMapTypeControlOptions = class;
  TMapTypeControlStyle = class;
  TNavigationControlOptions = class;                       //deprecated
  TNavigationControlStyle = class;                         //deprecated
  TOverviewMapControlOptions = class;
  TPanControlOptions = class;
  TRotateControlOptions = class;
  TScaleControlOptions = class;
  TScaleControlStyle = class;
  TStreetViewControlOptions = class;
  TZoomControlOptions = class;
  TZoomControlStyle = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TControlPosition'}{$ENDIF}
  ///<summary>
  ///  Identifiers used to specify the placement of controls on the map. Controls
  ///  are positioned relative to other controls in the same layout position.
  ///  Controls that are added first are positioned closer to the edge of the map.<br/>
  ///  <c>+----------------+</c><br/>
  ///  <c>+&#160;TL&#160;&#160;&#160;&#160;TC&#160;&#160;&#160;&#160;TR +</c><br/>
  ///  <c>+&#160;LT&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;RT&#160;+</c><br/>
  ///  <c>+&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;+</c><br/>
  ///  <c>+&#160;LC&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;RC&#160;+</c><br/>
  ///  <c>+&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;+</c><br/>
  ///  <c>+&#160;LB&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;RB&#160;+</c><br/>
  ///  <c>+&#160;BL&#160;&#160;&#160;&#160;BC&#160;&#160;&#160;&#160;BR&#160;+</c><br/>
  ///  <c>+----------------+</c><br/>
  ///  Elements in the top or bottom row flow towards the middle. Elements at the
  ///  left or right sides flow downwards.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmControls|TControlPosition.Bottom">Bottom</see> (deprecated)</item>
  ///    <item><see cref="gmControls|TControlPosition.BottomCenter">BottomCenter</see></item>
  ///    <item><see cref="gmControls|TControlPosition.BottomLeft">BottomLeft</see></item>
  ///    <item><see cref="gmControls|TControlPosition.BottomRight">BottomRight</see></item>
  ///    <item><see cref="gmControls|TControlPosition.Left">Left</see> (deprecated)</item>
  ///    <item><see cref="gmControls|TControlPosition.LeftBottom">LeftBottom</see></item>
  ///    <item><see cref="gmControls|TControlPosition.LeftCenter">LeftCenter</see></item>
  ///    <item><see cref="gmControls|TControlPosition.LeftTop">LeftTop</see></item>
  ///    <item><see cref="gmControls|TControlPosition.Right">Right</see> (deprecated)</item>
  ///    <item><see cref="gmControls|TControlPosition.RightBottom">RightBottom</see></item>
  ///    <item><see cref="gmControls|TControlPosition.RightCenter">RightCenter</see></item>
  ///    <item><see cref="gmControls|TControlPosition.RightTop">RightTop</see></item>
  ///    <item><see cref="gmControls|TControlPosition.Top">Top</see> (deprecated)</item>
  ///    <item><see cref="gmControls|TControlPosition.TopCenter">TopCenter</see></item>
  ///    <item><see cref="gmControls|TControlPosition.TopLeft">TopLeft</see></item>
  ///    <item><see cref="gmControls|TControlPosition.TopRight">TopRight</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmAPI|TMaps.ControlPosition">Google.Maps.ControlPosition</seealso>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#ControlPosition">
  ///  ControlPosition (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#ControlPosition">
  ///  ControlPosition (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TControlPosition = class(TCustomJScriptObject)
  private
    procedure SetBottom(Value: Integer);
    procedure SetBottomCenter(Value:Integer);
    procedure SetBottomLeft(Value: Integer);
    procedure SetBottomRight(Value: Integer);
    procedure SetLeft(Value: Integer);
    procedure SetLeftBottom(Value: Integer);
    procedure SetLeftCenter(Value: Integer);
    procedure SetLeftTop(Value: Integer);
    procedure SetRight(Value: Integer);
    procedure SetRightBottom(Value: Integer);
    procedure SetRightCenter(Value: Integer);
    procedure SetRightTop(Value: Integer);
    procedure SetTop(Value: Integer);
    procedure SetTopCenter(Value: Integer);
    procedure SetTopLeft(Value: Integer);
    procedure SetTopRight(Value: Integer);
    function GetBottom: Integer;
    function GetBottomCenter: Integer;
    function GetBottomLeft: Integer;
    function GetBottomRight: Integer;
    function GetLeft: Integer;
    function GetLeftBottom: Integer;
    function GetLeftCenter: Integer;
    function GetLeftTop: Integer;
    function GetRight: Integer;
    function GetRightBottom: Integer;
    function GetRightCenter: Integer;
    function GetRightTop: Integer;
    function GetTop: Integer;
    function GetTopCenter: Integer;
    function GetTopLeft: Integer;
    function GetTopRight: Integer;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Bottom'}{$ENDIF}
    ///<remarks>
    ///  <note type="important">
    ///    Deprecated.<br/>
    ///    Use <see cref="gmControls|TControlPosition.BottomCenter">BottomCenter</see>.
    ///  </note>
    ///</remarks>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Bottom: Integer read GetBottom write SetBottom;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation BottomCenter'}{$ENDIF}
    ///<summary>
    ///  Element are positioned in the center of the bottom row.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property BottomCenter: Integer read GetBottomCenter write SetBottomCenter;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation BottomLeft'}{$ENDIF}
    ///<summary>
    ///  Element are positioned in the bottom left and flow towards the middle.
    ///  Element are positioned to the right of the Google logo.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property BottomLeft: Integer read GetBottomLeft write SetBottomLeft;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation BottomRight'}{$ENDIF}
    ///<summary>
    ///  Element are positioned in the bottom right and flow towards the middle.
    ///  Element are positioned to the left of the copyrights.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property BottomRight: Integer read GetBottomRight write SetBottomRight;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Left'}{$ENDIF}
    ///<remarks>
    ///  <note type="important">
    ///    Deprecated.<br/>
    ///    Use <see cref="gmControls|TControlPosition.LeftCenter">LeftCenter</see>.
    ///  </note>
    ///</remarks>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Left: Integer read GetLeft write SetLeft;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LeftBottom'}{$ENDIF}
    ///<summary>
    ///  Element are positioned on the left, above bottom-left elements, and
    ///  flow upwards.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property LeftBottom: Integer read GetLeftBottom write SetLeftBottom;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LeftCenter'}{$ENDIF}
    ///<summary>
    ///  Elements are positioned in the center of the left side.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property LeftCenter: Integer read GetLeftCenter write SetLeftCenter;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LeftTop'}{$ENDIF}
    ///<summary>
    ///  Elements are positioned on the left, below top-left elements, and flow
    ///  downwards.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property LeftTop: Integer read GetLeftTop write SetLeftTop;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Right'}{$ENDIF}
    ///<remarks>
    ///  <note type="important">
    ///    Deprecated.<br/>
    ///    Use <see cref="gmControls|TControlPosition.RightCenter">RightCenter</see>.
    ///  </note>
    ///</remarks>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Right: Integer read GetRight write SetRight;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RightBottom'}{$ENDIF}
    ///<summary>
    ///  Elements are positioned on the right, above bottom-right elements, and
    ///  flow upwards.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property RightBottom: Integer read GetRightBottom write SetRightBottom;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RightCenter'}{$ENDIF}
    ///<summary>
    ///  Elements are positioned in the center of the right side.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property RightCenter: Integer read GetRightCenter write SetRightCenter;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RightTop'}{$ENDIF}
    ///<summary>
    ///  Elements are positioned on the right, below top-right elements, and
    ///  flow downwards.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property RightTop: Integer read GetRightTop write SetRightTop;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Top'}{$ENDIF}
    ///<remarks>
    ///  <note type="important">
    ///    Deprecated.<br/>
    ///    Use <see cref="gmControls|TControlPosition.TopCenter">TopCenter</see>.
    ///  </note>
    ///</remarks>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Top: Integer read GetTop write SetTop;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation TopCenter'}{$ENDIF}
    ///<summary>
    ///  Elements are positioned in the center of the top row.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property TopCenter: Integer read GetTopCenter write SetTopCenter;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation TopLeft'}{$ENDIF}
    ///<summary>
    ///  Elements are positioned in the top left and flow towards the middle.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property TopLeft: Integer read GetTopLeft write SetTopLeft;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation TopRight'}{$ENDIF}
    ///<summary>
    ///  Elements are positioned in the top right and flow towards the middle.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property TopRight: Integer read GetTopRight write SetTopRight;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMapTypeControlOptions'}{$ENDIF}
  ///<summary>
  ///  Options for the rendering of the map type control.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmControls|TMapTypeControlOptions.MapTypeIDs">MapTypeIDs</see></item>
  ///    <item>
  ///      <see cref="gmControls|TMapTypeControlOptions.Position">Position</see>
  ///      &#160;<b>default</b>&#160;
  ///      <see cref="gmControls|TControlPosition.TopRight">TopRight</see>
  ///    </item>
  ///    <item>
  ///      <see cref="gmControls|TMapTypeControlOptions.Style">Style</see>
  ///      &#160;<b>default</b>&#160;
  ///      <see cref="gmControls|TMapTypeControlStyle.Default">Default</see>
  ///    </item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#MapTypeControlOptions">
  ///  MapTypeControlOptions (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#MapTypeControlOptions">
  ///  MapTypeControlOptions (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMapTypeControlOptions = class(TCustomJScriptObject)
  private
    FMapTypeIDs: TStringArray;
    procedure SetMapTypeIDs(Value: TStringArray);
    procedure SetPosition(Value: Integer);
    procedure SetStyle(Value: Integer);
    function GetMapTypeIDs: TStringArray;
    function GetPosition: Integer;
    function GetStyle: Integer;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MapTypeIDs'}{$ENDIF}
    ///<summary>
    ///  IDs of map types to show in the control.
    ///</summary>
    ///<seealso cref="JScriptObjects|TStringArray">TStringArray</seealso>
    ///<seealso cref="gmControls|TMapTypeControlOptions">TMapTypeControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MapTypeIDs: TStringArray read GetMapTypeIDs write SetMapTypeIDs;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  Position ID. Used to specify the position of the control on the map.
    ///  The default position is <c>TopRight</c>.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ControlPosition">Google.Maps.ControlPosition</seealso>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    ///<seealso cref="gmControls|TMapTypeControlOptions">TMapTypeControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: Integer read GetPosition write SetPosition;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Style'}{$ENDIF}
    ///<summary>
    ///  Style ID. Used to select what style of map type control to display.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.MapTypeControlStyle">Google.Maps.MapTypeControlStyle</seealso>
    ///<seealso cref="gmControls|TMapTypeControlStyle">TMapTypeControlStyle</seealso>
    ///<seealso cref="gmControls|TMapTypeControlOptions">TMapTypeControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Style: Integer read GetStyle write SetStyle;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMapTypeControlStyle'}{$ENDIF}
  ///<summary>
  ///  Identifiers for common MapTypesControls.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmControls|TMapTypeControlStyle.Default">Default</see></item>
  ///    <item><see cref="gmControls|TMapTypeControlStyle.DropdownMenu">DropdownMenu</see></item>
  ///    <item><see cref="gmControls|TMapTypeControlStyle.HorizontalBar">HorizontalBar</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmAPI|TMaps.MapTypeControlStyle">Google.Maps.MapTypeControlStyle</seealso>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#MapTypeControlStyle">
  ///  MapTypeControlStyle (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#MapTypeControlStyle">
  ///  MapTypeControlStyle (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMapTypeControlStyle = class(TCustomJScriptObject)
  private
    function GetDefault: Integer;
    function GetDropdownMenu: Integer;
    function GetHorizontalBar: Integer;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Default'}{$ENDIF}
    ///<summary>
    ///  Uses the default map type control. The control which <c>Default</c> maps
    ///  to will vary according to window size and other factors. It may change
    ///  in future versions of the API.
    ///</summary>
    ///<seealso cref="gmControls|TMapTypeControlStyle">TMapTypeControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Default: Integer read GetDefault;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DropdownMenu'}{$ENDIF}
    ///<summary>
    ///  A dropdown menu for the screen realestate conscious.
    ///</summary>
    ///<seealso cref="gmControls|TMapTypeControlStyle">TMapTypeControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DropdownMenu: Integer read GetDropdownMenu;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation HorizontalBar'}{$ENDIF}
    ///<summary>
    ///  The standard horizontal radio buttons bar.
    ///</summary>
    ///<seealso cref="gmControls|TMapTypeControlStyle">TMapTypeControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property HorizontalBar: Integer read GetHorizontalBar;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TNavigationControlOptions'}{$ENDIF}
  ///<summary>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item>
  ///      <see cref="gmControls|TNavigationControlOptions.Position">Position</see>
  ///      &#160;<b>default</b>&#160;
  ///      <see cref="gmControls|TControlPosition.TopLeft">TopLeft</see>
  ///    </item>
  ///    <item>
  ///      <see cref="gmControls|TNavigationControlOptions.Style">Style</see>
  ///      &#160;<b>default</b>&#160;
  ///      <see cref="gmControls|TNavigationControlStyle.Default">Default</see>
  ///    </item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="important">
  ///    Deprecated since version 3.3.<br/>
  ///    Use <see cref="gmControls|TZoomControlOptions">TZoomControlOptions</see>
  ///    or <see cref="gmControls|TPanControlOptions">TPanControlOptions</see>.
  ///  </note>
  ///</remarks>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TNavigationControlOptions = class(TCustomJScriptObject)
  private
    procedure SetPosition(Value: Integer);
    procedure SetStyle(Value: Integer);
    function GetPosition: Integer;
    function GetStyle: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  Position ID. Used to specify the position of the control on the map.
    ///  The default position is <c>TopLeft</c>.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ControlPosition">Google.Maps.ControlPosition</seealso>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    ///<seealso cref="gmControls|TNavigationControlOptions">TNavigationControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: Integer read GetPosition write SetPosition;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Style'}{$ENDIF}
    ///<summary>
    ///  Style ID. Used to select what style of zoom control to display.
    ///  The default style is <see cref="gmControls|TNavigationControlStyle.Default">Default</see>.
    ///</summary>
    ///<seealso cref="gmControls|TNavigationControlStyle">TNavigationControlStyle</seealso>
    ///<seealso cref="gmControls|TNavigationControlOptions">TNavigationControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Style: Integer read GetStyle write SetStyle;
  end; //deprecated 'Use TZoomControlOptions or TPanControlOptions';

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TNavigationControlStyle'}{$ENDIF}
  ///<summary>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmControls|TNavigationControlStyle.Android">Android</see></item>
  ///    <item><see cref="gmControls|TNavigationControlStyle.Default">Default</see></item>
  ///    <item><see cref="gmControls|TNavigationControlStyle.Small">Small</see></item>
  ///    <item><see cref="gmControls|TNavigationControlStyle.ZoomPan">ZoomPan</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="important">
  ///    Deprecated since version 3.3.<br/>
  ///    Use <see cref="gmControls|TZoomControlStyle">TZoomControlStyle</see>.
  ///  </note>
  ///</remarks>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TNavigationControlStyle = class(TCustomJScriptObject)
  private
    procedure SetAndroid(Value: Integer);
    procedure SetDefault(Value: Integer);
    procedure SetHi(Value: Integer);
    procedure SetSmall(Value: Integer);
    procedure SetZoomPan(Value: Integer);
    function GetAndroid: Integer;
    function GetDefault: Integer;
    function GetHi: Integer;
    function GetSmall: Integer;
    function GetZoomPan: Integer;
  protected
    property Hi: Integer read GetHi write SetHi;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Android'}{$ENDIF}
    ///<seealso cref="gmControls|TNavigationControlStyle">TNavigationControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Android: Integer read GetAndroid write SetAndroid;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Default'}{$ENDIF}
    ///<seealso cref="gmControls|TNavigationControlStyle">TNavigationControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Default: Integer read GetDefault write SetDefault;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Small'}{$ENDIF}
    ///<seealso cref="gmControls|TNavigationControlStyle">TNavigationControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Small: Integer read GetSmall write SetSmall;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZoomPan'}{$ENDIF}
    ///<seealso cref="gmControls|TNavigationControlStyle">TNavigationControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZoomPan: Integer read GetZoomPan write SetZoomPan;
  end; //deprecated 'Use TZoomControlStyle';

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TOverviewMapControlOptions'}{$ENDIF}
  ///<summary>
  ///  Options for the rendering of the Overview Map control.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item>
  ///      <see cref="gmControls|TOverviewMapControlOptions.Opened">Opened</see>
  ///      &#160;<b>default</b>&#160;false
  ///    </item>
  ///  </list>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TOverviewMapControlOptions = class(TCustomJScriptObject)
  private
    procedure SetOpened(Value: Boolean);
    function GetOpened: Boolean;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Opened'}{$ENDIF}
    ///<summary>
    ///  Whether the control should display in opened mode or collapsed (minimized)
    ///  mode. By default, the control is closed.
    ///</summary>
    ///<seealso cref="gmControls|TOverviewMapControlOptions">TOverviewMapControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Opened: Boolean read GetOpened write SetOpened;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPanControlOptions'}{$ENDIF}
  ///<summary>
  ///  Options for the rendering of the pan control.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item>
  ///      <see cref="gmControls|TPanControlOptions.Position">Position</see>
  ///      &#160;<b>default</b>&#160;<see cref="gmControls|TControlPosition.TopLeft">TopLeft</see>
  ///    </item>
  ///  </list>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPanControlOptions = class(TCustomJScriptObject)
  private
    procedure SetPosition(Value: Integer);
    function GetPosition: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  Position ID. Used to specify the position of the control on the map.
    ///  The default position is <c>TopLeft</c>.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ControlPosition">Google.Maps.ControlPosition</seealso>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    ///<seealso cref="gmControls|TPanControlOptions">TPanControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: Integer read GetPosition write SetPosition;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TRotateControlOptions'}{$ENDIF}
  ///<summary>
  ///  Options for the rendering of the rotate control.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item>
  ///      <see cref="gmControls|TRotateControlOptions.Position">Position</see>
  ///      &#160;<b>default</b>&#160;<see cref="gmControls|TControlPosition.TopLeft">TopLeft</see>
  ///    </item>
  ///  </list>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TRotateControlOptions = class(TCustomJScriptObject)
  private
    procedure SetPosition(Value: Integer);
    function GetPosition: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  Position ID. Used to specify the position of the control on the map.
    ///  The default position is <c>TopLeft</c>.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ControlPosition">Google.Maps.ControlPosition</seealso>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    ///<seealso cref="gmControls|TRotateControlOptions">TRotateControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: Integer read GetPosition write SetPosition;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TScaleControlOptions'}{$ENDIF}
  ///<summary>
  ///  Options for the rendering of the scale control.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item>
  ///      <see cref="gmControls|TScaleControlOptions.Position">Position</see>
  ///      &#160;<b>default</b>&#160;<see cref="gmControls|TControlPosition.TopLeft">BottomLeft</see>
  ///    </item>
  ///    <item>
  ///      <see cref="gmControls|TScaleControlOptions.Style">Style</see>
  ///      &#160;<b>default</b>&#160;<see cref="gmControls|TScaleControlStyle.Default">Default</see>
  ///    </item>
  ///  </list>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TScaleControlOptions = class(TCustomJScriptObject)
  private
    procedure SetPosition(Value: Integer);
    procedure SetStyle(Value: Integer);
    function GetPosition: Integer;
    function GetStyle: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  Position ID. Used to specify the position of the control on the map.
    ///  The default position is <c>BottomLeft</c>.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ControlPosition">Google.Maps.ControlPosition</seealso>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    ///<seealso cref="gmControls|TScaleControlOptions">TScaleControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: Integer read GetPosition write SetPosition;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Style'}{$ENDIF}
    ///<summary>
    ///  Style ID. Used to select what style of map type control to display.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ScaleControlStyle">Google.Maps.ScaleControlStyle</seealso>
    ///<seealso cref="gmControls|TScaleControlStyle">TScaleControlStyle</seealso>
    ///<seealso cref="gmControls|TScaleControlOptions">TScaleControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Style: Integer read GetStyle write SetStyle;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TScaleControlStyle'}{$ENDIF}
  ///<summary>
  ///  Identifiers for scale control IDs.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmControls|TScaleControlStyle.Default">Default</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmAPI|TMaps.ScaleControlStyle">Google.Maps.ScaleControlStyle</seealso>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#ScaleControlStyle">
  ///  ScaleControlStyle (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#ScaleControlStyle">
  ///  ScaleControlStyle (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TScaleControlStyle = class(TCustomJScriptObject)
  private
    function GetDefault: Integer;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Default'}{$ENDIF}
    ///<summary>
    ///  The standard scale control.
    ///</summary>
    ///<seealso cref="gmControls|TScaleControlStyle">TScaleControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Default: Integer read GetDefault;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TStreetViewControlOptions'}{$ENDIF}
  ///<summary>
  ///  Options for the rendering of the Street View pegman control on the map.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item>
  ///      <see cref="gmControls|TStreetViewControlOptions.Position">Position</see>
  ///      &#160;<b>default</b>&#160;
  ///      <see cref="gmControls|TControlPosition.TopLeft">TopLeft</see>
  ///    </item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#StreetViewControlOptions">
  ///  StreetViewControlOptions (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#StreetViewControlOptions">
  ///  StreetViewControlOptions (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TStreetViewControlOptions = class(TCustomJScriptObject)
  private
    procedure SetPosition(Value: Integer);
    function GetPosition: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  Position ID. Used to specify the position of the control on the map.
    ///  The default position is <c>TopLeft</c>.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ControlPosition">Google.Maps.ControlPosition</seealso>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    ///<seealso cref="gmControls|TStreetViewControlOptions">TStreetViewControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: Integer read GetPosition write SetPosition;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TZoomControlOptions'}{$ENDIF}
  ///<summary>
  ///  Options for the rendering of the zoom control on the map.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item>
  ///      <see cref="gmControls|TControlOptions.Position">Position</see>
  ///      &#160;<b>default</b>&#160;
  ///      <see cref="gmControls|TControlPosition.TopLeft">TopLeft</see>
  ///    </item>
  ///    <item>
  ///      <see cref="gmControls|TControlOptions.Style">Style</see>
  ///      &#160;<b>default</b>&#160;
  ///      <see cref="gmControls|TZoomControlStyle.Default">Default</see>
  ///    </item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#ZoomControlOptions">
  ///  ZoomControlOptions (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#ZoomControlOptions">
  ///  ZoomControlOptions (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TZoomControlOptions = class(TCustomJScriptObject)
  private
    procedure SetPosition(Value: Integer);
    procedure SetStyle(Value: Integer);
    function GetPosition: Integer;
    function GetStyle: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  Position ID. Used to specify the position of the control on the map.
    ///  The default position is <c>TopLeft</c>.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ControlPosition">Google.Maps.ControlPosition</seealso>
    ///<seealso cref="gmControls|TControlPosition">TControlPosition</seealso>
    ///<seealso cref="gmControls|TZoomControlOptions">TZoomControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: Integer read GetPosition write SetPosition;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Style'}{$ENDIF}
    ///<summary>
    ///  Style ID. Used to select what style of zoom control to display.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.ZoomControlStyle">Google.Maps.ZoomControlStyle</seealso>
    ///<seealso cref="gmControls|TZoomControlOptions">TZoomControlOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Style: Integer read GetStyle write SetStyle;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TZoomControlStyle'}{$ENDIF}
  ///<summary>
  ///  Identifiers for the zoom control.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmControls"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmControls|TZoomControlStyle.Default">Default</see></item>
  ///    <item><see cref="gmControls|TZoomControlStyle.Large">Large</see></item>
  ///    <item><see cref="gmControls|TZoomControlStyle.Small">Small</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmAPI|TMaps.ZoomControlStyle">Google.Maps.ZoomControlStyle</seealso>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#ZoomControlStyle">
  ///  ZoomControlStyle (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#ZoomControlStyle">
  ///  ZoomControlStyle (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TZoomControlStyle = class(TCustomJScriptObject)
  private
    function GetDefault: Integer;
    function GetLarge: Integer;
    function GetSmall: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Default'}{$ENDIF}
    ///<summary>
    ///  The default zoom control. The control which Default maps to will vary
    ///  according to map size and other factors. It may change in future
    ///  versions of the API.
    ///</summary>
    ///<seealso cref="gmControls|TZoomControlStyle">TZoomControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Default: Integer read GetDefault;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Large'}{$ENDIF}
    ///<summary>
    ///  The larger control, with the zoom slider in addition to +/- buttons.
    ///</summary>
    ///<seealso cref="gmControls|TZoomControlStyle">TZoomControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Large: Integer read GetLarge;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Small'}{$ENDIF}
    ///<summary>
    ///  A small control with buttons to zoom in and out.
    ///</summary>
    ///<seealso cref="gmControls|TZoomControlStyle">TZoomControlStyle</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Small: Integer read GetSmall;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TControlPosition

class function TControlPosition.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[ControlPositionProp_BottomCenter,
                                 ControlPositionProp_BottomLeft,
                                 ControlPositionProp_BottomRight]);
end;

function TControlPosition.GetBottomCenter: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_BottomCenter);
end;

procedure TControlPosition.SetBottomCenter(Value: Integer);
begin
  SetProp(ControlPositionProp_BottomCenter,Value,true);
end;

function TControlPosition.GetBottomLeft: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_BottomLeft);
end;

procedure TControlPosition.SetBottomLeft(Value: Integer);
begin
  SetProp(ControlPositionProp_BottomLeft,Value,true);
end;

function TControlPosition.GetBottomRight: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_BottomRight);
end;

procedure TControlPosition.SetBottomRight(Value: Integer);
begin
  SetProp(ControlPositionProp_BottomRight,Value,true);
end;

function TControlPosition.GetBottom: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_Bottom);
end;

procedure TControlPosition.SetBottom(Value: Integer);
begin
  SetProp(ControlPositionProp_Bottom,Value,true);
end;

function TControlPosition.GetLeftBottom: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_LeftBottom);
end;

procedure TControlPosition.SetLeftBottom(Value: Integer);
begin
  SetProp(ControlPositionProp_LeftBottom,Value,true);
end;

function TControlPosition.GetLeftCenter: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_LeftCenter);
end;

procedure TControlPosition.SetLeftCenter(Value: Integer);
begin
  SetProp(ControlPositionProp_LeftCenter,Value,true);
end;

function TControlPosition.GetLeftTop: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_LeftTop);
end;

procedure TControlPosition.SetLeftTop(Value: Integer);
begin
  SetProp(ControlPositionProp_LeftTop,Value,true);
end;

function TControlPosition.GetLeft: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_Left);
end;

procedure TControlPosition.SetLeft(Value: Integer);
begin
  SetProp(ControlPositionProp_Left,Value,true);
end;

function TControlPosition.GetRightBottom: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_RightBottom);
end;

procedure TControlPosition.SetRightBottom(Value: Integer);
begin
  SetProp(ControlPositionProp_RightBottom,Value,true);
end;

function TControlPosition.GetRightCenter: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_RightCenter);
end;

procedure TControlPosition.SetRightCenter(Value: Integer);
begin
  SetProp(ControlPositionProp_RightCenter,Value,true);
end;

function TControlPosition.GetRightTop: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_RightTop);
end;

procedure TControlPosition.SetRightTop(Value: Integer);
begin
  SetProp(ControlPositionProp_RightTop,Value,true);
end;

function TControlPosition.GetRight: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_Right);
end;

procedure TControlPosition.SetRight(Value: Integer);
begin
  SetProp(ControlPositionProp_Right,Value,true);
end;

function TControlPosition.GetTopCenter: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_TopCenter);
end;

procedure TControlPosition.SetTopCenter(Value: Integer);
begin
  SetProp(ControlPositionProp_TopCenter,Value,true);
end;

function TControlPosition.GetTopLeft: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_TopLeft);
end;

procedure TControlPosition.SetTopLeft(Value: Integer);
begin
  SetProp(ControlPositionProp_TopLeft,Value,true);
end;

function TControlPosition.GetTopRight: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_TopRight);
end;

procedure TControlPosition.SetTopRight(Value: Integer);
begin
  SetProp(ControlPositionProp_TopRight,Value,true);
end;

function TControlPosition.GetTop: Integer;
begin
  Result:=GetPropAsInteger(ControlPositionProp_Top);
end;

procedure TControlPosition.SetTop(Value: Integer);
begin
  SetProp(ControlPositionProp_Top,Value,true);
end;

//------------------------------------------------------------------------------
//TMapTypeControlOptions

destructor TMapTypeControlOptions.Destroy;
begin
  FMapTypeIDs.Free;
  inherited;
end;

function TMapTypeControlOptions.GetMapTypeIDs: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(MapTypeControlOptionsProp_MapTypeIDs,
                                       TStringArray,FMapTypeIDs,true));
end;

function TMapTypeControlOptions.GetPosition: Integer;
begin
  Result:=0;
   if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ControlPosition.TopRight;
  Result:=GetPropAsInteger(MapTypeControlOptionsProp_Position,Result);
end;

function TMapTypeControlOptions.GetStyle: Integer;
begin
  Result:=0;
  if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.MapTypeControlStyle.Default;
  Result:=GetPropAsInteger(MapTypeControlOptionsProp_Style,Result);
end;

procedure TMapTypeControlOptions.SetMapTypeIDs(Value: TStringArray);
begin
  SetProp(MapTypeControlOptionsProp_MapTypeIDs,Value,true);
end;

procedure TMapTypeControlOptions.SetPosition(Value: Integer);
begin
  SetProp(MapTypeControlOptionsProp_Position,Value,true);
end;

procedure TMapTypeControlOptions.SetStyle(Value: Integer);
begin
  SetProp(MapTypeControlOptionsProp_Style,Value,true);
end;

//------------------------------------------------------------------------------
//TMapTypeControlStyle

class function TMapTypeControlStyle.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MapTypeControlStyleProp_Default,
                                 MapTypeControlStyleProp_DropdownMenu,
                                 MapTypeControlStyleProp_HorizontalBar]);
end;

function TMapTypeControlStyle.GetDefault: Integer;
begin
  Result:=GetPropAsInteger(MapTypeControlStyleProp_Default);
end;

function TMapTypeControlStyle.GetDropdownMenu: Integer;
begin
  Result:=GetPropAsInteger(MapTypeControlStyleProp_DropdownMenu);
end;

function TMapTypeControlStyle.GetHorizontalBar: Integer;
begin
  Result:=GetPropAsInteger(MapTypeControlStyleProp_HorizontalBar);
end;

//------------------------------------------------------------------------------
//TNavigationControlOptions

function TNavigationControlOptions.GetPosition: Integer;
begin
  Result:=0;
   if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ControlPosition.TopLeft;
  Result:=GetPropAsInteger(NavigationControlOptionsProp_Position,Result);
end;

function TNavigationControlOptions.GetStyle: Integer;
begin
  Result:=0;
  if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.NavigationControlStyle.Default;
  Result:=GetPropAsInteger(NavigationControlOptionsProp_Style,Result);
end;

procedure TNavigationControlOptions.SetPosition(Value: Integer);
begin
  SetProp(NavigationControlOptionsProp_Position,Value,true);
end;

procedure TNavigationControlOptions.SetStyle(Value: Integer);
begin
  SetProp(NavigationControlOptionsProp_Style,Value,true);
end;

//------------------------------------------------------------------------------
//TNavigationControlStyle

class function TNavigationControlStyle.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[NavigationControlStyleProp_Default,
                                 NavigationControlStyleProp_Android,
                                 NavigationControlStyleProp_Small]);
end;

function TNavigationControlStyle.GetAndroid: Integer;
begin
  Result:=GetPropAsInteger(NavigationControlStyleProp_Android);
end;

procedure TNavigationControlStyle.SetAndroid(Value: Integer);
begin
  SetProp(NavigationControlStyleProp_Android,Value,true);
end;

function TNavigationControlStyle.GetDefault: Integer;
begin
  Result:=GetPropAsInteger(NavigationControlStyleProp_Default);
end;

procedure TNavigationControlStyle.SetDefault(Value: Integer);
begin
  SetProp(NavigationControlStyleProp_Default,Value,true);
end;

function TNavigationControlStyle.GetHi: Integer;
begin
  Result:=GetPropAsInteger(NavigationControlStyleProp_Hi);
end;

procedure TNavigationControlStyle.SetHi(Value: Integer);
begin
  SetProp(NavigationControlStyleProp_Hi,Value,true);
end;

function TNavigationControlStyle.GetSmall: Integer;
begin
  Result:=GetPropAsInteger(NavigationControlStyleProp_Small);
end;

procedure TNavigationControlStyle.SetSmall(Value: Integer);
begin
  SetProp(NavigationControlStyleProp_Small,Value,true);
end;

function TNavigationControlStyle.GetZoomPan: Integer;
begin
  Result:=GetPropAsInteger(NavigationControlStyleProp_ZoomPan);
end;

procedure TNavigationControlStyle.SetZoomPan(Value: Integer);
begin
  SetProp(NavigationControlStyleProp_ZoomPan,Value,true);
end;

//------------------------------------------------------------------------------
//TOverviewMapControlOptions

function TOverviewMapControlOptions.GetOpened: Boolean;
begin
  Result:=GetPropAsBoolean(OverviewMapControlOptionsProp_Opened);
end;

procedure TOverviewMapControlOptions.SetOpened(Value: Boolean);
begin
  SetProp(OverviewMapControlOptionsProp_Opened,Value,true);
end;

//------------------------------------------------------------------------------
//TPanControlOptions

function TPanControlOptions.GetPosition: Integer;
begin
  Result:=0;
   if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ControlPosition.TopLeft;
  Result:=GetPropAsInteger(PanControlOptionsProp_Position,Result);
end;

procedure TPanControlOptions.SetPosition(Value: Integer);
begin
  SetProp(PanControlOptionsProp_Position,Value,true);
end;

//------------------------------------------------------------------------------
//TRotateControlOptions

function TRotateControlOptions.GetPosition: Integer;
begin
  Result:=0;
   if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ControlPosition.TopLeft;
  Result:=GetPropAsInteger(RotateControlOptionsProp_Position,Result);
end;

procedure TRotateControlOptions.SetPosition(Value: Integer);
begin
  SetProp(RotateControlOptionsProp_Position,Value,true);
end;

//------------------------------------------------------------------------------
//TScaleControlOptions

function TScaleControlOptions.GetPosition: Integer;
begin
  Result:=0;
   if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ControlPosition.BottomLeft;
  Result:=GetPropAsInteger(ScaleControlOptionsProp_Position,Result);
end;

function TScaleControlOptions.GetStyle: Integer;
begin
  Result:=0;
  if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ScaleControlStyle.Default;
  Result:=GetPropAsInteger(ScaleControlOptionsProp_Style,Result);
end;

procedure TScaleControlOptions.SetPosition(Value: Integer);
begin
  SetProp(ScaleControlOptionsProp_Position,Value,true);
end;

procedure TScaleControlOptions.SetStyle(Value: Integer);
begin
  SetProp(ScaleControlOptionsProp_Style,Value,true);
end;

//------------------------------------------------------------------------------
//TScaleControlStyle

class function TScaleControlStyle.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
           ObjectHasMembers(Disp,[ScaleControlStyleProp_Default]);
end;

function TScaleControlStyle.GetDefault: Integer;
begin
  Result:=GetPropAsInteger(ScaleControlStyleProp_Default);
end;

//------------------------------------------------------------------------------
//TStreetViewControlOptions

function TStreetViewControlOptions.GetPosition: Integer;
begin
  Result:=0;
   if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ControlPosition.TopLeft;
  Result:=GetPropAsInteger(StreetViewControlOptionsProp_Position,Result);
end;

procedure TStreetViewControlOptions.SetPosition(Value: Integer);
begin
  SetProp(StreetViewControlOptionsProp_Position,Value,true);
end;

//------------------------------------------------------------------------------
//TZoomControlOptions

function TZoomControlOptions.GetPosition: Integer;
begin
  Result:=0;
   if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ControlPosition.TopLeft;
  Result:=GetPropAsInteger(ZoomControlOptionsProp_Position,Result);
end;

function TZoomControlOptions.GetStyle: Integer;
begin
  Result:=0;
  if assigned(Script) and (Script is TScript)
    then Result:=TScript(Script).Google.Maps.ZoomControlStyle.Default;
  Result:=GetPropAsInteger(ZoomControlOptionsProp_Style,Result);
end;

procedure TZoomControlOptions.SetPosition(Value: Integer);
begin
  SetProp(ZoomControlOptionsProp_Position,Value,true);
end;

procedure TZoomControlOptions.SetStyle(Value: Integer);
begin
  SetProp(ZoomControlOptionsProp_Style,Value,true);
end;

//------------------------------------------------------------------------------
//TZoomControlStyle

function TZoomControlStyle.GetDefault: Integer;
begin
  Result:=GetPropAsInteger(ZoomControlStyleProp_Default);
end;

function TZoomControlStyle.GetLarge: Integer;
begin
  Result:=GetPropAsInteger(ZoomControlStyleProp_Large);
end;

function TZoomControlStyle.GetSmall: Integer;
begin
  Result:=GetPropAsInteger(ZoomControlStyleProp_Small);
end;

end.

