unit gmEvents;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, Forms, SysUtils, Menus, Windows, Messages,
  {$IFDEF DELPHI6_UP}Variants,{$ENDIF} ActiveX, BrowserTools,
  {$IFDEF USE_ANONYMOUS_METHODS}AnonymousMethodsTools,{$ENDIF} DispObject,
  JScriptObjects, gmBase, gmAdditionalConstants, gmAdditionalMessages,
  HTMLObjects;

type
  TMapsEventListener = class;

  TEvent = class(TCustomJScriptObject)
  private
    procedure SetMb(Value: IDispatch);
    procedure SetRe(Value: Boolean);
    function GetMb: IDispatch;
    function GetRe: Boolean;
  protected
    procedure Hi;
    function H(const a, b, c, d: OleVariant): OleVariant;
    function X(const a, b, c, d: OleVariant): OleVariant;
    function Xc(const a, b: OleVariant): OleVariant;
    function Bind(const a, b, c, d: OleVariant): OleVariant;
    function Forward(const a, b, c: OleVariant): OleVariant;
    property Mb: IDispatch read GetMb write SetMb;
    property Re: Boolean read GetRe write SetRe;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    procedure ClearInstanceListeners(Instance: TCustomJScriptObject);
    procedure ClearListeners(Instance: TCustomJScriptObject; EventName: String);
    procedure RemoveListener(Listener: TMapsEventListener);
    procedure Trigger(Instance: TCustomJScriptObject; EventName: String; const Args: array of OleVariant);
    function AddDomListener(Instance: TDispObject; EventName: String; Handler: TJScriptFunction): TMapsEventListener;
    function AddDomListenerOnce(Instance: TDispObject; EventName: String; Handler: TJScriptFunction): TMapsEventListener;
    function AddListener(Instance: TDispObject; EventName: String; Handler: TJScriptFunction): TMapsEventListener;
    function AddListenerOnce(Instance: TDispObject; EventName: String; Handler: TJScriptFunction): TMapsEventListener;
  end;

  TMapsEventListener = class(TCustomJScriptObject)
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMouseEvent'}{$ENDIF}
  ///<summary>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMouseEvent = class(TCustomJScriptObject)
  private
    FLatLng: TLatLng;
    function GetLatLng: TLatLng;
  protected
  public
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    property LatLng: TLatLng read GetLatLng;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TInsertAtEvent'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; Index: Index) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmEvents"/>
  ///  <para/>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TInsertAtEvent = TProc<TObject,Integer>;
  {$ELSE}
  TInsertAtEvent = procedure(Sender: TObject; Index: Integer) of object;
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TRemoveAtEvent'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; Index: Integer; Element:
  ///  </c><see cref="DispObject|TDispObject"/>)<c> of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmEvents"/>
  ///  <para/>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TRemoveAtEvent = TProc<TObject,Integer,TDispObject>;
  {$ELSE}
  TRemoveAtEvent = procedure(Sender: TObject; Index: Integer; Element: TDispObject) of object;
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TSetAtEvent'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; Index: Integer; Element:
  ///  </c><see cref="DispObject|TDispObject"/>)<c> of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmEvents"/>
  ///  <para/>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TSetAtEvent = TRemoveAtEvent;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMapMouseEvent'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; MouseEvent: </c>
  ///  <see cref="gmEvents|TMouseEvent"/><c>)
  ///  of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmEvents"/>
  ///  <para/>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TMapMouseEvent = TProc<TObject,TMouseEvent>;
  {$ELSE}
  TMapMouseEvent = procedure(Sender: TObject; MouseEvent: TMouseEvent) of object;
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMapEventEvent'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; Event: </c>
  ///  <see cref="HTMLObjects|TEvent"/><c>) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmEvents"/>
  ///  <para/>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TMapEventEvent = TProc<TObject,HTMLObjects.TEvent>;
  {$ELSE}
  TMapEventEvent = procedure(Sender: TObject; Event: HTMLObjects.TEvent) of object;
  {$ENDIF}

  {$IFDEF USE_ANONYMOUS_METHODS}
  TNotifyEvent = TProc<TObject>;
  {$ELSE}
  TNotifyEvent = Classes.TNotifyEvent;
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMapEvent'}{$ENDIF}
  ///<summary>
  ///  <c>procedure of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmEvents"/>
  ///  <para/>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TMapEvent = TProc;
  {$ELSE}
  TMapEvent = procedure of object;
  {$ENDIF}

  TMessageHelper = class(TObject)
  private
    FHWnd: HWND;
  protected
    procedure WndMethod(var Msg: TMessage); virtual;
  public
    constructor Create;
    destructor Destroy; override;
    property Handle: HWND read FHWnd;
  end;

procedure AddListener(Script: TCustomScript; Instance: TCustomJScriptObject;
                      const EventName: String; Handler: TJScriptFunction;
                      var Listener: TMapsEventListener); overload;
procedure AddListener(Script: TCustomScript;  Instance: TCustomJScriptObject;
                      const EventName: String;
                      var Listener: TMapsEventListener;
                      var ListenerFunction: TJScriptFunction;
                      Receiver: TExternalProc); overload;
procedure RemoveListener(Script: TCustomScript; var Listener: TMapsEventListener);
procedure ShowPopupMenu(PopupMenu: TPopupMenu);
{$IFDEF USE_ANONYMOUS_METHODS}
function AddRefToEvent(const Event: Classes.TNotifyEvent): Classes.TNotifyEvent; overload;
function MakeEvent(const NotifyProc: TNotifyProc): Classes.TNotifyEvent;
procedure ReleaseEvent(const Event);
{$ENDIF}

implementation

uses
  gmAPI;

procedure AddListener(Script: TCustomScript; Instance: TCustomJScriptObject;
                      const EventName: String; Handler: TJScriptFunction;
                      var Listener: TMapsEventListener);
begin
  if assigned(Script) and (Script is TScript) and assigned(Handler) and
     not assigned(Listener)
    then Listener:=TScript(Script).Google.Maps.Event.AddListener(Instance,EventName,Handler);
end;

procedure AddListener(Script: TCustomScript; Instance: TCustomJScriptObject;
                      const EventName: String;
                      var Listener: TMapsEventListener;
                      var ListenerFunction: TJScriptFunction;
                      Receiver: TExternalProc);
begin
  if assigned(Script) and (Script is TScript) and assigned(Receiver) then
  begin
    if not assigned(ListenerFunction)
      then ListenerFunction:=CreateJScriptFunction(Script,Receiver);
    AddListener(Script,Instance,EventName,ListenerFunction,Listener);
  end;
end;

procedure RemoveListener(Script: TCustomScript; var Listener: TMapsEventListener);
begin
  if assigned(Script) and (Script is TScript) and assigned(Listener) then
  begin
    TScript(Script).Google.Maps.Event.RemoveListener(Listener);
    FreeAndNil(Listener);
  end;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
function AddRefToEvent(const Event: Classes.TNotifyEvent): Classes.TNotifyEvent;
begin
  Result:=Event;
  AddRefMethPtr(Result);
end;

function MakeEvent(const NotifyProc: TNotifyProc): Classes.TNotifyEvent;
begin
  Result:=AnonymousMethodsTools.MakeEvent(NotifyProc);
end;

procedure ReleaseEvent(const Event);
begin
  ReleaseMethPtr(Event);
end;
{$ENDIF}

const
  WM_SHOWPOPUPMENU = WM_USER+100;

type
  TWMPopupMessage = packed record
    Msg: Cardinal;
    PopupMenu: TPopupMenu;                                 //32 Bit Pointer (64 Bit ???)
    Unused1, Unused2: Word;
    Result: Longint;
  end;

  TPopupHelper = class(TMessageHelper)
  private
    procedure WMPopupMenu(var Msg: TWMPopupMessage); message WM_SHOWPOPUPMENU;
  end;

constructor TMessageHelper.Create;
begin
  inherited;
  {$HINTS OFF}
  FHWnd:=AllocateHWnd(WndMethod);                          //deprecated D2005..D2007 ???
  {$HINTS ON}
end;

destructor TMessageHelper.Destroy;
begin
  {$HINTS OFF}
  DeallocateHWnd(FHWnd);                                   //deprecated D2005..D2007 ???
  {$HINTS ON}
  inherited;
end;

procedure TMessageHelper.WndMethod(var Msg: TMessage);
begin
  Dispatch(Msg);
end;

procedure TPopupHelper.WMPopupMenu(var Msg: TWMPopupMessage);
var
  Point: Windows.TPoint;
begin
  if assigned(Msg.PopupMenu) then
  begin
    GetCursorPos(Point);
    Msg.PopupMenu.Popup(Point.x,Point.y);
  end;
  Msg.Result:=1;
end;

var
  PopupHelper: TPopupHelper;                               //Hilfsobjekt fr den Empfang von Windows-Nachrichten

procedure ShowPopupMenu(PopupMenu: TPopupMenu);
begin
  if not assigned(PopupMenu)
    then Exit;
  if not assigned(PopupHelper)
    then PopupHelper:=TPopupHelper.Create;
  PostMessage(PopupHelper.Handle,WM_SHOWPOPUPMENU,Integer(PopupMenu),0);
end;

//------------------------------------------------------------------------------
//TEvent

class function TEvent.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[EventMethod_AddDomListener,
                                 EventMethod_AddDomListener,
                                 EventMethod_AddDomListenerOnce]);
end;

function TEvent.AddDomListener(Instance: TDispObject; EventName: String; Handler: TJScriptFunction): TMapsEventListener;
var
  DomListener: OleVariant;
begin
  Result:=nil;
  if not assigned(Instance)
    then raise Exception.Create(ExceptionNoInstance);
  if not assigned(Handler)
    then raise Exception.Create(ExceptionNoHandler);
  DomListener:=Exec(EventMethod_AddDomListener,[Instance.Disp,EventName,Handler.Disp]);
  if VarIsValidDispatch(DomListener)
    then Result:=TMapsEventListener.Create(Script,DomListener);
end;

function TEvent.AddDomListenerOnce(Instance: TDispObject; EventName: String; Handler: TJScriptFunction): TMapsEventListener;
var
  DomListener: OleVariant;
begin
  Result:=nil;
  if not assigned(Instance)
    then raise Exception.Create(ExceptionNoInstance);
  if not assigned(Handler)
    then raise Exception.Create(ExceptionNoHandler);
  DomListener:=Exec(EventMethod_AddDomListenerOnce,[Instance.Disp,EventName,Handler.Disp]);
  if VarIsValidDispatch(DomListener)
    then Result:=TMapsEventListener.Create(Script,DomListener);
end;

function TEvent.AddListener(Instance: TDispObject; EventName: String; Handler: TJScriptFunction): TMapsEventListener;
var
  Listener: OleVariant;
begin
  Result:=nil;
  if not assigned(Instance)
    then raise Exception.Create(ExceptionNoInstance);
  if not assigned(Handler)
    then raise Exception.Create(ExceptionNoHandler);
  Listener:=Exec(EventMethod_AddListener,[Instance.Disp,EventName,Handler.Disp]);
  if VarIsValidDispatch(Listener)
    then Result:=TMapsEventListener.Create(Script,Listener);
end;

function TEvent.AddListenerOnce(Instance: TDispObject; EventName: String; Handler: TJScriptFunction): TMapsEventListener;
var
  Listener: OleVariant;
begin
  Result:=nil;
  if not assigned(Instance)
    then raise Exception.Create(ExceptionNoInstance);
  if not assigned(Handler)
    then raise Exception.Create(ExceptionNoHandler);
  Listener:=Exec(EventMethod_AddListenerOnce,[Instance.Disp,EventName,Handler.Disp]);
  if VarIsValidDispatch(Listener)
    then Result:=TMapsEventListener.Create(Script,Listener);
end;

function TEvent.Bind(const a, b, c, d: OleVariant): OleVariant;
begin
  Result:=Exec(EventMethod_Bind,[a,b,c,d]);
end;

procedure TEvent.ClearInstanceListeners(Instance: TCustomJScriptObject);
begin
  if not assigned(Instance)
    then raise Exception.Create(ExceptionNoInstance);
  Exec(EventMethod_ClearInstanceListeners,[Instance.Disp]);
end;

procedure TEvent.ClearListeners(Instance: TCustomJScriptObject; EventName: String);
begin
  if not assigned(Instance)
    then raise Exception.Create(ExceptionNoInstance);
  Exec(EventMethod_ClearListeners,[Instance.Disp,EventName]);
end;

function TEvent.Forward(const a, b, c: OleVariant): OleVariant;
begin
  Result:=Exec(EventMethod_Forward,[a,b,c]);
end;

function TEvent.H(const a, b, c, d: OleVariant): OleVariant;
begin
  Result:=Exec(EventMethod_H,[a,b,c,d]);
end;

procedure TEvent.Hi;
begin
  Exec(EventMethod_Hi);
end;

function TEvent.GetMb: IDispatch;
begin
  Result:=GetPropAsDispatch(EventProp_Mb);
end;

procedure TEvent.SetMb(Value: IDispatch);
begin
  if assigned(Value)
    then SetProp(EventProp_Mb,Value,true);
end;

function TEvent.GetRe: Boolean;
begin
  Result:=GetPropAsBoolean(EventProp_Re);
end;

procedure TEvent.SetRe(Value: Boolean);
begin
  SetProp(EventProp_Re,Value,true);
end;

procedure TEvent.RemoveListener(Listener: TMapsEventListener);
begin
  Exec(EventMethod_RemoveListener,Listener);
end;

procedure TEvent.Trigger(Instance: TCustomJScriptObject; EventName: String; const Args: array of OleVariant);
{$IFNDEF CAN_OVERLOAD}
var
  a: array[0..1] of OleVariant;
{$ENDIF}
begin
  if not assigned(Instance)
    then raise Exception.Create(ExceptionNoInstance);
  {$IFNDEF CAN_OVERLOAD}
  a[0]:=Instance.Disp;
  a[1]:=EventName;
  Exec(EventMethod_Trigger,Append(a,Args));
  {$ELSE}
  Exec(EventMethod_Trigger,Append([Instance.Disp,EventName],Args));
  {$ENDIF}
end;

function TEvent.X(const a, b, c, d: OleVariant): OleVariant;
begin
  Result:=Exec(EventMethod_X,[a,b,c,d]);
end;

function TEvent.Xc(const a, b: OleVariant): OleVariant;
begin
  Result:=Exec(EventMethod_Xc,[a,b]);
end;

//------------------------------------------------------------------------------
//TMapsEventListener

class function TMapsEventListener.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=false; {(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[,,]); }
end;

//------------------------------------------------------------------------------
//TMouseEvent

destructor TMouseEvent.Destroy;
begin
  FLatLng.Free;
  inherited;
end;

class function TMouseEvent.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MouseEventProp_LatLng]);
end;

function TMouseEvent.GetLatLng: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(MouseEventProp_LatLng,TLatLng,FLatLng));
end;

initialization

finalization
  PopupHelper.Free;

end.

