///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Adsense|TAdFormat"/></item>
///    <item><see cref="gmLibraries.Adsense|TAdsense"/></item>
///    <item><see cref="gmLibraries.Adsense|TAdUnit"/></item>
///    <item><see cref="gmLibraries.Adsense|TAdUnitOptions"/></item>
///  </list>
///  <para><b>Functions</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Adsense|New(TAdUnit)">New: TAdUnit</see></item>
///  </list>
///</summary>
unit gmLibrariesAdsense;

{$INCLUDE gmConfig.inc}

interface

uses
  Classes, SysUtils, JScriptObjects, HTMLObjects, BrowserTools, gmMap, gmMVC,
  gmEvents, gmAdditionalConstants;

type
  TAdFormat = class;
  TAdsense = class;
  TAdUnit = class;
  TAdUnitOptions = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TAdsense'}{$ENDIF}
  ///<summary>
  ///  Library.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Adsense"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Adsense|TAdsense.AdUnit(TAdUnitOptions)">AdUnit</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Adsense|TAdsense.AdFormat">AdFormat</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmAPI|TMaps.Adsense">Google.Maps.Adsense</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TAdsense = class(TCustomJScriptObject)
  private
    FAdFormat: TAdFormat;
    function GetAdFormat: TAdFormat;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AdUnit'}{$ENDIF}
    ///<summary>
    ///  Creates an AdSense for Content display ad on the associated map.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdsense"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function AdUnit(Opts: TAdUnitOptions): TAdUnit;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AdFormat'}{$ENDIF}
    ///<seealso cref="gmLibraries.Adsense|TAdsense"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property AdFormat: TAdFormat read GetAdFormat;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TAdFormat'}{$ENDIF}
  ///<summary>
  ///  Identifiers used to specify an AdSense For Content format.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Adsense"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.Banner">Banner</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.Button">Button</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.HalfBanner">HalfBanner</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.LargeRectangle">LargeRectangle</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.Leaderboard">Leaderboard</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.MediumRectangle">MediumRectangle</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.Skyscraper">Skyscraper</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.SmallRectangle">SmallRectangle</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.SmallSquare">SmallSquare</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.Square">Square</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.VerticalBanner">VerticalBanner</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdFormat.WideSkyscraper">WideSkyscraper</see></item>
  ///  </list>
  ///</summary>
  ///<example>See <see cref="gmLibraries.Adsense|TAdUnit"/></example>
  ///<seealso cref="gmLibraries.Adsense|TAdsense">TAdsense</seealso>
  ///<seealso href="https://google.com/adsense/adformats">Google AdSense Ad Formats</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#AdFormat">AdFormat</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TAdFormat = class(TCustomJScriptObject)
  private
    procedure SetBanner(Value: String);
    procedure SetButton(Value: String);
    procedure SetHalfBanner(Value: String);
    procedure SetLargeRectangle(Value: String);
    procedure SetLeaderboard(Value: String);
    procedure SetMediumRectangle(Value: String);
    procedure SetSkyscraper(Value: String);
    procedure SetSmallRectangle(Value: String);
    procedure SetSmallSquare(Value: String);
    procedure SetSquare(Value: String);
    procedure SetVerticalBanner(Value: String);
    procedure SetWideSkyscraper(Value: String);
    function GetBanner: String;
    function GetButton: String;
    function GetHalfBanner: String;
    function GetLargeRectangle: String;
    function GetLeaderboard: String;
    function GetMediumRectangle: String;
    function GetSkyscraper: String;
    function GetSmallRectangle: String;
    function GetSmallSquare: String;
    function GetSquare: String;
    function GetVerticalBanner: String;
    function GetWideSkyscraper: String;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Banner'}{$ENDIF}
    ///<summary>
    ///  A horizontal "banner" ad. (468x60px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Banner: String read GetBanner write SetBanner;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Button'}{$ENDIF}
    ///<summary>
    ///  A small ad. (125x125px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Button: String read GetButton write SetButton;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation HalfBanner'}{$ENDIF}
    ///<summary>
    ///  A smaller horizontal "banner" ad. (234x60px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property HalfBanner: String read GetHalfBanner write SetHalfBanner;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LargeRectangle'}{$ENDIF}
    ///<summary>
    ///  A large rectangulare ad. (336x280px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property LargeRectangle: String read GetLargeRectangle write SetLargeRectangle;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Leaderboard'}{$ENDIF}
    ///<summary>
    ///  A fully horizontal display area. (728x90px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Leaderboard: String read GetLeaderboard write SetLeaderboard;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MediumRectangle'}{$ENDIF}
    ///<summary>
    ///  A medium rectangular ad. (300x250px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MediumRectangle: String read GetMediumRectangle write SetMediumRectangle;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Skyscraper'}{$ENDIF}
    ///<summary>
    ///  A large vertical ad. (120x600px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Skyscraper: String read GetSkyscraper write SetSkyscraper;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SmallRectangle'}{$ENDIF}
    ///<summary>
    ///  A small rectangular ad. (180x150px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property SmallRectangle: String read GetSmallRectangle write SetSmallRectangle;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SmallSquare'}{$ENDIF}
    ///<summary>
    ///  A smaller square ad. (200x200px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property SmallSquare: String read GetSmallSquare write SetSmallSquare;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Square'}{$ENDIF}
    ///<summary>
    ///  A square ad with large type. (250x250px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Square: String read GetSquare write SetSquare;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation VerticalBanner'}{$ENDIF}
    ///<summary>
    ///  A medium-sized vertical ad. (120x240px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property VerticalBanner: String read GetVerticalBanner write SetVerticalBanner;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation WideSkyscraper'}{$ENDIF}
    ///<summary>
    ///  A wide, vertical ad using larger type. (160x600px)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdFormat"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property WideSkyscraper: String read GetWideSkyscraper write SetWideSkyscraper;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TAdUnit'}{$ENDIF}
  ///<summary>
  ///  Implements AdSense for Content advertising on an associated map. To use
  ///  an <c>TAdUnit</c>, you must obtain and specify an <i>AdSense for Content
  ///  publisher ID</i> within the AdUnit's constructor options. This class extends
  ///  <c>TMVCObject</c>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Adsense"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Adsense|TAdsense.AdUnit(TAdUnitOptions)">Google.Maps.AdSense.AdUnit</see></item>
  ///  </list>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.GetChannelNumber">GetChannelNumber</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.GetContainer">GetContainer</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.GetFormat">GetFormat</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.GetMap">GetMap</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.GetPosition">GetPosition</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.GetPublisherID">GetPublisherID</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.SetChannelNumber(String)">SetChannelNumber</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.SetFormat(TAdFormat)">SetFormat</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.SetMap(TMap)">SetMap</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.SetPosition(Integer)">SetPosition</see></item>
  ///  </list>
  ///  <para><b>Events</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.OnChannelNumberChanged">OnChannelNumberChanged</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.OnFormatChanged">OnFormatChanged</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.OnMapChanged">OnMapChanged</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnit.OnPositionChanged">OnPositionChanged</see></item>
  ///  </list>
  ///</summary>
  ///<example>
  ///  <code lang="Delphi">
  ///procedure TForm1.InitMap(Sender: TObject);
  ///var
  ///  AdUnitOptions: TAdUnitOptions;
  ///  [...]
  ///begin
  ///  with TScript(Sender) do
  ///  begin
  ///    [...]
  ///    AdUnitOptions:=TAdUnitOptions.Create;
  ///    with AdUnitOptions do
  ///    begin
  ///      Format:=Google.Maps.Adsense.AdFormat.HalfBanner;
  ///      Map:=Maps[0];
  ///      Position:=Google.Maps.ControlPosition.TopCenter;
  ///      PublisherID:=MyID;
  ///    end;
  ///    New(Google.Maps.AdSense.AdUnit(AdUnitOptions));
  ///    [...]
  ///  end;
  ///end;</code>
  ///</example>
  ///<seealso cref="gmLibraries.Adsense|TAdsense">TAdsense</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#AdUnit">AdUnit</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TAdUnit = class(TMVCObject)
  private
    FContainer: TNode;
    FFormat: TAdFormat;
    FMap: TMap;
    FOnChannelNumberChanged: TNotifyEvent;
    FOnChannelNumberChangedListener: TMapsEventListener;
    FOnChannelNumberChangedFunction: TJScriptFunction;
    FOnFormatChanged: TNotifyEvent;
    FOnFormatChangedListener: TMapsEventListener;
    FOnFormatChangedFunction: TJScriptFunction;
    FOnMapChanged: TNotifyEvent;
    FOnMapChangedListener: TMapsEventListener;
    FOnMapChangedFunction: TJScriptFunction;
    FOnPositionChanged: TNotifyEvent;
    FOnPositionChangedListener: TMapsEventListener;
    FOnPositionChangedFunction: TJScriptFunction;
    procedure SetOnChannelNumberChanged(Value: TNotifyEvent);
    procedure SetOnFormatChanged(Value: TNotifyEvent);
    procedure SetOnMapChanged(Value: TNotifyEvent);
    procedure SetOnPositionChanged(Value: TNotifyEvent);
  protected
    procedure FreeNotify(Sender: TObject); override;
    class function New(Script: TCustomScript; Container: TNode): IDispatch; reintroduce; overload;
    class function New(Script: TCustomScript; Container: TNode; Opts: TAdUnitOptions): IDispatch; reintroduce; overload;
    //JavaScript event handler:
    procedure ChannelNumberChanged;
    procedure FormatChanged;
    procedure MapChanged;
    procedure PositionChanged;
  public
    constructor Create(Opts: TAdUnitOptions); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TAdUnitOptions); reintroduce; overload;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetChannelNumber'}{$ENDIF}
    ///<summary>
    ///  Specifies the channel number for this AdUnit. Channel numbers are
    ///  optional and can be created for Google AdSense tracking.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetChannelNumber(ChannelNumber: String);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetFormat'}{$ENDIF}
    ///<summary>
    ///  Specifies the display format for this AdUnit.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetFormat(Format: TAdFormat);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetMap'}{$ENDIF}
    ///<summary>
    ///  Associates this AdUnit with the specified map. Ads will be targeted to
    ///  the map's viewport. The map must be specified in order to display ads.
    ///</summary>
    ///<seealso cref="gmMap|TMap"/>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetMap(Map: TMap);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPosition'}{$ENDIF}
    ///<summary>
    ///  Sets the ControlPosition at which to display the AdUnit on the map. If
    ///  the position is set to null, the AdUnit is removed from the map.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition"/>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPosition(Position: Integer);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetChannelNumber'}{$ENDIF}
    ///<summary>
    ///  Returns the channel number in use by this AdUnit.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition"/>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetChannelNumber: String;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetContainer'}{$ENDIF}
    ///<summary>
    ///  Returns the containing element of the AdUnit.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode"/>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetContainer: TNode;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetFormat'}{$ENDIF}
    ///<summary>
    ///  Returns the format in use by this AdUnit.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetFormat: TAdFormat;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetMap'}{$ENDIF}
    ///<summary>
    ///  Returns the map to which this AdUnit's ads are targeted.
    ///</summary>
    ///<seealso cref="gmMap|TMap"/>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetMap: TMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetPosition'}{$ENDIF}
    ///<summary>
    ///  Returns the ControlPosition at which this AdUnit is displayed on the map.
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition"/>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetPosition: Integer;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetPublisherID'}{$ENDIF}
    ///<summary>
    ///  Returns the specified AdSense For Content publisher ID.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetPublisherID: String;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnChannelNumberChanged'}{$ENDIF}
    ///<summary>
    ///  This event is fired when the AdUnit's channelNumber property changes.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnChannelNumberChanged: TNotifyEvent read FOnChannelNumberChanged write SetOnChannelNumberChanged;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnFormatChanged'}{$ENDIF}
    ///<summary>
    ///  This event is fired when the AdUnit's format property changes.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnFormatChanged: TNotifyEvent read FOnFormatChanged write SetOnFormatChanged;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnMapChanged'}{$ENDIF}
    ///<summary>
    ///  This event is fired when the AdUnit's map property changes.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnMapChanged: TNotifyEvent read FOnMapChanged write SetOnMapChanged;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnPositionChanged'}{$ENDIF}
    ///<summary>
    ///  This event is fired when the AdUnit's position property changes.
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnit"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnPositionChanged: TNotifyEvent read FOnPositionChanged write SetOnPositionChanged;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TAdUnitOptions'}{$ENDIF}
  ///<summary>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Adsense"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Adsense|TAdUnitOptions.ChannelNumber">ChannelNumber</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnitOptions.Format">Format</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnitOptions.Map">Map</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnitOptions.Position">Position</see></item>
  ///    <item><see cref="gmLibraries.Adsense|TAdUnitOptions.PublisherID">PublisherID</see></item>
  ///  </list>
  ///</summary>
  ///<example>See <see cref="gmLibraries.Adsense|TAdUnit"/></example>
  ///<seealso cref="gmLibraries.Adsense|TAdsense">TAdsense</seealso>
  ///<seealso cref="gmLibraries.Adsense|TAdUnit">TAdUnit</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#AdUnitOptions">AdUnitOptions</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TAdUnitOptions = class(TCustomJScriptObject)
  private
    FMap: TMap;
    procedure SetChannelNumber(Value: String);
    procedure SetFormat(Value: String);
    procedure SetMap(Value: TMap);
    procedure SetPosition(Value: Integer);
    procedure SetPublisherID(Value: String);
    procedure SetVisible(Value: Boolean);
    function GetChannelNumber: String;
    function GetFormat: String;
    function GetMap: TMap;
    function GetPosition: Integer;
    function GetPublisherID: String;
    function GetVisible: Boolean;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ChannelNumber'}{$ENDIF}
    ///<summary>
    ///  The AdSense For Content channel number for tracking the performance of
    ///  this AdUnit. It must be stored as a string as it will typically be a
    ///  large UINT64. (Optional)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnitOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ChannelNumber: String read GetChannelNumber write SetChannelNumber;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Format'}{$ENDIF}
    ///<summary>
    ///  The Format of the AdUnit. See <see href="https://google.com/adsense/adformats"/>.
    ///  (Optional)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnitOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Format: String read GetFormat write SetFormat;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Map'}{$ENDIF}
    ///<summary>
    ///  The map associated with this AdUnit.
    ///</summary>
    ///<seealso cref="gmMap|TMap"/>
    ///<seealso cref="gmLibraries.Adsense|TAdUnitOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Map: TMap read GetMap write SetMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  The position of the AdUnit. If specified, the AdUnit will be displayed
    ///  at this position. Otherwise, it will not be added to the map. (Optional)
    ///</summary>
    ///<seealso cref="gmControls|TControlPosition"/>
    ///<seealso cref="gmLibraries.Adsense|TAdUnitOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: Integer read GetPosition write SetPosition;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PublisherID'}{$ENDIF}
    ///<summary>
    ///  Your AdSense for Content publisher ID. Required and must be set at the
    ///  time of initialization. (Required)
    ///</summary>
    ///<seealso cref="gmLibraries.Adsense|TAdUnitOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PublisherID: String read GetPublisherID write SetPublisherID;
    property Visible: Boolean read GetVisible write SetVisible;
  end;

{$IFDEF USE_REGIONS}{$REGION 'Documentation New'}{$ENDIF}
///<summary>
///  <para><b>Unit</b></para>
///    <see cref="gmLibraries.Adsense"/>
///  <para/>
///</summary>
{$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
function New(Value: TAdUnit): TAdUnit; overload;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TAdsense

destructor TAdsense.Destroy;
begin
  FAdFormat.Free;
  inherited;
end;

function New(Value: TAdUnit): TAdUnit;
begin
  Result:=Value;
end;

function TAdsense.AdUnit(Opts: TAdUnitOptions): TAdUnit;
begin
  Result:=TAdUnit.Create(Script,Opts);
end;

function TAdsense.GetAdFormat: TAdFormat;
begin
  Result:=TAdFormat(GetPropAsObject(AdsenseProp_AdFormat,TAdFormat,FAdFormat));
end;

//------------------------------------------------------------------------------
//TAdFormat

function TAdFormat.GetBanner: String;
begin
  Result:=GetPropAsString(AdFormatProp_Banner);
end;

function TAdFormat.GetButton: String;
begin
  Result:=GetPropAsString(AdFormatProp_Button);
end;

function TAdFormat.GetHalfBanner: String;
begin
  Result:=GetPropAsString(AdFormatProp_HalfBanner);
end;

function TAdFormat.GetLargeRectangle: String;
begin
  Result:=GetPropAsString(AdFormatProp_LargeRectangle);
end;

function TAdFormat.GetLeaderboard: String;
begin
  Result:=GetPropAsString(AdFormatProp_Leaderboard);
end;

function TAdFormat.GetMediumRectangle: String;
begin
  Result:=GetPropAsString(AdFormatProp_MediumRectangle);
end;

function TAdFormat.GetSkyscraper: String;
begin
  Result:=GetPropAsString(AdFormatProp_Skyscraper);
end;

function TAdFormat.GetSmallRectangle: String;
begin
  Result:=GetPropAsString(AdFormatProp_SmallRectangle);
end;

function TAdFormat.GetSmallSquare: String;
begin
  Result:=GetPropAsString(AdFormatProp_SmallSquare);
end;

function TAdFormat.GetSquare: String;
begin
  Result:=GetPropAsString(AdFormatProp_Square);
end;

function TAdFormat.GetVerticalBanner: String;
begin
  Result:=GetPropAsString(AdFormatProp_VerticalBanner);
end;

function TAdFormat.GetWideSkyscraper: String;
begin
  Result:=GetPropAsString(AdFormatProp_WideSkyscraper);
end;

procedure TAdFormat.SetBanner(Value: String);
begin
  SetProp(AdFormatProp_Banner,Value,true);
end;

procedure TAdFormat.SetButton(Value: String);
begin
  SetProp(AdFormatProp_Button,Value,true);
end;

procedure TAdFormat.SetHalfBanner(Value: String);
begin
  SetProp(AdFormatProp_HalfBanner,Value,true);
end;

procedure TAdFormat.SetLargeRectangle(Value: String);
begin
  SetProp(AdFormatProp_LargeRectangle,Value,true);
end;

procedure TAdFormat.SetLeaderboard(Value: String);
begin
  SetProp(AdFormatProp_Leaderboard,Value,true);
end;

procedure TAdFormat.SetMediumRectangle(Value: String);
begin
  SetProp(AdFormatProp_MediumRectangle,Value,true);
end;

procedure TAdFormat.SetSkyscraper(Value: String);
begin
  SetProp(AdFormatProp_Skyscraper,Value,true);
end;

procedure TAdFormat.SetSmallRectangle(Value: String);
begin
  SetProp(AdFormatProp_SmallRectangle,Value,true);
end;

procedure TAdFormat.SetSmallSquare(Value: String);
begin
  SetProp(AdFormatProp_SmallSquare,Value,true);
end;

procedure TAdFormat.SetSquare(Value: String);
begin
  SetProp(AdFormatProp_Square,Value,true);
end;

procedure TAdFormat.SetVerticalBanner(Value: String);
begin
  SetProp(AdFormatProp_VerticalBanner,Value,true);
end;

procedure TAdFormat.SetWideSkyscraper(Value: String);
begin
  SetProp(AdFormatProp_WideSkyscraper,Value,true);
end;

//------------------------------------------------------------------------------
//TAdUnit

constructor TAdUnit.Create(Opts: TAdUnitOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TAdUnit.Create(Script: TCustomScript; Opts: TAdUnitOptions);
begin
  if not assigned(Script) or not (Script is TScript)
    then raise Exception.Create(ExceptionNoScript);
  FContainer:=Script.Document.CreateDivNode;
  FContainer.AddNotify(Self);
  inherited Create(Script,New(Script,FContainer,Opts));
end;

destructor TAdUnit.Destroy;
begin
  SetOnChannelNumberChanged(nil);
  SetOnFormatChanged(nil);
  SetOnMapChanged(nil);
  SetOnPositionChanged(nil);
  FContainer.Free;
  FFormat.Free;
  FMap.Free;
  inherited;
end;

class function TAdUnit.New(Script: TCustomScript; Container: TNode): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Adsense.Disp,
                              AdsenseMethod_AdUnit,Container.Disp,true);
end;

class function TAdUnit.New(Script: TCustomScript; Container: TNode; Opts: TAdUnitOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Adsense.Disp,
                                     AdsenseMethod_AdUnit,
                                     {$IFDEF CAN_OVERLOAD}
                                     [Container.Disp,Opts.Disp]
                                     {$ELSE}
                                     Container.Disp,Opts.Disp
                                     {$ENDIF}
                                     ,true);
end;

procedure TAdUnit.ChannelNumberChanged;
begin
  CallEvent(FOnChannelNumberChanged);
end;

procedure TAdUnit.FormatChanged;
begin
  CallEvent(FOnFormatChanged);
end;

procedure TAdUnit.FreeNotify(Sender: TObject);
begin
  inherited;
  if FContainer=Sender
    then FContainer:=nil;
end;

function TAdUnit.GetChannelNumber: String;
begin
  Result:=VarAsString(Exec(AdUnitMethod_GetChannelNumber));
end;

function TAdUnit.GetContainer: TNode;
begin
  Result:=TNode(GetObjectWithExec(AdUnitMethod_GetContainer,TNode,FContainer));
end;

function TAdUnit.GetFormat: TAdFormat;
begin
  Result:=TAdFormat(GetObjectWithExec(AdUnitMethod_GetFormat,TAdFormat,FFormat));
end;

function TAdUnit.GetMap: TMap;
begin
  Result:=TMap(GetObjectWithExec(AdUnitMethod_GetMap,TMap,FMap));
end;

function TAdUnit.GetPosition: Integer;
begin
  Result:=VarAsInteger(Exec(AdUnitMethod_GetPosition));
end;

function TAdUnit.GetPublisherID: String;
begin
  Result:=VarAsString(Exec(AdUnitMethod_GetPublisherID));
end;

procedure TAdUnit.MapChanged;
begin
  CallEvent(FOnMapChanged);
end;

procedure TAdUnit.PositionChanged;
begin
  CallEvent(FOnPositionChanged);
end;

procedure TAdUnit.SetChannelNumber(ChannelNumber: String);
begin
  Exec(AdUnitMethod_SetChannelNumber,[ChannelNumber]);
end;

procedure TAdUnit.SetFormat(Format: TAdFormat);
begin
  Exec(AdUnitMethod_SetFormat,Format);
end;

procedure TAdUnit.SetMap(Map: TMap);
begin
  Exec(AdUnitMethod_SetMap,Map);
end;

procedure TAdUnit.SetOnChannelNumberChanged(Value: TNotifyEvent);
begin
  FOnChannelNumberChanged:=Value;
  RemoveListener(FOnChannelNumberChangedListener);
  if assigned(FOnChannelNumberChanged)
    then AddListener(AdUnitEvent_ChannelNumberChanged,FOnChannelNumberChangedListener,
                     FOnChannelNumberChangedFunction,ChannelNumberChanged);
end;

procedure TAdUnit.SetOnFormatChanged(Value: TNotifyEvent);
begin
  FOnFormatChanged:=Value;
  RemoveListener(FOnFormatChangedListener);
  if assigned(FOnFormatChanged)
    then AddListener(AdUnitEvent_FormatChanged,FOnFormatChangedListener,
                     FOnFormatChangedFunction,FormatChanged);
end;

procedure TAdUnit.SetOnMapChanged(Value: TNotifyEvent);
begin
  FOnMapChanged:=Value;
  RemoveListener(FOnMapChangedListener);
  if assigned(FOnMapChanged)
    then AddListener(AdUnitEvent_MapChanged,FOnMapChangedListener,
                     FOnMapChangedFunction,MapChanged);
end;

procedure TAdUnit.SetOnPositionChanged(Value: TNotifyEvent);
begin
  FOnPositionChanged:=Value;
  RemoveListener(FOnPositionChangedListener);
  if assigned(FOnPositionChanged)
    then AddListener(AdUnitEvent_PositionChanged,FOnPositionChangedListener,
                     FOnPositionChangedFunction,PositionChanged);
end;

procedure TAdUnit.SetPosition(Position: Integer);
begin
  Exec(AdUnitMethod_SetPosition,[Position]);
end;

//------------------------------------------------------------------------------
//TAdUnitOptions

constructor TAdUnitOptions.Create;
begin
  inherited;
end;

constructor TAdUnitOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TAdUnitOptions.Destroy;
begin
  FMap.Free;
  inherited;
end;

function TAdUnitOptions.GetChannelNumber: String;
begin
  Result:=GetPropAsString(AdUnitOptionsProp_ChannelNumber);
end;

function TAdUnitOptions.GetFormat: String;
begin
  Result:=GetPropAsString(AdUnitOptionsProp_Format);
end;

function TAdUnitOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(AdUnitOptionsProp_Map,TMap,FMap));
end;

function TAdUnitOptions.GetPosition: Integer;
begin
  Result:=GetPropAsInteger(AdUnitOptionsProp_Position);
end;

function TAdUnitOptions.GetPublisherID: String;
begin
  Result:=GetPropAsString(AdUnitOptionsProp_PublisherID);
end;

function TAdUnitOptions.GetVisible: Boolean;
begin
  Result:=GetPropAsBoolean(AdUnitOptionsProp_Visible);
end;

procedure TAdUnitOptions.SetChannelNumber(Value: String);
begin
  SetProp(AdUnitOptionsProp_ChannelNumber,Value,true);
end;

procedure TAdUnitOptions.SetFormat(Value: String);
begin
  SetProp(AdUnitOptionsProp_Format,Value,true);
end;

procedure TAdUnitOptions.SetMap(Value: TMap);
begin
  SetProp(AdUnitOptionsProp_Map,Value,true);
end;

procedure TAdUnitOptions.SetPosition(Value: Integer);
begin
  SetProp(AdUnitOptionsProp_Position,Value,true);
end;

procedure TAdUnitOptions.SetPublisherID(Value: String);
begin
  SetProp(AdUnitOptionsProp_PublisherID,Value,true);
end;

procedure TAdUnitOptions.SetVisible(Value: Boolean);
begin
  SetProp(AdUnitOptionsProp_Visible,Value,true);
end;

end.
