///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Geometry|TEncoding"/></item>
///    <item><see cref="gmLibraries.Geometry|TGeometry"/></item>
///    <item><see cref="gmLibraries.Geometry|TSpherical"/></item>
///  </list>
///  <para/>
///  <para><b>Constants</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Geometry|EarthRadius"/></item>
///  </list>
///</summary>
unit gmLibrariesGeometry;

interface

{$INCLUDE gmConfig.inc}

uses
  BrowserTools, JScriptObjects, gmBase, gmMVC, gmAdditionalConstants;

const
  {$IFDEF USE_REGIONS}{$REGION 'Documentation EarthRadius'}{$ENDIF}
  ///<summary>
  ///  6378137 m
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  EarthRadius = 6378137;                                   //Meter

type
  TEncoding = class;
  TGeometry = class;
  TSpherical = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TEncoding'}{$ENDIF}
  ///<summary>
  ///  Utilities for polyline encoding and decoding.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Geometry"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Geometry|TEncoding.DecodePath(String)">DecodePath</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TEncoding.EncodePath(TLatLngArray)">EncodePath(TLatLngArray)</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TEncoding.EncodePath(TLatLngMVCArray)">EncodePath(TLatLngMVCArray)</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmLibraries.Geometry|TGeometry">TGeometry</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/utilities/polylinealgorithm.html">
  ///  Encoded Polyline Algorithm Format
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Encoding">
  ///  Encoding
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TEncoding = class(TCustomJScriptObject)
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DecodePath'}{$ENDIF}
    ///<summary>
    ///  Decodes an encoded path string into a sequence of
    ///  <see cref="gmBase|TLatLng">TLatLng</see>'s.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray">TLatLngArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TEncoding">TEncoding</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function DecodePath(EncodedPath: String): TLatLngArray;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DecodePath'}{$ENDIF}
    ///<summary>
    ///  Encodes a sequence of <see cref="gmBase|TLatLng">TLatLng</see>'s
    ///  into an encoded path string.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray">TLatLngArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TEncoding">TEncoding</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function EncodePath(Path: TLatLngArray): String; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DecodePath'}{$ENDIF}
    ///<summary>
    ///  Encodes a sequence of <see cref="gmBase|TLatLng">TLatLng</see>'s
    ///  into an encoded path string.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TEncoding">TEncoding</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function EncodePath(Path: TLatLngMVCArray): String; overload;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TGeometry'}{$ENDIF}
  ///<summary>
  ///  Library.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Geometry"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Geometry|TGeometry.Encoding">Encoding</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TGeometry.Spherical">Spherical</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/geometry.html">
  ///  Geometry
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Geometry">
  ///  Encoding
  ///</seealso>
  ///<seealso cref="gmAPI|TMaps.Geometry">Google.Maps.Geometry</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TGeometry = class(TCustomJScriptObject)
  private
    FEncoding: TEncoding;
    FSpherical: TSpherical;
    function GetEncoding: TEncoding;
    function GetSpherical: TSpherical;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Encoding'}{$ENDIF}
    ///<summary>
    ///  Utilities for polyline encoding and decoding.
    ///</summary>
    ///<seealso cref="gmLibraries.Geometry|TGeometry">TGeometry</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Encoding: TEncoding read GetEncoding;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Spherical'}{$ENDIF}
    ///<summary>
    ///  Utility functions for computing geodesic angles, distances and areas.
    ///</summary>
    ///<seealso cref="gmLibraries.Geometry|TGeometry">TGeometry</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Spherical: TSpherical read GetSpherical;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TSpherical'}{$ENDIF}
  ///<summary>
  ///  Utility functions for computing geodesic angles, distances and areas.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Geometry"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeArea(TLatLngArray,Integer)">ComputeArea(TLatLngArray,Integer)</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeArea(TLatLngMVCArray,Integer)">ComputeArea(TLatLngMVCArray,Integer)</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeDistanceBetween(TLatLng,TLatLng,Integer)">ComputeDistanceBetween</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeHeading(TLatLng,TLatLng)"></see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeLength(TLatLngArray,Integer)">ComputeLength(TLatLngArray,Integer)</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeLength(TLatLngMVCArray,Integer)">ComputeLength(TLatLngMVCArray,Integer)</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeOffset(TLatLng,Double,Double,Integer)">ComputeOffset</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeSignedArea(TLatLngArray,Integer)">ComputeSignedArea(TLatLngArray,Integer)</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.ComputeSignedArea(TLatLngMVCArray,Integer)">ComputeSignedArea(TLatLngMVCArray,Integer)</see></item>
  ///    <item><see cref="gmLibraries.Geometry|TSpherical.Interpolate(TLatLng,TLatLng,Double)">Interpolate</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Spherical">
  ///  Spherical
  ///</seealso>
  ///<seealso cref="gmLibraries.Geometry|EarthRadius">EarthRadius</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TSpherical = class(TCustomJScriptObject)
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeArea'}{$ENDIF}
    ///<summary>
    ///  Computes the area of the given loop. Loops must be closed.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray">TLatLngArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeArea(Loop: TLatLngArray; Radius: Integer = EarthRadius): Double; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeArea'}{$ENDIF}
    ///<summary>
    ///  Computes the area of the given loop. Loops must be closed.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeArea(Loop: TLatLngMVCArray; Radius: Integer = EarthRadius): Double; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeDistanceBetween'}{$ENDIF}
    ///<summary>
    ///  Computes the distance between two <see cref="gmBase|TLatLng">TLatLng</see>'s.
    ///</summary>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeDistanceBetween(From, {$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}: TLatLng; Radius: Integer = EarthRadius): Double;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeHeading'}{$ENDIF}
    ///<summary>
    ///  Computes the heading from one <see cref="gmBase|TLatLng">TLatLng</see>
    ///  to another <i>TLatLng</i>.
    ///</summary>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeHeading(From, {$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}: TLatLng): Double;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeLength'}{$ENDIF}
    ///<summary>
    ///  Computes the length of the given path.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray">TLatLngArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeLength(Path: TLatLngArray; Radius: Integer = EarthRadius): Double; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeLength'}{$ENDIF}
    ///<summary>
    ///  Computes the length of the given path.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeLength(Path: TLatLngMVCArray; Radius: Integer = EarthRadius): Double; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeOffset'}{$ENDIF}
    ///<summary>
    ///  Computes the <see cref="gmBase|TLatLng">TLatLng</see> produced by starting from a
    ///  given <i>TLatLng</i> and heading a given distance.
    ///</summary>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeOffset(From: TLatLng; Distance, Heading: Double; Radius: Integer = EarthRadius): TLatLng;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeSignedArea'}{$ENDIF}
    ///<summary>
    ///  Computes the signed area of the given loop. Loops must be closed. The
    ///  signed area may be used to determine the orientation of a loop.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray">TLatLngArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeSignedArea(Loop: TLatLngArray; Radius: Integer = EarthRadius): Double; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ComputeSignedArea'}{$ENDIF}
    ///<summary>
    ///  Computes the signed area of the given loop. Loops must be closed. The
    ///  signed area may be used to determine the orientation of a loop.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function ComputeSignedArea(Loop: TLatLngMVCArray; Radius: Integer = EarthRadius): Double; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Interpolate'}{$ENDIF}
    ///<summary>
    ///  Travels a fraction of the way from one
    ///  <see cref="gmBase|TLatLng">TLatLng</see> to another <i>TLatLng</i>.
    ///</summary>
    ///<seealso cref="gmLibraries.Geometry|TSpherical">TSpherical</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Interpolate(From, {$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}: TLatLng; Fraction: Double): TLatLng;
  end;

implementation

//------------------------------------------------------------------------------
//TEncoding

function TEncoding.DecodePath(EncodedPath: String): TLatLngArray;
begin
  Result:=TLatLngArray(CreateObjectWrapper(Script,TLatLngArray,
                                           BrowserTools.Exec(Disp,EncodingMethod_DecodePath,
                                                             EncodedPath,true)));
end;

function TEncoding.EncodePath(Path: TLatLngArray): String;
begin
  Result:=BrowserTools.Exec(Disp,EncodingMethod_EncodePath,Path.DispEx,true);
end;

function TEncoding.EncodePath(Path: TLatLngMVCArray): String;
begin
  Result:=BrowserTools.Exec(Disp,EncodingMethod_EncodePath,Path.DispEx,true);
end;

//------------------------------------------------------------------------------
//TGeometry

destructor TGeometry.Destroy;
begin
  FEncoding.Free;
  FSpherical.Free;
  inherited;
end;

function TGeometry.GetEncoding: TEncoding;
begin
  Result:=TEncoding(GetPropAsObject(GeometryProp_Encoding,TEncoding,FEncoding));
end;

function TGeometry.GetSpherical: TSpherical;
begin
  Result:=TSpherical(GetPropAsObject(GeometryProp_Spherical,TSpherical,FSpherical));
end;

//------------------------------------------------------------------------------
//TSpherical

function TSpherical.ComputeArea(Loop: TLatLngMVCArray; Radius: Integer): Double;
begin
  Result:=BrowserTools.Exec(Disp,SphericalMethod_ComputeArea,[Loop.DispEx,Radius],true);
end;

function TSpherical.ComputeArea(Loop: TLatLngArray; Radius: Integer): Double;
begin
  Result:=BrowserTools.Exec(Disp,SphericalMethod_ComputeArea,[Loop.DispEx,Radius],true);
end;

function TSpherical.ComputeDistanceBetween(From, {$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}: TLatLng;
                                           Radius: Integer): Double;
begin
  Result:=BrowserTools.Exec(Disp,SphericalMethod_ComputeDistanceBetween,
                            [From.DispEx,{$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}.DispEx,Radius],
                            true);
end;

function TSpherical.ComputeHeading(From, {$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}: TLatLng): Double;
begin
  Result:=BrowserTools.Exec(Disp,SphericalMethod_ComputeHeading,
                           [From.DispEx,{$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}.DispEx],
                           true);
end;

function TSpherical.ComputeLength(Path: TLatLngMVCArray; Radius: Integer): Double;
begin
  Result:=BrowserTools.Exec(Disp,SphericalMethod_ComputeLength,[Path.DispEx,Radius],true);
end;

function TSpherical.ComputeLength(Path: TLatLngArray; Radius: Integer): Double;
begin
  Result:=BrowserTools.Exec(Disp,SphericalMethod_ComputeLength,[Path.DispEx,Radius],true);
end;

function TSpherical.ComputeOffset(From: TLatLng; Distance, Heading: Double; Radius: Integer): TLatLng;
begin
  Result:=TLatLng(CreateObjectWrapper(Script,TLatLng,
                                      BrowserTools.Exec(Disp,SphericalMethod_ComputeOffset,
                                                        [From.DispEx,Distance,Heading,Radius],
                                                        true)));                        
end;

function TSpherical.ComputeSignedArea(Loop: TLatLngMVCArray; Radius: Integer): Double;
begin
  Result:=BrowserTools.Exec(Disp,SphericalMethod_ComputeSignedArea,[Loop.DispEx,Radius],true);
end;

function TSpherical.ComputeSignedArea(Loop: TLatLngArray; Radius: Integer): Double;
begin
  Result:=BrowserTools.Exec(Disp,SphericalMethod_ComputeSignedArea,[Loop.DispEx,Radius],true);
end;

function TSpherical.Interpolate(From, {$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}: TLatLng; Fraction: Double): TLatLng;
begin
  Result:=TLatLng(CreateObjectWrapper(Script,TLatLng,
                                      BrowserTools.Exec(Disp,SphericalMethod_Interpolate,
                                      [From.DispEx,{$IFDEF AND_OP}&To{$ELSE}To_{$ENDIF}.DispEx,Fraction],
                                      true)));
end;

end.
