///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Panoramio|TPanoramio"/></item>
///    <item><see cref="gmLibraries.Panoramio|TPanoramioFeature"/></item>
///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer"/></item>
///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayerOptions"/></item>
///    <item><see cref="gmLibraries.Panoramio|TPanoramioMouseEvent"/></item>
///  </list>
///  <para><b>Types</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Panoramio|TPanoramioClickEvent"/></item>
///  </list>
///  <para><b>Functions</b></para>
///  <list type="bullet">
///    <item><see cref="gmLibraries.Panoramio|New(TPanoramioLayer)">New: TPanoramioLayer</see></item>
///  </list>
///</summary>
unit gmLibrariesPanoramio;

{$INCLUDE gmConfig.inc}

interface

uses
  SysUtils, {$IFDEF DEBUG}Dialogs,{$ENDIF} BrowserTools,
  {$IFDEF USE_ANONYMOUS_METHODS}AnonymousMethodsTools,{$ENDIF} JScriptObjects,
  gmBase, gmMap, gmMVC, gmLayers, gmEvents, gmAdditionalConstants;

type
  TPanoramio = class;
  TPanoramioFeature = class;
  TPanoramioLayer = class;
  TPanoramioLayerOptions = class;
  TPanoramioMouseEvent = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPanoramio'}{$ENDIF}
  ///<summary>
  ///  Library.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Panoramio"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramio.PanoramioLayer(TPanoramioLayerOptions)">PanoramioLayer</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html">Panoramio</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPanoramio = class(TCustomJScriptObject)
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PanoramioLayer'}{$ENDIF}
    ///<summary>
    ///  A layer that displays data from Panoramio.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramio"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function PanoramioLayer(Opts: TPanoramioLayerOptions = nil): TPanoramioLayer;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPanoramioFeature'}{$ENDIF}
  ///<summary>
  ///  Describes a single Panoramio feature.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Panoramio"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioFeature.Author">Author</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioFeature.PhotoID">PhotoID</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioFeature.Title">Title</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioFeature.URL">URL</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioFeature.UserID">UserID</see></item>
  ///  </list>
  ///</summary>
  ///<example>See <see cref="gmLibraries.Panoramio|TPanoramioLayer">TPanoramioLayer</see></example>
  ///<seealso cref="gmLibraries.Panoramio|TPanoramioMouseEvent"/>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PanoramioFeature">PanoramioFeature</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPanoramioFeature = class(TCustomJScriptObject)
  private
    procedure SetAuthor(Value: String);
    procedure SetPhotoID(Value: Integer);
    procedure SetTitle(Value: String);
    procedure SetURL(Value: String);
    procedure SetUserID(Value: Integer);
    function GetAuthor: String;
    function GetPhotoID: Integer;
    function GetTitle: String;
    function GetURL: String;
    function GetUserID: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Author'}{$ENDIF}
    ///<summary>
    ///  The username of the user who uploaded this photo.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioFeature"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Author: String read GetAuthor write SetAuthor;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PhotoID'}{$ENDIF}
    ///<summary>
    ///  The unique identifier for this photo, as used in the Panoramio API.
    ///</summary>
    ///<seealso href="http://www.panoramio.com/api/widget/api.html">Panoramio API</seealso>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioFeature"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PhotoID: Integer read GetPhotoID write SetPhotoID;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Title'}{$ENDIF}
    ///<summary>
    ///  The title of the photo.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioFeature"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Title: String read GetTitle write SetTitle;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation URL'}{$ENDIF}
    ///<summary>
    ///  The URL of the photo.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioFeature"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property URL: String read GetURL write SetURL;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UserID'}{$ENDIF}
    ///<summary>
    ///  The unique identifier for the user who uploaded this photo, as used in
    ///  the Panoramio API.
    ///</summary>
    ///<seealso href="http://www.panoramio.com/api/widget/api.html">Panoramio API</seealso>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioFeature"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UserID: Integer read GetUserID write SetUserID;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPanoramioClickEvent'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; PanoramioMouseEvent: </c>
  ///  <see cref="gmLibraries.Panoramio|TPanoramioMouseEvent"/><c>) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Panoramio"/>
  ///  <para/>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TPanoramioClickEvent = TProc<TObject,TPanoramioMouseEvent>;
  {$ELSE}
  TPanoramioClickEvent = procedure(Sender: TObject; PanoramioMouseEvent: TPanoramioMouseEvent) of object;
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPanoramioLayer'}{$ENDIF}
  ///<summary>
  ///  A PanoramioLayer displays photos from Panoramio as a rendered layer.
  ///  This class extends <see cref="gmMVC|TMVCObject"/>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Panoramio"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmLayers|TLayer"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramio.PanoramioLayer(TPanoramioLayerOptions)">Google.Maps.Panoramio.PanoramioLayer</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.GetMap">GetMap</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.GetTag">GetTag</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.GetUserID">GetUserID</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.SetMap(TMap)">SetMap</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.SetOptions(TPanoramioLayerOptions)">SetOptions</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.SetTag(String)">SetTag</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.SetUserID(String)">SetUserID</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.Map">Map</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.Tag">Tag</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.UserID">UserID</see></item>
  ///  </list>
  ///  <para><b>Events</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayer.OnClick">OnClick</see></item>
  ///  </list>
  ///</summary>
  ///<example>
  ///  <code lang="Delphi">
  ///procedure TForm1.InitMap(Sender: TObject);
  ///var
  ///  PanoramioLayer: TPanoramioLayer;
  ///  PanoramioLayerOptions: TPanoramioLayerOptions;
  ///  [...]
  ///begin
  ///  with TScript(Sender) do
  ///  begin
  ///    [...]
  ///    PanoramioLayerOptions:=TPanoramioLayerOptions.Create;
  ///    with PanoramioLayerOptions do
  ///    begin
  ///      Clickable:=true;
  ///      Map:=Maps[0];
  ///      SuppressInfoWindows:=true;
  ///    end;
  ///    PanoramioLayer:=New(Google.Maps.Panoramio.PanoramioLayer(PanoramioLayerOptions));
  ///    PanoramioLayer.OnClick:=PanoramioLayerClick;
  ///    [...]
  ///  end;
  ///end;
  ///
  ///procedure TForm1.PanoramioLayerClick(Sender: TObject; PanoramioMouseEvent: TPanoramioMouseEvent);
  ///begin
  ///  Label1.Caption:=PanoramioMouseEvent.FeatureDetails.Title;
  ///end;</code>
  ///</example>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PanoramioLayer">PanoramioLayer</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPanoramioLayer = class(TLayer)
  private
    FMap: TMap;
    FOnClick: TPanoramioClickEvent;
    FOnClickListener: TMapsEventListener;
    FOnClickFunction: TJScriptFunction;
    procedure SetOnClick(Value: TPanoramioClickEvent);
  protected
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; Opts: TPanoramioLayerOptions): IDispatch; reintroduce; overload;
    //JavaScript event handler:
    procedure Click;
  public
    constructor Create(Opts: TPanoramioLayerOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TPanoramioLayerOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetMap'}{$ENDIF}
    ///<summary>
    ///  Renders the layer on the specified map. If map is set to nil, the layer
    ///  will be removed.
    ///</summary>
    ///<seealso cref="gmMap|TMap"/>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetMap(Map: TMap);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetOptions'}{$ENDIF}
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetOptions(Options: TPanoramioLayerOptions); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetTag'}{$ENDIF}
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetTag(Tag: String);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetUserID'}{$ENDIF}
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetUserID(UserID: String);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetMap'}{$ENDIF}
    ///<summary>
    ///  Returns the map on which this layer is displayed.
    ///</summary>
    ///<seealso cref="gmMap|TMap"/>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetMap: TMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetTag'}{$ENDIF}
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetTag: String;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetUserID'}{$ENDIF}
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetUserID: String;
    //Additional Delphi properties:
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Map'}{$ENDIF}
    ///<summary>
    ///  Additional Delphi property.
    ///</summary>
    ///<seealso cref="gmMap|TMap"/>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Map: TMap read GetMap write SetMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Tag'}{$ENDIF}
    ///<summary>
    ///  Additional Delphi property.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Tag: String read GetTag write SetTag;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UserID'}{$ENDIF}
    ///<summary>
    ///  Additional Delphi property.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UserID: String read GetUserID write SetUserID;
    //Events:
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnClick'}{$ENDIF}
    ///<summary>
    ///  This event is fired when a feature in the layer is clicked.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnClick: TPanoramioClickEvent read FOnClick write SetOnClick;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPanoramioLayerOptions'}{$ENDIF}
  ///<summary>
  ///  This object defines the properties that can be set on a PanoramioLayer object.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Panoramio"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayerOptions.Clickable">Clickable</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayerOptions.Map">Map</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayerOptions.SuppressInfoWindows">SuppressInfoWindows</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayerOptions.Tag">Tag</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioLayerOptions.UserID">UserID</see></item>
  ///  </list>
  ///</summary>
  ///<example>See <see cref="gmLibraries.Panoramio|TPanoramioLayer">TPanoramioLayer</see></example>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PanoramioLayerOptions">PanoramioLayerOptions</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPanoramioLayerOptions = class(TCustomJScriptObject)
  private
    FMap: TMap;
    procedure SetClickable(Value: Boolean);
    procedure SetMap(Value: TMap);
    procedure SetSuppressInfoWindows(Value: Boolean);
    procedure SetTag(Value: String);
    procedure SetUserID(Value: String);
    function GetClickable: Boolean;
    function GetMap: TMap;
    function GetSuppressInfoWindows: Boolean;
    function GetTag: String;
    function GetUserID: String;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Clickable'}{$ENDIF}
    ///<summary>
    ///  If true, the layer receives mouse events. Default value is true.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayerOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Clickable: Boolean read GetClickable write SetClickable;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Map'}{$ENDIF}
    ///<summary>
    ///  The map on which to display the layer.
    ///</summary>
    ///<seealso cref="gmMap|TMap"/>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayerOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Map: TMap read GetMap write SetMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SuppressInfoWindows'}{$ENDIF}
    ///<summary>
    ///  Suppress the rendering of info windows when layer features are clicked.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayerOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property SuppressInfoWindows: Boolean read GetSuppressInfoWindows write SetSuppressInfoWindows;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Tag'}{$ENDIF}
    ///<summary>
    ///  A panoramio tag used to filter the photos which are displayed. Only
    ///  photos which have been tagged with the supplied string will be shown.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayerOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Tag: String read GetTag write SetTag;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UserID'}{$ENDIF}
    ///<summary>
    ///  A Panoramio user ID. If provided, only photos by this user will be
    ///  displayed on the map. If both a tag and user ID are provided, the tag
    ///  will take precedence.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioLayerOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UserID: String read GetUserID write SetUserID;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPanoramioMouseEvent'}{$ENDIF}
  ///<summary>
  ///  The properties of a mouse event on a PanoramioLayer.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmLibraries.Panoramio"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioMouseEvent.FeatureDetails">FeatureDetails</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioMouseEvent.InfoWindowHTML">InfoWindowHTML</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioMouseEvent.LatLng">LatLng</see></item>
  ///    <item><see cref="gmLibraries.Panoramio|TPanoramioMouseEvent.PixelOffset">PixelOffset</see></item>
  ///  </list>
  ///</summary>
  ///<example>See <see cref="gmLibraries.Panoramio|TPanoramioLayer">TPanoramioLayer</see></example>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PanoramioMouseEvent">PanoramioMouseEvent</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPanoramioMouseEvent = class(TCustomJScriptObject)
  private
    FFeatureDetails: TPanoramioFeature;
    FLatLng: TLatLng;
    FPixelOffset: TSize;
    procedure SetFeatureDetails(Value: TPanoramioFeature);
    procedure SetInfoWindowHTML(Value: String);
    procedure SetLatLng(Value: TLatLng);
    procedure SetPixelOffset(Value: TSize);
    function GetFeatureDetails: TPanoramioFeature;
    function GetInfoWindowHTML: String;
    function GetLatLng: TLatLng;
    function GetPixelOffset: TSize;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation FeatureDetails'}{$ENDIF}
    ///<summary>
    ///  A PanoramioFeature object containing information about the clicked
    ///  feature.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioMouseEvent"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property FeatureDetails: TPanoramioFeature read GetFeatureDetails write SetFeatureDetails;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InfoWindowHTML'}{$ENDIF}
    ///<summary>
    ///  Pre-rendered HTML content to display within a feature's InfoWindow
    ///  when clicked.
    ///</summary>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioMouseEvent"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property InfoWindowHTML: String read GetInfoWindowHTML write SetInfoWindowHTML;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LatLng'}{$ENDIF}
    ///<summary>
    ///  The position at which to anchor an info window on the clicked feature.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioMouseEvent"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property LatLng: TLatLng read GetLatLng write SetLatLng;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PixelOffset'}{$ENDIF}
    ///<summary>
    ///  The offset to apply to an info window anchored on the clicked feature.
    ///</summary>
    ///<seealso cref="gmBase|TSize"/>
    ///<seealso cref="gmLibraries.Panoramio|TPanoramioMouseEvent"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PixelOffset: TSize read GetPixelOffset write SetPixelOffset;
  end;

{$IFDEF USE_REGIONS}{$REGION 'Documentation New'}{$ENDIF}
///<summary>
///  <para><b>Unit</b></para>
///    <see cref="gmLibraries.Panoramio"/>
///  <para/>
///</summary>
{$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
function New(Value: TPanoramioLayer): TPanoramioLayer; overload;

implementation

uses
  gmAPI;

function New(Value: TPanoramioLayer): TPanoramioLayer;     //Syntax-Helper
begin
  Result:=Value;
end;

//------------------------------------------------------------------------------
//TPanoramio

function TPanoramio.PanoramioLayer(Opts: TPanoramioLayerOptions): TPanoramioLayer;
begin
  Result:=TPanoramioLayer.Create(Script,Opts);
end;

//------------------------------------------------------------------------------
//TPanoramioFeature

function TPanoramioFeature.GetAuthor: String;
begin
  Result:=GetPropAsString(PanoramioFeatureProp_Author);
end;

function TPanoramioFeature.GetPhotoID: Integer;
begin
  Result:=GetPropAsInteger(PanoramioFeatureProp_PhotoID);
end;

function TPanoramioFeature.GetTitle: String;
begin
  Result:=GetPropAsString(PanoramioFeatureProp_Title);
end;

function TPanoramioFeature.GetURL: String;
begin
  Result:=GetPropAsString(PanoramioFeatureProp_URL);
end;

function TPanoramioFeature.GetUserID: Integer;
begin
  Result:=GetPropAsInteger(PanoramioFeatureProp_UserID);
end;

procedure TPanoramioFeature.SetAuthor(Value: String);
begin
  SetProp(PanoramioFeatureProp_Author,Value,true);
end;

procedure TPanoramioFeature.SetPhotoID(Value: Integer);
begin
  SetProp(PanoramioFeatureProp_PhotoID,Value,true);
end;

procedure TPanoramioFeature.SetTitle(Value: String);
begin
  SetProp(PanoramioFeatureProp_Title,Value,true);
end;

procedure TPanoramioFeature.SetURL(Value: String);
begin
  SetProp(PanoramioFeatureProp_URL,Value,true);
end;

procedure TPanoramioFeature.SetUserID(Value: Integer);
begin
  SetProp(PanoramioFeatureProp_UserID,Value,true);
end;

//------------------------------------------------------------------------------
//TPanoramioLayer

constructor TPanoramioLayer.Create(Opts: TPanoramioLayerOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TPanoramioLayer.Create(Script: TCustomScript; Opts: TPanoramioLayerOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TPanoramioLayer.Destroy;
begin
  SetOnClick(nil);
  FMap.Free;
  inherited;
end;

class function TPanoramioLayer.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Panoramio.Disp,
                              PanoramioMethod_PanoramioLayer,true);

end;

class function TPanoramioLayer.New(Script: TCustomScript;
                                   Opts: TPanoramioLayerOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Panoramio.Disp,
                                     PanoramioMethod_PanoramioLayer,Opts.Disp,
                                     true);
end;

procedure TPanoramioLayer.Click;
var
  EventObject: TPanoramioMouseEvent;
begin
  if assigned(FOnClick) then
  begin
    try
      EventObject:=TPanoramioMouseEvent(CreateObjectWrapper(Script,
                                                            TPanoramioMouseEvent,
                                                            FOnClickFunction.Arguments[0]));
      try
        FOnClick(self,EventObject);
      finally
        //EventObject.Free;
      end;
    except
      {$IFDEF DEBUG}
      ShowMessage(Exception(ExceptObject).Message);
      {$ENDIF}
    end;
  end;
end;

function TPanoramioLayer.GetMap: TMap;
var
  Map: OleVariant;
begin
  Result:=nil;
  Map:=Exec(PanoramioLayerMethod_GetMap);
  if VarIsValidDispatch(Map) and TMap.IsObjectFromDisp(Map) then
  begin
    if assigned(FMap)
      then FMap.Disp:=Map
      else FMap:=TMap(CreateObjectWrapper(Script,TMap,Map));
    Result:=FMap;
  end;
end;

function TPanoramioLayer.GetTag: String;
begin
  Result:=VarAsString(Exec(PanoramioLayerMethod_GetTag));
end;

function TPanoramioLayer.GetUserID: String;
begin
  Result:=VarAsString(Exec(PanoramioLayerMethod_GetUserID));
end;

procedure TPanoramioLayer.SetMap(Map: TMap);
begin
  Exec(PanoramioLayerMethod_SetMap,Map);
end;

procedure TPanoramioLayer.SetOnClick(Value: TPanoramioClickEvent);
begin
  FOnClick:=Value;
  RemoveListener(FOnClickListener);
  if assigned(FOnClick)
    then AddListener(PanoramioLayerEvent_Click,FOnClickListener,FOnClickFunction,Click);
end;

procedure TPanoramioLayer.SetOptions(Options: TPanoramioLayerOptions);
begin
  Exec(PanoramioLayerMethod_SetOptions,Options);
end;

procedure TPanoramioLayer.SetTag(Tag: String);
begin
  Exec(PanoramioLayerMethod_SetTag,[Tag]);
end;

procedure TPanoramioLayer.SetUserID(UserID: String);
begin
  Exec(PanoramioLayerMethod_SetUserID,[UserID]);
end;

//------------------------------------------------------------------------------
//TPanoramioLayerOptions

constructor TPanoramioLayerOptions.Create;
begin
  inherited;
end;

constructor TPanoramioLayerOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TPanoramioLayerOptions.Destroy;
begin
  FMap.Free;
  inherited;
end;

function TPanoramioLayerOptions.GetClickable: Boolean;
begin
  Result:=GetPropAsBoolean(PanoramioLayerOptionsProp_Clickable);
end;

function TPanoramioLayerOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(PanoramioLayerOptionsProp_Map,TMap,FMap));
end;

function TPanoramioLayerOptions.GetSuppressInfoWindows: Boolean;
begin
  Result:=GetPropAsBoolean(PanoramioLayerOptionsProp_SuppressInfoWindows);
end;

function TPanoramioLayerOptions.GetTag: String;
begin
  Result:=GetPropAsString(PanoramioLayerOptionsProp_Tag);
end;

function TPanoramioLayerOptions.GetUserID: String;
begin
  Result:=GetPropAsString(PanoramioLayerOptionsProp_UserID);
end;

procedure TPanoramioLayerOptions.SetClickable(Value: Boolean);
begin
  SetProp(PanoramioLayerOptionsProp_Clickable,Value,true);
end;

procedure TPanoramioLayerOptions.SetMap(Value: TMap);
begin
  SetProp(PanoramioLayerOptionsProp_Map,Value,true);
end;

procedure TPanoramioLayerOptions.SetSuppressInfoWindows(Value: Boolean);
begin
  SetProp(PanoramioLayerOptionsProp_SuppressInfoWindows,Value,true);
end;

procedure TPanoramioLayerOptions.SetTag(Value: String);
begin
  SetProp(PanoramioLayerOptionsProp_Tag,Value,true);
end;

procedure TPanoramioLayerOptions.SetUserID(Value: String);
begin
  SetProp(PanoramioLayerOptionsProp_UserID,Value,true);
end;

//------------------------------------------------------------------------------
//TPanoramioMouseEvent

destructor TPanoramioMouseEvent.Destroy;
begin
  FFeatureDetails.Free;
  FLatLng.Free;
  FPixelOffset.Free;
  inherited;
end;

function TPanoramioMouseEvent.GetFeatureDetails: TPanoramioFeature;
begin
  Result:=TPanoramioFeature(GetPropAsObject(PanoramioMouseEventProp_FeatureDetails,
                                            TPanoramioFeature,FFeatureDetails));
end;

function TPanoramioMouseEvent.GetInfoWindowHTML: String;
begin
  Result:=GetPropAsString(PanoramioMouseEventProp_InfoWindowHTML);
end;

function TPanoramioMouseEvent.GetLatLng: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(PanoramioMouseEventProp_LatLng,
                                  TLatLng,FLatLng));
end;

function TPanoramioMouseEvent.GetPixelOffset: TSize;
begin
  Result:=TSize(GetPropAsObject(PanoramioMouseEventProp_PixelOffset,
                                TSize,FPixelOffset));
end;

procedure TPanoramioMouseEvent.SetFeatureDetails(Value: TPanoramioFeature);
begin
  SetProp(PanoramioMouseEventProp_FeatureDetails,Value,true);
end;

procedure TPanoramioMouseEvent.SetInfoWindowHTML(Value: String);
begin
  SetProp(PanoramioMouseEventProp_InfoWindowHTML,Value,true);
end;

procedure TPanoramioMouseEvent.SetLatLng(Value: TLatLng);
begin
  SetProp(PanoramioMouseEventProp_LatLng,Value,true);
end;

procedure TPanoramioMouseEvent.SetPixelOffset(Value: TSize);
begin
  SetProp(PanoramioMouseEventProp_PixelOffset,Value,true);
end;

end.
