///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmMVC|TMVCArray"/></item>
///    <item><see cref="gmMVC|TMVCObject"/></item>
///  </list>
///  <para/>
///  <para><b>Additional Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmMVC|TLatLngMVCArray"/></item>
///    <item><see cref="gmMVC|TLatLngMVCArrayMVCArray"/></item>
///    <item><see cref="gmMVC|TMVCEventObject"/></item>
///    <item><see cref="gmMVC|TNodeMVCArray"/></item>
///    <item><see cref="gmMVC|TNodeMVCArrayArray"/></item>
///    <item><see cref="gmMVC|TObjectMVCArray"/></item>
///  </list>
///  <para/>
///  <para><b>Types</b></para>
///  <list type="bullet">
///    <item><see cref="gmMVC|TCallbackMethod"/></item>
///  </list>
///</summary>
unit gmMVC;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF}
  {$IFDEF DEBUG}Dialogs,{$ENDIF} DispObject, BrowserTools, JScriptObjects,
  HTMLObjects, gmBase, gmEvents, gmAdditionalConstants;

type
  TLatLngMVCArray = class;                                 //MVCArray.<LatLng>
  TLatLngMVCArrayMVCArray = class;                         //MVCArray.<MVCArray.<LatLng>>
  TNodeMVCArray = class;                                   //MVCArray.<Node>
  TNodeMVCArrayArray = class;                              //Array.<MVCArray.<Node>>
  TObjectMVCArray = class;
  TMVCArray = class;
  TMVCEventObject = class;
  TMVCObject = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMVCObject'}{$ENDIF}
  ///<summary>
  ///  Base class implementing KVO.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmMVC"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmAPI|TMaps.MVCObject">Google.Maps.MVCObject</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TMVCObject.BindTo(String,TMVCObject)">BindTo(String,TMVCObject)</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.BindTo(String,TMVCObject,String)">BindTo(String,TMVCObject,String)</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.BindTo(String,TMVCObject,String,Boolean)">BindTo(String,TMVCObject,String,Boolean)</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.Changed(String)">Changed</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.Get(String)">Get</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.Notify(String)">Notify</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.Set_(String,OleVariant)">Set</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.SetValues(TCustomJScriptObject)">SetValues</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.Unbind(String)">Unbind</see></item>
  ///    <item><see cref="gmMVC|TMVCObject.UnbindAll">UnbindAll</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#MVCObject">
  ///  MVCObject (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#MVCObject">
  ///  MVCObject (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMVCObject = class(TCustomJScriptObject)
  protected
    procedure AddListener(const EventName: String; var Listener: TMapsEventListener;
                          var ListenerFunction: TJScriptFunction; Receiver: TExternalProc);
    procedure CallEvent(Event: TNotifyEvent); overload;
    procedure CallEvent(Event: TMapMouseEvent; EventData: IDispatch); overload;
    procedure CallEvent(Event: TMapEventEvent; EventData: IDispatch); overload;
    procedure ClearInstanceListeners;
    procedure RemoveListener(var Listener: TMapsEventListener);
    procedure SetOptions(Options: TCustomJScriptObject); virtual;
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation BindTo'}{$ENDIF}
    ///<summary>
    ///  Binds a View to a Model.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure BindTo(const Key: String; Target: TMVCObject); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation BindTo'}{$ENDIF}
    ///<summary>
    ///  Binds a View to a Model.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure BindTo(const Key: String; Target: TMVCObject; const TargetKey: String); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation BindTo'}{$ENDIF}
    ///<summary>
    ///  Binds a View to a Model.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure BindTo(const Key: String; Target: TMVCObject; const TargetKey: String; NoNotify: Boolean); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Changed'}{$ENDIF}
    ///<summary>
    ///  Generic handler for state changes. Override this in derived classes to
    ///  handle arbitrary state changes.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Changed(const Key: String);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Notify'}{$ENDIF}
    ///<summary>
    ///  Notify all observers of a change on this property. This notifies both
    ///  objects that are bound to the object's property as well as the object that
    ///  it is bound to.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Notify(const Key: String);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Set'}{$ENDIF}
    ///<summary>
    ///  Sets a value.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure {$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}(const Key: String; const Value: OleVariant); virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetValues'}{$ENDIF}
    ///<summary>
    ///  Sets a collection of key-value pairs.
    ///</summary>
    ///<seealso cref="JScriptObjects|TCustomJScriptObject">TCustomJScriptObject</seealso>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetValues(Values: TCustomJScriptObject);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Unbind'}{$ENDIF}
    ///<summary>
    ///  Removes a binding. Unbinding will set the unbound property to the current
    /// value. The object will not be notified, as the value has not changed.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Unbind(const Key: String);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UnbindAll'}{$ENDIF}
    ///<summary>
    ///  Removes all bindings.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure UnbindAll;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Get'}{$ENDIF}
    ///<summary>
    ///  Gets a value.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Get(const Key: String): OleVariant;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TCallbackMethod'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject">TObject</see><c>; Element: </c>
  ///  <see cref="DispObject|TDispObject">TDispObject</see>;
  ///  <c>Index: Integer) of object;</c>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TCallbackMethod = procedure(Sender: TObject; Element: TDispObject;
                              Index: Integer) of object;
  {$IFDEF USE_REGIONS}{$REGION 'Documentation TCallbackMethodA'}{$ENDIF}
  ///<summary>
  ///  <c>reference to procedure(Sender: </c><see cref="TObject">TObject</see><c>;
  ///  Element: </c>
  ///  <see cref="DispObject|TDispObject">TDispObject</see>;
  ///  <c>Index: Integer);</c>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TCallbackMethodA = reference to procedure(Sender: TObject; Element: TDispObject;
                                            Index: Integer);
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMVCArray'}{$ENDIF}
  ///<summary>
  ///  This class extends MVCObject.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmMVC"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmAPI|TMaps.MVCArray(array of OleVariant)">Google.Maps.MVCArray</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TMVCArray.Clear">Clear</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.Count">Count</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.ForEach(TCallbackMethod)">ForEach</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.GetArray">GetArray</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.GetAt(Integer)">GetAt</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.GetLength">GetLength</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.InsertAt(Integer,OleVariant)">InsertAt</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.Pop">Pop</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.Push(OleVariant)">Push(OleVariant)</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.Push(TDispObject)">Push(TDispObject)</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.RemoveAt(Integer)">RemoveAt</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.SetAt(Integer,OleVariant)">SetAt</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TMVCArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///  <para><b>Events</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TMVCArray.OnInsertAt">OnInsertAt</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.OnRemoveAt">OnRemoveAt</see></item>
  ///    <item><see cref="gmMVC|TMVCArray.OnSetAt">OnSetAt</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#MVCArray">
  ///  MVCArray (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#MVCArray">
  ///  MVCArray (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMVCArray = class(TMVCObject)
  private
    FCallback: TCallbackMethod;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FCallbackA: TCallbackMethodA;
    {$ENDIF}
    FCallbackFunction: TJScriptFunction;
    FOnInsertAt: TInsertAtEvent;
    FOnInsertAtListener: TMapsEventListener;
    FOnInsertAtFunction: TJScriptFunction;
    FOnRemoveAt: TRemoveAtEvent;
    FOnRemoveAtListener: TMapsEventListener;
    FOnRemoveAtFunction: TJScriptFunction;
    FOnSetAt: TSetAtEvent;
    FOnSetAtListener: TMapsEventListener;
    FOnSetAtFunction: TJScriptFunction;
    procedure SetOnInsertAt(Value: TInsertAtEvent);
    procedure SetOnRemoveAt(Value: TRemoveAtEvent);
    procedure SetOnSetAt(Value: TSetAtEvent);
  protected
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; const Elements: array of OleVariant): IDispatch; reintroduce; overload;
    //von JScript aufgerufene Event-Handler:
    //JavaScript event handler:
    procedure ForEachHandler;
    procedure InsertAtHandler;
    procedure RemoveAtHandler;
    procedure SetAtHandler;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    constructor Create(const Elements: array of OleVariant); reintroduce; overload; virtual;
    constructor Create(Script: TCustomScript; const Elements: array of OleVariant); reintroduce; overload; virtual;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Clear'}{$ENDIF}
    ///<summary>
    ///  Removes all elements from the array.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Clear;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ForEach'}{$ENDIF}
    ///<summary>
    ///  Iterate over each element, calling the provided callback. The callback
    ///  is called for each element like: <c>Callback(MVCArray,Element,Index)</c>.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure ForEach(Callback: TCallbackMethod); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ForEach'}{$ENDIF}
    ///<summary>
    ///  Iterate over each element, calling the provided callback. The callback
    ///  is called for each element like: <c>Callback(MVCArray,Element,Index)</c>.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure ForEach(Callback: TCallbackMethodA); overload;
    {$ENDIF}
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InsertAt'}{$ENDIF}
    ///<summary>
    ///  Inserts an element at the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure InsertAt(i: Integer; const Elem: OleVariant); overload; virtual;
    procedure InsertAt(i: Integer; const Elem: TDispObject); overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetAt'}{$ENDIF}
    ///<summary>
    ///  Sets an element at the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetAt(i: Integer; const Elem: OleVariant); virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Count'}{$ENDIF}
    ///<summary>
    ///  Returns the number of elements in this array.
    ///</summary>
    ///<remarks>
    ///  Additional Delphi method. Not available in Google Maps API.
    ///</remarks>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Count: Integer;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetArray'}{$ENDIF}
    ///<summary>
    ///  Returns a reference to the underlying Array. Warning: If the Array is
    ///  mutated, no events will be fired by this object.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetArray: OleVariant;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetAt'}{$ENDIF}
    ///<summary>
    ///  Get an element at the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetAt(i: Integer): OleVariant; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetLength'}{$ENDIF}
    ///<summary>
    ///  Returns the number of elements in this array.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetLength: Integer;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Pop'}{$ENDIF}
    ///<summary>
    ///  Removes the last element of the array and returns that element.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Pop: OleVariant; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Push'}{$ENDIF}
    ///<summary>
    ///  Adds one element to the end of the array and returns the new length of
    ///  the array.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Push(const Elem: OleVariant): Integer; overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Push'}{$ENDIF}
    ///<summary>
    ///  Adds one element to the end of the array and returns the new length of
    ///  the array.
    ///</summary>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Push(Elem: TDispObject): Integer; overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RemoveAt'}{$ENDIF}
    ///<summary>
    ///  Removes an element from the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function RemoveAt(i: Integer): OleVariant; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Get/set an element at the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: OleVariant read GetAt write SetAt;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnInsertAt'}{$ENDIF}
    ///<summary>
    ///  This event is fired when <i>InsertAt</i> is called. The event passes
    ///  the index that was passed to <i>InsertAt</i>.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnInsertAt: TInsertAtEvent read FOnInsertAt write SetOnInsertAt;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnRemoveAt'}{$ENDIF}
    ///<summary>
    ///  This event is fired when <i>RemoveAt</i> is called. The event passes
    ///  the index that was passed to <i>RemoveAt</i> and the element that was
    ///  removed from the array.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnRemoveAt: TRemoveAtEvent read FOnRemoveAt write SetOnRemoveAt;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnSetAt'}{$ENDIF}
    ///<summary>
    ///  This event is fired when <i>SetAt</i> is called. The event passes
    ///  the index that was passed to <i>SetAt</i> and the element that was
    ///  previously in the array at that index.
    ///</summary>
    ///<seealso cref="gmMVC|TMVCArray">TMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnSetAt: TSetAtEvent read FOnSetAt write SetOnSetAt;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMVCEventObject'}{$ENDIF}
  ///<summary>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmMVC"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///  <para/>
  ///  <para><b>Events</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TMVCEventObject.OnClick">OnClick</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnDblClick">OnDblClick</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseDown">OnMouseDown</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseMove">OnMouseMove</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseOut">OnMouseOut</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseOver">OnMouseOver</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseUp">OnMouseUp</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnRightClick">OnRightClick</see></item>
  ///  </list>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMVCEventObject = class(TMVCObject)
  private
    FOnClick: TMapMouseEvent;
    FOnClickListener: TMapsEventListener;
    FOnClickFunction: TJScriptFunction;
    FOnDblClick: TMapMouseEvent;
    FOnDblClickListener: TMapsEventListener;
    FOnDblClickFunction: TJScriptFunction;
    FOnMouseDown: TMapMouseEvent;
    FOnMouseDownListener: TMapsEventListener;
    FOnMouseDownFunction: TJScriptFunction;
    FOnMouseMove: TMapMouseEvent;
    FOnMouseMoveListener: TMapsEventListener;
    FOnMouseMoveFunction: TJScriptFunction;
    FOnMouseOut: TMapMouseEvent;
    FOnMouseOutListener: TMapsEventListener;
    FOnMouseOutFunction: TJScriptFunction;
    FOnMouseOver: TMapMouseEvent;
    FOnMouseOverListener: TMapsEventListener;
    FOnMouseOverFunction: TJScriptFunction;
    FOnMouseUp: TMapMouseEvent;
    FOnMouseUpListener: TMapsEventListener;
    FOnMouseUpFunction: TJScriptFunction;
    FOnRightClick: TMapMouseEvent;
    FOnRightClickListener: TMapsEventListener;
    FOnRightClickFunction: TJScriptFunction;
    procedure SetOnClick(Value: TMapMouseEvent);
    procedure SetOnDblClick(Value: TMapMouseEvent);
    procedure SetOnMouseDown(Value: TMapMouseEvent);
    procedure SetOnMouseMove(Value: TMapMouseEvent);
    procedure SetOnMouseOut(Value: TMapMouseEvent);
    procedure SetOnMouseOver(Value: TMapMouseEvent);
    procedure SetOnMouseUp(Value: TMapMouseEvent);
    procedure SetOnRightClick(Value: TMapMouseEvent);
  protected
    constructor Create(Script: TCustomScript; Dispatch: IDispatch); reintroduce; virtual;
    //von JScript aufgerufene Event-Handler:
    //JavaScript event handler:
    procedure Click;
    procedure DblClick;
    procedure MouseDown;
    procedure MouseMove;
    procedure MouseOut;
    procedure MouseOver;
    procedure MouseUp;
    procedure RightClick;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnClick'}{$ENDIF}
    ///<summary>
    ///  Dieses Ereignis tritt ein, wenn das DOM-Click-Ereignis fr das Objekt
    ///  ausgelst wird.<br/><br/>
    ///  This event is fired when the DOM click event is fired on the Object.
    ///</summary>
    ///<seealso cref="gmEvents|TMapMouseEvent">TMapMouseEvent</seealso>
    ///<seealso cref="gmMVC|TMVCEventObject">TMVCEventObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnClick: TMapMouseEvent read FOnClick write SetOnClick;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnDblClick'}{$ENDIF}
    ///<summary>
    ///  Dieses Ereignis tritt ein, wenn das DOM-DblClick-Ereignis fr das Objekt
    ///  ausgelst wird.<br/><br/>
    ///  This event is fired when the DOM dblclick event is fired on the Object.
    ///</summary>
    ///<seealso cref="gmEvents|TMapMouseEvent">TMapMouseEvent</seealso>
    ///<seealso cref="gmMVC|TMVCEventObject">TMVCEventObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnDblClick: TMapMouseEvent read FOnDblClick write SetOnDblClick;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnMouseDown'}{$ENDIF}
    ///<summary>
    ///  Dieses Ereignis tritt ein, wenn das DOM-MouseDown-Ereignis fr das Objekt
    ///  ausgelst wird.<br/><br/>
    ///  This event is fired when the DOM mousedown event is fired on the Object.
    ///</summary>
    ///<seealso cref="gmEvents|TMapMouseEvent">TMapMouseEvent</seealso>
    ///<seealso cref="gmMVC|TMVCEventObject">TMVCEventObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnMouseDown: TMapMouseEvent read FOnMouseDown write SetOnMouseDown;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnMouseMove'}{$ENDIF}
    ///<summary>
    ///  Dieses Ereignis tritt ein, wenn das DOM-MouseMove-Ereignis fr das Objekt
    ///  ausgelst wird.<br/><br/>
    ///  This event is fired when the DOM mousemove event is fired on the Object.
    ///</summary>
    ///<seealso cref="gmEvents|TMapMouseEvent">TMapMouseEvent</seealso>
    ///<seealso cref="gmMVC|TMVCEventObject">TMVCEventObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnMouseMove: TMapMouseEvent read FOnMouseMove write SetOnMouseMove;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnMouseOut'}{$ENDIF}
    ///<summary>
    ///  Dieses Ereignis tritt ein, wenn das DOM-MouseOut-Ereignis fr das Objekt
    ///  ausgelst wird.<br/><br/>
    ///  This event is fired on Object mouseout.
    ///</summary>
    ///<seealso cref="gmEvents|TMapMouseEvent">TMapMouseEvent</seealso>
    ///<seealso cref="gmMVC|TMVCEventObject">TMVCEventObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnMouseOut: TMapMouseEvent read FOnMouseOut write SetOnMouseOut;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnMouseOver'}{$ENDIF}
    ///<summary>
    ///  Dieses Ereignis tritt ein, wenn das DOM-MouseOver-Ereignis fr das Objekt
    ///  ausgelst wird.<br/><br/>
    ///  This event is fired when the DOM mouseover event is fired on the Object.
    ///</summary>
    ///<seealso cref="gmEvents|TMapMouseEvent">TMapMouseEvent</seealso>
    ///<seealso cref="gmMVC|TMVCEventObject">TMVCEventObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnMouseOver: TMapMouseEvent read FOnMouseOver write SetOnMouseOver;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnMouseUp'}{$ENDIF}
    ///<summary>
    ///  Dieses Ereignis tritt ein, wenn das DOM-MouseUp-Ereignis fr das Objekt
    ///  ausgelst wird.<br/><br/>
    ///  This event is fired when the DOM mouseup event is fired on the Object.
    ///</summary>
    ///<seealso cref="gmEvents|TMapMouseEvent">TMapMouseEvent</seealso>
    ///<seealso cref="gmMVC|TMVCEventObject">TMVCEventObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnMouseUp: TMapMouseEvent read FOnMouseUp write SetOnMouseUp;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnRightClick'}{$ENDIF}
    ///<summary>
    ///  Dieses Ereignis wird ausgelst, wenn mit der rechten Maustaste auf das
    ///  Objekt geklickt wird.<br/><br/>
    ///  This event is fired when the Object is right-clicked on.
    ///</summary>
    ///<seealso cref="gmEvents|TMapMouseEvent">TMapMouseEvent</seealso>
    ///<seealso cref="gmMVC|TMVCEventObject">TMVCEventObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnRightClick: TMapMouseEvent read FOnRightClick write SetOnRightClick;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TObjectMVCArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: MVCArray.&#60;DispObject&#62;
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmMVC"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCArray"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TObjectMVCArray.Add">Add</see></item>
  ///    <item><see cref="gmMVC|TObjectMVCArray.InsertAt(Integer,TDispObject)">InsertAt</see></item>
  ///    <item><see cref="gmMVC|TObjectMVCArray.SetAt(Integer,TDispObject)">SetAt</see></item>
  ///    <item><see cref="gmMVC|TObjectMVCArray.GetAt(Integer)">GetAt</see></item>
  ///    <item><see cref="gmMVC|TObjectMVCArray.Pop">Pop</see></item>
  ///    <item><see cref="gmMVC|TObjectMVCArray.Push(TDispObject)">Push</see></item>
  ///    <item><see cref="gmMVC|TObjectMVCArray.RemoveAt(Integer)">RemoveAt</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TObjectMVCArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TObjectMVCArray = class(TMVCArray)
  private
    FItems: TStringList;
    function GetObject(Index: Integer; NewDispatch: IDispatch; Extract: Boolean = false): TDispObject;
  protected
    procedure AddObject(Elem: TDispObject);
    procedure SetDisp(Value: IDispatch); override;
    function CreateObject(Dispatch: IDispatch): TDispObject; virtual; abstract;
  public
    destructor Destroy; override;
    procedure AfterConstruction; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InsertAt'}{$ENDIF}
    ///<summary>
    ///  Inserts an <i>TDispObject</i> at the specified index.
    ///</summary>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure InsertAt(i: Integer; Elem: TDispObject); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetAt'}{$ENDIF}
    ///<summary>
    ///  Sets an <i>TDispObject</i> at the specified index.
    ///</summary>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetAt(i: Integer; Elem: TDispObject); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<summary>
    ///  Adds one <c>TDispObject</c> to the end of the array.
    ///</summary>
    ///<remarks>
    ///  Additional Delphi method. Not available in Google Maps API.
    ///</remarks>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    //function Add: TDispObject; virtual; abstract;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetAt'}{$ENDIF}
    ///<summary>
    ///  Get an <i>TDispObject</i> at the specified index.
    ///</summary>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetAt(i: Integer): TDispObject; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Pop'}{$ENDIF}
    ///<summary>
    ///  Removes the last <i>TDispObject</i> of the array and returns that element.
    ///</summary>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Pop: TDispObject; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Push'}{$ENDIF}
    ///<summary>
    ///  Adds one <i>TDispObject</i> to the end of the array and returns the new length of
    ///  the array.
    ///</summary>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Push(Elem: TDispObject): Integer; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RemoveAt'}{$ENDIF}
    ///<summary>
    ///  Removes an <i>TDispObject</i> from the specified index.
    ///</summary>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function RemoveAt(i: Integer): TDispObject; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Get/set an <i>TDispObject</i> at the specified index.
    ///</summary>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TDispObject read GetAt write SetAt; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TLatLngMVCArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: MVCArray.&#60;LatLng&#62;
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmMVC"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TObjectMVCArray"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TLatLngMVCArray.Add(Double,Double,Boolean)">Add</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArray.InsertAt(Integer,TLatLng)">InsertAt</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArray.SetAt(Integer,TLatLng)">SetAt</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArray.GetAt(Integer)">GetAt</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArray.Pop">Pop</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArray.Push(TLatLng)">Push</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArray.RemoveAt(Integer)">RemoveAt</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TObjectMVCArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="note">
  ///    Im Google Maps API nicht vorhanden.<br/><br/>
  ///    Not available in the Google Maps API.
  ///  </note>
  ///</remarks>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TLatLngMVCArray = class(TObjectMVCArray)
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InsertAt'}{$ENDIF}
    ///<summary>
    ///  Inserts an <i>TLatLng</i> at the specified index.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure InsertAt(i: Integer; Elem: TLatLng); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetAt'}{$ENDIF}
    ///<summary>
    ///  Sets an <i>TLatLng</i> at the specified index.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetAt(i: Integer; Elem: TLatLng); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<summary>
    ///  Adds one <c>TLatLng</c> to the end of the array.
    ///</summary>
    ///<remarks>
    ///  Additional Delphi method. Not available in Google Maps API.
    ///</remarks>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(Lat, Lng: Double; NoWrap: Boolean = true): TLatLng; reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetAt'}{$ENDIF}
    ///<summary>
    ///  Get an <i>TLatLng</i> at the specified index.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetAt(i: Integer): TLatLng; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Pop'}{$ENDIF}
    ///<summary>
    ///  Removes the last <i>TLatLng</i> of the array and returns that element.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Pop: TLatLng; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Push'}{$ENDIF}
    ///<summary>
    ///  Adds one <i>TLatLng</i> to the end of the array and returns the new length of
    ///  the array.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Push(Elem: TLatLng): Integer; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RemoveAt'}{$ENDIF}
    ///<summary>
    ///  Removes an <i>TLatLng</i> from the specified index.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function RemoveAt(i: Integer): TLatLng; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Get/set an <i>TLatLng</i> at the specified index.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TLatLng read GetAt write SetAt; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TLatLngMVCArrayMVCArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: MVCArray.&#60;MVCArray.&#60;LatLng&#62;&#62;
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmMVC"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TObjectMVCArray"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TLatLngMVCArrayMVCArray.InsertAt(Integer,TLatLngMVCArray)">InsertAt</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArrayMVCArray.SetAt(Integer,TLatLngMVCArray)">SetAt</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArrayMVCArray.GetAt(Integer)">GetAt</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArrayMVCArray.Pop">Pop</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArrayMVCArray.Push(TLatLngMVCArray)">Push</see></item>
  ///    <item><see cref="gmMVC|TLatLngMVCArrayMVCArray.RemoveAt(Integer)">RemoveAt</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TLatLngMVCArrayMVCArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="note">
  ///    Im Google Maps API nicht vorhanden.<br/><br/>
  ///    Not available in the Google Maps API.
  ///  </note>
  ///</remarks>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TLatLngMVCArrayMVCArray = class(TObjectMVCArray)
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InsertAt'}{$ENDIF}
    ///<summary>
    ///  Inserts an <i>TLatLngMVCArray</i> at the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure InsertAt(i: Integer; Elem: TLatLngMVCArray); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetAt'}{$ENDIF}
    ///<summary>
    ///  Sets an <i>TLatLngMVCArray</i> at the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetAt(i: Integer; Elem: TLatLngMVCArray); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<summary>
    ///  Adds one <c>TLatLngMVCArray</c> to the end of the array.
    ///</summary>
    ///<remarks>
    ///  Additional Delphi method. Not available in Google Maps API.
    ///</remarks>
    ///<seealso cref="DispObject|TDispObject">TDispObject</seealso>
    ///<seealso cref="gmMVC|TObjectMVCArray">TObjectMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add: TLatLngMVCArray; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetAt'}{$ENDIF}
    ///<summary>
    ///  Get an <i>TLatLngMVCArray</i> at the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetAt(i: Integer): TLatLngMVCArray; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Pop'}{$ENDIF}
    ///<summary>
    ///  Removes the last <i>TLatLngMVCArray</i> of the array and returns that element.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Pop: TLatLngMVCArray; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Push'}{$ENDIF}
    ///<summary>
    ///  Adds one <i>TLatLngMVCArray</i> to the end of the array and returns the
    ///  new length of the array.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Push(Elem: TLatLngMVCArray): Integer; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RemoveAt'}{$ENDIF}
    ///<summary>
    ///  Removes an <i>TLatLngMVCArray</i> from the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function RemoveAt(i: Integer): TLatLngMVCArray; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Get/set an <i>TLatLngMVCArray</i> at the specified index.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TLatLngMVCArray read GetAt write SetAt; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TNodeMVCArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: MVCArray.&#60;Node&#62;
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmMVC"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TObjectMVCArray"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TNodeMVCArray.Add(String)">Add</see></item>
  ///    <item><see cref="gmMVC|TNodeMVCArray.InsertAt(Integer,TNode)">InsertAt</see></item>
  ///    <item><see cref="gmMVC|TNodeMVCArray.SetAt(Integer,TNode)">SetAt</see></item>
  ///    <item><see cref="gmMVC|TNodeMVCArray.GetAt(Integer)">GetAt</see></item>
  ///    <item><see cref="gmMVC|TNodeMVCArray.Pop">Pop</see></item>
  ///    <item><see cref="gmMVC|TNodeMVCArray.Push(TNode)">Push</see></item>
  ///    <item><see cref="gmMVC|TNodeMVCArray.RemoveAt(Integer)">RemoveAt</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TNodeMVCArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="note">
  ///    Im Google Maps API nicht vorhanden.<br/><br/>
  ///    Not available in the Google Maps API.
  ///  </note>
  ///</remarks>
  ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TNodeMVCArray = class(TObjectMVCArray)
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InsertAt'}{$ENDIF}
    ///<summary>
    ///  Inserts an <i>TNode</i> at the specified index.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure InsertAt(i: Integer; Elem: TNode); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetAt'}{$ENDIF}
    ///<summary>
    ///  Sets an <i>TNode</i> at the specified index.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetAt(i: Integer; Elem: TNode); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<summary>
    ///  Adds one <c>TNode</c> to the end of the array.
    ///</summary>
    ///<remarks>
    ///  Additional Delphi method. Not available in Google Maps API.
    ///</remarks>
    ///<seealso cref="HTMLObjects|TNode"></seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(ElementType: String): TNode; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetAt'}{$ENDIF}
    ///<summary>
    ///  Get an <i>TNode</i> at the specified index.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetAt(i: Integer): TNode; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Pop'}{$ENDIF}
    ///<summary>
    ///  Removes the last <i>TNode</i> of the array and returns that element.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Pop: TNode; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Push'}{$ENDIF}
    ///<summary>
    ///  Adds one <i>TNode</i> to the end of the array and returns the new length of
    ///  the array.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Push(Elem: TNode): Integer; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RemoveAt'}{$ENDIF}
    ///<summary>
    ///  Removes an <i>TNode</i> from the specified index.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function RemoveAt(i: Integer): TNode; reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Get/set an <i>TNode</i> at the specified index.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TNode read GetAt write SetAt;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TNodeMVCArrayArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&#60;MVCArray.&#60;Node&#62;&#62;
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmMVC"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TObjectMVCArray"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TNodeMVCArrayArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <note type="note">
  ///    Im Google Maps API nicht vorhanden.<br/><br/>
  ///    Not available in the Google Maps API.
  ///  </note>
  ///</remarks>
  ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TNodeMVCArrayArray = class(TObjectArray)
  private
    procedure SetItem(Index: Integer; Value: TNodeMVCArray);
    function GetItem(Index: Integer): TNodeMVCArray;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Get/set an <i>TNodeMVCArray</i> at the specified index.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArray">TNodeMVCArray</seealso>
    ///<seealso cref="gmMVC|TNodeMVCArrayArray">TNodeMVCArrayArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TNodeMVCArray read GetItem write SetItem; default;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TMVCObject

constructor TMVCObject.Create;
begin
  inherited;
end;

constructor TMVCObject.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TMVCObject.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_MVCObject,true);
end;

class function TMVCObject.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MVCObjectMethod_BindTo,
                                 MVCObjectMethod_Changed,
                                 MVCObjectMethod_Get]);
end;

procedure TMVCObject.AddListener(const EventName: String; var Listener: TMapsEventListener;
                                 var ListenerFunction: TJScriptFunction; Receiver: TExternalProc);
begin
  gmEvents.AddListener(Script,Self,EventName,Listener,ListenerFunction,Receiver);
end;

procedure TMVCObject.ClearInstanceListeners;
begin
  if Script is TScript
    then TScript(Script).Google.Maps.Event.ClearInstanceListeners(Self);
end;

procedure TMVCObject.RemoveListener(var Listener: TMapsEventListener);
begin
  gmEvents.RemoveListener(Script,Listener);
end;

procedure TMVCObject.CallEvent(Event: TNotifyEvent);
begin
  try
    if assigned(Event)
      then Event(Self);
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

procedure TMVCObject.CallEvent(Event: TMapMouseEvent; EventData: IDispatch);
var
  EventObject: TMouseEvent;
begin
  try
    if assigned(Event) then
    begin
      EventObject:=TMouseEvent(CreateObjectWrapper(Script,TMouseEvent,EventData));
      try
        Event(Self,EventObject);
      finally
        //EventObject.Free;       -> to save it for future use in anonymous methods
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

procedure TMVCObject.CallEvent(Event: TMapEventEvent; EventData: IDispatch);
var
  EventObject: HTMLObjects.TEvent;
begin
  try
    if assigned(Event) then
    begin
      EventObject:=HTMLObjects.TEvent(CreateObjectWrapper(Script,
                                                          HTMLObjects.TEvent,
                                                          EventData));
      try
        Event(Self,EventObject);
      finally
        //EventObject.Free;       -> to save it for future use in anonymous methods
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

procedure TMVCObject.BindTo(const Key: String; Target: TMVCObject);
begin
  Exec(MVCObjectMethod_BindTo,[Key,Target.Disp]);
end;

procedure TMVCObject.BindTo(const Key: String; Target: TMVCObject; const TargetKey: String);
begin
  Exec(MVCObjectMethod_BindTo,[Key,Target.Disp,TargetKey]);
end;

procedure TMVCObject.BindTo(const Key: String; Target: TMVCObject; const TargetKey: String; NoNotify: Boolean);
begin
  Exec(MVCObjectMethod_BindTo,[Key,Target.Disp,TargetKey,NoNotify]);
end;

procedure TMVCObject.Changed(const Key: String);
begin
  Exec(MVCObjectMethod_Changed,[Key]);
end;

function TMVCObject.Get(const Key: String): OleVariant;
begin
  Result:=Exec(MVCObjectMethod_Get,[Key]);
end;

procedure TMVCObject.Notify(const Key: String);
begin
  Exec(MVCObjectMethod_Notify,[Key]);
end;

procedure TMVCObject.SetOptions(Options: TCustomJScriptObject);
begin
  Exec(MVCObjectMethod_SetOptions,[Options.Disp]);
end;

procedure TMVCObject.{$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}(const Key: String; const Value: OleVariant);
begin
  Exec(MVCObjectMethod_Set,[Key,Value]);
end;

procedure TMVCObject.SetValues(Values: TCustomJScriptObject);
begin
  if not assigned(Values)
    then Exec(MVCObjectMethod_SetValues)
    else Exec(MVCObjectMethod_SetValues,[Values.Disp]);
end;

procedure TMVCObject.Unbind(const Key: String);
begin
  Exec(MVCObjectMethod_Unbind,[Key]);
end;

procedure TMVCObject.UnbindAll;
begin
  Exec(MVCObjectMethod_UnbindAll);
end;

//------------------------------------------------------------------------------
//TMVCEventObject

destructor TMVCEventObject.Destroy;
begin
  SetOnClick(nil);
  SetOnDblClick(nil);
  SetOnMouseDown(nil);
  SetOnMouseMove(nil);
  SetOnMouseOut(nil);
  SetOnMouseOver(nil);
  SetOnMouseUp(nil);
  SetOnRightClick(nil);
  FOnClickFunction.Free;
  FOnDblClickFunction.Free;
  FOnMouseDownFunction.Free;
  FOnMouseMoveFunction.Free;
  FOnMouseOutFunction.Free;
  FOnMouseOverFunction.Free;
  FOnMouseUpFunction.Free;
  FOnRightClickFunction.Free;
  inherited;
end;

constructor TMVCEventObject.Create(Script: TCustomScript; Dispatch: IDispatch);
begin
  inherited;
end;

procedure TMVCEventObject.Click;
begin
  CallEvent(FOnClick,FOnClickFunction.Arguments[0]);
end;

procedure TMVCEventObject.DblClick;
begin
  CallEvent(FOnDblClick,FOnDblClickFunction.Arguments[0]);
end;

procedure TMVCEventObject.MouseDown;
begin
  CallEvent(FOnMouseDown,FOnMouseDownFunction.Arguments[0]);
end;

procedure TMVCEventObject.MouseMove;
begin
  CallEvent(FOnMouseMove,FOnMouseMoveFunction.Arguments[0]);
end;

procedure TMVCEventObject.MouseOut;
begin
  CallEvent(FOnMouseOut,FOnMouseOutFunction.Arguments[0]);
end;

procedure TMVCEventObject.MouseOver;
begin
  CallEvent(FOnMouseOver,FOnMouseOverFunction.Arguments[0]);
end;

procedure TMVCEventObject.MouseUp;
begin
  CallEvent(FOnMouseUp,FOnMouseUpFunction.Arguments[0]);
end;

procedure TMVCEventObject.RightClick;
begin
  CallEvent(FOnRightClick,FOnRightClickFunction.Arguments[0]);
end;

procedure TMVCEventObject.SetOnClick(Value: TMapMouseEvent);
begin
  FOnClick:=Value;
  RemoveListener(FOnClickListener);
  if assigned(FOnClick)
    then AddListener(Event_Click,FOnClickListener,FOnClickFunction,Click);
end;

procedure TMVCEventObject.SetOnDblClick(Value: TMapMouseEvent);
begin
  FOnDblClick:=Value;
  RemoveListener(FOnDblClickListener);
  if assigned(FOnDblClick)
    then AddListener(Event_DblClick,FOnDblClickListener,FOnDblClickFunction,DblClick);
end;

procedure TMVCEventObject.SetOnMouseDown(Value: TMapMouseEvent);
begin
  FOnMouseDown:=Value;
  RemoveListener(FOnMouseDownListener);
  if assigned(FOnMouseDown)
    then AddListener(Event_MouseDown,FOnMouseDownListener,FOnMouseDownFunction,MouseDown);
end;

procedure TMVCEventObject.SetOnMouseMove(Value: TMapMouseEvent);
begin
  FOnMouseMove:=Value;
  RemoveListener(FOnMouseMoveListener);
  if assigned(FOnMouseMove)
    then AddListener(Event_MouseMove,FOnMouseMoveListener,FOnMouseMoveFunction,MouseMove);
end;

procedure TMVCEventObject.SetOnMouseOut(Value: TMapMouseEvent);
begin
  FOnMouseOut:=Value;
  RemoveListener(FOnMouseOutListener);
  if assigned(FOnMouseOut)
    then AddListener(Event_MouseOut,FOnMouseOutListener,FOnMouseOutFunction,MouseOut);
end;

procedure TMVCEventObject.SetOnMouseOver(Value: TMapMouseEvent);
begin
  FOnMouseOver:=Value;
  RemoveListener(FOnMouseOverListener);
  if assigned(FOnMouseOver)
    then AddListener(Event_MouseOver,FOnMouseOverListener,FOnMouseOverFunction,MouseOver);
end;

procedure TMVCEventObject.SetOnMouseUp(Value: TMapMouseEvent);
begin
  FOnMouseUp:=Value;
  RemoveListener(FOnMouseUpListener);
  if assigned(FOnMouseUp)
    then AddListener(Event_MouseUp,FOnMouseUpListener,FOnMouseUpFunction,MouseUp);
end;

procedure TMVCEventObject.SetOnRightClick(Value: TMapMouseEvent);
begin
  FOnRightClick:=Value;
  RemoveListener(FOnRightClickListener);
  if assigned(FOnRightClick)
    then AddListener(Event_RightClick,FOnRightClickListener,FOnRightClickFunction,RightClick);
end;

//------------------------------------------------------------------------------
//TMVCArray

constructor TMVCArray.Create;
begin
  inherited;
end;

constructor TMVCArray.Create(Script: TCustomScript);
begin
  inherited;
end;

constructor TMVCArray.Create(const Elements: array of OleVariant);
begin
  Create(JScriptObjects.Script,Elements);
end;

constructor TMVCArray.Create(Script: TCustomScript; const Elements: array of OleVariant);
begin
  inherited Create(Script,New(Script,Elements));
end;

destructor TMVCArray.Destroy;
begin
  SetOnInsertAt(nil);
  SetOnRemoveAt(nil);
  SetOnSetAt(nil);
  FOnInsertAtFunction.Free;
  FOnRemoveAtFunction.Free;
  FOnSetAtFunction.Free;
  FCallbackFunction.Free;
  inherited;
end;

class function TMVCArray.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MVCArrayMethod_ForEach,
                                 MVCArrayMethod_GetArray,
                                 MVCArrayMethod_GetAt]);
end;

class function TMVCArray.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_MVCArray,true);
end;

class function TMVCArray.New(Script: TCustomScript; const Elements: array of OleVariant): IDispatch;
begin
  CheckMaps(Script);
  if Length(Elements)=0
    then Result:=New(Script)
    else Result:=BrowserTools.{$IFDEF CAN_OVERLOAD}Create{$ELSE}Create_{$ENDIF}(
                              TScript(Script).Google.Maps.Disp,
                              MapsMethod_MVCArray,Elements,true);
end;

function TMVCArray.Count: Integer;
begin
  Result:=GetLength;
end;

procedure TMVCArray.Clear;
begin
  Exec(MVCArrayMethod_Clear);
end;

procedure TMVCArray.ForEach(Callback: TCallbackMethod);
begin
  if assigned(Callback) then
  begin
    if not assigned(FCallbackFunction)
      then FCallbackFunction:=CreateJScriptFunction(Script,ForEachHandler);
    FCallback:=Callback;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FCallbackA:=nil;
    {$ENDIF}
    Exec(MVCArrayMethod_ForEach,[FCallbackFunction.Disp]);
  end;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TMVCArray.ForEach(Callback: TCallbackMethodA);
begin
  if assigned(Callback) then
  begin
    if not assigned(FCallbackFunction)
      then FCallbackFunction:=CreateJScriptFunction(Script,ForEachHandler);
    FCallback:=nil;
    FCallbackA:=Callback;
    Exec(MVCArrayMethod_ForEach,[FCallbackFunction.Disp]);
  end;
end;
{$ENDIF}

procedure TMVCArray.ForEachHandler;
var
  Element: TDispObject;
begin
  if (FCallbackFunction.Arguments.Count>=2) and
     (assigned(FCallback) {$IFDEF USE_ANONYMOUS_METHODS}or assigned(FCallbackA){$ENDIF}) then
  begin
    try
      Element:=CreateObjectWrapper(Script,TDispObject,FCallbackFunction.Arguments[0]);
      try
        {$IFDEF USE_ANONYMOUS_METHODS}
        if assigned(FCallbackA)
          then FCallbackA(Self,Element,FCallbackFunction.Arguments[1])
          else
        {$ENDIF}
        FCallback(Self,Element,FCallbackFunction.Arguments[1]);
      finally
        Element.Free;
      end;
    except
      {$IFDEF DEBUG}
      ShowMessage(Exception(ExceptObject).Message);
      {$ENDIF}
    end;
  end;
end;

function TMVCArray.GetArray: OleVariant;
begin
  Result:=Exec(MVCArrayMethod_GetArray);
end;

function TMVCArray.GetAt(i: Integer): OleVariant;
begin
  Result:=Exec(MVCArrayMethod_GetAt,[i]);
end;

function TMVCArray.GetLength: Integer;
begin
  Result:=Exec(MVCArrayMethod_GetLength);
end;

procedure TMVCArray.InsertAt(i: Integer; const Elem: OleVariant);
begin
  Exec(MVCArrayMethod_InsertAt,[i,Elem]);
end;

procedure TMVCArray.InsertAt(i: Integer; const Elem: TDispObject);
begin
  if assigned(Elem)
    then InsertAt(i,Elem.Disp)
    else InsertAt(i,Null);
end;

procedure TMVCArray.InsertAtHandler;
begin
  if assigned(FOnInsertAt) then
  begin
    try
      FOnInsertAt(Self,FOnInsertAtFunction.Arguments[0]);
    except
      {$IFDEF DEBUG}
      ShowMessage(Exception(ExceptObject).Message);
      {$ENDIF}
    end;
  end;
end;

function TMVCArray.Pop: OleVariant;
begin
  Result:=Exec(MVCArrayMethod_Pop);
end;

function TMVCArray.Push(const Elem: OleVariant): Integer;
begin
  Result:=Exec(MVCArrayMethod_Push,Elem);
end;

function TMVCArray.Push(Elem: TDispObject): Integer;
begin
  Result:=GetLength;
  if assigned(Elem)
    then Result:=Push(Elem.Disp);
end;

function TMVCArray.RemoveAt(i: Integer): OleVariant;
begin
  Result:=Exec(MVCArrayMethod_RemoveAt,[i]);
end;

procedure TMVCArray.RemoveAtHandler;
var
  Element: TDispObject;
begin
  try
    Element:=CreateObjectWrapper(Script,TDispObject,FOnRemoveAtFunction.Arguments[1]);
    try
      FOnRemoveAt(Self,FOnRemoveAtFunction.Arguments[0],Element);
    finally
      Element.Free;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

procedure TMVCArray.SetAt(i: Integer; const Elem: OleVariant);
begin
  Exec(MVCArrayMethod_SetAt,[i,Elem]);
end;

procedure TMVCArray.SetAtHandler;
var
  Element: TDispObject;
begin
  try
    Element:=CreateObjectWrapper(Script,TDispObject,FOnSetAtFunction.Arguments[1]);
    try
      FOnSetAt(Self,FOnSetAtFunction.Arguments[0],Element);
    finally
      Element.Free;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

procedure TMVCArray.SetOnInsertAt(Value: TInsertAtEvent);
begin
  FOnInsertAt:=Value;
  RemoveListener(FOnInsertAtListener);
  if assigned(FOnInsertAt)
    then AddListener(MVCArrayEvent_InsertAt,FOnInsertAtListener,
                     FOnInsertAtFunction,InsertAtHandler);
end;

procedure TMVCArray.SetOnRemoveAt(Value: TRemoveAtEvent);
begin
  FOnRemoveAt:=Value;
  RemoveListener(FOnRemoveAtListener);
  if assigned(FOnRemoveAt)
    then AddListener(MVCArrayEvent_RemoveAt,FOnRemoveAtListener,
                     FOnRemoveAtFunction,RemoveAtHandler);
end;

procedure TMVCArray.SetOnSetAt(Value: TSetAtEvent);
begin
  FOnSetAt:=Value;
  RemoveListener(FOnSetAtListener);
  if assigned(FOnSetAt)
    then AddListener(MVCArrayEvent_SetAt,FOnSetAtListener,
                     FOnSetAtFunction,SetAtHandler);
end;

//------------------------------------------------------------------------------
//TObjectMVCArray

procedure TObjectMVCArray.AfterConstruction;
begin
  inherited;
  FItems:=TStringList.Create{$IFDEF DELPHI2009_UP}(true){$ENDIF};
end;

destructor TObjectMVCArray.Destroy;
begin
  {$IFNDEF DELPHI2009_UP}
  if assigned(FItems) then
    while FItems.Count>0 do
    begin
      FItems.Objects[0].Free;
      FItems.Delete(0);
    end;
  {$ENDIF}
  FreeAndNil(FItems);
  inherited;
end;

function TObjectMVCArray.GetObject(Index: Integer; NewDispatch: IDispatch; Extract: Boolean): TDispObject;
var
  IndexStr: String;
begin
  Result:=nil;
  IndexStr:=IntToStr(Index);
  if FItems.IndexOf(IndexStr)>=0 then
  begin
    Result:=TDispObject(FItems.Objects[FItems.IndexOf(IndexStr)]);
    if assigned(Result)
      then Result.Disp:=NewDispatch;
    if Extract then
    begin
      {$IFDEF DELPHI2009_UP}
      FItems.OwnsObjects:=false;
      {$ENDIF}
      FItems.Delete(FItems.IndexOf(IndexStr));
      {$IFDEF DELPHI2009_UP}
      FItems.OwnsObjects:=true;
      {$ENDIF}
    end;
  end;
end;

procedure TObjectMVCArray.SetDisp(Value: IDispatch);
var
  OldDisp: IDispatch;
begin
  OldDisp:=Disp;
  inherited;
  if (OldDisp<>Value) and assigned(FItems)                 //JScript-Objekt wurde neu zugewiesen
    then FItems.Clear;                                     //-> schon erstellte Delphi-Wrapper-Objekte lschen
end;

procedure TObjectMVCArray.AddObject(Elem: TDispObject);
begin
  if assigned(Elem) then
  begin
    Push(Elem);                                            //Objekt dem Array hinzufgen
    FItems.AddObject(IntToStr(Count-1),Elem);              //Objekt bernehmen
  end;
end;

function TObjectMVCArray.GetAt(i: Integer): TDispObject;
var
  Dispatch: OleVariant;
  IndexStr: String;
begin
  Result:=nil;
  Dispatch:=inherited GetAt(i);
  if VarIsValidDispatch(Dispatch) then
  begin
    IndexStr:=IntToStr(i);
    if FItems.IndexOf(IndexStr)<0 then                     //noch kein zugehriges Delphi-Objekt angelegt
    begin
      Result:=CreateObject(Dispatch);
      FItems.AddObject(IndexStr,Result);
    end else Result:=GetObject(i,Dispatch);
  end;
end;

procedure TObjectMVCArray.InsertAt(i: Integer; Elem: TDispObject);
begin
  if assigned(Elem)
    then inherited InsertAt(i,Elem.Disp)
    else inherited InsertAt(i,Null);
end;

function TObjectMVCArray.Pop: TDispObject;
var
  Dispatch: OleVariant;
  IndexStr: String;
begin
  Result:=nil;
  Dispatch:=inherited Pop;
  if VarIsValidDispatch(Dispatch) then
  begin
    IndexStr:=IntToStr(GetLength);
    if FItems.IndexOf(IndexStr)<0                          //noch kein zugehriges Delphi-Objekt angelegt
      then Result:=CreateObject(Dispatch)
      else Result:=GetObject(GetLength,Dispatch,true);
  end;
end;

function TObjectMVCArray.Push(Elem: TDispObject): Integer;
begin
  if assigned(Elem)
    then Result:=inherited Push(Elem.Disp)
    else Result:=inherited Push(Null);
end;

function TObjectMVCArray.RemoveAt(i: Integer): TDispObject;
var
  Dispatch: OleVariant;
  IndexStr: String;
begin
  Result:=nil;
  Dispatch:=inherited RemoveAt(i);
  if VarIsValidDispatch(Dispatch) then
  begin
    IndexStr:=IntToStr(i);
    if FItems.IndexOf(IndexStr)<0                          //noch kein zugehriges Delphi-Objekt angelegt
      then Result:=CreateObject(Dispatch)
      else Result:=GetObject(i,Dispatch,true);
  end;
end;

procedure TObjectMVCArray.SetAt(i: Integer; Elem: TDispObject);
begin
  if assigned(Elem)
    then inherited SetAt(i,Elem.Disp)
    else inherited SetAt(i,Null);
end;

//------------------------------------------------------------------------------
//TMVCLatLngArray

function TLatLngMVCArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=CreateObjectWrapper(Script,TLatLng,Dispatch);
end;

function TLatLngMVCArray.Add(Lat, Lng: Double; NoWrap: Boolean): TLatLng;
begin
  Result:=TLatLng.Create(Script,Lat,Lng,NoWrap);
  AddObject(Result);
end;

procedure TLatLngMVCArray.InsertAt(i: Integer; Elem: TLatLng);
begin
  inherited InsertAt(i,Elem);
end;

procedure TLatLngMVCArray.SetAt(i: Integer; Elem: TLatLng);
begin
  inherited SetAt(i,Elem);
end;

function TLatLngMVCArray.GetAt(i: Integer): TLatLng;
begin
  Result:=TLatLng(inherited GetAt(i));
end;

function TLatLngMVCArray.Pop: TLatLng;
begin
  Result:=TLatLng(inherited Pop);
end;

function TLatLngMVCArray.Push(Elem: TLatLng): Integer;
begin
  Result:=inherited Push(Elem);
end;

function TLatLngMVCArray.RemoveAt(i: Integer): TLatLng;
begin
  Result:=TLatLng(inherited RemoveAt(i));
end;

//------------------------------------------------------------------------------
//TLatLngMVCArrayMVCArray

function TLatLngMVCArrayMVCArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=CreateObjectWrapper(Script,TLatLngMVCArray,Dispatch);
end;

function TLatLngMVCArrayMVCArray.Add: TLatLngMVCArray;
{$IFDEF DELPHIXE2}
var
  Elements: array of OleVariant;
{$ENDIF}
begin
  {$IFDEF DELPHIXE2}
  SetLength(Elements,0);
  Result:=TLatLngMVCArray.Create(Script,Elements);         //[] -> Compiler Error
  {$ELSE}
  Result:=TLatLngMVCArray.Create(Script,[]);
  {$ENDIF}
  AddObject(Result);
end;

function TLatLngMVCArrayMVCArray.GetAt(i: Integer): TLatLngMVCArray;
begin
  Result:=TLatLngMVCArray(inherited GetAt(i));
end;

procedure TLatLngMVCArrayMVCArray.InsertAt(i: Integer; Elem: TLatLngMVCArray);
begin
  inherited InsertAt(i,Elem);
end;

function TLatLngMVCArrayMVCArray.Pop: TLatLngMVCArray;
begin
  Result:=TLatLngMVCArray(inherited Pop);
end;

function TLatLngMVCArrayMVCArray.Push(Elem: TLatLngMVCArray): Integer;
begin
  Result:=inherited Push(Elem);
end;

function TLatLngMVCArrayMVCArray.RemoveAt(i: Integer): TLatLngMVCArray;
begin
  Result:=TLatLngMVCArray(inherited RemoveAt(i));
end;

procedure TLatLngMVCArrayMVCArray.SetAt(i: Integer; Elem: TLatLngMVCArray);
begin
  inherited SetAt(i,Elem);
end;

//------------------------------------------------------------------------------
//TMVCNodeArray

function TNodeMVCArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=CreateObjectWrapper(Script,TNode,Dispatch);
end;

function TNodeMVCArray.Add(ElementType: String): TNode;
begin
  Result:=Script.Document.CreateElement(ElementType);
  AddObject(Result);
end;

procedure TNodeMVCArray.InsertAt(i: Integer; Elem: TNode);
begin
  inherited InsertAt(i,Elem);
end;

procedure TNodeMVCArray.SetAt(i: Integer; Elem: TNode);
begin
  inherited SetAt(i,Elem);
end;

function TNodeMVCArray.GetAt(i: Integer): TNode;
begin
  Result:=TNode(inherited GetAt(i));
end;

function TNodeMVCArray.Pop: TNode;
begin
  Result:=TNode(inherited Pop);
end;

function TNodeMVCArray.Push(Elem: TNode): Integer;
begin
  Result:=inherited Push(Elem);
end;

function TNodeMVCArray.RemoveAt(i: Integer): TNode;
begin
  Result:=TNode(inherited RemoveAt(i));
end;

//------------------------------------------------------------------------------
//TMVCNodeArrayArray

function TNodeMVCArrayArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TNodeMVCArray.Create(Script,Dispatch);
end;

function TNodeMVCArrayArray.GetItem(Index: Integer): TNodeMVCArray;
begin
  Result:=TNodeMVCArray(inherited Items[Index]);
end;

procedure TNodeMVCArrayArray.SetItem(Index: Integer; Value: TNodeMVCArray);
begin
  inherited Items[Index]:=Value;
end;

end.





