unit gmMap;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, Controls, Menus, SysUtils, {$IFDEF DEBUG}Dialogs,{$ENDIF} Graphics,
  {$IFDEF DELPHI6_UP}Variants,{$ENDIF} BrowserTools, DispObject, JScriptObjects,
  HTMLObjects, gmBase, gmMapTypes, gmMVC, gmStreetView, gmEvents, gmControls,
  gmServicesMaxZoom, gmAdditionalConstants, gmAdditionalMessages;

type
  TMap = class;
  TMapOptions = class;
  TMapTypeID = class;

  TMap = class(TMVCObject)
  private
    FBounds: TLatLngBounds;
    FCenter: TLatLng;
    FContainer: TNode;
    FControls: TNodeMVCArrayArray;
    FMapTypeRegistry: TMapTypeRegistry;
    FOverlayMapTypes: TMVCArray;
    FPopupMenu: TPopupMenu;
    FProjection: TProjection;
    FStreetViewPanorama: TStreetViewPanorama;
    FOnBoundsChanged: TNotifyEvent;
    FOnBoundsChangedListener: TMapsEventListener;
    FOnBoundsChangedFunction: TJScriptFunction;
    FOnCenterChanged: TNotifyEvent;
    FOnCenterChangedListener: TMapsEventListener;
    FOnCenterChangedFunction: TJScriptFunction;
    FOnClick: TMapMouseEvent;
    FOnClickListener: TMapsEventListener;
    FOnClickFunction: TJScriptFunction;
    FOnDblClick: TMapMouseEvent;
    FOnDblClickListener: TMapsEventListener;
    FOnDblClickFunction: TJScriptFunction;
    FOnDrag: TNotifyEvent;
    FOnDragListener: TMapsEventListener;
    FOnDragFunction: TJScriptFunction;
    FOnDragEnd: TNotifyEvent;
    FOnDragEndListener: TMapsEventListener;
    FOnDragEndFunction: TJScriptFunction;
    FOnDragStart: TNotifyEvent;
    FOnDragStartListener: TMapsEventListener;
    FOnDragStartFunction: TJScriptFunction;
    FOnHeadingChanged: TNotifyEvent;
    FOnHeadingChangedListener: TMapsEventListener;
    FOnHeadingChangedFunction: TJScriptFunction;
    FOnIdle: TNotifyEvent;
    FOnIdleListener: TMapsEventListener;
    FOnIdleFunction: TJScriptFunction;
    FOnMapTypeIDChanged: TNotifyEvent;
    FOnMapTypeIDChangedListener: TMapsEventListener;
    FOnMapTypeIDChangedFunction: TJScriptFunction;
    FOnMouseMove: TMapMouseEvent;
    FOnMouseMoveListener: TMapsEventListener;
    FOnMouseMoveFunction: TJScriptFunction;
    FOnMouseOut: TMapMouseEvent;
    FOnMouseOutListener: TMapsEventListener;
    FOnMouseOutFunction: TJScriptFunction;
    FOnMouseOver: TMapMouseEvent;
    FOnMouseOverListener: TMapsEventListener;
    FOnMouseOverFunction: TJScriptFunction;
    FOnProjectionChanged: TNotifyEvent;
    FOnProjectionChangedListener: TMapsEventListener;
    FOnProjectionChangedFunction: TJScriptFunction;
    FOnResize: TNotifyEvent;
    FOnResizeListener: TMapsEventListener;
    FOnResizeFunction: TJScriptFunction;
    FOnRightClick: TMapMouseEvent;
    FOnRightClickListener: TMapsEventListener;
    FOnRightClickFunction: TJScriptFunction;
    FOnTilesLoaded: TNotifyEvent;
    FOnTilesLoadedListener: TMapsEventListener;
    FOnTilesLoadedFunction: TJScriptFunction;
    FOnTiltChanged: TNotifyEvent;
    FOnTiltChangedListener: TMapsEventListener;
    FOnTiltChangedFunction: TJScriptFunction;
    FOnZoomChanged: TNotifyEvent;
    FOnZoomChangedListener: TMapsEventListener;
    FOnZoomChangedFunction: TJScriptFunction;
    procedure Set__e3_(Value: IDispatch);
    procedure SetA(Value: IDispatch);
    procedure SetAlign(Value: TAlign);
    procedure SetB(Value: IDispatch);
    procedure SetC(Value: IDispatch);
    procedure SetControls(Value: TNodeMVCArrayArray);
    procedure SetD(Value: IDispatch);
    procedure SetG(Value: IDispatch);
    procedure SetGmAccessors(Value: IDispatch);
    procedure SetGmBindings(Value: IDispatch);
    procedure SetHeight(Value: Integer);
    procedure SetI(Value: IDispatch);
    procedure SetLeft(Value: Integer);
    procedure SetMapTypeControl(Value: Boolean);
    procedure SetMapTypes(Value: TMapTypeRegistry);
    procedure SetNavigationControl(Value: Boolean);
    procedure SetOverlayMapTypes(Value: TMVCArray);
    procedure SetPopupMenu(Value: TPopupMenu);
    procedure SetScaleControl(Value: Boolean);
    procedure SetStreetViewControl(Value: Boolean);
    procedure SetTa(Value: IDispatch);
    procedure SetTop(Value: Integer);
    procedure SetWidth(Value: Integer);
    procedure SetXa(Value: IDispatch);
    procedure SetOnBoundsChanged(Value: TNotifyEvent);
    procedure SetOnCenterChanged(Value: TNotifyEvent);
    procedure SetOnClick(Value: TMapMouseEvent);
    procedure SetOnDblClick(Value: TMapMouseEvent);
    procedure SetOnDrag(Value: TNotifyEvent);
    procedure SetOnDragEnd(Value: TNotifyEvent);
    procedure SetOnDragStart(Value: TNotifyEvent);
    procedure SetOnHeadingChanged(Value: TNotifyEvent);
    procedure SetOnIdle(Value: TNotifyEvent);
    procedure SetOnMapTypeIDChanged(Value: TNotifyEvent);
    procedure SetOnMouseMove(Value: TMapMouseEvent);
    procedure SetOnMouseOut(Value: TMapMouseEvent);
    procedure SetOnMouseOver(Value: TMapMouseEvent);
    procedure SetOnProjectionChanged(Value: TNotifyEvent);
    procedure SetOnResize(Value: TNotifyEvent);
    procedure SetOnRightClick(Value: TMapMouseEvent);
    procedure SetOnTilesLoaded(Value: TNotifyEvent);
    procedure SetOnTiltChanged(Value: TNotifyEvent);
    procedure SetOnZoomChanged(Value: TNotifyEvent);
    function Get__e3_: IDispatch;
    function GetA: IDispatch;
    function GetAlign: TAlign;
    function GetB: IDispatch;
    function GetC: IDispatch;
    function GetControls: TNodeMVCArrayArray;
    function GetD: IDispatch;
    function GetG: IDispatch;
    function GetGmAccessors: IDispatch;
    function GetGmBindings: IDispatch;
    function GetHeight: Integer;
    function GetI: IDispatch;
    function GetLeft: Integer;
    function GetMapTypeControl: Boolean;
    function GetMapTypes: TMapTypeRegistry;
    function GetNavigationControl: Boolean;
    function GetOverlayMapTypes: TMVCArray;
    function GetScaleControl: Boolean;
    function GetStreetViewControl: Boolean;
    function GetTa: IDispatch;
    function GetTop: Integer;
    function GetWidth: Integer;
    function GetXa: IDispatch;
  protected
    class function New(Script: TCustomScript; MapDiv: TNode; Opts: TMapOptions): IDispatch; reintroduce;
    //offenbar interne Objekte und Methoden:
    function N: OleVariant;
    function P: OleVariant;
    property __e3_: IDispatch read Get__e3_ write Set__e3_;
    property A: IDispatch read GetA write SetA;
    property B: IDispatch read GetB write SetB;
    property C: IDispatch read GetC write SetC;
    property D: IDispatch read GetD write SetD;
    property G: IDispatch read GetG write SetG;
    property I: IDispatch read GetI write SetI;
    property GmAccessors: IDispatch read GetGmAccessors write SetGmAccessors;
    property GmBindings: IDispatch read GetGmBindings write SetGmBindings;
    property Ta: IDispatch read GetTa write SetTa;
    property Xa: IDispatch read GetXa write SetXa;
    //von JScript aufgerufene Event-Handler:
    procedure BoundsChanged;
    procedure CenterChanged;
    procedure Click;
    procedure DblClick;
    procedure Drag;
    procedure Dragend;
    procedure DragStart;
    procedure HeadingChanged;
    procedure Idle;
    procedure MapTypeIDChanged;
    procedure MouseMove;
    procedure MouseOut;
    procedure MouseOver;
    procedure ProjectionChanged;
    procedure Resize;
    procedure RightClick;
    procedure TilesLoaded;
    procedure TiltChanged;
    procedure ZoomChanged;
  public
    constructor Create(Opts: TMapOptions); overload;
    constructor Create(MapDiv: TNode; Opts: TMapOptions); overload;
    constructor Create(Script: TCustomScript; Opts: TMapOptions); overload;
    constructor Create(Script: TCustomScript; MapDiv: TNode; Opts: TMapOptions); overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Passt die Karten an die angegebenen Grenzen an:
    //Sets the viewport to contain the given bounds:
    procedure FitBounds(Bounds: TLatLngBounds);
    //Verschiebt den Mittelpunkt der Karte um die in Pixeln angegebene Entfernung.
    //Wenn die Entfernung geringer ist als sowohl die Breite als auch die Hhe
    //der Karte, wird der bergang gleitend animiert. Beachten Sie, dass die
    //Koordinaten der Karte von Westen nach Osten (x-Werte) und von Norden nach
    //Sden (y-Werte) grer werden:
    //Changes the center of the map by the given distance in pixels. If the distance
    //is less than both the width and height of the map, the transition will be
    //smoothly animated. Note that the map coordinate system increases from west
    //to east (for x values) and north to south (for y values):
    procedure PanBy(x, y: Integer);
    //Verschiebt das Zentrum der Karte auf die angegebene LatLng. Wenn die
    //Verschiebung geringer ist als sowohl die Breite als auch die Hhe der Karte,
    //wird der bergang gleitend animiert:
    //Changes the center of the map to the given LatLng. If the change is less than
    //both the width and height of the map, the transition will be smoothly animated:
    procedure PanTo(LatLng: TLatLng);
    //Schwenkt die Karte um das geringstntige Stck, sodass sie die angegebenen
    //LatLngBounds enthlt. Es wird nicht garantiert, wo sich die Grenzen auf der
    //Karte befinden, sondern nur, dass so viel wie mglich von den Grenzen
    //sichtbar ist. Falls sich die Grenzen auf der Karte befinden, werden sie
    //innerhalb des Bereiches positioniert, der durch den Kartentyp und die
    //Navigationssteuerelemente begrenzt wird. Falls die Grenzen ber die Karte
    //hinausgehen, wird die Karte so verschoben, dass die nordwestliche Ecke der
    //Grenzen sichtbar ist. Wenn die Verschiebung der Kartenposition geringer ist als
    //sowohl die Breite als auch die Hhe der Karte, wird der bergang gleitend animiert:
    //Pans the map by the minimum amount necessary to contain the given LatLngBounds.
    //It makes no guarantee where on the map the bounds will be, except that as much
    //of the bounds as possible will be visible. The bounds will be positioned inside
    //the area bounded by the map type and navigation (pan, zoom, and Street View)
    //controls, if they are present on the map. If the bounds is larger than the map,
    //the map will be shifted to include the northwest corner of the bounds. If the
    //change in the map's position is less than both the width and height of the map,
    //the transition will be smoothly animated:
    procedure PanToBounds(LatLngBounds: TLatLngBounds);
    //Developers should trigger this event on the map when the div changes size:
    //google.maps.event.trigger(map,'resize')
    procedure Resized;
    //Setzt das Kartenzentrum auf die angegebenen Koorinaten:
    procedure SetCenter(LatLng: TLatLng);
    //Sets the compass heading for aerial imagery measured in degrees from cardinal
    //direction North:
    procedure SetHeading(Heading: Integer);
    //Setzt den angezeigten Kartentyp:
    procedure SetMapTypeID(MapTypeID: String);
    //Setzt die Karteneigenschaften entsprechend der bergebenen Optionen:
    procedure SetOptions(Options: TMapOptions); reintroduce; virtual;
    //Verknpft ein StreetViewPanorama mit der Karte. Dieses Panorama berschreibt
    //das standardmige StreetViewPanorama. Somit kann also ein externes Panorama
    //auerhalb der Karte mit der Karte verknpft werden. Wenn fr das Panorama
    //der Wert null festgelegt wird, wird wieder das standardmige eingebettete
    //Panorama mit der Karte verknpft:
    //Binds a StreetViewPanorama to the map. This panorama overrides the default
    //StreetViewPanorama, allowing the map to bind to an external panorama outside
    //of the map. Setting the panorama to nil binds the default embedded panorama
    //back to the map:
    procedure SetStreetView(Panorama: TStreetViewPanorama);
    //Sets the angle of incidence for aerial imagery (available for SATELLITE and
    //HYBRID map types) measured in degrees from the viewport plane to the map plane.
    //The only supported values are 0, indicating no angle of incidence (no tilt),
    //and 45 indicating a tilt of 45deg:
    procedure SetTilt(Tilt: Integer);
    procedure SetZoom(Zoom: Integer);
    //Gibt die Breite-/Lnge-Grenzen des aktuellen Darstellungsbereichs wieder.
    //Wenn die Karte noch nicht initialisiert ist (d. h. mapType ist immer noch null)
    //oder wenn das Zentrum oder die Vergrerung noch nicht festgelegt worden sind,
    //ist das Ergebnis null:
    //Returns the lat/lng bounds of the current viewport. If the map is not yet
    //initialized (i.e. the mapType is still null), or center and zoom have not
    //been set then the result is null or undefined:
    function GetBounds: TLatLngBounds;
    //Gibt die Koordinaten des Kartenmittelpunktes an:
    function GetCenter: TLatLng;
    //Gibt den Container der Karte an:
    function GetDiv: TNode;
    //Returns the compass heading of aerial imagery. The heading value is measured
    //in degrees (clockwise) from cardinal direction North:
    function GetHeading: Integer;
    //Gibt den momentan angezeigten Kartentyp an:
    function GetMapTypeID: String;
    //Gibt die aktuelle Projection zurck. Wenn die Karte noch nicht initiiert ist,
    //das heit wenn der mapType noch null ist, ist das Ergebnis null. berwachen
    //Sie projection_changed und vergewissern Sie sich, dass der Wert nicht null ist:
    //Returns the current Projection. If the map is not yet initialized (i.e. the
    //mapType is still null) then the result is null. Listen to projection_changed
    //and check its value to ensure it is not nil:
    function GetProjection: TProjection;
    //ab Version 3.3.1a nicht mehr vorhanden:
    //function GetRotatable: OleVariant;
    //Gibt das mit der Karte verknpfte standardmige StreetViewPanorama zurck;
    //dabei kann es sich um ein in die Karte eingebettetes Standardpanorama handeln
    //oder um das Panorama, das durch setStreetView() festgelegt wurde. nderungen
    //an der streetViewControl der Karte spiegeln sich in der Anzeige des
    //verknpften Panoramas wider:
    //Returns the default StreetViewPanorama bound to the map, which may be a default
    //panorama embedded within the map, or the panorama set using setStreetView().
    //Changes to the map's streetViewControl will be reflected in the display of
    //such a bound panorama:
    function GetStreetView: TStreetViewPanorama;
    //Returns the angle of incidence for aerial imagery (available for SATELLITE
    //and HYBRID map types) measured in degrees from the viewport plane to the map
    //plane. A value of 0 indicates no angle of incidence (no tilt) while 45
    //imagery will return a value of 45:
    function GetTilt: Integer;
    //Gibt die aktuelle Zoomstufe der Karte an:
    function GetZoom: Integer;
    //zustzliche Eigenschaft zu den Methoden GetCenter und SetCenter:
    property Align: TAlign read GetAlign write SetAlign;
    property Bounds: TLatLngBounds read GetBounds write FitBounds;
    property Center: TLatLng read GetCenter write SetCenter;
    property Container: TNode read GetDiv;
    //Fgt der Karte zustzliche Steuerelemente hinzu. Wenn Sie der Karte ein
    //Steuerelement hinzufgen mchten, fgen Sie die <div> des Steuerelements
    //entsprechend der ControlPosition, an der es dargestellt werden soll, zum
    //MVCArray hinzu:
    //Additional controls to attach to the map. To add a control to the map, add
    //the control's <div> to the MVCArray corresponding to the ControlPosition
    //where it should be rendered:
    property Controls: TNodeMVCArrayArray read GetControls write SetControls;
    property Heading: Integer read GetHeading write SetHeading;
    property Height: Integer read GetHeight write SetHeight;
    property Left: Integer read GetLeft write SetLeft;
    //zustzliche Eigenschaft zu den Methoden GetMapTypeID und SetMapTypeID:
    property MapTypeID: String read GetMapTypeID write SetMapTypeID;
    //zustzlich:
    property MapTypeControl: Boolean read GetMapTypeControl write SetMapTypeControl;
    //Eine Registrierung mit MapType-Instanzen, geordnet nach Zeichenfolgen-ID:
    //A registry of MapType instances by string ID:
    property MapTypes: TMapTypeRegistry read GetMapTypes write SetMapTypes;
    //zustzlich:
    property NavigationControl: Boolean read GetNavigationControl write SetNavigationControl;
    //Zustzliche Kartentypen fr Overlays:
    //Additional map types to overlay:
    property OverlayMapTypes: TMVCArray read GetOverlayMapTypes write SetOverlayMapTypes;
    //zustzlich:
    property PopupMenu: TPopupMenu read FPopupMenu write SetPopupMenu;
    //zustzlich:
    property Projection: TProjection read GetProjection;
    //zustzlich:
    property ScaleControl: Boolean read GetScaleControl write SetScaleControl;
    //zustzlich:
    property StreetViewControl: Boolean read GetStreetViewControl write SetStreetViewControl;
    //zustzliche Eigenschaft zu den Methoden GetStreetView und SetStreetView:
    property StreetView: TStreetViewPanorama read GetStreetView write SetStreetView;
    //zustzliche Eigenschaft zu den Methoden GetTilt und SetTilt:
    property Tilt: Integer read GetTilt write SetTilt;
    property Top: Integer read GetTop write SetTop;
    property Width: Integer read GetWidth write SetWidth;
    //zustzliche Eigenschaft zu den Methoden GetZoom und SetZoom:
    property Zoom: Integer read GetZoom write SetZoom;
    //Dieses Ereignis wird ausgelst, wenn sich die Darstellungsbereichsgrenzen gendert haben:
    //This event is fired when the viewport bounds have changed:
    property OnBoundsChanged: TNotifyEvent read FOnBoundsChanged write SetOnBoundsChanged;
    //Dieses Ereignis wird ausgelst, wenn sich die center-Eigenschaft der Karte ndert:
    //This event is fired when the map center property changes:
    property OnCenterChanged: TNotifyEvent read FOnCenterChanged write SetOnCenterChanged;
    //Dieses Ereignis wird ausgelst, wenn der Nutzer auf die Karte klickt
    //(jedoch nicht, wenn er auf eine Markierung oder ein Infofenster klickt):
    //This event is fired when the user clicks on the map (but not when they click
    //on a marker or infowindow):
    property OnClick: TMapMouseEvent read FOnClick write SetOnClick;
    //Dieses Ereignis wird ausgelst, wenn der Nutzer auf die Karte doppelklickt.
    //Zustzlich wird davor ein Klick-Ereignis ausgelst:
    //This event is fired when the user double-clicks on the map. Note that the click
    //event will also fire, right before this one:
    property OnDblClick: TMapMouseEvent read FOnDblClick write SetOnDblClick;
    //Dieses Ereignis wird wiederholt ausgelst, whrend der Nutzer die Karte verschiebt:
    //This event is repeatedly fired while the user drags the map:
    property OnDrag: TNotifyEvent read FOnDrag write SetOnDrag;
    //Dieses Ereignis wird ausgelst, wenn der Nutzer das Verschieben der Karte beendet:
    //This event is fired when the user stops dragging the map:
    property OnDragEnd: TNotifyEvent read FOnDragend write SetOnDragEnd;
    //Dieses Ereignis wird ausgelst, wenn der Nutzer beginnt, die Karte zu verschieben:
    //This event is fired when the user starts dragging the map:
    property OnDragStart: TNotifyEvent read FOnDragStart write SetOnDragStart;
    //This event is fired when the map heading property changes:
    property OnHeadingChanged: TNotifyEvent read FOnHeadingChanged write SetOnHeadingChanged;
    //Dieses Ereignis wird ausgelst, wenn die Karte nach dem Schwenken oder Zoomen inaktiv wird:
    //This event is fired when the map becomes idle after panning or zooming:
    property OnIdle: TNotifyEvent read FOnIdle write SetOnIdle;
    //Dieses Ereignis wird ausgelst, wenn sich die mapTypeId-Eigenschaft ndert:
    //This event is fired when the mapTypeId property changes:
    property OnMapTypeIDChanged: TNotifyEvent read FOnMapTypeIDChanged write SetOnMapTypeIDChanged;
    //Dieses Ereignis wird immer dann ausgelst, wenn der Nutzer den Mauszeiger
    //ber dem Kartencontainer bewegt:
    //This event is fired whenever the user's mouse moves over the map container:
    property OnMouseMove: TMapMouseEvent read FOnMouseMove write SetOnMouseMove;
    //Dieses Ereignis wird ausgelst, wenn der Nutzer mit dem Mauszeiger den Kartencontainer verlsst:
    //This event is fired when the user's mouse exits the map container:
    property OnMouseOut: TMapMouseEvent read FOnMouseOut write SetOnMouseOut;
    //Dieses Ereignis wird ausgelst, wenn der Nutzer den Mauszeiger in den Kartencontainer hineinbewegt:
    //This event is fired when the user's mouse enters the map container:
    property OnMouseOver: TMapMouseEvent read FOnMouseOver write SetOnMouseOver;
    //Dieses Ereignis wird ausgelst, wenn sich die "projection" gendert hat:
    //This event is fired when the projection has changed:
    property OnProjectionChanged: TNotifyEvent read FOnProjectionChanged write SetOnProjectionChanged;
    //Entwickler sollten dieses Ereignis fr die Karte auslsen, wenn sich die Gre der
    //DIV ndert: google.maps.event.trigger(map,'resize'):
    //Developers should trigger this event on the map when the div changes size:
    //google.maps.event.trigger(map,'resize'):
    property OnResize: TNotifyEvent read FOnResize write SetOnResize;
    //Dieses Ereignis tritt ein, wenn das DOM-Kontextmenereignis fr den Kartencontainer
    //ausgelst wird:
    //This event is fired when the DOM contextmenu event is fired on the map container:
    property OnRightClick: TMapMouseEvent read FOnRightClick write SetOnRightClick;
    //Dieses Ereignis wird ausgelst, wenn die sichtbaren Kacheln vollstndig geladen sind:
    //This event is fired when the visible tiles have finished loading:
    property OnTilesLoaded: TNotifyEvent read FOnTilesLoaded write SetOnTilesLoaded;
    //This event is fired when the map tilt property changes:
    property OnTiltChanged: TNotifyEvent read FOnTiltChanged write SetOnTiltChanged;
    //Dieses Ereignis wird ausgelst, wenn sich die zoom-Eigenschaft der Karte gendert hat:
    //This event is fired when the map zoom property changes:
    property OnZoomChanged: TNotifyEvent read FOnZoomChanged write SetOnZoomChanged;
  end;

  TMapOptions = class(TCustomJScriptObject)
  private
    FAlign: TAlign;
    FCenter: TLatLng;
    FHeight: Integer;
    FLeft: Integer;
    FMapTypeControlOptions: TMapTypeControlOptions;
    FNavigationControlOptions: TNavigationControlOptions;
    FOverviewMapControlOptions: TOverviewMapControlOptions;
    FPanControlOptions: TPanControlOptions;
    FRotateControlOptions: TRotateControlOptions;
    FScaleControlOptions: TScaleControlOptions;
    FStreetView: TStreetViewPanorama;
    FStreetViewControlOptions: TStreetViewControlOptions;
    FStyles: TMapTypeStyleArray;
    FTop: Integer;
    FWidth: Integer;
    FZoomControlOptions: TZoomControlOptions;
    procedure SetBackgroundColor(Value: TColor);
    procedure SetCenter(Value: TLatLng);
    procedure SetDisableDefaultUI(Value: Boolean);
    procedure SetDisableDoubleClickZoom(Value: Boolean);
    procedure SetDraggable(Value: Boolean);
    procedure SetDraggableCursor(Value: String);
    procedure SetDraggingCursor(Value: String);
    procedure SetHeading(Value: Integer);
    procedure SetKeyboardShortcuts(Value: Boolean);
    procedure SetMapTypeControl(Value: Boolean);
    procedure SetMapTypeControlOptions(Value: TMapTypeControlOptions);
    procedure SetMapTypeID(Value: String);
    procedure SetMaxZoom(Value: Integer);
    procedure SetMinZoom(Value: Integer);
    procedure SetNavigationControl(Value: Boolean);
    procedure SetNavigationControlOptions(Value: TNavigationControlOptions);
    procedure SetNoClear(Value: Boolean);
    procedure SetOverviewMapControl(Value: Boolean);
    procedure SetOverviewMapControlOptions(Value: TOverviewMapControlOptions);
    procedure SetPanControl(Value: Boolean);
    procedure SetPanControlOptions(Value: TPanControlOptions);
    procedure SetRotateControl(Value: Boolean);
    procedure SetRotateControlOptions(Value: TRotateControlOptions);
    procedure SetScaleControl(Value: Boolean);
    procedure SetScaleControlOptions(Value: TScaleControlOptions);
    procedure SetScrollWheel(Value: Boolean);
    procedure SetStreetView(Value: TStreetViewPanorama);
    procedure SetStreetViewControl(Value: Boolean);
    procedure SetStreetViewControlOptions(Value: TStreetViewControlOptions);
    procedure SetStyles(Value: TMapTypeStyleArray);
    procedure SetTilt(Value: Integer);
    procedure SetZoom(Value: Integer);
    procedure SetZoomControl(Value: Boolean);
    procedure SetZoomControlOptions(Value: TZoomControlOptions);
    function GetBackgroundColor: TColor;
    function GetCenter: TLatLng;
    function GetDisableDefaultUI: Boolean;
    function GetDisableDoubleClickZoom: Boolean;
    function GetDraggable: Boolean;
    function GetDraggableCursor: String;
    function GetDraggingCursor: String;
    function GetHeading: Integer;
    function GetKeyboardShortcuts: Boolean;
    function GetMapTypeControl: Boolean;
    function GetMapTypeControlOptions: TMapTypeControlOptions;
    function GetMapTypeID: String;
    function GetMaxZoom: Integer;
    function GetMinZoom: Integer;
    function GetNavigationControl: Boolean;
    function GetNavigationControlOptions: TNavigationControlOptions;
    function GetNoClear: Boolean;
    function GetOverviewMapControl: Boolean;
    function GetOverviewMapControlOptions: TOverviewMapControlOptions;
    function GetPanControl: Boolean;
    function GetPanControlOptions: TPanControlOptions;
    function GetRotateControl: Boolean;
    function GetRotateControlOptions: TRotateControlOptions;
    function GetScaleControl: Boolean;
    function GetScaleControlOptions: TScaleControlOptions;
    function GetScrollWheel: Boolean;
    function GetStreetView: TStreetViewPanorama;
    function GetStreetViewControl: Boolean;
    function GetStreetViewControlOptions: TStreetViewControlOptions;
    function GetStyles: TMapTypeStyleArray;
    function GetTilt: Integer;
    function GetZoom: Integer;
    function GetZoomControl: Boolean;
    function GetZoomControlOptions: TZoomControlOptions;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    constructor Create(LatLng: TLatLng; MapTypeID: String; Zoom: Integer); overload;
    constructor Create(CenterX, CenterY: Double; MapTypeID: String; Zoom: Integer); overload;
    constructor Create(Script: TCustomScript; CenterX, CenterY: Double; MapTypeID: String; Zoom: Integer); overload;
    destructor Destroy; override;
    procedure AfterConstruction; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    property Align: TAlign read FAlign write FAlign;
    //Fr den Hintergrund der Karten-div verwendete Farbe. Diese Farbe wird angezeigt,
    //wenn noch keine Kacheln geladen wurden und der Nutzer die Karte schwenkt. Diese
    //Option kann nur festgelegt werden, wenn die Karte initiiert ist:
    //Color used for the background of the Map div. This color will be visible when
    //tiles have not yet loaded as the user pans. This option can only be set when
    //the map is initialized:
    property BackgroundColor: TColor read GetBackgroundColor write SetBackgroundColor;
    //Anfngliches Kartenzentrum. Erforderlich:
    //The initial Map center. Required:
    property Center: TLatLng read GetCenter write SetCenter;
    //Aktiviert/inaktiviert alle Standard-UI. Kann im Einzelfall auer Kraft gesetzt werden:
    //Enables/disables all default UI. May be overridden individually:
    property DisableDefaultUI: Boolean read GetDisableDefaultUI write SetDisableDefaultUI;
    //Aktiviert/deaktiviert das Zoomen und Zentrieren durch Doppelklick. Standardmig aktiviert:
    //Enables/disables zoom and center on double click. Enabled by default:
    property DisableDoubleClickZoom: Boolean read GetDisableDoubleClickZoom write SetDisableDoubleClickZoom;
    //Falls falsch, kann die Karte nicht verschoben werden. Verschieben ist standardmig aktiviert:
    //If false, prevents the map from being dragged. Dragging is enabled by default:
    property Draggable: Boolean read GetDraggable write SetDraggable;
    //Der Name oder der URL des Cursors, der fr ein verschiebbares Objekt angezeigt werden soll:
    //The name or url of the cursor to display on a draggable object:
    property DraggableCursor: String read GetDraggableCursor write SetDraggableCursor;
    //Der Name oder der URL des Cursors, der angezeigt werden soll, wenn ein Objekt verschoben wird:
    //The name or url of the cursor to display when an object is dragging:
    property DraggingCursor: String read GetDraggingCursor write SetDraggingCursor;
    //The heading for aerial imagery in degrees measured clockwise from cardinal direction North.
    //Headings are snapped to the nearest available angle for which imagery is available:
    property Heading: Integer read GetHeading write SetHeading;
    property Height: Integer read FHeight write FHeight;
    //Falls falsch, kann die Karte nicht ber die Tastatur gesteuert werden. Tastaturkrzel
    //sind standardmig aktiviert:
    //If false, prevents the map from being controlled by the keyboard. Keyboard
    //shortcuts are enabled by default:
    property KeyboardShortcuts: Boolean read GetKeyboardShortcuts write SetKeyboardShortcuts;
    property Left: Integer read FLeft write FLeft;
    //Anfnglicher Aktivierungs-/Inaktivierungsstatus der Kartentypsteuerung:
    //The initial enabled/disabled state of the Map type control:
    property MapTypeControl: Boolean read GetMapTypeControl write SetMapTypeControl;
    //Anfngliche Anzeigeoptionen fr die Kartentypsteuerung:
    //The initial display options for the Map type control:
    property MapTypeControlOptions: TMapTypeControlOptions read GetMapTypeControlOptions write SetMapTypeControlOptions;
    //Anfngliche mapTypeId der Karte. Erforderlich:
    //The initial Map mapTypeId. Required:
    property MapTypeID: String read GetMapTypeID write SetMapTypeID;
    //The maximum zoom level which will be displayed on the map. If omitted, or set to null,
    //the maximum zoom from the current map type is used instead.
    //Since Version 3.3.:
    property MaxZoom: Integer read GetMaxZoom write SetMaxZoom;
    //The minimum zoom level which will be displayed on the map. If omitted, or set to null,
    //the minimum zoom from the current map type is used instead.
    //Since Version 3.3.:
    property MinZoom: Integer read GetMinZoom write SetMinZoom;
    //Anfnglicher Aktivierungs-/Inaktivierungsstatus der Navigationssteuerung.
    //ab Version 3.3 veraltet
    //deprecated 'V3.3: Use ZoomControl and PanControl':
    property NavigationControl: Boolean read GetNavigationControl write SetNavigationControl;
    //Anfngliche Anzeigeoptionen fr die Navigationssteuerung.
    //ab Version 3.3 veraltet
    //deprecated 'V3.3: Use ZoomControlOption and PanControlOptions':
    property NavigationControlOptions: TNavigationControlOptions read GetNavigationControlOptions write SetNavigationControlOptions;
    //Falls wahr, den Inhalt der Karten-div nicht lschen:
    //If true, do not clear the contents of the Map div:
    property NoClear: Boolean read GetNoClear write SetNoClear;
    //The enabled/disabled state of the Overview Map control:
    property OverviewMapControl: Boolean read GetOverviewMapControl write SetOverviewMapControl;
    //The display options for the Overview Map control:
    property OverviewMapControlOptions: TOverviewMapControlOptions read GetOverviewMapControlOptions write SetOverviewMapControlOptions;
    //The enabled/disabled state of the pan control.
    //Since Version 3.3.:
    property PanControl: Boolean read GetPanControl write SetPanControl;
    //The display options for the pan control.
    //Since ab Version 3.3.:
    property PanControlOptions: TPanControlOptions read GetPanControlOptions write SetPanControlOptions;
    //The enabled/disabled state of the Rotate control:
    property RotateControl: Boolean read GetRotateControl write SetRotateControl;
    //The display options for the Rotate control:
    property RotateControlOptions: TRotateControlOptions read GetRotateControlOptions write SetRotateControlOptions;
    //Anfnglicher Aktivierungs-/Inaktivierungsstatus der Mastabssteuerung:
    //The initial enabled/disabled state of the Scale control:
    property ScaleControl: Boolean read GetScaleControl write SetScaleControl;
    //Anfngliche Anzeigeoptionen fr die Mastabssteuerung:
    //The initial display options for the Scale control:
    property ScaleControlOptions: TScaleControlOptions read GetScaleControlOptions write SetScaleControlOptions;
    //Falls falsch, ist das Zoomen der Karte mit dem Scrollrad inaktiviert.
    //Das Scrollrad ist standardmig aktiviert:
    //If false, disables scrollwheel zooming on the map. The scrollwheel is
    //enabled by default:
    property ScrollWheel: Boolean read GetScrollWheel write SetScrollWheel;
    //StreetViewPanorama, das angezeigt werden soll, wenn das Street View-Mnnchen
    //auf die Karte gezogen wird. Falls kein Panorama angegeben ist, wird ein
    //Standard-StreetViewPanorama in der div der Karte angezeigt, sobald das Street
    //View-Mnnchen auf die Karte gezogen wird:
    //A StreetViewPanorama to display when the Street View pegman is dropped on the
    //map. If no panorama is specified, a default StreetViewPanorama will be displayed
    //in the map's div when the pegman is dropped:
    property StreetView: TStreetViewPanorama read GetStreetView write SetStreetView;
    //Der anfngliche Status (aktiviert/deaktiviert) des Street View-Mnnchen-Steuerelements:
    //The initial enabled/disabled state of the Street View Pegman control:
    property StreetViewControl: Boolean read GetStreetViewControl write SetStreetViewControl;
    //The initial display options for the Street View Pegman control:
    property StreetViewControlOptions: TStreetViewControlOptions read GetStreetViewControlOptions write SetStreetViewControlOptions;
    //Styles to apply to each of the default map types. Note that styles will
    //apply only to the labels and geometry in Satellite/Hybrid and Terrain modes:
    property Styles: TMapTypeStyleArray read GetStyles write SetStyles;
    //The angle of incidence of the map as measured in degrees from the viewport
    //plane to the map plane. The only currently supported values are 0, indicating
    //no angle of incidence (no tilt), and 45, indicating a tilt of 45deg;. 45deg;
    //imagery is only available for SATELLITE and HYBRID map types, within some
    //locations, and at some zoom levels:
    property Tilt: Integer read GetTilt write SetTilt;
    property Top: Integer read FTop write FTop;
    property Width: Integer read FWidth write FWidth;
    //Anfngliche Zoomstufe der Karte. Erforderlich:
    //The initial Map zoom level. Required:
    property Zoom: Integer read GetZoom write SetZoom;
    //The enabled/disabled state of the zoom control:
    property ZoomControl: Boolean read GetZoomControl write SetZoomControl;
    //The display options for the zoom control:
    property ZoomControlOptions: TZoomControlOptions read GetZoomControlOptions write SetZoomControlOptions;
  end;

  TMapTypeID = class(TCustomJScriptObject)
  private
    procedure SetDi(Value: String);
    procedure SetHybrid(Value: String);
    procedure SetRoadmap(Value: String);
    procedure SetSatellite(Value: String);
    procedure SetTerrain(Value: String);
    function GetDi: String;
    function GetHybrid: String;
    function GetRoadmap: String;
    function GetSatellite: String;
    function GetTerrain: String;
  protected
    property Di: String read GetDi write SetDi;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Bei diesem Kartentyp wird eine transparente Ebene mit den Hauptstraen auf
    //Satellitenbildern angezeigt:
    //This map type displays a transparent layer of major streets on satellite images:
    property Hybrid: String read GetHybrid write SetHybrid;
    //Bei diesem Kartentyp wird eine normale Straenkarte angezeigt:
    //This map type displays a normal street map:
    property Roadmap: String read GetRoadmap write SetRoadmap;
    //Bei diesem Kartentyp werden Satellitenbilder angezeigt:
    //This map type displays satellite images:
    property Satellite: String read GetSatellite write SetSatellite;
    //Bei diesem Kartentyp werden Karten mit physischen Merkmalen wie Gelnde
    //und Vegetation angezeigt:
    //This map type displays maps with physical features such as terrain and vegetation:
    property Terrain: String read GetTerrain write SetTerrain;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TMap

constructor TMap.Create(Opts: TMapOptions);
begin
  Create(JScriptObjects.Script,nil,Opts);
end;

constructor TMap.Create(MapDiv: TNode; Opts: TMapOptions);
begin
  Create(JScriptObjects.Script,MapDiv,Opts);
end;

constructor TMap.Create(Script: TCustomScript; Opts: TMapOptions);
begin
  Create(Script,nil,Opts);
end;

constructor TMap.Create(Script: TCustomScript; MapDiv: TNode; Opts: TMapOptions);
begin
  if not assigned(Script) or not (Script is TScript)
    then raise Exception.Create(ExceptionNoScript);
  if not assigned(MapDiv) then
  begin
    MapDiv:=Script.Document.CreateDivNode;
    MapDiv.Style.BorderWidth:='0px';
    MapDiv.Style.Margin:='0px';
    MapDiv.Style.Padding:='0px';
    Script.Document.Body.AppendChild(MapDiv);
    FContainer:=MapDiv;
  end;
  if not assigned(MapDiv)
    then raise Exception.Create(ExceptionNoDiv);
  if not assigned(Opts)
    then raise Exception.Create(ExceptionNoOpts);
  with MapDiv do
  begin
    Style.Position:='absolute';
    Left:=Opts.Left;
    Top:=Opts.Top;
    Width:=Opts.Width;
    Height:=Opts.Height;
    Align:=Opts.Align;
  end;
  inherited Create(Script,New(Script,MapDiv,Opts));
end;

destructor TMap.Destroy;
begin
  SetOnBoundsChanged(nil);
  SetOnCenterChanged(nil);
  SetOnClick(nil);
  SetOnDblClick(nil);
  SetOnDrag(nil);
  SetOnDragend(nil);
  SetOnDragStart(nil);
  SetOnHeadingChanged(nil);
  SetOnIdle(nil);
  SetOnMapTypeIDChanged(nil);
  SetOnMouseMove(nil);
  SetOnMouseOut(nil);
  SetOnMouseOver(nil);
  SetOnProjectionChanged(nil);
  SetOnResize(nil);
  SetOnRightClick(nil);
  SetOnTilesLoaded(nil);
  SetOnTiltChanged(nil);
  SetOnZoomChanged(nil);
  FOnBoundsChangedFunction.Free;
  FOnCenterChangedFunction.Free;
  FOnClickFunction.Free;
  FOnDblClickFunction.Free;
  FOnDragFunction.Free;
  FOnDragendFunction.Free;
  FOnDragStartFunction.Free;
  FOnHeadingChangedFunction.Free;
  FOnIdleFunction.Free;
  FOnMapTypeIDChangedFunction.Free;
  FOnMouseMoveFunction.Free;
  FOnMouseOutFunction.Free;
  FOnMouseOverFunction.Free;
  FOnProjectionChangedFunction.Free;
  FOnResizeFunction.Free;
  FOnRightClickFunction.Free;
  FOnTilesLoadedFunction.Free;
  FOnTiltChangedFunction.Free;
  FOnZoomChangedFunction.Free;
  FBounds.Free;
  FCenter.Free;
  FContainer.Free;
  FControls.Free;
  FMapTypeRegistry.Free;
  FOverlayMapTypes.Free;
  FProjection.Free;
  FStreetViewPanorama.Free;
  inherited;
end;

class function TMap.New(Script: TCustomScript; MapDiv: TNode; Opts: TMapOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(MapDiv)
    then raise Exception.Create(ExceptionNoDiv);
  if not assigned(Opts)
    then raise Exception.Create(ExceptionNoOpts);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,MapsMethod_Map,
                              {$IFDEF CAN_OVERLOAD}
                              [MapDiv.Disp,Opts.Disp]
                              {$ELSE}
                              MapDiv.Disp,Opts.Disp
                              {$ENDIF}
                              ,true);
end;

class function TMap.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MapMethod_GetCenter,
                                 MapMethod_SetCenter,
                                 MapMethod_FitBounds]);
end;

function TMap.Get__e3_: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_E3);
end;

procedure TMap.Set__e3_(Value: IDispatch);
begin
  SetProp(MapProp_E3,Value);
end;

function TMap.GetA: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_A);
end;

procedure TMap.SetA(Value: IDispatch);
begin
  SetProp(MapProp_A,Value);
end;

function TMap.GetAlign: TAlign;
begin
  Result:=GetDiv.Align;
end;

procedure TMap.SetAlign(Value: TAlign);
begin
  if GetDiv.Align<>Value then
  begin
    GetDiv.Align:=Value;
    Resized;
  end;
end;

function TMap.GetB: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_B);
end;

procedure TMap.SetB(Value: IDispatch);
begin
  SetProp(MapProp_B,Value);
end;

function TMap.GetC: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_C);
end;

procedure TMap.SetC(Value: IDispatch);
begin
  SetProp(MapProp_C,Value);
end;

function TMap.GetCenter: TLatLng;
var
  Center: OleVariant;
begin
  Result:=nil;
  Center:=Exec(MapMethod_GetCenter);
  if VarIsValidDispatch(Center) then
  begin
    if assigned(FCenter)
      then FCenter.Disp:=Center
      else FCenter:=TLatLng(CreateObjectWrapper(Script,TLatLng,Center));
    Result:=FCenter;
  end;
end;

procedure TMap.SetCenter(LatLng: TLatLng);
begin
  Exec(MapMethod_SetCenter,LatLng);
end;

function TMap.GetControls: TNodeMVCArrayArray;
begin
  Result:=TNodeMVCArrayArray(GetPropAsObject(MapProp_Controls,TNodeMVCArrayArray,
                                             FControls));
end;

procedure TMap.SetControls(Value: TNodeMVCArrayArray);
begin
  SetProp(MapProp_Controls,Value);
end;

function TMap.GetD: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_D);
end;

procedure TMap.SetD(Value: IDispatch);
begin
  SetProp(MapProp_D,Value);
end;

procedure TMap.FitBounds(Bounds: TLatLngBounds);
begin
  if assigned(Bounds)
    then Exec(MapMethod_FitBounds,[Bounds.Disp]);
end;

function TMap.GetG: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_G);
end;

procedure TMap.SetG(Value: IDispatch);
begin
  SetProp(MapProp_G,Value);
end;

function TMap.GetBounds: TLatLngBounds;
var
  Bounds: OleVariant;
begin
  Result:=nil;
  Bounds:=Exec(MapMethod_GetBounds);
  if VarIsValidDispatch(Bounds) then
  begin
    if assigned(FBounds)
      then FBounds.Disp:=Bounds
      else FBounds:=TLatLngBounds(CreateObjectWrapper(Script,TLatLngBounds,Bounds));
    Result:=FBounds;
  end;
end;

function TMap.GetDiv: TNode;
begin
  Result:=TNode(GetObjectWithExec(MapMethod_GetDiv,TNode,FContainer));
end;

function TMap.GetProjection: TProjection;
var
  Projection: OleVariant;
begin
  Result:=nil;
  Projection:=Exec(MapMethod_GetProjection);
  if VarIsValidDispatch(Projection) then
  begin
    if assigned(FProjection)
      then FProjection.Disp:=Projection
      else FProjection:=TProjection(CreateObjectWrapper(Script,TProjection,Projection));
    Result:=FProjection;
  end;
end;
{ ab Version 3.3.1a nicht mehr vorhanden
function TMap.GetRotatable: OleVariant;
begin
  Result:=Exec(MapMethod_GetRotatable);
end;
}
function TMap.GetGmAccessors: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_GmAccessors);
end;

procedure TMap.SetGmAccessors(Value: IDispatch);
begin
  SetProp(MapProp_GmAccessors,Value);
end;

function TMap.GetGmBindings: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_GmBindings);
end;

procedure TMap.SetGmBindings(Value: IDispatch);
begin
  SetProp(MapProp_GmBindings,Value);
end;

function TMap.GetHeading: Integer;
begin
  Result:=Exec(MapMethod_GetHeading);
end;

procedure TMap.SetHeading(Heading: Integer);
begin
  Exec(MapMethod_SetHeading,[Heading]);
end;

function TMap.GetHeight: Integer;
begin
  Result:=GetDiv.Height;
end;

procedure TMap.SetHeight(Value: Integer);
begin
  if GetDiv.Height<>Value then
  begin
    GetDiv.Height:=Value;
    Resized;
  end;
end;

function TMap.GetMapTypeControl: Boolean;
begin
  Result:=VarAsBoolean(Get(MapKey_MapTypeControl));
end;

procedure TMap.SetMapTypeControl(Value: Boolean);
begin
  {$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}(MapKey_MapTypeControl,Value);
end;

function TMap.GetI: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_I);
end;

procedure TMap.SetI(Value: IDispatch);
begin
  SetProp(MapProp_I,Value);
end;

function TMap.GetLeft: Integer;
begin
  Result:=GetDiv.Left;
end;

procedure TMap.SetLeft(Value: Integer);
begin
  GetDiv.Left:=Value;
end;

function TMap.GetMapTypeID: String;
begin
  Result:=Exec(MapMethod_GetMapTypeID);
end;

procedure TMap.SetMapTypeID(MapTypeID: String);
begin
  Exec(MapMethod_SetMapTypeID,[MapTypeID]);
end;

function TMap.GetNavigationControl: Boolean;
begin
  Result:=VarAsBoolean(Get(MapKey_NavigationControl));
end;

procedure TMap.SetNavigationControl(Value: Boolean);
begin
  {$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}(MapKey_NavigationControl,Value);
end;

function TMap.GetMapTypes: TMapTypeRegistry;
begin
  Result:=TMapTypeRegistry(GetPropAsObject(MapProp_MapTypes,TMapTypeRegistry,FMapTypeRegistry));
end;

procedure TMap.SetMapTypes(Value: TMapTypeRegistry);
begin
  SetProp(MapProp_MapTypes,Value);
end;

function TMap.GetOverlayMapTypes: TMVCArray;
begin
  Result:=TMVCArray(GetPropAsObject(MapProp_OverlayMapTypes,TMVCArray,FOverlayMapTypes));
end;

procedure TMap.SetOverlayMapTypes(Value: TMVCArray);
begin
  SetProp(MapProp_OverlayMapTypes,Value);
end;

function TMap.GetScaleControl: Boolean;
begin
  Result:=VarAsBoolean(Get(MapKey_ScaleControl));
end;

procedure TMap.SetScaleControl(Value: Boolean);
begin
  {$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}(MapKey_ScaleControl,Value);
end;

function TMap.GetStreetViewControl: Boolean;
begin
  Result:=VarAsBoolean(Get(MapKey_StreetViewControl));
end;

procedure TMap.SetStreetViewControl(Value: Boolean);
begin
  {$IFDEF AND_OP}&Set{$ELSE}Set_{$ENDIF}(MapKey_StreetViewControl,Value);
end;

function TMap.N: OleVariant;
begin
  Result:=Exec(MapMethod_N);
end;

function TMap.P: OleVariant;
begin
  Result:=Exec(MapMethod_P);
end;

procedure TMap.PanBy(x, y: Integer);
begin
  Exec(MapMethod_PanBy,[x,y]);
end;

procedure TMap.PanTo(LatLng: TLatLng);
begin
  if assigned(LatLng)
    then Exec(MapMethod_PanTo,[LatLng.Disp]);
end;

procedure TMap.PanToBounds(LatLngBounds: TLatLngBounds);
begin
  if assigned(LatLngBounds)
    then Exec(MapMethod_PanToBounds,[LatLngBounds.Disp]);
end;

procedure TMap.SetOptions(Options: TMapOptions);
begin
  if assigned(Options)
    then Exec(MapMethod_SetOptions,[Options.Disp]);
end;

function TMap.GetStreetView: TStreetViewPanorama;
var
  StreetView: OleVariant;
begin
  Result:=nil;
  StreetView:=Exec(MapMethod_GetStreetView);
  if VarIsValidDispatch(StreetView) then
  begin
    if assigned(FStreetViewPanorama)
      then FStreetViewPanorama.Disp:=StreetView
      else FStreetViewPanorama:=TStreetViewPanorama(CreateObjectWrapper(Script,
                                                                        TStreetViewPanorama,
                                                                        StreetView));
    Result:=FStreetViewPanorama;
  end;
end;

procedure TMap.SetStreetView(Panorama: TStreetViewPanorama);
begin
  Exec(MapMethod_SetStreetView,Panorama);
end;

function TMap.GetTa: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_Ta);
end;

procedure TMap.SetTa(Value: IDispatch);
begin
  SetProp(MapProp_Ta,Value);
end;

function TMap.GetTilt: Integer;
begin
  Result:=Exec(MapMethod_GetTilt);
end;

procedure TMap.SetTilt(Tilt: Integer);
begin
  Exec(MapMethod_SetTilt,[Tilt]);
end;

function TMap.GetTop: Integer;
begin
  Result:=GetDiv.Top;
end;

procedure TMap.SetTop(Value: Integer);
begin
  GetDiv.Top:=Value;
end;

function TMap.GetWidth: Integer;
begin
  Result:=GetDiv.Width;
end;

procedure TMap.SetWidth(Value: Integer);
begin
  GetDiv.Width:=Value;
end;

function TMap.GetXa: IDispatch;
begin
  Result:=GetPropAsDispatch(MapProp_Xa);
end;

procedure TMap.SetXa(Value: IDispatch);
begin
  SetProp(MapProp_Xa,Value);
end;

function TMap.GetZoom: Integer;
begin
  Result:=Exec(MapMethod_GetZoom);
end;

procedure TMap.SetZoom(Zoom: Integer);
begin
  Exec(MapMethod_SetZoom,[Zoom]);
end;

procedure TMap.BoundsChanged;
begin
  CallEvent(FOnBoundsChanged);
end;

procedure TMap.CenterChanged;
begin
  CallEvent(FOnCenterChanged);
end;

procedure TMap.Click;
begin
  CallEvent(FOnClick,FOnClickFunction.Arguments[0]);
end;

procedure TMap.DblClick;
begin
  CallEvent(FOnDblClick,FOnDblClickFunction.Arguments[0]);
end;

procedure TMap.Drag;
begin
  CallEvent(FOnDrag);
end;

procedure TMap.Dragend;
begin
  CallEvent(FOnDragend);
end;

procedure TMap.DragStart;
begin
  CallEvent(FOnDragStart);
end;

procedure TMap.HeadingChanged;
begin
  CallEvent(FOnHeadingChanged);
end;

procedure TMap.Idle;
begin
  CallEvent(FOnIdle);
end;

procedure TMap.MapTypeIDChanged;
begin
  CallEvent(FOnMapTypeIDChanged);
end;

procedure TMap.MouseMove;
begin
  CallEvent(FOnMouseMove,FOnMouseMoveFunction.Arguments[0]);
end;

procedure TMap.MouseOut;
begin
  CallEvent(FOnMouseOut,FOnMouseOutFunction.Arguments[0]);
end;

procedure TMap.MouseOver;
begin
  CallEvent(FOnMouseOver,FOnMouseOverFunction.Arguments[0]);
end;

procedure TMap.ProjectionChanged;
begin
  CallEvent(FOnProjectionChanged);
end;

procedure TMap.Resize;
begin
  CallEvent(FOnResize);
end;

procedure TMap.Resized;
begin
  TScript(Script).Google.Maps.Event.Trigger(Self,'resize',[]);
end;

procedure TMap.RightClick;
begin
  CallEvent(FOnRightClick,FOnRightClickFunction.Arguments[0]);
  if assigned(FPopupMenu) then
  begin
    //FPopupMenu.Popup(Round(Point.x),Round(Point.y));
    //-> fhrt zu Fehlern in der JScript-DLL!
    //Deshalb asynchrone Anzeige des PopupMens
    //(Entkopplung von der JScript-Funktion):
    ShowPopupMenu(FPopupMenu);                             //Anzeige an der aktuellen Cursorposition
  end;
end;

procedure TMap.TilesLoaded;
begin
  CallEvent(FOnTilesLoaded);
end;

procedure TMap.TiltChanged;
begin
  CallEvent(FOnTiltChanged);
end;

procedure TMap.ZoomChanged;
begin
  CallEvent(FOnZoomChanged);
end;

procedure TMap.SetOnBoundsChanged(Value: TNotifyEvent);
begin
  FOnBoundsChanged:=Value;
  RemoveListener(FOnBoundsChangedListener);
  if assigned(FOnBoundsChanged)
    then AddListener(MapEvent_BoundsChanged,FOnBoundsChangedListener,
                     FOnBoundsChangedFunction,BoundsChanged);
end;

procedure TMap.SetOnCenterChanged(Value: TNotifyEvent);
begin
  FOnCenterChanged:=Value;
  RemoveListener(FOnCenterChangedListener);
  if assigned(FOnCenterChanged)
    then AddListener(MapEvent_CenterChanged,FOnCenterChangedListener,
                     FOnCenterChangedFunction,CenterChanged);
end;

procedure TMap.SetOnClick(Value: TMapMouseEvent);
begin
  FOnClick:=Value;
  RemoveListener(FOnClickListener);
  if assigned(FOnClick)
    then AddListener(MapEvent_Click,FOnClickListener,FOnClickFunction,Click);
end;

procedure TMap.SetOnDblClick(Value: TMapMouseEvent);
begin
  FOnDblClick:=Value;
  RemoveListener(FOnDblClickListener);
  if assigned(FOnDblClick)
    then AddListener(MapEvent_DblClick,FOnDblClickListener,
                     FOnDblClickFunction,DblClick);
end;

procedure TMap.SetOnDrag(Value: TNotifyEvent);
begin
  FOnDrag:=Value;
  RemoveListener(FOnDragListener);
  if assigned(FOnDrag)
    then AddListener(MapEvent_Drag,FOnDragListener,FOnDragFunction,Drag);
end;

procedure TMap.SetOnDragEnd(Value: TNotifyEvent);
begin
  FOnDragEnd:=Value;
  RemoveListener(FOnDragEndListener);
  if assigned(FOnDragEnd)
    then AddListener(MapEvent_DragEnd,FOnDragEndListener,FOnDragEndFunction,DragEnd);
end;

procedure TMap.SetOnDragStart(Value: TNotifyEvent);
begin
  FOnDragStart:=Value;
  RemoveListener(FOnDragStartListener);
  if assigned(FOnDragStart)
    then AddListener(MapEvent_DragStart,FOnDragStartListener,
                     FOnDragStartFunction,DragStart);
end;

procedure TMap.SetOnHeadingChanged(Value: TNotifyEvent);
begin
  FOnHeadingChanged:=Value;
  RemoveListener(FOnHeadingChangedListener);
  if assigned(FOnHeadingChanged)
    then AddListener(MapEvent_HeadingChanged,FOnHeadingChangedListener,
                     FOnHeadingChangedFunction,HeadingChanged);
end;

procedure TMap.SetOnIdle(Value: TNotifyEvent);
begin
  FOnIdle:=Value;
  RemoveListener(FOnIdleListener);
  if assigned(FOnIdle)
    then AddListener(MapEvent_Idle,FOnIdleListener,FOnIdleFunction,Idle);
end;

procedure TMap.SetOnMapTypeIDChanged(Value: TNotifyEvent);
begin
  FOnMapTypeIDChanged:=Value;
  RemoveListener(FOnMapTypeIDChangedListener);
  if assigned(FOnMapTypeIDChanged)
    then AddListener(MapEvent_MapTypeIDChanged,FOnMapTypeIDChangedListener,
                     FOnMapTypeIDChangedFunction,MapTypeIDChanged);
end;

procedure TMap.SetOnMouseMove(Value: TMapMouseEvent);
begin
  FOnMouseMove:=Value;
  RemoveListener(FOnMouseMoveListener);
  if assigned(FOnMouseMove)
    then AddListener(MapEvent_MouseMove,FOnMouseMoveListener,
                     FOnMouseMoveFunction,MouseMove);
end;

procedure TMap.SetOnMouseOut(Value: TMapMouseEvent);
begin
  FOnMouseOut:=Value;
  RemoveListener(FOnMouseOutListener);
  if assigned(FOnMouseOut)
    then AddListener(MapEvent_MouseOut,FOnMouseOutListener,
                     FOnMouseOutFunction,MouseOut);
end;

procedure TMap.SetOnMouseOver(Value: TMapMouseEvent);
begin
  FOnMouseOver:=Value;
  RemoveListener(FOnMouseOverListener);
  if assigned(FOnMouseOver)
    then AddListener(MapEvent_MouseOver,FOnMouseOverListener,
                     FOnMouseOverFunction,MouseOver);
end;

procedure TMap.SetOnProjectionChanged(Value: TNotifyEvent);
begin
  FOnProjectionChanged:=Value;
  RemoveListener(FOnProjectionChangedListener);
  if assigned(FOnProjectionChanged)
    then AddListener(MapEvent_ProjectionChanged,FOnProjectionChangedListener,
                     FOnProjectionChangedFunction,ProjectionChanged);
end;

procedure TMap.SetOnResize(Value: TNotifyEvent);
begin
  FOnResize:=Value;
  RemoveListener(FOnResizeListener);
  if assigned(FOnResize)
    then AddListener(MapEvent_Resize,FOnResizeListener,FOnResizeFunction,Resize);
end;

procedure TMap.SetOnRightClick(Value: TMapMouseEvent);
begin
  FOnRightClick:=Value;
  if assigned(FOnRightClick) and not assigned(FOnRightClickListener)
    then AddListener(MapEvent_RightClick,FOnRightClickListener,
                     FOnRightClickFunction,RightClick);    //neuen Listener erstellen
  if not assigned(FPopupMenu) and not assigned(FOnRightClick)
    then RemoveListener(FOnRightClickListener);            //Listener wird nicht mehr bentigt -> enfernen
end;

procedure TMap.SetOnTilesLoaded(Value: TNotifyEvent);
begin
  FOnTilesLoaded:=Value;
  RemoveListener(FOnTilesLoadedListener);
  if assigned(FOnTilesLoaded)
    then AddListener(MapEvent_TilesLoaded,FOnTilesLoadedListener,
                     FOnTilesLoadedFunction,TilesLoaded);
end;

procedure TMap.SetOnTiltChanged(Value: TNotifyEvent);
begin
  FOnTiltChanged:=Value;
  RemoveListener(FOnTiltChangedListener);
  if assigned(FOnTiltChanged)
    then AddListener(MapEvent_TiltChanged,FOnTiltChangedListener,
                     FOnTiltChangedFunction,TiltChanged);
end;

procedure TMap.SetOnZoomChanged(Value: TNotifyEvent);
begin
  FOnZoomChanged:=Value;
  RemoveListener(FOnZoomChangedListener);
  if assigned(FOnZoomChanged)
    then AddListener(MapEvent_ZoomChanged,FOnZoomChangedListener,
                     FOnZoomChangedFunction,ZoomChanged);
end;

procedure TMap.SetPopupMenu(Value: TPopupMenu);
begin
  if FPopupMenu<>Value then
  begin
    FPopupMenu:=Value;
    if assigned(FPopupMenu) and not assigned(FOnRightClickListener)
      then AddListener(MapEvent_RightClick,FOnRightClickListener,
                       FOnRightClickFunction,RightClick);  //neuen Listener erstellen
    if not assigned(FPopupMenu) and not assigned(FOnRightClick)
      then RemoveListener(FOnRightClickListener);          //Listener wird nicht mehr bentigt -> enfernen
  end;
end;

//------------------------------------------------------------------------------
//TMapOptions

constructor TMapOptions.Create;
begin
  inherited;
end;

constructor TMapOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

constructor TMapOptions.Create(LatLng: TLatLng; MapTypeID: String; Zoom: Integer);
var
  CenterX, CenterY: Double;
begin
  CenterX:=0;
  CenterY:=0;
  if assigned(LatLng) then
  begin
    CenterX:=LatLng.Lat;
    CenterY:=LatLng.Lng;
  end;
  Create(JScriptObjects.Script,CenterX,CenterY,MapTypeID,Zoom);
end;

constructor TMapOptions.Create(CenterX, CenterY: Double; MapTypeID: String; Zoom: Integer);
begin
  Create(JScriptObjects.Script,CenterX,CenterY,MapTypeID,Zoom);
end;

constructor TMapOptions.Create(Script: TCustomScript; CenterX, CenterY: Double; MapTypeID: String; Zoom: Integer);
var
  LatLng: TLatLng;
begin
  inherited Create(Script);                                          //leeres Objekt anlegen
  if Script is TScript then
  begin
    //notwendige Werte speichern:
    LatLng:=TScript(Script).Google.Maps.LatLng(CenterX,CenterY,true);//neues LatLng-Objekt erstellen
    try
      SetCenter(LatLng);                                             //Kartenzentrum speichern
    finally
      LatLng.Free;
    end;
  end;
  SetMapTypeID(MapTypeID);
  SetZoom(Zoom);
end;

destructor TMapOptions.Destroy;
begin
  FCenter.Free;
  FMapTypeControlOptions.Free;
  FNavigationControlOptions.Free;
  FOverviewMapControlOptions.Free;
  FPanControlOptions.Free;
  FRotateControlOptions.Free;
  FScaleControlOptions.Free;
  FStreetView.Free;
  FStreetViewControlOptions.Free;
  FStyles.Free;
  FZoomControlOptions.Free;
  inherited;
end;

procedure TMapOptions.AfterConstruction;
begin
  inherited;
  FWidth:=200;
  FHeight:=150;
  FAlign:=alClient;
end;

class function TMapOptions.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MapOptionsProp_BackgroundColor,
                                 MapOptionsProp_Center,
                                 MapOptionsProp_DisableDefaultUI]);
end;

function TMapOptions.GetBackgroundColor: TColor;
begin
  Result:=WebColorStrToColor(GetPropAsString(MapOptionsProp_BackgroundColor,ColorToWebColorStr(clNone)));
end;

procedure TMapOptions.SetBackgroundColor(Value: TColor);
begin
  SetProp(MapOptionsProp_BackgroundColor,ColorToWebColorStr(Value),true);
end;

function TMapOptions.GetCenter: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(MapOptionsProp_Center,TLatLng,FCenter,true));
end;

procedure TMapOptions.SetCenter(Value: TLatLng);
begin
  SetProp(MapOptionsProp_Center,Value,true);
end;

function TMapOptions.GetDisableDefaultUI: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_DisableDefaultUI);
end;

procedure TMapOptions.SetDisableDefaultUI(Value: Boolean);
begin
  SetProp(MapOptionsProp_DisableDefaultUI,Value,true);
end;

function TMapOptions.GetDisableDoubleClickZoom: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_DisableDoubleClickZoom,true);
end;

procedure TMapOptions.SetDisableDoubleClickZoom(Value: Boolean);
begin
  SetProp(MapOptionsProp_DisableDoubleClickZoom,Value,true);
end;

function TMapOptions.GetDraggable: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_Draggable,true);
end;

procedure TMapOptions.SetDraggable(Value: Boolean);
begin
  SetProp(MapOptionsProp_Draggable,Value,true);
end;

function TMapOptions.GetDraggableCursor: String;
begin
  Result:=GetPropAsString(MapOptionsProp_DraggableCursor);
end;

procedure TMapOptions.SetDraggableCursor(Value: String);
begin
  SetProp(MapOptionsProp_DraggableCursor,Value,false);
end;

function TMapOptions.GetDraggingCursor: String;
begin
  Result:=GetPropAsString(MapOptionsProp_DraggingCursor);
end;

procedure TMapOptions.SetDraggingCursor(Value: String);
begin
  SetProp(MapOptionsProp_DraggingCursor,Value,true);
end;

function TMapOptions.GetHeading: Integer;
begin
  Result:=GetPropAsInteger(MapOptionsProp_Heading);
end;

procedure TMapOptions.SetHeading(Value: Integer);
begin
  SetProp(MapOptionsProp_Heading,Value,true);
end;

function TMapOptions.GetKeyboardShortcuts: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_KeyboardShortcuts,true);
end;

procedure TMapOptions.SetKeyboardShortcuts(Value: Boolean);
begin
  SetProp(MapOptionsProp_KeyboardShortcuts,Value,true);
end;

function TMapOptions.GetMapTypeControl: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_MapTypeControl,true);
end;

procedure TMapOptions.SetMapTypeControl(Value: Boolean);
begin
  SetProp(MapOptionsProp_MapTypeControl,Value,true);
end;

function TMapOptions.GetMapTypeControlOptions: TMapTypeControlOptions;
begin
  Result:=TMapTypeControlOptions(GetPropAsObject(MapOptionsProp_MapTypeControlOptions,
                                                 TMapTypeControlOptions,
                                                 FMaptypeControlOptions,true));
end;

procedure TMapOptions.SetMapTypeControlOptions(Value: TMapTypeControlOptions);
begin
  SetProp(MapOptionsProp_MapTypeControlOptions,Value,true);
end;

function TMapOptions.GetMapTypeID: String;
begin
  Result:=GetPropAsString(MapOptionsProp_MapTypeId);
end;

procedure TMapOptions.SetMapTypeID(Value: String);
begin
  SetProp(MapOptionsProp_MapTypeId,Value,true);
end;

function TMapOptions.GetMaxZoom: Integer;
begin
  Result:=GetPropAsInteger(MapOptionsProp_MaxZoom);
end;

procedure TMapOptions.SetMaxZoom(Value: Integer);
begin
  SetProp(MapOptionsProp_MaxZoom,Value,true);
end;

function TMapOptions.GetMinZoom: Integer;
begin
  Result:=GetPropAsInteger(MapOptionsProp_MinZoom);
end;

procedure TMapOptions.SetMinZoom(Value: Integer);
begin
  SetProp(MapOptionsProp_MinZoom,Value,true);
end;

function TMapOptions.GetNavigationControl: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_NavigationControl,true);
end;

procedure TMapOptions.SetNavigationControl(Value: Boolean);
begin
  SetProp(MapOptionsProp_NavigationControl,Value,true);
end;

function TMapOptions.GetNavigationControlOptions: TNavigationControlOptions;
begin
  Result:=TNavigationControlOptions(GetPropAsObject(MapOptionsProp_NavigationControlOptions,
                                                    TNavigationControlOptions,
                                                    FNavigationControlOptions,true));
end;

procedure TMapOptions.SetNavigationControlOptions(Value: TNavigationControlOptions);
begin
  SetProp(MapOptionsProp_NavigationControlOptions,Value,true);
end;

function TMapOptions.GetNoClear: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_NoClear);
end;

procedure TMapOptions.SetNoClear(Value: Boolean);
begin
  SetProp(MapOptionsProp_NoClear,Value,true);
end;

function TMapOptions.GetOverviewMapControl: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_OverviewMapControl);
end;

procedure TMapOptions.SetOverviewMapControl(Value: Boolean);
begin
  SetProp(MapOptionsProp_OverviewMapControl,Value,true);
end;

function TMapOptions.GetOverviewMapControlOptions: TOverviewMapControlOptions;
begin
  Result:=TOverviewMapControlOptions(GetPropAsObject(MapOptionsProp_OverviewMapControlOptions,
                                                     TOverviewMapControlOptions,
                                                     FOverviewMapControlOptions,true));
end;

procedure TMapOptions.SetOverviewMapControlOptions(Value: TOverviewMapControlOptions);
begin
  SetProp(MapOptionsProp_OverviewMapControlOptions,Value,true);
end;

function TMapOptions.GetPanControl: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_PanControl,true);
end;

procedure TMapOptions.SetPanControl(Value: Boolean);
begin
  SetProp(MapOptionsProp_PanControl,Value,true);
 end;

function TMapOptions.GetPanControlOptions: TPanControlOptions;
begin
  Result:=TPanControlOptions(GetPropAsObject(MapOptionsProp_PanControlOptions,
                                             TPanControlOptions,
                                             FPanControlOptions,true));
end;

procedure TMapOptions.SetPanControlOptions(Value: TPanControlOptions);
begin
  SetProp(MapOptionsProp_PanControlOptions,Value,true);
end;

function TMapOptions.GetRotateControl: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_RotateControl);
end;

procedure TMapOptions.SetRotateControl(Value: Boolean);
begin
  SetProp(MapOptionsProp_RotateControl,Value,true);
end;

function TMapOptions.GetRotateControlOptions: TRotateControlOptions;
begin
  Result:=TRotateControlOptions(GetPropAsObject(MapOptionsProp_RotateControlOptions,
                                               TRotateControlOptions,
                                               FRotateControlOptions,true));
end;

procedure TMapOptions.SetRotateControlOptions(Value: TRotateControlOptions);
begin
  SetProp(MapOptionsProp_RotateControlOptions,Value,true);
end;

function TMapOptions.GetScaleControl: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_ScaleControl);
end;

procedure TMapOptions.SetScaleControl(Value: Boolean);
begin
  SetProp(MapOptionsProp_ScaleControl,Value,true);
end;

function TMapOptions.GetScaleControlOptions: TScaleControlOptions;
begin
  Result:=TScaleControlOptions(GetPropAsObject(MapOptionsProp_ScaleControlOptions,
                                               TScaleControlOptions,
                                               FScaleControlOptions,true));
end;

procedure TMapOptions.SetScaleControlOptions(Value: TScaleControlOptions);
begin
  SetProp(MapOptionsProp_ScaleControlOptions,Value,true);
end;

function TMapOptions.GetScrollWheel: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_ScrollWheel,true);
end;

procedure TMapOptions.SetScrollWheel(Value: Boolean);
begin
  SetProp(MapOptionsProp_ScrollWheel,Value,true);
end;

function TMapOptions.GetStreetView: TStreetViewPanorama;
begin
  Result:=TStreetViewPanorama(GetPropAsObject(MapOptionsProp_StreetView,
                              TStreetViewPanorama,FStreetView,true));
end;

procedure TMapOptions.SetStreetView(Value: TStreetViewPanorama);
begin
  SetProp(MapOptionsProp_StreetView,Value,true);
end;

function TMapOptions.GetStreetViewControl: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_StreetViewControl,true);
end;

procedure TMapOptions.SetStreetViewControl(Value: Boolean);
begin
  SetProp(MapOptionsProp_StreetViewControl,Value,true);
end;

function TMapOptions.GetStreetViewControlOptions: TStreetViewControlOptions;
begin
  Result:=TStreetViewControlOptions(GetPropAsObject(MapOptionsProp_StreetViewControlOptions,
                                                    TStreetViewControlOptions,
                                                    FStreetViewControlOptions,true));
end;

procedure TMapOptions.SetStreetViewControlOptions(Value: TStreetViewControlOptions);
begin
  SetProp(MapOptionsProp_StreetViewControlOptions,Value,true);
end;

function TMapOptions.GetStyles: TMapTypeStyleArray;
begin
  Result:=TMapTypeStyleArray(GetPropAsObject(MapOptionsProp_Styles,
                                             TMapTypeStyleArray,FStyles,true));
end;

procedure TMapOptions.SetStyles(Value: TMapTypeStyleArray);
begin
  SetProp(MapOptionsProp_Styles,Value,true);
end;

function TMapOptions.GetTilt: Integer;
begin
  Result:=GetPropAsInteger(MapOptionsProp_Tilt);
end;

procedure TMapOptions.SetTilt(Value: Integer);
begin
  SetProp(MapOptionsProp_Tilt,Value,true);
end;

function TMapOptions.GetZoom: Integer;
begin
  Result:=GetPropAsInteger(MapOptionsProp_Zoom,1);
end;

procedure TMapOptions.SetZoom(Value: Integer);
begin
  SetProp(MapOptionsProp_Zoom,Value,true);
end;

function TMapOptions.GetZoomControl: Boolean;
begin
  Result:=GetPropAsBoolean(MapOptionsProp_ZoomControl,true);
end;

procedure TMapOptions.SetZoomControl(Value: Boolean);
begin
  SetProp(MapOptionsProp_ZoomControl,Value,true);
end;

function TMapOptions.GetZoomControlOptions: TZoomControlOptions;
begin
  Result:=TZoomControlOptions(GetPropAsObject(MapOptionsProp_ZoomControlOptions,
                                              TZoomControlOptions,
                                              FZoomControlOptions,true));
end;

procedure TMapOptions.SetZoomControlOptions(Value: TZoomControlOptions);
begin
  SetProp(MapOptionsProp_ZoomControlOptions,Value,true);
end;

//------------------------------------------------------------------------------
//TMapTypeID

class function TMapTypeID.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MapTypeIDProp_Hybrid,
                                 MapTypeIDProp_Roadmap,
                                 MapTypeIDProp_Satellite]);
end;

function TMapTypeID.GetDi: String;
begin
  Result:=GetPropAsString(MapTypeIDProp_Di);
end;

procedure TMapTypeID.SetDi(Value: String);
begin
  SetProp(MapTypeIDProp_Di,Value);
end;

function TMapTypeID.GetHybrid: String;
begin
  Result:=GetPropAsString(MapTypeIDProp_Hybrid);
end;

procedure TMapTypeID.SetHybrid(Value: String);
begin
  SetProp(MapTypeIDProp_Hybrid,Value);
end;

function TMapTypeID.GetRoadmap: String;
begin
  Result:=GetPropAsString(MapTypeIDProp_Roadmap);
end;

procedure TMapTypeID.SetRoadmap(Value: String);
begin
  SetProp(MapTypeIDProp_Roadmap,Value);
end;

function TMapTypeID.GetSatellite: String;
begin
  Result:=GetPropAsString(MapTypeIDProp_Satellite);
end;

procedure TMapTypeID.SetSatellite(Value: String);
begin
  SetProp(MapTypeIDProp_Satellite,Value);
end;

function TMapTypeID.GetTerrain: String;
begin
  Result:=GetPropAsString(MapTypeIDProp_Terrain);
end;

procedure TMapTypeID.SetTerrain(Value: String);
begin
  SetProp(MapTypeIDProp_Terrain,Value);
end;

end.

