unit gmOverlaysBase;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils, {$IFDEF DEBUG}Dialogs,{$ENDIF}
  {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Graphics, BrowserTools, HTMLObjects,
  JScriptObjects, DispObject, gmBase, gmMVC, gmMap, gmStreetView, gmEvents,
  gmMapTypes, gmAdditionalConstants, gmAdditionalMessages;

type
  TAnimation = class;
  TOverlayView = class;
  TRectangle = class;
  TRectangleOptions = class;
  TCircle = class;
  TCircleOptions = class;
  TGroundOverlay = class;
  TGroundOverlayOptions = class;

  //Animations that can be played on a marker. Use the setAnimation method on
  //Marker or the animation option to play an animation.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#Animation
  TAnimation = class(TCustomJScriptObject)
  private
    function GetBounce: Integer;
    function GetDrop: Integer;
    function GetWj: Integer;
    function GetXj: Integer;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Marker bounces until animation is stopped.
    property Bounce: Integer read GetBounce;
    //Marker falls from the top of the map ending with a small bounce.
    property Drop: Integer read GetDrop;
    property Wj: Integer read GetWj;
    property Xj: Integer read GetXj;
  end;

  TCircle = class(TMVCEventObject)
  private
    FBounds: TLatLngBounds;
    FCenter: TLatLng;
    FMap: TMap;
  protected
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; Opts: TCircleOptions): IDispatch; reintroduce; overload;
  public
    constructor Create(Opts: TCircleOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TCircleOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    procedure SetCenter(Center: TLatLng);
    procedure SetMap(Map: TMap);
    procedure SetOptions(Options: TCircleOptions); reintroduce;
    procedure SetRadius(Radius: Double);
    function GetBounds: TLatLngBounds;
    function GetCenter: TLatLng;
    function GetMap: TMap;
    function GetRadius: Double;
    property Center: TLatLng read GetCenter write SetCenter;
    property Bounds: TLatLngBounds read GetBounds;
    property Radius: Double read GetRadius write SetRadius;
    property Map: TMap read GetMap write SetMap;
  end;

  TCircleOptions = class(TCustomJScriptObject)
  private
    FCenter: TLatLng;
    FMap: TMap;
    procedure SetCenter(Value: TLatLng);
    procedure SetClickable(Value: Boolean);
    procedure SetFillColor(Value: TColor);
    procedure SetFillOpacity(Value: Double);
    procedure SetMap(Value: TMap);
    procedure SetRadius(Value: Double);
    procedure SetStrokeColor(Value: TColor);
    procedure SetStrokeOpacity(Value: Double);
    procedure SetStrokeWeight(Value: Integer);
    procedure SetZIndex(Value: Integer);
    function GetCenter: TLatLng;
    function GetClickable: Boolean;
    function GetFillColor: TColor;
    function GetFillOpacity: Double;
    function GetMap: TMap;
    function GetRadius: Double;
    function GetStrokeColor: TColor;
    function GetStrokeOpacity: Double;
    function GetStrokeWeight: Integer;
    function GetZIndex: Integer;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    property Center: TLatLng read GetCenter write SetCenter;
    property Clickable: Boolean read GetClickable write SetClickable;
    property FillColor: TColor read GetFillColor write SetFillColor;
    property FillOpacity: Double read GetFillOpacity write SetFillOpacity;
    property Map: TMap read GetMap write SetMap;
    property Radius: Double read GetRadius write SetRadius;
    property StrokeColor: TColor read GetStrokeColor write SetStrokeColor;
    property StrokeOpacity: Double read GetStrokeOpacity write SetStrokeOpacity;
    property StrokeWeight: Integer read GetStrokeWeight write SetStrokeWeight;
    property ZIndex: Integer read GetZIndex write SetZIndex;
  end;

  TGroundOverlay = class(TMVCObject)
  private
    FBounds: TLatLngBounds;
    FMap: TMap;
    FOnClick: TMapMouseEvent;
    FOnClickFunction: TJScriptFunction;
    FOnClickListener: TMapsEventListener;
    procedure SetOnClick(Value: TMapMouseEvent);
  protected
    class function New(Script: TCustomScript; const URL: String; Bounds: TLatLngBounds): IDispatch; reintroduce; overload;
    class function New(Script: TCustomScript; const URL: String; Bounds: TLatLngBounds; Opts: TGroundOverlayOptions): IDispatch; reintroduce; overload;
    //von JScript aufgerufene Event-Handler:
    procedure Click;
  public
    constructor Create(const URL: String; Bounds: TLatLngBounds; Opts: TGroundOverlayOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; const URL: String; Bounds: TLatLngBounds; Opts: TGroundOverlayOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    //Gibt das Boden-Overlay auf der angegebenen Karte wieder. Falls fr die
    //Karte der Wert "null" festgelegt wird, wird das Overlay entfernt:
    procedure SetMap(Map: TMap);
    //Ruft die LatLngBounds dieses Overlays ab:
    function GetBounds: TLatLngBounds;
    //Map Gibt die Karte zurck, auf der dieses Boden-Overlay angezeigt wird:
    function GetMap: TMap;
    //Ruft die URL des projizierten Bildes ab:
    function GetUrl: String;
    //Dieses Ereignis tritt ein, wenn das DOM-click-Ereignis beim GroundOverlay
    //ausgelst wird:
    property OnClick: TMapMouseEvent read FOnClick write SetOnClick;
  end;

  TGroundOverlayOptions = class(TCustomJScriptObject)
  private
    FMap: TMap;
    procedure SetClickable(Value: Boolean);
    procedure SetMap(Value: TMap);
    function GetClickable: Boolean;
    function GetMap: TMap;
  public
    destructor Destroy; override;
    //Falls true, kann das Boden-Overlay click-Ereignisse empfangen:
    property Clickable: Boolean read GetClickable write SetClickable;
    //Die Karte, auf der das Overlay angezeigt wird:
    property Map: TMap read GetMap write SetMap;
  end;

  TMapCanvasProjection = class(TCustomJScriptObject)
  private
    FLatLng1, FLatLng2: TLatLng;
    FPoint1, FPoint2: TPoint;
  public
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Berechnet die geografischen Koordinaten aus den Pixelkoordinaten im Container der Karte:
    function FromContainerPixelToLatLng(Pixel: TPoint): TLatLng;
    //Berechnet die geografischen Koordinaten aus den Pixelkoordinaten im div-Element,
    //das die verschiebbare Karte enthlt:
    function FromDivPixelToLatLng(Pixel: TPoint): TLatLng;
    //Berechnet die Pixelkoordinaten des angegebenen geografischen Standorts im
    //DOM-Element des ueren Containers der Karte:
    function FromLatLngToContainerPixel(LatLng: TLatLng): TPoint;
    //Berechnet die Pixelkoordinaten des angegebenen geografischen Standorts im
    //DOM-Element, das die verschiebbare Karte enthlt:
    function FromLatLngToDivPixel(LatLng: TLatLng): TPoint;
    //Berechnet die Breite der Welt in Pixeln bei der aktuellen Zoom-Einstellung:
    function GetWorldWidth: Integer;
  end;

  TMapPanes = class(TCustomJScriptObject)
  private
    FFloatPane: TNode;
    FFloatShadow: TNode;
    FMapPane: TNode;
    FOverlayImage: TNode;
    FOverlayLayer: TNode;
    FOverlayMouseTarget: TNode;
    FOverlayShadow: TNode;
    procedure SetFloatPane(Value: TNode);
    procedure SetFloatShadow(Value: TNode);
    procedure SetMapPane(Value: TNode);
    procedure SetOverlayImage(Value: TNode);
    procedure SetOverlayLayer(Value: TNode);
    procedure SetOverlayMouseTarget(Value: TNode);
    procedure SetOverlayShadow(Value: TNode);
    function GetFloatPane: TNode;
    function GetFloatShadow: TNode;
    function GetMapPane: TNode;
    function GetOverlayImage: TNode;
    function GetOverlayLayer: TNode;
    function GetOverlayMouseTarget: TNode;
    function GetOverlayShadow: TNode;
  public
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Dieses Fenster enthlt das Infofenster. Es befindet sich ber allen
    //Karten-Overlays. (Fenster 6):
    property FloatPane: TNode read GetFloatPane write SetFloatPane;
    //Dieses Fenster enthlt den Schatten des Infofensters. Es befindet sich
    //oberhalb von overlayImage, sodass sich Markierungen im Schatten des
    //Infofensters befinden knnen. (Fenster 4):
    property FloatShadow: TNode read GetFloatShadow write SetFloatShadow;
    //Dies ist das unterste Fenster; es befindet sich direkt ber den Kacheln.
    //(Fenster 0):
    property MapPane: TNode read GetMapPane write SetMapPane;
    //Dieses Fenster enthlt die Bilder fr den Vordergrund der Markierung.
    //(Fenster 3):
    property OverlayImage: TNode read GetOverlayImage write SetOverlayImage;
    //Dieses Fenster enthlt Polylinien, Polygone, Boden-Overlays und
    //Kachelebenen-Overlays. (Fenster 1):
    property OverlayLayer: TNode read GetOverlayLayer write SetOverlayLayer;
    //Dieses Fenster enthlt transparente Elemente, die DOM-Mausereignisse fr
    //die Markierungen empfangen. Es befindet sich oberhalb des floatShadow,
    //sodass auf Markierungen im Schatten des Infofensters geklickt werden kann.
    //(Fenster 5):
    property OverlayMouseTarget: TNode read GetOverlayMouseTarget write SetOverlayMouseTarget;
    //Dieses Fenster enthlt die Markierungsschatten. (Fenster 2):
    property OverlayShadow: TNode read GetOverlayShadow write SetOverlayShadow;
  end;
(*
  TMapTypeStyles = class(TObjectArray)
  private
    procedure SetItem(Index: Integer; Value: TMapTypeStyle);
    function GetItem(Index: Integer): TMapTypeStyle;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    constructor Create; reintroduce; overload;
    constructor Create(Script: TCustomScript); reintroduce; overload;
    function AddStyle: TMapTypeStyle; reintroduce;
    function Insert(Index: Integer): TMapTypeStyle; reintroduce;
    property Items[Index: Integer]: TMapTypeStyle read GetItem write SetItem; default;
  end;

  TMapTypeStylesCollection = class(TObject)
  private
    FItems: TStringList;
    FScript: TCustomScript;
    function GetStyles(Name: String): TMapTypeStyles;
  public
    constructor Create; overload;
    constructor Create(Script: TCustomScript); overload;
    destructor Destroy; override;
    function AddStyles(Name: String): TMapTypeStyles;
    property Styles[Name: String]: TMapTypeStyles read GetStyles; default;
    property Items: TStringList read FItems;
  end;

  TMapTypeStyleElementType = class(TObject)
  public
    //Wendet die Regel auf alle Elemente der angegebenen Funktion an:
    function All: String;
    //Wendet die Regel auf die Geometrie der Funktion an:
    function Geometry: String;
    //Wendet die Regel auf die Labels der Funktion an:
    function Labels: String;
  end;

  TMapTypeStyleFeatureType = class(TObject)
  public
    //Wendet die Regel auf Verwaltungsgebiete an:
    function Administrative: String;
    //Wendet die Regel auf Lnder an:
    function Administrative_Country: String;
    //Wendet die Regel auf Landgrundstcke an:
    function Administrative_Land_Parcel: String;
    //Wendet die Regel auf Orte an:
    function Administrative_Locality: String;
    //Wendet die Regel auf Nahbereiche/Stadtviertel an:
    function Administrative_Neighborhood: String;
    //Wendet die Regel auf Bundeslnder/Provinzen an:
    function Administrative_Province: String;
    //Wendet die Regel auf alle Selektortypen an:
    function All: String;
    //Wendet die Regel auf Landschaften an:
    function Landscape: String;
    //Wendet die Regel auf von Menschen geschaffene Strukturen an:
    function Landscape_Man_Made: String;
    //Wendet die Regel auf natrliche Elemente an:
    function Landscape_Natural: String;
    //Wendet die Regel auf interessante Orte an:
    function Poi: String;
    //Wendet die Regel auf Touristenattraktionen an:
    function Poi_Attraction: String;
    //Wendet die Regel auf Unternehmen an:
    function Poi_Business: String;
    //Wendet die Regel auf Regierungsgebude an:
    function Poi_Government: String;
    //Wendet die Regel auf Notdienste an (Krankenhuser, Apotheken, Polizei, rzte usw.):
    function Poi_Medical: String;
    //Wendet die Regel auf Parks an:
    function Poi_Park: String;
    //Wendet die Regel auf religise Orte an, wie Kirchen, Tempel oder Moscheen:
    function Poi_Place_Of_Worship: String;
    //Wendet die Regel auf Schulen und andere Bildungseinrichtungen an:
    function Poi_School: String;
    //Wendet die Regel auf Sportanlagen an:
    function Poi_Sports_Complex: String;
    //Wendet die Regel auf alle Straen an:
    function Road: String;
    //Wendet die Regel auf Straen mit hohem Verkehrsaufkommen an:
    function Road_Arterial: String;
    //Wendet die Regel auf Autobahnen an:
    function Road_Highway: String;
    //Wendet die Regel auf rtliche Straen an:
    function Road_Local: String;
    //Wendet die Regel auf alle Stationen und Linien der ffentlichen Verkehrsmittel an:
    function Transit: String;
    //Wendet die Regel auf Linien der ffentlichen Verkehrsmittel an:
    function Transit_Line: String;
    //Wendet die Regel auf alle Stationen der ffentlichen Verkehrsmittel an:
    function Transit_Station: String;
    //Wendet die Regel auf Flughfen an:
    function Transit_Station_Airport: String;
    //Wendet die Regel auf Bushaltestellen an:
    function Transit_Station_Bus: String;
    //Wendet die Regel auf Bahnhfe an:
    function Transit_Station_Rail: String;
    //Wendet die Regel auf Gewsser an:
    function Water: String;
  end;
*)

  //Von dieser Klasse sollte die Vererbung ausgehen; legen Sie dazu fr den
  //Prototyp Ihres Overlays new OverlayView.prototype fest. Sie mssen drei
  //Methoden implementieren: onAdd(), draw(), and onRemove(). Mit der
  //add()-Methode erstellen Sie DOM-Objekte und Sie als untergeordnete Objekte
  //an die Fenster anhngen. Mit der draw()-Methode positionieren Sie diese
  //Elemente. Mit der onRemove()-Methode entfernen Sie diese Objekte aus dem DOM.
  //Rufen Sie setMap() mit einem gltigen Kartenobjekt auf, um das Aufrufen der
  //onAdd()-Methode auszulsen. Rufen Sie auerdem setMap(null) auf, um die
  //onRemove()-Methode auszulsen. Die setMap()-Methode kann bei der Konstruktion
  //oder jederzeit danach aufgerufen werden, wenn der Overlay nach dem Entfernen
  //wieder angezeigt werden soll. Die draw()-Methode wird aufgerufen, wann immer
  //eine Karteneigenschaft sich ndert, die die Position des Elements verndern
  //knnte, wie z. B. Zoom, Mittelpunkt oder Kartentyp.
  TOverlayView = class(TMVCObject)
  private
    FMap: TMap;
    FPanes: TMapPanes;
    FProjection: TMapCanvasProjection;
    FOnAddFunction: TJScriptFunction;
    FDrawFunction: TJScriptFunction;
    FOnRemoveFunction: TJScriptFunction;
    procedure SetMapInt(Value: TMap);
  protected
    //von JScript aufgerufene Event-Handler:
    procedure DrawHandler;
    procedure OnAddHandler;
    procedure OnRemoveHandler;
    constructor Create; reintroduce; overload;
    constructor Create(Script: TCustomScript); reintroduce; overload;
  public
    destructor Destroy; override;
    procedure AfterConstruction; override;
    //Implementieren Sie diese Methode, um den Overlay zu zeichnen oder zu
    //aktualisieren. Diese Methode wird nach onAdd() aufgerufen und wenn fr die
    //Position von projection.fromLatLngToPixel() ein neuer Wert fr BreiteLnge
    //zurckgegeben wrde. Dies kommt vor, wenn Zoom, Mittelpunkt oder Kartentyp
    //verndert wird. Bei Verschieben oder ndern der Gre wird sie nicht
    //unbedingt aufgerufen:
    procedure Draw; virtual; abstract;
    //Fgt das Overlay zur Karte oder zum Panorama hinzu:
    procedure SetMap(Map: TMap); overload;
    procedure SetMap(Map: TStreetViewPanorama); overload;
    //Implementieren Sie diese Methode, um die Overlay-DOM-Elemente zu
    //initialisieren. Diese Methode wird einmal aufgerufen, nachdem setMap() mit
    //einer gltigen Karte aufgerufen wurde. Zu diesem Zeitpunkt sind Fenster
    //und Projektionen bereits initialisiert:
    procedure OnAdd; virtual; abstract;
    //Implementieren Sie diese Methode, um Elemente aus dem DOM zu entfernen.
    //Diese Methode wird einmal nach dem Aufrufen von setMap(null) aufgerufen:
    procedure OnRemove; virtual; abstract;
    function GetMap: TMap;
    //Gibt die Fenster zurck, in denen dieser Overlay wiedergegeben werden kann.
    //Nur verfgbar, nachdem Zeichnen aufgerufen wurde:
    function GetPanes: TMapPanes;
    //Gibt das Objekt KartenLeinwandProjektion zurck, das mit dieser
    //OverlayAnsicht assoziiert ist. Nur verfgbar, nachdem Zeichnen aufgerufen wurde:
    function GetProjection: TMapCanvasProjection;
    property Map: TMap read GetMap write SetMapInt;
  end;

  TRectangle = class(TMVCEventObject)
  private
    FBounds: TLatLngBounds;
    FMap: TMap;
  protected
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; Opts: TRectangleOptions): IDispatch; reintroduce; overload;
  public
    constructor Create(Opts: TRectangleOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TRectangleOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    procedure SetBounds(Bounds: TLatLngBounds);
    procedure SetMap(Map: TMap);
    procedure SetOptions(Options: TRectangleOptions); reintroduce;
    function GetBounds: TLatLngBounds;
    function GetMap: TMap;
    property Bounds: TLatLngBounds read GetBounds;
    property Map: TMap read GetMap write SetMap;
  end;

  TRectangleOptions = class(TCustomJScriptObject)
  private
    FBounds: TLatLngBounds;
    FMap: TMap;
    procedure SetBounds(Value: TLatLngBounds);
    procedure SetClickable(Value: Boolean);
    procedure SetFillColor(Value: TColor);
    procedure SetFillOpacity(Value: Double);
    procedure SetMap(Value: TMap);
    procedure SetStrokeColor(Value: TColor);
    procedure SetStrokeOpacity(Value: Double);
    procedure SetStrokeWeight(Value: Integer);
    procedure SetZIndex(Value: Integer);
    function GetBounds: TLatLngBounds;
    function GetClickable: Boolean;
    function GetFillColor: TColor;
    function GetFillOpacity: Double;
    function GetMap: TMap;
    function GetStrokeColor: TColor;
    function GetStrokeOpacity: Double;
    function GetStrokeWeight: Integer;
    function GetZIndex: Integer;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    property Bounds: TLatLngBounds read GetBounds write SetBounds;
    property Clickable: Boolean read GetClickable write SetClickable;
    property FillColor: TColor read GetFillColor write SetFillColor;
    property FillOpacity: Double read GetFillOpacity write SetFillOpacity;
    property Map: TMap read GetMap write SetMap;
    property StrokeColor: TColor read GetStrokeColor write SetStrokeColor;
    property StrokeOpacity: Double read GetStrokeOpacity write SetStrokeOpacity;
    property StrokeWeight: Integer read GetStrokeWeight write SetStrokeWeight;
    property ZIndex: Integer read GetZIndex write SetZIndex;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TAnimation

class function TAnimation.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[AnimationProp_Bounce,
                                 AnimationProp_Drop{,
                                 AnimationProp_Oi,
                                 AnimationProp_Qi}]);
end;

function TAnimation.GetBounce: Integer;
begin
  Result:=GetPropAsInteger(AnimationProp_Bounce);
end;

function TAnimation.GetDrop: Integer;
begin
  Result:=GetPropAsInteger(AnimationProp_Drop);
end;

function TAnimation.GetWj: Integer;
begin
  Result:=GetPropAsInteger(AnimationProp_Wj);
end;

function TAnimation.GetXj: Integer;
begin
  Result:=GetPropAsInteger(AnimationProp_Xj);
end;

//------------------------------------------------------------------------------
//TCircle

constructor TCircle.Create(Opts: TCircleOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TCircle.Create(Script: TCustomScript; Opts: TCircleOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TCircle.Destroy;
begin
  FBounds.Free;
  FCenter.Free;
  FMap.Free;
  inherited;
end;

class function TCircle.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_Circle,true);
end;

class function TCircle.New(Script: TCustomScript; Opts: TCircleOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_Circle,Opts.Disp,true);
end;

class function TCircle.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[CircelMethod_GetBounds,
                                 CircelMethod_GetCenter]);
end;

function TCircle.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetObjectWithExec(CircelMethod_GetBounds,
                                          TLatLngBounds,FBounds));
end;

function TCircle.GetCenter: TLatLng;
begin
  Result:=TLatLng(GetObjectWithExec(CircelMethod_GetCenter,TLatLng,FCenter));
end;

procedure TCircle.SetCenter(Center: TLatLng);
begin
  Exec(CircelMethod_SetCenter,Center);
end;

function TCircle.GetRadius: Double;
begin
  Result:=Exec(CircelMethod_GetRadius);
end;

procedure TCircle.SetRadius(Radius: Double);
begin
  Exec(CircelMethod_SetRadius,[Radius]);
end;

function TCircle.GetMap: TMap;
begin
  Result:=TMap(GetObjectWithExec(CircelMethod_GetMap,TMap,FMap));
end;

procedure TCircle.SetMap(Map: TMap);
begin
  Exec(CircelMethod_SetMap,Map);
end;

procedure TCircle.SetOptions(Options: TCircleOptions);
begin
  if assigned(Options)
    then Exec(CircelMethod_SetOptions,[Options.Disp]);
end;

//------------------------------------------------------------------------------
//TCircleOptions

constructor TCircleOptions.Create;
begin
  inherited;
end;

constructor TCircleOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TCircleOptions.Destroy;
begin
  FCenter.Free;
  FMap.Free;
  inherited;
end;

class function TCircleOptions.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[CircleOptionsProp_Center,
                                 CircleOptionsProp_Clickable,
                                 CircleOptionsProp_FillColor]);
end;

function TCircleOptions.GetCenter: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(CircleOptionsProp_Center,TLatLng,FCenter));
end;

procedure TCircleOptions.SetCenter(Value: TLatLng);
begin
  SetProp(CircleOptionsProp_Center,Value,true);
end;

function TCircleOptions.GetClickable: Boolean;
begin
  Result:=GetPropAsBoolean(CircleOptionsProp_Clickable);
end;

procedure TCircleOptions.SetClickable(Value: Boolean);
begin
  SetProp(CircleOptionsProp_Clickable,Value,true);
end;

function TCircleOptions.GetFillColor: TColor;
begin
  Result:=WebColorStrToColor(GetPropAsString(CircleOptionsProp_FillColor,
                                             ColorToWebColorStr(clNone)));
end;

procedure TCircleOptions.SetFillColor(Value: TColor);
begin
  SetProp(CircleOptionsProp_FillColor,ColorToWebColorStr(Value),true);
end;

function TCircleOptions.GetFillOpacity: Double;
begin
  Result:=GetPropAsDouble(CircleOptionsProp_FillOpacity);
end;

procedure TCircleOptions.SetFillOpacity(Value: Double);
begin
  SetProp(CircleOptionsProp_FillOpacity,Value,true);
end;

function TCircleOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(CircleOptionsProp_Map,TMap,FMap));
end;

procedure TCircleOptions.SetMap(Value: TMap);
begin
  SetProp(CircleOptionsProp_Map,Value,true);
end;

function TCircleOptions.GetRadius: Double;
begin
  Result:=GetPropAsDouble(CircleOptionsProp_Radius);
end;

procedure TCircleOptions.SetRadius(Value: Double);
begin
  SetProp(CircleOptionsProp_Radius,Value,true);
end;

function TCircleOptions.GetStrokeColor: TColor;
begin
  Result:=WebColorStrToColor(GetPropAsString(CircleOptionsProp_StrokeColor,
                                             ColorToWebColorStr(clNone)));
end;

procedure TCircleOptions.SetStrokeColor(Value: TColor);
begin
  SetProp(CircleOptionsProp_StrokeColor,ColorToWebColorStr(Value),true);
end;

function TCircleOptions.GetStrokeOpacity: Double;
begin
  Result:=GetPropAsDouble(CircleOptionsProp_StrokeOpacity);
end;

procedure TCircleOptions.SetStrokeOpacity(Value: Double);
begin
  SetProp(CircleOptionsProp_StrokeOpacity,Value,true);
end;

function TCircleOptions.GetStrokeWeight: Integer;
begin
  Result:=GetPropAsInteger(CircleOptionsProp_StrokeWeight);
end;

procedure TCircleOptions.SetStrokeWeight(Value: Integer);
begin
  SetProp(CircleOptionsProp_StrokeWeight,Value,true);
end;

function TCircleOptions.GetZIndex: Integer;
begin
  Result:=GetPropAsInteger(CircleOptionsProp_ZIndex);
end;

procedure TCircleOptions.SetZIndex(Value: Integer);
begin
  SetProp(CircleOptionsProp_ZIndex,Value,true);
end;

//------------------------------------------------------------------------------
//TGroundOverlay

constructor TGroundOverlay.Create(const URL: String; Bounds: TLatLngBounds;
                                  Opts: TGroundOverlayOptions);
begin
  Create(JScriptObjects.Script,URL,Bounds,Opts);
end;

constructor TGroundOverlay.Create(Script: TCustomScript; const URL: String;
                                  Bounds: TLatLngBounds; Opts: TGroundOverlayOptions);
begin
  inherited Create(Script,New(Script,URL,Bounds,Opts));
end;

destructor TGroundOverlay.Destroy;
begin
  SetOnClick(nil);
  FOnClickFunction.Free;
  FBounds.Free;
  FMap.Free;
  inherited;
end;

class function TGroundOverlay.New(Script: TCustomScript; const URL: String;
                                  Bounds: TLatLngBounds): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Bounds)
    then raise Exception.Create(ExceptionNoBounds);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_GroundOverlay,
                              {$IFDEF CAN_OVERLOAD}
                              [URL,Bounds.Disp]
                              {$ELSE}
                              URL,Bounds.Disp
                              {$ENDIF}
                              ,true);
end;

class function TGroundOverlay.New(Script: TCustomScript; const URL: String;
                                  Bounds: TLatLngBounds;
                                  Opts: TGroundOverlayOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Bounds)
    then raise Exception.Create(ExceptionNoBounds);
  if not assigned(Opts)
    then Result:=New(Script,URL,Bounds)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_GroundOverlay,
                                     {$IFDEF CAN_OVERLOAD}
                                     [URL,Bounds.Disp,Opts.Disp]
                                     {$ELSE}
                                     URL,Bounds.Disp,Opts.Disp
                                     {$ENDIF}
                                     ,true);
end;

procedure TGroundOverlay.SetMap(Map: TMap);
begin
  if assigned(Map)
    then Exec(GroundOverlayMethod_SetMap,Map)
    else Exec(GroundOverlayMethod_SetMap,[Null]);
end;

function TGroundOverlay.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetObjectWithExec(GroundOverlayMethod_GetBounds,
                                          TLatLngBounds,FBounds));
end;

function TGroundOverlay.GetMap: TMap;
begin
  Result:=TMap(GetObjectWithExec(GroundOverlayMethod_GetMap,TMap,FMap));
end;

function TGroundOverlay.GetUrl: String;
begin
  Result:=VarAsString(Exec(GroundOverlayMethod_GetUrl));
end;

procedure TGroundOverlay.Click;
begin
  CallEvent(FOnClick,FOnClickFunction.Arguments[0]);
end;

procedure TGroundOverlay.SetOnClick(Value: TMapMouseEvent);
begin
  FOnClick:=Value;
  RemoveListener(FOnClickListener);
  if assigned(FOnClick)
    then AddListener(Event_Click,FOnClickListener,FOnClickFunction,Click);
end;

//------------------------------------------------------------------------------
//TGroundOverlayOptions

destructor TGroundOverlayOptions.Destroy;
begin
  FMap.Free;
  inherited;
end;

function TGroundOverlayOptions.GetClickable: Boolean;
begin
  Result:=GetPropAsBoolean(GroundOverlayOptionsProp_Clickable);
end;

procedure TGroundOverlayOptions.SetClickable(Value: Boolean);
begin
  SetProp(GroundOverlayOptionsProp_Clickable,Value,true);
end;

function TGroundOverlayOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(GroundOverlayOptionsProp_Map,TMap,FMap));
end;

procedure TGroundOverlayOptions.SetMap(Value: TMap);
begin
  SetProp(GroundOverlayOptionsProp_Map,Value,true);
end;

//------------------------------------------------------------------------------
//TMapCanvasProjection

destructor TMapCanvasProjection.Destroy;
begin
  FLatLng1.Free;
  FLatLng2.Free;
  FPoint1.Free;
  FPoint2.Free;
  inherited;
end;

class function TMapCanvasProjection.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MapCanvasProjectionMethod_FromContainerPixelToLatLng,
                                 MapCanvasProjectionMethod_FromDivPixelToLatLng,
                                 MapCanvasProjectionMethod_FromLatLngToContainerPixel]);
end;

function TMapCanvasProjection.FromContainerPixelToLatLng(Pixel: TPoint): TLatLng;
var
  LatLng: OleVariant;
begin
  Result:=nil;
  LatLng:=Exec(MapCanvasProjectionMethod_FromContainerPixelToLatLng,Pixel);
  if VarIsValidDispatch(LatLng) then
  begin
    if assigned(FLatLng1)
      then FLatLng1.Disp:=LatLng
      else FLatLng1:=TLatLng(CreateObjectWrapper(Script,TLatLng,LatLng));
    Result:=FLatLng1;
  end;
end;

function TMapCanvasProjection.FromDivPixelToLatLng(Pixel: TPoint): TLatLng;
var
  LatLng: OleVariant;
begin
  Result:=nil;
  LatLng:=Exec(MapCanvasProjectionMethod_FromDivPixelToLatLng,Pixel);
  if VarIsValidDispatch(LatLng) then
  begin
    if assigned(FLatLng1)
      then FLatLng2.Disp:=LatLng
      else FLatLng2:=TLatLng(CreateObjectWrapper(Script,TLatLng,LatLng));
    Result:=FLatLng2;
  end;
end;

function TMapCanvasProjection.FromLatLngToContainerPixel(LatLng: TLatLng): TPoint;
var
  Point: OleVariant;
begin
  Result:=nil;
  Point:=Exec(MapCanvasProjectionMethod_FromLatLngToContainerPixel,LatLng);
  if VarIsValidDispatch(Point) then
  begin
    if assigned(FPoint1)
      then FPoint1.Disp:=Point
      else FPoint1:=TPoint(CreateObjectWrapper(Script,TPoint,Point));
    Result:=FPoint1;
  end;
end;

function TMapCanvasProjection.FromLatLngToDivPixel(LatLng: TLatLng): TPoint;
var
  Point: OleVariant;
begin
  Result:=nil;
  Point:=Exec(MapCanvasProjectionMethod_FromLatLngToDivPixel,LatLng);
  if VarIsValidDispatch(Point) then
  begin
    if assigned(FPoint2)
      then FPoint2.Disp:=Point
      else FPoint2:=TPoint(CreateObjectWrapper(Script,TPoint,Point));
    Result:=FPoint2;
  end;
end;

function TMapCanvasProjection.GetWorldWidth: Integer;
begin
  Result:=Exec(MapCanvasProjectionMethod_GetWorldWidth);
end;

//------------------------------------------------------------------------------
//TMapPanes

destructor TMapPanes.Destroy;
begin
  FFloatPane.Free;
  FFloatShadow.Free;
  FMapPane.Free;
  FOverlayImage.Free;
  FOverlayLayer.Free;
  FOverlayMouseTarget.Free;
  FOverlayShadow.Free;
  inherited;
end;

class function TMapPanes.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MapPanesProp_FloatPane,
                                 MapPanesProp_FloatShadow,
                                 MapPanesProp_MapPane]);
end;

function TMapPanes.GetFloatPane: TNode;
begin
  Result:=TNode(GetPropAsObject(MapPanesProp_FloatPane,TNode,FFloatPane));
end;

procedure TMapPanes.SetFloatPane(Value: TNode);
begin
  SetProp(MapPanesProp_FloatPane,Value);
end;

function TMapPanes.GetFloatShadow: TNode;
begin
  Result:=TNode(GetPropAsObject(MapPanesProp_FloatShadow,TNode,FFloatShadow));
end;

procedure TMapPanes.SetFloatShadow(Value: TNode);
begin
  SetProp(MapPanesProp_FloatShadow,Value);
end;

function TMapPanes.GetMapPane: TNode;
begin
  Result:=TNode(GetPropAsObject(MapPanesProp_MapPane,TNode,FMapPane));
end;

procedure TMapPanes.SetMapPane(Value: TNode);
begin
  SetProp(MapPanesProp_MapPane,Value);
end;

function TMapPanes.GetOverlayImage: TNode;
begin
  Result:=TNode(GetPropAsObject(MapPanesProp_OverlayImage,TNode,FOverlayImage));
end;

procedure TMapPanes.SetOverlayImage(Value: TNode);
begin
  SetProp(MapPanesProp_OverlayImage,Value);
end;

function TMapPanes.GetOverlayLayer: TNode;
begin
  Result:=TNode(GetPropAsObject(MapPanesProp_OverlayLayer,TNode,FOverlayLayer));
end;

procedure TMapPanes.SetOverlayLayer(Value: TNode);
begin
  SetProp(MapPanesProp_OverlayLayer,Value);
end;

function TMapPanes.GetOverlayMouseTarget: TNode;
begin
  Result:=TNode(GetPropAsObject(MapPanesProp_OverlayMouseTarget,TNode,FOverlayMouseTarget));
end;

procedure TMapPanes.SetOverlayMouseTarget(Value: TNode);
begin
  SetProp(MapPanesProp_OverlayMouseTarget,Value);
end;

function TMapPanes.GetOverlayShadow: TNode;
begin
  Result:=TNode(GetPropAsObject(MapPanesProp_OverlayShadow,TNode,FOverlayShadow));
end;

procedure TMapPanes.SetOverlayShadow(Value: TNode);
begin
  SetProp(MapPanesProp_OverlayShadow,Value);
end;

//------------------------------------------------------------------------------
//TMapTypeStyleElementType
(*
function TMapTypeStyleElementType.All: String;
begin
end;

function TMapTypeStyleElementType.Geometry: String;
begin
end;

function TMapTypeStyleElementType.Labels: String;
begin
end;
*)
//------------------------------------------------------------------------------
//TMapTypeStyles
(*
constructor TMapTypeStyles.Create;
begin
  Create(JScriptObjects.Script);
end;

constructor TMapTypeStyles.Create(Script: TCustomScript);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script);
end;

function TMapTypeStyles.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TMapTypeStyle(CreateObjectWrapper(Script,TMapTypeStyle,Dispatch));
end;

function TMapTypeStyles.GetItem(Index: Integer): TMapTypeStyle;
begin
  Result:=TMapTypeStyle(inherited Items[Index]);
end;

procedure TMapTypeStyles.SetItem(Index: Integer; Value: TMapTypeStyle);
begin
  inherited Items[Index]:=Value;
end;

function TMapTypeStyles.AddStyle: TMapTypeStyle;
begin
  Result:=Insert(Count);
end;

function TMapTypeStyles.Insert(Index: Integer): TMapTypeStyle;
begin
  Result:=TMapTypeStyle.Create;
  inherited Insert(Index,Result);
end;

//------------------------------------------------------------------------------
//TMapTypeStyles

constructor TMapTypeStylesCollection.Create;
begin
  Create(JScriptObjects.Script);
end;

constructor TMapTypeStylesCollection.Create(Script: TCustomScript);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create;
  FItems:=TStringList.Create{$IFDEF DELPHI2009_UP}(true){$ENDIF};
  FScript:=Script;
end;

destructor TMapTypeStylesCollection.Destroy;
begin
{$IFNDEF DELPHI2009_UP}
  if assigned(FItems) then
    while FItems.Count>0 do
    begin
      FItems.Objects[0].Free;
      FItems.Delete(0);
    end;
{$ENDIF}
  FreeAndNil(FItems);
  inherited;
end;

function TMapTypeStylesCollection.AddStyles(Name: String): TMapTypeStyles;
begin
  Result:=TMapTypeStyles.Create(Script);
  FItems.AddObject(Name,Result);
end;

function TMapTypeStylesCollection.GetStyles(Name: String): TMapTypeStyles;
var
  Index: Integer;
begin
  Result:=nil;
  Index:=FItems.IndexOf(Name);
  if Index>=0
   then Result:=TMapTypeStyles(FItems.Objects[Index]);
end;
*)
//------------------------------------------------------------------------------
//TOverlayView

constructor TOverlayView.Create;
begin
  Create(JScriptObjects.Script);
end;

constructor TOverlayView.Create(Script: TCustomScript);
begin
  if not assigned(Script) or not (Script is TScript)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script,BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                              MapsMethod_OverlayView,true));
end;

destructor TOverlayView.Destroy;
begin
  SetProp(OverlayViewEvent_OnAdd,Null);
  SetProp(OverlayViewEvent_Draw,Null);
  SetProp(OverlayViewEvent_OnRemove,Null);
  FOnAddFunction.Free;
  FDrawFunction.Free;
  FOnRemoveFunction.Free;
  FMap.Free;
  FPanes.Free;
  FProjection.Free;
  inherited;
end;

procedure TOverlayView.AfterConstruction;
begin
  inherited;
  FOnAddFunction:=CreateJScriptFunction(Script,OnAddHandler);
  FDrawFunction:=CreateJScriptFunction(Script,DrawHandler);
  FOnRemoveFunction:=CreateJScriptFunction(Script,OnRemoveHandler);
  SetProp(OverlayViewEvent_OnAdd,FOnAddFunction,true);
  SetProp(OverlayViewEvent_Draw,FDrawFunction,true);
  SetProp(OverlayViewEvent_OnRemove,FOnRemoveFunction,true);
end;

procedure TOverlayView.OnAddHandler;
begin
  try
    OnAdd;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

procedure TOverlayView.DrawHandler;
begin
  try
    Draw;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

procedure TOverlayView.OnRemoveHandler;
begin
  try
    OnRemove;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

procedure TOverlayView.SetMap(Map: TMap);
begin
  Exec(OverlayViewMethod_SetMap,Map);
end;

procedure TOverlayView.SetMap(Map: TStreetViewPanorama);
begin
  Exec(OverlayViewMethod_SetMap,Map);
end;

function TOverlayView.GetMap: TMap;
begin
  Result:=TMap(GetObjectWithExec(OverlayViewMethod_GetMap,TMap,FMap));
end;

function TOverlayView.GetPanes: TMapPanes;
begin
  Result:=TMapPanes(GetObjectWithExec(OverlayViewMethod_GetPanes,TMapPanes,FPanes));
end;

function TOverlayView.GetProjection: TMapCanvasProjection;
begin
  Result:=TMapCanvasProjection(GetObjectWithExec(OverlayViewMethod_GetProjection,
                                                 TMapCanvasProjection,FProjection));
end;

//------------------------------------------------------------------------------
//TRectangle

constructor TRectangle.Create(Opts: TRectangleOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TRectangle.Create(Script: TCustomScript; Opts: TRectangleOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TRectangle.Destroy;
begin
  FBounds.Free;
  FMap.Free;
  inherited;
end;

class function TRectangle.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_Rectangle,true);
end;

class function TRectangle.New(Script: TCustomScript; Opts: TRectangleOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_Rectangle,Opts.Disp,true);
end;

class function TRectangle.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[RectangleMethod_GetBounds,
                                 RectangleMethod_GetMap]);
end;

function TRectangle.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetObjectWithExec(RectangleMethod_GetBounds,
                                          TLatLngBounds,FBounds));
end;

function TRectangle.GetMap: TMap;
begin
  Result:=TMap(GetObjectWithExec(RectangleMethod_GetMap,TMap,FMap));
end;

procedure TRectangle.SetBounds(Bounds: TLatLngBounds);
begin
  if assigned(Bounds)
    then Exec(RectangleMethod_SetBounds,[Bounds.Disp]);
end;

procedure TRectangle.SetMap(Map: TMap);
begin
  Exec(RectangleMethod_SetMap,Map);
end;

procedure TOverlayView.SetMapInt(Value: TMap);
begin
  SetMap(Value);
end;

procedure TRectangle.SetOptions(Options: TRectangleOptions);
begin
  if assigned(Options)
    then Exec(RectangleMethod_SetOptions,[Options.Disp]);
end;

//------------------------------------------------------------------------------
//TRectangleOptions

constructor TRectangleOptions.Create;
begin
  inherited;
end;

constructor TRectangleOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TRectangleOptions.Destroy;
begin
  FBounds.Free;
  FMap.Free;
  inherited;
end;

class function TRectangleOptions.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[RectangleOptionsProp_Bounds,
                                 RectangleOptionsProp_Clickable,
                                 RectangleOptionsProp_FillColor]);
end;

function TRectangleOptions.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetPropAsObject(RectangleOptionsProp_Bounds,
                                        TLatLngBounds,FBounds));
end;

procedure TRectangleOptions.SetBounds(Value: TLatLngBounds);
begin
  SetProp(RectangleOptionsProp_Bounds,Value,true);
end;

function TRectangleOptions.GetClickable: Boolean;
begin
  Result:=GetPropAsBoolean(RectangleOptionsProp_Clickable,true);
end;

procedure TRectangleOptions.SetClickable(Value: Boolean);
begin
  SetProp(RectangleOptionsProp_Clickable,Value,true);
end;

function TRectangleOptions.GetFillColor: TColor;
begin
  Result:=WebColorStrToColor(GetPropAsString(RectangleOptionsProp_FillColor,
                                             ColorToWebColorStr(clNone)));
end;

procedure TRectangleOptions.SetFillColor(Value: TColor);
begin
  SetProp(RectangleOptionsProp_FillColor,ColorToWebColorStr(Value),true);
end;

function TRectangleOptions.GetFillOpacity: Double;
begin
  Result:=GetPropAsDouble(RectangleOptionsProp_FillOpacity);
end;

procedure TRectangleOptions.SetFillOpacity(Value: Double);
begin
  SetProp(RectangleOptionsProp_FillOpacity,Value,true);
end;

function TRectangleOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(RectangleOptionsProp_Map,TMap,FMap));
end;

procedure TRectangleOptions.SetMap(Value: TMap);
begin
  SetProp(RectangleOptionsProp_Map,Value,true);
end;

function TRectangleOptions.GetStrokeColor: TColor;
begin
  Result:=WebColorStrToColor(GetPropAsString(RectangleOptionsProp_StrokeColor,
                                             ColorToWebColorStr(clNone)));
end;

procedure TRectangleOptions.SetStrokeColor(Value: TColor);
begin
  SetProp(RectangleOptionsProp_StrokeColor,ColorToWebColorStr(Value),true)
end;

function TRectangleOptions.GetStrokeOpacity: Double;
begin
  Result:=GetPropAsDouble(RectangleOptionsProp_StrokeOpacity);
end;

procedure TRectangleOptions.SetStrokeOpacity(Value: Double);
begin
  SetProp(RectangleOptionsProp_StrokeOpacity,Value,true);
end;

function TRectangleOptions.GetStrokeWeight: Integer;
begin
  Result:=GetPropAsInteger(RectangleOptionsProp_StrokeWeight);
end;

procedure TRectangleOptions.SetStrokeWeight(Value: Integer);
begin
  SetProp(RectangleOptionsProp_StrokeWeight,Value,true);
end;

function TRectangleOptions.GetZIndex: Integer;
begin
  Result:=GetPropAsInteger(RectangleOptionsProp_ZIndex);
end;

procedure TRectangleOptions.SetZIndex(Value: Integer);
begin
  SetProp(RectangleOptionsProp_ZIndex,Value,true);
end;

end.
