///<summary>
///  Diese Unit enthlt die Objekte zur Anzeige von Info-Fenstern.
///  <para/>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmInfoWindow|TInfoWindow">TInfoWindow</see></item>
///    <item><see cref="gmInfoWindow|TInfoWindowOptions">TInfoWindowOptions</see></item>
///  </list>
///</summary>
///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html">
///Google Maps API</seealso>.

unit gmOverlaysInfoWindow;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils, BrowserTools, JScriptObjects, HTMLObjects, DispObject,
  gmBase, gmMVC, gmMap, gmStreetView, gmEvents, gmAdditionalConstants;

type
  TInfoWindowOptions = class;
  TInfoWindow = class;

  TInfoWindow = class(TMVCObject)
  private
    FNode: TNode;
    FPosition: TLatLng;
    FSize: TSize;
    FOnCloseClick: TNotifyEvent;
    FOnCloseClickListener: TMapsEventListener;
    FOnCloseClickFunction: TJScriptFunction;
    FOnContentChanged: TNotifyEvent;
    FOnContentChangedListener: TMapsEventListener;
    FOnContentChangedFunction: TJScriptFunction;
    FOnDomReady: TNotifyEvent;
    FOnDomReadyListener: TMapsEventListener;
    FOnDomReadyFunction: TJScriptFunction;
    FOnPositionChanged: TNotifyEvent;
    FOnPositionChangedListener: TMapsEventListener;
    FOnPositionChangedFunction: TJScriptFunction;
    FOnZIndexChanged: TNotifyEvent;
    FOnZIndexChangedListener: TMapsEventListener;
    FOnZIndexChangedFunction: TJScriptFunction;
    procedure SetContentAsString(Value: String);
    procedure SetOnCloseClick(Value: TNotifyEvent);
    procedure SetOnContentChanged(Value: TNotifyEvent);
    procedure SetOnDomReady(Value: TNotifyEvent);
    procedure SetOnPositionChanged(Value: TNotifyEvent);
    procedure SetOnZIndexChanged(Value: TNotifyEvent);
  protected
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; Opts: TInfoWindowOptions): IDispatch; reintroduce; overload;
    //von JScript aufgerufene Event-Handler:
    procedure CloseClick;
    procedure ContentChanged;
    procedure DomReady;
    procedure PositionChanged;
    procedure ZIndexChanged;
  public
    constructor Create(Opts: TInfoWindowOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TInfoWindowOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Close'}{$ENDIF}
    ///<summary>
    ///  Schliet das Infofenster, indem es aus der DOM-Struktur entfernt wird.
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Close;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Open'}{$ENDIF}
    ///<summary>
    ///  ffnet das InfoWindow in der angegebenen Karte. Optional kann das
    ///  Fenster mit einem Anker verbunden werden. Im Kern-API ist der einzige
    ///  Anker die Markierungsklasse. Als Anker kann deshalb auch jedes MVCObjekt
    ///  fungieren, das die Positionseigenschaft aufweist und - wahlweise - pixelBounds
    ///  zur Berechnung der pixelOffset (siehe InfoWindowOptions).
    ///</summary>
    ///<seealso cref="gmMap|TMap">TMap</seealso>
    ///<seealso cref="gmMVC|TMVCObject">TMVCObject</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Open(Map: TMap; Anchor: TMVCObject = nil); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Open'}{$ENDIF}
    ///<summary>
    ///  ffnet das InfoWindow im angegebenen Panorama. Optional kann das
    ///  Fenster mit einem Anker verbunden werden. Im Kern-API ist der einzige
    ///  Anker die Markierungsklasse. Als Anker kann deshalb auch jedes MVCObjekt
    ///  fungieren, das die Positionseigenschaft aufweist und - wahlweise - pixelBounds
    ///  zur Berechnung der pixelOffset (siehe InfoWindowOptions).
    ///</summary>
    ///<seealso cref="gmStreetView|TStreetViewPanorama">TStreeViewPanorama</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Open(Map: TStreetViewPanorama; Anchor: TMVCObject = nil); overload;
    //Der Original-Funktion "setContent()" kann sowohl einen String als auch ein
    //Node-Objekt bergeben werden:
    procedure SetContent(Content: String); overload;
    procedure SetContent(Content: TNode); overload;
    procedure SetOptions(Options: TInfoWindowOptions); reintroduce;
    procedure SetPosition(Position: TLatLng);
    procedure SetSize(Size: TSize);                        //nicht dokumentiert
    procedure SetZIndex(ZIndex: Integer);
    //Die Original-Funktion "getContent()" kann sowohl einen String als auch ein
    //Node-Objekt zurckgeben. Die Delphi-Methode "GetContent" liefert den String
    //oder - im Falle eines Objektes - den Wert "[object Object]".
    //Um das Objekt - falls vorhanden - zu ermitteln, mu die Methode "GetContentNode"
    //verwendet werden:
    function GetContent: String;
    function GetContentNode: TNode;                        //nicht im JScript-Objekt vorhanden
    function GetPosition: TLatLng;
    function GetSize: TSize;                               //nicht dokumentiert
    function GetZIndex: Integer;
    //zustzliche Delphi-Eigenschaften:
    property Content: String read GetContent write SetContentAsString;
    property Position: TLatLng read GetPosition write SetPosition;
    property Size: TSize read GetSize write SetSize;
    property ZIndex: Integer read GetZIndex write SetZIndex;
    //Dieses Ereignis wird ausgelst, wenn auf die Schalflche Schlieen geklickt wurde:
    property OnCloseClick: TNotifyEvent read FOnCloseClick write SetOnCloseClick;
    //Dieses Ereignis wird ausgelst, wenn sich die Eigenschaft Inhalt ndert:
    property OnContentChanged: TNotifyEvent read FOnContentChanged write SetOnContentChanged;
    //Dieses Ereignis wird ausgelst, wenn die <div>, die den Inhalt des InfoWindows
    //enthlt, an das DOM angehngt wird. Mglicherweise mchten Sie dieses Ereignis
    //berwachen, falls Sie Ihren Infofenster-Inhalt dynamisch erstellen:
    property OnDomReady: TNotifyEvent read FOnDomReady write SetOnDomReady;
    //Dieses Ereignis wird ausgelst, wenn sich die Eigenschaft Position ndert:
    property OnPositionChanged: TNotifyEvent read FOnPositionChanged write SetOnPositionChanged;
    //Dieses Ereignis wird ausgelst, wenn sich der zIndex des InfoWindow ndert:
    property OnZIndexChanged: TNotifyEvent read FOnZIndexChanged write SetOnZIndexChanged;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TInfoWindowOptions'}{$ENDIF}
  ///<summary>
  ///  Objekt zur Initialisierung und zur nderung von Einstellungen eines
  ///  Info-Fensters.
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.Create">Create</see></item>
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.Create(TCustomScript)">Create(Script)</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.Content">Content</see></item>
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.ContentNode">ContentNode</see></item>
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.DisableAutoPan">DisableAutoPan</see></item>
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.MaxWidth">MaxWidth</see></item>
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.PixelOffset">PixelOffset</see></item>
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.Position">Position</see></item>
  ///    <item><see cref="gmInfoWindow|TInfoWindowOptions.ZIndex">ZIndex</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#InfoWindowOptions">
  ///  InfoWindowOptions
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TInfoWindowOptions = class(TCustomJScriptObject)
  private
    FContentNode: TNode;
    FPixelOffset: TSize;
    FPosition: TLatLng;
    procedure SetContent(Value: String);
    procedure SetContentNode(Value: TNode);
    procedure SetDisableAutoPan(Value: Boolean);
    procedure SetMaxWidth(Value: Integer);
    procedure SetPixelOffset(Value: TSize);
    procedure SetPosition(Value: TLatLng);
    procedure SetZIndex(Value: Integer);
    function GetContent: String;
    function GetContentNode: TNode;
    function GetDisableAutoPan: Boolean;
    function GetMaxWidth: Integer;
    function GetPixelOffset: TSize;
    function GetPosition: TLatLng;
    function GetZIndex: Integer;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Content'}{$ENDIF}
    ///<summary>
    ///  Im InfoFenster anzuzeigender Inhalt. Dies kann ein HTML-Element sein,
    ///  eine reine Zeichenfolge oder eine Zeichenfolge, die HTML enthlt. Die
    ///  Gre des InfoFensters wird an seinen Inhalt angepasst. Um ausdrcklich
    ///  eine Gre fr den Inhalt zu bestimmen, muss festgelegt werden, dass
    ///  der Inhalt ein HTML-Element mit dieser Gre sein soll.
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Content: String read GetContent write SetContent;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ContentNode'}{$ENDIF}
    ///<summary>
    ///  Erweitert die Eigenschaft <i>Content</i>, um ein HTML-Element
    ///  setzen/lesen zu knnen. Das ist notwendig, da in Delphi Eigenschaften nicht
    ///  berladen werden knnen.
    ///</summary>
    ///<seealso cref="HTMLObjects|TNode">TNode</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ContentNode: TNode read GetContentNode write SetContentNode;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DisableAutoPan'}{$ENDIF}
    ///<summary>
    ///  Aktiviert/deaktiviert automatisches Schwenken beim ffnen. Standardmig
    ///  schwenkt das Infofenster die Karte, so dass es beim ffnen vollstndig
    ///  sichtbar ist.
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DisableAutoPan: Boolean read GetDisableAutoPan write SetDisableAutoPan;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MaxWidth'}{$ENDIF}
    ///<summary>
    ///  Die maximale Breite des Infofensters, ungeachtet der Breite des Inhalts.
    ///  Dieser Wert wird nur dann bercksichtigt, wenn er vor dem Aufrufen von
    ///  <see cref="gmInfoWindow|TInfoWindow.Open(TMap,TMVCObject)">Open</see>
    ///  festgelegt wurde. Um die maximale Breite gleichzeitig mit dem Inhalt zu
    ///  ndern, kann <see cref="gmInfoWindow|TInfoWindow.Close">Close</see>,
    ///  dann <see cref="gmInfoWindow|TInfoWindow.SetOptions(TInfoWindowOptions)">
    ///  SetOptions</see> und anschlieend
    ///  <see cref="gmInfoWindow|TInfoWindow.Open(TMap,TMVCObject)">Open</see>
    ///  aufgerufen werden.
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MaxWidth: Integer read GetMaxWidth write SetMaxWidth;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PixelOffset'}{$ENDIF}
    ///<summary>
    ///  Der Abstand in Pixeln zwischen der Spitze des Infofensters und dem Punkt
    ///  auf der Karte, an dessen geografischen Koordinaten das Infofenster verankert
    ///  ist. Wenn ein InfoWindow mit einem Anker geffnet wird, wird <i>PixelOffset</i>
    ///  anhand des oberen Zentrums der Ankergrenzen berechnet.
    ///</summary>
    ///<seealso cref="gmBase|TSize">TSize</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PixelOffset: TSize read GetPixelOffset write SetPixelOffset;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Position'}{$ENDIF}
    ///<summary>
    ///  Der LatLng-Wert, bei der diesem InfoWindow angezeigt werden soll. Wenn
    ///  das InfoWindow mit einem Anker geffnet wird, wird stattdessen die
    ///  Ankerposition verwendet.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng">TLatLng</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Position: TLatLng read GetPosition write SetPosition;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZIndex'}{$ENDIF}
    ///<summary>
    ///  Smtliche InfoWindow-Elemente werden auf der Karte in der Reihenfolge
    ///  des ZIndex angezeigt, wobei Elemente mit hheren Werten vor Elementen
    ///  mit niedrigeren Werten rangieren. Standardmig werden Fenster
    ///  entsprechend ihrer geografischen Breite angezeigt, wobei die mit
    ///  niedrigeren Breiten vor denen mit hheren Breiten liegen.
    ///  InfoWindows-Elemente werden immer vor Markierungen angezeigt.
    ///</summary>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZIndex: Integer read GetZIndex write SetZIndex;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TInfoWindow

constructor TInfoWindow.Create(Opts: TInfoWindowOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TInfoWindow.Create(Script: TCustomScript; Opts: TInfoWindowOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TInfoWindow.Destroy;
begin
  SetOnCloseClick(nil);
  SetOnContentChanged(nil);
  SetOnDomReady(nil);
  SetOnPositionChanged(nil);
  SetOnZIndexChanged(nil);
  FOnCloseClickFunction.Free;
  FOnContentChangedFunction.Free;
  FOnDomReadyFunction.Free;
  FOnPositionChangedFunction.Free;
  FOnZIndexChangedFunction.Free;
  FNode.Free;
  FPosition.Free;
  FSize.Free;
  inherited;
end;

class function TInfoWindow.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[InfoWindowMethod_Close,
                                 InfoWindowMethod_Open,
                                 InfoWindowMethod_SetContent]);
end;

class function TInfoWindow.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_InfoWindow,true);
end;

class function TInfoWindow.New(Script: TCustomScript;
                               Opts: TInfoWindowOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_InfoWindow,Opts.Disp,true);
end;

procedure TInfoWindow.CloseClick;
begin
  CallEvent(FOnCloseClick);
end;

procedure TInfoWindow.ContentChanged;
begin
  CallEvent(FOnContentChanged);
end;

procedure TInfoWindow.DomReady;
begin
  CallEvent(FOnDomReady);
end;

procedure TInfoWindow.PositionChanged;
begin
  CallEvent(FOnPositionChanged);
end;

procedure TInfoWindow.ZIndexChanged;
begin
  CallEvent(FOnZIndexChanged);
end;

procedure TInfoWindow.SetOnCloseClick(Value: TNotifyEvent);
begin
  FOnCloseClick:=Value;
  RemoveListener(FOnCloseClickListener);
  if assigned(FOnCloseClick)
    then AddListener(InfoWindowEvent_CloseClick,FOnCloseClickListener,
                     FOnCloseClickFunction,CloseClick);
end;

procedure TInfoWindow.SetOnContentChanged(Value: TNotifyEvent);
begin
  FOnContentChanged:=Value;
  RemoveListener(FOnContentChangedListener);
  if assigned(FOnContentChanged)
    then AddListener(InfoWindowEvent_ContentChanged,FOnContentChangedListener,
                     FOnContentChangedFunction,ContentChanged);
end;

procedure TInfoWindow.SetOnDomReady(Value: TNotifyEvent);
begin
  FOnDomReady:=Value;
  RemoveListener(FOnDomReadyListener);
  if assigned(FOnDomReady)
    then AddListener(InfoWindowEvent_DomReady,FOnDomReadyListener,
                     FOnDomReadyFunction,DomReady);
end;

procedure TInfoWindow.SetOnPositionChanged(Value: TNotifyEvent);
begin
  FOnPositionChanged:=Value;
  RemoveListener(FOnPositionChangedListener);
  if assigned(FOnPositionChanged)
    then AddListener(InfoWindowEvent_PositionChanged,FOnPositionChangedListener,
                     FOnPositionChangedFunction,PositionChanged);
end;

procedure TInfoWindow.SetOnZIndexChanged(Value: TNotifyEvent);
begin
  FOnZIndexChanged:=Value;
  RemoveListener(FOnZIndexChangedListener);
  if assigned(FOnZIndexChanged)
    then AddListener(InfoWindowEvent_ZIndexChanged,FOnZIndexChangedListener,
                     FOnZIndexChangedFunction,ZIndexChanged);
end;

procedure TInfoWindow.Close;
begin
  Exec(InfoWindowMethod_Close);
end;

procedure TInfoWindow.Open(Map: TMap; Anchor: TMVCObject = nil);
begin
  if assigned(Map) then
  begin
    if assigned(Anchor)
      then Exec(InfoWindowMethod_Open,[Map.Disp,Anchor.Disp])
      else Exec(InfoWindowMethod_Open,[Map.Disp]);
  end;
end;

procedure TInfoWindow.Open(Map: TStreetViewPanorama; Anchor: TMVCObject = nil);
begin
  if assigned(Map) then
  begin
    if assigned(Anchor)
      then Exec(InfoWindowMethod_Open,[Map.Disp,Anchor.Disp])
      else Exec(InfoWindowMethod_Open,[Map.Disp]);
  end;
end;

procedure TInfoWindow.SetContent(Content: String);
begin
  Exec(InfoWindowMethod_SetContent,[Content]);
end;

procedure TInfoWindow.SetContent(Content: TNode);
begin
  if assigned(Content)
    then Exec(InfoWindowMethod_SetContent,[Content.Disp]);
end;

//zustzlich, da in lteren Delphi-Versionen Property-Methoden nicht berladen
//werden drfen: SetContent(Content: String), SetContent(Content: TNode)
procedure TInfoWindow.SetContentAsString(Value: String);
begin
  SetContent(Value);
end;

procedure TInfoWindow.SetOptions(Options: TInfoWindowOptions);
begin
  if assigned(Options)
    then Exec(InfoWindowMethod_SetOptions,[Options.Disp]);
end;

procedure TInfoWindow.SetPosition(Position: TLatLng);
begin
  if assigned(Position)
    then Exec(InfoWindowMethod_SetPosition,[Position.Disp]);
end;

procedure TInfoWindow.SetSize(Size: TSize);
begin
  if assigned(Size)
    then Exec(InfoWindowMethod_SetSize,[Size.Disp]);
end;

procedure TInfoWindow.SetZIndex(ZIndex: Integer);
begin
  Exec(InfoWindowMethod_SetZIndex,[ZIndex]);
end;

function TInfoWindow.GetContent: String;
begin
  Result:=VarAsString(Exec(InfoWindowMethod_GetContent));
end;

function TInfoWindow.GetContentNode: TNode;
var
  Node: OleVariant;
begin
  Result:=nil;
  Node:=Exec(InfoWindowMethod_GetContent);
  if VarIsValidDispatch(Node) then
  begin
    if assigned(FNode)
      then FNode.Disp:=Node
      else FNode:=TNode(CreateObjectWrapper(TNode,Node));
    Result:=FNode;
  end;
end;

function TInfoWindow.GetPosition: TLatLng;
var
  Position: OleVariant;
begin
  Result:=nil;
  Position:=Exec(InfoWindowMethod_GetPosition);
  if VarIsValidDispatch(Position) then
  begin
    if assigned(FPosition)
      then FPosition.Disp:=Position
      else FPosition:=TLatLng(CreateObjectWrapper(Script,TLatLng,Position));
    Result:=FPosition;
  end;
end;

function TInfoWindow.GetSize: TSize;
var
  Size: OleVariant;
begin
  Result:=nil;
  Size:=Exec(InfoWindowMethod_GetSize);
  if VarIsValidDispatch(Size) then
  begin
    if assigned(FSize)
      then FSize.Disp:=Size
      else FSize:=TSize(CreateObjectWrapper(Script,TSize,Size));
    Result:=FSize;
  end;
end;

function TInfoWindow.GetZIndex: Integer;
begin
  Result:=VarAsInteger(Exec(InfoWindowMethod_GetZIndex));
end;

//------------------------------------------------------------------------------
//TInfoWindowOptions

constructor TInfoWindowOptions.Create;
begin
  inherited;
end;

constructor TInfoWindowOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TInfoWindowOptions.Destroy;
begin
  FContentNode.Free;
  FPixelOffset.Free;
  FPosition.Free;
  inherited;
end;

class function TInfoWindowOptions.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[InfoWindowOptionsProp_Content,
                                 InfoWindowOptionsProp_DisableAutoPan,
                                 InfoWindowOptionsProp_MaxWidth]);
end;

function TInfoWindowOptions.GetContent: String;
begin
  Result:=GetPropAsString(InfoWindowOptionsProp_Content);
end;

procedure TInfoWindowOptions.SetContent(Value: String);
begin
  SetProp(InfoWindowOptionsProp_Content,Value,true);
end;

function TInfoWindowOptions.GetContentNode: TNode;
begin
  Result:=TNode(GetPropAsObject(InfoWindowOptionsProp_Content,TNode,FContentNode));
end;

procedure TInfoWindowOptions.SetContentNode(Value: TNode);
begin
  SetProp(InfoWindowOptionsProp_Content,Value,true);
end;

function TInfoWindowOptions.GetDisableAutoPan: Boolean;
begin
  Result:=GetPropAsBoolean(InfoWindowOptionsProp_DisableAutoPan);
end;

procedure TInfoWindowOptions.SetDisableAutoPan(Value: Boolean);
begin
  SetProp(InfoWindowOptionsProp_DisableAutoPan,Value,true);
end;

function TInfoWindowOptions.GetMaxWidth: Integer;
begin
  Result:=GetPropAsInteger(InfoWindowOptionsProp_MaxWidth);
end;

procedure TInfoWindowOptions.SetMaxWidth(Value: Integer);
begin
  SetProp(InfoWindowOptionsProp_MaxWidth,Value,true);
end;

function TInfoWindowOptions.GetPixelOffset: TSize;
begin
  Result:=TSize(GetPropAsObject(InfoWindowOptionsProp_PixelOffset,TSize,FPixelOffset));
end;

procedure TInfoWindowOptions.SetPixelOffset(Value: TSize);
begin
  SetProp(InfoWindowOptionsProp_PixelOffset,Value,true);
end;

function TInfoWindowOptions.GetPosition: TLatLng;
begin
  Result:=TLatLng(GetpropAsObject(InfoWindowOptionsProp_Position,TLatLng,FPosition));
end;

procedure TInfoWindowOptions.SetPosition(Value: TLatLng);
begin
  SetProp(InfoWindowOptionsProp_Position,Value,true);
end;

function TInfoWindowOptions.GetZIndex: Integer;
begin
  Result:=GetPropAsInteger(InfoWindowOptionsProp_ZIndex);
end;

procedure TInfoWindowOptions.SetZIndex(Value: Integer);
begin
  SetProp(InfoWindowOptionsProp_ZIndex,Value,true);
end;

end.

