unit gmOverlaysMarker;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} BrowserTools,
  JScriptObjects, gmBase, gmMVC, gmMap, gmStreetView, gmEvents,
  gmAdditionalConstants, gmAdditionalMessages, HTMLObjects;

type
  TMarker = class;
  TMarkerImage = class;
  TMarkerOptions = class;
  TMarkerShape = class;

  TMarker = class(TMVCObject)
  private
    FIconImage: TMarkerImage;
    FMap: TMap;
    FPosition: TLatLng;
    FShadowImage: TMarkerImage;
    FShape: TMarkerShape;
    FStreetViewPanorama: TStreetViewPanorama;
    FOnClick: TMapEventEvent;
    FOnClickListener: TMapsEventListener;
    FOnClickFunction: TJScriptFunction;
    FOnClickableChanged: TNotifyEvent;
    FOnClickableChangedListener: TMapsEventListener;
    FOnClickableChangedFunction: TJScriptFunction;
    FOnCursorChanged: TNotifyEvent;
    FOnCursorChangedListener: TMapsEventListener;
    FOnCursorChangedFunction: TJScriptFunction;
    FOnDblClick: TMapEventEvent;
    FOnDblClickListener: TMapsEventListener;
    FOnDblClickFunction: TJScriptFunction;
    FOnDrag: TMapMouseEvent;
    FOnDragListener: TMapsEventListener;
    FOnDragFunction: TJScriptFunction;
    FOnDragEnd: TMapMouseEvent;
    FOnDragEndListener: TMapsEventListener;
    FOnDragEndFunction: TJScriptFunction;
    FOnDraggableChanged: TNotifyEvent;
    FOnDraggableChangedListener: TMapsEventListener;
    FOnDraggableChangedFunction: TJScriptFunction;
    FOnDragStart: TMapMouseEvent;
    FOnDragStartListener: TMapsEventListener;
    FOnDragStartFunction: TJScriptFunction;
    FOnFlatChanged: TNotifyEvent;
    FOnFlatChangedListener: TMapsEventListener;
    FOnFlatChangedFunction: TJScriptFunction;
    FOnIconChanged: TNotifyEvent;
    FOnIconChangedListener: TMapsEventListener;
    FOnIconChangedFunction: TJScriptFunction;
    FOnMouseDown: TMapEventEvent;
    FOnMouseDownListener: TMapsEventListener;
    FOnMouseDownFunction: TJScriptFunction;
    FOnMouseOut: TMapEventEvent;
    FOnMouseOutListener: TMapsEventListener;
    FOnMouseOutFunction: TJScriptFunction;
    FOnMouseOver: TMapEventEvent;
    FOnMouseOverListener: TMapsEventListener;
    FOnMouseOverFunction: TJScriptFunction;
    FOnMouseUp: TMapEventEvent;
    FOnMouseUpListener: TMapsEventListener;
    FOnMouseUpFunction: TJScriptFunction;
    FOnPositionChanged: TNotifyEvent;
    FOnPositionChangedListener: TMapsEventListener;
    FOnPositionChangedFunction: TJScriptFunction;
    FOnRightClick: TMapEventEvent;
    FOnRightClickListener: TMapsEventListener;
    FOnRightClickFunction: TJScriptFunction;
    FOnShadowChanged: TNotifyEvent;
    FOnShadowChangedListener: TMapsEventListener;
    FOnShadowChangedFunction: TJScriptFunction;
    FOnShapeChanged: TNotifyEvent;
    FOnShapeChangedListener: TMapsEventListener;
    FOnShapeChangedFunction: TJScriptFunction;
    FOnTitleChanged: TNotifyEvent;
    FOnTitleChangedListener: TMapsEventListener;
    FOnTitleChangedFunction: TJScriptFunction;
    FOnVisibleChanged: TNotifyEvent;
    FOnVisibleChangedListener: TMapsEventListener;
    FOnVisibleChangedFunction: TJScriptFunction;
    FOnZIndexChanged: TNotifyEvent;
    FOnZIndexChangedListener: TMapsEventListener;
    FOnZIndexChangedFunction: TJScriptFunction;
    procedure SetIconAsString(Value: String);
    procedure SetIconImage(Value: TMarkerImage);
    procedure SetMapAsTMap(Value: TMap);
    procedure SetShadowAsString(Value: String);
    procedure SetShadowImage(Value: TMarkerImage);
    procedure SetStreetViewPanorama(Value: TStreetViewPanorama);
    procedure SetOnClick(Value: TMapEventEvent);
    procedure SetOnClickableChanged(Value: TNotifyEvent);
    procedure SetOnCursorChanged(Value: TNotifyEvent);
    procedure SetOnDblClick(Value: TMapEventEvent);
    procedure SetOnDrag(Value: TMapMouseEvent);
    procedure SetOnDragEnd(Value: TMapMouseEvent);
    procedure SetOnDraggableChanged(Value: TNotifyEvent);
    procedure SetOnDragStart(Value: TMapMouseEvent);
    procedure SetOnFlatChanged(Value: TNotifyEvent);
    procedure SetOnIconChanged(Value: TNotifyEvent);
    procedure SetOnMouseDown(Value: TMapEventEvent);
    procedure SetOnMouseOut(Value: TMapEventEvent);
    procedure SetOnMouseOver(Value: TMapEventEvent);
    procedure SetOnMouseUp(Value: TMapEventEvent);
    procedure SetOnPositionChanged(Value: TNotifyEvent);
    procedure SetOnRightClick(Value: TMapEventEvent);
    procedure SetOnShadowChanged(Value: TNotifyEvent);
    procedure SetOnShapeChanged(Value: TNotifyEvent);
    procedure SetOnTitleChanged(Value: TNotifyEvent);
    procedure SetOnVisibleChanged(Value: TNotifyEvent);
    procedure SetOnZIndexChanged(Value: TNotifyEvent);
  protected
    procedure Map_changed;
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; Opts: TMarkerOptions): IDispatch; reintroduce; overload;
    //von JScript aufgerufene Event-Handler:
    procedure Click;
    procedure ClickableChanged;
    procedure CursorChanged;
    procedure DblClick;
    procedure Drag;
    procedure DragEnd;
    procedure DraggableChanged;
    procedure DragStart;
    procedure FlatChanged;
    procedure IconChanged;
    procedure MouseDown;
    procedure MouseOut;
    procedure MouseOver;
    procedure MouseUp;
    procedure PositionChanged;
    procedure RightClick;
    procedure ShadowChanged;
    procedure ShapeChanged;
    procedure TitleChanged;
    procedure VisibleChanged;
    procedure ZIndexChanged;
  public
    constructor Create(Opts: TMarkerOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TMarkerOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    procedure SetAnimation(Animation: Integer);
    procedure SetClickable(Flag: Boolean);
    procedure SetCross(Cross: String);                     //nicht dokumentiert
    procedure SetCursor(Cursor: String);
    procedure SetDraggable(Flag: Boolean);
    procedure SetFlat(Flag: Boolean);
    procedure SetIcon(Icon: String); overload;
    procedure SetIcon(Icon: TMarkerImage); overload;
    procedure SetMap(Map: TMap); overload;
    procedure SetMap(Map: TStreetViewPanorama); overload;
    procedure SetOptions(Options: TMarkerOptions); reintroduce;
    procedure SetPosition(LatLng: TLatLng);
    procedure SetShadow(Shadow: String); overload;
    procedure SetShadow(Shadow: TMarkerImage); overload;
    procedure SetShape(Shape: TMarkerShape);
    //procedure SetTarget(const c: OleVariant);            //nicht dokumentiert
    procedure SetTitle(Title: String);
    procedure SetVisible(Visible: Boolean);
    procedure SetZIndex(ZIndex: Integer);
    function GetAnimation: Integer;
    function GetClickable: Boolean;
    function GetCross: String;                             //nicht dokumentiert
    function GetCursor: String;
    function GetDraggable: Boolean;
    function GetFlat: Boolean;
    //Die Original-Methode "getIcon" kann sowohl einen String, als auch ein
    //MarkerImage-Objekt liefern. Deshalb liefert die Delphi-Methode
    //"GetIcon" den Wert als String (im Falle eines Objektes "[object Object]")
    //und die zustzliche Methode "GetIconImage" das MarkerImage-Objekt
    //(falls vorhanden):
    function GetIcon: String;
    function GetIconImage: TMarkerImage;
    //Die Original-Methode "getMap" kann sowohl einen Map-Objekt, als auch ein
    //StreetViewPanorama-Objekt liefern. Deshalb liefert die Delphi-Methode
    //"GetMap" nur dann einen Wert, wenn der Marker einem Map-Objekt
    //zugewiesen wurde.
    function GetMap: TMap;
    function GetPosition: TLatLng;
    //Die Original-Methode "getShadow" kann sowohl einen String, als auch ein
    //MarkerImage-Objekt enthalten. Deshalb liefert die Delphi-Methode
    //"GetShadow" den Wert als String (im Falle eines Objektes "[object Object]")
    //und die zustzliche Eigenschaft "GetShadowImage" das MarkerImage-Objekt
    //(falls vorhanden):
    function GetShadow: String;
    function GetShadowImage: TMarkerImage;
    function GetShape: TMarkerShape;
    //function GetTarget: OleVariant;                        //nicht dokumentiert
    //Zustzliche Methode "GetStreetViewPanorama": Sie liefert nur dann einen
    //Wert, wenn der Marker einem StreetViewPanorama-Objekt zugewiesen wurde.
    //Siehe Methode "GetMap".
    function GetStreetViewPanorama: TStreetViewPanorama;
    function GetTitle: String;
    function GetVisible: Boolean;
    function GetZIndex: Integer;
    //Objektmethoden zustzlich als Eigenschaften:
    property Animation: Integer read GetAnimation write SetAnimation;
    property Clickable: Boolean read GetClickable write SetClickable;
    property Cross: String read GetCross write SetCross;
    property Cursor: String read GetCursor write SetCursor;
    property Draggable: Boolean read GetDraggable write SetDraggable;
    property Flat: Boolean read GetFlat write SetFlat;
    property Icon: String read GetIcon write SetIconAsString;
    property IconImage: TMarkerImage read GetIconImage write SetIconImage;
    property Map: TMap read GetMap write SetMapAsTMap;
    property Position: TLatLng read GetPosition write SetPosition;
    property Shadow: String read GetShadow write SetShadowAsString;
    property ShadowImage: TMarkerImage read GetShadowImage write SetShadowImage;
    property Shape: TMarkerShape read GetShape write SetShape;
    property StreetViewPanorama: TStreetViewPanorama read GetStreetViewPanorama write SetStreetViewPanorama;
    //property Target: OleVariant read GetTarget write SetTarget;
    property Title: String read GetTitle write SetTitle;
    property Visible: Boolean read GetVisible write SetVisible;
    property ZIndex: Integer read GetZIndex write SetZIndex;
    //Dieses Ereignis wird ausgelst, wenn auf das Markierungssymbol geklickt wurde:
    property OnClick: TMapEventEvent read FOnClick write SetOnClick;
    //Dieses Ereignis wird ausgelst, wenn sich die clickable-Eigenschaft der Markierung ndert:
    property OnClickableChanged: TNotifyEvent read FOnClickableChanged write SetOnClickableChanged;
    //Dieses Ereignis wird ausgelst, wenn sich die cursor-Eigenschaft der Markierung ndert:
    property OnCursorChanged: TNotifyEvent read FOnCursorChanged write SetOnCursorChanged;
    //Dieses Ereignis wird ausgelst, wenn doppelt auf das Markierungssymbol geklickt wurde:
    property OnDblClick: TMapEventEvent read FOnDblClick write SetOnDblClick;
    //Dieses Ereignis wird wiederholt ausgelst, whrend der Nutzer die Markierung verschiebt:
    property OnDrag: TMapMouseEvent read FOnDrag write SetOnDrag;
    //Dieses Ereignis wird ausgelst, wenn der Nutzer das Verschieben der Markierung beendet:
    property OnDragEnd: TMapMouseEvent read FOnDragEnd write SetOnDragEnd;
    //Dieses Ereignis wird ausgelst, wenn sich die draggable-Eigenschaft der Markierung ndert:
    property OnDraggableChanged: TNotifyEvent read FOnDraggableChanged write SetOnDraggableChanged;
    //Dieses Ereignis wird ausgelst, wenn der Nutzer beginnt, die Markierung zu verschieben:
    property OnDragStart: TMapMouseEvent read FOnDragStart write SetOnDragStart;
    //Dieses Ereignis wird ausgelst, wenn sich die flat-Eigenschaft der Markierung ndert:
    property OnFlatChanged: TNotifyEvent read FOnFlatChanged write SetOnFlatChanged;
    //Dieses Ereignis wird ausgelst, wenn sich die icon-Eigenschaft der Markierung ndert:
    property OnIconChanged: TNotifyEvent read FOnIconChanged write SetOnIconChanged;
    //Dieses Ereignis wird ausgelst, wenn das DOM-mousedown-Ereignis fr das Markierungssymbol
    //eintritt:
    property OnMouseDown: TMapEventEvent read FOnMouseDown write SetOnMouseDown;
    //Dieses Ereignis wird ausgelst, wenn die Maus den Bereich des Markierungssymbols verlsst:
    property OnMouseOut: TMapEventEvent read FOnMouseOut write SetOnMouseOut;
    //Dieses Ereignis wird ausgelst, wenn die Maus in den Bereich des Markierungssymbols
    //hineinbewegt wird:
    property OnMouseOver: TMapEventEvent read FOnMouseOver write SetOnMouseOver;
    //Dieses Ereignis wird ausgelst, wenn das DOM-mouseup-Ereignis fr die Markierung eintritt:
    property OnMouseUp: TMapEventEvent read FOnMouseUp write SetOnMouseUp;
    //Dieses Ereignis wird ausgelst, wenn sich die position-Eigenschaft der Markierung ndert:
    property OnPositionChanged: TNotifyEvent read FOnPositionChanged write SetOnPositionChanged;
    //Dieses Ereignis wird ausgelst, wenn mit der rechten Maustaste auf die Markierung geklickt wird:
    property OnRightClick: TMapEventEvent read FOnRightClick write SetOnRightClick;
    //Dieses Ereignis wird ausgelst, wenn sich die Eigenschaft Schatten der Markierung ndert:
    property OnShadowChanged: TNotifyEvent read FOnShadowChanged write SetOnShadowChanged;
    //Dieses Ereignis wird ausgelst, wenn sich die Eigenschaft Form der Markierung ndert:
    property OnShapeChanged: TNotifyEvent read FOnShapeChanged write SetOnShapeChanged;
    //Dieses Ereignis wird ausgelst, wenn sich die title-Eigenschaft der Markierung ndert:
    property OnTitleChanged: TNotifyEvent read FOnTitleChanged write SetOnTitleChanged;
    //Dieses Ereignis wird ausgelst, wenn sich die visible-Eigenschaft der Markierung ndert:
    property OnVisibleChanged: TNotifyEvent read FOnVisibleChanged write SetOnVisibleChanged;
    //Dieses Ereignis wird ausgelst, wenn sich die zIndex-Eigenschaft der Markierung ndert:
    property OnZIndexChanged: TNotifyEvent read FOnZIndexChanged write SetOnZIndexChanged;
  end;

  TMarkerImage = class(TCustomJScriptObject)
  protected
    class function New(Script: TCustomScript; const URL: String; Size: TSize; Origin: TPoint; Anchor: TPoint; ScaledSize: TSize): IDispatch; reintroduce;
  public
    constructor Create(const URL: String; Size: TSize = nil; Origin: TPoint = nil; Anchor: TPoint = nil; ScaledSize: TSize = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; const URL: String; Size: TSize = nil; Origin: TPoint = nil; Anchor: TPoint = nil; ScaledSize: TSize = nil); reintroduce; overload;
  end;

  TMarkerOptions = class(TCustomJScriptObject)
  private
    FIconImage: TMarkerImage;
    FMap: TMap;
    FPosition: TLatLng;
    FShadowImage: TMarkerImage;
    FShape: TMarkerShape;
    FStreetViewPanorama: TStreetViewPanorama;
    procedure SetAnimation(Value: Integer);
    procedure SetClickable(Value: Boolean);
    procedure SetCross(Value: String);
    procedure SetCursor(Value: String);
    procedure SetDraggable(Value: Boolean);
    procedure SetFlat(Value: Boolean);
    procedure SetIcon(Value: String);
    procedure SetIconImage(Value: TMarkerImage);
    procedure SetMap(Value: TMap);
    procedure SetOptimized(Value: Boolean);
    procedure SetPosition(Value: TLatLng);
    procedure SetRaiseOnDrag(Value: Boolean);
    procedure SetShadow(Value: String);
    procedure SetShadowImage(Value: TMarkerImage);
    procedure SetShape(Value: TMarkerShape);
    procedure SetStreetViewPanorama(Value: TStreetViewPanorama);
    procedure SetTitle(Value: String);
    procedure SetVisible(Value: Boolean);
    procedure SetZIndex(Value: Integer);
    function GetAnimation: Integer;
    function GetClickable: Boolean;
    function GetCross: String;
    function GetCursor: String;
    function GetDraggable: Boolean;
    function GetFlat: Boolean;
    function GetIcon: String;
    function GetIconImage: TMarkerImage;
    function GetMap: TMap;
    function GetOptimized: Boolean;
    function GetPosition: TLatLng;
    function GetRaiseOnDrag: Boolean;
    function GetShadow: String;
    function GetShadowImage: TMarkerImage;
    function GetShape: TMarkerShape;
    function GetStreetViewPanorama: TStreetViewPanorama;
    function GetTitle: String;
    function GetVisible: Boolean;
    function GetZIndex: Integer;
  public
    constructor Create; overload; override;
    constructor Create(Position: TLatLng); reintroduce; overload;
    constructor Create(Script: TCustomScript); overload; override;
    constructor Create(Script: TCustomScript; Position: TLatLng); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Which animation to play when marker is added to a map:
    property Animation: Integer read GetAnimation write SetAnimation;
    //If true, the marker receives mouse and touch events. Default value is true:
    property Clickable: Boolean read GetClickable write SetClickable;
    property Cross: String read GetCross write SetCross;                  //???
    //Mouse cursor to show on hover:
    property Cursor: String read GetCursor write SetCursor;
    //If true, the marker can be dragged. Default value is false:
    property Draggable: Boolean read GetDraggable write SetDraggable;
    //If true, the marker shadow will not be displayed:
    property Flat: Boolean read GetFlat write SetFlat;
    //Die Original-Eigenschaft "icon" kann sowohl einen String, als auch ein
    //MarkerImage-Objekt enthalten. Deshalb liefert die Delphi-Eigenschaft
    //"Icon" den Wert als String (im Falle eines Objektes "[object Object]")
    //und die zustzliche Eigenschaft "IconImage" das MarkerImage-Objekt
    //(falls vorhanden):
    //Icon for the foreground:
    property Icon: String read GetIcon write SetIcon;
    property IconImage: TMarkerImage read GetIconImage write SetIconImage;
    //Die Original-Eigenschaft "map" kann sowohl einen Map-Objekt, als auch ein
    //StreetViewPanorama-Objekt enthalten. Deshalb liest und setzt die Delphi-
    //Eigenschaft "Map" den Wert als TMap-Objekt und die zustzliche Eigenschaft
    //"ShadowImage" als TStreeViewPanorama-Objekt:
    //Map on which to display Marker:
    property Map: TMap read GetMap write SetMap;
    //Optimization renders many markers as a single static element. Optimized
    //rendering is enabled by default. Disable optimized rendering for animated
    //GIFs or PNGs, or when each marker must be rendered as a separate DOM
    //element (advanced usage only):
    property Optimized: Boolean read GetOptimized write SetOptimized;
    //Marker position. Required:
    property Position: TLatLng read GetPosition write SetPosition;
    //If false, disables raising and lowering the marker on drag. This option is
    //true by default:
    property RaiseOnDrag: Boolean read GetRaiseOnDrag write SetRaiseOnDrag;
    //Die Original-Eigenschaft "shadow" kann sowohl einen String, als auch ein
    //MarkerImage-Objekt enthalten. Deshalb liefert die Delphi-Eigenschaft
    //"Shadow" den Wert als String (im Falle eines Objektes "[object Object]")
    //und die zustzliche Eigenschaft "ShadowImage" das MarkerImage-Objekt
    //(falls vorhanden):
    //If false, disables raising and lowering the marker on drag. This option is
    //true by default:
    property Shadow: String read GetShadow write SetShadow;
    property ShadowImage: TMarkerImage read GetShadowImage write SetShadowImage;
    //Image map region definition used for drag/click:
    property Shape: TMarkerShape read GetShape write SetShape;
    //Zustzliche Eigenschaft "StreetViewPanorama": Siehe Eigenschaft "Map".
    property StreetViewPanorama: TStreetViewPanorama read GetStreetViewPanorama write SetStreetViewPanorama;
    //Rollover text:
    property Title: String read GetTitle write SetTitle;
    //If true, the marker is visible:
    property Visible: Boolean read GetVisible write SetVisible;
    //All markers are displayed on the map in order of their zIndex, with higher
    //values displaying in front of markers with lower values. By default,
    //markers are displayed according to their vertical position on screen, with
    //lower markers appearing in front of markers further up the screen:
    property ZIndex: Integer read GetZIndex write SetZIndex;
  end;

  TMarkerShape = class(TCustomJScriptObject)
  private
    FCoords: TIntegerArray;
    procedure SetCoords(Value: TIntegerArray);
    procedure SetType(Value: String);
    function GetCoords: TIntegerArray;
    function GetType: String;
  protected
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    //Aus Google Maps API:
    //http://code.google.com/intl/de/apis/maps/documentation/javascript/reference.html#MarkerShape
    //Das Format dieses Attributs hngt vom type-Wert ab und befolgt die
    //W3-Spezifikation fr AREA-coords, die unter
    //http://www.w3.org/TR/REC-html40/struct/objects.html#adef-coords abrufbar ist.
    //"coord" ist ein Array mit Ganzzahlwerten, mit denen die Pixelposition der
    //Form im Bezug auf die linke obere Ecke des Zielbildes angeben wird. Die
    //Koordinaten hngen wie folgt vom type-Wert ab:
    //- circ oder circle: coord wird in der Form [x1,y1,r] angegeben, wobei x1,y2
    //  die Koordinaten des Kreismittelpunktes sind und r der Radius des Kreises ist.
    //- poly oder polygon: coord wird in der Form [x1,y1,x2,y2...xn,yn] angegeben,
    //  wobei jedes x,y-Paar fr die Koordinaten eines Scheitelpunkts des Polygons steht.
    //- rect oder rectangle: coord wird in der Form [x1,y1,x2,y2] angegeben, wobei
    //  x1,y1 die Koordinaten der linken oberen Ecke des Rechtecks bilden und x2,y2
    //  die Koordinaten der rechten unteren Ecke.
    //Anmerkung: alt "Coord", ab Version 3.4 (8.2.2011) umbenannt in "Coords"
    //deprecated 'V3.4.: Use TMarkerShape.Coords';
    property Coord: TIntegerArray read GetCoords write SetCoords;    //deprecated;
    property Coords: TIntegerArray read GetCoords write SetCoords;
    property {$IFDEF AND_OP}&Type{$ELSE}Type_{$ENDIF}: String read GetType write SetType;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TMarker

constructor TMarker.Create(Opts: TMarkerOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TMarker.Create(Script: TCustomScript; Opts: TMarkerOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TMarker.Destroy;
begin
  //Niemals (siehe Bemerkung zu TMap)!
  //ClearInstanceListeners;                                //alle Events entfernen
  //Deshalb nur die selbst erstellten Listener entfernen:
  SetOnClick(nil);
  SetOnClickableChanged(nil);
  SetOnCursorChanged(nil);
  SetOnDblClick(nil);
  SetOnDrag(nil);
  SetOnDragEnd(nil);
  SetOnDraggableChanged(nil);
  SetOnDragStart(nil);
  SetOnFlatChanged(nil);
  SetOnIconChanged(nil);
  SetOnMouseDown(nil);
  SetOnMouseOut(nil);
  SetOnMouseOver(nil);
  SetOnMouseUp(nil);
  SetOnPositionChanged(nil);
  SetOnRightClick(nil);
  SetOnShadowChanged(nil);
  SetOnShapeChanged(nil);
  SetOnTitleChanged(nil);
  SetOnVisibleChanged(nil);
  SetOnZIndexChanged(nil);
  FOnClickFunction.Free;
  FOnClickableChangedFunction.Free;
  FOnCursorChangedFunction.Free;
  FOnDblClickFunction.Free;
  FOnDragFunction.Free;
  FOnDragEndFunction.Free;
  FOnDraggableChangedFunction.Free;
  FOnDragStartFunction.Free;
  FOnFlatChangedFunction.Free;
  FOnIconChangedFunction.Free;
  FOnMouseDownFunction.Free;
  FOnMouseOutFunction.Free;
  FOnMouseOverFunction.Free;
  FOnMouseUpFunction.Free;
  FOnPositionChangedFunction.Free;
  FOnRightClickFunction.Free;
  FOnShadowChangedFunction.Free;
  FOnShapeChangedFunction.Free;
  FOnTitleChangedFunction.Free;
  FOnVisibleChangedFunction.Free;
  FOnZIndexChangedFunction.Free;
  FIconImage.Free;
  FMap.Free;
  FPosition.Free;
  FShadowImage.Free;
  FShape.Free;
  FStreetViewPanorama.Free;
  inherited;
end;

class function TMarker.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MarkerMethod_GetAnimation,
                                 MarkerMethod_SetAnimation,
                                 MarkerMethod_GetClickable]);
end;

class function TMarker.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_Marker,true);
end;

class function TMarker.New(Script: TCustomScript; Opts: TMarkerOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_Marker,Opts.Disp,true);
end;

function TMarker.GetAnimation: Integer;
begin
  Result:=VarAsInteger(Exec(MarkerMethod_GetAnimation));
end;

procedure TMarker.SetAnimation(Animation: Integer);
begin
  if Animation>0
    then Exec(MarkerMethod_SetAnimation,[Animation])
    else Exec(MarkerMethod_SetAnimation,[Null]);
end;

function TMarker.GetClickable: Boolean;
begin
  Result:=Exec(MarkerMethod_GetClickable);
end;

procedure TMarker.SetClickable(Flag: Boolean);
begin
  Exec(MarkerMethod_SetClickable,[Flag]);
end;

function TMarker.GetCross: String;
begin
  Result:=Exec(MarkerMethod_GetCross);
end;

procedure TMarker.SetCross(Cross: String);
begin
  Exec(MarkerMethod_SetCross,[Cross]);
end;

function TMarker.GetCursor: String;
begin
  Result:=Exec(MarkerMethod_GetCursor);
end;

procedure TMarker.SetCursor(Cursor: String);
begin
  Exec(MarkerMethod_SetCursor,[Cursor]);
end;

function TMarker.GetDraggable: Boolean;
begin
  Result:=Exec(MarkerMethod_GetDraggable);
end;

procedure TMarker.SetDraggable(Flag: Boolean);
begin
  Exec(MarkerMethod_SetDraggable,[Flag]);
end;

function TMarker.GetFlat: Boolean;
begin
  Result:=Exec(MarkerMethod_GetFlat);
end;

procedure TMarker.SetFlat(Flag: Boolean);
begin
  Exec(MarkerMethod_SetFlat,[Flag]);
end;

function TMarker.GetIcon: String;
begin
  Result:=Exec(MarkerMethod_GetIcon);
end;

function TMarker.GetIconImage: TMarkerImage;
var
  IconImage: OleVariant;
begin
  Result:=nil;
  IconImage:=Exec(MarkerMethod_GetIcon);
  if VarIsValidDispatch(IconImage) then
  begin
    if assigned(FIconImage)
      then FIconImage.Disp:=IconImage
      else FIconImage:=TMarkerImage(CreateObjectWrapper(Script,TMarkerImage,IconImage));
    Result:=FIconImage;
  end;
end;

procedure TMarker.SetIcon(Icon: String);
begin
  Exec(MarkerMethod_SetIcon,[Icon]);
end;

procedure TMarker.SetIcon(Icon: TMarkerImage);
begin
  if assigned(Icon)
    then Exec(MarkerMethod_SetIcon,[Icon.Disp]);
end;

//zustzlich, da in lteren Delphi-Versionen Property-Methoden nicht berladen
//werden drfen: SetIcon(Icon: String), SetIcon(Icon: TMarkerImage)
procedure TMarker.SetIconAsString(Value: String);
begin
  SetIcon(Value);
end;

procedure TMarker.SetIconImage(Value: TMarkerImage);
begin
  SetIcon(Value);
end;

function TMarker.GetMap: TMap;
var
  Map: OleVariant;
begin
  Result:=nil;
  Map:=Exec(MarkerMethod_GetMap);
  if VarIsValidDispatch(Map) then
  begin
    if assigned(FMap)
      then FMap.Disp:=Map
      else FMap:=TMap(CreateObjectWrapper(Script,TMap,Map));
    Result:=FMap;
  end;
end;

procedure TMarker.SetMap(Map: TMap);
begin
  Exec(MarkerMethod_SetMap,Map);
end;

procedure TMarker.SetMap(Map: TStreetViewPanorama);
begin
  Exec(MarkerMethod_SetMap,Map);
end;

//zustzlich, da in lteren Delphi-Versionen Property-Methoden nicht berladen
//werden drfen: SetMap(Map: TMap), SetMap(Map: TStreetViewPanorama)
procedure TMarker.SetMapAsTMap(Value: TMap);
begin
  SetMap(Value);
end;

procedure TMarker.SetStreetViewPanorama(Value: TStreetViewPanorama);
begin
  SetMap(Value);
end;

procedure TMarker.SetOptions(Options: TMarkerOptions);
begin
  if assigned(Options)
    then Exec(MarkerMethod_SetOptions,[Options.Disp]);
end;

function TMarker.GetPosition: TLatLng;
var
  Position: OleVariant;
begin
  Result:=nil;
  Position:=Exec(MarkerMethod_GetPosition);
  if VarIsValidDispatch(Position) then
  begin
    if assigned(FPosition)
      then FPosition.Disp:=Position
      else FPosition:=TLatLng(CreateObjectWrapper(Script,TLatLng,Position));
    Result:=FPosition;
  end;
end;

procedure TMarker.SetPosition(LatLng: TLatLng);
begin
  if assigned(LatLng)
    then Exec(MarkerMethod_SetPosition,[LatLng.Disp]);
end;

function TMarker.GetShadow: String;
begin
  Result:=Exec(MarkerMethod_GetShadow);
end;

function TMarker.GetShadowImage: TMarkerImage;
var
  ShadowImage: OleVariant;
begin
  Result:=nil;
  ShadowImage:=Exec(MarkerMethod_GetShadow);
  if VarIsValidDispatch(ShadowImage) then
  begin
    if assigned(FShadowImage)
      then FShadowImage.Disp:=ShadowImage
      else FShadowImage:=TMarkerImage(CreateObjectWrapper(Script,TMarkerImage,ShadowImage));
    Result:=FShadowImage;
  end;
end;

procedure TMarker.SetShadow(Shadow: String);
begin
  Exec(MarkerMethod_SetShadow,[Shadow]);
end;

procedure TMarker.SetShadow(Shadow: TMarkerImage);
begin
  if assigned(Shadow)
    then Exec(MarkerMethod_SetShadow,[Shadow.Disp]);
end;

//zustzlich, da in lteren Delphi-Versionen Property-Methoden nicht berladen
//werden drfen: SetShadow(Shadow: String), SetShadow(Shadow: TMarkerImage)
procedure TMarker.SetShadowAsString(Value: String);
begin
  SetShadow(Value);
end;

procedure TMarker.SetShadowImage(Value: TMarkerImage);
begin
  SetShadow(Value);
end;

function TMarker.GetShape: TMarkerShape;
var
  Shape: OleVariant;
begin
  Result:=nil;
  Shape:=Exec(MarkerMethod_GetShape);
  if VarIsValidDispatch(Shape) then
  begin
    if assigned(FShape)
      then FShape.Disp:=Shape
      else FShape:=TMarkerShape(CreateObjectWrapper(Script,TMarkerShape,Shape));
    Result:=FShape;
  end;
end;

procedure TMarker.SetShape(Shape: TMarkerShape);
begin
  Exec(MarkerMethod_SetShape,Shape);
end;

function TMarker.GetStreetViewPanorama: TStreetViewPanorama;
var
  StreetViewPanorama: OleVariant;
begin
  Result:=nil;
  StreetViewPanorama:=Exec(MarkerMethod_GetMap);
  if VarIsValidDispatch(StreetViewPanorama) then
  begin
    if assigned(FStreetViewPanorama)
      then FStreetViewPanorama.Disp:=StreetViewPanorama
      else FStreetViewPanorama:=TStreetViewPanorama(CreateObjectWrapper(Script,
                                                                        TStreetViewPanorama,
                                                                        StreetViewPanorama));
    Result:=FStreetViewPanorama;
  end;
end;
{
function TMarker.GetTarget: OleVariant;
begin
  Result:=Exec(MarkerMethod_GetTarget);
end;

procedure TMarker.SetTarget(c: OleVariant);
begin
  Exec(MarkerMethod_SetTarget,c);
end;
}
function TMarker.GetTitle: String;
begin
  Result:=Exec(MarkerMethod_GetTitle);
end;

procedure TMarker.SetTitle(Title: String);
begin
  Exec(MarkerMethod_SetTitle,[Title]);
end;

function TMarker.GetVisible: Boolean;
begin
  Result:=Exec(MarkerMethod_GetVisible);
end;

procedure TMarker.SetVisible(Visible: Boolean);
begin
  Exec(MarkerMethod_SetVisible,[Visible]);
end;

function TMarker.GetZIndex: Integer;
begin
  Result:=Exec(MarkerMethod_GetZIndex);
end;

procedure TMarker.SetZIndex(ZIndex: Integer);
begin
  Exec(MarkerMethod_SetZIndex,[ZIndex]);
end;

procedure TMarker.Map_changed;
begin
  Exec(MarkerMethod_Map_changed);
end;

procedure TMarker.Click;
begin
  CallEvent(FOnClick,FOnClickFunction.Arguments[0]);
end;

procedure TMarker.ClickableChanged;
begin
  CallEvent(FOnClickableChanged);
end;

procedure TMarker.CursorChanged;
begin
  CallEvent(FOnCursorChanged);
end;

procedure TMarker.DblClick;
begin
  CallEvent(FOnDblClick,FOnDblClickFunction.Arguments[0]);
end;

procedure TMarker.Drag;
begin
  CallEvent(FOnDrag,FOnDragFunction.Arguments[0]);
end;

procedure TMarker.DragEnd;
begin
  CallEvent(FOnDragEnd,FOnDragEndFunction.Arguments[0]);
end;

procedure TMarker.DraggableChanged;
begin
  CallEvent(FOnDraggableChanged);
end;

procedure TMarker.DragStart;
begin
  CallEvent(FOnDragStart,FOnDragStartFunction.Arguments[0]);
end;

procedure TMarker.FlatChanged;
begin
  CallEvent(FOnFlatChanged);
end;

procedure TMarker.IconChanged;
begin
  CallEvent(FOnIconChanged);
end;

procedure TMarker.MouseDown;
begin
  CallEvent(FOnMouseDown,FOnMouseDownFunction.Arguments[0]);
end;

procedure TMarker.MouseOut;
begin
  CallEvent(FOnMouseOut,FOnMouseOutFunction.Arguments[0]);
end;

procedure TMarker.MouseOver;
begin
  CallEvent(FOnMouseOver,FOnMouseOverFunction.Arguments[0]);
end;

procedure TMarker.MouseUp;
begin
  CallEvent(FOnMouseUp,FOnMouseUpFunction.Arguments[0]);
end;

procedure TMarker.PositionChanged;
begin
  CallEvent(FOnPositionChanged);
end;

procedure TMarker.RightClick;
begin
  CallEvent(FOnRightClick,FOnRightClickFunction.Arguments[0]);
end;

procedure TMarker.ShadowChanged;
begin
  CallEvent(FOnShadowChanged);
end;

procedure TMarker.ShapeChanged;
begin
  CallEvent(FOnShapeChanged);
end;

procedure TMarker.TitleChanged;
begin
  CallEvent(FOnTitleChanged);
end;

procedure TMarker.VisibleChanged;
begin
  CallEvent(FOnVisibleChanged);
end;

procedure TMarker.ZIndexChanged;
begin
  CallEvent(FOnZIndexChanged);
end;

procedure TMarker.SetOnClick(Value: TMapEventEvent);
begin
  FOnClick:=Value;
  RemoveListener(FOnClickListener);
  if assigned(FOnClick)
    then AddListener(MarkerEvent_Click,FOnClickListener,FOnClickFunction,Click);
end;

procedure TMarker.SetOnClickableChanged(Value: TNotifyEvent);
begin
  FOnClickableChanged:=Value;
  RemoveListener(FOnClickableChangedListener);
  if assigned(FOnClickableChanged)
    then AddListener(MarkerEvent_ClickableChanged,FOnClickableChangedListener,
                     FOnClickableChangedFunction,ClickableChanged);
end;

procedure TMarker.SetOnCursorChanged(Value: TNotifyEvent);
begin
  FOnCursorChanged:=Value;
  RemoveListener(FOnCursorChangedListener);
  if assigned(FOnCursorChanged)
    then AddListener(MarkerEvent_CursorChanged,FOnCursorChangedListener,
                     FOnCursorChangedFunction,CursorChanged);
end;

procedure TMarker.SetOnDblClick(Value: TMapEventEvent);
begin
  FOnDblClick:=Value;
  RemoveListener(FOnDblClickListener);
  if assigned(FOnDblClick)
    then AddListener(MarkerEvent_DblClick,FOnDblClickListener,
                     FOnDblClickFunction,DblClick);
end;

procedure TMarker.SetOnDrag(Value: TMapMouseEvent);
begin
  FOnDrag:=Value;
  RemoveListener(FOnDragListener);
  if assigned(FOnDrag)
    then AddListener(MarkerEvent_Drag,FOnDragListener,FOnDragFunction,Drag);
end;

procedure TMarker.SetOnDragEnd(Value: TMapMouseEvent);
begin
  FOnDragEnd:=Value;
  RemoveListener(FOnDragEndListener);
  if assigned(FOnDragEnd)
    then AddListener(MarkerEvent_DragEnd,FOnDragEndListener,FOnDragEndFunction,DragEnd);
end;

procedure TMarker.SetOnDraggableChanged(Value: TNotifyEvent);
begin
  FOnDraggableChanged:=Value;
  RemoveListener(FOnDraggableChangedListener);
  if assigned(FOnDraggableChanged)
    then AddListener(MarkerEvent_DraggableChanged,FOnDraggableChangedListener,
                     FOnDraggableChangedFunction,DraggableChanged);
end;

procedure TMarker.SetOnDragStart(Value: TMapMouseEvent);
begin
  FOnDragStart:=Value;
  RemoveListener(FOnDragStartListener);
  if assigned(FOnDragStart)
    then AddListener(MarkerEvent_DragStart,FOnDragStartListener,
                     FOnDragStartFunction,DragStart);
end;

procedure TMarker.SetOnFlatChanged(Value: TNotifyEvent);
begin
  FOnFlatChanged:=Value;
  RemoveListener(FOnFlatChangedListener);
  if assigned(FOnFlatChanged)
    then AddListener(MarkerEvent_FlatChanged,FOnFlatChangedListener,
                     FOnFlatChangedFunction,FlatChanged);
end;

procedure TMarker.SetOnIconChanged(Value: TNotifyEvent);
begin
  FOnIconChanged:=Value;
  RemoveListener(FOnIconChangedListener);
  if assigned(FOnIconChanged)
    then AddListener(MarkerEvent_IconChanged,FOnIconChangedListener,
                     FOnIconChangedFunction,IconChanged);
end;

procedure TMarker.SetOnMouseDown(Value: TMapEventEvent);
begin
  FOnMouseDown:=Value;
  RemoveListener(FOnMouseDownListener);
  if assigned(FOnMouseDown)
    then AddListener(MarkerEvent_MouseDown,FOnMouseDownListener,
                     FOnMouseDownFunction,MouseDown);
end;

procedure TMarker.SetOnMouseOut(Value: TMapEventEvent);
begin
  FOnMouseOut:=Value;
  RemoveListener(FOnMouseOutListener);
  if assigned(FOnMouseOut)
    then AddListener(MarkerEvent_MouseOut,FOnMouseOutListener,
                     FOnMouseOutFunction,MouseOut);
end;

procedure TMarker.SetOnMouseOver(Value: TMapEventEvent);
begin
  FOnMouseOver:=Value;
  RemoveListener(FOnMouseOverListener);
  if assigned(FOnMouseOver)
    then AddListener(MarkerEvent_MouseOver,FOnMouseOverListener,
                     FOnMouseOverFunction,MouseOver);
end;

procedure TMarker.SetOnMouseUp(Value: TMapEventEvent);
begin
  FOnMouseUp:=Value;
  RemoveListener(FOnMouseUpListener);
  if assigned(FOnMouseUp)
    then AddListener(MarkerEvent_MouseUp,FOnMouseUpListener,
                     FOnMouseUpFunction,MouseUp);
end;

procedure TMarker.SetOnPositionChanged(Value: TNotifyEvent);
begin
  FOnPositionChanged:=Value;
  RemoveListener(FOnPositionChangedListener);
  if assigned(FOnPositionChanged)
    then AddListener(MarkerEvent_PositionChanged,FOnPositionChangedListener,
                     FOnPositionChangedFunction,PositionChanged);
end;

procedure TMarker.SetOnRightClick(Value: TMapEventEvent);
begin
  FOnRightClick:=Value;
  RemoveListener(FOnRightClickListener);
  if assigned(FOnRightClick)
    then AddListener(MarkerEvent_RightClick,FOnRightClickListener,
                     FOnRightClickFunction,RightClick);
end;

procedure TMarker.SetOnShadowChanged(Value: TNotifyEvent);
begin
  FOnShadowChanged:=Value;
  RemoveListener(FOnShadowChangedListener);
  if assigned(FOnShadowChanged)
    then AddListener(MarkerEvent_ShadowChanged,FOnShadowChangedListener,
                     FOnShadowChangedFunction,ShadowChanged);
end;

procedure TMarker.SetOnShapeChanged(Value: TNotifyEvent);
begin
  FOnShapeChanged:=Value;
  RemoveListener(FOnShapeChangedListener);
  if assigned(FOnShapeChanged)
    then AddListener(MarkerEvent_ShapeChanged,FOnShapeChangedListener,
                     FOnShapeChangedFunction,ShapeChanged);
end;

procedure TMarker.SetOnTitleChanged(Value: TNotifyEvent);
begin
  FOnTitleChanged:=Value;
  RemoveListener(FOnTitleChangedListener);
  if assigned(FOnTitleChanged)
    then AddListener(MarkerEvent_TitleChanged,FOnTitleChangedListener,
                     FOnTitleChangedFunction,TitleChanged);
end;

procedure TMarker.SetOnVisibleChanged(Value: TNotifyEvent);
begin
  FOnVisibleChanged:=Value;
  RemoveListener(FOnVisibleChangedListener);
  if assigned(FOnVisibleChanged)
    then AddListener(MarkerEvent_VisibleChanged,FOnVisibleChangedListener,
                     FOnVisibleChangedFunction,VisibleChanged);
end;

procedure TMarker.SetOnZIndexChanged(Value: TNotifyEvent);
begin
  FOnZIndexChanged:=Value;
  RemoveListener(FOnZIndexChangedListener);
  if assigned(FOnZIndexChanged)
    then AddListener(MarkerEvent_ZIndexChanged,FOnZIndexChangedListener,
                     FOnZIndexChangedFunction,ZIndexChanged);
end;

//------------------------------------------------------------------------------
//TMarkerImage

constructor TMarkerImage.Create(const URL: String; Size: TSize; Origin: TPoint;
                                Anchor: TPoint; ScaledSize: TSize);
begin
  Create(JScriptObjects.Script,URL,Size,Origin,Anchor,ScaledSize);
end;

constructor TMarkerImage.Create(Script: TCustomScript; const URL: String; Size: TSize;
                                Origin: TPoint; Anchor: TPoint; ScaledSize: TSize);
begin
  inherited Create(Script,New(Script,URL,Size,Origin,Anchor,ScaledSize));
end;

class function TMarkerImage.New(Script: TCustomScript; const URL: String; Size: TSize;
                                Origin: TPoint; Anchor: TPoint; ScaledSize: TSize): IDispatch;
var
  SizeDisp, OriginDisp, AnchorDisp, ScaledSizeDisp: IDispatch;
begin
  CheckMaps(Script);
  if assigned(Size)
    then SizeDisp:=Size.Disp
    else SizeDisp:=nil;
  if assigned(Origin)
    then OriginDisp:=Origin.Disp
    else OriginDisp:=nil;
  if assigned(Anchor)
    then AnchorDisp:=Anchor.Disp
    else AnchorDisp:=nil;
  if assigned(ScaledSize)
    then ScaledSizeDisp:=ScaledSize.Disp
    else ScaledSizeDisp:=nil;
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_MarkerImage,
                              {$IFDEF CAN_OVERLOAD}
                              [URL,SizeDisp,OriginDisp,AnchorDisp,ScaledSizeDisp]
                              {$ELSE}
                              URL,SizeDisp,OriginDisp,AnchorDisp,ScaledSizeDisp
                              {$ENDIF}
                              ,true);
end;

//------------------------------------------------------------------------------
//TMarkerOptions

constructor TMarkerOptions.Create;
begin
  inherited;
end;

constructor TMarkerOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

constructor TMarkerOptions.Create(Position: TLatLng);
begin
  Create(JScriptObjects.Script,Position);
end;

constructor TMarkerOptions.Create(Script: TCustomScript; Position: TLatLng);
begin
  inherited Create(Script);
  SetPosition(Position);
end;

destructor TMarkerOptions.Destroy;
begin
  FIconImage.Free;
  FMap.Free;
  FPosition.Free;
  FShadowImage.Free;
  FShape.Free;
  FStreetViewPanorama.Free;
  inherited;
end;

class function TMarkerOptions.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MarkerOptionsProp_Clickable,
                                 MarkerOptionsProp_Cursor,
                                 MarkerOptionsProp_Draggable]);
end;

function TMarkerOptions.GetAnimation: Integer;
begin
  Result:=GetPropAsInteger(MarkerOptionsProp_Animation);
end;

procedure TMarkerOptions.SetAnimation(Value: Integer);
begin
  SetProp(MarkerOptionsProp_Animation,Value,true);
end;

function TMarkerOptions.GetClickable: Boolean;
begin
  Result:=GetPropAsBoolean(MarkerOptionsProp_Clickable,true);
end;

procedure TMarkerOptions.SetClickable(Value: Boolean);
begin
  SetProp(MarkerOptionsProp_Clickable,Value,true);
end;

function TMarkerOptions.GetCross: String;
begin
  Result:=GetPropAsString(MarkerOptionsProp_Cross);
end;

procedure TMarkerOptions.SetCross(Value: String);
begin
  SetProp(MarkerOptionsProp_Cross,Value,true);
end;

function TMarkerOptions.GetCursor: String;
begin
  Result:=GetPropAsString(MarkerOptionsProp_Cursor);
end;

procedure TMarkerOptions.SetCursor(Value: String);
begin
  SetProp(MarkerOptionsProp_Cursor,Value,true);
end;

function TMarkerOptions.GetDraggable: Boolean;
begin
  Result:=GetPropAsBoolean(MarkerOptionsProp_Draggable);
end;

procedure TMarkerOptions.SetDraggable(Value: Boolean);
begin
  SetProp(MarkerOptionsProp_Draggable,Value,true);
end;

function TMarkerOptions.GetFlat: Boolean;
begin
  Result:=GetPropAsBoolean(MarkerOptionsProp_Flat);
end;

procedure TMarkerOptions.SetFlat(Value: Boolean);
begin
  SetProp(MarkerOptionsProp_Flat,Value,true);
end;

function TMarkerOptions.GetIcon: String;
begin
  Result:=GetPropAsString(MarkerOptionsProp_Icon);
end;

procedure TMarkerOptions.SetIcon(Value: String);
begin
  SetProp(MarkerOptionsProp_Icon,Value,true);
end;

function TMarkerOptions.GetIconImage: TMarkerImage;
begin
  Result:=TMarkerImage(GetPropAsObject(MarkerOptionsProp_Icon,TMarkerImage,FIconImage));
end;

procedure TMarkerOptions.SetIconImage(Value: TMarkerImage);
begin
  SetProp(MarkerOptionsProp_Icon,Value,true);
end;

function TMarkerOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(MarkerOptionsProp_Map,TMap,FMap));
end;

procedure TMarkerOptions.SetMap(Value: TMap);
begin
  SetProp(MarkerOptionsProp_Map,Value,true);
end;

function TMarkerOptions.GetOptimized: Boolean;
begin
  Result:=GetPropAsBoolean(MarkerOptionsProp_Optimized,true);
end;

procedure TMarkerOptions.SetOptimized(Value: Boolean);
begin
  SetProp(MarkerOptionsProp_Optimized,Value,true);
end;

function TMarkerOptions.GetPosition: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(MarkerOptionsProp_Position,TLatLng,FPosition));
end;

procedure TMarkerOptions.SetPosition(Value: TLatLng);
begin
  SetProp(MarkerOptionsProp_Position,Value,true);
end;

function TMarkerOptions.GetRaiseOnDrag: Boolean;
begin
  Result:=GetPropAsBoolean(MarkerOptionsProp_RaiseOnDrag,true);
end;

procedure TMarkerOptions.SetRaiseOnDrag(Value: Boolean);
begin
  SetProp(MarkerOptionsProp_RaiseOnDrag,Value,true);
end;

function TMarkerOptions.GetShadow: String;
begin
  Result:=GetPropAsString(MarkerOptionsProp_Shadow);
end;

procedure TMarkerOptions.SetShadow(Value: String);
begin
  SetProp(MarkerOptionsProp_Shadow,Value,true);
end;

function TMarkerOptions.GetShadowImage: TMarkerImage;
begin
  Result:=TMarkerImage(GetPropAsObject(MarkerOptionsProp_Shadow,TMarkerImage,FShadowImage));
end;

procedure TMarkerOptions.SetShadowImage(Value: TMarkerImage);
begin
  SetProp(MarkerOptionsProp_Shadow,Value,true);
end;

function TMarkerOptions.GetShape: TMarkerShape;
begin
  Result:=TMarkerShape(GetPropAsObject(MarkerOptionsProp_Shape,TMarkerShape,FShape));
end;

procedure TMarkerOptions.SetShape(Value: TMarkerShape);
begin
  SetProp(MarkerOptionsProp_Shape,Value,true);
end;

function TMarkerOptions.GetStreetViewPanorama: TStreetViewPanorama;
begin
  Result:=TStreetViewPanorama(GetPropAsObject(MarkerOptionsProp_Map,TStreetViewPanorama,FStreetViewPanorama));
end;

procedure TMarkerOptions.SetStreetViewPanorama(Value: TStreetViewPanorama);
begin
  SetProp(MarkerOptionsProp_Map,Value,true);
end;

function TMarkerOptions.GetTitle: String;
begin
  Result:=GetPropAsString(MarkerOptionsProp_Title);
end;

procedure TMarkerOptions.SetTitle(Value: String);
begin
  SetProp(MarkerOptionsProp_Title,Value,true);
end;

function TMarkerOptions.GetVisible: Boolean;
begin
  Result:=GetPropAsBoolean(MarkerOptionsProp_Visible,true);
end;

procedure TMarkerOptions.SetVisible(Value: Boolean);
begin
  SetProp(MarkerOptionsProp_Visible,Value,true);
end;

function TMarkerOptions.GetZIndex: Integer;
begin
  Result:=GetPropAsInteger(MarkerOptionsProp_ZIndex);
end;

procedure TMarkerOptions.SetZIndex(Value: Integer);
begin
  SetProp(MarkerOptionsProp_ZIndex,Value,true);
end;

//------------------------------------------------------------------------------
//TMarkerShape

constructor TMarkerShape.Create;
begin
  inherited;
end;

constructor TMarkerShape.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TMarkerShape.Destroy;
begin
  FCoords.Free;
  inherited;
end;

class function TMarkerShape.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[MarkerShapeProp_Coord,
                                 MarkerShapeProp_Type]);
end;

function TMarkerShape.GetCoords: TIntegerArray;
begin
  Result:=TIntegerArray(GetPropAsObject(MarkerShapeProp_Coords,TIntegerArray,
                                        FCoords,true));
end;

procedure TMarkerShape.SetCoords(Value: TIntegerArray);
begin
  SetProp(MarkerShapeProp_Coords,Value,true);
end;

function TMarkerShape.GetType: String;
begin
  Result:=GetPropAsString(MarkerShapeProp_Type);
end;

procedure TMarkerShape.SetType(Value: String);
begin
  SetProp(MarkerShapeProp_Type,Value,true);
end;

end.

