///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmPolylinePolygon|TPolygon"/></item>
///    <item><see cref="gmPolylinePolygon|TPolygonOptions"/></item>
///    <item><see cref="gmPolylinePolygon|TPolyline"/></item>
///    <item><see cref="gmPolylinePolygon|TPolylineOptions"/></item>
///  </list>
///  <para/>
///  <para><b>Additional Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions"/></item>
///  </list>
///</summary>
unit gmOverlaysPolylinePolygon;

interface

{$INCLUDE gmConfig.inc}

uses
  {$IFDEF DELPHI6_UP}Variants,{$ENDIF} SysUtils, Graphics, Math, BrowserTools,
  JScriptObjects, gmMap, gmBase, gmMVC, gmEvents, gmAdditionalConstants;

type
  TPolygon = class;
  TPolygonOptions = class;
  TPolyline = class;
  TPolylineOptions = class;
  TPolylinePolygonOptions = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPolylinePolygonOptions'}{$ENDIF}
  ///<summary>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmPolylinePolygon"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Clickable">Clickable</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygonOptions.FillColor">FillColor</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygonOptions.FillOpacity">FillOpacity</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Geodesic">Geodesic</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Map">Map</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygonOptions.Paths">Paths</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeColor">StrokeColor</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeOpacity">StrokeOpacity</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeWeight">StrokeWeight</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.ZIndex">ZIndex</see></item>
  ///  </list>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPolylinePolygonOptions = class(TCustomJScriptObject)
  private
    FMap: TMap;
    procedure SetClickable(Value: Boolean);
    procedure SetGeodesic(Value: Boolean);
    procedure SetMap(Value: TMap);
    procedure SetStrokeColor(Value: TColor);
    procedure SetStrokeOpacity(Value: Double);
    procedure SetStrokeWeight(Value: Integer);
    procedure SetZIndex(Value: Integer);
    function GetClickable: Boolean;
    function GetGeodesic: Boolean;
    function GetMap: TMap;
    function GetStrokeColor: TColor;
    function GetStrokeOpacity: Double;
    function GetStrokeWeight: Integer;
    function GetZIndex: Integer;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Clickable'}{$ENDIF}
    ///<summary>
    ///  Indicates whether this Object handles click events. Defaults to true.
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolylinePolygonOptions">TPolylinePolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Clickable: Boolean read GetClickable write SetClickable;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Geodesic'}{$ENDIF}
    ///<summary>
    ///  Render each edge as a geodesic (a segment of a "great circle"). A geodesic
    ///  is the shortest path between two points along the surface of the Earth.
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolylinePolygonOptions">TPolylinePolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Geodesic: Boolean read GetGeodesic write SetGeodesic;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Map'}{$ENDIF}
    ///<summary>
    ///  Map on which to display Object.
    ///</summary>
    ///<seealso cref="gmMap|TMap">TMap</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolylinePolygonOptions">TPolylinePolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Map: TMap read GetMap write SetMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StrokeColor'}{$ENDIF}
    ///<summary>
    ///  The stroke color.
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolylinePolygonOptions">TPolylinePolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property StrokeColor: TColor read GetStrokeColor write SetStrokeColor;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StrokeOpacity'}{$ENDIF}
    ///<summary>
    ///  The stroke opacity between 0.0 and 1.0
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolylinePolygonOptions">TPolylinePolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property StrokeOpacity: Double read GetStrokeOpacity write SetStrokeOpacity;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StrokeWeight'}{$ENDIF}
    ///<summary>
    ///  The stroke width in pixels.
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolylinePolygonOptions">TPolylinePolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property StrokeWeight: Integer read GetStrokeWeight write SetStrokeWeight;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZIndex'}{$ENDIF}
    ///<summary>
    ///  The zIndex compared to other Objects.
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolylinePolygonOptions">TPolylinePolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZIndex: Integer read GetZIndex write SetZIndex;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPolygon'}{$ENDIF}
  ///<summary>
  ///  A polygon (like a polyline) defines a series of connected coordinates in
  ///  an ordered sequence; additionally, polygons form a closed loop and define
  ///  a filled region. This class extends MVCObject.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmPolylinePolygon"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCEventObject"/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmAPI|TMaps.Polygon(TPolygonOptions)">Google.Maps.Polygon</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmPolylinePolygon|TPolygon.GetMap">GetMap</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.GetPath">GetPath</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.GetPaths">GetPaths</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.SetMap(TMap)">SetMap</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.SetOptions(TPolygonOptions)">SetOptions</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.SetPath(TLatLngArray)">SetPath(TLatLngArray)</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.SetPath(TLatLngMVCArray)">SetPath(TLatLngMVCArray)</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.SetPaths(TLatLngArray)">SetPaths(TLatLngArray)</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.SetPaths(TLatLngMVCArray)">SetPaths(TLatLngMVCArray)</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.SetPaths(TLatLngArrayArray)">SetPaths(TLatLngArrayArray)</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.SetPaths(TLatLngMVCArrayMVCArray)">SetPaths(TLatLngMVCArrayMVCArray)</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmPolylinePolygon|TPolygon.Map">Map</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygon.Paths">Paths</see></item>
  ///  </list>
  ///  <para><b>Events</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TMVCEventObject.OnClick">OnClick</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnDblClick">OnDblClick</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseDown">OnMouseDown</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseMove">OnMouseMove</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseOut">OnMouseOut</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseOver">OnMouseOver</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseUp">OnMouseUp</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnRightClick">OnRightClick</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#Polygon">
  ///  Polygon (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Polygon">
  ///  Polygon (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPolygon = class(TMVCEventObject)
  private
    FMap: TMap;
    FPath: TLatLngMVCArray;
    FPaths: TLatLngMVCArrayMVCArray;
    procedure SetPathsEx(Value: TLatLngMVCArrayMVCArray);
  protected
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; Opts: TPolygonOptions): IDispatch; reintroduce; overload;
  public
    constructor Create(Opts: TPolygonOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TPolygonOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetMap'}{$ENDIF}
    ///<summary>
    ///  Renders this Polyline or Polygon on the specified map. If map is set to nil,
    ///  the Poly will be removed.
    ///</summary>
    ///<seealso cref="gmMap|TMap">TMap</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetMap(Map: TMap);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetOptions'}{$ENDIF}
    ///<summary>
    ///  See <see cref="gmPolylinePolygon|TPolygonOptions">TPolylineOptions</see>
    ///  for more details.
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetOptions(Options: TPolygonOptions); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPath'}{$ENDIF}
    ///<summary>
    ///  Sets the first path.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray">TLatLngArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPath(Path: TLatLngArray); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPath'}{$ENDIF}
    ///<summary>
    ///  Sets the first path.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPath(Path: TLatLngMVCArray); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPaths'}{$ENDIF}
    ///<summary>
    ///  Sets the path for this Polygon.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray">TLatLngArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPaths(Paths: TLatLngArray); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPaths'}{$ENDIF}
    ///<summary>
    ///  Sets the path for this Polygon.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPaths(Paths: TLatLngMVCArray); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPaths'}{$ENDIF}
    ///<summary>
    ///  Sets the path for this Polygon.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArrayArray">TLatLngArrayArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
   {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPaths(Paths: TLatLngArrayArray); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPaths'}{$ENDIF}
    ///<summary>
    ///  Sets the path for this Polygon.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPaths(Paths: TLatLngMVCArrayMVCArray); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetMap'}{$ENDIF}
    ///<summary>
    ///  Returns the map on which this poly is attached.
    ///</summary>
    ///<seealso cref="gmMap|TMap">TMap</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetMap: TMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetPath'}{$ENDIF}
    ///<summary>
    ///  Retrieves the first path.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetPath: TLatLngMVCArray;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetPaths'}{$ENDIF}
    ///<summary>
    ///  Retrieves the paths for this Polygon.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetPaths: TLatLngMVCArrayMVCArray;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Map'}{$ENDIF}
    ///<summary>
    ///  Map on which this poly is attached. If Map is set to nil,
    /// the Poly will be removed.
    ///</summary>
    ///<remarks>
    ///  Im Google Maps API nicht vorhanden.<br/><br/>
    ///  Not available in the Google Maps API.
    ///</remarks>
    ///<seealso cref="gmMap|TMap">TMap</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Map: TMap read GetMap write SetMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Paths'}{$ENDIF}
    ///<summary>
    ///  Paths for this Polygon.
    ///</summary>
    ///<remarks>
    ///  Im Google Maps API nicht vorhanden.<br/><br/>
    ///  Not available in the Google Maps API.
    ///</remarks>
    ///<seealso cref="gmMVC|TLatLngMVCArrayMVCArray">TLatLngMVCArrayMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolygon">TPolygon</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Paths: TLatLngMVCArrayMVCArray read GetPaths write SetPathsEx;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPolygonOptions'}{$ENDIF}
  ///<summary>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmPolylinePolygon"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmPolylinePolygon|TPolylinePolygonOptions"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Clickable">Clickable</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygonOptions.FillColor">FillColor</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygonOptions.FillOpacity">FillOpacity</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Geodesic">Geodesic</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Map">Map</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolygonOptions.Paths">Paths</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeColor">StrokeColor</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeOpacity">StrokeOpacity</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeWeight">StrokeWeight</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.ZIndex">ZIndex</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de/apis/maps/documentation/javascript/reference.html#PolygonOptions">
  ///  PolygonOptions (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PolygonOptions">
  ///  PolygonOptions (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPolygonOptions = class(TPolylinePolygonOptions)
  private
    FPaths: TLatLngMVCArrayMVCArray;
    procedure SetFillColor(Value: TColor);
    procedure SetFillOpacity(Value: Double);
    procedure SetPaths(Value: TLatLngMVCArrayMVCArray);
    function GetFillColor: TColor;
    function GetFillOpacity: Double;
    function GetPaths: TLatLngMVCArrayMVCArray;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation FillColor'}{$ENDIF}
    ///<summary>
    ///  The fill color.
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolygonOptions">TPolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property FillColor: TColor read GetFillColor write SetFillColor;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation FillOpacity'}{$ENDIF}
    ///<summary>
    ///  The fill opacity between 0.0 and 1.0
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolygonOptions">TPolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property FillOpacity: Double read GetFillOpacity write SetFillOpacity;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Paths'}{$ENDIF}
    ///<summary>
    ///  The ordered sequence of coordinates that designates a closed loop.
    ///  Unlike polylines, a polygon may consist of one or more paths. As a result,
    ///  the paths property may specify one or more arrays of <i>TLatLng</i> coordinates.
    ///  Simple polygons may be defined using a single array of <i>TLatLng</i>s. More complex
    ///  polygons may specify an array of arrays. Any simple arrays are convered into
    ///  <i>TMVCArray</i>s. Inserting or removing <i>TLatLng</i>s from the <i>TMVCArray</i>
    ///  will automatically update the polygon on the map.
    ///</summary>
    ///<seealso cref="gmPolylinePolygon|TPolygonOptions">TPolygonOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Paths: TLatLngMVCArrayMVCArray read GetPaths write SetPaths;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPolyline'}{$ENDIF}
  ///<summary>
  ///  A polyline is a linear overlay of connected line segments on the map.
  ///  This class extends <see cref="gmMVC|TMVCObject">TMVCObject</see>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmPolylinePolygon"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="gmMVC|TMVCEventObject"/>
  ///  <para/>
  ///  <para><b>Constructor</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmAPI|TMaps.Polyline(TPolylineOptions)">Google.Maps.Polyline</see></item>
  ///  </list>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmPolylinePolygon|TPolyline.GetMap">GetMap</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolyline.GetPath">GetPath</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolyline.SetMap(TMap)">SetMap</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolyline.SetOptions(TPolylineOptions)">SetOptions</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolyline.SetPath(TLatLngArray)">SetPath(TLatLngArray)</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolyline.SetPath(TLatLngMVCArray)">SetPath(TLatLngMVCArray)</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmPolylinePolygon|TPolyline.Map">Map</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolyline.Path">Path</see></item>
  ///  </list>
  ///  <para><b>Events</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmMVC|TMVCEventObject.OnClick">OnClick</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnDblClick">OnDblClick</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseDown">OnMouseDown</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseMove">OnMouseMove</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseOut">OnMouseOut</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseOver">OnMouseOver</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnMouseUp">OnMouseUp</see></item>
  ///    <item><see cref="gmMVC|TMVCEventObject.OnRightClick">OnRightClick</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#Polyline">
  ///  Polyline (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Polyline">
  ///  Polyline (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPolyline = class(TMVCEventObject)
  private
    FMap: TMap;
    FPath: TLatLngMVCArray;
    procedure SetPathEx(Value: TLatLngMVCArray);
  protected
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; Opts: TPolylineOptions): IDispatch; reintroduce; overload;
  public
    constructor Create(Opts: TPolylineOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TPolylineOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Clickable'}{$ENDIF}
    ///<summary>
    ///  Renders this Polyline on the specified map. If map is set to <i>nil</i>,
    /// the Poly will be removed.
    ///</summary>
    ///<seealso cref="gmMap|TMap">TMap</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolyline">TPolyline</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetMap(Map: TMap);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetOptions'}{$ENDIF}
    ///<seealso cref="gmPolylinePolygon|TPolylineOptions">TPolylineOptions</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolyline">TPolyline</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetOptions(Options: TPolylineOptions); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPath'}{$ENDIF}
    ///<summary>
    ///  Sets the first path. See <see cref="gmPolylinePolygon|TPolylineOptions">TPolylineOptions</see> for more details.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray">TLatLngArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolyline">TPolyline</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPath(Path: TLatLngArray); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPath'}{$ENDIF}
    ///<summary>
    ///  Sets the first path. See <see cref="gmPolylinePolygon|TPolylineOptions">TPolylineOptions</see> for more details.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolyline">TPolyline</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPath(Path: TLatLngMVCArray); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetMap'}{$ENDIF}
    ///<summary>
    ///  Returns the map on which this poly is attached.
    ///</summary>
    ///<seealso cref="gmMap|TMap">TMap</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolyline">TPolyline</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetMap: TMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetPath'}{$ENDIF}
    ///<summary>
    ///  Retrieves the first path.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolyline">TPolyline</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetPath: TLatLngMVCArray;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Map'}{$ENDIF}
    ///<summary>
    ///  Zustzliche Eigenschaft. Nicht im JScript-Objekt vorhanden.<br/><br/>
    ///  Additional Delphi property.<br/>
    ///  Returns the map on which this poly is attached.
    ///</summary>
    ///<seealso cref="gmMap|TMap">TMap</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolyline">TPolyline</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Map: TMap read GetMap write SetMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Path'}{$ENDIF}
    ///<summary>
    ///  Zustzliche Eigenschaft. Nicht im JScript-Objekt vorhanden.<br/><br/>
    ///  Additional Delphi property.<br/>
    ///  Retrieves the first path.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolyline">TPolyline</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Path: TLatLngMVCArray read GetPath write SetPathEx;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TPolylineOptions'}{$ENDIF}
  ///<summary>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmPolylinePolygon"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="gmPolylinePolygon|TPolylinePolygonOptions"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Clickable">Clickable</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Geodesic">Geodesic</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.Map">Map</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylineOptions.Path">Path</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeColor">StrokeColor</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeOpacity">StrokeOpacity</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.StrokeWeight">StrokeWeight</see></item>
  ///    <item><see cref="gmPolylinePolygon|TPolylinePolygonOptions.ZIndex">ZIndex</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de/apis/maps/documentation/javascript/reference.html#PolylineOptions">
  ///  PolylineOptions (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#PolylineOptions">
  ///  PolylineOptions (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TPolylineOptions = class(TPolylinePolygonOptions)
  private
    FPath: TLatLngMVCArray;
    procedure SetPath(Value: TLatLngMVCArray);
    function GetPath: TLatLngMVCArray;
  protected
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Path'}{$ENDIF}
    ///<summary>
    ///  The ordered sequence of coordinates of the Polyline. This path may be
    ///  specified using an <i>TMVCArray</i> of <i>TLatLng</i>'s.
    ///</summary>
    ///<seealso cref="gmMVC|TLatLngMVCArray">TLatLngMVCArray</seealso>
    ///<seealso cref="gmPolylinePolygon|TPolylineOptions">TPolylineOptions</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Path: TLatLngMVCArray read GetPath write SetPath;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TPolylinePolygonOptions

destructor TPolylinePolygonOptions.Destroy;
begin
  FMap.Free;
  inherited;
end;

function TPolylinePolygonOptions.GetClickable: Boolean;
begin
  Result:=GetPropAsBoolean(OptionsProp_Clickable,true);
end;

procedure TPolylinePolygonOptions.SetClickable(Value: Boolean);
begin
  SetProp(OptionsProp_Clickable,Value,true);
end;

function TPolylinePolygonOptions.GetGeodesic: Boolean;
begin
  Result:=GetPropAsBoolean(OptionsProp_Geodesic);
end;

procedure TPolylinePolygonOptions.SetGeodesic(Value: Boolean);
begin
  SetProp(OptionsProp_Geodesic,Value,true);
end;

function TPolylinePolygonOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(OptionsProp_Map,TMap,FMap));
end;

procedure TPolylinePolygonOptions.SetMap(Value: TMap);
begin
  SetProp(OptionsProp_Map,Value,true);
end;

function TPolylinePolygonOptions.GetStrokeColor: TColor;
begin
  Result:=WebColorStrToColor(GetPropAsString(OptionsProp_StrokeColor,ColorToWebColorStr(clBlack)));
end;

procedure TPolylinePolygonOptions.SetStrokeColor(Value: TColor);
begin
  SetProp(OptionsProp_StrokeColor,ColorToWebColorStr(Value),true);
end;

function TPolylinePolygonOptions.GetStrokeOpacity: Double;
begin
  Result:=GetPropAsDouble(OptionsProp_StrokeOpacity,1);
end;

procedure TPolylinePolygonOptions.SetStrokeOpacity(Value: Double);
begin
  SetProp(OptionsProp_StrokeOpacity,Min(Max(0,Value),1),true);
end;

function TPolylinePolygonOptions.GetStrokeWeight: Integer;
begin
  Result:=GetPropAsInteger(OptionsProp_StrokeWeight,3);
end;

procedure TPolylinePolygonOptions.SetStrokeWeight(Value: Integer);
begin
  SetProp(OptionsProp_StrokeWeight,Value,true);
end;

function TPolylinePolygonOptions.GetZIndex: Integer;
begin
  Result:=GetPropAsInteger(OptionsProp_ZIndex);
end;

procedure TPolylinePolygonOptions.SetZIndex(Value: Integer);
begin
  SetProp(OptionsProp_ZIndex,Value,true);
end;

//------------------------------------------------------------------------------
//TPolygon

constructor TPolygon.Create(Opts: TPolygonOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TPolygon.Create(Script: TCustomScript; Opts: TPolygonOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TPolygon.Destroy;
begin
  FMap.Free;
  FPath.Free;
  FPaths.Free;
  inherited;
end;

class function TPolygon.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[PolygonMethod_GetMap,
                                 PolygonMethod_SetMap,
                                 PolygonMethod_SetOptions]);
end;

class function TPolygon.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_Polygon,true);
end;

class function TPolygon.New(Script: TCustomScript; Opts: TPolygonOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_Polygon,Opts.Disp,true);
end;

function TPolygon.GetMap: TMap;
var
  Map: OleVariant;
begin
  Result:=nil;
  Map:=Exec(PolygonMethod_GetMap);
  if VarIsValidDispatch(Map) then
  begin
    if assigned(FMap)
      then FMap.Disp:=Map
      else FMap:=TMap(CreateObjectWrapper(Script,TMap,Map));
    Result:=FMap;
  end;
end;

procedure TPolygon.SetMap(Map: TMap);
begin
  Exec(PolygonMethod_SetMap,Map)
end;

procedure TPolygon.SetOptions(Options: TPolygonOptions);
begin
  if assigned(Options)
    then Exec(PolygonMethod_SetOptions,[Options.Disp]);
end;

function TPolygon.GetPath: TLatLngMVCArray;
var
  Path: OleVariant;
begin
  Result:=nil;
  Path:=Exec(PolygonMethod_GetPath);
  if VarIsValidDispatch(Path) then
  begin
    if assigned(FPath)
      then FPath.Disp:=Path
      else FPath:=TLatLngMVCArray(CreateObjectWrapper(Script,TLatLngMVCArray,Path));
    Result:=FPath;
  end;
end;

procedure TPolygon.SetPath(Path: TLatLngArray);
begin
  Exec(PolygonMethod_SetPath,Path);
end;

procedure TPolygon.SetPath(Path: TLatLngMVCArray);
begin
  Exec(PolygonMethod_SetPath,Path);
end;

function TPolygon.GetPaths: TLatLngMVCArrayMVCArray;
var
  Paths: OleVariant;
begin
  Result:=nil;
  Paths:=Exec(PolygonMethod_GetPaths);
  if VarIsValidDispatch(Paths) then
  begin
    if assigned(FPaths)
      then FPaths.Disp:=Paths
      else FPaths:=TLatLngMVCArrayMVCArray(CreateObjectWrapper(Script,
                                                               TLatLngMVCArrayMVCArray,
                                                               Paths));
    Result:=FPaths;
  end;
end;

procedure TPolygon.SetPaths(Paths: TLatLngArray);
begin
  Exec(PolygonMethod_SetPaths,Paths);
end;

procedure TPolygon.SetPaths(Paths: TLatLngMVCArray);
begin
  Exec(PolygonMethod_SetPaths,Paths);
end;

procedure TPolygon.SetPaths(Paths: TLatLngArrayArray);
begin
  Exec(PolygonMethod_SetPaths,Paths);
end;

procedure TPolygon.SetPaths(Paths: TLatLngMVCArrayMVCArray);
begin
  Exec(PolygonMethod_SetPaths,Paths);
end;

//Hilfsmethode fr ltere Compiler, die keine berladenen Property-Setter erlauben
procedure TPolygon.SetPathsEx(Value: TLatLngMVCArrayMVCArray);
begin
  SetPaths(Value);
end;

//------------------------------------------------------------------------------
//TPolygonOptions

constructor TPolygonOptions.Create;
begin
  inherited;
end;

constructor TPolygonOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TPolygonOptions.Destroy;
begin
  FPaths.Free;
  inherited;
end;

class function TPolygonOptions.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[PolygonOptionsProp_FillColor,
                                 PolygonOptionsProp_FillOpacity,
                                 PolygonOptionsProp_Paths]);
end;

function TPolygonOptions.GetFillColor: TColor;
begin
  Result:=WebColorStrToColor(GetPropAsString(PolygonOptionsProp_FillColor,ColorToWebColorStr(clBlack)));
end;

procedure TPolygonOptions.SetFillColor(Value: TColor);
begin
  SetProp(PolygonOptionsProp_FillColor,ColorToWebColorStr(Value),true);
end;

function TPolygonOptions.GetFillOpacity: Double;
begin
  Result:=GetPropAsDouble(PolygonOptionsProp_FillOpacity,0.3);
end;

procedure TPolygonOptions.SetFillOpacity(Value: Double);
begin
  SetProp(PolygonOptionsProp_FillOpacity,Min(Max(0,Value),1),true);
end;

function TPolygonOptions.GetPaths: TLatLngMVCArrayMVCArray;
begin
  Result:=TLatLngMVCArrayMVCArray(GetPropAsObject(PolygonOptionsProp_Paths,
                                                  TLatLngMVCArrayMVCArray,
                                                  FPaths,true));
end;

procedure TPolygonOptions.SetPaths(Value: TLatLngMVCArrayMVCArray);
begin
  SetProp(PolygonOptionsProp_Paths,Value,true);
end;

//------------------------------------------------------------------------------
//TPolyline

constructor TPolyline.Create(Opts: TPolylineOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TPolyline.Create(Script: TCustomScript; Opts: TPolylineOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TPolyline.Destroy;
begin
  FMap.Free;
  FPath.Free;
  inherited;
end;

class function TPolyline.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_Polyline,true);
end;

class function TPolyline.New(Script: TCustomScript; Opts: TPolylineOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_Polyline,Opts.Disp,true);
end;

class function TPolyline.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[PolylineMethod_GetMap,
                                 PolylineMethod_SetMap,
                                 PolylineMethod_SetOptions]);
end;

function TPolyline.GetMap: TMap;
var
  Map: OleVariant;
begin
  Result:=nil;
  Map:=Exec(PolylineMethod_GetMap);
  if VarIsValidDispatch(Map) then
  begin
    if assigned(FMap)
      then FMap.Disp:=Map
      else FMap:=TMap(CreateObjectWrapper(Script,TMap,Map));
    Result:=FMap;
  end;
end;

procedure TPolyline.SetMap(Map: TMap);
begin
  Exec(PolylineMethod_SetMap,Map);
end;

procedure TPolyline.SetOptions(Options: TPolylineOptions);
begin
  if assigned(Options)
    then Exec(PolylineMethod_SetOptions,[Options.Disp]);
end;

function TPolyline.GetPath: TLatLngMVCArray;
var
  Path: OleVariant;
begin
  Result:=nil;
  Path:=Exec(PolylineMethod_GetPath);
  if VarIsValidDispatch(Path) then
  begin
    if assigned(FPath)
      then FPath.Disp:=Path
      else FPath:=TLatLngMVCArray(CreateObjectWrapper(Script,TLatLngMVCArray,Path));
    Result:=FPath;
  end;
end;

procedure TPolyline.SetPath(Path: TLatLngArray);
begin
  Exec(PolylineMethod_SetPath,Path);
end;

procedure TPolyline.SetPath(Path: TLatLngMVCArray);
begin
  Exec(PolylineMethod_SetPath,Path);
end;

//Hilfsmethode fr ltere Compiler, die keine berladenen Property-Setter erlauben
procedure TPolyline.SetPathEx(Value: TLatLngMVCArray);
begin
  SetPath(Value);
end;

//------------------------------------------------------------------------------
//TPolylineOptions

constructor TPolylineOptions.Create;
begin
  inherited;
end;

constructor TPolylineOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TPolylineOptions.Destroy;
begin
  FPath.Free;
  inherited;
end;

class function TPolylineOptions.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[PolylineOptionsProp_Path]);
end;

function TPolylineOptions.GetPath: TLatLngMVCArray;
begin
  Result:=TLatLngMVCArray(GetPropAsObject(PolylineOptionsProp_Path,
                                          TLatLngMVCArray,FPath,true));
end;

procedure TPolylineOptions.SetPath(Value: TLatLngMVCArray);
begin
  SetProp(PolylineOptionsProp_Path,Value,true);
end;

end.
