///<summary>
///  Diese Unit enthlt die DirectionsService-Objekte des
///  <see href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html">
///  Google Maps API</see>'s.<br/><br/>
///  This unit contains DirectionsService objects of the
///  <see href="http://code.google.com/apis/maps/documentation/javascript/reference.html">
///  Google Maps API</see>.
///  <para/>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmServices.Directions|TDirectionsLeg"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsRequest"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsResult"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsRoute"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsService"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsStatus"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsStep"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsWayPoint"/></item>
///    <item><see cref="gmServices.Directions|TDistance"/></item>
///    <item><see cref="gmServices.Directions|TDuration"/></item>
///    <item><see cref="gmServices.Directions|TTravelMode"/></item>
///    <item><see cref="gmServices.Directions|TUnitSystem"/></item>
///  </list>
///  <para/>
///  <para><b>Additional Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmServices.Directions|TDirectionsLegArray"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsRouteArray"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsStepArray"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsWayPointArray"/></item>
///  </list>
///  <para><b>Deprecated Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmServices.Directions|TDirectionsDistance"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsDuration"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsTravelMode"/></item>
///    <item><see cref="gmServices.Directions|TDirectionsUnitSystem"/></item>
///  </list>
///</summary>
unit gmServicesDirections;

interface

{$INCLUDE gmConfig.inc}

uses
  SysUtils, {$IFDEF DEBUG}Dialogs,{$ENDIF} DispObject, BrowserTools,
  JScriptObjects, gmBase, gmAdditionalMessages, gmAdditionalConstants;

type
  TDirectionsDistance = class;
  TDirectionsDuration = class;
  TDirectionsLeg = class;
  TDirectionsLegArray = class;
  TDirectionsRequest = class;
  TDirectionsResult = class;
  TDirectionsRoute = class;
  TDirectionsRouteArray = class;
  TDirectionsService = class;
  TDirectionsStatus = class;
  TDirectionsStep = class;
  TDirectionsStepArray = class;
  TDirectionsTravelMode = class;
  TDirectionsUnitSystem = class;
  TDirectionsWayPoint = class;
  TDirectionsWayPointArray = class;
  TDistance = class;
  TDuration = class;
  TTravelMode = class;
  TUnitSystem = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistance'}{$ENDIF}
  ///<summary>
  ///  A representation of distance as a numeric value and a display string.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDistance.Text">Text</see></item>
  ///    <item><see cref="gmServices.Directions|TDistance.Value">Value</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsDistance">
  ///  Distance (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Distance">
  ///  Distance (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistance = class(TCustomJScriptObject)
  private
   function GetText: String;
   function GetValue: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Text'}{$ENDIF}
    ///<summary>
    ///  Eine Zeichenfolgendarstellung des Entfernungswertes unter Verwendung des
    ///  in der Anfrage angegebenen <see cref="TDirectionsUnitSystem">
    ///  DirectionsUnitSystems</see>.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDistance"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Text: String read GetText;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Value'}{$ENDIF}
    ///<summary>
    ///  Die Entfernung in Metern.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDistance"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Value: Integer read GetValue;
  end;

  TDirectionsDistance = class(TDistance);                  //deprecated

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDuration'}{$ENDIF}
  ///<summary>
  ///  A representation of duration as a numeric value and a display string.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDistance.Text">Text</see></item>
  ///    <item><see cref="gmServices.Directions|TDistance.Value">Value</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsDuration">
  ///  Duration (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#Duration">
  ///  Duration (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDuration = class(TCustomJScriptObject)
  private
   function GetText: String;
   function GetValue: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Text'}{$ENDIF}
    ///<summary>
    ///  Eine Zeichenfolgendarstellung des Wertes fr die Dauer.<br/><br/>
    ///  A string representation of the duration value.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDuration"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Text: String read GetText;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Value'}{$ENDIF}
    ///<summary>
    ///  Die Dauer in Sekunden.<br/><br/>
    ///  The duration in seconds.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDuration"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Value: Integer read GetValue;
  end;

  TDirectionsDuration = class(TDuration);                  //deprecated

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsLeg'}{$ENDIF}
  ///<summary>
  ///  A single leg consisting of a set of steps in a <c>TDirectionsResult</c>.
  ///  Some fields in the leg may not be returned for all requests. (This object
  ///  was formerly known as "DirectionsRoute".) Note that though this result is
  ///  "JSON-like," it is not strictly JSON, as it directly and indirectly
  ///  includes <c>TLatLng</c> objects.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsLeg.Distance">Distance</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsLeg.Duration">Duration</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsLeg.EndAddress">EndAddress</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsLeg.EndLocation">EndLocation</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsLeg.StartAddress">StartAddress</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsLeg.StartLocation">StartLocation</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsLeg.Steps">Steps</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsLeg.ViaWayPoints">ViaWayPoints</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmServices.Directions|TDirectionsResult"/>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsLeg">
  ///  DirectionsLeg (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsLeg">
  ///  DirectionsLeg (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsLeg = class(TCustomJScriptObject)
  private
    FDistance: TDistance;
    FDuration: TDuration;
    FEndLocation: TLatLng;
    FStartLocation: TLatLng;
    FSteps: TDirectionsStepArray;
    FViaWayPoints: TLatLngArray;
    function GetDistance: TDistance;
    function GetDuration: TDuration;
    function GetEndAddress: String;
    function GetEndLocation: TLatLng;
    function GetStartAddress: String;
    function GetStartLocation: TLatLng;
    function GetSteps: TDirectionsStepArray;
    function GetViaWayPoints: TLatLngArray;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Distance'}{$ENDIF}
    ///<summary>
    ///  Die Gesamtstrecke, die durch diesen Streckenabschnitt zurckgelegt wird.
    ///  Diese Eigenschaft ist mglicherweise nicht definiert, falls die
    ///  Entfernung nicht bekannt ist.<br/><br/>
    ///  The total distance covered by this leg. This property may be undefined
    ///  as the distance may be unknown.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Distance: TDistance read GetDistance;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Duration'}{$ENDIF}
    ///<summary>
    ///  Die Gesamtdauer fr diesen Streckenabschnitt. Diese Eigenschaft ist
    ///  mglicherweise nicht definiert, falls dieser Wert nicht bekannt ist.<br/><br/>
    ///  The total duration of this leg. This property may be undefined as
    ///  the duration may be unknown.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Duration: TDuration read GetDuration;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation EndAddress'}{$ENDIF}
    ///<summary>
    ///  Die Zieladresse dieses Streckenabschnitts.<br/><br/>
    ///  The address of the destination of this leg.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property EndAddress: String read GetEndAddress;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation EndLocation'}{$ENDIF}
    ///<summary>
    ///  Der DirectionsService berechnet die Route zwischen Standorten unter
    ///  Verwendung der nchstgelegenen Transportoption, normalerweise einer
    ///  Strae, bei den Start- und Endstandorten. EndLocation gibt das
    ///  tatschliche geocodierte Ziel an; dieses kann sich vom
    ///  <c>EndLocation</c> des letzten Schrittes unterscheiden, zum Beispiel
    ///  wenn die Strae sich nicht in der Nhe des Ziels dieses
    ///  Streckenabschnitts befindet.<br/><br/>
    ///  The DirectionsService calculates directions between locations by using
    ///  the nearest transportation option (usually a road) at the start and end
    ///  locations. <c>EndLocation</c> indicates the actual geocoded destination,
    ///  which may be different than the <c>EndLocation</c> of the last step if,
    ///  for example, the road is not near the destination of this leg.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property EndLocation: TLatLng read GetEndLocation;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StartAddress'}{$ENDIF}
    ///<summary>
    ///  Die Ursprungsadresse dieses Streckenabschnitts.<br/><br/>
    ///  The address of the origin of this leg.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property StartAddress: String read GetStartAddress;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StartLocation'}{$ENDIF}
    ///<summary>
    ///  Der DirectionsService berechnet die Route zwischen Standorten unter
    ///  Verwendung der nchstgelegenen Transportoption, normalerweise einer
    ///  Strae, bei den Start- und Endstandorten. <c>StartLocation</c>
    ///  gibt den tatschlichen geocodierten Ursprung an - dieser kann sich von
    ///  <c>StartLocation</c> des ersten Schrittes unterscheiden, zum Beispiel
    ///  wenn die Strae sich nicht in der Nhe des Ursprungs dieses
    ///  Streckenabschnitts befindet.<br/><br/>
    ///  The DirectionsService calculates directions between locations by using
    ///  the nearest transportation option (usually a road) at the start and end
    ///  locations. <c>StartLocation</c> indicates the actual geocoded origin,
    ///  which may be different than the <c>StartLocation</c> of the first step
    ///  if, for example, the road is not near the origin of this leg.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property StartLocation: TLatLng read GetStartLocation;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Steps'}{$ENDIF}
    ///<summary>
    ///  Ein Array mit <c>TDirectionsStep</c>'s, von denen jeder Informationen
    ///  ber die einzelnen Schritte in diesem Streckenabschnitt enthlt.<br/><br/>
    ///  An array of <c>TDirectionsStep</c>'s, each of which contains
    ///  information about the individual steps in this leg.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Steps: TDirectionsStepArray read GetSteps;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ViaWayPoints'}{$ENDIF}
    ///<summary>
    ///  An array of waypoints along this leg that were not specified in the
    ///  original request, either as a result of a user dragging the polyline
    ///  or selecting an alternate route.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngArray"/>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ViaWayPoints: TLatLngArray read GetViaWayPoints;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsLegArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;DirectionsLeg&gt;<br/><br/>
  ///  Objekt zur Verwaltung von <c>TDirectionsLeg</c>-Objekten.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsLegArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  In JavaScript nicht vorhanden.
  ///</remarks>
  ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsLegArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TDirectionsLeg;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Ein Array mit <c>TDirectionsLeg</c>-Objekten, von denen
    ///  jeder Informationen ber die einzelnen Schritte in diesem Streckenabschnitt
    ///  enthlt.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsLegArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TDirectionsLeg read GetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsRequest'}{$ENDIF}
  ///<summary>
  ///  A directions query to be sent to the DirectionsService.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.AvoidHighways">AvoidHighways</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.AvoidTolls">AvoidTolls</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.Destination">Destination</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.DestinationAsString">DestinationAsString</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.OptimizeWayPoints">OptimizeWayPoints</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.Origin">Origin</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.OriginAsString">OriginAsString</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.ProvideRouteAlternatives">ProvideRouteAlternatives</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.Region">Region</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.TravelMode">TravelMode</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.UnitSystem">UnitSystem</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRequest.WayPoints">WayPoints</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmServices.Directions|TDirectionsService"/>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsRequest">
  ///  DirectionsRequest (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsRequest">
  ///  DirectionsRequest (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsRequest = class(TCustomJScriptObject)
  private
    FDestination: TLatLng;
    FOrigin: TLatLng;
    FWayPoints: TDirectionsWayPointArray;
    procedure SetAvoidHighways(Value: Boolean);
    procedure SetAvoidTolls(Value: Boolean);
    procedure SetDestination(Value: TLatLng);
    procedure SetDestinationAsString(Value: String);
    procedure SetOptimizeWayPoints(Value: Boolean);
    procedure SetOrigin(Value: TLatLng);
    procedure SetOriginAsString(Value: String);
    procedure SetProvideRouteAlternatives(Value: Boolean);
    procedure SetRegion(Value: String);
    procedure SetTravelMode(Value: String);
    procedure SetUnitSystem(Value: Integer);
    procedure SetWayPoints(Value: TDirectionsWayPointArray);
    function GetAvoidHighways: Boolean;
    function GetAvoidTolls: Boolean;
    function GetDestination: TLatLng;
    function GetDestinationAsString: String;
    function GetOptimizeWayPoints: Boolean;
    function GetOrigin: TLatLng;
    function GetOriginAsString: String;
    function GetProvideRouteAlternatives: Boolean;
    function GetRegion: String;
    function GetTravelMode: String;
    function GetUnitSystem: Integer;
    function GetWayPoints: TDirectionsWayPointArray;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AvoidHighways'}{$ENDIF}
    ///<summary>
    ///  Falls <c>true</c>, wird der Routendienst angewiesen, Autobahnen wo
    ///  mglich zu vermeiden. Optional.<br/><br/>
    ///  If <c>true</c>, instructs the Directions service to avoid highways
    ///  where possible. Optional.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property AvoidHighways: Boolean read GetAvoidHighways write SetAvoidHighways;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AvoidTolls'}{$ENDIF}
    ///<summary>
    ///  Falls <c>true</c>, wird der Routendienst angewiesen, mautpflichtige
    ///  Straen wo mglich zu vermeiden. Optional.<br/><br/>
    ///  If <c>true</c>, instructs the Directions service to avoid toll roads
    ///  where possible. Optional.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property AvoidTolls: Boolean read GetAvoidTolls write SetAvoidTolls;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Destination'}{$ENDIF}
    ///<summary>
    ///  Zielstandort als <c>TLatLng</c>-Wert.<br/>
    ///  Einer der beiden Werte <c>Destination</c> oder
    ///  <c>DestinationAsString</c> ist erforderlich.<br/><br/>
    ///  Location of destination as <c>TLatLng</c>.<br/>
    ///  <c>Destination</c> or <c>DestinationAsString</c> is required.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest.DestinationAsString">
    ///  DestinationAsString
    ///</seealso>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Destination: TLatLng read GetDestination write SetDestination;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DestinationAsString'}{$ENDIF}
    ///<summary>
    ///  Zielstandort als Zeichenfolge, die geocodiert wird.<br/>
    ///  Einer der beiden Werte <c>Destination</c> oder
    ///  <c>DestinationAsString</c> ist erforderlich.<br/><br/>
    ///  Location of destination as a string to be geocoded.<br/>
    ///  <c>Destination</c> or <c>DestinationAsString</c> is required.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest.Destination">
    ///  Destination
    ///</seealso>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DestinationAsString: String read GetDestinationAsString write SetDestinationAsString;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OptimizeWayPoints'}{$ENDIF}
    ///<summary>
    ///  Falls als Wert <i>true</i> festgelegt wird, versucht der
    ///  DirectionService, die bereitgestellten Wegzwischenpunkte so neu zu
    ///  ordnen, dass die Gesamtkosten der Route minimiert werden. Falls die
    ///  Wegpunkte optimiert werden, berprfen Sie
    ///  <c>TDirectionsRoute.WayPointOrder</c> in der Antwort, um die neue
    ///  Reihenfolge zu ermitteln.<br/><br/>
    ///  If set to <c>true</c>, the DirectionService will attempt to re-order
    ///  the supplied intermediate waypoints to minimize overall cost of the
    ///  route. If waypoints are optimized, inspect
    ///  <c>TDirectionsRoute.WayPointOrder</c> in the response to determine the
    ///  new ordering.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRoute.WayPointOrder">
    ///  WayPointOrder
    ///</seealso>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OptimizeWayPoints: Boolean read GetOptimizeWayPoints write SetOptimizeWayPoints;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Origin'}{$ENDIF}
    ///<summary>
    ///  Ursprungsort als <c>TLatLng</c>-Wert.<br/>
    ///  Einer der beiden Werte <c>Origin</c> oder <c>OriginAsString</c> ist
    ///  erforderlich.<br/><br/>
    ///  Location of origin as a <c>TLatLng</c> value.<br/>
    ///  <c>Origin</c> or <c>OriginAsString</c> is required.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest.OriginAsString">
    ///  OriginAsString
    ///</seealso>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Origin: TLatLng read GetOrigin write SetOrigin;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OriginAsString'}{$ENDIF}
    ///<summary>
    ///  Ursprungsort als Zeichenfolge, die geocodiert wird.
    ///  Einer der beider Werte <c>Origin</c> oder <c>OriginAsString</c> ist
    ///  erforderlich.<br/><br/><br/>
    ///  Location of origin as a string to be geocoded.<br/>
    ///  <c>Origin</c> or <c>OriginAsString</c> is required.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest.Origin">Origin</seealso>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OriginAsString: String read GetOriginAsString write SetOriginAsString;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ProvideRouteAlternatives'}{$ENDIF}
    ///<summary>
    ///  Legt fest, ob alternative Routen bereitgestellt werden sollen oder
    ///  nicht. Optional.<br/><br/>
    ///  Whether or not route alternatives should be provided. Optional.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ProvideRouteAlternatives: Boolean read GetProvideRouteAlternatives write SetProvideRouteAlternatives;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Region'}{$ENDIF}
    ///<summary>
    ///  Regionscode fr die Gewichtung von Geocodierungsanfragen. Optional.<br/><br/>
    ///  Region code used as a bias for geocoding requests. Optional.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Region: String read GetRegion write SetRegion;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation TravelMode'}{$ENDIF}
    ///<summary>
    ///  Typ der angeforderten Route. Erforderlich.<br/><br/>
    ///  Type of routing requested. Required.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TTravelMode"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property TravelMode: String read GetTravelMode write SetTravelMode;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UnitSystem'}{$ENDIF}
    ///<summary>
    ///  Bevorzugtes Maeinheitensystem fr die Anzeige der Entfernung.
    ///  Standardmig wird das System des Ursprungslandes verwendet.<br/><br/>
    ///  Preferred unit system to use when displaying distance. Defaults to the
    ///  unit system used in the country of origin.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TUnitSystem"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UnitSystem: Integer read GetUnitSystem write SetUnitSystem;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation WayPoints'}{$ENDIF}
    ///<summary>
    ///  Array mit <c>TDirectionsWayPoint</c>-Werten. Es wird die Route berechnet,
    ///  die vom ursprnglichen Standort zum Zielstandort und dabei ber alle
    ///  Wegpunkte in diesem Array fhrt. Optional.<br/><br/>
    ///  Array of intermediate waypoints. Directions will be calculated from the
    ///  origin to the destination by way of each waypoint in this array.
    ///  Optional.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsWayPoint"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property WayPoints: TDirectionsWayPointArray read GetWayPoints write SetWayPoints;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsRoute'}{$ENDIF}
  ///<summary>
  ///  Eine einzelne Route mit einer Reihe von Streckenabschnitten im JSON-Format
  ///  in einem <c>TDirectionsResult</c>. Dieses Objekt hie frher
  ///  "DirectionsTrip".<br/><br/>
  ///  A single route containing a set of legs in a DirectionsResult. (This
  ///  object was formerly known as "DirectionsTrip".)<br/>
  ///  Note that though this object is "JSON-like," it is not strictly JSON, as
  ///  it directly and indirectly includes <c>TLatLng</c> objects.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsRoute.Bounds">Bounds</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRoute.Copyrights">Copyrights</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRoute.Legs">Legs</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRoute.OverviewPath">OverviewPath</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRoute.Warnings">Warnings</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsRoute.WayPointOrder">WayPointOrder</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsRoute">
  ///  DirectionsRoute (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsRoute">
  ///  DirectionsRoute (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsRoute = class(TCustomJScriptObject)
  private
    FBounds: TLatLngBounds;
    FLegs: TDirectionsLegArray;
    FOverviewPath: TLatLngArray;
    FWarnings: TStringArray;
    FWayPointOrder: TIntegerArray;
    function GetBounds: TLatLngBounds;
    function GetCopyrights: String;
    function GetLegs: TDirectionsLegArray;
    function GetOverviewPath: TLatLngArray;
    function GetWarnings: TStringArray;
    function GetWayPointOrder: TIntegerArray;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Bounds'}{$ENDIF}
    ///<summary>
    ///  Die Grenzen fr diese Route.<br/><br/>
    ///  The bounds for this route.
    ///</summary>
    ///<seealso cref="gmBase|TLatLngBounds"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRoute"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Bounds: TLatLngBounds read GetBounds;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Copyrights'}{$ENDIF}
    ///<summary>
    ///  Der Text zu den Urheberrechten, der fr diese Route angezeigt werden
    ///  soll.<br/><br/>
    ///  Copyrights text to be displayed for this route.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsRoute"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Copyrights: String read GetCopyrights;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Legs'}{$ENDIF}
    ///<summary>
    ///  Ein Array mit <c>TDirectionsLeg</c>'s, von denen jede Informationen
    ///  ber die darin enthaltenen Schritte enthlt. Fr jeden angegebenen
    ///  Wegpunkt oder Zielort ist genau ein Streckenabschnitt ("leg")
    ///  vorhanden. Eine Route ohne Wegpunkte enthlt also ein
    ///  <c>TDirectionsLeg</c> und eine Route mit einem Wegpunkt zwei. Diese
    ///  Eigenschaft hie frher "Routes".<br/><br/>
    ///  An array of <c>DirectionsLeg</c>'s, each of which contains information
    ///  about the steps of which it is composed. There will be one leg for each
    ///  waypoint or destination specified. So a route with no waypoints will
    ///  contain one <c>TDirectionsLeg</c> and a route with one waypoint will
    ///  contain two. (This property was formerly known as "routes".)
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsLeg"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRoute"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Legs: TDirectionsLegArray read GetLegs;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OverviewPath'}{$ENDIF}
    ///<summary>
    ///  Ein Array mit <c>TLatLng</c>-Werten, die den gesamten Verlauf der Route
    ///  wiedergeben. Der Pfad ist vereinfacht, damit er fr Kontexte geeignet
    ///  ist, in denen eine geringe Anzahl von Scheitelpunkten erforderlich ist,
    ///  beispielsweise in Static Maps API-URL's.<br/><br/>
    ///  An array of <c>TLatLng</c>'s representing the entire course of this
    ///  route. The path is simplified in order to make it suitable in contexts
    ///  where a small number of vertices is required (such as Static Maps API
    ///  URLs).
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRoute"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OverviewPath: TLatLngArray read GetOverviewPath;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Warnings'}{$ENDIF}
    ///<summary>
    ///  Warnungen, die angezeigt werden sollen, wenn diese Route dargestellt
    ///  wird.<br/><br/>
    ///  Warnings to be displayed when showing these directions.
    ///</summary>
    ///<seealso cref="JScriptObjects|TStringArray"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRoute"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Warnings: TStringArray read GetWarnings;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation WayPointOrder'}{$ENDIF}
    ///<summary>
    ///  Falls fr <c>OptimizeWayPoints</c> der Wert <c>true</c> festgelegt
    ///  wurde, enthlt dieses Feld die neue Anordnung der Eingabe-Wegpunkte.
    //   Wenn die Eingabe beispielsweise lautete:
    ///  <list type="bullet">
    ///    <item>Ursprungsort: Los Angeles</item>
    ///    <item>Wegpunkte: Dallas, Bangor, Phoenix</item>
    ///    <item>Zielort: New York</item>
    ///  </list>
    ///  und die optimierte Ausgabe wie folgt geordnet wurde:
    ///  <list type="bullet">
    ///    <item>Ursprungsort: Los Angeles</item>
    ///    <item>Wegpunkte: Phoenix, Dallas, Bangor</item>
    ///    <item>Zielort: New York</item>
    ///  </list>
    ///  dann besteht dieses Feld aus einem Array mit den Werten <c>[2,0,1]</c>.
    ///  Die Nummerierung der Wegpunkte ist Null basiert.<br/>
    ///  Falls bei einem beliebigen Eingabe-Wegpunkt fr <c>StopOver</c> der
    ///  Wert <c>false</c> festgelegt wurde, bleibt dieses Feld leer, da die
    ///  Routenoptimierung fr solche Anfragen nicht verfgbar ist.<br/><br/>
    ///  If <c>OptimizeWayPoints</c> was set to <c>true</c>, this field will
    ///  contain the re-ordered permutation of the input waypoints. For example,
    ///  if the input was:
    ///  <list type="bullet">
    ///    <item>Origin: Los Angeles</item>
    ///    <item>WayPoints: Dallas, Bangor, Phoenix</item>
    ///    <item>Destination: New York</item>
    ///  </list>
    ///  and the optimized output was ordered as follows:
    ///  <list type="bullet">
    ///    <item>Origin: Los Angeles</item>
    ///    <item>WayPoints: Phoenix, Dallas, Bangor</item>
    ///    <item>Destination: New York</item>
    ///  </list>
    ///  then this field will be an Array containing the values <c>[2,0,1]</c>.
    ///  Note that the numbering of waypoints is zero-based.
    ///  If any of the input waypoints has stopover set to <c>false</c>, this
    ///  field will be empty, since route optimization is not available for such
    ///  queries.
    ///</summary>
    ///<seealso cref="JScriptObjects|TIntegerArray"/>
    ///<seealso cref="gmServices.Directions|TDirectionsRoute"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property WayPointOrder: TIntegerArray read GetWayPointOrder;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsRouteArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;DirectionsRoute&gt;<br/><br/>
  ///  Objekt zur Verwaltung von <c>TDirectionsRoute</c>-Objekten.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsRouteArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  In JavaScript nicht vorhanden.
  ///</remarks>
  ///<seealso cref="gmServices.Directions|TDirectionsRoute"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsRouteArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TDirectionsRoute;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Ein Array mit <c>TDirectionsRoute</c>-Objekten.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsLegArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TDirectionsRoute read GetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsResult'}{$ENDIF}
  ///<summary>
  ///  The directions response retrieved from the directions server. You can
  ///  render these using a DirectionsRenderer or parse this object and render
  ///  it yourself. You must display the warnings and copyrights as noted in the
  ///  Maps API terms of service. Note that though this result is "JSON-like,"
  ///  it is not strictly JSON, as it indirectly includes <c>TLatLng</c> objects.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsResult.Routes">Routes</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmBase|TLatLng"/>
  ///<seealso cref="gmServices.Directions|TDirectionsService"/>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsResult">
  ///  DirectionsResult (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsResult">
  ///  DirectionsResult (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsResult = class(TCustomJScriptObject)
  private
    FRoutes: TDirectionsRouteArray;
    function GetRoutes: TDirectionsRouteArray;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Routes'}{$ENDIF}
    ///<summary>
    ///  Ein Array mit <c>TDirectionsRoute</c>-Objekten, von denen jede
    ///  Informationen ber die darin enthaltenen Streckenabschnitte und
    ///  Schritte enthlt. Es ist nur eine Route vorhanden, es sei denn, bei der
    ///  Anfrage war fr <c>ProvideRouteAlternatives</c> der Wert <c>true</c>
    ///  festgelegt.<br/>
    ///  Diese Eigenschaft hie frher "Trips".<br/><br/>
    ///  An array of <c>TDirectionsRoute</c>'s, each of which contains
    ///  information about the legs and steps of which it is composed. There
    ///  will only be one route unless the DirectionsRequest was made with
    ///  <c>ProvideRouteAlternatives</c> set to <c>true</c>.<br/>
    ///  This property was formerly known as "trips".
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Routes: TDirectionsRouteArray read GetRoutes;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsStatus'}{$ENDIF}
  ///<summary>
  ///  The status returned by the <c>TDirectionsService</c> on the completion
  ///  of a call to <c>Route()</c>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsStatus.InvalidRequest">InvalidRequest</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStatus.MaxWayPointsExceeded">MaxWayPointsExceeded</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStatus.NotFound">NotFound</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStatus.OK">OK</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStatus.OverQueryLimit">OverQueryLimit</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStatus.RequestDenied">RequestDenied</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStatus.UnknownError">UnknownError</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStatus.ZeroResults">ZeroResults</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmServices.Directions|TDirectionsService"/>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsStatus">
  ///  DirectionsStatus (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsStatus">
  ///  DirectionsStatus (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsStatus = class(TCustomJScriptObject)
  private
    procedure SetInvalidRequest(Value: String);
    procedure SetMaxWayPointsExceeded(Value: String);
    procedure SetNotFound(Value: String);
    procedure SetOK(Value: String);
    procedure SetOverQueryLimit(Value: String);
    procedure SetRequestDenied(Value: String);
    procedure SetUnknownError(Value: String);
    procedure SetZeroResults(Value: String);
    function GetInvalidRequest: String;
    function GetMaxWayPointsExceeded: String;
    function GetNotFound: String;
    function GetOK: String;
    function GetOverQueryLimit: String;
    function GetRequestDenied: String;
    function GetUnknownError: String;
    function GetZeroResults: String;
  protected
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InvalidRequest'}{$ENDIF}
    ///<summary>
    ///  Die DirectionsRequest ist ungltig.<br/><br/>
    ///  The DirectionsRequest provided was invalid.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property InvalidRequest: String read GetInvalidRequest write SetInvalidRequest;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MaxWayPointsExceeded'}{$ENDIF}
    ///<summary>
    ///  In der DirectionsRequest wurden zu viele DirectionsWayPoint angegeben.
    ///  Es sind maximal 8 Wegpunkte zulssig, plus Ursprungsort und Zielort.
    ///  <br/><br/>
    ///  Too many DirectionsWayPoints were provided in the DirectionsRequest.
    ///  The total allowed waypoints is 8, plus the origin and destination.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MaxWayPointsExceeded: String read GetMaxWayPointsExceeded write SetMaxWayPointsExceeded;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation NotFound'}{$ENDIF}
    ///<summary>
    ///  Mindestens einer der Orte (Ursprungsort, Zielort oder Wegpunkte) konnte
    ///  nicht geocodiert werden.<br/><br/>
    ///  At least one of the origin, destination, or waypoints could not be
    ///  geocoded.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property NotFound: String read GetNotFound write SetNotFound;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OK'}{$ENDIF}
    ///<summary>
    ///  Die Antwort enthlt eine gltige DirectionsResult.<br/><br/>
    ///  The response contains a valid DirectionsResult.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OK: String read GetOK write SetOK;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OverQueryLimit'}{$ENDIF}
    ///<summary>
    ///  Die Anfragebeschrnkungen wurden in einem zu geringen Zeitraum
    ///  berschritten.<br/><br/>
    ///  The webpage has gone over the requests limit in too short a period of
    ///  time.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OverQueryLimit: String read GetOverQueryLimit write SetOverQueryLimit;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RequestDenied'}{$ENDIF}
    ///<summary>
    ///  Es ist nicht gestattet, den Routendienst zu verwenden.<br/><br/>
    ///  The webpage is not allowed to use the directions service.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property RequestDenied: String read GetRequestDenied write SetRequestDenied;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UnknownError'}{$ENDIF}
    ///<summary>
    ///  Eine Routenanfrage konnte aufgrund eines Serverfehlers nicht
    ///  verarbeitet werden. Die Anfrage ist mglicherweise erfolgreich, wenn
    ///  sie erneut durchgefhrt wird.<br/><br/>
    ///  A directions request could not be processed due to a server error. The
    ///  request may succeed if you try again.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UnknownError: String read GetUnknownError write SetUnknownError;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZeroResults'}{$ENDIF}
    ///<summary>
    ///  Zwischen Ursprungsort und Zielort konnte keine Route gefunden werden.
    ///  <br/><br/>
    ///  No route could be found between the origin and destination.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZeroResults: String read GetZeroResults write SetZeroResults;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsStep'}{$ENDIF}
  ///<summary>
  ///  A single DirectionsStep in a <c>TDirectionsResult</c>. Some fields may be
  ///  undefined. Note that though this object is "JSON-like," it is not
  ///  strictly JSON, as it directly includes <c>TLatLng</c> objects.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsStep.Distance">Distance</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStep.Duration">Duration</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStep.EndLocation">EndLocation</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStep.Instructions">Instructions</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStep.Path">Path</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStep.StartLocation">StartLocation</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsStep.TravelMode">TravelMode</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmServices.Directions|TDirectionsResult"/>
  ///<seealso cref="gmBase|TLatLng"/>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsStep">
  ///  DirectionsStep (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsStep">
  ///  DirectionsStep (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsStep = class(TCustomJScriptObject)
  private
    FDistance: TDistance;
    FDuration: TDuration;
    FEndLocation: TLatLng;
    FPath: TLatLngArray;
    FStartLocation: TLatLng;
    function GetDistance: TDistance;
    function GetDuration: TDuration;
    function GetEndLocation: TLatLng;
    function GetInstructions: String;
    function GetPath: TLatLngArray;
    function GetStartLocation: TLatLng;
    function GetTravelMode: String;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Distance'}{$ENDIF}
    ///<summary>
    ///  Die Entfernung, die durch diesen Schritt zurckgelegt wird. Diese
    ///  Eigenschaft ist mglicherweise nicht definiert, falls die Entfernung
    ///  nicht bekannt ist.<br/><br/>
    ///  The distance covered by this step. This property may be undefined as
    ///  the distance may be unknown.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Distance: TDistance read GetDistance;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Duration'}{$ENDIF}
    ///<summary>
    ///  Die normalerweise zum Durchfhren dieses Schrittes erforderliche Zeit,
    ///  angegeben in Sekunden und Textform. Diese Eigenschaft ist
    ///  mglicherweise nicht definiert, falls die Dauer nicht bekannt ist.
    ///  <br/><br/>
    ///  The typical time required to perform this step in seconds and in text
    ///  form. This property may be undefined as the duration may be unknown.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Duration: TDuration read GetDuration;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation EndLocation'}{$ENDIF}
    ///<summary>
    ///  Der Endstandort dieses Schrittes.<br/><br/>
    ///  The ending location of this step.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property EndLocation: TLatLng read GetEndLocation;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Instructions'}{$ENDIF}
    ///<summary>
    ///  Anweisungen fr diesen Schritt.<br/><br/>
    ///  Instructions for this step.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Instructions: String read GetInstructions;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Path'}{$ENDIF}
    ///<summary>
    ///  Eine Folge von <c>TLatLng</c>-Werten, die den Verlauf dieses Schrittes
    ///  beschreiben.<br/><br/>
    ///  A sequence of LatLngs describing the course of this step.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmBase|TLatLngArray"/>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Path: TLatLngArray read GetPath;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StartLocation'}{$ENDIF}
    ///<summary>
    ///  Der Startstandort dieses Schrittes.<br/><br/>
    ///  The starting location of this step.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property StartLocation: TLatLng read GetStartLocation;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation TravelMode'}{$ENDIF}
    ///<summary>
    ///  The mode of travel used in this step.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TTravelMode"/>
    ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property TravelMode: String read GetTravelMode;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsStepArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;DirectionsStep&gt;<br/><br/>
  ///  Objekt zur Verwaltung von <c>TDirectionsStep</c>-Objekten.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsStepArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  In JavaScript nicht vorhanden.
  ///</remarks>
  ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsStepArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TDirectionsStep;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Ein Array mit <c>TDirectionsStep</c>-Objekten.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsLegArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TDirectionsStep read GetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsServiceCallback'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; Result: </c>
  ///  <see cref="gmServices.Directions|TDirectionsResult"/><c>; Status: </c>
  ///  <see cref="String"/><c>) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmServices.Directions|TDirectionsService"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsServiceCallback = procedure(Sender: TObject;
                                         Result: TDirectionsResult;
                                         Status: String) of object;
  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsServiceCallbackA'}{$ENDIF}
  ///<summary>
  ///  <c>reference to procedure(Sender: </c><see cref="TObject"/><c>; Result: </c>
  ///  <see cref="gmServices.Directions|TDirectionsResult"/><c>; Status: </c>
  ///  <see cref="String"/><c>);</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmServices.Directions|TDirectionsService"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TDirectionsServiceCallbackA = reference to procedure(Sender: TObject;
                                                       Result: TDirectionsResult;
                                                       Status: String);
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsService'}{$ENDIF}
  ///<summary>
  ///  A service for computing directions between two or more places.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsService.Route(TDirectionsRequest,TDirectionsServiceCallback)">Route(TDirectionsRequest,TDirectionsServiceCallback)</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsService.Route(TDirectionsRequest,TDirectionsServiceCallbackA)">Route(TDirectionsRequest,TDirectionsServiceCallbackA)</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsService">
  ///  DirectionsService (deutsch)
  ///</seealso>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsService">
  ///  DirectionsService (english)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsService = class(TCustomJScriptObject)
  private
    procedure CheckRequest(Request: TDirectionsRequest);
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; reintroduce; overload;
    constructor Create(Script: TCustomScript); reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Route'}{$ENDIF}
    ///<summary>
    ///  Sendet eine Suchanfrage fr eine Route.<br/><br/>
    ///  Issue a directions search request.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Route(Request: TDirectionsRequest; Callback: TDirectionsServiceCallback); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Route'}{$ENDIF}
    ///<summary>
    ///  Sendet eine Suchanfrage fr eine Route.<br/><br/>
    ///  Issue a directions search request.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure Route(Request: TDirectionsRequest; Callback: TDirectionsServiceCallbackA); overload;
    {$ENDIF}
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TTravelMode'}{$ENDIF}
  ///<summary>
  ///  The valid travel modes that can be specified in a
  ///  <c>TDirectionsRequest</c> as well as the travel modes returned in a
  ///  <c>TDirectionsStep</c>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TTravelMode.Bicycling">Bicycling</see></item>
  ///    <item><see cref="gmServices.Directions|TTravelMode.Driving">Driving</see></item>
  ///    <item><see cref="gmServices.Directions|TTravelMode.Walking">Walking</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  Renamed <c>TDirectionsTravelMode</c> to <c>TTravelMode</c> (v3.4 April 14, 2011)
  ///</remarks>
  ///<seealso cref="gmServices.Directions|TDirectionsRequest"/>
  ///<seealso cref="gmServices.Directions|TDirectionsStep"/>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#TravelMode">
  ///  TravelMode
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TTravelMode = class(TCustomJScriptObject)
  private
    procedure SetBicycling(Value: String);
    procedure SetDriving(Value: String);
    procedure SetWalking(Value: String);
    function GetBicycling: String;
    function GetDriving: String;
    function GetItem(Index: String): String;
    function GetWalking: String;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Bicycling'}{$ENDIF}
    ///<summary>
    ///  Kennzeichnet eine Routenanfrage fr Fahrrder.<br/><br/>
    ///  Specifies a bicycling directions request.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TTravelMode"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Bicycling: String read GetBicycling write SetBicycling;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Driving'}{$ENDIF}
    ///<summary>
    ///  Kennzeichnet eine Routenanfrage fr motorisierte Fahrzeuge.<br/><br/>
    ///  Specifies a driving directions request.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TTravelMode"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Driving: String read GetDriving write SetDriving;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Walking'}{$ENDIF}
    ///<summary>
    ///  Kennzeichnet eine Routenanfrage fr Fugnger.<br/><br/>
    ///  Specifies a walking directions request.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TTravelMode"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Walking: String read GetWalking write SetWalking;
    property Items[Index: String]: String read GetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsTravelMode'}{$ENDIF}
  ///<remarks>
  ///  Deprecated. Use <c>TTravelMode</c>.
  ///</remarks>
  ///<seealso cref="gmServices.Directions|TTravelMode"/>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsTravelMode">
  ///  DirectionsTravelMode (deutsch)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsTravelMode = class(TTravelMode);              //deprecated

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TUnitSystem'}{$ENDIF}
  ///<summary>
  ///  The valid unit systems that can be specified in a <c>TDirectionsRequest</c>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TUnitSystem.Imperial">Imperial</see></item>
  ///    <item><see cref="gmServices.Directions|TUnitSystem.Metric">Metric</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  Renamed <c>TDirectionsUnitSystem</c> to <c>TUnitSystem</c> (v3.4 April 14, 2011)
  ///</remarks>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#UnitSystem">
  ///  UnitSystem
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TUnitSystem = class(TCustomJScriptObject)
  private
    procedure SetImperial(Value: Integer);
    procedure SetMetric(Value: Integer);
    function GetImperial: Integer;
    function GetMetric: Integer;
  protected
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Imperial'}{$ENDIF}
    ///<summary>
    ///  Legt fest, dass Entfernungen im DirectionsResult in imperialen
    ///  Einheiten angegeben werden sollen.<br/><br/>
    ///  Specifies that distances in the DirectionsResult should be expressed
    ///  in imperial units.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TUnitSystem"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Imperial: Integer read GetImperial write SetImperial;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Metric'}{$ENDIF}
    ///<summary>
    ///  Legt fest, dass Entfernungen im DirectionsResult in metrischen
    ///  Einheiten angegeben werden sollen.<br/><br/>
    ///  Specifies that distances in the DirectionsResult should be expressed in
    ///  metric units.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TUnitSystem"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Metric: Integer read GetMetric write SetMetric;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsUnitSystem'}{$ENDIF}
  ///<remarks>
  ///  Deprecated. Use <c>TUnitSystem</c>.
  ///</remarks>
  ///<seealso cref="gmServices.Directions|TUnitSystem"/>
  ///<seealso href="http://code.google.com/intl/de-DE/apis/maps/documentation/javascript/reference.html#DirectionsUnitSystem">
  ///  DirectionsUnitSystem (deutsch)
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsUnitSystem = class(TUnitSystem);              //deprecated

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsWayPoint'}{$ENDIF}
  ///<summary>
  ///  A <c>TDirectionsWayPoint</c> represents a location between origin and
  ///  destination through which the trip should be routed.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsWayPoint.Location">Location</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsWayPoint.LocationAsString">LocationAsString</see></item>
  ///    <item><see cref="gmServices.Directions|TDirectionsWayPoint.StopOver">StopOver</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsWayPoint">
  ///  DirectionsWayPoint
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsWayPoint = class(TCustomJScriptObject)
  private
    FLocation: TLatLng;
    procedure SetLocation(Value: TLatLng);
    procedure SetLocationAsString(Value: String);
    procedure SetStopOver(Value: Boolean);
    function GetLocation: TLatLng;
    function GetLocationAsString: String;
    function GetStopOver: Boolean;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Location'}{$ENDIF}
    ///<summary>
    ///  Standort des Wegpunktes als <c>TLatLng</c>-Wert. Optional.<br/><br/>
    ///  WayPoint location as <c>TLatLng</c>. Optional.
    ///</summary>
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.Directions|TDirectionsWayPoint"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Location: TLatLng read GetLocation write SetLocation;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation LocationAsString'}{$ENDIF}
    ///<summary>
    ///  Standort des Wegpunktes als Text. Optional.<br/><br/>
    ///  WayPoint location as string. Optional.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsWayPoint"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property LocationAsString: String read GetLocationAsString write SetLocationAsString;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation StopOver'}{$ENDIF}
    ///<summary>
    ///  Falls <c>true</c>, ist der Wegpunkt ein Zwischenstopp zwischen
    ///  Ursprungsort und Zielort. Dies fhrt dazu, dass die Route in zwei Teile
    ///  geteilt wird.<br/>
    ///  Der Standardwert ist <c>true</c>. Optional.<br/><br/>
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsWayPoint"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property StopOver: Boolean read GetStopOver write SetStopOver;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDirectionsWayPointArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;DirectionsWayPoint&gt;<br/><br/>
  ///  Objekt zur Verwaltung von <c>TDirectionsWayPoint</c>-Objekten.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.Directions"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.Directions|TDirectionsWayPointArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  In JavaScript nicht vorhanden.
  ///</remarks>
  ///<seealso cref="gmServices.Directions|TDirectionsWayPoint"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDirectionsWayPointArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TDirectionsWayPoint;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<summary>
    ///  Ein Array mit <c>TDirectionsWayPoint</c>-Objekten.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsWayPointArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TDirectionsWayPoint read GetItem; default;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TDistance

function TDistance.GetText: String;
begin
  Result:=GetPropAsString(DistanceProp_Text);
end;

function TDistance.GetValue: Integer;
begin
  Result:=GetPropAsInteger(DistanceProp_Value);
end;

//------------------------------------------------------------------------------
//TDuration

function TDuration.GetText: String;
begin
  Result:=GetPropAsString(DurationProp_Text);
end;

function TDuration.GetValue: Integer;
begin
  Result:=GetPropAsInteger(DurationProp_Value);
end;

//------------------------------------------------------------------------------
//TDirectionsLeg

destructor TDirectionsLeg.Destroy;
begin
  FDistance.Free;
  FDuration.Free;
  FEndLocation.Free;
  FStartLocation.Free;
  FSteps.Free;
  FViaWayPoints.Free;
  inherited;
end;

function TDirectionsLeg.GetDistance: TDistance;
begin
  Result:=TDistance(GetPropAsObject(DirectionsLegProp_Distance,
                                    TDistance,FDistance));
end;

function TDirectionsLeg.GetDuration: TDuration;
begin
  Result:=TDuration(GetPropAsObject(DirectionsLegProp_Duration,
                                    TDuration,FDuration));
end;

function TDirectionsLeg.GetEndAddress: String;
begin
  Result:=GetPropAsString(DirectionsLegProp_EndAddress);
end;

function TDirectionsLeg.GetEndLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(DirectionsLegProp_EndLocation,
                                  TLatLng,FEndLocation));
end;

function TDirectionsLeg.GetStartAddress: String;
begin
  Result:=GetPropAsString(DirectionsLegProp_StartAddress);
end;

function TDirectionsLeg.GetStartLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(DirectionsLegProp_StartLocation,
                                  TLatLng,FStartLocation));
end;

function TDirectionsLeg.GetSteps: TDirectionsStepArray;
begin
  Result:=TDirectionsStepArray(GetPropAsObject(DirectionsLegProp_Steps,
                                               TDirectionsStepArray,FSteps));
end;

function TDirectionsLeg.GetViaWayPoints: TLatLngArray;
begin
  Result:=TLatLngArray(GetPropAsObject(DirectionsLegProp_ViaWayPoints,
                                       TLatLngArray,FViaWayPoints));
end;

//------------------------------------------------------------------------------
//TDirectionsLegArray

function TDirectionsLegArray.GetItem(Index: Integer): TDirectionsLeg;
begin
  Result:=TDirectionsLeg(inherited Items[Index]);
end;

function TDirectionsLegArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TDirectionsLeg.Create(Script,Dispatch);
end;

//------------------------------------------------------------------------------
//TDirectionsRequest

constructor TDirectionsRequest.Create;
begin
  inherited;
end;

constructor TDirectionsRequest.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TDirectionsRequest.Destroy;
begin
  FDestination.Free;
  FOrigin.Free;
  FWayPoints.Free;
  inherited;
end;

function TDirectionsRequest.GetAvoidHighways: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRequestProp_AvoidHighways);
end;

procedure TDirectionsRequest.SetAvoidHighways(Value: Boolean);
begin
  SetProp(DirectionsRequestProp_AvoidHighways,Value,true);
end;

function TDirectionsRequest.GetAvoidTolls: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRequestProp_AvoidTolls);
end;

procedure TDirectionsRequest.SetAvoidTolls(Value: Boolean);
begin
  SetProp(DirectionsRequestProp_AvoidTolls,Value,true);
end;

function TDirectionsRequest.GetDestination: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(DirectionsRequestProp_Destination,TLatLng,FDestination));
end;

procedure TDirectionsRequest.SetDestination(Value: TLatLng);
begin
  SetProp(DirectionsRequestProp_Destination,Value,true);
end;

function TDirectionsRequest.GetDestinationAsString: String;
begin
  Result:=GetPropAsString(DirectionsRequestProp_Destination);
end;

procedure TDirectionsRequest.SetDestinationAsString(Value: String);
begin
  SetProp(DirectionsRequestProp_Destination,Value,true);
end;

function TDirectionsRequest.GetOptimizeWayPoints: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRequestProp_OptimizeWayPoints);
end;

procedure TDirectionsRequest.SetOptimizeWayPoints(Value: Boolean);
begin
  SetProp(DirectionsRequestProp_OptimizeWayPoints,Value,true);
end;

function TDirectionsRequest.GetOrigin: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(DirectionsRequestProp_Origin,TLatLng,FOrigin));
end;

procedure TDirectionsRequest.SetOrigin(Value: TLatLng);
begin
  SetProp(DirectionsRequestProp_Origin,Value,true);
end;

function TDirectionsRequest.GetOriginAsString: String;
begin
  Result:=GetPropAsString(DirectionsRequestProp_Origin);
end;

procedure TDirectionsRequest.SetOriginAsString(Value: String);
begin
  SetProp(DirectionsRequestProp_Origin,Value,true);
end;

function TDirectionsRequest.GetProvideRouteAlternatives: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRequestProp_ProvideRouteAlternatives);
end;

procedure TDirectionsRequest.SetProvideRouteAlternatives(Value: Boolean);
begin
  SetProp(DirectionsRequestProp_ProvideRouteAlternatives,Value,true);
end;

function TDirectionsRequest.GetRegion: String;
begin
  Result:=GetPropAsString(DirectionsRequestProp_Region);
end;

procedure TDirectionsRequest.SetRegion(Value: String);
begin
  SetProp(DirectionsRequestProp_Region,Value,true);
end;

function TDirectionsRequest.GetTravelMode: String;
begin
  Result:=GetPropAsString(DirectionsRequestProp_TravelMode);
end;

procedure TDirectionsRequest.SetTravelMode(Value: String);
begin
  SetProp(DirectionsRequestProp_TravelMode,Value,true);
end;

function TDirectionsRequest.GetUnitSystem: Integer;
begin
  Result:=GetPropAsInteger(DirectionsRequestProp_UnitSystem);
end;

procedure TDirectionsRequest.SetUnitSystem(Value: Integer);
begin
  SetProp(DirectionsRequestProp_UnitSystem,Value,true);
end;

function TDirectionsRequest.GetWayPoints: TDirectionsWayPointArray;
begin
  Result:=TDirectionsWayPointArray(GetPropAsObject(DirectionsRequestProp_WayPoints,
                                                   TDirectionsWayPointArray,
                                                   FWayPoints,true));
end;

procedure TDirectionsRequest.SetWayPoints(Value: TDirectionsWayPointArray);
begin
  SetProp(DirectionsRequestProp_WayPoints,Value,true);
end;

//------------------------------------------------------------------------------
//TDirectionsRoute

destructor TDirectionsRoute.Destroy;
begin
  FBounds.Free;
  FLegs.Free;
  FOverviewPath.Free;
  FWarnings.Free;
  FWayPointOrder.Free;
  inherited;
end;

function TDirectionsRoute.GetBounds: TLatLngBounds;
begin
  Result:=TLatLngBounds(GetPropAsObject(DirectionsRouteProp_Bounds,TLatLngBounds,FBounds));
end;

function TDirectionsRoute.GetCopyrights: String;
begin
  Result:=GetPropAsString(DirectionsRouteProp_Copyrights);
end;

function TDirectionsRoute.GetLegs: TDirectionsLegArray;
begin
  Result:=TDirectionsLegArray(GetPropAsObject(DirectionsRouteProp_Legs,TDirectionsLegArray,FLegs));
end;

function TDirectionsRoute.GetOverviewPath: TLatLngArray;
begin
  Result:=TLatLngArray(GetPropAsObject(DirectionsRouteProp_OverviewPath,TLatLngArray,FOverviewPath));
end;

function TDirectionsRoute.GetWarnings: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(DirectionsRouteProp_Warnings,TStringArray,FWarnings));
end;

function TDirectionsRoute.GetWayPointOrder: TIntegerArray;
begin
  Result:=TIntegerArray(GetPropAsObject(DirectionsRouteProp_WayPointOrder,TIntegerArray,FWayPointOrder));
end;

//------------------------------------------------------------------------------
//TDirectionsRouteArray

function TDirectionsRouteArray.GetItem(Index: Integer): TDirectionsRoute;
begin
  Result:=TDirectionsRoute(inherited Items[Index]);
end;

function TDirectionsRouteArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TDirectionsRoute.Create(Script,Dispatch);
end;

//------------------------------------------------------------------------------
//TDirectionsResult.

destructor TDirectionsResult.Destroy;
begin
  FRoutes.Free;
  inherited;
end;

function TDirectionsResult.GetRoutes: TDirectionsRouteArray;
begin
  Result:=TDirectionsRouteArray(GetPropAsObject(DirectionsResultProp_Routes,
                                TDirectionsRouteArray,FRoutes));
end;

//------------------------------------------------------------------------------
//TDirectionsStatus

class function TDirectionsStatus.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[DirectionsStatusProp_InvalidRequest,
                                 DirectionsStatusProp_MaxWayPointsExceeded,
                                 DirectionsStatusProp_NotFound]);
end;

function TDirectionsStatus.GetInvalidRequest: String;
begin
  Result:=GetPropAsString(DirectionsStatusProp_InvalidRequest);
end;

procedure TDirectionsStatus.SetInvalidRequest(Value: String);
begin
  SetProp(DirectionsStatusProp_InvalidRequest,Value);
end;

function TDirectionsStatus.GetMaxWayPointsExceeded: String;
begin
  Result:=GetPropAsString(DirectionsStatusProp_MaxWayPointsExceeded);
end;

procedure TDirectionsStatus.SetMaxWayPointsExceeded(Value: String);
begin
  SetProp(DirectionsStatusProp_MaxWayPointsExceeded,Value);
end;

function TDirectionsStatus.GetNotFound: String;
begin
  Result:=GetPropAsString(DirectionsStatusProp_NotFound);
end;

procedure TDirectionsStatus.SetNotFound(Value: String);
begin
  SetProp(DirectionsStatusProp_NotFound,Value);
end;

function TDirectionsStatus.GetOK: String;
begin
  Result:=GetPropAsString(DirectionsStatusProp_OK);
end;

procedure TDirectionsStatus.SetOK(Value: String);
begin
  SetProp(DirectionsStatusProp_OK,Value);
end;

function TDirectionsStatus.GetOverQueryLimit: String;
begin
  Result:=GetPropAsString(DirectionsStatusProp_OverQueryLimit);
end;

procedure TDirectionsStatus.SetOverQueryLimit(Value: String);
begin
  SetProp(DirectionsStatusProp_OverQueryLimit,Value);
end;

function TDirectionsStatus.GetRequestDenied: String;
begin
  Result:=GetPropAsString(DirectionsStatusProp_RequestDenied);
end;

procedure TDirectionsStatus.SetRequestDenied(Value: String);
begin
  SetProp(DirectionsStatusProp_RequestDenied,Value);
end;

function TDirectionsStatus.GetUnknownError: String;
begin
  Result:=GetPropAsString(DirectionsStatusProp_UnknownError);
end;

procedure TDirectionsStatus.SetUnknownError(Value: String);
begin
  SetProp(DirectionsStatusProp_UnknownError,Value);
end;

function TDirectionsStatus.GetZeroResults: String;
begin
  Result:=GetPropAsString(DirectionsStatusProp_ZeroResults);
end;

procedure TDirectionsStatus.SetZeroResults(Value: String);
begin
  SetProp(DirectionsStatusProp_ZeroResults,Value);
end;

//------------------------------------------------------------------------------
//TDirectionsStep

destructor TDirectionsStep.Destroy;
begin
  FDistance.Free;
  FDuration.Free;
  FEndLocation.Free;
  FPath.Free;
  FStartLocation.Free;
  inherited;
end;

function TDirectionsStep.GetDistance: TDistance;
begin
  Result:=TDistance(GetPropAsObject(DirectionsStepProp_Distance,
                                    TDistance,FDistance));
end;

function TDirectionsStep.GetDuration: TDuration;
begin
  Result:=TDuration(GetPropAsObject(DirectionsStepProp_Duration,
                                    TDuration,FDuration));
end;

function TDirectionsStep.GetEndLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(DirectionsStepProp_EndLocation,
                                  TLatLng,FEndLocation));
end;

function TDirectionsStep.GetInstructions: String;
begin
  Result:=GetPropAsString(DirectionsStepProp_Instructions);
end;

function TDirectionsStep.GetPath: TLatLngArray;
begin
  Result:=TLatLngArray(GetPropAsObject(DirectionsStepProp_Path,
                                       TLatLngArray,FPath));
end;

function TDirectionsStep.GetStartLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(DirectionsStepProp_StartLocation,
                                  TLatLng,FStartLocation));
end;

function TDirectionsStep.GetTravelMode: String;
begin
  Result:=GetPropAsString(DirectionsStepProp_TravelMode);
end;

//------------------------------------------------------------------------------
//TDirectionsStepArray

function TDirectionsStepArray.GetItem(Index: Integer): TDirectionsStep;
begin
  Result:=TDirectionsStep(inherited Items[Index]);
end;

function TDirectionsStepArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TDirectionsStep.Create(Script,Dispatch);
end;

//------------------------------------------------------------------------------
//TTravelMode

class function TTravelMode.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[TravelModeProp_Bicycling,
                                 TravelModeProp_Driving,
                                 TravelModeProp_Walking]);
end;

function TTravelMode.GetBicycling: String;
begin
  Result:=GetPropAsString(TravelModeProp_Bicycling);
end;

procedure TTravelMode.SetBicycling(Value: String);
begin
  SetProp(TravelModeProp_Bicycling,Value);
end;

function TTravelMode.GetDriving: String;
begin
  Result:=GetPropAsString(TravelModeProp_Driving);
end;

function TTravelMode.GetItem(Index: String): String;
begin
  Result:='';
  if UpperCase(Index)='BICYCLING'
    then Result:=GetBicycling;
  if UpperCase(Index)='DRIVING'
    then Result:=GetDriving;
  if UpperCase(Index)='WALKING'
    then Result:=GetWalking;
end;

procedure TTravelMode.SetDriving(Value: String);
begin
  SetProp(TravelModeProp_Driving,Value);
end;

function TTravelMode.GetWalking: String;
begin
  Result:=GetPropAsString(TravelModeProp_Walking);
end;

procedure TTravelMode.SetWalking(Value: String);
begin
  SetProp(TravelModeProp_Walking,Value);
end;

//------------------------------------------------------------------------------
//TDirectionsService

constructor TDirectionsService.Create;
begin
  inherited;
end;

constructor TDirectionsService.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TDirectionsService.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_DirectionsService,true);
end;

type
  TDirectionsServiceCallbackObject = class(TCallbackObject)
  private
    FCallback: TDirectionsServiceCallback;
    FDirectionsService: TDirectionsService;
  protected
    procedure Callback; override;
  public
    constructor Create(DirectionsService: TDirectionsService;
                       Request: TDirectionsRequest;
                       Callback: TDirectionsServiceCallback); reintroduce;
  end;

constructor TDirectionsServiceCallbackObject.Create(DirectionsService: TDirectionsService;
                                                    Request: TDirectionsRequest;
                                                    Callback: TDirectionsServiceCallback);
begin
  inherited Create(DirectionsService.Script);
  FCallback:=Callback;
  FDirectionsService:=DirectionsService;
  FDirectionsService.Exec(DirectionsServiceMethod_Route,[Request.Disp,JScriptFunction.Disp]);
end;

procedure TDirectionsServiceCallbackObject.Callback;
var
  DirectionsResult: TDirectionsResult;
begin
  try
    if ArgumentCount>=2 then
    begin
      DirectionsResult:=nil;
      if VarIsValidDispatch(Arguments[0])
        then DirectionsResult:=TDirectionsResult(CreateObjectWrapper(Script,
                                                                     TDirectionsResult,
                                                                     Arguments[0]));
      try
        FCallback(FDirectionsService,DirectionsResult,Arguments[1]);
      finally
        DirectionsResult.Free;
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TDirectionsService.CheckRequest(Request: TDirectionsRequest);
begin
  if not assigned(Request)
    then raise Exception.Create(ExceptionNoRequest);
  if not Request.PropertyExists(DirectionsRequestProp_Origin)
    then raise Exception.Create(ExceptionNoOrigin);
  if not Request.PropertyExists(DirectionsRequestProp_Destination)
    then raise Exception.Create(ExceptionNoDestination);
  CheckMaps(Script);
  if not Request.PropertyExists(DirectionsRequestProp_TravelMode)
    then Request.TravelMode:=TScript(Script).Google.Maps.TravelMode.Driving; //Property is requiret.
end;

procedure TDirectionsService.Route(Request: TDirectionsRequest;
                                   Callback: TDirectionsServiceCallback);
begin
  if assigned(Request) and assigned(Callback) then
  begin
    CheckRequest(Request);
    TDirectionsServiceCallbackObject.Create(Self,Request,Callback);
  end;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
type
  TDirectionsServiceCallbackObjectA = class(TCallbackObject)
  private
    FCallback: TDirectionsServiceCallbackA;
    FDirectionsService: TDirectionsService;
  protected
    procedure Callback; override;
  public
    constructor Create(DirectionsService: TDirectionsService;
                       Request: TDirectionsRequest;
                       Callback: TDirectionsServiceCallbackA); reintroduce;
  end;

constructor TDirectionsServiceCallbackObjectA.Create(DirectionsService: TDirectionsService;
                                                     Request: TDirectionsRequest;
                                                     Callback: TDirectionsServiceCallbackA);
begin
  inherited Create(DirectionsService.Script);
  FCallback:=Callback;
  FDirectionsService:=DirectionsService;
  FDirectionsService.Exec(DirectionsServiceMethod_Route,
                      {$IFDEF CAN_OVERLOAD}
                      [Request.Disp,JScriptFunction.Disp]
                      {$ELSE}
                      Request.Disp,JScriptFunction.Disp
                      {$ENDIF});
end;

procedure TDirectionsServiceCallbackObjectA.Callback;
var
  DirectionsResult: TDirectionsResult;
begin
  try
    if ArgumentCount>=2 then
    begin
      DirectionsResult:=nil;
      if VarIsValidDispatch(Arguments[0])
        then DirectionsResult:=TDirectionsResult(CreateObjectWrapper(Script,
                                                                     TDirectionsResult,
                                                                     Arguments[0]));
      try
        FCallback(FDirectionsService,DirectionsResult,Arguments[1]);
      finally
        DirectionsResult.Free;
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TDirectionsService.Route(Request: TDirectionsRequest;
                                   Callback: TDirectionsServiceCallbackA);
begin
  if assigned(Request) and assigned(Callback) then
  begin
    CheckRequest(Request);
    TDirectionsServiceCallbackObjectA.Create(Self,Request,Callback);
  end;
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TUnitSystem

class function TUnitSystem.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[UnitSystemProp_Imperial,
                                 UnitSystemProp_Metric]);
end;

function TUnitSystem.GetImperial: Integer;
begin
  Result:=GetPropAsInteger(UnitSystemProp_Imperial);
end;

procedure TUnitSystem.SetImperial(Value: Integer);
begin
  SetProp(UnitSystemProp_Imperial,Value);
end;

function TUnitSystem.GetMetric: Integer;
begin
  Result:=GetPropAsInteger(UnitSystemProp_Metric);
end;

procedure TUnitSystem.SetMetric(Value: Integer);
begin
  SetProp(UnitSystemProp_Metric,Value);
end;

//------------------------------------------------------------------------------
//TDirectionsWayPoint

constructor TDirectionsWayPoint.Create;
begin
  inherited;
end;

constructor TDirectionsWayPoint.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TDirectionsWayPoint.Destroy;
begin
  FLocation.Free;
  inherited;
end;

function TDirectionsWayPoint.GetLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(DirectionsWayPointProp_Location,TLatLng,FLocation));
end;

procedure TDirectionsWayPoint.SetLocation(Value: TLatLng);
begin
  SetProp(DirectionsWayPointProp_Location,Value,true);
end;

function TDirectionsWayPoint.GetLocationAsString: String;
begin
  Result:=GetPropAsString(DirectionsWayPointProp_Location);
end;

procedure TDirectionsWayPoint.SetLocationAsString(Value: String);
begin
  SetProp(DirectionsWayPointProp_Location,Value,true);
end;

function TDirectionsWayPoint.GetStopOver: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsWayPointProp_StopOver);
end;

procedure TDirectionsWayPoint.SetStopOver(Value: Boolean);
begin
  SetProp(DirectionsWayPointProp_StopOver,Value,true);
end;

//------------------------------------------------------------------------------
//TDirectionsWayPointArray

function TDirectionsWayPointArray.GetItem(Index: Integer): TDirectionsWayPoint;
begin
  Result:=TDirectionsWayPoint(inherited Items[Index]);
end;

function TDirectionsWayPointArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TDirectionsWayPoint.Create(Script,Dispatch);
end;

end.
