unit gmServicesDirectionsRenderer;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils,
  BrowserTools, DispObject, JScriptObjects, HTMLObjects, gmBase, gmMVC, gmMap,
  gmEvents, gmOverlaysMarker, gmOverlaysPolylinePolygon, gmOverlaysInfoWindow,
  gmServicesDirections, gmAdditionalConstants;

type
  TDirectionsRenderer = class;
  TDirectionsRendererOptions = class;

  //Renders directions retrieved in the form of a DirectionsResult object
  //retrieved from the DirectionsService. This class extends MVCObject.
  //http://code.google.com/apis/maps/documentation/javascript/reference.html#DirectionsRenderer
  //Creates the renderer with the given options. Directions can be rendered on
  //a map (as visual overlays) or additionally on a <div> panel (as textual
  //instructions).
  ///<summary>
  ///  Blub
  ///</summary>
  TDirectionsRenderer = class(TMVCObject)
  private
    FDirections: TDirectionsResult;
    FMap: TMap;
    FPanel: TNode;
    FOnDirectionsChanged: TNotifyEvent;
    FOnDirectionsChangedListener: TMapsEventListener;
    FOnDirectionsChangedFunction: TJScriptFunction;
    procedure SetOnDirectionsChanged(Value: TNotifyEvent);
  protected
    class function New(Script: TCustomScript): IDispatch; overload; override;
    class function New(Script: TCustomScript; Opts: TDirectionsRendererOptions): IDispatch; reintroduce; overload;
    //JavaScript event handler:
    procedure DirectionsChanged;
  public
    constructor Create(Opts: TDirectionsRendererOptions = nil); reintroduce; overload;
    constructor Create(Script: TCustomScript; Opts: TDirectionsRendererOptions = nil); reintroduce; overload;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetDirections'}{$ENDIF}
    ///<summary>
    ///  Legt fest, dass der Renderer das Ergebnis vom DirectionsService
    ///  verwenden soll. Wenn auf diese Weise gltige Routeninformationen
    ///  festgelegt werden, wird die Route in der dafr vorgesehenen Karte und
    ///  Bereich des Renderers angezeigt.<br/><br/>
    ///  Set the renderer to use the result from the DirectionsService. Setting
    ///  a valid set of directions in this manner will display the directions
    ///  on the renderer's designated map and panel.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetDirections(Directions: TDirectionsResult);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetMap'}{$ENDIF}
    ///<summary>
    ///  Mit dieser Methode wird die Karte angegeben, auf der die Route angezeigt
    ///  werden soll. Mit <c>nil</c> wird die Route von der Karte entfernt.<br/><br/>
    ///  This method specifies the map on which directions will be rendered.
    ///  Pass <c>nil</c> to remove the directions from the map.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetMap(Map: TMap);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetOptions'}{$ENDIF}
    ///<summary>
    ///  Change the options settings of this DirectionsRenderer after
    ///  initialization.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetOptions(Opts: TDirectionsRendererOptions); reintroduce;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetPanel'}{$ENDIF}
    ///<summary>
    ///  Mit dieser Methode wird die Route in einem <c>&lt;div&gt;</c>-Element
    ///  wiedergegeben. Mit <c>nil</c> wird der Inhalt aus dem Bereich
    ///  entfernt.<br/><br/>
    ///  This method renders the directions in a &lt;div&gt;. Pass <c>nil</c> to
    ///  remove the content from the panel.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetPanel(Panel: TNode);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SetRouteIndex'}{$ENDIF}
    ///<summary>
    ///  Legt den auf Null basierenden Index der Route im wiederzugebenden
    ///  DirectionsResult-Objekt fest. Standardmig wird die erste Route im
    ///  Array zurckgegeben.<br/><br/>
    ///  Set the (zero-based) index of the route in the DirectionsResult object to
    ///  render. By default, the first route in the array will be rendered.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure SetRouteIndex(RouteIndex: Integer);
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetDirections'}{$ENDIF}
    ///<summary>
    ///  Gibt die aktuellen Routeninformationen des Renderers wieder.<br/><br/>
    ///  Returns the renderer's current set of directions.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetDirections: TDirectionsResult;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetMap'}{$ENDIF}
    ///<summary>
    ///  Gibt die Karte wieder, auf der das DirectionsResult dargestellt wird.
    ///  <br/><br/>
    ///  Returns the map on which the DirectionsResult is rendered.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetMap: TMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetPanel'}{$ENDIF}
    ///<summary>
    ///  Gibt das <c>&lt;div&gt;</c>-Bereichselement wieder, in dem das
    ///  DirectionsResult dargestellt wird.<br/><br/>
    ///  Returns the panel <c>&lt;div&gt;</c> in which the DirectionsResult is rendered.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetPanel: TNode;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetRouteIndex'}{$ENDIF}
    ///<summary>
    ///  Gibt den aktuellen, auf Null basierenden Routenindex wieder, der von
    ///  diesem DirectionsRenderer-Objekt verwendet wird.<br/><br/>
    ///  Returns the current (zero-based) route index in use by this
    ///  DirectionsRenderer object.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function GetRouteIndex: Integer;
    //zustzliche Delphi-Eigenschaften:
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Directions'}{$ENDIF}
    ///<summary>
    ///  Aktuelle Routeninformationen des Renderers.<br/><br/>
    ///  The renderer's current set of directions.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Directions: TDirectionsResult read GetDirections write SetDirections;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Map'}{$ENDIF}
    ///<summary>
    ///  Karte, auf der die Route angezeigt wird. Mit <c>nil</c> wird die Route
    ///  von der Karte entfernt.<br/><br/>
    ///  Map on which directions will be rendered. Pass <c>nil</c> to remove the
    ///  directions from the map.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Map: TMap read GetMap write SetMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Panel'}{$ENDIF}
    ///<summary>
    ///  <c>&lt;div&gt;</c>-Element, in dem die Route wiedergegeben wird. Mit
    ///  <c>nil</c> wird der Inhalt aus diesem Bereich entfernt.<br/><br/>
    ///  &lt;div&gt; node, in which the directions is rendered. Set <c>nil</c> to
    ///  remove the content from the panel.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Panel: TNode read GetPanel write SetPanel;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RouteIndex'}{$ENDIF}
    ///<summary>
    ///  Auf Null basierenden Index der Route im wiederzugebenden
    ///  <c>TDirectionsResult</c>-Objekt. Standardmig wird die erste Route im
    ///  Array angezeigt.<br/><br/>
    ///  Zero-based index of the route in the DirectionsResult object to
    ///  render. By default, the first route in the array will be rendered.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property RouteIndex: Integer read GetRouteIndex write SetRouteIndex;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OnDirectionsChanged'}{$ENDIF}
    ///<summary>
    ///  This event is fired when the rendered directions change, either when a
    ///  new <c>TDirectionsResult</c> is set or when the user finishes dragging
    ///  a change to the directions path.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRenderer"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OnDirectionsChanged: TNotifyEvent read FOnDirectionsChanged write SetOnDirectionsChanged;
  end;

  TDirectionsRendererOptions = class(TCustomJScriptObject)
  private
    FDirections: TDirectionsResult;
    FInfoWindow: TInfoWindow;
    FMap: TMap;
    FMarkerOptions: TMarkerOptions;
    FPanel: TNode;
    FPolylineOptions: TPolylineOptions;
    procedure SetDirections(Value: TDirectionsResult);
    procedure SetDraggable(Value: Boolean);
    procedure SetHideRouteList(Value: Boolean);
    procedure SetInfoWindow(Value: TInfoWindow);
    procedure SetMap(Value: TMap);
    procedure SetMarkerOptions(Value: TMarkerOptions);
    procedure SetPanel(Value: TNode);
    procedure SetPolylineOptions(Value: TPolylineOptions);
    procedure SetPreserveViewport(Value: Boolean);
    procedure SetRouteIndex(Value: Integer);
    procedure SetSuppressBicyclingLayer(Value: Boolean);
    procedure SetSuppressInfoWindows(Value: Boolean);
    procedure SetSuppressMarkers(Value: Boolean);
    procedure SetSuppressPolylines(Value: Boolean);
    function GetDirections: TDirectionsResult;
    function GetDraggable: Boolean;
    function GetHideRouteList: Boolean;
    function GetInfoWindow: TInfoWindow;
    function GetMap: TMap;
    function GetMarkerOptions: TMarkerOptions;
    function GetPanel: TNode;
    function GetPolylineOptions: TPolylineOptions;
    function GetPreserveViewport: Boolean;
    function GetRouteIndex: Integer;
    function GetSuppressBicyclingLayer: Boolean;
    function GetSuppressInfoWindows: Boolean;
    function GetSuppressMarkers: Boolean;
    function GetSuppressPolylines: Boolean;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Directions'}{$ENDIF}
    ///<summary>
    ///  Die Route, die auf der Karte und/oder in einem <c>&lt;div&gt;</c>-Bereich
    ///  angezeigt werden soll und die als <c>TDirectionsResult</c>-Objekt
    ///  vom DirectionsService abgerufen wird.<br/><br/>
    ///  The directions to display on the map and/or in a <c>&lt;div&gt;</c> panel,
    ///  retrieved as a <c>TDirectionsResult</c> object from DirectionsService.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDirectionsResult"/>
    ///<seealso cref="gmServices.Directions|TDirectionsService"/>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Directions: TDirectionsResult read GetDirections write SetDirections;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Draggable'}{$ENDIF}
    ///<summary>
    ///  If true, allows the user to drag and modify the paths of routes
    ///  rendered by this DirectionsRenderer.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Draggable: Boolean read GetDraggable write SetDraggable;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation HideRouteList'}{$ENDIF}
    ///<summary>
    ///  Diese Eigenschaft gibt an, ob der Renderer eine UI fr die Auswahl
    ///  zwischen alternativen Routen zur Verfgung stellen soll. Standardmig
    ///  besitzt diese Markierung den Wert <c>false</c> und im zugehrigen
    ///  Bereich der Route wird eine Liste von Routen angezeigt, zwischen denen
    ///  der Nutzer auswhlen kann. Um diese Liste zu verbergen,mu fr
    ///  <c>HideRouteList</c> der Wert <c>true</c> festgelegt werden.<br/><br/>
    ///  This property indicates whether the renderer should provide UI to select
    ///  amongst alternative routes. By default, this flag is <c>false</c> and a
    ///  user-selectable list of routes will be shown in the directions'
    ///  associated panel. To hide that list, set hideRouteList to <c>true</c>.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property HideRouteList: Boolean read GetHideRouteList write SetHideRouteList;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InfoWindow'}{$ENDIF}
    ///<summary>
    ///  The InfoWindow in which to render text information when a marker is
    ///  clicked. Existing info window content will be overwritten and its
    ///  position moved. If no info window is specified, the DirectionsRenderer
    ///  will create and use its own info window. This property will be ignored
    ///  if <c>SuppressInfoWindows</c> is set to <c>true</c>.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property InfoWindow: TInfoWindow read GetInfoWindow write SetInfoWindow;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Map'}{$ENDIF}
    ///<summary>
    ///  Karte, auf der die Route angezeigt werden soll.<br/><br/>
    ///  Map on which to display the directions.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Map: TMap read GetMap write SetMap;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MarkerOptions'}{$ENDIF}
    ///<summary>
    ///  Optionen fr die Markierungen. Diese Optionen werden fr alle
    ///  Markierungen verwendet, die vom DirectionsRenderer zurckgegeben werden.
    ///  <br/><br/>
    ///  Options for the markers. All markers rendered by the DirectionsRenderer
    ///  will use these options.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MarkerOptions: TMarkerOptions read GetMarkerOptions write SetMarkerOptions;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Panel'}{$ENDIF}
    ///<summary>
    ///  Das <c>&lt;div&gt;</c>-Element, in dem die Routenschritte angezeigt
    ///  werden sollen.<br/><br/>
    ///  The <c>&lt;div&gt;</c> in which to display the directions steps.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Panel: TNode read GetPanel write SetPanel;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PolylineOptions'}{$ENDIF}
    ///<summary>
    ///  Optionen fr die Polylinien. Diese Optionen werden fr alle Polylinien
    ///  verwendet, die vom DirectionsRenderer zurckgegeben werden.<br/><br/>
    ///  Options for the polylines. All polylines rendered by the
    ///  DirectionsRenderer will use these options.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PolylineOptions: TPolylineOptions read GetPolylineOptions write SetPolylineOptions;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation PreserveViewport'}{$ENDIF}
    ///<summary>
    ///  Standardmig wird die Eingabekarte bezglich des Markierungsrahmens
    ///  dieser Routeninformationen zentriert und gezoomt. Wenn fr diese Option
    ///  der Wert <c>true</c> festgelegt wird, bleibt der Darstellungsbereich
    ///  unverndert, es sei denn, der Zoomwert und der Mittelpunkt der Karte
    ///  wurden nie festgelegt.<br/><br/>
    ///  By default, the input map is centered and zoomed to the bounding box of
    ///  this set of directions. If this option is set to true, the viewport is
    ///  left unchanged, unless the map's center and zoom were never set.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property PreserveViewport: Boolean read GetPreserveViewport write SetPreserveViewport;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RouteIndex'}{$ENDIF}
    ///<summary>
    ///  Der Index der Route innerhalb des DirectionsResult-Objekts. Der
    ///  Standardwert ist 0.<br/><br/>
    ///  The index of the route within the DirectionsResult object. The default
    ///  value is 0.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property RouteIndex: Integer read GetRouteIndex write SetRouteIndex;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SuppressBicyclingLayer'}{$ENDIF}
    ///<summary>
    ///  Unterdrckt die Wiedergabe der BicyclingLayer, wenn Routen fr Fahrrder
    ///  angefragt werden.<br/><br/>
    ///  Suppress the rendering of the BicyclingLayer when bicycling directions
    ///  are requested.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property SuppressBicyclingLayer: Boolean read GetSuppressBicyclingLayer write SetSuppressBicyclingLayer;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SuppressInfoWindows'}{$ENDIF}
    ///<summary>
    ///  Unterdrckt die Wiedergabe von Infofenstern.<br/><br/>
    ///  Suppress the rendering of the BicyclingLayer when bicycling directions
    //are requested.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property SuppressInfoWindows: Boolean read GetSuppressInfoWindows write SetSuppressInfoWindows;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SuppressMarkers'}{$ENDIF}
    ///<summary>
    ///  Unterdrckt die Wiedergabe von Markierungen.<br/><br/>
    ///  Suppress the rendering of markers.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property SuppressMarkers: Boolean read GetSuppressMarkers write SetSuppressMarkers;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation SuppressPolylines'}{$ENDIF}
    ///<summary>
    ///  Unterdrckt die Wiedergabe von Polylinien.<br/><br/>
    ///  Suppress the rendering of polylines.
    ///</summary>
    ///<seealso cref="gmServices.DirectionsRenderer|TDirectionsRendererOptions"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property SuppressPolylines: Boolean read GetSuppressPolylines write SetSuppressPolylines;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TDirectionsRenderer

constructor TDirectionsRenderer.Create(Opts: TDirectionsRendererOptions);
begin
  Create(JScriptObjects.Script,Opts);
end;

constructor TDirectionsRenderer.Create(Script: TCustomScript; Opts: TDirectionsRendererOptions);
begin
  inherited Create(Script,New(Script,Opts));
end;

destructor TDirectionsRenderer.Destroy;
begin
  SetOnDirectionsChanged(nil);
  FDirections.Free;
  FMap.Free;
  FPanel.Free;
  inherited;
end;

class function TDirectionsRenderer.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_DirectionsRenderer,true);
end;

class function TDirectionsRenderer.New(Script: TCustomScript;
                                       Opts: TDirectionsRendererOptions): IDispatch;
begin
  CheckMaps(Script);
  if not assigned(Opts)
    then Result:=New(Script)
    else Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                                     MapsMethod_DirectionsRenderer,Opts.Disp,true);
end;

function TDirectionsRenderer.GetDirections: TDirectionsResult;
begin
  Result:=TDirectionsResult(GetObjectWithExec(DirectionsRendererMethod_GetDirections,
                                              TDirectionsResult,FDirections));
end;

procedure TDirectionsRenderer.SetDirections(Directions: TDirectionsResult);
begin
  Exec(DirectionsRendererMethod_SetDirections,Directions);
end;

function TDirectionsRenderer.GetMap: TMap;
begin
  Result:=TMap(GetObjectWithExec(DirectionsRendererMethod_GetMap,TMap,FMap));
end;

procedure TDirectionsRenderer.SetMap(Map: TMap);
begin
  Exec(DirectionsRendererMethod_SetMap,Map);
end;

procedure TDirectionsRenderer.SetOptions(Opts: TDirectionsRendererOptions);
begin
  inherited SetOptions(Opts);
end;

procedure TDirectionsRenderer.DirectionsChanged;
begin
  CallEvent(FOnDirectionsChanged);
end;

procedure TDirectionsRenderer.SetOnDirectionsChanged(Value: TNotifyEvent);
begin
  FOnDirectionsChanged:=Value;
  RemoveListener(FOnDirectionsChangedListener);
  if assigned(FOnDirectionsChanged)
    then AddListener(DirectionsRendererEvent_DirectionsChanged,
                     FOnDirectionsChangedListener,
                     FOnDirectionsChangedFunction,
                     DirectionsChanged);
end;

function TDirectionsRenderer.GetPanel: TNode;
begin
  Result:=TNode(GetObjectWithExec(DirectionsRendererMethod_GetPanel,TNode,FPanel));
end;

procedure TDirectionsRenderer.SetPanel(Panel: TNode);
begin
  Exec(DirectionsRendererMethod_SetPanel,Panel);
end;

function TDirectionsRenderer.GetRouteIndex: Integer;
begin
  Result:=VarAsInteger(Exec(DirectionsRendererMethod_GetRouteIndex));
end;

procedure TDirectionsRenderer.SetRouteIndex(RouteIndex: Integer);
begin
  Exec(DirectionsRendererMethod_SetRouteIndex,Map);
end;

//------------------------------------------------------------------------------
//TDirectionsRendererOptions

constructor TDirectionsRendererOptions.Create;
begin
  inherited;
end;

constructor TDirectionsRendererOptions.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TDirectionsRendererOptions.Destroy;
begin
  FDirections.Free;
  FMap.Free;
  FMarkerOptions.Free;
  FPanel.Free;
  FPolylineOptions.Free;
  inherited;
end;

function TDirectionsRendererOptions.GetDirections: TDirectionsResult;
begin
  Result:=TDirectionsResult(GetPropAsObject(DirectionsRendererOptionsProp_Directions,
                                           TDirectionsResult,FDirections));
end;

procedure TDirectionsRendererOptions.SetDirections(Value: TDirectionsResult);
begin
  SetProp(DirectionsRendererOptionsProp_Directions,Value,true);
end;

function TDirectionsRendererOptions.GetDraggable: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRendererOptionsProp_Draggable);
end;

procedure TDirectionsRendererOptions.SetDraggable(Value: Boolean);
begin
  SetProp(DirectionsRendererOptionsProp_Draggable,Value,true);
end;

function TDirectionsRendererOptions.GetHideRouteList: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRendererOptionsProp_HideRouteList);
end;

procedure TDirectionsRendererOptions.SetHideRouteList(Value: Boolean);
begin
  SetProp(DirectionsRendererOptionsProp_HideRouteList,Value,true);
end;

function TDirectionsRendererOptions.GetInfoWindow: TInfoWindow;
begin
  Result:=TInfoWindow(GetPropAsObject(DirectionsRendererOptionsProp_InfoWindow,
                                      TInfoWindow,FInfoWindow));
end;

procedure TDirectionsRendererOptions.SetInfoWindow(Value: TInfoWindow);
begin
  SetProp(DirectionsRendererOptionsProp_InfoWindow,Value,true);
end;

function TDirectionsRendererOptions.GetMap: TMap;
begin
  Result:=TMap(GetPropAsObject(DirectionsRendererOptionsProp_Map,TMap,FMap));
end;

procedure TDirectionsRendererOptions.SetMap(Value: TMap);
begin
  SetProp(DirectionsRendererOptionsProp_Map,Value,true);
end;

function TDirectionsRendererOptions.GetMarkerOptions: TMarkerOptions;
begin
  Result:=TMarkerOptions(GetPropAsObject(DirectionsRendererOptionsProp_MarkerOptions,
                                         TMarkerOptions,FMarkerOptions,true));
end;

procedure TDirectionsRendererOptions.SetMarkerOptions(Value: TMarkerOptions);
begin
  SetProp(DirectionsRendererOptionsProp_MarkerOptions,Value,true);
end;

function TDirectionsRendererOptions.GetPanel: TNode;
begin
  Result:=TNode(GetPropAsObject(DirectionsRendererOptionsProp_Panel,
                                TNode,FPanel));
end;

procedure TDirectionsRendererOptions.SetPanel(Value: TNode);
begin
  SetProp(DirectionsRendererOptionsProp_Panel,Value,true);
end;

function TDirectionsRendererOptions.GetPolylineOptions: TPolylineOptions;
begin
  Result:=TPolylineOptions(GetPropAsObject(DirectionsRendererOptionsProp_PolylineOptions,
                                           TPolylineOptions,FPolylineOptions,true));
end;

procedure TDirectionsRendererOptions.SetPolylineOptions(Value: TPolylineOptions);
begin
  SetProp(DirectionsRendererOptionsProp_PolylineOptions,Value,true);
end;

function TDirectionsRendererOptions.GetRouteIndex: Integer;
begin
  Result:=GetPropAsInteger(DirectionsRendererOptionsProp_RouteIndex);
end;

procedure TDirectionsRendererOptions.SetRouteIndex(Value: Integer);
begin
  SetProp(DirectionsRendererOptionsProp_RouteIndex,Value,true);
end;

procedure TDirectionsRendererOptions.SetPreserveViewport(Value: Boolean);
begin
  SetProp(DirectionsRendererOptionsProp_PreserveViewport,Value,true);
end;

function TDirectionsRendererOptions.GetPreserveViewport: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRendererOptionsProp_PreserveViewport);
end;

function TDirectionsRendererOptions.GetSuppressBicyclingLayer: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRendererOptionsProp_SuppressBicyclingLayer);
end;

procedure TDirectionsRendererOptions.SetSuppressBicyclingLayer(Value: Boolean);
begin
  SetProp(DirectionsRendererOptionsProp_SuppressBicyclingLayer,Value,true);
end;

function TDirectionsRendererOptions.GetSuppressInfoWindows: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRendererOptionsProp_SuppressInfoWindows);
end;

procedure TDirectionsRendererOptions.SetSuppressInfoWindows(Value: Boolean);
begin
  SetProp(DirectionsRendererOptionsProp_SuppressInfoWindows,Value,true);
end;

function TDirectionsRendererOptions.GetSuppressMarkers: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRendererOptionsProp_SuppressMarkers);
end;

procedure TDirectionsRendererOptions.SetSuppressMarkers(Value: Boolean);
begin
  SetProp(DirectionsRendererOptionsProp_SuppressMarkers,Value,true);
end;

function TDirectionsRendererOptions.GetSuppressPolylines: Boolean;
begin
  Result:=GetPropAsBoolean(DirectionsRendererOptionsProp_SuppressPolylines);
end;

procedure TDirectionsRendererOptions.SetSuppressPolylines(Value: Boolean);
begin
  SetProp(DirectionsRendererOptionsProp_SuppressPolylines,Value,true);
end;

end.
