///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixElementStatus"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixRequest"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponse"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElement"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseRow"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixService"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/></item>
///  </list>
///  <para><b>Additional Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElementArray"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseRowArray"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TLatLngStringArray"/></item>
///  </list>
///  <para><b>Types</b></para>
///  <list type="bullet">
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixServiceCallback"/></item>
///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixServiceCallbackA"/></item>
///  </list>
///</summary>
unit gmServicesDistanceMatrix;

{$INCLUDE gmConfig.inc}

interface

uses
   SysUtils, {$IFDEF DEBUG}Dialogs,{$ENDIF} BrowserTools, JScriptObjects,
   DispObject, gmBase, gmServicesDirections, gmAdditionalConstants,
   gmAdditionalMessages;

const
  MaxDestinations = 25;                                    //max. destinations per request
  MaxOrigins      = 25;                                    //max. origins per request
  MaxElements     = 100;                                   //max. elements per request (origins times destinations)

type
  TDistanceMatrixElementStatus = class;
  TDistanceMatrixRequest = class;
  TDistanceMatrixResponse = class;
  TDistanceMatrixResponseElement = class;
  TDistanceMatrixResponseElementArray = class;
  TDistanceMatrixResponseRow = class;
  TDistanceMatrixResponseRowArray = class;
  TDistanceMatrixService = class;
  TDistanceMatrixStatus = class;
  TLatLngStringArray = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixServiceCallback'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; Response: </c>
  ///  <see cref="gmServices.DistanceMatrix|TDistanceMatrixResponse"/><c>; Status: </c>
  ///  <see cref="String"/><c>) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixService"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixServiceCallback = procedure(Sender: TObject;
                                             Response: TDistanceMatrixResponse;
                                             Status: String) of object;
  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixServiceCallbackA'}{$ENDIF}
  ///<summary>
  ///  <c>reference to procedure(Sender: </c><see cref="TObject"/><c>; Response: </c>
  ///  <see cref="gmServices.DistanceMatrix|TDistanceMatrixResponse"/><c>; Status: </c>
  ///  <see cref="String"/><c>);</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixService"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TDistanceMatrixServiceCallbackA = reference to procedure(Sender: TObject;
                                                           Response: TDistanceMatrixResponse;
                                                           Status: String);
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixElementStatus'}{$ENDIF}
  ///<summary>
  ///  The element-level status about a particular origin-destination pairing
  ///  returned by the DistanceMatrixService upon completion of a distance
  ///  matrix request.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixElementStatus.NotFound">NotFound</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixElementStatus.OK">OK</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixElementStatus.ZeroResults">ZeroResults</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DistanceMatrixElementStatus">
  ///  DistanceMatrixElementStatus
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixElementStatus = class(TCustomJScriptObject)
  private
    function GetNotFound: String;
    function GetOK: String;
    function GetZeroResults: String;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation NotFound'}{$ENDIF}
    ///<summary>
    ///  The origin and/or destination of this pairing could not be geocoded.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixElementStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property NotFound: String read GetNotFound;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OK'}{$ENDIF}
    ///<summary>
    ///  The response contains a valid result.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixElementStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OK: String read GetOK;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation ZeroResults'}{$ENDIF}
    ///<summary>
    ///  No route could be found between the origin and destination.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixElementStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property ZeroResults: String read GetZeroResults;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixRequest'}{$ENDIF}
  ///<summary>
  ///  A distance matrix query sent by the DistanceMatrixService containing
  ///  arrays of origin and destination locations, and various options for
  ///  computing metrics.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixRequest.AvoidHighways">AvoidHighways</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixRequest.AvoidTolls">AvoidTolls</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixRequest.Destinations">Destinations</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixRequest.Origins">Origins</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixRequest.Region">Region</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixRequest.TravelMode">TravelMode</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixRequest.UnitSystem">UnitSystem</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DistanceMatrixRequest">
  ///  DistanceMatrixRequest
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixRequest = class(TCustomJScriptObject)
  private
    FDestinations: TLatLngStringArray;
    FOrigins: TLatLngArray;
    procedure SetAvoidHighways(Value: Boolean);
    procedure SetAvoidTolls(Value: Boolean);
    procedure SetDestinations(Value: TLatLngStringArray);
    procedure SetOrigins(Value: TLatLngStringArray);
    procedure SetRegion(Value: String);
    procedure SetTravelMode(Value: String);
    procedure SetUnitSystem(Value: Integer);
    function GetAvoidHighways: Boolean;
    function GetAvoidTolls: Boolean;
    function GetDestinations: TLatLngStringArray;
    function GetOrigins: TLatLngStringArray;
    function GetRegion: String;
    function GetTravelMode: String;
    function GetUnitSystem: Integer;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    procedure AfterConstruction; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AvoidHighways'}{$ENDIF}
    ///<summary>
    ///  If true, instructs the Distance Matrix service to avoid highways where
    ///  possible. Optional.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property AvoidHighways: Boolean read GetAvoidHighways write SetAvoidHighways;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation AvoidTolls'}{$ENDIF}
    ///<summary>
    ///  If true, instructs the Distance Matrix service to avoid toll roads
    ///  where possible. Optional.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property AvoidTolls: Boolean read GetAvoidTolls write SetAvoidTolls;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Destinations'}{$ENDIF}
    ///<summary>
    ///  An array containing destination address strings and/or LatLngs, to
    ///  which to calculate distance and time. Required.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Destinations: TLatLngStringArray read GetDestinations write SetDestinations;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Origins'}{$ENDIF}
    ///<summary>
    ///  An array containing origin address strings and/or LatLngs, from which
    ///  to calculate distance and time. Required.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Origins: TLatLngStringArray read GetOrigins write SetOrigins;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Region'}{$ENDIF}
    ///<summary>
    ///  Region code used as a bias for geocoding requests. Optional.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Region: String read GetRegion write SetRegion;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation TravelMode'}{$ENDIF}
    ///<summary>
    ///  Type of routing requested. Required.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property TravelMode: String read GetTravelMode write SetTravelMode;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UnitSystem'}{$ENDIF}
    ///<summary>
    ///  Preferred unit system to use when displaying distance. Optional;
    ///  defaults to metric.
    ///</summary>
    ///<seealso cref="gmAPI|TMaps.UnitSystem"/>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixRequest"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UnitSystem: Integer read GetUnitSystem write SetUnitSystem;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixResponse'}{$ENDIF}
  ///<summary>
  ///  The response to a DistanceMatrixService request, consisting of the
  ///  formatted origin and destination addresses, and a sequence of
  ///  <c>TDistanceMatrixResponseRow</c>'s, one for each corresponding origin
  ///  address.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponse.DestinationAddresses">DestinationAddresses</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponse.OriginAddresses">OriginAddresses</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponse.Rows">Rows</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DistanceMatrixResponse">
  ///  DistanceMatrixResponse
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixResponse = class(TCustomJScriptObject)
  private
    FDestinationAddresses: TStringArray;
    FOriginAddresses: TStringArray;
    FRows: TDistanceMatrixResponseRowArray;
    procedure SetDestinationAddresses(Value: TStringArray);
    procedure SetOriginAddresses(Value: TStringArray);
    procedure SetRows(Value: TDistanceMatrixResponseRowArray);
    function GetDestinationAddresses: TStringArray;
    function GetOriginAddresses: TStringArray;
    function GetRows: TDistanceMatrixResponseRowArray;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation DestinationAddresses'}{$ENDIF}
    ///<summary>
    ///  The formatted destination addresses.
    ///</summary>
    ///<seealso cref="JScriptObjects|TStringArray"/>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponse"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property DestinationAddresses: TStringArray read GetDestinationAddresses write SetDestinationAddresses;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OriginAddresses'}{$ENDIF}
    ///<summary>
    ///  The formatted origin addresses.
    ///</summary>
    ///<seealso cref="JScriptObjects|TStringArray"/>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponse"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OriginAddresses: TStringArray read GetOriginAddresses write SetOriginAddresses;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Rows'}{$ENDIF}
    ///<summary>
    ///  The rows of the matrix, corresponding to the origin addresses.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponse"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Rows: TDistanceMatrixResponseRowArray read GetRows write SetRows;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixResponseElement'}{$ENDIF}
  ///<summary>
  ///  A single element of a response to a DistanceMatrixService request, which
  ///  contains the duration and distance from one origin to one destination.
  ///  address.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElement.Distance">Distance</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElement.Duration">Duration</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElement.Status">Status</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DistanceMatrixResponseElement">
  ///  DistanceMatrixResponseElement
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixResponseElement = class(TCustomJScriptObject)
  private
    FDistance: TDistance;
    FDuration: TDuration;
    procedure SetDistance(Value: TDistance);
    procedure SetDuration(Value: TDuration);
    procedure SetStatus(Value: String);
    function GetDistance: TDistance;
    function GetDuration: TDuration;
    function GetStatus: String;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Distance'}{$ENDIF}
    ///<summary>
    ///  The distance for this origin-destination pairing. This property may be
    ///  undefined as the distance may be unknown.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDistance"/>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElement"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Distance: TDistance read GetDistance write SetDistance;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Duration'}{$ENDIF}
    ///<summary>
    ///  The duration for this origin-destination pairing. This property may be
    ///  undefined as the duration may be unknown.
    ///</summary>
    ///<seealso cref="gmServices.Directions|TDuration"/>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElement"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Duration: TDuration read GetDuration write SetDuration;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Status'}{$ENDIF}
    ///<summary>
    ///  The status of this particular origin-destination pairing.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElement"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Status: String read GetStatus write SetStatus;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixResponseElementArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;DistanceMatrixResponseElement&gt;<br/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElementArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponseRow"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixResponseElementArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TDistanceMatrixResponseElement;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponseElementArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TDistanceMatrixResponseElement read GetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixResponseRow'}{$ENDIF}
  ///<summary>
  ///  A row of the response to a DistanceMatrixService request, consisting of
  ///  a sequence of <c>TDistanceMatrixResponseElement</c>'s, one for each corresponding
  ///  destination address.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseRow.Elements">Elements</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DistanceMatrixResponseRow">
  ///  DistanceMatrixResponseRow
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixResponseRow = class(TCustomJScriptObject)
  private
    FElements: TDistanceMatrixResponseElementArray;
    procedure SetElements(Value: TDistanceMatrixResponseElementArray);
    function GetElements: TDistanceMatrixResponseElementArray;
  public
    destructor Destroy; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Elements'}{$ENDIF}
    ///<summary>
    ///  The row's elements, corresponding to the destination addresses.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponseRow"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Elements: TDistanceMatrixResponseElementArray read GetElements write SetElements;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixResponseRowArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;DistanceMatrixResponseRow&gt;<br/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TObjectArray"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixResponseRowArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponseRow"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixResponseRowArray = class(TObjectArray)
  private
    function GetItem(Index: Integer): TDistanceMatrixResponseRow;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixResponseRowArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: TDistanceMatrixResponseRow read GetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixService'}{$ENDIF}
  ///<summary>
  ///  A service for computing distances between multiple origins and destinations.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item>
  ///      <see cref="gmServices.DistanceMatrix|TDistanceMatrixService.GetDistanceMatrix(TDistanceMatrixRequest,TDistanceMatrixServiceCallback)">
  ///        GetDistanceMatrix
  ///      </see>
  ///    </item>
  ///    <item>
  ///      <see cref="gmServices.DistanceMatrix|TDistanceMatrixService.GetDistanceMatrix(TDistanceMatrixRequest,TDistanceMatrixServiceCallbackA)">
  ///        GetDistanceMatrix
  ///      </see>
  ///    </item>
  ///  </list>
  ///</summary>
  ///<remarks>
  ///  <para><b>Quotas</b></para>
  //   <para/>
  ///  The following usage limits are in place for the Distance Matrix service:
  ///  <list type="bullet">
  ///    <item>25 origins and/or 25 destinations per request</item>
  ///    <item>100 elements per request (origins times destinations)</item>
  ///  </list>
  ///  Requests are also rate limited. If too many elements are requested within
  ///  a certain time period, an <c>OVER_QUERY_LIMIT</c> response code will be returned.
  ///</remarks>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/services.html#distance_matrix">Distance Matrix</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixService = class(TCustomJScriptObject)
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; reintroduce; overload;
    constructor Create(Script: TCustomScript); reintroduce; overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetDistanceMatrix'}{$ENDIF}
    ///<summary>
    ///  Issues a distance matrix request.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure GetDistanceMatrix(Request: TDistanceMatrixRequest; Callback: TDistanceMatrixServiceCallback); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetDistanceMatrix'}{$ENDIF}
    ///<summary>
    ///  Issues a distance matrix request.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure GetDistanceMatrix(Request: TDistanceMatrixRequest; Callback: TDistanceMatrixServiceCallbackA); overload;
    {$ENDIF}
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TDistanceMatrixStatus'}{$ENDIF}
  ///<summary>
  ///  The top-level status about the request in general returned by the
  ///  DistanceMatrixService upon completion of a distance matrix request.
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixStatus.InvalidRequest">InvalidRequest</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixStatus.MaxDimensionsExceeded">MaxDimensionsExceeded</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixStatus.MaxElementsExceeded">MaxElementsExceeded</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixStatus.OK">OK</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixStatus.OverQueryLimit">OverQueryLimit</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixStatus.RequestDenied">RequestDenied</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TDistanceMatrixStatus.UnknownError">UnknownError</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#DistanceMatrixStatus">
  ///  DistanceMatrixStatus
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TDistanceMatrixStatus = class(TCustomJScriptObject)
  private
    function GetInvalidRequest: String;
    function GetMaxDimensionsExceeded: String;
    function GetMaxElementsExceeded: String;
    function GetOK: String;
    function GetOverQueryLimit: String;
    function GetRequestDenied: String;
    function GetUnknownError: String;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation InvalidRequest'}{$ENDIF}
    ///<summary>
    ///  The provided request was invalid.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property InvalidRequest: String read GetInvalidRequest;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MaxDimensionsExceeded'}{$ENDIF}
    ///<summary>
    ///  The request contains more than 25 origins, or more than 25
    ///  destinations.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MaxDimensionsExceeded: String read GetMaxDimensionsExceeded;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation MaxElementsExceeded'}{$ENDIF}
    ///<summary>
    ///  The product of origins and destinations exceeds the per-query limit.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property MaxElementsExceeded: String read GetMaxElementsExceeded;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OK'}{$ENDIF}
    ///<summary>
    ///  The response contains a valid result.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OK: String read GetOK;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OverQueryLimit'}{$ENDIF}
    ///<summary>
    ///  Too many elements have been requested within the allowed time period.
    ///  The request should succeed if you try again after a reasonable amount
    ///  of time.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OverQueryLimit: String read GetOverQueryLimit;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation RequestDenied'}{$ENDIF}
    ///<summary>
    ///  The service denied use of the Distance Matrix service by your program.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property RequestDenied: String read GetRequestDenied;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation UnknownError'}{$ENDIF}
    ///<summary>
    ///  A Distance Matrix request could not be processed due to a server error.
    ///  The request may succeed if you try again.
    ///</summary>
    ///<seealso cref="gmServices.DistanceMatrix|TDistanceMatrixStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property UnknownError: String read GetUnknownError;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TLatLngStringArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&lt;LatLng||string&gt;<br/>
  ///  An array containing strings and/or <c>TLatLng</c>'s
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.DistanceMatrix"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.DistanceMatrix|TLatLngStringArray.Insert(Integer,String)">Insert(Integer,String)</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TLatLngStringArray.Insert(Integer,TLatLng)">Insert(Integer,TLatLng)</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TLatLngStringArray.Add(String)">Add(String)</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TLatLngStringArray.Add(array of String)">Add(array of String)</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TLatLngStringArray.Add(TLatLng)">Add(TLatLng)</see></item>
  ///    <item><see cref="gmServices.DistanceMatrix|TLatLngStringArray.Add(array of TLatLng)">Add(array of TLatLng)</see></item>
  ///  </list>
  ///</summary>
  ///<seealso cref="gmBase|TLatLng"/>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TLatLngStringArray = class(TJScriptArrayEx)
  private
    FMaxPoints: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Insert'}{$ENDIF}
    ///<seealso cref="gmServices.DistanceMatrix|TLatLngStringArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Insert(Index: Integer; Value: String); reintroduce; overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Insert'}{$ENDIF}
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.DistanceMatrix|TLatLngStringArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Insert(Index: Integer; Value: TLatLng); reintroduce; overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<seealso cref="gmServices.DistanceMatrix|TLatLngStringArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(Value: String): Integer; reintroduce; overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<seealso cref="gmServices.DistanceMatrix|TLatLngStringArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(Values: array of String): Integer; reintroduce; overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<seealso cref="gmBase|TLatLng"/>
    ///<seealso cref="gmServices.DistanceMatrix|TLatLngStringArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(Value: TLatLng): Integer; reintroduce; overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<seealso cref="gmServices.DistanceMatrix|TLatLngStringArray"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(Values: array of TLatLng): Integer; reintroduce; overload; virtual;
  end;

var
  RequestElements: Integer = 0;                            //Elements count (origins times destinations)

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TDistanceMatrixElementStatus

function TDistanceMatrixElementStatus.GetNotFound: String;
begin
  Result:=GetPropAsString(DistanceMatrixElementStatusProp_NotFound);
end;

function TDistanceMatrixElementStatus.GetOK: String;
begin
  Result:=GetPropAsString(DistanceMatrixElementStatusProp_OK);
end;

function TDistanceMatrixElementStatus.GetZeroResults: String;
begin
  Result:=GetPropAsString(DistanceMatrixElementStatusProp_ZeroResults);
end;

//------------------------------------------------------------------------------
//TDistanceMatrixRequest

constructor TDistanceMatrixRequest.Create;
begin
  inherited;
end;

procedure TDistanceMatrixRequest.AfterConstruction;
begin
  inherited;
  //Required properties:
  GetDestinations;
  GetOrigins;
  TravelMode:=TScript(Script).Google.Maps.TravelMode.Driving;
end;

constructor TDistanceMatrixRequest.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TDistanceMatrixRequest.Destroy;
begin
  FDestinations.Free;
  FOrigins.Free;
  inherited;
end;

function TDistanceMatrixRequest.GetAvoidHighways: Boolean;
begin
  Result:=GetPropAsBoolean(DistanceMatrixRequestProp_AvoidHighways);
end;

function TDistanceMatrixRequest.GetAvoidTolls: Boolean;
begin
  Result:=GetPropAsBoolean(DistanceMatrixRequestProp_AvoidTolls);
end;

function TDistanceMatrixRequest.GetDestinations: TLatLngStringArray;
begin
  Result:=TLatLngStringArray(GetPropAsObject(DistanceMatrixRequestProp_Destinations,
                                             TLatLngStringArray,
                                             FDestinations,true));
  Result.FMaxPoints:=MaxDestinations;
end;

function TDistanceMatrixRequest.GetOrigins: TLatLngStringArray;
begin
  Result:=TLatLngStringArray(GetPropAsObject(DistanceMatrixRequestProp_Origins,
                                             TLatLngStringArray,
                                             FOrigins,true));
  Result.FMaxPoints:=MaxOrigins;
end;

function TDistanceMatrixRequest.GetRegion: String;
begin
  Result:=GetPropAsString(DistanceMatrixRequestProp_Region);
end;

function TDistanceMatrixRequest.GetTravelMode: String;
begin
  Result:=GetPropAsString(DistanceMatrixRequestProp_TravelMode);
end;

function TDistanceMatrixRequest.GetUnitSystem: Integer;
begin
  Result:=GetPropAsInteger(DistanceMatrixRequestProp_UnitSystem);
end;

procedure TDistanceMatrixRequest.SetAvoidHighways(Value: Boolean);
begin
  SetProp(DistanceMatrixRequestProp_AvoidHighways,Value,true);
end;

procedure TDistanceMatrixRequest.SetAvoidTolls(Value: Boolean);
begin
  SetProp(DistanceMatrixRequestProp_AvoidTolls,Value,true);
end;

procedure TDistanceMatrixRequest.SetDestinations(Value: TLatLngStringArray);
begin
  SetProp(DistanceMatrixRequestProp_Destinations,Value,true);
end;

procedure TDistanceMatrixRequest.SetOrigins(Value: TLatLngStringArray);
begin
  SetProp(DistanceMatrixRequestProp_Origins,Value,true);
end;

procedure TDistanceMatrixRequest.SetRegion(Value: String);
begin
  SetProp(DistanceMatrixRequestProp_Region,Value,true);
end;

procedure TDistanceMatrixRequest.SetTravelMode(Value: String);
begin
  SetProp(DistanceMatrixRequestProp_TravelMode,Value,true);
end;

procedure TDistanceMatrixRequest.SetUnitSystem(Value: Integer);
begin
  SetProp(DistanceMatrixRequestProp_UnitSystem,Value,true);
end;

//------------------------------------------------------------------------------
//TDistanceMatrixResponse

destructor TDistanceMatrixResponse.Destroy;
begin
  FDestinationAddresses.Free;
  FOriginAddresses.Free;
  FRows.Free;
  inherited;
end;

function TDistanceMatrixResponse.GetDestinationAddresses: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(DistanceMatrixResponseProp_DestinationAddresses,
                                       TStringArray,FDestinationAddresses));
end;

function TDistanceMatrixResponse.GetOriginAddresses: TStringArray;
begin
  Result:=TStringArray(GetPropAsObject(DistanceMatrixResponseProp_OriginAddresses,
                                       TStringArray,FOriginAddresses));
end;

function TDistanceMatrixResponse.GetRows: TDistanceMatrixResponseRowArray;
begin
  Result:=TDistanceMatrixResponseRowArray(GetPropAsObject(DistanceMatrixResponseProp_Rows,
                                          TDistanceMatrixResponseRowArray,FRows));
end;

procedure TDistanceMatrixResponse.SetDestinationAddresses(Value: TStringArray);
begin
  SetProp(DistanceMatrixResponseProp_DestinationAddresses,Value,true);
end;

procedure TDistanceMatrixResponse.SetOriginAddresses(Value: TStringArray);
begin
  SetProp(DistanceMatrixResponseProp_OriginAddresses,Value,true);
end;

procedure TDistanceMatrixResponse.SetRows(Value: TDistanceMatrixResponseRowArray);
begin
  SetProp(DistanceMatrixResponseProp_Rows,Value,true);
end;

//------------------------------------------------------------------------------
//TDistanceMatrixResponseElement

destructor TDistanceMatrixResponseElement.Destroy;
begin
  FDistance.Free;
  FDuration.Free;
  inherited;
end;

function TDistanceMatrixResponseElement.GetDistance: TDistance;
begin
  Result:=TDistance(GetPropAsObject(DistanceMatrixResponseElementProp_Distance,
                                    TDistance,FDistance));
end;

function TDistanceMatrixResponseElement.GetDuration: TDuration;
begin
  Result:=TDuration(GetPropAsObject(DistanceMatrixResponseElementProp_Duration,
                                    TDuration,FDuration));
end;

function TDistanceMatrixResponseElement.GetStatus: String;
begin
  Result:=GetPropAsString(DistanceMatrixResponseElementProp_Status);
end;

procedure TDistanceMatrixResponseElement.SetDistance(Value: TDistance);
begin
  SetProp(DistanceMatrixResponseElementProp_Distance,Value,true);
end;

procedure TDistanceMatrixResponseElement.SetDuration(Value: TDuration);
begin
  SetProp(DistanceMatrixResponseElementProp_Duration,Value,true);
end;

procedure TDistanceMatrixResponseElement.SetStatus(Value: String);
begin
  SetProp(DistanceMatrixResponseElementProp_Status,Value,true);
end;

//------------------------------------------------------------------------------
//TDistanceMatrixResponseElementArray

function TDistanceMatrixResponseElementArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TDistanceMatrixResponseElement.Create(Script,Dispatch);
end;

function TDistanceMatrixResponseElementArray.GetItem(Index: Integer): TDistanceMatrixResponseElement;
begin
  Result:=TDistanceMatrixResponseElement(inherited Items[Index]);
end;

//------------------------------------------------------------------------------
//TDistanceMatrixResponseRow

destructor TDistanceMatrixResponseRow.Destroy;
begin
  FElements.Free;
  inherited;
end;

function TDistanceMatrixResponseRow.GetElements: TDistanceMatrixResponseElementArray;
begin
  Result:=TDistanceMatrixResponseElementArray(
          GetPropAsObject(DistanceMatrixResponseRowProp_Elements,
                          TDistanceMatrixResponseElementArray,FElements));
end;

procedure TDistanceMatrixResponseRow.SetElements(Value: TDistanceMatrixResponseElementArray);
begin
  SetProp(DistanceMatrixResponseRowProp_Elements,Value,true);
end;

//------------------------------------------------------------------------------
//TDistanceMatrixResponseRowArray

function TDistanceMatrixResponseRowArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=TDistanceMatrixResponseRow.Create(Script,Dispatch);
end;

function TDistanceMatrixResponseRowArray.GetItem(Index: Integer): TDistanceMatrixResponseRow;
begin
  Result:=TDistanceMatrixResponseRow(inherited Items[Index]);
end;

//------------------------------------------------------------------------------
//TDistanceMatrixService

constructor TDistanceMatrixService.Create;
begin
  inherited;
end;

constructor TDistanceMatrixService.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TDistanceMatrixService.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_DistanceMatrixService,true);
end;

type
  TDistanceMatrixServiceCallbackObject = class(TCallbackObject)
  private
    FCallback: TDistanceMatrixServiceCallback;
    FDistanceMatrixService: TDistanceMatrixService;
  protected
    procedure Callback; override;
  public
    constructor Create(DistanceMatrixService: TDistanceMatrixService;
                       Request: TDistanceMatrixRequest;
                       Callback: TDistanceMatrixServiceCallback); reintroduce;
  end;

constructor TDistanceMatrixServiceCallbackObject.Create(DistanceMatrixService: TDistanceMatrixService;
                                                        Request: TDistanceMatrixRequest;
                                                        Callback: TDistanceMatrixServiceCallback);
begin
  inherited Create(DistanceMatrixService.Script);
  FCallback:=Callback;
  FDistanceMatrixService:=DistanceMatrixService;
  FDistanceMatrixService.Exec(DistanceMatrixServiceMethod_GetDistanceMatrix,
                              [Request.Disp,JScriptFunction.Disp]);
end;

procedure TDistanceMatrixServiceCallbackObject.Callback;
var
  DistanceMatrixResponse: TDistanceMatrixResponse;
begin
  try
    if ArgumentCount>=2 then
    begin
      DistanceMatrixResponse:=nil;
      if VarIsValidDispatch(Arguments[0])
        then DistanceMatrixResponse:=TDistanceMatrixResponse.Create(Script,Arguments[0]);
      try
        FCallback(FDistanceMatrixService,DistanceMatrixResponse,Arguments[1]);
      finally
        DistanceMatrixResponse.Free;
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TDistanceMatrixService.GetDistanceMatrix(Request: TDistanceMatrixRequest;
                                                   Callback: TDistanceMatrixServiceCallback);
var
  Elements: Integer;
begin
  if assigned(Request) and assigned(Callback) then
  begin
    Elements:=Request.Origins.Count*Request.Destinations.Count;
    if Elements>MaxElements
      then raise Exception.Create(ExceptionToManyElements);
    Inc(RequestElements,Elements);
    TDistanceMatrixServiceCallbackObject.Create(Self,Request,Callback);
  end;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
type
  TDistanceMatrixServiceCallbackObjectA = class(TCallbackObject)
  private
    FCallback: TDistanceMatrixServiceCallbackA;
    FDistanceMatrixService: TDistanceMatrixService;
  protected
    procedure Callback; override;
  public
    constructor Create(DistanceMatrixService: TDistanceMatrixService;
                       Request: TDistanceMatrixRequest;
                       Callback: TDistanceMatrixServiceCallbackA); reintroduce;
  end;

constructor TDistanceMatrixServiceCallbackObjectA.Create(DistanceMatrixService: TDistanceMatrixService;
                                                         Request: TDistanceMatrixRequest;
                                                         Callback: TDistanceMatrixServiceCallbackA);
begin
  inherited Create(DistanceMatrixService.Script);
  FCallback:=Callback;
  FDistanceMatrixService:=DistanceMatrixService;
  FDistanceMatrixService.Exec(DistanceMatrixServiceMethod_GetDistanceMatrix,
                              {$IFDEF CAN_OVERLOAD}
                              [Request.Disp,JScriptFunction.Disp]
                              {$ELSE}
                              Request.Disp,JScriptFunction.Disp
                              {$ENDIF});
end;

procedure TDistanceMatrixServiceCallbackObjectA.Callback;
var
  DistanceMatrixResponse: TDistanceMatrixResponse;
begin
  try
    if ArgumentCount>=2 then
    begin
      DistanceMatrixResponse:=nil;
      if VarIsValidDispatch(Arguments[0])
        then DistanceMatrixResponse:=TDistanceMatrixResponse.Create(Script,Arguments[0]);
      try
        FCallback(FDistanceMatrixService,DistanceMatrixResponse,Arguments[1]);
      finally
        DistanceMatrixResponse.Free;
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TDistanceMatrixService.GetDistanceMatrix(Request: TDistanceMatrixRequest;
                                                   Callback: TDistanceMatrixServiceCallbackA);
var
  Elements: Integer;
begin
  if assigned(Request) and assigned(Callback) then
  begin
    Elements:=Request.Origins.Count*Request.Destinations.Count;
    if Elements>MaxElements
      then raise Exception.Create(ExceptionToManyElements);
    Inc(RequestElements,Elements);
    TDistanceMatrixServiceCallbackObjectA.Create(Self,Request,Callback);
  end;
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TDistanceMatrixStatus

function TDistanceMatrixStatus.GetInvalidRequest: String;
begin
  Result:=GetPropAsString(DistanceMatrixStatusProp_InvalidRequest);
end;

function TDistanceMatrixStatus.GetMaxDimensionsExceeded: String;
begin
  Result:=GetPropAsString(DistanceMatrixStatusProp_MaxDimensionsExceeded);
end;

function TDistanceMatrixStatus.GetMaxElementsExceeded: String;
begin
  Result:=GetPropAsString(DistanceMatrixStatusProp_MaxElementsExceeded);
end;

function TDistanceMatrixStatus.GetOK: String;
begin
  Result:=GetPropAsString(DistanceMatrixStatusProp_OK);
end;

function TDistanceMatrixStatus.GetOverQueryLimit: String;
begin
  Result:=GetPropAsString(DistanceMatrixStatusProp_OverQueryLimit);
end;

function TDistanceMatrixStatus.GetRequestDenied: String;
begin
  Result:=GetPropAsString(DistanceMatrixStatusProp_RequestDenied);
end;

function TDistanceMatrixStatus.GetUnknownError: String;
begin
  Result:=GetPropAsString(DistanceMatrixStatusProp_UnknownError);
end;

//------------------------------------------------------------------------------
//TLatLngStringArray

function TLatLngStringArray.Add(Values: array of String): Integer;
var
  n: Integer;
begin
  Result:=-1;
  for n:=Low(Values) to High(Values) do
    Result:=Add(Values[n]);
end;

function TLatLngStringArray.Add(Value: String): Integer;
begin
  if (FMaxPoints>0) and (Count=FMaxPoints)
    then raise Exception.Create(ExceptionToManyPoints);
  Result:=inherited Add(Value);
end;

function TLatLngStringArray.Add(Values: array of TLatLng): Integer;
var
  n: Integer;
begin
  Result:=-1;
  for n:=Low(Values) to High(Values) do
    Result:=Add(Values[n]);
end;

function TLatLngStringArray.Add(Value: TLatLng): Integer;
begin
  if (FMaxPoints>0) and (Count=FMaxPoints)
    then raise Exception.Create(ExceptionToManyPoints);
  Result:=inherited Add(Value);
end;

procedure TLatLngStringArray.Insert(Index: Integer; Value: String);
begin
  if (FMaxPoints>0) and (Count=FMaxPoints)
    then raise Exception.Create(ExceptionToManyPoints);
  inherited Insert(Index,Value);
end;

procedure TLatLngStringArray.Insert(Index: Integer; Value: TLatLng);
begin
  if (FMaxPoints>0) and (Count=FMaxPoints)
    then raise Exception.Create(ExceptionToManyPoints);
  inherited Insert(Index,Value);
end;

end.
