//http://code.google.com/intl/de/apis/maps/documentation/elevation/
//Die Verwendung des Google Elevation APIs unterliegt einem Suchanfragenlimit
//von 25,000 Anfragen pro Tag. Beachten Sie, dass Sie im Rahmen einer gegebenen
//Suchanfrage Hhendaten fr mehrere Positionen abfragen knnen. Diese werden
//als getrennte Anfragen gezhlt. Innerhalb einer einzelnen Anfrage knnen Sie
//bis zu 512 Standorte abfragen. Dieses Limit besteht, um Missbrauch und/oder
//Zweckentfremdung des Elevation APIs zu verhindern, und kann jederzeit ohne
//Vorankndigung gendert werden. Darber hinaus gilt ein Limit der Anfragerate,
//um den Missbrauch des Dienstes zu verhindern. Wenn Sie das 24-Stunden-Limit
//berschreiten oder den Dienst auf andere Weise missbrauchen, knnen Sie das
//Elevation API mglicherweise vorbergehend nicht verwenden. Sollten Sie das
//Limit weiterhin berschreiten, kann Ihr Zugriff auf das Elevation API
//gesperrt werden. (07.01.2011)

unit gmServicesElevation;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils, {$IFDEF DEBUG}Dialogs,{$ENDIF}
  {$IFDEF DELPHI6_UP}Variants,{$ENDIF} Math, DispObject, BrowserTools,
  JScriptObjects, gmBase, gmAdditionalConstants, gmAdditionalMessages;

const
  //maximale Anzahl von Punkten innerhalb einer Anfrage:
  //normalerweise 512 (siehe oben);
  //Tests ergaben aber einen Wert von rund 400. Bei mehr Punkten kommt der
  //Fehler UNKNOWN_ERROR. Empfehlenswert sind aber 100 Werte pro Abfrage und
  //eine Pause von etwa 8 Sekunden nach jeweils 1000 Abfragen:
  MaxPointsPerRequest                             = 100;

  //maximale Anzahl von Zwischenpunkten entlang eines Pfades:
  MaxSamplesPerPath                               = 1024;

type
  TPathElevationRequest = class;
  TLocationElevationRequest = class;

  TElevationResult = class(TCustomJScriptObject)
  private
    FLatLng: TLatLng;
    function GetElevation: Double;
    function GetLocation: TLatLng;
  public
    destructor Destroy; override;
    //Die Hhe dieses Punktes der Erde in Metern ber dem Meeresspiegel:
    property Elevation: Double read GetElevation;
    //Der Standort dieses Hhenergebnisses:
    property Location: TLatLng read GetLocation;
  end;

  //JavaScript: Array.<TElevationResult>
  TElevationResultArray = class(TObjectArray)
  private
    procedure SetItem(Index: Integer; Value: TElevationResult);
    function GetItem(Index: Integer): TElevationResult;
  protected
    function CreateObject(Dispatch: IDispatch): TDispObject; override;
  public
    property Items[Index: Integer]: TElevationResult read GetItem write SetItem; default;
  end;

  TElevationServiceCallback = procedure(Sender: TObject;
                              Result: TElevationResultArray;
                              Status: String) of object;
  {$IFDEF USE_ANONYMOUS_METHODS}
  TElevationServiceCallbackA = reference to procedure(Sender: TObject;
                                                      Result: TElevationResultArray;
                                                      Status: String);
  {$ENDIF}

  TElevationService = class(TCustomJScriptObject)
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; reintroduce; overload;
    constructor Create(Script: TCustomScript); reintroduce; overload;
    //Sendet eine Anfrage fr Hhenangaben entlang des Pfades; es werden in
    //gleichmigen Abstnden entlang des Pfades Beispiele mit Hhendaten zurckgegeben:
    procedure GetElevationAlongPath(Request: TPathElevationRequest; Callback: TElevationServiceCallback); overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure GetElevationAlongPath(Request: TPathElevationRequest; Callback: TElevationServiceCallbackA); overload;
    {$ENDIF}
    //Sendet eine Hhenanfrage fr eine Liste diskreter Standorte:
    procedure GetElevationForLocations(Request: TLocationElevationRequest; Callback: TElevationServiceCallback); overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure GetElevationForLocations(Request: TLocationElevationRequest; Callback: TElevationServiceCallbackA); overload;
    {$ENDIF}
  end;

  TElevationStatus = class(TCustomJScriptObject)
  private
    procedure SetInvalidRequest(Value: String);
    procedure SetKi(Value: String);
    procedure SetOK(Value: String);
    procedure SetOverQueryLimit(Value: String);
    procedure SetRequestDenied(Value: String);
    procedure SetUnknownError(Value: String);
    function GetInvalidRequest: String;
    function GetKi: String;
    function GetOK: String;
    function GetOverQueryLimit: String;
    function GetRequestDenied: String;
    function GetUnknownError: String;
  protected
    property Ki: String read GetKi write SetKi;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    property InvalidRequest: String read GetInvalidRequest write SetInvalidRequest;
    property OK: String read GetOK write SetOK;
    property OverQueryLimit: String read GetOverQueryLimit write SetOverQueryLimit;
    property RequestDenied: String read GetRequestDenied write SetRequestDenied;
    property UnknownError: String read GetUnknownError write SetUnknownError;
  end;

  TLocationElevationRequest = class(TCustomJScriptObject)
  private
    FLocations: TLatLngArray;
    procedure SetLocations(Value: TLatLngArray);
    function GetLocations: TLatLngArray;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    //Die diskreten Standorte, fr die Hhendaten abgerufen werden sollen:
    property Locations: TLatLngArray read GetLocations write SetLocations;
  end;

  TPathElevationRequest = class(TCustomJScriptObject)
  private
    FPath: TLatLngArray;
    procedure SetPath(Value: TLatLngArray);
    procedure SetSamples(Value: Integer);
    function GetPath: TLatLngArray;
    function GetSamples: Integer;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    destructor Destroy; override;
    //Der Pfad, entlang dessen Hhenwerte gesammelt werden sollen:
    property Path: TLatLngArray read GetPath write SetPath;
    //Die Anzahl der gleich weit voneinander entfernten Punkte entlang des Pfades,
    //fr die Hhendaten abgerufen werden sollen, einschlielich der Endpunkte.
    //Der Wert fr die Anzahl der Beispiele muss zwischen 2 und 1024 betragen.
    //Erforderlich:
    property Samples: Integer read GetSamples write SetSamples;
  end;

var
  ElevationRequests: Integer = 0;                          //Gesamtanzahl der Anfragen

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TElevationResult

destructor TElevationResult.Destroy;
begin
  FLatLng.Free;
  inherited;
end;

function TElevationResult.GetElevation: Double;
begin
  Result:=GetPropAsDouble(ElevationResultProp_Elevation);
end;

function TElevationResult.GetLocation: TLatLng;
begin
  Result:=TLatLng(GetPropAsObject(ElevationResultProp_Location,TLatLng,FLatLng));
end;

//------------------------------------------------------------------------------
//TElevationResults

function TElevationResultArray.CreateObject(Dispatch: IDispatch): TDispObject;
begin
  Result:=CreateObjectWrapper(Script,TElevationResult,Dispatch);
end;

function TElevationResultArray.GetItem(Index: Integer): TElevationResult;
begin
  Result:=TElevationResult(inherited Items[Index]);
end;

procedure TElevationResultArray.SetItem(Index: Integer; Value: TElevationResult);
begin
  inherited Items[Index]:=Value;
end;

//------------------------------------------------------------------------------
//TElevationService

constructor TElevationService.Create;
begin
  inherited;
end;

constructor TElevationService.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TElevationService.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_ElevationService,true);
end;

type
  TElevationServiceCallbackObject = class(TCallbackObject)
  private
    FCallback: TElevationServiceCallback;
    FElevationService: TElevationService;
  protected
    procedure Callback; override;
  public
    constructor Create(ElevationService: TElevationService;
                       Request: TPathElevationRequest;
                       Callback: TElevationServiceCallback); reintroduce; overload;
    constructor Create(ElevationService: TElevationService;
                       Request: TLocationElevationRequest;
                       Callback: TElevationServiceCallback); reintroduce; overload;
  end;

constructor TElevationServiceCallbackObject.Create(ElevationService: TElevationService;
                                                   Request: TPathElevationRequest;
                                                   Callback: TElevationServiceCallback);
begin
  inherited Create(ElevationService.Script);
  FCallback:=Callback;
  FElevationService:=ElevationService;
  FElevationService.Exec(ElevationServiceMethod_GetElevationAlongPath,
                         [Request.Disp,JScriptFunction.Disp]);
end;

constructor TElevationServiceCallbackObject.Create(ElevationService: TElevationService;
                                                   Request: TLocationElevationRequest;
                                                   Callback: TElevationServiceCallback);
begin
  inherited Create(ElevationService.Script);
  FCallback:=Callback;
  FElevationService:=ElevationService;
  FElevationService.Exec(ElevationServiceMethod_GetElevationForLocations,
                         [Request.Disp,JScriptFunction.Disp]);
end;

procedure TElevationServiceCallbackObject.Callback;
var
  ElevationResult: TElevationResultArray;
begin
  try
    if ArgumentCount>=2 then
    begin
      ElevationResult:=nil;
      if VarIsValidDispatch(Arguments[0])
        then ElevationResult:=TElevationResultArray(CreateObjectWrapper(Script,
                                                                        TElevationResultArray,
                                                                        Arguments[0]));
      try
        FCallback(FElevationService,ElevationResult,Arguments[1]);
      finally
        ElevationResult.Free;
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TElevationService.GetElevationAlongPath(Request: TPathElevationRequest;
                                                  Callback: TElevationServiceCallback);
begin
  if assigned(Request) and assigned(Callback) then
  begin
    Inc(ElevationRequests,(Request.Samples-1)*(Request.Path.Count-1)+1);
    TElevationServiceCallbackObject.Create(Self,Request,Callback);
  end;
end;

procedure TElevationService.GetElevationForLocations(Request: TLocationElevationRequest;
                                                     Callback: TElevationServiceCallback);
begin
  if assigned(Request) and assigned(Callback) then
  begin
    Inc(ElevationRequests,Request.Locations.Count);
    TElevationServiceCallbackObject.Create(Self,Request,Callback);
  end;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
type
  TElevationServiceCallbackObjectA = class(TCallbackObject)
  private
    FCallback: TElevationServiceCallbackA;
    FElevationService: TElevationService;
  protected
    procedure Callback; override;
  public
    constructor Create(ElevationService: TElevationService;
                       Request: TPathElevationRequest;
                       Callback: TElevationServiceCallbackA); reintroduce; overload;
    constructor Create(ElevationService: TElevationService;
                       Request: TLocationElevationRequest;
                       Callback: TElevationServiceCallbackA); reintroduce; overload;
  end;

constructor TElevationServiceCallbackObjectA.Create(ElevationService: TElevationService;
                                                    Request: TPathElevationRequest;
                                                    Callback: TElevationServiceCallbackA);
begin
  inherited Create(ElevationService.Script);
  FCallback:=Callback;
  FElevationService:=ElevationService;
  FElevationService.Exec(ElevationServiceMethod_GetElevationAlongPath,
                         {$IFDEF CAN_OVERLOAD}
                         [Request.Disp,JScriptFunction.Disp]
                         {$ELSE}
                         Request.Disp,JScriptFunction.Disp
                         {$ENDIF});
end;

constructor TElevationServiceCallbackObjectA.Create(ElevationService: TElevationService;
                                                    Request: TLocationElevationRequest;
                                                    Callback: TElevationServiceCallbackA);
begin
  inherited Create(ElevationService.Script);
  FCallback:=Callback;
  FElevationService:=ElevationService;
  FElevationService.Exec(ElevationServiceMethod_GetElevationForLocations,
                         {$IFDEF CAN_OVERLOAD}
                         [Request.Disp,JScriptFunction.Disp]
                         {$ELSE}
                         Request.Disp,JScriptFunction.Disp
                         {$ENDIF});
end;

procedure TElevationServiceCallbackObjectA.Callback;
var
  ElevationResult: TElevationResultArray;
begin
  try
    if ArgumentCount>=2 then
    begin
      ElevationResult:=nil;
      if VarIsValidDispatch(Arguments[0])
        then ElevationResult:=TElevationResultArray(CreateObjectWrapper(Script,
                                                                        TElevationResultArray,
                                                                        Arguments[0]));
      try
        FCallback(FElevationService,ElevationResult,Arguments[1]);
      finally
        ElevationResult.Free;
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TElevationService.GetElevationAlongPath(Request: TPathElevationRequest;
                                                  Callback: TElevationServiceCallbackA);
begin
  if assigned(Request) and assigned(Callback) then
  begin
    Inc(ElevationRequests,(Request.Samples-1)*(Request.Path.Count-1)+1);
    TElevationServiceCallbackObjectA.Create(Self,Request,Callback);
  end;
end;

procedure TElevationService.GetElevationForLocations(Request: TLocationElevationRequest;
                                                     Callback: TElevationServiceCallbackA);
begin
  if assigned(Request) and assigned(Callback) then
  begin
    Inc(ElevationRequests,Request.Locations.Count);
    TElevationServiceCallbackObjectA.Create(Self,Request,Callback);
  end;
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TElevationStatus

class function TElevationStatus.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=(GetJSObjectType(Disp)=otObject) and
          ObjectHasMembers(Disp,[ElevationStatusProp_InvalidRequest,
                                 ElevationStatusProp_Ok,
                                 ElevationStatusProp_OverQueryLimit]);
end;

function TElevationStatus.GetInvalidRequest: String;
begin
  Result:=GetPropAsString(ElevationStatusProp_InvalidRequest);
end;

procedure TElevationStatus.SetInvalidRequest(Value: String);
begin
  SetProp(ElevationStatusProp_InvalidRequest,Value);
end;

function TElevationStatus.GetKi: String;
begin
  Result:=GetPropAsString(ElevationStatusProp_Ki);
end;

procedure TElevationStatus.SetKi(Value: String);
begin
  SetProp(ElevationStatusProp_Ki,Value);
end;

function TElevationStatus.GetOK: String;
begin
  Result:=GetPropAsString(ElevationStatusProp_OK);
end;

procedure TElevationStatus.SetOK(Value: String);
begin
  SetProp(ElevationStatusProp_OK,Value);
end;

function TElevationStatus.GetOverQueryLimit: String;
begin
  Result:=GetPropAsString(ElevationStatusProp_OverQueryLimit);
end;

procedure TElevationStatus.SetOverQueryLimit(Value: String);
begin
  SetProp(ElevationStatusProp_OverQueryLimit,Value);
end;

function TElevationStatus.GetRequestDenied: String;
begin
  Result:=GetPropAsString(ElevationStatusProp_RequestDenied);
end;

procedure TElevationStatus.SetRequestDenied(Value: String);
begin
  SetProp(ElevationStatusProp_RequestDenied,Value);
end;

function TElevationStatus.GetUnknownError: String;
begin
  Result:=GetPropAsString(ElevationStatusProp_UnknownError);
end;

procedure TElevationStatus.SetUnknownError(Value: String);
begin
  SetProp(ElevationStatusProp_UnknownError,Value);
end;

//------------------------------------------------------------------------------
//TLocationElevationRequest

constructor TLocationElevationRequest.Create;
begin
  inherited;
end;

constructor TLocationElevationRequest.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TLocationElevationRequest.Destroy;
begin
  FLocations.Free;
  inherited;
end;

function TLocationElevationRequest.GetLocations: TLatLngArray;
begin
  Result:=TLatLngArray(GetPropAsObject(LocationElevationRequestProp_Locations,
                                       TLatLngArray,FLocations,true));
  if assigned(Result)
    then Result.MaxPoints:=MaxPointsPerRequest;            //max. Punktanzahl
end;

procedure TLocationElevationRequest.SetLocations(Value: TLatLngArray);
begin
  SetProp(LocationElevationRequestProp_Locations,Value,true);
end;

//------------------------------------------------------------------------------
//TPathElevationRequest

constructor TPathElevationRequest.Create;
begin
  inherited;
end;

constructor TPathElevationRequest.Create(Script: TCustomScript);
begin
  inherited;
end;

destructor TPathElevationRequest.Destroy;
begin
  FPath.Free;
  inherited;
end;

function TPathElevationRequest.GetPath: TLatLngArray;
begin
  Result:=TLatLngArray(GetPropAsObject(PathElevationRequestProp_Path,
                                       TLatLngArray,FPath,true));
  if assigned(Result)
    then Result.MaxPoints:=MaxPointsPerRequest;            //max. Punktanzahl
end;

procedure TPathElevationRequest.SetPath(Value: TLatLngArray);
begin
  FreeAndNil(FPath);
  SetProp(PathElevationRequestProp_Path,Value,true);
end;

function TPathElevationRequest.GetSamples: Integer;
begin
  Result:=GetPropAsInteger(PathElevationRequestProp_Samples);
end;

procedure TPathElevationRequest.SetSamples(Value: Integer);
begin
  if Value>MaxSamplesPerPath
    then raise Exception.Create(ExceptionToManySamplesPerPath);
  SetProp(PathElevationRequestProp_Samples,Value,true);
end;

end.
