///<summary>
///  <para><b>Google Maps API Classes</b></para>
///  <list type="bullet">
///    <item><see cref="gmServices.MaxZoom|TMaxZoomResult"/></item>
///    <item><see cref="gmServices.MaxZoom|TMaxZoomService"/></item>
///    <item><see cref="gmServices.MaxZoom|TMaxZoomStatus"/></item>
///  </list>
///  <para><b>Types</b></para>
///  <list type="bullet">
///    <item><see cref="gmServices.MaxZoom|TMaxZoomServiceCallback"/></item>
///    <item><see cref="gmServices.MaxZoom|TMaxZoomServiceCallbackA"/></item>
///  </list>
///</summary>
unit gmServicesMaxZoom;

interface

{$INCLUDE gmConfig.inc}

uses
  {$IFDEF DEBUG}SysUtils, Dialogs,{$ENDIF} BrowserTools, JScriptObjects, gmBase,
  gmAdditionalConstants;

type
  TMaxZoomResult = class;
  TMaxZoomService = class;
  TMaxZoomStatus = class;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMaxZoomResult'}{$ENDIF}
  ///<summary>
  ///  A MaxZoom result in JSON format retrieved from the MaxZoomService.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.MaxZoom"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.MaxZoom|TMaxZoomResult.Status">Status</see></item>
  ///    <item><see cref="gmServices.MaxZoom|TMaxZoomResult.Zoom">Zoom</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#MaxZoomResult">
  ///  MaxZoomResult
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMaxZoomResult = class(TCustomJScriptObject)
  private
    function GetStatus: String;
    function GetZoom: Integer;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Status'}{$ENDIF}
    ///<summary>
    ///  Status der Abfrage.<br/><br/>
    ///  Status of the request.
    ///</summary>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomStatus"/>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Status: String read GetStatus;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Zoom'}{$ENDIF}
    ///<summary>
    ///  Max. Zoom-Stufe fr die gegebene LatLng-Koordinate.<br/><br/>
    ///  The maximum zoom level found at the given LatLng.
    ///</summary>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomResult"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Zoom: Integer read GetZoom;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMaxZoomServiceCallback'}{$ENDIF}
  ///<summary>
  ///  <c>procedure(Sender: </c><see cref="TObject"/><c>; Response: </c>
  ///  <see cref="gmServices.MaxZoom|TMaxZoomResult"/><c>) of object;</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.MaxZoom"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmServices.MaxZoom|TMaxZoomService"/>
  ///<seealso cref="gmServices.MaxZoom|TMaxZoomService.GetMaxZoomAtLatLng(TLatLng,TMaxZoomServiceCallback)">
  ///  TPlacesService.GetDetails
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMaxZoomServiceCallback = procedure(Sender: TObject; Response: TMaxZoomResult) of object;
  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMaxZoomServiceCallbackA'}{$ENDIF}
  ///<summary>
  ///  <c>reference to procedure(Sender: </c><see cref="TObject"/><c>; Response: </c>
  ///  <see cref="gmServices.MaxZoom|TMaxZoomResult"/><c>);</c>
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.MaxZoom"/>
  ///  <para/>
  ///</summary>
  ///<seealso cref="gmServices.MaxZoom|TMaxZoomService"/>
  ///<seealso cref="gmServices.MaxZoom|TMaxZoomService.GetMaxZoomAtLatLng(TLatLng,TMaxZoomServiceCallbackA)">
  ///  TPlacesService.GetDetails
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  {$IFDEF USE_ANONYMOUS_METHODS}
  TMaxZoomServiceCallbackA = reference to procedure(Sender: TObject; Response: TMaxZoomResult);
  {$ENDIF}

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMaxZoomService'}{$ENDIF}
  ///<summary>
  ///  A service for obtaining the highest zoom level at which satellite imagery
  ///  is available for a given location.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.MaxZoom"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.MaxZoom|TMaxZoomService.GetMaxZoomAtLatLng(TLatLng,TMaxZoomServiceCallback)">GetMaxZoomAtLatLng(TLatLng,TMaxZoomServiceCallback)</see></item>
  ///    <item><see cref="gmServices.MaxZoom|TMaxZoomService.GetMaxZoomAtLatLng(TLatLng,TMaxZoomServiceCallbackA)">GetMaxZoomAtLatLng(TLatLng,TMaxZoomServiceCallbackA)</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#MaxZoomService">
  ///  MaxZoomService
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMaxZoomService = class(TCustomJScriptObject)
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetMaxZoomAtLatLng'}{$ENDIF}
    ///<summary>
    ///  Returns the maximum zoom level available at a particular LatLng for the
    ///  Satellite map type. As this request is asynchronous, you must pass a
    ///  callback function which will be executed upon completion of the request,
    ///  being passed a <c>TMaxZoomResult</c>.
    ///</summary>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure GetMaxZoomAtLatLng(LatLng: TLatLng; Callback: TMaxZoomServiceCallback); overload;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation GetMaxZoomAtLatLng'}{$ENDIF}
    ///<summary>
    ///  Returns the maximum zoom level available at a particular LatLng for the
    ///  Satellite map type. As this request is asynchronous, you must pass a
    ///  callback function which will be executed upon completion of the request,
    ///  being passed a <c>TMaxZoomResult</c>.
    ///</summary>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomService"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure GetMaxZoomAtLatLng(LatLng: TLatLng; Callback: TMaxZoomServiceCallbackA); overload;
    {$ENDIF}
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TMaxZoomStatus'}{$ENDIF}
  ///<summary>
  ///  The status returned by the <c>TMaxZoomService</c> on the completion of a
  ///  call to <c>GetMaxZoomAtLatLng</c>.
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="gmServices.MaxZoom"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TCustomJScriptObject"/><br/>
  ///  <para/>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="gmServices.MaxZoom|TMaxZoomStatus.Error">Error</see></item>
  ///    <item><see cref="gmServices.MaxZoom|TMaxZoomStatus.OK">OK</see></item>
  ///  </list>
  ///</summary>
  ///<seealso href="http://code.google.com/apis/maps/documentation/javascript/reference.html#MaxZoomStatus">
  ///  MaxZoomStatus
  ///</seealso>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TMaxZoomStatus = class(TCustomJScriptObject)
  private
    procedure SetError(Value: String);
    procedure SetOK(Value: String);
    function GetError: String;
    function GetOK: String;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Error'}{$ENDIF}
    ///<summary>
    ///  There was a problem contacting the Google servers.
    ///</summary>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Error: String read GetError write SetError;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation OK'}{$ENDIF}
    ///<summary>
    ///  The response contains a valid MaxZoomResult.
    ///</summary>
    ///<seealso cref="gmServices.MaxZoom|TMaxZoomStatus"/>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property OK: String read GetOK write SetOK;
  end;

implementation

uses
  gmAPI;

//------------------------------------------------------------------------------
//TMaxZoomResult

function TMaxZoomResult.GetStatus: String;
begin
  Result:=GetPropAsString(MaxZoomResultProp_Status);
end;

function TMaxZoomResult.GetZoom: Integer;
begin
  Result:=GetPropAsInteger(MaxZoomResultProp_Zoom);
end;

//------------------------------------------------------------------------------
//TMaxZoomService

constructor TMaxZoomService.Create;
begin
  inherited;
end;

constructor TMaxZoomService.Create(Script: TCustomScript);
begin
  inherited;
end;

class function TMaxZoomService.New(Script: TCustomScript): IDispatch;
begin
  CheckMaps(Script);
  Result:=BrowserTools.Create(TScript(Script).Google.Maps.Disp,
                              MapsMethod_MaxZoomService,true);
end;

type
  TMaxZoomServiceCallbackObject = class(TCallbackObject)
  private
    FCallback: TMaxZoomServiceCallback;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FCallbackA: TMaxZoomServiceCallbackA;
    {$ENDIF}
    FMaxZoomService: TMaxZoomService;
  protected
    procedure Callback; override;
  public
    constructor Create(MaxZoomService: TMaxZoomService; LatLng: TLatLng); reintroduce; overload;
    constructor Create(MaxZoomService: TMaxZoomService; LatLng: TLatLng;
                       Callback: TMaxZoomServiceCallback); reintroduce; overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    constructor Create(MaxZoomService: TMaxZoomService; LatLng: TLatLng;
                       Callback: TMaxZoomServiceCallbackA); reintroduce; overload;
    {$ENDIF}
  end;

constructor TMaxZoomServiceCallbackObject.Create(MaxZoomService: TMaxZoomService;
                                                 LatLng: TLatLng);
begin
  inherited Create(MaxZoomService.Script);
  FMaxZoomService:=MaxZoomService;
  FMaxZoomService.Exec(MaxZoomServiceMethod_GetMaxZoomAtLatLng,[LatLng.Disp,JScriptFunction.Disp]);
end;

constructor TMaxZoomServiceCallbackObject.Create(MaxZoomService: TMaxZoomService;
                                                 LatLng: TLatLng;
                                                 Callback: TMaxZoomServiceCallback);
begin
  FCallback:=Callback;
  Create(MaxZoomService,LatLng);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
constructor TMaxZoomServiceCallbackObject.Create(MaxZoomService: TMaxZoomService;
                                                 LatLng: TLatLng;
                                                 Callback: TMaxZoomServiceCallbackA);
begin
  FCallbackA:=Callback;
  Create(MaxZoomService,LatLng);
end;
{$ENDIF}

procedure TMaxZoomServiceCallbackObject.Callback;
var
  MaxZoomResult: TMaxZoomResult;
begin
  try
    if ArgumentCount>=1 then
    begin
      MaxZoomResult:=nil;
      if VarIsValidDispatch(Arguments[0])
        then MaxZoomResult:=TMaxZoomResult(CreateObjectWrapper(Script,
                                                               TMaxZoomResult,
                                                               Arguments[0]));
      try
        {$IFDEF USE_ANONYMOUS_METHODS}
        if assigned(FCallbackA)
          then FCallbackA(FMaxZoomService,MaxZoomResult)
          else
        {$ENDIF}
        if assigned(FCallback)
          then FCallback(FMaxZoomService,MaxZoomResult);
      finally
        //MaxZoomResult.Free;     -> to save it for future use in anonymous methods
      end;
    end;
  except
    {$IFDEF DEBUG}
    ShowMessage(Exception(ExceptObject).Message);
    {$ENDIF}
  end;
  Ready:=true;
end;

procedure TMaxZoomService.GetMaxZoomAtLatLng(LatLng: TLatLng;
                                             Callback: TMaxZoomServiceCallback);
begin
  if assigned(LatLng) and assigned(Callback)
    then TMaxZoomServiceCallbackObject.Create(Self,LatLng,Callback);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
procedure TMaxZoomService.GetMaxZoomAtLatLng(LatLng: TLatLng;
                                             Callback: TMaxZoomServiceCallbackA);
begin
  if assigned(LatLng) and assigned(Callback)
    then TMaxZoomServiceCallbackObject.Create(Self,LatLng,Callback);
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TMaxZoomStatus

function TMaxZoomStatus.GetError: String;
begin
  Result:=GetPropAsString(MaxZoomStatusProp_Error);
end;

function TMaxZoomStatus.GetOK: String;
begin
  Result:=GetPropAsString(MaxZoomStatusProp_OK);
end;

procedure TMaxZoomStatus.SetError(Value: String);
begin
  SetProp(MaxZoomStatusProp_Error,Value,true);
end;

procedure TMaxZoomStatus.SetOk(Value: String);
begin
  SetProp(MaxZoomStatusProp_OK,Value,true);
end;

end.
