//Mason Wheeler: Anonymous method as TInterfacedObject
//http://tech.turbu-rpg.com/30/under-the-hood-of-an-anonymous-method
//Barry Kelly:
//http://blog.barrkel.com/2010/01/using-anonymous-methods-in-method.html
unit AnonymousMethodsTools;

interface

uses
  SysUtils, Classes;

type
  TNotifyProc = TProc<TObject>;

function IsMethRef(Meth: TMethod): Boolean;
function AddRefMethRef(const MethRef): Integer;
function AddRefMethPtr(const MethPtr): Integer;
function ReleaseMethRef(const MethRef): Integer;
function ReleaseMethPtr(const MethPtr): Integer;
function RefCount(const MethRef): Integer;
function FrameSize(const MethRef): Integer;
function MethFrameObject(const MethRef): TInterfacedObject;
function MakeEvent(const NotifyProc: TNotifyProc): TNotifyEvent; overload;
procedure MethRefToMethPtr(const MethRef; var MethPtr);
procedure ReleaseEvent(const Event);
procedure ReleaseMethod(const MethRef);

implementation

type
  TInterfacedObjectAccess = class(TInterfacedObject);
  TVTable = array[0..3] of Pointer;
  PVTable = ^TVTable;
  PPVTable = ^PVTable;

function MethFrameObject(const MethRef): TInterfacedObject;
var
  Meth: TObject;
  Intf: IInterface absolute MethRef;
begin
  Result:=nil;
  Meth:=Intf as TObject;
  if assigned(Meth) and (Meth is TInterfacedObject)
    then Result:=TInterfacedObject(Meth);
end;

function AddRefMethRef(const MethRef): Integer;
var
  Meth: TObject;
  Intf: IInterface absolute MethRef;
begin
  Result:=0;
  Meth:=Intf as TObject;
  if assigned(Meth) and (Meth is TInterfacedObject)
    then Result:=TInterfacedObjectAccess(Meth)._AddRef;
end;

function ReleaseMethRef(const MethRef): Integer;
var
  Meth: TObject;
  Intf: IInterface absolute MethRef;
begin
  Result:=0;
  Meth:=Intf as TObject;
  if assigned(Meth) and (Meth is TInterfacedObject)
    then Result:=TInterfacedObjectAccess(Meth)._Release;
end;

function RefCount(const MethRef): Integer;
var
  MethFrameObj: TInterfacedObject;
begin
  Result:=0;
  MethFrameObj:=MethFrameObject(MethRef);
  if assigned(MethFrameObj)
    then Result:=MethFrameObj.RefCount;
end;

function FrameSize(const MethRef): Integer;
var
  MethFrameObj: TInterfacedObject;
begin
  Result:=0;
  MethFrameObj:=MethFrameObject(MethRef);
  if assigned(MethFrameObj)
    then Result:=MethFrameObj.InstanceSize;
end;

function IsMethRef(Meth: TMethod): Boolean;
begin
  Result:=false;
  if assigned(Meth.Code) and assigned(Meth.Data)
    then Result:=Meth.Code=PPVtable(Meth.Data)^^[3];
end;

function AddRefMethPtr(const MethPtr): Integer;
var
  Meth: TMethod;
begin
  Result:=0;
  Meth:=TMethod(MethPtr);
  if IsMethRef(Meth) and assigned(Meth.Data)
    then Result:=AddRefMethRef(Meth.Data);
end;

function ReleaseMethPtr(const MethPtr): Integer;
var
  Meth: TMethod;
begin
  Result:=0;
  Meth:=TMethod(MethPtr);
  if IsMethRef(Meth)
    then Result:=ReleaseMethRef(Meth.Data);
end;

procedure ReleaseMethod(const MethRef);
begin
  ReleaseMethRef(MethRef);
end;

procedure ReleaseEvent(const Event);
begin
  ReleaseMethPtr(Event);
end;

procedure MethRefToMethPtr(const MethRef; var MethPtr);
begin
  //3 is offset of Invoke, after QI, AddRef, Release
  TMethod(MethPtr).Code:=PPVtable(MethRef)^^[3];
  TMethod(MethPtr).Data:=Pointer(MethRef);
  //Additional: Prevent release of captured variables:
  AddRefMethPtr(MethPtr);
end;

function MakeEvent(const NotifyProc: TNotifyProc): TNotifyEvent; overload;
begin
  MethRefToMethPtr(NotifyProc,Result);
end;

end.
