unit Base64;

interface

uses
  Classes;

function Encode(const s: AnsiString): AnsiString;
function Decode(const s: AnsiString): AnsiString;
function Encode64(s: String): String; overload;
function Encode64(Src, Dest: TStream): Boolean; overload;
function Decode64(s: String): String; overload;
function Decode64(Stream: TStream): Boolean; overload;

implementation

function Decode(const s: AnsiString): AnsiString;
const
  Map: array[AnsiChar] of Byte = (
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 62,  0,  0,  0, 63,
    52, 53, 54, 55, 56, 57, 58, 59, 60, 61,  0,  0,  0,  0,  0,  0,
     0,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
    15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25,  0,  0,  0,  0,  0,
     0, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
    41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0);
var
  n: LongInt;
begin
  case Length(s) of
    2: begin
         n:=Map[S[1]]+(Map[s[2]] shl 6);
         SetLength(Result,1);
         Move(n,Result[1],Length(Result));
       end;
    3: begin
         n:=Map[S[1]]+(Map[s[2]] shl 6)+(Map[s[3]] shl 12);
         SetLength(Result,2);
         Move(n,Result[1],Length(Result))
       end;
    4: begin
         n:= Map[s[1]]+(Map[s[2]] shl 6)+(Map[s[3]] shl 12)+(Map[s[4]] shl 18);
         SetLength(Result,3);
         Move(n,Result[1],Length(Result))
       end
  end
end;

function Encode(const s: AnsiString): AnsiString;
const
  Map: array[0..63] of AnsiChar = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
var
  n: LongInt;
begin
  n:=0;
  Move(s[1],n,Length(s));
  case Length(s) of
    1: Result:=Map[n mod 64]+Map[(n shr 6) mod 64];
    2: Result:=Map[n mod 64]+Map[(n shr 6) mod 64]+Map[(n shr 12) mod 64];
    3: Result:=Map[n mod 64]+Map[(n shr 6) mod 64]+Map[(n shr 12) mod 64]+Map[(n shr 18) mod 64]
  end
end;

const
  Codes64 = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+/';

function Encode64(s: String): String;
var
  i: Integer;
  a: Integer;
  x: Integer;
  b: Integer;
begin
  Result:='';
  a:=0;
  b:=0;
  for i:=1 to Length(s) do
  begin
    x:= Ord(s[i]);
    b:=b*256+x;
    a:=a+8;
    while a>=6 do
    begin
      a:=a - 6;
      x:=b div (1 shl a);
      b:=b mod (1 shl a);
      Result:=Result+Codes64[x+1];
    end;
  end;
  if a>0 then
  begin
    x:=b shl (6-a);
    Result:=Result+Codes64[x+1];
  end;
end;

function Decode64(s: String): String;
var
  i: Integer;
  a: Integer;
  x: Integer;
  b: Integer;
begin
  Result:='';
  a:=0;
  b:=0;
  for i:=1 to Length(s) do
  begin
    x:=Pos(s[i],Codes64)-1;
    if x>=0 then
    begin
      b:=b*64+x;
      a:=a+6;
      if a>=8 then
      begin
        a:=a-8;
        x:=b shr a;
        b:=b mod (1 shl a);
        x:=x mod 256;
        Result:=Result+Chr(x);
      end;
    end else Exit;
  end;
end;

const
  Map: array[0..63] of AnsiChar = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'+
                                  'abcdefghijklmnopqrstuvwxyz'+
                                  '0123456789+/';

function Encode64(Src, Dest: TStream): Boolean;
var
  Size: Integer;
  Data: array[0..2] of Byte;
  EncodedData: array[0..3] of AnsiChar;
begin
  Result:=false;
  if not assigned(Src) or not assigned(Dest)
    then Exit;
  while Src.Position<Src.Size do
  begin
    Size:=Src.Read(Data,SizeOf(Data));
    EncodedData[0]:=Map[Data[0] shr 2];
    EncodedData[1]:=Map[((Data[0] shl 4) or (Data[1] shr 4)) and $3F];
    EncodedData[2]:=Map[((Data[1] shl 2) or (Data[2] shr 6)) and $3F];
    EncodedData[3]:=Map[Data[2] and $3F];
    case Size of
      0: Exit;
      1: begin
           EncodedData[2]:='=';
           EncodedData[3]:='=';
         end;
      2: EncodedData[3]:='=';
      3: ;
    end;
    Dest.Write(EncodedData,SizeOf(EncodedData));
  end;
end;

function Decode64(Stream: TStream): Boolean;
begin
  Result:=false;
  if not assigned(Stream)
    then Exit;
end;

end.
