unit BrowserTools;

interface

{$INCLUDE gmConfig.inc}

uses
  Forms, SysUtils, Classes, Contnrs, ActiveX,  Windows, OleCtrls, Dialogs,
  DocHostUIHandler, Graphics, WinInet, Base64
  {$IFDEF USE_PNGIMG}, PNGImage{$ENDIF}
  {$IFDEF USE_GIFIMG}, GIFImg{$ENDIF}
  {$IFDEF DELPHI6_UP}, Variants{$ENDIF}
  {$IFDEF DELPHI7_UP}, Themes{$ENDIF}
  {$IFDEF DELPHI2006_UP}, ObjAuto{$ENDIF};

const
  CRLF       = #13+#10;
  {$IFNDEF DELPHI6_UP}     
  sLineBreak = CRLF;
  {$ENDIF}

  EmptyPage  = '<html>'+CRLF+
               '  <head>'+CRLF+
               '  </head>'+CRLF+
               '  <body>'+CRLF+
               '  </body>'+CRLF+
               '</html>';

//IDispatchEx ist in Delphi 6 offenbar fehlerhaft implementiert
//-> deshalb Nutzung der Definition aus der Unit ActiveX erst ab Delphi 7:
{.$IFNDEF DELPHI6_UP}
{$IFNDEF DELPHI7_UP}
  DISPATCH_CONSTRUCT      = $4000;

  fdexNameCaseSensitive   = $1;
  fdexNameEnsure          = $2;
  fdexNameImplicit        = $4;
  fdexNameCaseInsensitive = $8;
{$ENDIF}

  READYSTATE_UNINITIALIZED = $00000000;
  READYSTATE_LOADING       = $00000001;
  READYSTATE_LOADED        = $00000002;
  READYSTATE_INTERACTIVE   = $00000003;
  READYSTATE_COMPLETE      = $00000004;

type
{.$IFNDEF DELPHI6_UP}
{$IFNDEF DELPHI7_UP}
  IServiceProvider = interface(IUnknown)
    ['{6d5140c1-7436-11ce-8034-00aa006009fa}']
    function QueryService(const rsid, iid: TGuid; out Obj): HResult; stdcall;
  end;

  PServiceProvider = ^IServiceProvider;

  IDispatchEx = interface(IDispatch)
    ['{A6EF9860-C720-11D0-9337-00A0C90DCAA9}']
    function GetDispID(const bstrName: TBSTR; const grfdex: DWORD; out id: TDispID): HResult; stdcall;
    function InvokeEx(const id: TDispID; const lcid: LCID; const wflags: WORD; const pdp: PDispParams;
      out varRes: OleVariant; out pei: TExcepInfo; const pspCaller: PServiceProvider): HResult; stdcall;
    function DeleteMemberByName(const bstr: TBSTR; const grfdex: DWORD): HResult; stdcall;
    function DeleteMemberByDispID(const id: TDispID): HResult; stdcall;
    function GetMemberProperties(const id: TDispID; const grfdexFetch: DWORD; out grfdex: DWORD): HResult; stdcall;
    function GetMemberName(const id: TDispID; out bstrName: TBSTR): HResult; stdcall;
    function GetNextDispID(const grfdex: DWORD; const id: TDispID; out nid: TDispID): HResult; stdcall;
    function GetNameSpaceParent(out unk: IUnknown): HResult; stdcall;
  end;
{$ENDIF}

  TInvokeType = (itMethod, itGet, itSet, itNew);
  TOperation = (opInsert, opChanged, opRemove);

  TExternalProc = procedure of object;
  TExternalFunc = function: Variant of object;

  TExternalMethod = class(TObject)
  private
    FProc: TExternalProc;
    FFunc: TExternalFunc;
    FInstance: TObject;
    FExternalName: String;
    FName: String;
    FDispId: Integer;
    FDisp: IDispatch;
    function GetExternalName: String;
  public
    constructor Create(Disp: IDispatch; DispId: Integer; ExternalName: String; Proc: TExternalProc); overload;
    constructor Create(Disp: IDispatch; DispId: Integer; ExternalName: String; Func: TExternalFunc); overload;
    function Call: Variant;
    property Proc: TExternalProc read FProc write FProc;
    property Func: TExternalFunc read FFunc write FFunc;
    property Instance: TObject read FInstance;
    property Name: String read FName;
    property ExternalName: String read GetExternalName;
    property DispID: Integer read FDispId;
    property Disp: IDispatch read FDisp;                   //Interface der zugehrigen JScript-Funktion
  end;

  TExternalMethods = class(TInterfacedObject, IDispatch)
  private
    FItems: TObjectList;
    FID: Integer;
    function FindFunc(ExternalName: String; Method: TMethod): TExternalMethod; overload;
    function GetFunc(DispID: Integer): TExternalMethod;
    function GetItem(Index: Integer): TExternalMethod;
  protected
    //IDispatch:
    function GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount: Integer; LocaleID: Integer; DispIDs: Pointer): HRESULT; virtual; stdcall;
    function GetTypeInfo(Index: Integer; LocaleID: Integer; out TypeInfo): HRESULT; stdcall;
    function GetTypeInfoCount(out Count: Integer): HRESULT; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult: Pointer; ExcepInfo: Pointer; ArgErr: Pointer): HRESULT; virtual; stdcall;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Reset;
    function AddRef: Integer;
    function Release: Integer;
    function RegisterMethod(Disp: IDispatch; Method: TExternalProc; ExternalName: String = ''): TExternalMethod; overload;
    function RegisterMethod(Disp: IDispatch; Method: TExternalFunc; ExternalName: String = ''): TExternalMethod; overload;
    function UnregisterMethod(Method: TExternalProc; ExternalName: String = ''): Boolean; overload;
    function UnregisterMethod(Method: TExternalFunc; ExternalName: String = ''): Boolean; overload;
    function UnregisterMethod(Disp: IDispatch): Boolean; overload;
    function Count: Integer;
    function IndexOf(ExternalName: String): Integer;
    property Items[Index: Integer]: TExternalMethod read GetItem; default;
  end;

  THostOption = (hoScrollBars, ho3DBorder, hoAllowTextSelection);
  THostOptions = set of THostOption;

  TDocHostUIHandler = class;

  TNoticationComponent = class(TComponent)
  private
    FOwner: TDocHostUIHandler;
  protected
    procedure Notification(AComponent: TComponent; Operation: Classes.TOperation); override;
  private
    constructor Create(Owner: TDocHostUIHandler); reintroduce;
  end;

  TTranslateURLEvent = function(Sender: TObject; var URL: String): Boolean of object;
  TTranslateAcceleratorEvent = function(Sender: TObject; Accelerator: Integer): Boolean of object;
  TShowContextMenuEvent = function(Sender: TObject; Point: TPoint): Boolean of object;

  TDocHostUIHandler = class(TObject,{$IFDEF DELPHI6_UP}IInterface{$ELSE}IUnknown{$ENDIF},IDocHostUIHandler)
  private
    FBrowser: TOleControl;
    FCSS: String;
    FExternalMethods: TExternalMethods;
    FNoticationComponent: TNoticationComponent;
    FOptions: THostOptions;
    FOnShowContextMenu: TShowContextMenuEvent;
    FOnTranslateURL: TTranslateURLEvent;
    FOnTranslateAccelerator: TTranslateAcceleratorEvent;
    procedure SetBrowser(Value: TOleControl);
    procedure SetExternalMethods(Value: TExternalMethods);
    function StoreOptions: Boolean;
  protected
    //IInterface
    function QueryInterface(const IID: TGUID; out Obj): HResult; virtual; stdcall;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    //IDocHostUIHandler:
    function GetHostInfo(var pInfo: TDocHostUIInfo): HResult; stdcall;
    function ShowContextMenu(const dwID: DWORD; const ppt: PPOINT; const pcmdtReserved: {$IFDEF DELPHI6_UP}IInterface{$ELSE}IUnknown{$ENDIF}; const pdispReserved: IDispatch): HResult; stdcall;
    function ShowUI(const dwID: DWORD; const pActiveObject: IOleInPlaceActiveObject; const pCommandTarget: IOleCommandTarget; const pFrame: IOleInPlaceFrame; const pDoc: IOleInPlaceUIWindow): HResult; stdcall;
    function HideUI: HResult; stdcall;
    function UpdateUI: HResult; stdcall;
    function EnableModeless(const fEnable: BOOL): HResult; stdcall;
    function OnDocWindowActivate(const fActivate: BOOL): HResult; stdcall;
    function OnFrameWindowActivate(const fActivate: BOOL): HResult; stdcall;
    function ResizeBorder(const prcBorder: PRECT; const pUIWindow: IOleInPlaceUIWindow; const fFrameWindow: BOOL): HResult; stdcall;
    function TranslateAccelerator(const lpMsg: PMSG; const pguidCmdGroup: PGUID; const nCmdID: DWORD): HResult; stdcall;
    function GetOptionKeyPath(var pchKey: POLESTR; const dw: DWORD): HResult; stdcall;
    function GetDropTarget(const pDropTarget: IDropTarget; out ppDropTarget: IDropTarget): HResult; stdcall;
    function GetExternal(out ppDispatch: IDispatch): HResult; stdcall;
    function TranslateUrl(const dwTranslate: DWORD; const pchURLIn: POLESTR; var ppchURLOut: POLESTR): HResult; stdcall;
    function FilterDataObject(const pDO: IDataObject; out ppDORet: IDataObject): HResult; stdcall;
  public
    constructor Create;
    destructor Destroy; override;
    property Browser: TOleControl read FBrowser write SetBrowser;
    property CSS: String read FCSS write FCSS;
    property ExternalMethods: TExternalMethods read FExternalMethods write SetExternalMethods;
    property Options: THostOptions read FOptions write FOptions stored StoreOptions;
    property OnShowContextMenu: TShowContextMenuEvent read FOnShowContextMenu write FOnShowContextMenu;
    property OnTranslateURL: TTranslateURLEvent read FOnTranslateURL write FOnTranslateURL;
    property OnTranslateAccelerator: TTranslateAcceleratorEvent read FOnTranslateAccelerator write FOnTranslateAccelerator;
  end;

function VarIsValidDispatch(const v: Variant): Boolean;
function VarAsDispatch(const v: Variant): IDispatch;
function VarAsBoolean(const v: Variant; Default: Boolean = false): Boolean;
function VarAsString(const v: Variant; Default: String = ''): String;
function VarAsDouble(const v: Variant; Default: Double = 0): Double;
function VarAsInteger(const v: Variant; Default: Integer = 0): Integer;
function SetProxy(Server: String): Boolean;
function IsConnected(URL: String): Boolean;
function Script(WebBrowser: TOleControl): IDispatch;
function ScriptCanUsed(Script: IDispatch): Boolean;
function Invoke(Dispatch: IDispatch; DispID: Integer; InvokeType: TInvokeType; const InvokeParams: array of OleVariant; CanExcept: Boolean = true): OleVariant; overload;
function Invoke(Dispatch: IDispatch; DispID: Integer; InvokeType: TInvokeType; CanExcept: Boolean = true): OleVariant; overload;
function Invoke(Dispatch: IDispatch; const InvokeName: String; InvokeType: TInvokeType; const InvokeParams: array of OleVariant; CanExcept: Boolean = true): OleVariant; overload;
function Invoke(Dispatch: IDispatch; const InvokeName: String; InvokeType: TInvokeType; CanExcept: Boolean = true): OleVariant; overload;
function Exec(Dispatch: IDispatch; const Script: String; const Params: array of OleVariant; CanExcept: Boolean = false): OleVariant; overload;
function Exec(Dispatch: IDispatch; const Script: String; const Param: OleVariant; CanExcept: Boolean = false): OleVariant; overload;
function Exec(Dispatch: IDispatch; const Script: String; CanExcept: Boolean = false): OleVariant; overload;
function Exec(Dispatch: IDispatch; const Params: array of OleVariant; CanExcept: Boolean = false): OleVariant; overload;
function Exec(Dispatch: IDispatch; const Param: OleVariant; CanExcept: Boolean = false): OleVariant; overload;
function Exec(Dispatch: IDispatch; CanExcept: Boolean = false): OleVariant; overload;
function Eval(WebBrowser: TOleControl; const Script: String): OleVariant;
function Create(Dispatch: IDispatch; const Name: String; CanExcept: Boolean {$IFDEF CAN_OVERLOD}= false{$ENDIF}): IDispatch; overload;
function Create(Dispatch: IDispatch; const Name: String; const Param: OleVariant; CanExcept: Boolean {$IFDEF CAN_OVERLOAD}= false{$ENDIF}): IDispatch; overload;
{$IFDEF CAN_OVERLOAD}
function Create(Dispatch: IDispatch; const Name: String; const Params: array of OleVariant; CanExcept: Boolean = false): IDispatch; overload;
{$ELSE}
function Create_(Dispatch: IDispatch; const Name: String; const Params: array of OleVariant; CanExcept: Boolean): IDispatch; overload;
function Create(Dispatch: IDispatch; const Name: String; const Param1, Param2: OleVariant; CanExcept: Boolean): IDispatch; overload;
function Create(Dispatch: IDispatch; const Name: String; const Param1, Param2, Param3: OleVariant; CanExcept: Boolean): IDispatch; overload;
function Create(Dispatch: IDispatch; const Name: String; const Param1, Param2, Param3, Param4: OleVariant; CanExcept: Boolean): IDispatch; overload;
function Create(Dispatch: IDispatch; const Name: String; const Param1, Param2, Param3, Param4, Param5: OleVariant; CanExcept: Boolean): IDispatch; overload;
{$ENDIF}
function CreateJSFunction(Script: IDispatch; const FuncDef: String): IDispatch; overload;
function CreateJSFunction(Script: IDispatch; const FuncDef: String; const Params: array of String): IDispatch; overload;
function CreateJSFunction(Script: IDispatch; const FuncDef: String; const Param: String): IDispatch; overload;
function AddJScript(Document: IDispatch; const URL, Src: String; ID: String = ''): IDispatch;
function GetDispID(Dispatch: IDispatch; const InvokeName: String; var DispID: Integer; CanExcept: Boolean = false): Boolean;
function GetProp(Dispatch: IDispatch; const Name: String; CanExcept: Boolean): OleVariant; overload;
function GetProp(Dispatch: IDispatch; const Name: String; const Param: OleVariant; CanExcept: Boolean): OleVariant; overload;
function GetProp(Dispatch: IDispatch; DispID: Integer; CanExcept: Boolean): OleVariant; overload;
function LoadBrowserFromSource(Browser: TOleControl; const Source: String; Wait: Boolean = false): Boolean;
function LoadBrowserFromStream(Browser: TOleControl; Stream: TStream; Wait: Boolean = false): Boolean;
procedure Alert(const Val: String); overload;
procedure Alert(Val: Integer); overload;
procedure Alert(Val: Double); overload;
procedure Alert(Val: TObject); overload;
procedure SetProp(Dispatch: IDispatch; const Name: String; const Param: OleVariant; CanExcept: Boolean = false); overload;
procedure SetProp(Dispatch: IDispatch; DispID: Integer; const Param: OleVariant; CanExcept: Boolean = false); overload;
function ColorToWebColorStr(Color: TColor): String;
function WebColorStrToColor(WebColor: String): TColor;
function RGBToColor(R, G, B : Byte): TColor;
function Encode64(Data: TStream; MediaType: String): String; overload;
function Encode64(Image: TBitmap): String; overload;
{$IFDEF USE_PNGIMG}
function Encode64(Image: {$IFDEF TPNGImage}TPNGImage{$ELSE}TPNGObject{$ENDIF}): String; overload;
{$ENDIF}
{$IFDEF USE_GIFIMG}
function Encode64(Image: TGIFImage): String; overload;
{$ENDIF}

implementation

function Encode64(Data: TStream; MediaType: String): String;
var
  Stream: TStringStream;
begin
  Result:='';
  if not assigned(Data)
    then Exit;
  Stream:=TStringStream.Create('data:'+MediaType+';base64,');
  try
    Stream.Seek(0,soFromEnd);
    Encode64(Data,Stream);
    Stream.Seek(0,soFromBeginning);
    Result:=Stream.DataString;
  finally
    Stream.Free;
  end;
end;

function Encode64(Image: TBitmap): String;
var
  Stream: TMemoryStream;
begin
  Result:='';
  if not assigned(Image)
    then Exit;
  Stream:=TMemoryStream.Create;
  try
    Image.SaveToStream(Stream);
    Stream.Seek(0,soFromBeginning);
    Result:=Encode64(Stream,'image/bmp');
  finally
    Stream.Free;
  end;
end;

{$IFDEF USE_PNGIMG}
function Encode64(Image: {$IFDEF TPNGImage}TPNGImage{$ELSE}TPNGObject{$ENDIF}): String;
var
  Stream: TMemoryStream;
begin
  Result:='';
  if not assigned(Image)
    then Exit;
  Stream:=TMemoryStream.Create;
  try
    Image.SaveToStream(Stream);
    Stream.Seek(0,soFromBeginning);
    Result:=Encode64(Stream,'image/png');
  finally
    Stream.Free;
  end;
end;
{$ENDIF}

{$IFDEF USE_GIFIMG}
function Encode64(Image: TGIFImage): String;
var
  Stream: TMemoryStream;
begin
  Result:='';
  if not assigned(Image)
    then Exit;
  Stream:=TMemoryStream.Create;
  try
    Image.SaveToStream(Stream);
    Stream.Seek(0,soFromBeginning);
    Result:=Encode64(Stream,'image/gif');
  finally
    Stream.Free;
  end;
end;
{$ENDIF}

//setzt einen Proxy fr die in der Anwendung enthaltenen TWebBrowser-Komponenten
//http://support.microsoft.com/default.aspx?scid=kb;en-us;226473
function SetProxy(Server: String): Boolean;                //Server z.B. '127.0.0.1:8080' oder ''
type
  INTERNET_PER_CONN_OPTION = record
    dwOption: DWORD;
    Value: record
      case Integer of
        1: (dwValue: DWORD);
        2: (pszValue: {$IFDEF DELPHI2009_UP}PWideChar{$ELSE}PAnsiChar{$ENDIF});
        3: (ftValue: TFileTime);
      end;
  end;
  LPINTERNET_PER_CONN_OPTION = ^INTERNET_PER_CONN_OPTION;
  INTERNET_PER_CONN_OPTION_LIST = record
    dwSize: DWORD;
    pszConnection: LPTSTR;
    dwOptionCount: DWORD;
    dwOptionError: DWORD;
    pOptions: LPINTERNET_PER_CONN_OPTION;
  end;
  LPINTERNET_PER_CONN_OPTION_LIST = ^INTERNET_PER_CONN_OPTION_LIST;
const
  INTERNET_PER_CONN_FLAGS               = 1;
  INTERNET_PER_CONN_PROXY_SERVER        = 2;
  INTERNET_PER_CONN_PROXY_BYPASS        = 3;
  INTERNET_PER_CONN_AUTOCONFIG_URL      = 4;
  INTERNET_PER_CONN_AUTODISCOVERY_FLAGS = 5;
  PROXY_TYPE_DIRECT                     = $00000001;
  PROXY_TYPE_PROXY                      = $00000002;
  PROXY_TYPE_AUTO_PROXY_URL             = $00000004;
  PROXY_TYPE_AUTO_DETECT                = $00000008;
  INTERNET_OPTION_REFRESH               = 37;
  INTERNET_OPTION_PER_CONNECTION_OPTION = 75;
  INTERNET_OPTION_SETTINGS_CHANGED      = 39;
var
  Agent: String;
  BufSize: DWORD;
  hInternet: WinInet.HINTERNET;
  OptionsList: INTERNET_PER_CONN_OPTION_LIST;
begin
  Result:=false;
  BufSize:=SizeOf(OptionsList);
  OptionsList.dwSize:=BufSize;
  OptionsList.pszConnection:=nil;                               //nil -> LAN, sonst Verbindungsname
  OptionsList.dwOptionCount:=3;                                 //3 Optionen werden gesetzt
  OptionsList.pOptions:=AllocMem(3*SizeOf(INTERNET_PER_CONN_OPTION));
  try
    if not assigned(OptionsList.pOptions)
      then Exit;
    OptionsList.pOptions^.dwOption:=INTERNET_PER_CONN_FLAGS;
    OptionsList.pOptions^.Value.dwValue:=PROXY_TYPE_DIRECT or PROXY_TYPE_PROXY;
    inc(OptionsList.pOptions);
    OptionsList.pOptions^.dwOption:=INTERNET_PER_CONN_PROXY_SERVER;
    OptionsList.pOptions^.Value.pszValue:=PChar(Server);
    inc(OptionsList.pOptions);
    OptionsList.pOptions^.dwOption:=INTERNET_PER_CONN_PROXY_BYPASS;
    OptionsList.pOptions^.Value.pszValue:='local';
    dec(OptionsList.pOptions,2);
    Agent:=ExtractFileName(Application.ExeName);
    hInternet:=InternetOpen({$IFDEF DELPHI2009_UP}PWideChar{$ELSE}PAnsiChar{$ENDIF}
                            (Agent),INTERNET_OPEN_TYPE_DIRECT,nil,nil,0);
    try                                                         //Optionen setzen
      Result:=InternetSetOption(hInternet,INTERNET_OPTION_PER_CONNECTION_OPTION,
                                @OptionsList,BufSize);
      InternetSetOption(hInternet,INTERNET_OPTION_REFRESH,nil,0);
    finally
      InternetCloseHandle(hInternet);
    end;
  finally
    FreeMem(OptionsList.pOptions);                              //Speicher freigeben
  end;
end;

{$IFNDEF DELPHI6_UP}
const
  INTERNET_CONNECTION_OFFLINE         = 32;
  INTERNET_CONNECTION_CONFIGURED      = 64;
{$ENDIF}

function IsConnected(URL: String): Boolean;
var
  Agent: String;
  InetState: DWORD;
  hInternet, hReqUrl: WinInet.HINTERNET;
begin
  Result:=InternetGetConnectedState(@InetState,0);
  if Result and (InetState and INTERNET_CONNECTION_CONFIGURED<>0) then
  begin
    //Es existiert eine gltige Verbindung -> Test auf Ereichbarkeit der gewnschten URL:
    Agent:=ExtractFileName(Application.ExeName);
    hInternet:=InternetOpen({$IFDEF DELPHI2009_UP}PWideChar{$ELSE}PAnsiChar{$ENDIF}
                            (Agent),INTERNET_OPEN_TYPE_PRECONFIG,nil,nil,0);
    try
      hReqUrl:=InternetOpenURL(hInternet,{$IFDEF DELPHI2009_UP}PWideChar{$ELSE}
                               PAnsiChar{$ENDIF}(URL),nil,0,0,0);
      try
        Result:=assigned(hReqUrl);
      finally
        InternetCloseHandle(hReqUrl);
      end;
    finally
      InternetCloseHandle(hInternet);
    end;
  end else if InetState and INTERNET_CONNECTION_OFFLINE<>0
    then Result:=false;
end;

const
  UnassignedMethod: TMethod = (Code: nil; Data: nil);

function AssignedMethod(Method: TMethod): Boolean;
begin
  Result:=assigned(Method.Code) and assigned(Method.Data);
end;

function RGBToColor(R, G, B : Byte): TColor;
begin
  Result:=((R and $FF) shl 16)+((G and $FF) shl 8)+(B and $FF);
end;

const
  TransparentColor = 'transparent';

function ColorToWebColorStr(Color: TColor): String;
var
  RGB: Integer;
begin
  if Color<>clNone then
  begin
    RGB:=ColorToRGB(Color);
    Result:=UpperCase(Format('#%.2x%.2x%.2x',[GetRValue(RGB),
                                              GetGValue(RGB),
                                              GetBValue(RGB)]));
  end else Result:=TransparentColor;
end;

function WebColorStrToColor(WebColor: String): TColor;
begin
  Result:=0;
  if WebColor=''
    then Exit;
  if WebColor=TransparentColor then
  begin
    Result:=clNone;
    Exit;
  end;
  if WebColor[1]='#'
    then Delete(WebColor,1,1);
  if Length(WebColor)<>6
    then Exit;
  Result:=RGB(StrToIntDef('$'+Copy(WebColor,1,2),0),
              StrToIntDef('$'+Copy(WebColor,3,2),0),
              StrToIntDef('$'+Copy(WebColor,5,2),0));
end;

procedure Alert(const Val: String);
begin
  ShowMessage(Val);
end;

procedure Alert(Val: Integer);
begin
  Alert(IntToStr(Val));
end;

procedure Alert(Val: Double);
begin
  Alert(FloatToStr(Val));
end;

procedure Alert(Val: TObject);
begin
  if assigned(Val)
    then Alert(Val.{$IFDEF DELPHI2009_UP}ToString{$ELSE}ClassName{$ENDIF})
    else Alert('unassigned');
end;

//Test, ob v einen Dispatch-Zeiger enthlt, der nicht nil ist
function VarIsValidDispatch(const v: Variant): Boolean;
begin
  Result:=not {$IFDEF DELPHI6_UP}VarIsClear(v){$ELSE}VarIsEmpty(v){$ENDIF} and
          {$IFDEF DELPHI6_UP}VarIsType(v,varDispatch){$ELSE}(VarType(v)=varDispatch){$ENDIF} and
          assigned({$IFDEF DELPHI6_UP}FindVarData(v).VDispatch{$ELSE}TVarData(v).VDispatch{$ENDIF});
end;

function VarAsDispatch(const v: Variant): IDispatch;
begin
  try
    Result:=VarAsType(v,varDispatch);
  except
    Result:=nil;
  end;
end;

function VarAsBoolean(const v: Variant; Default: Boolean = false): Boolean;
begin
  try
    Result:=VarAsType(v,varBoolean);
  except
    Result:=Default;
  end;
end;

function VarAsString(const v: Variant; Default: String = ''): String;
begin
  try
    Result:=VarAsType(v,varOleStr);
  except
    Result:=Default;
  end;
end;

function VarAsDouble(const v: Variant; Default: Double = 0): Double;
begin
  try
    Result:=VarAsType(v,varDouble);
  except
    Result:=Default;
  end;
end;

function VarAsInteger(const v: Variant; Default: Integer = 0): Integer;
begin
  try
    Result:=VarAsType(v,varInteger);
  except
    Result:=Default;
  end;
end;

function Invoke(Dispatch: IDispatch; DispID: Integer; InvokeType: TInvokeType; const InvokeParams: array of OleVariant; CanExcept: Boolean = true): OleVariant; overload;
var
  n, ArgSize, Flags: Integer;
  hr: HRESULT;
  Params: TDispParams;
  DispIDNamed: Integer;
  DispatchEx: IDispatchEx;
  ExcepInfo: TExcepInfo;
begin
  Result:=Unassigned;
  try
    with Params do
    begin
      cArgs:=Length(InvokeParams);
      ArgSize:=cArgs*sizeof(OleVariant);
      GetMem(rgvarg,ArgSize);
      FillChar(rgvarg^,ArgSize,0);
      for n:=0 to cArgs-1 do
        OleVariant(rgvarg^[cArgs-n-1]):=InvokeParams[n];
      rgdispidNamedArgs:=nil;
      cNamedArgs:=0;
    end;
    case InvokeType of
      itMethod: Flags:=DISPATCH_METHOD;
      itGet   : Flags:=DISPATCH_PROPERTYGET;
      itSet   :
      begin
        //Sonderbehandlung!!! (siehe: http://msdn.microsoft.com/en-us/library/ms221479.aspx - Comments)
        DispIDNamed:=DISPID_PROPERTYPUT;
        Params.cNamedArgs:=1;
        Params.rgdispidNamedArgs:=@dispidNamed;
        {case VarType(InvokeParams[0]) of
          varDispatch: Flags:=DISPATCH_PROPERTYPUTREF;     //sonst wird der Wert nicht als Zeiger gesetzt
          else} Flags:=DISPATCH_PROPERTYPUT or DISPATCH_PROPERTYPUTREF;
//        end;
      end;
      itNew: Flags:=DISPATCH_CONSTRUCT;                    //neu in IDispatchEx
      else Flags:=0;
    end;
    FillChar(ExcepInfo,sizeof(ExcepInfo),0);
    if Supports(Dispatch,IDispatchEx,DispatchEx)
      then hr:=DispatchEx.InvokeEx(DispID,0{LOCALE_USER_DEFAULT},Flags,@Params,Result,ExcepInfo,nil)
      else hr:=Dispatch.Invoke(DispId,GUID_NULL,0{LOCALE_USER_DEFAULT},Flags,Params,@Result,@ExcepInfo,nil);
    if CanExcept and not Succeeded(hr)
      then raise Exception.Create(SysErrorMessage(Cardinal(hr)));
  finally
    FreeMem(Params.rgvarg);
  end;
end;

function Invoke(Dispatch: IDispatch; DispID: Integer; InvokeType: TInvokeType; CanExcept: Boolean = true): OleVariant; overload;
begin
  Result:=Invoke(Dispatch,DispID,InvokeType,[],CanExcept);
end;

function GetDispID(Dispatch: IDispatch; const InvokeName: String; var DispID: Integer; CanExcept: Boolean = false): Boolean;
var
  hr: HRESULT;
  DispatchEx: IDispatchEx;
  PName: PWideChar;
  InvokeNameW: WideString;
begin
  DispID:=-1;                                                   //das knnte durchaus eine gltige ID sein!
  Result:=false;
  if not assigned(Dispatch)
    then Exit;
  if Supports(Dispatch,IDispatchEx,DispatchEx) then
  begin
    PName:=SysAllocString(PWideChar(WideString(InvokeName)));   //WideString(): sicherheitshalber fr lterer Compiler
    try
      hr:=DispatchEx.GetDispID(PName,fdexNameCaseSensitive,DispID);
    finally
      SysFreeString(PName);
    end;
  end else
  begin
    InvokeNameW:=InvokeName;                                    //sicherheitshalber fr lterer Compiler
    hr:=Dispatch.GetIDsOfNames(GUID_NULL,@InvokeNameW,1,0{LOCALE_USER_DEFAULT},@DispId);
  end;
  if Succeeded(hr)
    then Result:=true
    else if CanExcept
      then raise Exception.Create(SysErrorMessage(Cardinal(hr)));
end;

function Invoke(Dispatch: IDispatch; const InvokeName: String; InvokeType: TInvokeType; const InvokeParams: array of OleVariant; CanExcept: Boolean = true): OleVariant; overload;
var
  DispID: Integer;
begin
  Result:=Unassigned;
  if not assigned(Dispatch)
    then Exit;
  if GetDispID(Dispatch,InvokeName,DispID,CanExcept)
    then Result:=Invoke(Dispatch,DispID,InvokeType,InvokeParams,CanExcept);
end;

function Invoke(Dispatch: IDispatch; const InvokeName: String; InvokeType: TInvokeType; CanExcept: Boolean = true): OleVariant; overload;
begin
  Result:=Invoke(Dispatch,InvokeName,InvokeType,[],CanExcept);
end;

function Exec(Dispatch: IDispatch; const Script: String; const Param: OleVariant; CanExcept: Boolean = false): OleVariant;
begin
  Result:=Exec(Dispatch,Script,[Param],CanExcept);
end;

function Exec(Dispatch: IDispatch; const Script: String; const Params: array of OleVariant; CanExcept: Boolean = false): OleVariant;
begin
  Result:=Invoke(Dispatch,Script,itMethod,Params,CanExcept);
end;

function Exec(Dispatch: IDispatch; const Script: String; CanExcept: Boolean = false): OleVariant;
begin
  Result:=Exec(Dispatch,Script,[],CanExcept);
end;

function Exec(Dispatch: IDispatch; const Params: array of OleVariant; CanExcept: Boolean = false): OleVariant;
begin
  Result:=Exec(Dispatch,'call',Params,CanExcept);
end;

function Exec(Dispatch: IDispatch; const Param: OleVariant; CanExcept: Boolean = false): OleVariant;
begin
  Result:=Exec(Dispatch,[Param],CanExcept);
end;

function Exec(Dispatch: IDispatch; CanExcept: Boolean = false): OleVariant;
begin
  Result:=Exec(Dispatch,[],CanExcept);
end;

function Document(WebBrowser: TOleControl): IDispatch;
var
  Document: OleVariant;
begin
  Result:=nil;
  if assigned(WebBrowser) then
  begin
    Document:=GetProp(WebBrowser.DefaultDispatch,'Document'{203},false);
    if VarIsValidDispatch(Document)
      then Result:=Document;
  end;
end;

function Script(WebBrowser: TOleControl): IDispatch;
var
  Document: IDispatch;
  Script: OleVariant;
begin
  Result:=nil;
  if assigned(WebBrowser) then
  begin
    Document:=BrowserTools.Document(WebBrowser);
    Script:=GetProp(Document,'Script',false);
    if VarIsValidDispatch(Script)
      then Result:=Script;
  end;
end;

function ScriptCanUsed(Script: IDispatch): Boolean;
var
  InvokeName: String;
  DispId: SYSINT;
begin
  Result:=false;
  if not assigned(Script)
    then Exit;
  InvokeName:='eval';
  Result:=Succeeded(Script.GetIDsOfNames(GUID_NULL,@InvokeName,1,
                    LOCALE_USER_DEFAULT,@DispId));
end;

function Eval(WebBrowser: TOleControl; const Script: String): OleVariant;
begin
  Result:=Exec(BrowserTools.Script(WebBrowser),'eval',[Script]);
end;

//ltere Compiler haben Probleme beim Aufruf berladener
//Prozeduren/Funktionen mit offenen Arrays als Parameter. Hier wrde z.B. aus
//dem bergebenen Array eine einziges Variant-Array gemacht und die falsche
//Funktion mit einem Parameter) aufgerufen. Deshalb die folgende Hilfsfunktion
//fr ltere Compiler:
{$IFDEF CAN_OVERLOAD}
function Create(Dispatch: IDispatch; const Name: String;
                const Params: array of OleVariant;
                CanExcept: Boolean): IDispatch;
begin
  Result:=Invoke(Dispatch,Name,itNew,Params,CanExcept);
end;
{$ELSE}
function Create_(Dispatch: IDispatch; const Name: String;
                 const Params: array of OleVariant;
                 CanExcept: Boolean): IDispatch;
begin
  Result:=Invoke(Dispatch,Name,itNew,Params,CanExcept);
end;

function Create(Dispatch: IDispatch; const Name: String;
                const Param1, Param2: OleVariant;
                CanExcept: Boolean): IDispatch; overload;
var
  Params: array[0..1] of OleVariant;
begin
  Params[0]:=Param1;
  Params[1]:=Param2;
  Result:=Invoke(Dispatch,Name,itNew,Params,CanExcept);
end;

function Create(Dispatch: IDispatch; const Name: String;
                const Param1, Param2, Param3: OleVariant;
                CanExcept: Boolean): IDispatch;
var
  Params: array[0..2] of OleVariant;
begin
  Params[0]:=Param1;
  Params[1]:=Param2;
  Params[2]:=Param3;
  Result:=Invoke(Dispatch,Name,itNew,Params,CanExcept);
end;

function Create(Dispatch: IDispatch; const Name: String;
                const Param1, Param2, Param3, Param4: OleVariant;
                CanExcept: Boolean): IDispatch;
var
  Params: array[0..3] of OleVariant;
begin
  Params[0]:=Param1;
  Params[1]:=Param2;
  Params[2]:=Param3;
  Params[3]:=Param4;
  Result:=Invoke(Dispatch,Name,itNew,Params,CanExcept);
end;

function Create(Dispatch: IDispatch; const Name: String;
                const Param1, Param2, Param3, Param4, Param5: OleVariant;
                CanExcept: Boolean): IDispatch;
var
  Params: array[0..4] of OleVariant;
begin
  Params[0]:=Param1;
  Params[1]:=Param2;
  Params[2]:=Param3;
  Params[3]:=Param4;
  Params[4]:=Param5;
  Result:=Invoke(Dispatch,Name,itNew,Params,CanExcept);
end;
{$ENDIF}

function Create(Dispatch: IDispatch; const Name: String;
                const Param: OleVariant; CanExcept: Boolean): IDispatch;
begin
  Result:={$IFDEF CAN_OVERLOAD}Create{$ELSE}Create_{$ENDIF}
          (Dispatch,Name,[Param],CanExcept);
end;

function Create(Dispatch: IDispatch; const Name: String; CanExcept: Boolean): IDispatch;
begin
  Result:={$IFDEF CAN_OVERLOAD}Create{$ELSE}Create_{$ENDIF}
          (Dispatch,Name,[],CanExcept);
end;

function CreateJSFunction(Script: IDispatch; const FuncDef: String;
                          const Params: array of String): IDispatch;
var
  n: Integer;
  FuncDesc: String;
begin
  FuncDesc:='';
  for n:=0 to High(Params) do
  begin
    if FuncDesc<>''
      then FuncDesc:=FuncDesc+', ';
    FuncDesc:=FuncDesc+'"'+Params[n]+'"';
  end;
  if FuncDesc<>''
    then FuncDesc:=FuncDesc+', ';
  FuncDesc:=FuncDesc+'"'+FuncDef+'"';                      //die Funktionslogik kommt erst zum Schlu
  Result:=Exec(Script,'eval','new Function('+FuncDesc+')');

//Etwas Theorie: JavaScript-Funktion als Objekt
//  <script type="text/javascript">
//Definition der Funktion:
//    var a=new Function("Param1","Param2","alert(this); alert(Param1); return Param2");
//"normaler" Funktionsaufruf:
//    alert(a(1,2));                -> Anzeige: [object]
//                                  -> Anzeige: 1
//                                  -> Anzeige: 2
//    alert(eval(a(1,2)));          -> Anzeige: [object]
//                                  -> Anzeige: 1
//                                  -> Anzeige: 2
//Aufruf der Funktion als Objekt:
//    alert(a.call(0,1,2));         -> Anzeige: 0           -> zustzlicher Parameter!!!
//                                  -> Anzeige: 1
//                                  -> Anzeige: 2
//Aufruf ohne Parameter liefert Funktionsdefinition:
//    alert(a);                     -> Anzeige: function anonymous(Param1, Param2) {alert(this); alert(Param1); return Param2}
//    alert(eval(a));               -> Anzeige: function anonymous(Param1, Param2) {alert(this); alert(Param1); return Param2}
//  </script>
end;

function CreateJSFunction(Script: IDispatch; const FuncDef: String; const Param: String): IDispatch;
begin
  Result:=CreateJSFunction(Script,FuncDef,[Param]);
end;

function CreateJSFunction(Script: IDispatch; const FuncDef: String): IDispatch;
begin
  Result:=CreateJSFunction(Script,FuncDef,[]);
end;

function GetProp(Dispatch: IDispatch; const Name: String; CanExcept: Boolean): OleVariant;
begin
  Result:=Invoke(Dispatch,Name,itGet,[],CanExcept);
end;

function GetProp(Dispatch: IDispatch; const Name: String; const Param: OleVariant; CanExcept: Boolean): OleVariant;
begin
  Result:=Invoke(Dispatch,Name,itGet,[Param],CanExcept);
end;

function GetProp(Dispatch: IDispatch; DispID: Integer; CanExcept: Boolean): OleVariant;
begin
  Result:=Invoke(Dispatch,DispID,itGet,[],CanExcept);
end;

procedure SetProp(Dispatch: IDispatch; const Name: String; const Param: OleVariant; CanExcept: Boolean = false);
begin
  Invoke(Dispatch,Name,itSet,Param,CanExcept);
end;

procedure SetProp(Dispatch: IDispatch; DispID: Integer; const Param: OleVariant; CanExcept: Boolean = false); overload;
begin
  Invoke(Dispatch,DispID,itSet,Param,CanExcept);
end;

//erstellt ein neues JavaScript-Objekt aus einer externen Quelle (URL) oder aus einem Skript (Src)
function AddJScript(Document: IDispatch; const URL, Src: String; ID: String = ''): IDispatch;
var
  Body: IDispatch;
begin
  Result:=Exec(Document,'createElement',['script']);
  SetProp(Result,'id',ID);
  SetProp(Result,'type','text/javascript');
  SetProp(Result,'src',URL);                               //externer Code kommt in die Eigenschaft src des Skript-Elements
  SetProp(Result,'text',Src);                              //interner Code kommt in die Eigenschaft text des Skript-Elements
  Body:=GetProp(Document,'body',false);
  Exec(Body,'appendChild',Result);
end;

function LoadBrowserFromSource(Browser: TOleControl; const Source: String; Wait: Boolean = false): Boolean;
var
  Stream: TStringStream;
begin
  Result:=false;
  if assigned(Browser) then
  begin
    Stream:=TStringStream.Create(Source);
    try
      Stream.Seek(0,soFromBeginning);
      Result:=LoadBrowserFromStream(Browser,Stream,Wait);
    finally
      Stream.Free;
    end;
  end;
end;

function LoadBrowserFromStream(Browser: TOleControl; Stream: TStream; Wait: Boolean = false): Boolean;
var
  PersistStreamInit: IPersistStreamInit;
  ReadyState: Integer;
begin
  Result:=false;
  if assigned(Browser) and assigned(Stream) then
  begin
    if Supports(Document(Browser),IPersistStreamInit,PersistStreamInit) then
      if Succeeded(PersistStreamInit.InitNew)
        then Result:=Succeeded(PersistStreamInit.Load(TStreamAdapter.Create(Stream)));
    if Result and Wait then
      repeat
        ReadyState:=VarAsInteger(GetProp(Browser.DefaultDispatch,'ReadyState'{-525},false),READYSTATE_COMPLETE);
        if ReadyState<READYSTATE_COMPLETE
          then Forms.Application.ProcessMessages;          //warten, bis die Seite komplett geladen wurde
      until ReadyState>=READYSTATE_COMPLETE;
  end;
end;

//------------------------------------------------------------------------------
//TExternalFunc

constructor TExternalMethod.Create(Disp: IDispatch; DispId: Integer; ExternalName: String; Proc: TExternalProc);
begin
  inherited Create;
  FProc:=Proc;
  FInstance:=TMethod(Proc).Data;
  FExternalName:=ExternalName;
  FDispId:=DispID;
  FDisp:=Disp;
end;

constructor TExternalMethod.Create(Disp: IDispatch; DispId: Integer; ExternalName: String; Func: TExternalFunc);
begin
  inherited Create;
  FFunc:=Func;
  FInstance:=TMethod(Func).Data;
  FExternalName:=ExternalName;
  FDispId:=DispID;
  FDisp:=Disp;
end;

function TExternalMethod.GetExternalName: String;
begin
  if FExternalName<>''
    then Result:=FExternalName
    else Result:=FName;
end;

function TExternalMethod.Call: Variant;
begin
  Result:=unassigned;
  try
    if assigned(FProc)
      then FProc                                           //Aufruf der Objektmethode ohne Rckgabewert (Prozedur)
      else if assigned(FFunc)
        then Result:=FFunc;                                //Aufruf der Objektmethode mit Rckgabewert (Funktion)
  except
    {$IFDEF DEBUG}                                         //Exceptions drfen nicht nach auen dringen
    ShowMessage(FName+': '+Exception(ExceptObject).Message);
    {$ENDIF}
  end;
end;

//------------------------------------------------------------------------------
//TExternalFunctions

constructor TExternalMethods.Create;
begin
  inherited Create;
  FItems:=TObjectList.Create(true);
  FID:=1;
end;

destructor TExternalMethods.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TExternalMethods.AddRef: Integer;
begin
  Result:=_AddRef;
end;

function TExternalMethods.Release: Integer;
begin
  Result:=_Release;
end;

//->IDispatch
function TExternalMethods.GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount: Integer; LocaleID: Integer; DispIDs: Pointer): HRESULT;
var
  n: Integer;
  Func: TExternalMethod;
begin
  Result:=NOERROR;
  for n:=0 to NameCount-1 do
  begin
    Func:=FindFunc(POleStrList(Names)^[n],UnassignedMethod);
    if assigned(Func)
      then PDispIdList(DispIDs)^[n]:=Func.DispId
      else begin
        PDispIdList(DispIDs)^[n]:=DISPID_UNKNOWN;
        Result:=HResult(DISP_E_UNKNOWNNAME);
      end;
  end;
end;

function TExternalMethods.GetTypeInfo(Index: Integer; LocaleID: Integer; out TypeInfo): HRESULT;
begin
  Result:=E_NOTIMPL;
end;

function TExternalMethods.GetTypeInfoCount(out Count: Integer): HRESULT;
begin
  Result:=E_NOTIMPL;
end;

function TExternalMethods.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult: Pointer; ExcepInfo: Pointer; ArgErr: Pointer): HRESULT;
type
  TVariantArray = array[0..65535] of Variant;
  PVariantArray = ^TVariantArray;
  TIntegerArray = array[0..65535] of Integer;
  PIntegerArray = ^TIntegerArray;
var
  Func: TExternalMethod;
  RResult: OleVariant;
begin
  Func:=GetFunc(DispID);
  if assigned(Func) then
  begin
    Result:=S_OK;
    RResult:=Unassigned;
    if Flags and DISPATCH_METHOD<>0 then
    begin
      RResult:=Func.Call;
      if assigned(VarResult)                               //Rckgabewert erwartet?
        then OleVariant(VarResult^):=RResult;              //-> Rckgabewert liefern
    end;
    if Flags and DISPATCH_PROPERTYGET<>0 then
    begin
      if assigned(VarResult) then
      begin
        if VarIsEmpty(RResult)
          //Methode ausfhren:
          then OleVariant(VarResult^):=Func.Call
          else OleVariant(VarResult^):=RResult;
      end else Result:=DISP_E_EXCEPTION;
    end;
    if Flags and DISPATCH_PROPERTYPUT<>0
      then Result:=E_NOTIMPL;
  end else Result:=E_NOTIMPL;
end;

procedure TExternalMethods.Clear;
begin
  FItems.Clear;
end;

procedure TExternalMethods.Reset;
begin
  Clear;
  FID:=1;
end;

function TExternalMethods.FindFunc(ExternalName: String; Method: TMethod): TExternalMethod;
var
  n: Integer;
  Func: TExternalMethod;
  FuncMethod: TMethod;
begin
  Result:=nil;
  if (ExternalName='') and not assigned(Method.Code) and not assigned(Method.Data)
    then Exit;
  for n:=FItems.Count-1 downto 0 do                        //von hinten suchen, um berladene Funktionen zu untersttzen
  begin
    Func:=TExternalMethod(FItems[n]);
    FuncMethod:=UnassignedMethod;
    if assigned(Func.Proc)
      then FuncMethod:=TMethod(Func.Proc);
    if assigned(Func.Func)
      then FuncMethod:=TMethod(Func.Func);
    if (((ExternalName<>'') and (Func.ExternalName=ExternalName)) or (ExternalName='')) and
      ((AssignedMethod(Method) and (FuncMethod.Data=Method.Data) and (FuncMethod.Data=Method.Data)) or not AssignedMethod(Method)) then
    begin
      Result:=Func;
      Exit;
    end;
  end;
end;

function TExternalMethods.RegisterMethod(Disp: IDispatch; Method: TExternalProc; ExternalName: String = ''): TExternalMethod;
begin
  Result:=nil;
  if not assigned(Method) or (ExternalName='') or assigned(FindFunc(ExternalName,TMethod(Method)))
    then Exit;                                             //diese Kombination gibt es schon
  Result:=TExternalMethod.Create(Disp,FID,ExternalName,Method);
  FItems.Add(Result);
  inc(FID);
end;

function TExternalMethods.RegisterMethod(Disp: IDispatch; Method: TExternalFunc; ExternalName: String = ''): TExternalMethod;
begin
  Result:=nil;
  if not assigned(Method) or (ExternalName='') or assigned(FindFunc(ExternalName,TMethod(Method)))
    then Exit;                                             //diese Kombination gibt es schon
  Result:=TExternalMethod.Create(Disp,FID,ExternalName,Method);
  FItems.Add(Result);
  inc(FID);
end;

function TExternalMethods.UnregisterMethod(Method: TExternalProc; ExternalName: String = ''): Boolean;
var
  Func: TExternalMethod;
begin
  Result:=false;
  while true do
  begin
    Func:=FindFunc(ExternalName,TMethod(Method));
    if assigned(Func) then
    begin
      FItems.Remove(Func);
      Result:=true;
    end else Exit
  end;
end;

function TExternalMethods.UnregisterMethod(Method: TExternalFunc; ExternalName: String = ''): Boolean;
var
  Func: TExternalMethod;
begin
  Result:=false;
  while true do
  begin
    Func:=FindFunc(ExternalName,TMethod(Method));
    if assigned(Func) then
    begin
      FItems.Remove(Func);
      Result:=true;
    end else Exit
  end;
end;

function TExternalMethods.UnregisterMethod(Disp: IDispatch): Boolean;
var
  n: Integer;
  Func: TExternalMethod;
begin
  Result:=false;
  if not assigned(Disp)
    then Exit;
  for n:=FItems.Count-1 downto 0 do
  begin
    Func:=Items[n];
    if Func.Disp=Disp
      then FItems.Remove(Func);
  end;
end;

function TExternalMethods.Count: Integer;
begin
  Result:=FItems.Count;
end;

function TExternalMethods.IndexOf(ExternalName: String): Integer;
begin
  Result:=FItems.IndexOf(FindFunc(ExternalName,UnassignedMethod));
end;

function TExternalMethods.GetFunc(DispID: Integer): TExternalMethod;
var
  n: Integer;
  Func: TExternalMethod;
begin
  Result:=nil;
  for n:=0 to FItems.Count-1 do
  begin
    Func:=GetItem(n);
    if Func.DispID=DispID then
    begin
      Result:=Func;
      Exit;
    end;
  end;
end;

function TExternalMethods.GetItem(Index: Integer): TExternalMethod;
begin
  Result:=nil;
  if (Index>=0) and (Index<Count)
    then Result:=TExternalMethod(FItems[Index]);
end;

//------------------------------------------------------------------------------
//TNoticationComponent

constructor TNoticationComponent.Create(Owner: TDocHostUIHandler);
begin
  inherited Create(nil);
  FOwner:=Owner;
end;

procedure TNoticationComponent.Notification(AComponent: TComponent; Operation: Classes.TOperation);
begin
  inherited;
  if assigned(FOwner) and (FOwner.FBrowser=AComponent) and (Operation=Classes.opRemove)
    then FOwner.FBrowser:=nil;
end;

//------------------------------------------------------------------------------
//TDocHostUIHandler

constructor TDocHostUIHandler.Create;
begin
  inherited;
  FNoticationComponent:=TNoticationComponent.Create(self);
  FOptions:=[hoAllowTextSelection];
end;

destructor TDocHostUIHandler.Destroy;
begin
  SetBrowser(nil);
  SetExternalMethods(nil);
  FNoticationComponent.Free;
  inherited;
end;

function TDocHostUIHandler.StoreOptions: Boolean;
begin
  Result:=not (FOptions=[hoAllowTextSelection]);
end;

procedure TDocHostUIHandler.SetBrowser(Value: TOleControl);
  procedure SetUIHandler(Value: IDocHostUIHandler);
  var
    Document: OleVariant;
    CustomDoc: ICustomDoc;
  begin
    Document:=BrowserTools.Document(FBrowser);
    if VarIsValidDispatch(Document) and (IDispatch(Document).QueryInterface(ICustomDoc,CustomDoc)=S_OK)
      then CustomDoc.SetUIHandler(Value);
  end;
begin
  if FBrowser<>Value then
  begin
    if assigned(FBrowser) then
    begin
      FBrowser.RemoveFreeNotification(FNoticationComponent);
      SetUIHandler(nil);
    end;
    FBrowser:=Value;
    if assigned(FBrowser) then
    begin
      FBrowser.FreeNotification(FNoticationComponent);
      SetUIHandler(self as IDocHostUIHandler);
    end;
  end;
end;

procedure TDocHostUIHandler.SetExternalMethods(Value: TExternalMethods);
begin
  if FExternalMethods<>Value then
  begin
    if assigned(FExternalMethods)
      then FExternalMethods.Release;
    FExternalMethods:=Value;
    if assigned(FExternalMethods)
      then FExternalMethods.AddRef;
  end;
end;

//->IInterface
function TDocHostUIHandler.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if GetInterface(IID,Obj)
    then Result:=S_OK
    else Result:=E_NOINTERFACE;
end;

function TDocHostUIHandler._AddRef: Integer;
begin
  Result:=-1;
end;

function TDocHostUIHandler._Release: Integer;
begin
  Result:=-1;
end;
//<-IInterface

//->IDocHostUIHandler
function TDocHostUIHandler.EnableModeless(const fEnable: BOOL): HResult;
begin
  Result:=S_OK;
end;

function TDocHostUIHandler.FilterDataObject(const pDO: IDataObject; out ppDORet: IDataObject): HResult;
begin
  ppDORet:=nil;
  Result:=S_FALSE;
end;

function TDocHostUIHandler.GetDropTarget(const pDropTarget: IDropTarget; out ppDropTarget: IDropTarget): HResult;
begin
  ppDropTarget:=nil;
  Result:=E_FAIL;
end;

function TDocHostUIHandler.GetExternal(out ppDispatch: IDispatch): HResult;
begin
  if assigned(FExternalMethods) then
  begin
    ppDispatch:=FExternalMethods as IDispatch;
    Result:=S_OK;
  end else Result:=E_FAIL;
end;

function TDocHostUIHandler.GetHostInfo(var pInfo: TDocHostUIInfo): HResult;
var
  StrLen: Integer;
begin
  try
    ZeroMemory(@pInfo,sizeof(TDocHostUIInfo));
    pInfo.cbSize:=sizeof(TDocHostUIInfo);
    if not (hoScrollBars in FOptions) then
      pInfo.dwFlags:=pInfo.dwFlags or DOCHOSTUIFLAG_SCROLL_NO;
    if not (ho3DBorder in FOptions) then
      pInfo.dwFlags:=pInfo.dwFlags or DOCHOSTUIFLAG_NO3DBORDER;
    if not (hoAllowTextSelection in FOptions) then
      pInfo.dwFlags:=pInfo.dwFlags or DOCHOSTUIFLAG_DIALOG;
    {$IFDEF DELPHI7_UP}
    if {$IFDEF DELPHIXE2_UP}StyleServices.Available{$ELSE}ThemeServices.ThemesEnabled{$ENDIF}
      then pInfo.dwFlags:=pInfo.dwFlags or DOCHOSTUIFLAG_THEME
      else if {$IFDEF DELPHIXE2_UP}StyleServices.Available{$ELSE}ThemeServices.ThemesAvailable{$ENDIF}
        then{$ENDIF} pInfo.dwFlags:=pInfo.dwFlags or DOCHOSTUIFLAG_NOTHEME;
    StrLen:=Length(CSS)+1;
    pInfo.pchHostCss:=CoTaskMemAlloc(StrLen*sizeof(WideChar));
    if assigned(pInfo.pchHostCss)
      then StringToWideChar(FCSS,pInfo.pchHostCss,StrLen)
      else raise Exception.Create('Can''t allocate CSS string');
    Result:=S_OK;
  except
    Result:=E_FAIL;
  end;
end;

function TDocHostUIHandler.GetOptionKeyPath(var pchKey: POLESTR; const dw: DWORD): HResult;
begin
  Result:=E_FAIL;
end;

function TDocHostUIHandler.HideUI: HResult;
begin
  Result:=S_OK;
end;

function TDocHostUIHandler.OnDocWindowActivate(const fActivate: BOOL): HResult;
begin
  Result:=S_OK;
end;

function TDocHostUIHandler.OnFrameWindowActivate(const fActivate: BOOL): HResult;
begin
  Result:=S_OK;
end;

function TDocHostUIHandler.ResizeBorder(const prcBorder: PRECT; const pUIWindow: IOleInPlaceUIWindow; const fFrameWindow: BOOL): HResult;
begin
  Result:=S_FALSE;
end;

function TDocHostUIHandler.ShowContextMenu(const dwID: DWORD; const ppt: PPOINT; const pcmdtReserved: {$IFDEF DELPHI6_UP}IInterface{$ELSE}IUnknown{$ENDIF}; const pdispReserved: IDispatch): HResult;
begin
  Result:=S_FALSE;
  if assigned(FOnShowContextMenu) then
    if FOnShowContextMenu(Self,ppt^)
      then Result:=S_OK
      else Result:=S_FALSE;
end;

function TDocHostUIHandler.TranslateAccelerator(const lpMsg: PMSG; const pguidCmdGroup: PGUID; const nCmdID: DWORD): HResult;
begin
  Result:=S_FALSE;
  if assigned(FOnTranslateAccelerator) then
    if FOnTranslateAccelerator(Self,lpmsg.wParam)
      then Result:=S_OK
      else Result:=S_FALSE;
end;

function TDocHostUIHandler.TranslateUrl(const dwTranslate: DWORD; const pchURLIn: POLESTR; var ppchURLOut: POLESTR): HResult;
var
  URLIn: String;
  URLOut: Pointer;
  StrLen: Integer;
begin
  Result:=S_FALSE;
  if assigned(FOnTranslateURL) and assigned(pchURLIn) then
  begin
    URLIn:=String(pchURLIn);                               //WideString -> AnsiString/WideString (je nach Compiler)
    if FOnTranslateURL(self,URLIn) then
    begin
      StrLen:=Length(URLIn)+1;
      URLOut:=CoTaskMemAlloc(StrLen*SizeOf(WideChar));
      if assigned(URLOut) then
      begin
        StringToWideChar(URLIn,URLOut,StrLen);
        ppchURLOut:=URLOut;
        Result:=S_OK;
      end else raise Exception.Create('Can''t allocate URL string');
    end;
  end;
end;

function TDocHostUIHandler.UpdateUI: HResult;
begin
  Result:=S_OK;
end;

function TDocHostUIHandler.ShowUI(const dwID: DWORD; const pActiveObject: IOleInPlaceActiveObject; const pCommandTarget: IOleCommandTarget; const pFrame: IOleInPlaceFrame; const pDoc: IOleInPlaceUIWindow): HResult;
begin
  try
    if FBrowser.CanFocus then
      FBrowser.SetFocus;
  except
    on E: EInvalidOperation do ;
  end;
  Result:=S_OK;
end;
//<-IDocHostUIHandler

end.
