unit DispObject;

interface

{$INCLUDE gmConfig.inc}

uses
  Classes, SysUtils, {$IFDEF DELPHI6_UP}Variants,{$ENDIF} ActiveX, Windows,
  BrowserTools;

resourcestring
  ExceptionNoInterface                = 'No Interface';
  ExceptionInvalidConstructorCalling  = 'No Interface.'+sLineBreak+
                                        'Don''t create object ''%s'' directly.';

type
  TDispObject = class;
  TDispObjectClass = class of TDispObject;

  INotifyInterface = interface({$IFDEF DELPHI6_UP}IInterface{$ELSE}IUnknown{$ENDIF})
    ['{6971F829-8011-4E6C-B6BD-FAA86048E059}']
    procedure FreeNotification(Sender: INotifyInterface);
    procedure RegisterNotification(AInterface: INotifyInterface);
    procedure UnregisterNotification(AInterface: INotifyInterface);
    function GetObject: TObject;
  end;

  TNotifyObject = class(TObject, INotifyInterface)
  private
    FNotifications: TInterfaceList;
  protected
    procedure DelObjectRef(AInterface: INotifyInterface; var AObject); overload;
    procedure DelObjectRef(Instance: TObject; var AObject); overload;
    procedure FreeNotify(Sender: TObject); virtual;
    function GetObject: TObject;
    //Interface-Methoden:
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    procedure FreeNotification(Sender: INotifyInterface);
    procedure RegisterNotification(AInterface: INotifyInterface);
    procedure UnregisterNotification(AInterface: INotifyInterface);
  public
    procedure BeforeDestruction; override;
    procedure AddNotify(AInterface: INotifyInterface);
    procedure RemoveNotify(AInterface: INotifyInterface);
  end;

  TDispObject = class(TNotifyObject)
  private
    FCreatedAsWrapper: Boolean;
    FData: TObject;
    FDisp: IDispatch;
    FTag: LongInt;
    FOwnsData: Boolean;
    function GetDispEx: IDispatchEx;
  protected
    constructor Create(Dispatch: IDispatch; CreateAsWrapper: Boolean = false);
    procedure AddProp(const Name: String; Value: TDispObject); overload;
    procedure SetDisp(Value: IDispatch); virtual;
    procedure SetProp(const Name: String; const Value: OleVariant; CreateIfNotExists: Boolean = false); overload;
    procedure SetProp(const Name: String; DispObject: TDispObject; CreateIfNotExists: Boolean = false); overload;
    procedure SetProp(DispID: Integer; const Value: OleVariant); overload;
    function Exec(const Name: String): OleVariant; overload;
    function Exec(const Name: String; DispObject: TDispObject): OleVariant; overload;
    //function Exec(const Name: String; const Param: OleVariant): OleVariant; overload;
    function Exec(const Name: String; const Params: array of OleVariant): OleVariant; overload;
    function AddProp(const Name: String): Boolean; overload;
    function DelProp(const Name: String): Boolean;
    function GetProp(const Name: String; const Param: String = ''): OleVariant;
    function GetPropAsVariant(const Name: String): OleVariant;
    function GetPropAsDispatch(const Name: String): IDispatch;
    function GetPropAsObject(const Name: String; PropClass: TDispObjectClass; var PropObject): TDispObject; virtual;
    function GetPropAsBoolean(const Name: String; Default: Boolean = false): Boolean;
    function GetPropAsInteger(const Name: String; Default: Integer = 0): Integer;
    function GetPropAsString(const Name: String; const Default: String = ''): String;
    function GetPropAsDouble(const Name: String; Default: Double = 0): Double;
    function GetObjectWithExec(const Name: String; ObjectClass: TDispObjectClass; var AObject): TDispObject; overload;
    function GetObjectWithExec(const Name: String; ObjectClass: TDispObjectClass; Param: TDispObject; var AObject): TDispObject; overload;
    property CreatedAsWrapper: Boolean read FCreatedAsWrapper;
  public
    destructor Destroy; override;
    procedure AfterConstruction; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; virtual;
    function AsVariant: Variant;
    function MethodExists(const Name: String): Boolean;
    function PropertyExists(const Name: String): Boolean;
    property Data: TObject read FData write FData;
    property Disp: IDispatch read FDisp write SetDisp;
    property DispEx: IDispatchEx read GetDispEx;
    property OwnsData: Boolean read FOwnsData write FOwnsData;
    property Tag: LongInt read FTag write FTag;
  end;

procedure ReleaseObject(var DispObject);
function GetDispObject(Dispatch: IDispatch): TDispObject;
function CreateObjectWrapper(AClass: TDispObjectClass; Dispatch: IDispatch): TDispObject; overload;
function New(AObject: TDispObject): TDispObject; overload; //JavaScript like Constructor

implementation

var
  DispObjects: TList = nil;

//Umweg ber diese Helper-Funktion, da der Constructor von TDispObject absichtlich
//nicht public ist: Ein TDispObject-Objekt existieren nicht ohne zugehriges
//JScript-Objekt (IDispatch-Interface) und mu immer in einem Skript-Kontext
//erstellt werden. Das geschieht ber TCustomScript.CreateXXX oder z.B. die
//Konstruktoren im TGoogle.Maps.XXX.
function CreateObjectWrapper(AClass: TDispObjectClass; Dispatch: IDispatch): TDispObject;
begin
  Result:=nil;
  if assigned(AClass)
    then Result:=AClass.Create(Dispatch,true);
end;

function GetDispObject(Dispatch: IDispatch): TDispObject;
var
  n: Integer;
  DispObject: TDispObject;
begin
  Result:=nil;
  if not assigned(DispObjects) or not assigned(Dispatch)
    then Exit;
  for n:=0 to DispObjects.Count-1 do
  begin
    DispObject:=TDispObject(DispObjects[n]);
    if DispObject.Disp=Dispatch then
    begin
      Result:=DispObject;
      Exit;
    end;
  end;
end;

function New(AObject: TDispObject): TDispObject;
begin
  Result:=AObject;
end;

procedure RegisterObject(AObject: TDispObject);
begin
  if not assigned(AObject)
    then Exit;
  if not assigned(DispObjects)
    then DispObjects:=TList.Create;
  if DispObjects.IndexOf(AObject)<0
    then DispObjects.Add(AObject);
end;

procedure UnregisterObject(AObject: TDispObject);
begin
  if not assigned(AObject) or not assigned(DispObjects)
    then Exit;
  DispObjects.Remove(AObject);
  if DispObjects.Count=0
    then FreeAndNil(DispObjects);
end;

procedure ReleaseObject(var DispObject);
var
  Obj: TObject;
begin
  Obj:=TObject(DispObject);
  if assigned(Obj) and assigned(DispObjects) and (DispObjects.IndexOf(Obj)>=0)
    then FreeAndNil(Obj);
end;

//------------------------------------------------------------------------------
//TNotifyObject

procedure TNotifyObject.BeforeDestruction;
var
  NotifyInterface: INotifyInterface;
begin
  if assigned(FNotifications) then
    while FNotifications.Count>0 do
    begin
      NotifyInterface:=INotifyInterface(FNotifications[0]);
      FNotifications.Delete(0);
      NotifyInterface.FreeNotification(self);
    end;
  FreeAndNil(FNotifications);
end;

function TNotifyObject.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if GetInterface(IID,Obj)
    then Result:=S_OK
    else Result:=E_NOINTERFACE;
end;

function TNotifyObject._AddRef: Integer;
begin
  Result:=-1;
end;

function TNotifyObject._Release: Integer;
begin
  Result:=-1;
end;

procedure TNotifyObject.FreeNotification(Sender: INotifyInterface);
begin
  if assigned(Sender) then
  begin
    FreeNotify(Sender.GetObject);                          //zur besseren Handhabung ber virtuelle Methode
    if assigned(FNotifications)
      then FNotifications.Remove(Sender);
  end;
end;

procedure TNotifyObject.AddNotify(AInterface: INotifyInterface);
begin
  RegisterNotification(AInterface);
end;

procedure TNotifyObject.RegisterNotification(AInterface: INotifyInterface);
begin
  if assigned(AInterface) then
  begin
    if not assigned(FNotifications)
      then FNotifications:=TInterfaceList.Create;
    if FNotifications.IndexOf(AInterface)<0 then
    begin
      FNotifications.Add(AInterface);
      AInterface.RegisterNotification(self);
    end;
  end;
end;

procedure TNotifyObject.RemoveNotify(AInterface: INotifyInterface);
begin
  UnregisterNotification(AInterface);
end;

procedure TNotifyObject.UnregisterNotification(AInterface: INotifyInterface);
var
  n: Integer;
begin
  if assigned(FNotifications) then
  begin
    n:=FNotifications.IndexOf(AInterface);
    if n>=0 then
    begin
      FNotifications.Delete(n);
      AInterface.UnregisterNotification(self);
    end;
  end;
  if FNotifications.Count=0
    then FreeAndNil(FNotifications);
end;

procedure TNotifyObject.DelObjectRef(AInterface: INotifyInterface; var AObject);
begin
  if assigned(AInterface)
    then DelObjectRef(AInterface.GetObject,AObject);
end;

procedure TNotifyObject.DelObjectRef(Instance: TObject; var AObject);
begin
  if Instance=TObject(AObject)
    then TObject(AObject):=nil;                            //nur die Objektreferez lschen
end;

function TNotifyObject.GetObject: TObject;
begin
  Result:=self;
end;

procedure TNotifyObject.FreeNotify(Sender: TObject);
begin
end;

//------------------------------------------------------------------------------
//TDispObject

constructor TDispObject.Create(Dispatch: IDispatch; CreateAsWrapper: Boolean);
begin
  inherited Create;
  if not assigned(Dispatch)
    then raise Exception.Create(ExceptionNoInterface);
  FOwnsData:=true;
  FCreatedAsWrapper:=CreateAsWrapper;
  SetDisp(Dispatch);
  RegisterObject(self);                                    //damit kein Objekt verloren geht
end;
{
constructor TDispObject.Create(Script: TCustomScript);
begin
  Create(Script,Unassigned);
end;

constructor TDispObject.Create(Script: TCustomScript; const Value: OleVariant);
begin
  if not assigned(Script)
    then raise Exception.Create(NoScript);
  if Value=Unassigned
    then Create(Invoke(FDispatch,'Object',itNew)
    else Create(Invoke(FDispatch,'Object',itNew,[Value]);
end;
}
destructor TDispObject.Destroy;
begin
  if FOwnsData
    then FData.Free;
  UnregisterObject(self);
  SetDisp(nil);
  inherited;
end;

procedure TDispObject.AfterConstruction;
begin
  inherited;
  //es wurde TObject.Create aufgerufen statt TDispObject.Create(Dispatch):
  if not assigned(FDisp)
    then raise Exception.Create(Format(ExceptionInvalidConstructorCalling,[ClassName]));
end;

class function TDispObject.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=false;
end;

function TDispObject.AsVariant: Variant;
begin
  Result:=Disp;
end;

procedure TDispObject.SetDisp(Value: IDispatch);
begin
  if FDisp<>Value then
  begin
    if assigned(FDisp)
      then FDisp._Release;
    FDisp:=Value;
    if assigned(FDisp)
      then FDisp._AddRef;
  end;
end;

function TDispObject.GetDispEx: IDispatchEx;
begin
  Result:=nil;
  Supports(FDisp,IDispatchEx,Result);
end;

function TDispObject.PropertyExists(const Name: String): Boolean;
var
  DispID: Integer;
begin
  Result:=GetDispID(FDisp,Name,DispID);
end;

function TDispObject.MethodExists(const Name: String): Boolean;
begin
  Result:=PropertyExists(Name);
end;

procedure TDispObject.SetProp(const Name: String; const Value: OleVariant; CreateIfNotExists: Boolean = false);
begin
  if CreateIfNotExists and not PropertyExists(Name)
    then AddProp(Name);
  BrowserTools.SetProp(FDisp,Name,Value,{false}true);
end;

procedure TDispObject.SetProp(const Name: String; DispObject: TDispObject; CreateIfNotExists: Boolean = false);
begin
  if assigned(DispObject)
    then SetProp(Name,DispObject.Disp,CreateIfNotExists)
    else SetProp(Name,Null,CreateIfNotExists);
end;

procedure TDispObject.SetProp(DispID: Integer; const Value: OleVariant);
begin
  BrowserTools.SetProp(FDisp,DispID,Value,false);
end;

function TDispObject.Exec(const Name: String): OleVariant;
begin
  Result:=BrowserTools.Exec(FDisp,Name,[],true);
end;

function TDispObject.Exec(const Name: String; DispObject: TDispObject): OleVariant;
begin
  if assigned(DispObject)
    then Result:=Exec(Name,[DispObject.Disp])
    else Result:=Exec(Name,[Null]);
end;
{
function TDispObject.Exec(const Name: String; const Param: OleVariant): OleVariant;
begin
  Result:=BrowserTools.Exec(FDisp,Name,Param,true);
end;
}
function TDispObject.Exec(const Name: String; const Params: array of OleVariant): OleVariant;
begin
  Result:=BrowserTools.Exec(FDisp,Name,Params,true);
end;

function TDispObject.GetProp(const Name: String; const Param: String = ''): OleVariant;
begin
  if Param=''
    then Result:=BrowserTools.GetProp(FDisp,Name,true)
    else Result:=BrowserTools.GetProp(FDisp,Name,Param,true);
end;

function TDispObject.GetPropAsVariant(const Name: String): OleVariant;
begin
  Result:=Unassigned;
  if PropertyExists(Name)
    then Result:=GetProp(Name);
end;

function TDispObject.GetPropAsBoolean(const Name: String; Default: Boolean = false): Boolean;
begin
  Result:=Default;
  if PropertyExists(Name)
    then Result:=VarAsBoolean(GetProp(Name),Default);
end;

function TDispObject.GetPropAsInteger(const Name: String; Default: Integer = 0): Integer;
begin
  Result:=Default;
  if PropertyExists(Name)
    then Result:=VarAsInteger(GetProp(Name),Default);
end;

function TDispObject.GetPropAsString(const Name: String; const Default: String = ''): String;
begin
  Result:=Default;
  if PropertyExists(Name)
    then Result:=VarAsString(GetProp(Name),Default);
end;

function TDispObject.GetPropAsDouble(const Name: String; Default: Double = 0): Double;
begin
  Result:=Default;
  if PropertyExists(Name)
    then Result:=VarAsDouble(GetProp(Name),Default);
end;

function TDispObject.GetPropAsDispatch(const Name: String): IDispatch;
begin
  Result:=nil;
  if PropertyExists(Name)
    then Result:=VarAsDispatch(GetProp(Name));
end;

function TDispObject.GetPropAsObject(const Name: String; PropClass: TDispObjectClass; var PropObject): TDispObject;
var
  Value: OleVariant;
begin
  Result:=nil;
  Value:=GetPropAsVariant(Name);
  if VarIsValidDispatch(Value) then
  begin
    if assigned(TObject(PropObject)) and (TObject(PropObject) is TDispObject)
      then TDispObject(PropObject).Disp:=Value
      else TObject(PropObject):=CreateObjectWrapper(PropClass,Value);
    Result:=TDispObject(PropObject);
  end;
end;

function TDispObject.GetObjectWithExec(const Name: String; ObjectClass: TDispObjectClass;
                                       var AObject): TDispObject;
begin
  Result:=GetObjectWithExec(Name,ObjectClass,nil,AObject);
end;

function TDispObject.GetObjectWithExec(const Name: String; ObjectClass: TDispObjectClass;
                                       Param: TDispObject; var AObject): TDispObject;
var
  Value: OleVariant;
begin
  Result:=nil;
  if MethodExists(Name) then
  begin
    if assigned(Param)
      then Value:=Exec(Name,Param)
      else Value:=Exec(Name);
    if VarIsValidDispatch(Value) then
    begin
      if assigned(TObject(AObject)) and (TObject(AObject) is TDispObject)
        then TDispObject(AObject).Disp:=Value
        else TObject(AObject):=CreateObjectWrapper(ObjectClass,Value);
      Result:=TDispObject(AObject);
    end;
  end;
end;

function TDispObject.AddProp(const Name: String): Boolean;
var
  DispatchEx: IDispatchEx;
  PName: PWideChar;
  DispID: Integer;
begin
  Result:=false;
  if Supports(FDisp,IDispatchEx,DispatchEx) then
  begin
    PName:=SysAllocString(PWideChar(WideString(Name)));    //WideString(): sicherheitshalber fr lterer Compiler
    try
      if Succeeded(DispatchEx.GetDispID(PName,fdexNameCaseSensitive or fdexNameEnsure,DispID)) then
      begin
        SetProp(DispID,varEmpty);                          //wichtig - sonst ist die neue Eigenschaft spter nicht ber ihren Namen ansprechbar!!!
        Result:=true;
      end;
    finally
      SysFreeString(PName);
    end;
  end;
end;

procedure TDispObject.AddProp(const Name: String; Value: TDispObject);
begin
  SetProp(Name,Value,true);
end;

function TDispObject.DelProp(const Name: String): Boolean;
var
  DispatchEx: IDispatchEx;
  PName: PWideChar;
begin
  Result:=false;
  if Supports(FDisp,IDispatchEx,DispatchEx) then
  begin
    PName:=SysAllocString(PWideChar(WideString(Name)));    //WideString(): sicherheitshalber fr lterer Compiler
    try
      Result:=Succeeded(DispatchEx.DeleteMemberByName(PName,fdexNameCaseSensitive));
    finally
      SysFreeString(PName);
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure DeleteJSObjects;
var
  DispObject: TDispObject;
begin
  while assigned(DispObjects) and (DispObjects.Count>0) do
  begin
    DispObject:=TDispObject(DispObjects[0]);
    ReleaseObject(DispObject);
  end;
end;

initialization

finalization
  DeleteJSObjects;
end.
