unit JScriptObjects;

interface

{$INCLUDE gmConfig.inc}

uses
  Forms, SysUtils, Contnrs, Classes, Dialogs, OleCtrls, SHDocVw, Math, Windows,
  DispObject, HTMLObjects, BrowserTools {$IFDEF DELPHI6_UP}, Variants{$ENDIF};

const
  EmptyPage                          = 'about:blank';

  ExceptionConstructorNeedsArguments = 'Constructor needs arguments';
  ExceptionNoDivNode                 = 'No DIV node';
  ExceptionNoFunction                = 'No function';
  ExceptionNoMethod                  = 'No method';
  ExceptionNoScript                  = 'No script';
  ExceptionIndexOutOfRange           = 'Index out of range';

  ArgumentsProp_Callee               = 'callee';
  ArgumentsProp_Length               = 'length';

  ArrayMethod_Concat                 = 'concat';
  ArrayMethod_Join                   = 'join';
  ArrayMethod_Pop                    = 'pop';
  ArrayMethod_Push                   = 'push';
  ArrayMethod_Reverse                = 'reverse';
  ArrayMethod_Shift                  = 'shift';
  ArrayMethod_Slice                  = 'slice';
  ArrayMethod_Sort                   = 'sort';
  ArrayMethod_Splice                 = 'splice';
  ArrayMethod_Unshift                = 'unshift';
  ArrayProp_Length                   = 'length';

  ErrorProp_Description              = 'description';
  ErrorProp_Message                  = 'message';
  ErrorProp_Name                     = 'name';
  ErrorProp_Number                   = 'number';

  FunctionMethod_Apply               = 'apply';
  FunctionMethod_Call                = 'call';
  FunctionProp_Arguments             = 'arguments';
  FunctionProp_Caller                = 'caller';
  FunctionProp_Length                = 'length';

  ObjectMethod_HasOwnProperty        = 'hasOwnProperty';
  ObjectMethod_IsPrototypeOf         = 'isPrototypeOf';
  ObjectMethod_ToLocaleString        = 'toLocaleString';
  ObjectMethod_ToString              = 'toString';
  ObjectMethod_ValueOf               = 'valueOf';
  ObjectProp_Constructor             = 'constructor';
  ObjectProp_PropertyIsEnumerable    = 'propertyIsEnumerable';
  ObjectProp_Prototype               = 'prototype';

  RegExprArrayProp_Input             = 'input';
  RegExprArrayProp_Index             = 'index';
  RegExprArrayProp_LastIndex         = 'lastIndex';

  RegularExpressionMethod_Compile    = 'compile';
  RegularExpressionMethod_Exec       = 'exec';
  RegularExpressionMethod_Test       = 'test';
  RegularExpressionProp_Global       = 'global';
  RegularExpressionProp_IgnoreCase   = 'ignoreCase';
  RegularExpressionProp_Multiline    = 'multiline';
  RegularExpressionProp_Source       = 'source';

  ScriptMethod_Array                 = 'Array';
  ScriptMethod_Eval                  = 'eval';
  ScriptMethod_Function              = 'Function';
  ScriptMethod_Object                = 'Object';
  ScriptMethod_RegExp                = 'RegExp';
  ScriptMethod_String                = 'String';
  ScriptMethod_TypeOf                = 'typeof';
  ScriptProp_Document                = 'document';
  ScriptProp_Navigator               = 'navigator';

  StringMethod_CharAt                = 'charAt';
  StringMethod_CharCodeAt            = 'charCodeAt';
  StringMethod_Concat                = 'concat';
  StringMethod_IndexOf               = 'indexOf';
  StringMethod_LastIndexOf           = 'lastIndexOf';
  StringMethod_Match                 = 'match';
  StringMethod_Replace               = 'replace';
  StringMethod_Search                = 'search';
  StringMethod_Slice                 = 'slice';
  StringMethod_Split                 = 'split';
  StringMethod_SubStr                = 'substr';
  StringMethod_SubString             = 'substring';
  StringMethod_ToLowerCase           = 'toLowerCase';
  StringMethod_ToUpperCase           = 'toUpperCase';

type
  TJSObjectType = (otNoObject, otUnknown, otError, otArguments, otObject, otFunction, otArray, otString);
  TJScriptString = class;
  TCustomScript = class;
  OleArray = array of OleVariant;

  TScriptObject = class(TDispObject)
  private
    FScript: TCustomScript;
  protected
    constructor Create; reintroduce; overload; virtual;
    constructor Create(Script: TCustomScript); reintroduce; overload; virtual;
    constructor Create(Script: TCustomScript; Dispatch: IDispatch); reintroduce; overload; virtual;
    procedure FreeNotify(Sender: TObject); override;
    procedure SetScript(Script: TCustomScript);
    function GetPropAsObject(const Name: String; PropClass: TDispObjectClass;
                             var PropObject; CreateIfNotExists: Boolean = false): TDispObject; reintroduce; virtual;
    function GetObjectWithExec(const Method: String; ObjectClass: TDispObjectClass;
                               var AObject): TDispObject; overload;
    function GetObjectWithExec(const Method: String; ObjectClass: TDispObjectClass;
                               Param: TDispObject; var AObject): TDispObject; overload;
    class function New(Script: TCustomScript): IDispatch; virtual; abstract;
    property Script: TCustomScript read FScript;
  public
    procedure AfterConstruction; override;
  end;

  TScriptObjectClass = class of TScriptObject;

  TJScriptError = class(TScriptObject)
  private
    function GetDescription: String;
    function GetMessage: String;
    function GetName: String;
    function GetNumber: Integer;
  protected
    //class function New(Script: TCustomScript): IDispatch; override;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    property Description: String read GetDescription;
    property Message: String read GetMessage;
    property Name: String read GetName;
    property Number: Integer read GetNumber;
  end;

  TCustomJScriptObject = class(TScriptObject)
  private
    procedure SetProperty(Name: String; const Value: OleVariant);
    //function GetConstructor: IDispatch;
    function GetProperty(Name: String): OleVariant;
    //function GetPropertyIsEnumerable(PropName: String): Boolean;
    function GetPrototype: OleVariant;
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    function ConstructorObj: IDispatch;
    function HasOwnProperty(const PropName: String): Boolean;
    function IsPrototypeOf(Object2: IDispatch): Boolean;
    function PropertyIsEnumerable(const PropName: String): Boolean;
    function ToLocaleString: String;
    function ToString(Radix: Integer = 0): String; reintroduce; virtual;
    function ValueOf: OleVariant;
    //property ConstructorObj: IDispatch read GetConstructor;
    property Prototype: OleVariant read GetPrototype;
    //property PropertyIsEnumerable[PropName: String]: Boolean read GetPropertyIsEnumerable;
    property Properties[Name: String]: OleVariant read GetProperty write SetProperty; default;
  end;

  TJScriptObject = class(TCustomJScriptObject)
  public
    constructor Create; overload; override;
    constructor Create(Script: TCustomScript); overload; override;
    constructor Create(const Values: array of String); reintroduce; overload;
    constructor Create(Script: TCustomScript; const Values: array of String); reintroduce; overload;
  end;

  TJScriptArguments = class(TScriptObject)
  private
    function GetCallee: IDispatch;
    function GetArgument(Index: Integer): OleVariant;
    function GetLength: Integer;
  public
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    function Count: Integer;
    property Callee: IDispatch read GetCallee;
    property Length: Integer read GetLength;
    property Arguments[Index: Integer]: OleVariant read GetArgument; default;
  end;

  {$IFDEF USE_ANONYMOUS_METHODS}
  TProcEx = reference to procedure(Sender: TObject);
  TFuncEx = reference to function(Sender: TObject): Variant;
  {$ENDIF}

  TJScriptFunction = class(TCustomJScriptObject)
  private
    FArguments: TJScriptArguments;
    {$IFDEF USE_ANONYMOUS_METHODS}
    FProc: TProc;
    FFunc: TFunc<Variant>;
    FProcEx: TProcEx;
    FFuncEx: TFuncEx;
    {$ENDIF}
    procedure SetLength(Value: Integer);
    function GetArguments: TJScriptArguments;
    function GetCaller: String;
    function GetLength: Integer;
  protected
    {$IFDEF USE_ANONYMOUS_METHODS}
    procedure InternalCallbackProc;
    function InternalCallbackFunc: Variant;
    {$ENDIF}
  public
    constructor Create(Method: TExternalProc; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Method: TExternalFunc; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(const Arguments: array of String; const Body: String); reintroduce; overload;
    constructor Create(Script: TCustomScript; Method: TExternalProc; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; Method: TExternalFunc; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; const Arguments: array of String; const Body: String); reintroduce; overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    constructor Create(Method: TProc; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Method: TFunc<Variant>; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Method: TProcEx; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Method: TFuncEx; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; Method: TProc; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; Method: TFunc<Variant>; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; Method: TProcEx; const ExternalName: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; Method: TFuncEx; const ExternalName: String = ''); reintroduce; overload;
    {$ENDIF}
    destructor Destroy; override;
    procedure AfterConstruction; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    function Apply(ThisObj: IDispatch; const ArgArray: array of OleVariant): OleVariant;
    function Call(ThisObj: IDispatch; const ArgArray: array of OleVariant): OleVariant;
    property Arguments: TJScriptArguments read GetArguments;
    property Caller: String read GetCaller;
    property Length: Integer read GetLength write SetLength;
  end;

  TJScriptArray = class(TCustomJScriptObject)
  private
    FConcat: TJScriptArray;
    FReverse: TJScriptArray;
    FSlice: TJScriptArray;
    FSplice: TJScriptArray;
    procedure SetItem(Name: String; const Value: OleVariant);
    procedure SetLength(Value: Integer);
    function GetItem(Name: String): OleVariant;
    function GetLength: Integer;
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create(Length: Integer = 0); reintroduce; overload;
    constructor Create(const Items: array of OleVariant); reintroduce; overload;
    constructor Create(Script: TCustomScript; Length: Integer = 0); reintroduce; overload;
    constructor Create(Script: TCustomScript; const Items: array of OleVariant); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    procedure Sort(SortFunction: TJScriptFunction = nil);
    procedure Unshift(const Items: array of OleVariant);
    function Concat(const Items: array of OleVariant): TJScriptArray;
    function Count: Integer;                               //zustzlich, da in Delphi gebruchlicher als "Length"
    function Join(Separator: String = ''): String;
    function Pop: OleVariant;
    function Push(const Item: OleVariant): Integer; overload;
    function Push(const Item: TScriptObject): Integer; overload;
    function {$IFDEF CAN_OVERLOAD}Push{$ELSE}PushArray{$ENDIF}(const Items: array of OleVariant): Integer; overload;
    function Push(const Items: array of TScriptObject): Integer; overload;
    function Reverse: TJScriptArray;
    function Shift: OleVariant;
    function Slice(StartIndex: Integer; EndIndex: Integer = 0): TJScriptArray; //orig.: slice(start,[end])
    function Splice(Start, DeleteCount: Integer): TJScriptArray; overload;
    function Splice(Start, DeleteCount: Integer; const Items: array of OleVariant): TJScriptArray; overload;
    property Length: Integer read GetLength write SetLength;
    property Items[Name: String]: OleVariant read GetItem write SetItem; default;
  end;

  //Delphi-Erweiterungen fr das Array-Objekt:
  TJScriptArrayEx = class(TJScriptArray)
  public
    procedure Insert(Index: Integer; const Value: OleVariant); overload; virtual;
    procedure Insert(Index: Integer; Value: TDispObject); overload; virtual;
    function Add(const Value: OleVariant): Integer; overload; virtual;
    function Add(Value: TDispObject): Integer; overload; virtual;
    function Count: Integer;
  end;

  TObjectArray = class(TJScriptArrayEx)
  private
    FItems: TStringList;
    procedure SetItem(Index: Integer; Value: TDispObject);
    function GetItem(Index: Integer): TDispObject;
  protected
    procedure SetDisp(Value: IDispatch); override;
    function CreateObject(Dispatch: IDispatch): TDispObject; virtual; abstract;
  public
    destructor Destroy; override;
    property Items[Index: Integer]: TDispObject read GetItem write SetItem; default;
  end;

  TIntegerArray = class(TJScriptArrayEx)
  private
    function GetItem(Index: Integer): Integer;
  public
    procedure Insert(Index: Integer; Value: Integer); reintroduce; virtual;
    function Add(Value: Integer): Integer; reintroduce; overload; virtual;
    function Add(Values: array of Integer): Integer; reintroduce; overload; virtual;
    property Items[Index: Integer]: Integer read GetItem; default;
  end;

  TDoubleArray = class(TJScriptArrayEx)
  private
    function GetItem(Index: Integer): Double;
  public
    procedure Insert(Index: Integer; Value: Double); reintroduce; virtual;
    function Add(Value: Double): Integer; reintroduce; overload; virtual;
    function Add(Values: array of Double): Integer; reintroduce; overload; virtual;
    property Items[Index: Integer]: Double read GetItem; default;
  end;

  {$IFDEF USE_REGIONS}{$REGION 'Documentation TStringArray'}{$ENDIF}
  ///<summary>
  ///  JavaScript: Array.&#60;string&#62;
  ///  <para/>
  ///  <para><b>Unit</b></para>
  ///    <see cref="JScriptObjects"/>
  ///  <para/>
  ///  <para><b>Hierarchy</b></para>
  ///    <see cref="TObject"/><br/>
  ///    &#160;<see cref="DispObject|TNotifyObject"/><br/>
  ///    &#160;&#160;<see cref="DispObject|TDispObject"/><br/>
  ///    &#160;&#160;&#160;<see cref="JScriptObjects|TScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptObject"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArray"/><br/>
  ///    &#160;&#160;&#160;&#160;&#160;&#160;<see cref="JScriptObjects|TJScriptArrayEx"/><br/>
  ///  <para/>
  ///  <para><b>Methods</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="JScriptObjects|TStringArray.Insert(Integer,String)">Insert</see></item>
  ///    <item><see cref="JScriptObjects|TStringArray.Add(String)">Add(String)</see></item>
  ///    <item><see cref="JScriptObjects|TStringArray.Add(array of String)">Add(array of String)</see></item>
  ///  </list>
  ///  <para><b>Properties</b></para>
  ///  <list type="bullet">
  ///    <item><see cref="JScriptObjects|TStringArray.Items(Integer)">Items</see></item>
  ///  </list>
  ///</summary>
  {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
  TStringArray = class(TJScriptArrayEx)
  private
    function GetItem(Index: Integer): String;
  public
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Insert'}{$ENDIF}
    ///<seealso cref="JScriptObjects|TStringArray">TStringArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    procedure Insert(Index: Integer; Value: String); reintroduce; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<seealso cref="JScriptObjects|TStringArray">TStringArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(Value: String): Integer; reintroduce; overload; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Add'}{$ENDIF}
    ///<seealso cref="JScriptObjects|TStringArray">TStringArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    function Add(Values: array of String): Integer; reintroduce; overload; virtual;
    function ToString: String; reintroduce; virtual;
    {$IFDEF USE_REGIONS}{$REGION 'Documentation Items'}{$ENDIF}
    ///<seealso cref="JScriptObjects|TStringArray">TStringArray</seealso>
    {$IFDEF USE_REGIONS}{$ENDREGION}{$ENDIF}
    property Items[Index: Integer]: String read GetItem; default;
  end;

  TRegExprArray = class(TJScriptArray)
  private
    function GetInput: String;
    function GetIndex: Integer;
    function GetLastIndex: Integer;
  public
    property Input: String read GetInput;
    property Index: Integer read GetIndex;
    property LastIndex: Integer read GetLastIndex;
  end;

  TJScriptRegularExpression = class(TScriptObject)
  private
    FExec: TRegExprArray;
    function GetGlobal: Boolean;
    function GetIgnoreCase: Boolean;
    function GetMultiline: Boolean;
    function GetSource: String;
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create(const Pattern: String; const Flags: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; const Pattern: String; const Flags: String = ''); reintroduce; overload;
    destructor Destroy; override;
    procedure Compile(Pattern: String; Flags: String = '');
    function Exec(Str: String): TRegExprArray; overload;
    function Exec(Str: TJScriptString): TRegExprArray; overload;
    function Test(Str: String): Boolean;
    property Global: Boolean read GetGlobal;
    property IgnoreCase: Boolean read GetIgnoreCase;
    property Multiline: Boolean read GetMultiline;
    property Source: String read GetSource;
  end;

  TJScriptString = class(TJScriptArray)
  private
    FMatch: TRegExprArray;
    FSplit: TJScriptArray;
  protected
    class function New(Script: TCustomScript): IDispatch; override;
  public
    constructor Create(const Value: String = ''); reintroduce; overload;
    constructor Create(Script: TCustomScript; const Value: String = ''); reintroduce; overload;
    destructor Destroy; override;
    class function IsObjectFromDisp(Disp: IDispatch): Boolean; override;
    function CharAt(Index: Integer): Char;
    function CharCodeAt(Index: Integer): Integer;
    function Concat(String2: String): String; overload;
    function Concat(Strings: array of String): String; overload;
    function IndexOf(SubString: String; StartIndex: Integer = 0): Integer;
    function LastIndexOf(SubString: String; StartIndex: Integer = -1): Integer;
    function Match(RgExp: String): TRegExprArray; overload;
    function Match(RgExp: TJScriptRegularExpression): TRegExprArray; overload;
    function Replace(RgExp: String; ReplaceText: String): String; overload;
    function Replace(RgExp: TJScriptRegularExpression; ReplaceText: String): String; overload;
    function Replace(RgExp: String; ReplaceText: TJScriptFunction): String; overload;
    function Replace(RgExp: TJScriptRegularExpression; ReplaceText: TJScriptFunction): String; overload;
    function Search(RgExp: String): Integer; overload;
    function Search(RgExp: TJScriptRegularExpression): Integer; overload;
    function Slice(StartIndex: Integer): String; overload;
    function Slice(StartIndex: Integer; EndIndex: Integer): String; overload;
    function Split(Separator: String = ''; Limit: Integer = -1): TJScriptArray; overload;
    function Split(Separator: TJScriptRegularExpression = nil; Limit: Integer = -1): TJScriptArray; overload;
    function SubStr(Start: Integer; Length: Integer = -1): String;
    function SubString(Start: Integer; Length: Integer = -1): String;
    function ToLowerCase: String;
    function ToUpperCase: String;
  end;

  TCallbackObject = class(TObject)
  private
    FJScriptFunction: TJScriptFunction;
    FReady: Boolean;
    FScript: TCustomScript;
    function GetArgument(Index: Integer): OleVariant;
  protected
    procedure Callback; virtual; abstract;
  public
    constructor Create(Script: TCustomScript); virtual;
    destructor Destroy; override;
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;
    function ArgumentCount: Integer;
    property JScriptFunction: TJScriptFunction read FJScriptFunction;
    property Arguments[Index: Integer]: OleVariant read GetArgument;
    property Ready: Boolean read FReady write FReady;
    property Script: TCustomScript read FScript;
  end;

  TCallbackManager = class(TObject)
  private
    FItems: TObjectList;
    procedure Add(CallbackObject: TCallbackObject);
    procedure Remove(CallbackObject: TCallbackObject);
  public
    constructor Create;
    destructor Destroy; override;
    procedure CleanUp;
  end;

  TShowContextMenuEvent = procedure(Sender: TObject; Point: TPoint) of object;

  TTimerEvent = class(TObject)
  private
    FEventTime: Cardinal;
    FID: Integer;
  public
    constructor Create(Interval: Cardinal);
    procedure Execute; virtual; abstract;
  end;

  {$IFDEF USE_ANONYMOUS_METHODS}
  TProcTimerEvent = class(TTimerEvent)
  private
    FProc: TProc;
  public
    constructor Create(Method: TProc; Interval: Cardinal);
    procedure Execute; override;
  end;
  {$ENDIF}

  TExternalProcTimerEvent = class(TTimerEvent)
  private
    FProc: TExternalProc;
  public
    constructor Create(Method: TExternalProc; Interval: Cardinal);
    procedure Execute; override;
  end;

  TTimerThread = class(TThread)
  private
    FCurrentID: Integer;
    FEvents: TThreadList;
  protected
    procedure Execute; override;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Delete(ID: Integer);
    function Add(TimerEvent: TTimerEvent): Integer;
  end;

  TCustomScript = class(TDispObject)
  private
    FCallbackManager: TCallbackManager;
    FExternalMethods: TExternalMethods;
    FDocHostUIHandler: TDocHostUIHandler;
    FDocument: TDocument;
    FNavigator: TNavigator;
    FTimerThread: TTimerThread;
    FTypeOfFunc: TJScriptFunction;
    FOnReloadAccelerator: TNotifyEvent;
    FOnShowContextMenu: TShowContextMenuEvent;
    FOnTranslateURL: TTranslateURLEvent;
    procedure MakeFunctionNameUnique(var FunctionName: String);
    procedure SetVar(Name: String; const Value: OleVariant);
    function GetDocument: TDocument;
    function GetNavigator: TNavigator;
    function GetVar(Name: String): OleVariant;
  protected
    procedure FreeNotify(Sender: TObject); override;
    function ShowContextMenu(Sender: TObject; Point: TPoint): Boolean; virtual;
    function TranslateAccelerator(Sender: TObject; Accelerator: Integer): Boolean; virtual;
    function TranslateURL(Sender: TObject; var URL: String): Boolean; virtual;
    //Funktionen zur Erstellung neuer JScript-Objekte:
    function NewJScriptArray(Length: Integer = 0): IDispatch; overload;
    function NewJScriptArray(const Items: array of OleVariant): IDispatch; overload;
    function NewJScriptFunction(Method: TExternalProc; ExternalName: String = ''): IDispatch; overload;
    function NewJScriptFunction(Method: TExternalFunc; ExternalName: String = ''): IDispatch; overload;
    function NewJScriptFunction(const Arguments: array of String; Body: String): IDispatch; overload;
    function NewJScriptObject: IDispatch;
    function NewJScriptRegExp(const Pattern: String; const Flags: String = ''): IDispatch;
    function NewJScriptString(const Value: String = ''): IDispatch;
    property CallbackManager: TCallbackManager read FCallbackManager;
    property OnShowContextMenu: TShowContextMenuEvent read FOnShowContextMenu write FOnShowContextMenu;
    property OnReloadAccelerator: TNotifyEvent read FOnReloadAccelerator write FOnReloadAccelerator;
    property OnTranslateURL: TTranslateURLEvent read FOnTranslateURL write FOnTranslateURL;
  public
    constructor Create(Browser: TOleControl); reintroduce;
    destructor Destroy; override;
    procedure AddVar(const Name: String; const Value: OleVariant); overload;
    procedure AddVar(const Name: String; Value: TDispObject); overload;
    procedure ClearTimeout(TimeoutID: Integer);
    procedure DelVar(const Name: String);
    procedure RegisterObject(ScriptObject: TScriptObject); virtual;
    function Eval(const Script: String): OleVariant;
    function LoadLib(Source: TStream): Boolean; overload;
    function LoadLib(URL: String; Timeout: Cardinal = 0): Boolean; overload;
    function LoadLib(Instance: THandle; ResName: String; ResType: PChar): Boolean; overload;
    function LoadEmptyPage(Wait: Boolean = true): Boolean; overload;
    function LoadPage(Source: String; Wait: Boolean = true): Boolean; overload;
    function LoadPage(Source: TStream; Wait: Boolean = true): Boolean; overload;
    function NewArray(Length: Integer = 0): TJScriptArray; overload;
    function NewArray(const Arguments: array of OleVariant): TJScriptArray; overload;
    function NewRegExp(const Pattern: String; const Flags: String = ''): TJScriptRegularExpression;
    function NewFunction(Method: TExternalProc; const ExternalName: String = ''): TJScriptFunction; overload;
    function NewFunction(Method: TExternalFunc; const ExternalName: String = ''): TJScriptFunction; overload;
    function NewFunction(const Arguments: array of String; const Body: String): TJScriptFunction; overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    function NewFunction(Method: TProc; ExternalName: String = ''): TJScriptFunction; overload;
    function NewFunction(Method: TFunc<Variant>; ExternalName: String = ''): TJScriptFunction; overload;
    function NewFunction(Method: TProcEx; ExternalName: String = ''): TJScriptFunction; overload;
    function NewFunction(Method: TFuncEx; ExternalName: String = ''): TJScriptFunction; overload;
    {$ENDIF}
    function NewObject: TCustomJScriptObject;
    function NewString(const Value: String = ''): TJScriptString;
    function SetTimeout(Method: TExternalProc; Interval: Integer): Integer; overload;
    {$IFDEF USE_ANONYMOUS_METHODS}
    function SetTimeout(Method: TProc; Interval: Integer): Integer; overload;
    {$ENDIF}
    function TypeOf(const AObject: OleVariant): String; overload;
    function TypeOf(AObject: TDispObject): String; overload;
    property DocHostUIHandler: TDocHostUIHandler read FDocHostUIHandler;
    property Document: TDocument read GetDocument;
    property ExternalMethods: TExternalMethods read FExternalMethods;
    property Navigator: TNavigator read GetNavigator;
    property Vars[Name: String]: OleVariant read GetVar write SetVar; default;
  end;

procedure Alert(Val: TCustomJScriptObject); overload;
function GetJSObjectType(Dispatch: IDispatch; Document: IDispatch = nil): TJSObjectType;
function GetJSObjectClass(ObjectType: TJSObjectType): TDispObjectClass;
function CreateJSObject(ObjectType: TJSObjectType; Dispatch: IDispatch): TDispObject;
function CreateObjectWrapper(Script: TCustomScript; AClass: TDispObjectClass; Dispatch: IDispatch): TDispObject; overload;
function CreateJScriptFunction(Script: TCustomScript; Method: TExternalProc; ExternalName: String = ''): TJScriptFunction; overload;
function CreateJScriptFunction(Script: TCustomScript; Method: TExternalFunc; ExternalName: String = ''): TJScriptFunction; overload;
function JSObjectTypeToStr(ObjectType: TJSObjectType): String;
function ObjectHasMembers(Dispatch: IDispatch; Members: array of String): Boolean;
function Append(const a: OleVariant; const b: array of OleVariant): OleArray; overload;
function Append(const a, b: array of OleVariant): OleArray; overload;
function Append(const a: array of String; const b: String): OleArray; overload;
function VarToDisp(const Value: OleVariant): IDispatch;
function New(Script: TCustomScript; ScriptObject: TScriptObjectClass): IDispatch;
function Script: TCustomScript; overload;
function Script(Browser: TOleControl): TCustomScript; overload;

implementation

type
  TCustomScriptCollection = class(TObject)
  private
    FItems: TList;
    function GetItem(Browser: TOleControl): TCustomScript;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(Item: TCustomScript);
    procedure Remove(Item: TCustomScript);
    property Items[Browser: TOleControl]: TCustomScript read GetItem; default;
  end;

var
  Scripts: TCustomScriptCollection = nil;

//------------------------------------------------------------------------------
//TCustomScriptCollection

constructor TCustomScriptCollection.Create;
begin
  inherited;
  FItems:=TList.Create;
end;

destructor TCustomScriptCollection.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure TCustomScriptCollection.Add(Item: TCustomScript);
begin
  if assigned(Item) and (FItems.IndexOf(Item)<0)
    then FItems.Add(Item);
end;

function TCustomScriptCollection.GetItem(Browser: TOleControl): TCustomScript;
var
  n: Integer;
  Script: TCustomScript;
begin
  Result:=nil;
  if not assigned(Browser) and (FItems.Count>0) then
  begin
    Result:=TCustomScript(FItems[FItems.Count-1]);
    Exit;
  end;
  for n:=0 to FItems.Count-1 do
  begin
    Script:=TCustomScript(FItems[n]);
    if assigned(Script) and (Script.DocHostUIHandler.Browser=Browser) then
    begin
      Result:=Script;
      Exit;
    end;
  end;
end;

procedure TCustomScriptCollection.Remove(Item: TCustomScript);
begin
  FItems.Remove(Item);
end;

function Script: TCustomScript;
begin
  Result:=Script(nil);
end;

function Script(Browser: TOleControl): TCustomScript;
begin
  Result:=nil;
  if assigned(Scripts)
    then Result:=Scripts[Browser];
end;

//------------------------------------------------------------------------------

{$IFNDEF DELPHI6_UP}
function TryStrToInt(const S: string; out Value: Integer): Boolean;
var
  Code: Integer;
begin
  Val(S,Value,Code);
  Result:=Code=0;
end;

function TryStrToFloat(const S: string; out Value: Double): Boolean;
var
  Code: Integer;
begin
  Val(S,Value,Code);
  Result:=Code=0;
end;

function TryStrToBool(const S: string; out Value: Boolean): Boolean;
begin
  Result:=false;
  if LowerCase(S)='true' then
  begin
    Value:=true;
    Result:=true;
  end else if LowerCase(S)='false' then
  begin
    Value:=false;
    Result:=true;
  end;
end;
{$ENDIF}

function New(Script: TCustomScript; ScriptObject: TScriptObjectClass): IDispatch;
begin
  Result:=nil;
  if assigned(Script) and assigned(ScriptObject)
    then Result:=ScriptObject.New(Script);
end;

procedure Alert(Val: TCustomJScriptObject);
begin
  if assigned(Val)
    then Alert(Val.ToString)
    else Alert('unassigned');
end;

function VarToDisp(const Value: OleVariant): IDispatch;
begin
  try
    Result:=VarAsType(Value,varDispatch);
  except
    Result:=nil;
  end;
end;

function Append(const a: OleVariant; const b: array of OleVariant): OleArray;
var
  n: Integer;
begin
  SetLength(Result,Length(b)+1);
  if
    {$IFDEF DELPHI6_UP}
    VarIsType(a,varDispatch) and not assigned(FindVarData(a).VDispatch)
    {$ELSE}
    (VarType(a)=varDispatch) and not assigned(TVarData(a).VDispatch)
    {$ENDIF}
    then Result[0]:=Unassigned
    else Result[0]:=a;
  for n:=0 to High(b) do
    Result[n+1]:=b[n];
end;

function Append(const a, b: array of OleVariant): OleArray;
var
  n: Integer;
begin
  System.SetLength(Result,Length(a)+Length(b));
  for n:=0 to High(a) do
    Result[n]:=a[n];
  for n:=0 to High(b) do
    Result[n+Length(a)]:=b[n];
end;

function Append(const a: array of String; const b: String): OleArray; overload;
var
  n: Integer;
begin
  System.SetLength(Result,Length(a)+1);
  for n:=0 to High(a) do
    Result[n]:=a[n];
  Result[High(Result)]:=b;
end;

function ObjectHasMembers(Dispatch: IDispatch; Members: array of String): Boolean;
var
  n, DispID: Integer;
begin
  Result:=false;
  if not assigned(Dispatch)
    then Exit;
  for n:=0 to High(Members) do
    if not GetDispID(Dispatch,Members[n],DispID)
      then Exit;
  Result:=true;
end;

function GetJSObjectType(Dispatch: IDispatch; Document: IDispatch = nil): TJSObjectType;
  function IsConstructor(ObjName: String): Boolean;
  var
    ConstObj: OleVariant;
    Script, Dispatch, IConstructor: IDispatch;
  begin
    Result:=false;
    Script:=GetProp(Document,'script',false);
    IConstructor:=GetProp(Dispatch,'constructor',false);
    ConstObj:=GetProp(Script,ObjName,false);
    if {$IFDEF DELPHI6_UP}VarIsType(ConstObj,varDispatch){$ELSE}VarType(ConstObj)=varDispatch{$ENDIF} then
    begin
      Dispatch:=VarAsType(ConstObj,varDispatch);
      Result:=IConstructor=Dispatch;
    end;
  end;
begin
  Result:=otNoObject;
  if not assigned(Dispatch)
    then Exit;
  Result:=otUnknown;
  //Test auf Error-Objekt
  if ObjectHasMembers(Dispatch,['description','message','name','number']) then
  begin
    Result:=otError;
    Exit;
  end;
  //Test auf Function-Objekt
  if ObjectHasMembers(Dispatch,[//'prototype',             ist entgegen der MSDN-Dokumentation nicht immer vorhanden!!!
                                'apply','call']) then
  begin
    Result:=otFunction;
    Exit;
  end;
  //Test auf Array-Objekt
  if ObjectHasMembers(Dispatch,[//'prototype',             ist entgegen der MSDN-Dokumentation nicht immer vorhanden!!!
                                'sort','unshift','concat','join','pop','push',
                                'reverse','shift','slice','splice','length']) then
  begin
    Result:=otArray;
    Exit;
  end;
  //Test auf Arguments-Objekt
  if ObjectHasMembers(Dispatch,['callee','length']) then
  begin
    Result:=otArguments;
    Exit;
  end;
  //Test auf String-Objekt
  if ObjectHasMembers(Dispatch,[//'prototype',             ist entgegen der MSDN-Dokumentation nicht immer vorhanden!!!
                                'constructor','length','anchor','big','blink',
                                'bold','charAt','charCodeAt','concat'{...}]) then
  begin
    Result:=otString;
    Exit;
  end;
  //ToDo: andere Objekte analysieren
  //Test auf Object-Objekt
  if ObjectHasMembers(Dispatch,[//'prototype',             ist entgegen der MSDN-Dokumentation nicht immer vorhanden!!!
                                'constructor','propertyIsEnumerable',
                                'isPrototypeOf','hasOwnProperty','toLocaleString','toString','valueOf']) then
  begin
    Result:=otObject;
    Exit;
  end;
end;

function JSObjectTypeToStr(ObjectType: TJSObjectType): String;
begin
  case ObjectType of
    otNoObject: Result:='NoObject';
    otUnknown : Result:='Unknown';
    otError   : Result:='Error';
    otObject  : Result:='Object';
    otFunction: Result:='Function';
    otArray   : Result:='Array';
    else Result:='Unknown';
  end;
end;

function GetJSObjectClass(ObjectType: TJSObjectType): TDispObjectClass;
begin
  case ObjectType of
    otNoObject: Result:=nil;
    otUnknown : Result:=TDispObject;
    otError   : Result:=TJScriptError;
    otObject  : Result:=TCustomJScriptObject;
    otFunction: Result:=TJScriptFunction;
    otArray   : Result:=TJScriptArray;
    else Result:=nil;
  end;
end;

function CreateJSObject(ObjectType: TJSObjectType; Dispatch: IDispatch): TDispObject;
var
  DispObjectClass: TDispObjectClass;
begin
  Result:=nil;
  DispObjectClass:=GetJSObjectClass(ObjectType);
  if assigned(DispObjectClass)
    then Result:=CreateObjectWrapper(DispObjectClass,Dispatch);
end;

function CreateObjectWrapper(Script: TCustomScript; AClass: TDispObjectClass; Dispatch: IDispatch): TDispObject;
begin
  Result:=nil;
  if assigned(AClass) then
  begin
    Result:=DispObject.CreateObjectWrapper(AClass,Dispatch);
    if assigned(Result) and (Result is TScriptObject)
      then TScriptObject(Result).SetScript(Script);
  end;
end;

function CreateJScriptFunction(Script: TCustomScript; Method: TExternalProc; ExternalName: String = ''): TJScriptFunction; overload;
begin
  Result:=nil;
  if not assigned(Script) or not assigned(Method)
    then Exit;
  Result:=Script.NewFunction(Method,ExternalName);
end;

function CreateJScriptFunction(Script: TCustomScript; Method: TExternalFunc; ExternalName: String = ''): TJScriptFunction; overload;
begin
  Result:=nil;
  if not assigned(Script) or not assigned(Method)
    then Exit;
  Result:=Script.NewFunction(Method,ExternalName);
end;

//------------------------------------------------------------------------------
//TScriptObject

constructor TScriptObject.Create;
begin
  Create(JScriptObjects.Script);
end;

constructor TScriptObject.Create(Script: TCustomScript);
begin
  Create(Script,New(Script));                              //create a new JScript object
end;

procedure TScriptObject.AfterConstruction;
begin
  inherited;
  if not CreatedAsWrapper                                  //as a wrapper for an existing object
    then Script.RegisterObject(Self);
end;

constructor TScriptObject.Create(Script: TCustomScript; Dispatch: IDispatch);
begin
  inherited Create(Dispatch);
  SetScript(Script);
end;

procedure TScriptObject.SetScript(Script: TCustomScript);
begin
  if FScript<>Script then
  begin
    if assigned(FScript)
      then FScript.UnregisterNotification(Self);
    FScript:=Script;
    if assigned(FScript)
      then FScript.RegisterNotification(Self);
  end;
end;

function TScriptObject.GetPropAsObject(const Name: String; PropClass: TDispObjectClass;
                                       var PropObject; CreateIfNotExists: Boolean): TDispObject;
begin
  if not PropertyExists(Name) and CreateIfNotExists and assigned(PropClass) and
     PropClass.InheritsFrom(TScriptObject)
    then SetProp(Name,TScriptObjectClass(PropClass).New(Script),true);
  Result:=inherited GetPropAsObject(Name,PropClass,PropObject);
  if assigned(Result) and (Result is TScriptObject)
    then TScriptObject(Result).SetScript(FScript);
end;

function TScriptObject.GetObjectWithExec(const Method: String; ObjectClass: TDispObjectClass;
                                         var AObject): TDispObject;
begin
  Result:=inherited GetObjectWithExec(Method,ObjectClass,AObject);
  if assigned(Result) and (Result is TScriptObject)
    then TScriptObject(Result).SetScript(FScript);
end;

function TScriptObject.GetObjectWithExec(const Method: String; ObjectClass: TDispObjectClass;
                                         Param: TDispObject; var AObject): TDispObject;
begin
  Result:=inherited GetObjectWithExec(Method,ObjectClass,Param,AObject);
  if assigned(Result) and (Result is TScriptObject)
    then TScriptObject(Result).SetScript(FScript);
end;

procedure TScriptObject.FreeNotify(Sender: TObject);
begin
  inherited;
  DelObjectRef(Sender,FScript);                            //Objektreferenz entfernen
  if not assigned(FScript)
    then Disp:=nil;                                        //sicherheiteshabler: ohne Script geht nichts mehr
end;

//------------------------------------------------------------------------------
//TJScriptError
{
class function TJScriptError.New(Script: TCustomScript): IDispatch;
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  Script.???;
end;
}
class function TJScriptError.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=GetJSObjectType(Disp)=otError;
end;

function TJScriptError.GetDescription: String;
begin
  Result:=GetPropAsString(ErrorProp_Description);
end;

function TJScriptError.GetMessage: String;
begin
  Result:=GetPropAsString(ErrorProp_Message);
end;

function TJScriptError.GetName: String;
begin
  Result:=GetPropAsString(ErrorProp_Name);
end;

function TJScriptError.GetNumber: Integer;
begin
  Result:=GetPropAsInteger(ErrorProp_Number);
end;

//------------------------------------------------------------------------------
//TCustomJScriptObject

class function TCustomJScriptObject.New(Script: TCustomScript): IDispatch;
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  Result:=Script.NewJScriptObject;
end;

class function TCustomJScriptObject.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=GetJSObjectType(Disp)=otObject;
end;
{
function TJSObject.GetConstructor: IDispatch;
begin
  Result:=GetPropAsDispatch('constructor');
end;
}
function TCustomJScriptObject.ConstructorObj: IDispatch;
begin
  Result:=GetPropAsDispatch(ObjectProp_Constructor);
end;

function TCustomJScriptObject.GetPrototype: OleVariant;
begin
  Result:=GetPropAsVariant(ObjectProp_Prototype);
end;
{
function TJSObject.GetPropertyIsEnumerable(PropName: String): Boolean;
begin
  Result:=VarAsBoolean(GetProp('propertyIsEnumerable',PropName));
end;
}
function TCustomJScriptObject.PropertyIsEnumerable(const PropName: String): Boolean;
begin
  Result:=VarAsBoolean(GetProp(ObjectProp_PropertyIsEnumerable,PropName));
end;

function TCustomJScriptObject.HasOwnProperty(const PropName: String): Boolean;
begin
  Result:=VarAsBoolean(Exec(ObjectMethod_HasOwnProperty,[PropName]));
end;

function TCustomJScriptObject.IsPrototypeOf(Object2: IDispatch): Boolean;
begin
  Result:=VarAsBoolean(Exec(ObjectMethod_IsPrototypeOf,[Object2]));
end;

function TCustomJScriptObject.ToLocaleString: String;
begin
  Result:=VarAsString(Exec(ObjectMethod_ToLocaleString));
end;

function TCustomJScriptObject.ToString(Radix: Integer = 0): String;
begin
  Result:=VarAsString(Exec(ObjectMethod_ToString,[Radix]));
end;

function TCustomJScriptObject.ValueOf: OleVariant;
begin
  Result:=Exec(ObjectMethod_ValueOf);
end;

function TCustomJScriptObject.GetProperty(Name: String): OleVariant;
begin
  Result:=GetPropAsVariant(Name);
end;

procedure TCustomJScriptObject.SetProperty(Name: String; const Value: OleVariant);
begin
  if not PropertyExists(Name)
    then AddProp(Name);
  SetProp(Name,Value);
end;

//------------------------------------------------------------------------------
//TJScriptObject

constructor TJScriptObject.Create;
begin
  inherited;
end;

constructor TJScriptObject.Create(Script: TCustomScript);
begin
  inherited;
end;

constructor TJScriptObject.Create(const Values: array of String);
begin
  Create(JScriptObjects.Script,Values);
end;

constructor TJScriptObject.Create(Script: TCustomScript; const Values: array of String);
var
  n, m, i: Integer;
  d: Double;
  b: Boolean;
  s: String;
  Value: Variant;
  ValueList: TStringList;
begin
  inherited Create(Script);
  //Initialisierung des Objektes mit Eigenschaften und den dazugehrigen Werten
  //analog zum JavaScript-Syntax:
  //{a: 1, b: 'Test'} -> ['a: 1','b: Test']
  if Length(Values)>0 then
  begin
    ValueList:=TStringList.Create;
    try
      for n:=0 to High(Values) do
      begin
        s:=Values[n];
        m:=Pos(':',s);
        if m>0
          then s[m]:='=';                                  //kein StringReplace() wegen lterer Compiler
        ValueList.Add(s);
      end;
      for n:=0 to ValueList.Count-1 do
      begin
      {$IFDEF DELPHI7_UP}
        s:=Trim(ValueList.ValueFromIndex[n]);
      {$ELSE}
        s:=ValueList.Strings[n];
        if Pos('=',s)>0
          then Delete(s,1,Pos('=',s));
        s:=Trim(s);
      {$ENDIF}
        if s=''
          then Value:=Null                                 //Null
          else if TryStrToInt(s,i)
            then Value:=i                                  //Integer
            else if TryStrToFloat(s,d)
              then Value:=d                                //Double
              else if TryStrToBool(s,b)
                then Value:=b                              //Boolean
                else Value:=s;                             //String
        SetProp(ValueList.Names[n],Value,true);            //neue Eigenschaft setzen bzw. erstellen, falls noch nicht vorhanden
      end;
    finally
      ValueList.Free;
    end;
  end;
end;

//------------------------------------------------------------------------------
//TJScriptArguments

class function TJScriptArguments.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=GetJSObjectType(Disp)=otNoObject
end;

function TJScriptArguments.GetCallee: IDispatch;
begin
  Result:=GetPropAsDispatch(ArgumentsProp_Callee);
end;

function TJScriptArguments.Count: Integer;
begin
  Result:=GetLength;
end;

function TJScriptArguments.GetArgument(Index: Integer): OleVariant;
begin
  Result:=GetPropAsVariant(IntToStr(Index));
end;

function TJScriptArguments.GetLength: Integer;
begin
  Result:=GetPropAsInteger(ArgumentsProp_Length);
end;

//------------------------------------------------------------------------------
//TJScriptFunction

constructor TJScriptFunction.Create(Method: TExternalProc; const ExternalName: String);
begin
  Create(JScriptObjects.Script,Method,ExternalName);
end;

constructor TJScriptFunction.Create(Method: TExternalFunc; const ExternalName: String);
begin
  Create(JScriptObjects.Script,Method,ExternalName);
end;

constructor TJScriptFunction.Create(const Arguments: array of String; const Body: String);
begin
  Create(JScriptObjects.Script,Arguments,Body);
end;

constructor TJScriptFunction.Create(Script: TCustomScript; Method: TExternalProc; const ExternalName: String);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script,Script.NewJScriptFunction(Method,ExternalName));
end;

constructor TJScriptFunction.Create(Script: TCustomScript; Method: TExternalFunc; const ExternalName: String);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script,Script.NewJScriptFunction(Method,ExternalName));
end;

constructor TJScriptFunction.Create(Script: TCustomScript; const Arguments: array of String; const Body: String);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script,Script.NewJScriptFunction(Arguments,Body));
end;

{$IFDEF USE_ANONYMOUS_METHODS}
constructor TJScriptFunction.Create(Method: TProc; const ExternalName: String = '');
begin
  Create(JScriptObjects.Script,Method,ExternalName);
end;

constructor TJScriptFunction.Create(Method: TFunc<Variant>; const ExternalName: String = '');
begin
  Create(JScriptObjects.Script,Method,ExternalName);
end;

constructor TJScriptFunction.Create(Method: TProcEx; const ExternalName: String = '');
begin
  Create(JScriptObjects.Script,Method,ExternalName);
end;

constructor TJScriptFunction.Create(Method: TFuncEx; const ExternalName: String = '');
begin
  Create(JScriptObjects.Script,Method,ExternalName);
end;

constructor TJScriptFunction.Create(Script: TCustomScript; Method: TProc; const ExternalName: String = '');
begin
  if not assigned(Method)
    then raise Exception.Create(ExceptionNoMethod);
  FProc:=Method;
  Create(Script,InternalCallbackProc,ExternalName);
end;

constructor TJScriptFunction.Create(Script: TCustomScript; Method: TFunc<Variant>; const ExternalName: String = '');
begin
  if not assigned(Method)
    then raise Exception.Create(ExceptionNoMethod);
  FFunc:=Method;
  Create(Script,InternalCallbackFunc,ExternalName);
end;

constructor TJScriptFunction.Create(Script: TCustomScript; Method: TProcEx; const ExternalName: String = '');
begin
  if not assigned(Method)
    then raise Exception.Create(ExceptionNoMethod);
  FProcEx:=Method;
  Create(Script,InternalCallbackProc,ExternalName);
end;

constructor TJScriptFunction.Create(Script: TCustomScript; Method: TFuncEx; const ExternalName: String = '');
begin
  if not assigned(Method)
    then raise Exception.Create(ExceptionNoMethod);
  FFuncEx:=Method;
  Create(Script,InternalCallbackFunc,ExternalName);
end;
{$ENDIF}

destructor TJScriptFunction.Destroy;
begin
  if assigned(Script) and assigned(Script.ExternalMethods)
    then Script.ExternalMethods.UnregisterMethod(Disp);
  FArguments.Free;
  inherited;
end;

procedure TJScriptFunction.AfterConstruction;
begin
  inherited;
  RegisterNotification(Script);
end;

class function TJScriptFunction.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=GetJSObjectType(Disp)=otFunction;
end;

{$IFDEF USE_ANONYMOUS_METHODS}
function TJScriptFunction.InternalCallbackFunc: Variant;
begin
  Result:=Null;
  if assigned(FFuncEx)
    then Result:=FFuncEx(Self)
    else if assigned(FFunc)
      then Result:=FFunc;
end;

procedure TJScriptFunction.InternalCallbackProc;
begin
  if assigned(FProcEx)
    then FProcEx(Self)
    else if assigned(FProc)
      then FProc;
end;
{$ENDIF}

function TJScriptFunction.GetArguments: TJScriptArguments;
begin
  Result:=TJScriptArguments(GetPropAsObject(FunctionProp_Arguments,TJScriptArguments,FArguments));
end;

function TJScriptFunction.GetCaller: String;
begin
  Result:=GetPropAsString(FunctionProp_Caller);
end;

function TJScriptFunction.GetLength: Integer;
begin
  Result:=GetPropAsInteger(FunctionProp_Length);
end;

procedure TJScriptFunction.SetLength(Value: Integer);
begin
  SetProp(FunctionProp_Length,Value);
end;

function TJScriptFunction.Apply(ThisObj: IDispatch; const ArgArray: array of OleVariant): OleVariant;
begin
  Result:=Exec(FunctionMethod_Apply,Append(ThisObj,ArgArray));
end;

function TJScriptFunction.Call(ThisObj: IDispatch; const ArgArray: array of OleVariant): OleVariant;
begin
  Result:=Exec(FunctionMethod_Call,Append(ThisObj,ArgArray));
end;

//------------------------------------------------------------------------------
//TJScriptArray

constructor TJScriptArray.Create(Length: Integer);
begin
  Create(JScriptObjects.Script,Length);
end;

constructor TJScriptArray.Create(const Items: array of OleVariant);
begin
  Create(JScriptObjects.Script,Items);
end;

constructor TJScriptArray.Create(Script: TCustomScript; Length: Integer);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script,Script.NewJScriptArray(Length));
end;

constructor TJScriptArray.Create(Script: TCustomScript; const Items: array of OleVariant);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script,Script.NewJScriptArray(Items));
end;

destructor TJScriptArray.Destroy;
begin
  FConcat.Free;
  FReverse.Free;
  FSlice.Free;
  FSplice.Free;
  inherited;
end;

class function TJScriptArray.New(Script: TCustomScript): IDispatch;
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  Result:=Script.NewJScriptArray;
end;

class function TJScriptArray.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=GetJSObjectType(Disp)=otArray;
end;

procedure TJScriptArray.Sort(SortFunction: TJScriptFunction = nil);
begin
  if not assigned(SortFunction)
    then Exec(ArrayMethod_Sort)
    else Exec(ArrayMethod_Sort,[SortFunction.Disp]);
end;

procedure TJScriptArray.Unshift(const Items: array of OleVariant);
begin
  Exec(ArrayMethod_Unshift,Items);
end;

function TJScriptArray.Concat(const Items: array of OleVariant): TJScriptArray;
begin
  Result:=TJScriptArray(GetObjectWithExec(ArrayMethod_Concat,TJScriptArray,FConcat));
end;

function TJScriptArray.Count: Integer;
begin
  Result:=Length;
end;

function TJScriptArray.Join(Separator: String = ''): String;
begin
  if Separator=''
    then Result:=VarAsString(Exec(ArrayMethod_Join))
    else Result:=VarAsString(Exec(ArrayMethod_Join,[Separator]));
end;

function TJScriptArray.Pop: OleVariant;
begin
  Result:=Exec(ArrayMethod_Pop);
end;

function TJScriptArray.Push(const Item: OleVariant): Integer;
{$IFNDEF CAN_OVERLOAD}
var
  a: array of OleVariant;
{$ENDIF}
begin
  {$IFNDEF CAN_OVERLOAD}
  System.SetLength(a,1);
  a[0]:=Item;
  Result:=PushArray(a);                          //sonst ruft der Delphi 5 Compiler wieder Push(const Item: OleVariant) auf! :-(
  {$ELSE}
  Result:=Push([Item]);
  {$ENDIF}
end;

function TJScriptArray.{$IFDEF CAN_OVERLOAD}Push{$ELSE}PushArray{$ENDIF}
                       (const Items: array of OleVariant): Integer;
begin
  Result:=VarAsInteger(Exec(ArrayMethod_Push,Items));
end;

function TJScriptArray.Push(const Item: TScriptObject): Integer;
begin
  Result:=-1;
  if assigned(Item)
    then Result:=Push(Item.Disp);
end;

function TJScriptArray.Push(const Items: array of TScriptObject): Integer;
var
  n: Integer;
  VariantArray: array of OleVariant;
begin
  System.SetLength(VariantArray,System.Length(Items));
  for n:=0 to High(Items) do
    if assigned(Items[n])
      then VariantArray[n]:=Items[n].Disp
      else VariantArray[n]:=Null;
  Result:=Push(VariantArray);
  System.SetLength(VariantArray,0);
end;

function TJScriptArray.Reverse: TJScriptArray;
begin
  Result:=TJScriptArray(GetObjectWithExec(ArrayMethod_Reverse,TJScriptArray,FReverse));
end;

function TJScriptArray.Shift: OleVariant;
begin
  Result:=Exec(ArrayMethod_Shift);
end;

//orig.: slice(start,[end])
function TJScriptArray.Slice(StartIndex: Integer; EndIndex: Integer = 0): TJScriptArray;
var
  Slice: OleVariant;
begin
  Result:=nil;
  if EndIndex=0
    then Slice:=Exec(ArrayMethod_Slice,[StartIndex])
    else Slice:=Exec(ArrayMethod_Slice,[StartIndex,EndIndex]);
  if VarIsValidDispatch(Slice) then
  begin
    if assigned(FSlice)
      then FSlice.Disp:=Slice
      else FSlice:=TJScriptArray(CreateObjectWrapper(Script,TJScriptArray,Slice));
    Result:=FSlice;
  end;
end;

function TJScriptArray.Splice(Start, DeleteCount: Integer): TJScriptArray;
var
  Splice: OleVariant;
begin
  Result:=nil;
  Splice:=Exec(ArrayMethod_Splice,[Start,DeleteCount]);
  if VarIsValidDispatch(Splice) then
  begin
    if assigned(FSplice)
      then FSplice.Disp:=Splice
      else FSplice:=TJScriptArray(CreateObjectWrapper(Script,TJScriptArray,Splice));
    Result:=FSplice;
  end;
end;

function TJScriptArray.Splice(Start, DeleteCount: Integer; const Items: array of OleVariant): TJScriptArray;
var
  Splice: OleVariant;
  {$IFNDEF CAN_OVERLOAD}
  a: array[0..1] of OleVariant;
  {$ENDIF}
begin
  Result:=nil;
  {$IFNDEF CAN_OVERLOAD}
  a[0]:=Start;
  a[1]:=DeleteCount;
  Splice:=Exec(ArrayMethod_Splice,Append(a,Items));
  {$ELSE}
  Splice:=Exec(ArrayMethod_Splice,Append([Start,DeleteCount],Items));
  {$ENDIF}
  if VarIsValidDispatch(Splice) then
  begin
    if assigned(FSplice)
      then FSplice.Disp:=Splice
      else FSplice:=TJScriptArray(CreateObjectWrapper(Script,TJScriptArray,Splice));
    Result:=FSplice;
  end;
end;

function TJScriptArray.GetItem(Name: String): OleVariant;
begin
  Result:=GetPropAsVariant(Name);
end;

procedure TJScriptArray.SetItem(Name: String; const Value: OleVariant);
begin
  SetProp(Name,Value);
end;

function TJScriptArray.GetLength: Integer;
begin
  Result:=GetPropAsInteger(ArrayProp_Length);
end;

procedure TJScriptArray.SetLength(Value: Integer);
begin
  SetProp(ArrayProp_Length,Value);
end;

//------------------------------------------------------------------------------
//TObjectArray

destructor TObjectArray.Destroy;
begin
  {$IFNDEF DELPHI2009_UP}
  if assigned(FItems) then
    while FItems.Count>0 do
    begin
      FItems.Objects[0].Free;
      FItems.Delete(0);
    end;
  {$ENDIF}
  FreeAndNil(FItems);
  inherited;
end;

procedure TObjectArray.SetDisp(Value: IDispatch);
var
  OldDisp: IDispatch;
begin
  OldDisp:=Disp;
  inherited;
  if (OldDisp<>Value) and assigned(FItems)                 //JScript-Objekt wurde neu zugewiesen
    then FItems.Clear;                                     //-> schon erstellte Delphi-Wrapper-Objekte lschen
end;

function TObjectArray.GetItem(Index: Integer): TDispObject;
var
  IndexStr: String;
  Dispatch: OleVariant;
begin
  Result:=nil;
  if not assigned(FItems)
    then FItems:=TStringList.Create{$IFDEF DELPHI2009_UP}(true){$ENDIF};
  if (Index>=0) and (Index<Count) then
  begin
    IndexStr:=IntToStr(Index);
    if FItems.IndexOf(IndexStr)<0 then                     //noch kein zugehriges Delphi-Objekt angelegt
    begin
      Dispatch:=inherited Items[IndexStr];                 //JScript-Objekt ermitteln
      if VarIsValidDispatch(Dispatch)
        then FItems.AddObject(IndexStr,
                              CreateObject(Dispatch));     //Delphi-Wrapper anlegen
    end;
    if FItems.IndexOf(IndexStr)>=0
      then Result:=TDispObject(FItems.Objects[FItems.IndexOf(IndexStr)]);
  end;
end;

procedure TObjectArray.SetItem(Index: Integer; Value: TDispObject);
var
  IndexStr: String;
begin
  if (Index>=0) and (Index<Count) then
  begin
    IndexStr:=IntToStr(Index);
    if assigned(Value)
      then inherited Items[IndexStr]:=Value.Disp
      else inherited Items[IndexStr]:=Null;
  end;
end;

//------------------------------------------------------------------------------
//TJScriptArrayEx

procedure TJScriptArrayEx.Insert(Index: Integer; const Value: OleVariant);
begin
  Index:=Min(Count,Max(0,Index));
  Splice(Index,0,[Value]);
end;

procedure TJScriptArrayEx.Insert(Index: Integer; Value: TDispObject);
begin
  if assigned(Value)
    then Insert(Index,Value.Disp)
    else Insert(Index,Null);
end;

function TJScriptArrayEx.Add(const Value: OleVariant): Integer;
begin
  Result:=Count;
  Insert(Result,Value);
end;

function TJScriptArrayEx.Add(Value: TDispObject): Integer;
begin
  if assigned(Value)
    then Result:=Add(Value.Disp)
    else Result:=Add(Null);
end;

function TJScriptArrayEx.Count: Integer;
begin
  Result:=Length;
end;

//------------------------------------------------------------------------------
//TIntegerArray

function TIntegerArray.GetItem(Index: Integer): Integer;
begin
  if (Index>=0) and (Index<Count)
    then Result:=VarAsInteger(inherited Items[IntToStr(Index)])
    else raise Exception.Create(ExceptionIndexOutOfRange);
end;

procedure TIntegerArray.Insert(Index: Integer; Value: Integer);
begin
  inherited Insert(Index,Value);
end;

function TIntegerArray.Add(Value: Integer): Integer;
begin
  Result:=inherited Add(Value);
end;

function TIntegerArray.Add(Values: array of Integer): Integer;
var
  n: Integer;
begin
  Result:=-1;
  for n:=0 to High(Values) do
    Result:=Add(Values[n]);
end;

//------------------------------------------------------------------------------
//TDoubleArray

function TDoubleArray.Add(Value: Double): Integer;
begin
  Result:=inherited Add(Value);
end;

function TDoubleArray.Add(Values: array of Double): Integer;
var
  n: Integer;
begin
  Result:=-1;
  for n:=0 to High(Values) do
    Result:=Add(Values[n]);
end;

function TDoubleArray.GetItem(Index: Integer): Double;
begin
  if (Index>=0) and (Index<Count)
    then Result:=VarAsDouble(inherited Items[IntToStr(Index)])
    else raise Exception.Create(ExceptionIndexOutOfRange);
end;

procedure TDoubleArray.Insert(Index: Integer; Value: Double);
begin
  inherited Insert(Index,Value);
end;

//------------------------------------------------------------------------------
//TStringArray

function TStringArray.GetItem(Index: Integer): String;
//var
//  JScriptString: TJScriptString;
begin
  Result:='';
  if (Index>=0) and (Index<Count) then
  begin
    //Strings werden als OleVariant und NICHT als JavaScript-String-Objekt eingefgt!!!
    //JScriptString:=TJScriptString(CreateObjectWrapper(Script,TJScriptString,
    //                              inherited Items[IntToStr(Index)]));
    //try
    //  Result:=JScriptString.ToString;
    //finally
    //  JScriptString.Free;
    //end;
    Result:=VarToStr(inherited Items[IntToStr(Index)]);
  end;
end;

procedure TStringArray.Insert(Index: Integer; Value: String);
begin
  inherited Insert(Index,Value);                           //Strings werden als OleVariant und NICHT als JavaScript-String-Objekt eingefgt!!!
end;

function TStringArray.ToString: String;
var
  n: Integer;
begin
  Result:='';
  for n:=0 to Count-1 do
  begin
    if System.Length(Result)>0
      then Result:=Result+',';
    Result:=Result+Items[n];
  end;
end;

function TStringArray.Add(Value: String): Integer;
begin
  Result:=Count;
  Insert(Result,Value);
end;

function TStringArray.Add(Values: array of String): Integer;
var
  n: Integer;
begin
  Result:=Count;
  for n:=0 to System.Length(Values)-1 do
    Result:=Add(Values[n]);
end;

//------------------------------------------------------------------------------
//TJScriptRegExprArray

function TRegExprArray.GetInput: String;
begin
  Result:=GetPropAsString(RegExprArrayProp_Input);
end;

function TRegExprArray.GetIndex: Integer;
begin
  Result:=GetPropAsInteger(RegExprArrayProp_Index);
end;

function TRegExprArray.GetLastIndex: Integer;
begin
  Result:=GetPropAsInteger(RegExprArrayProp_LastIndex);
end;

//------------------------------------------------------------------------------
//TJScriptRegularExpression

constructor TJScriptRegularExpression.Create(const Pattern, Flags: String);
begin
  Create(JScriptObjects.Script,Pattern, Flags);
end;

constructor TJScriptRegularExpression.Create(Script: TCustomScript; const Pattern, Flags: String);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script,Script.NewJScriptRegExp(Pattern,Flags));
end;

destructor TJScriptRegularExpression.Destroy;
begin
  FExec.Free;
  inherited;
end;

class function TJScriptRegularExpression.New(Script: TCustomScript): IDispatch;
begin
  raise Exception.Create(ExceptionConstructorNeedsArguments);
end;

procedure TJScriptRegularExpression.Compile(Pattern: String; Flags: String = '');
begin
  if Flags=''
    then Exec(RegularExpressionMethod_Compile,[Pattern])
    else Exec(RegularExpressionMethod_Compile,[Pattern,Flags]);
end;

function TJScriptRegularExpression.Exec(Str: String): TRegExprArray;
var
  Exec: OleVariant;
begin
  Result:=nil;
  Exec:=Self.Exec(RegularExpressionMethod_Exec,[Str]);
  if VarIsValidDispatch(Exec) then
  begin
    if assigned(FExec)
      then FExec.Disp:=Exec
      else FExec:=TRegExprArray(CreateObjectWrapper(Script,TRegExprArray,Exec));
    Result:=FExec;
  end;
end;

function TJScriptRegularExpression.Exec(Str: TJScriptString): TRegExprArray;
var
  Exec: OleVariant;
begin
  Result:=nil;
  Exec:=Self.Exec(RegularExpressionMethod_Exec,Str);
  if VarIsValidDispatch(Exec) then
  begin
    if assigned(FExec)
      then FExec.Disp:=Exec
      else FExec:=TRegExprArray(CreateObjectWrapper(Script,TRegExprArray,Exec));
    Result:=FExec;
  end;
end;

function TJScriptRegularExpression.GetGlobal: Boolean;
begin
  Result:=GetPropAsBoolean(RegularExpressionProp_Global);
end;

function TJScriptRegularExpression.GetIgnoreCase: Boolean;
begin
  Result:=GetPropAsBoolean(RegularExpressionProp_IgnoreCase);
end;

function TJScriptRegularExpression.GetMultiline: Boolean;
begin
  Result:=GetPropAsBoolean(RegularExpressionProp_Multiline);
end;

function TJScriptRegularExpression.GetSource: String;
begin
  Result:=GetPropAsString(RegularExpressionProp_Source);
end;

function TJScriptRegularExpression.Test(Str: String): Boolean;
begin
  Result:=Exec(RegularExpressionMethod_Test,[Str]);
end;

//------------------------------------------------------------------------------
//TJScriptString

constructor TJScriptString.Create(const Value: String);
begin
  Create(JScriptObjects.Script,Value);
end;

constructor TJScriptString.Create(Script: TCustomScript; const Value: String);
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  inherited Create(Script,Script.NewJScriptString(Value));
end;

destructor TJScriptString.Destroy;
begin
  FMatch.Free;
  FSplit.Free;
  inherited;
end;

class function TJScriptString.New(Script: TCustomScript): IDispatch;
begin
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  Result:=Script.NewJScriptString;
end;

class function TJScriptString.IsObjectFromDisp(Disp: IDispatch): Boolean;
begin
  Result:=GetJSObjectType(Disp)=otString;
end;

function TJScriptString.CharAt(Index: Integer): Char;
var
  ResChar: OleVariant;
  {$IFNDEF CAN_OVERLOAD}
  w: Word;
  {$ENDIF}
begin
  Result:=#0;
  ResChar:=Exec(StringMethod_CharAt,[Index]);
  {$IFNDEF CAN_OVERLOAD}
  w:=VarType(ResChar);
  if (w=varOleStr) or (w=varString)
  {$ELSE}
  if VarIsStr(ResChar)
  {$ENDIF}
    and (System.Length(ResChar)>0)
      then Result:=String(ResChar)[1];
end;

function TJScriptString.CharCodeAt(Index: Integer): Integer;
begin
  Result:=Exec(StringMethod_CharCodeAt,[Index]);
end;

function TJScriptString.Concat(String2: String): String;
{$IFNDEF CAN_OVERLOAD}
var
  a: array[0..0] of String;
{$ENDIF}
begin
  {$IFNDEF CAN_OVERLOAD}
  a[0]:=String2;
  Result:=Concat(a);
  {$ELSE}
  Result:=Concat([String2]);
  {$ENDIF}
end;

function TJScriptString.Concat(Strings: array of String): String;
var
  n: Integer;
  OleStrings: array of OleVariant;
begin
  Result:='';
  System.SetLength(OleStrings,System.Length(Strings));
  for n:=0 to System.Length(Strings)-1 do
    OleStrings[n]:=Strings[n];
  Result:=VarAsString(Exec(StringMethod_Concat,OleStrings));
end;

function TJScriptString.IndexOf(SubString: String; StartIndex: Integer = 0): Integer;
begin
  if StartIndex=0
    then Result:=Exec(StringMethod_IndexOf,[SubString])
    else Result:=Exec(StringMethod_IndexOf,[SubString,StartIndex]);
end;

function TJScriptString.LastIndexOf(SubString: String; StartIndex: Integer = -1): Integer;
begin
  if StartIndex=-1
    then Result:=Exec(StringMethod_LastIndexOf,[SubString])
    else Result:=Exec(StringMethod_LastIndexOf,[SubString,StartIndex]);
end;

function TJScriptString.Match(RgExp: String): TRegExprArray;
var
  Match: OleVariant;
begin
  Result:=nil;
  Match:=Exec(StringMethod_Match,[RgExp]);
  if VarIsValidDispatch(Match) then
  begin
    if assigned(FMatch)
      then FMatch.Disp:=Match
      else FMatch:=TRegExprArray(CreateObjectWrapper(Script,TRegExprArray,Match));
    Result:=FMatch;
  end;
end;

function TJScriptString.Match(RgExp: TJScriptRegularExpression): TRegExprArray;
var
  Match: OleVariant;
begin
  Result:=nil;
  Match:=Exec(StringMethod_Match,RgExp);
  if VarIsValidDispatch(Match) then
  begin
    if assigned(FMatch)
      then FMatch.Disp:=Match
      else FMatch:=TRegExprArray(CreateObjectWrapper(Script,TRegExprArray,Match));
    Result:=FMatch;
  end;
end;

function TJScriptString.Replace(RgExp: String; ReplaceText: String): String;
begin
  Result:=VarAsString(Exec(StringMethod_Replace,[RgExp,ReplaceText]));
end;

function TJScriptString.Replace(RgExp: TJScriptRegularExpression; ReplaceText: String): String;
begin
  Result:='';
  if assigned(RgExp)
    then Replace:=VarAsString(Exec(StringMethod_Replace,[RgExp.Disp,ReplaceText]));
end;

function TJScriptString.Replace(RgExp: String; ReplaceText: TJScriptFunction): String;
begin
  Result:='';
  if assigned(ReplaceText)
    then Replace:=VarAsString(Exec(StringMethod_Replace,[RgExp,ReplaceText.Disp]));
end;

function TJScriptString.Replace(RgExp: TJScriptRegularExpression; ReplaceText: TJScriptFunction): String;
begin
  Result:='';
  if assigned(RgExp) and assigned(ReplaceText)
    then Result:=VarAsString(Exec(StringMethod_Replace,[RgExp.Disp,ReplaceText.Disp]));
end;

function TJScriptString.Search(RgExp: String): Integer;
begin
  Result:=Exec(StringMethod_Search,[RgExp]);
end;

function TJScriptString.Search(RgExp: TJScriptRegularExpression): Integer;
begin
  Result:=-1;
  if assigned(RgExp)
    then Result:=Exec(StringMethod_Search,RgExp);
end;

function TJScriptString.Slice(StartIndex: Integer): String;
begin
  //http://msdn.microsoft.com/en-us/library/6w1bzf9f(v=VS.85).aspx
  //The slice method returns a String object containing the specified portion of stringObj.
  //Das stimmt NICHT: Zurckgegeben wird kein String-Objekt, sondern nur ein einfacher String!!!
  Result:=VarAsString(Exec(StringMethod_Slice,[StartIndex]));
end;

function TJScriptString.Slice(StartIndex: Integer; EndIndex: Integer): String;
begin
  //Result: siehe Bemerkung oben
  Result:=VarAsString(Exec(StringMethod_Slice,[StartIndex,EndIndex]));
end;

function TJScriptString.Split(Separator: String = ''; Limit: Integer = -1): TJScriptArray;
var
  Split: OleVariant;
begin
  Result:=nil;
  if Limit=-1
    then Split:=Exec(StringMethod_Split,[Separator])
    else Split:=Exec(StringMethod_Split,[Separator,Limit]);
  if VarIsValidDispatch(Split) then
  begin
    if assigned(FSplit)
      then FSplit.Disp:=Split
      else FSplit:=TJScriptArray(CreateObjectWrapper(Script,TJScriptArray,Split));
    Result:=FSplit;
  end;
end;

function TJScriptString.Split(Separator: TJScriptRegularExpression = nil; Limit: Integer = -1): TJScriptArray;
var
  Split: OleVariant;
begin
  Result:=nil;
  if not assigned(Separator)
    then Exit;
  if Limit=-1
    then Split:=Exec(StringMethod_Split,Separator)
    else Split:=Exec(StringMethod_Split,[Separator.Disp,Limit]);
  if VarIsValidDispatch(Split) then
  begin
    if assigned(FSplit)
      then FSplit.Disp:=Split
      else FSplit:=TJScriptArray(CreateObjectWrapper(Script,TJScriptArray,Split));
    Result:=FSplit;
  end;
end;

function TJScriptString.SubStr(Start: Integer; Length: Integer = -1): String;
begin
  if Length=-1
    then Result:=VarAsString(Exec(StringMethod_SubStr,[Start]))
    else Result:=VarAsString(Exec(StringMethod_SubStr,[Start,Length]));
end;

function TJScriptString.SubString(Start: Integer; Length: Integer = -1): String;
begin
  //http://msdn.microsoft.com/en-us/library/3cz15ahb(v=VS.85).aspx
  //Returns the substring at the specified location within a String object.
  //Das stimmt NICHT: Zurckgegeben wird kein String-Objekt, sondern nur ein einfacher String!!!
  if Length=-1
    then Result:=VarAsString(Exec(StringMethod_SubString,[Start]))
    else Result:=VarAsString(Exec(StringMethod_SubString,[Start,Length]));
end;

function TJScriptString.ToLowerCase: String;
begin
  Result:=VarAsString(Exec(StringMethod_ToLowerCase));
end;

function TJScriptString.ToUpperCase: String;
begin
  Result:=VarAsString(Exec(StringMethod_ToUpperCase));
end;

//------------------------------------------------------------------------------
//TCallbackObject

constructor TCallbackObject.Create(Script: TCustomScript);
begin
  inherited Create;
  if not assigned(Script)
    then raise Exception.Create(ExceptionNoScript);
  FScript:=Script;
  FJScriptFunction:=Script.NewFunction(Callback);
end;

destructor TCallbackObject.Destroy;
begin
  FJScriptFunction.Free;
  inherited;
end;

procedure TCallbackObject.AfterConstruction;
begin
  inherited;
  FScript.FCallbackManager.Add(Self);
end;

procedure TCallbackObject.BeforeDestruction;
begin
  FScript.FCallbackManager.Remove(Self);
  inherited;
end;

function TCallbackObject.GetArgument(Index: Integer): OleVariant;
begin
  Result:=FJScriptFunction.Arguments.Arguments[Index];
end;

function TCallbackObject.ArgumentCount: Integer;
begin
  Result:=FJScriptFunction.Arguments.Count;
end;

//------------------------------------------------------------------------------
//TCallbackManager

constructor TCallbackManager.Create;
begin
  inherited Create;
  FItems:=TObjectList.Create(true);
end;

destructor TCallbackManager.Destroy;
begin
  FItems.Free;
  inherited;
end;

procedure TCallbackManager.Add(CallbackObject: TCallbackObject);
begin
  if assigned(CallbackObject) and (FItems.IndexOf(CallbackObject)<0)
    then FItems.Add(CallbackObject);
  CleanUp;
end;

procedure TCallbackManager.Remove(CallbackObject: TCallbackObject);
begin
  FItems.Extract(CallbackObject);
end;

procedure TCallbackManager.CleanUp;
var
  n: Integer;
  CallbackObject: TCallbackObject;
begin
  n:=0;
  while n<FItems.Count do
  begin
    CallbackObject:=TCallbackObject(FItems[n]);
    if CallbackObject.FReady
      then FItems.Remove(CallbackObject)
      else Inc(n);
  end;
end;

//------------------------------------------------------------------------------
//TTimerEvent

constructor TTimerEvent.Create(Interval: Cardinal);
begin
  inherited Create;
  FEventTime:=GetTickCount+Interval;
end;

//------------------------------------------------------------------------------
//TProcTimerEvent

{$IFDEF USE_ANONYMOUS_METHODS}
constructor TProcTimerEvent.Create(Method: TProc; Interval: Cardinal);
begin
  inherited Create(Interval);
  FProc:=Method;
end;

procedure TProcTimerEvent.Execute;
begin
  if assigned(FProc)
    then FProc;
end;
{$ENDIF}

//------------------------------------------------------------------------------
//TExternalProcTimerEvent

constructor TExternalProcTimerEvent.Create(Method: TExternalProc; Interval: Cardinal);
begin
  inherited Create(Interval);
  FProc:=Method;
end;

procedure TExternalProcTimerEvent.Execute;
begin
  if assigned(FProc)
    then FProc;
end;

//------------------------------------------------------------------------------
//TTimerThread

constructor TTimerThread.Create;
begin
  inherited Create(false);
  FEvents:=TThreadList.Create;
end;

destructor TTimerThread.Destroy;
begin
  Clear;
  FEvents.Free;
  inherited;
end;

function TTimerThread.Add(TimerEvent: TTimerEvent): Integer;
begin
  Result:=-1;
  if assigned(TimerEvent) then
  begin
    FEvents.Add(TimerEvent);
    TimerEvent.FID:=FCurrentID;
    Result:=TimerEvent.FID;
    Inc(FCurrentID);
  end;
end;

procedure TTimerThread.Clear;
begin
  with FEvents.LockList do
  try
    while Count>0 do
    begin
      TTimerEvent(Items[0]).Free;
      Delete(0);
    end;
  finally
    FEvents.UnlockList;
  end;
end;

procedure TTimerThread.Delete(ID: Integer);
var
  n: Integer;
  TimerEvent: TTimerEvent;
begin
  with FEvents.LockList do
  try
    for n:=0 to Count-1 do
    begin
      TimerEvent:=TTimerEvent(Items[n]);
      if TimerEvent.FID=ID then
      begin
        TimerEvent.Free;
        Delete(n);
        Exit;
      end;
    end;
  finally
    FEvents.UnlockList;
  end;
end;

procedure TTimerThread.Execute;
var
  n: Integer;
  Event: TTimerEvent;
  Time: Cardinal;
begin
  while not Terminated do
  begin
    Time:=GetTickCount;
    with FEvents.LockList do
    try
      n:=0;
      while n<Count do
      begin
        Event:=TTimerEvent(Items[0]);
        if Time>=Event.FEventTime then
        begin
          Synchronize(Event.Execute);
          Event.Free;
          Delete(n);
        end else Inc(n);
      end;
    finally
      FEvents.UnlockList;
    end;
    Sleep(10);
  end;
end;

//------------------------------------------------------------------------------
//TCustomScript

constructor TCustomScript.Create(Browser: TOleControl);
var
  EmptyParam: OleVariant;
begin
  if not assigned(BrowserTools.Script(Browser)) and assigned(Browser)
    then ({$IFDEF DELPHI6_UP}
          IInterface
          {$ELSE}
          IUnknown
          {$ENDIF}(Browser.OleObject) as IWebBrowser2).Navigate(EmptyPage,
                                                                EmptyParam,
                                                                EmptyParam,
                                                                EmptyParam,
                                                                EmptyParam);
  inherited Create(BrowserTools.Script(Browser));          //Script-COM-Objekt der Browser-Komponente ermitteln
  FCallbackManager:=TCallbackManager.Create;
  FExternalMethods:=TExternalMethods.Create;
  FExternalMethods.AddRef;
  FDocHostUIHandler:=TDocHostUIHandler.Create;             //Handler zur Steuerung des Browser-Layouts
  FDocHostUIHandler.Browser:=Browser;
  FDocHostUIHandler.ExternalMethods:=FExternalMethods;
  FDocHostUIHandler.OnShowContextMenu:=ShowContextMenu;
  FDocHostUIHandler.OnTranslateAccelerator:=TranslateAccelerator;
  FDocHostUIHandler.OnTranslateURL:=TranslateURL;
  FTimerThread:=TTimerThread.Create;
  Scripts.Add(Self);                                       //in globaler Variable speichern
end;

destructor TCustomScript.Destroy;
begin
  if assigned(Scripts)
    then Scripts.Remove(Self);
  FTimerThread.Free;
  FCallbackManager.Free;
  FExternalMethods.Release;
  FDocHostUIHandler.Free;
  FDocument.Free;
  FNavigator.Free;
  FTypeOfFunc.Free;
  inherited;
end;

procedure TCustomScript.AddVar(const Name: String; const Value: OleVariant);
begin
  if Name<>'' then
  begin
    if not PropertyExists(Name)
      then AddProp(Name);
    SetProp(Name,Value);                                   //benannte Variable hinzufgen
  end;
end;

procedure TCustomScript.AddVar(const Name: String; Value: TDispObject);
begin
  if assigned(Value)
    then AddVar(Name,Value.Disp)
    else AddVar(Name,unassigned);
end;

procedure TCustomScript.DelVar(const Name: String);
begin
  //ToDo: Im Script-Objekt lassen sich Variablen zwar erstellen und (neu) zuweisen,
  //aber aus irgendwelchen Grnden nicht wieder lschen.
  //DispatchEx.DeleteMemberByName/DispID liefert "Nicht implementiert". ???
  //Eine Mglichkeit wre z.B. die Belegung der Variable ber eine spezielle JavaScript-Funktion:
  //function SetVar(Name,Val) {eval(var Name=Val)};
  if (Name<>'') and PropertyExists(Name)
    then DelProp(Name);                                    //benannte Variable lschen
end;

procedure TCustomScript.ClearTimeout(TimeoutID: Integer);
begin
  FTimerThread.Delete(TimeoutID);
end;

function TCustomScript.Eval(const Script: String): OleVariant;
begin
  try
    Result:=Exec(ScriptMethod_Eval,[Script]);
  except
    Result:=Null;
  end;
end;

function TCustomScript.GetVar(Name: String): OleVariant;
begin
  Result:=GetPropAsVariant(Name);
end;

//http://ntt.cc/2008/02/10/4-ways-to-dynamically-load-external-javascriptwith-source.html
function TCustomScript.LoadLib(Source: TStream): Boolean;
var
  Node: TScriptNode;
  Stream: TStringStream;
begin
  Result:=false;
  if assigned(Source) and (Source.Size>0) then
  begin
    Stream:=TStringStream.Create('');
    try
      Stream.CopyFrom(Source,0);
      Node:=Document.CreateScriptNode;
      try
        Node.Type_:='text/javascript';
        Node.Defer:=true;
        Node.Text:=Stream.DataString;
        Document.Head.AppendChild(Node);
        Eval(Node.Text);
      finally
        Node.Free;
      end;
    finally
      Stream.Free;
    end;
  end;
end;

function TCustomScript.LoadLib(URL: String; Timeout: Cardinal): Boolean;
var
  Node: TScriptNode;
  StartTime: Cardinal;
begin
  Result:=false;
  if URL<>'' then
  begin
    Node:=Document.CreateScriptNode;
    try
      Node.Type_:='text/javascript';
      Node.Src:=URL;
      Document.Head.AppendChild(Node);
      if Timeout>0 then
      begin
        StartTime:=GetTickCount;
        while (Node.ReadyState<>'complete') and (GetTickCount-StartTime<Timeout) do
          Application.ProcessMessages;                     //Sleep(50);
      end;
      Result:=Node.ReadyState='complete';
    finally
      Node.Free;
    end;
  end;
end;

function TCustomScript.LoadLib(Instance: THandle; ResName: String; ResType: PChar): Boolean;
var
  ResStream: TResourceStream;
begin
  ResStream:=TResourceStream.Create(Instance,ResName,ResType);
  try
    Result:=LoadLib(ResStream);
  finally
    ResStream.Free;
  end;
end;

function TCustomScript.LoadEmptyPage(Wait: Boolean = true): Boolean;
begin
  Result:=LoadPage(BrowserTools.EmptyPage,Wait);
end;

function TCustomScript.LoadPage(Source: String; Wait: Boolean): Boolean;
var
  Stream: TStringStream;
begin
  Result:=false;
  if Length(Source)>0 then
  begin
    Stream:=TStringStream.Create(Source);
    try
      Result:=LoadPage(Stream,Wait);
    finally
      Stream.Free;
    end;
  end;
end;

function TCustomScript.LoadPage(Source: TStream; Wait: Boolean): Boolean;
begin
  Result:=false;
  if assigned(Source)
    then Result:=LoadBrowserFromStream(DocHostUIHandler.Browser,Source,Wait)
end;

function TCustomScript.SetTimeout(Method: TExternalProc; Interval: Integer): Integer;
begin
  Result:=-1;
  if assigned(Method)
    then Result:=FTimerThread.Add(TExternalProcTimerEvent.Create(Method,Interval));
end;

{$IFDEF USE_ANONYMOUS_METHODS}
function TCustomScript.SetTimeout(Method: TProc; Interval: Integer): Integer;
begin
  Result:=-1;
  if assigned(Method)
    then Result:=FTimerThread.Add(TProcTimerEvent.Create(Method,Interval));
end;
{$ENDIF}

procedure TCustomScript.SetVar(Name: String; const Value: OleVariant);
begin
  AddVar(Name,Value);
end;

function TCustomScript.GetDocument: TDocument;
begin
  Result:=TDocument(GetPropAsObject(ScriptProp_Document,TDocument,FDocument));
end;

function TCustomScript.GetNavigator: TNavigator;
begin
  Result:=TNavigator(GetPropAsObject(ScriptProp_Navigator,TNavigator,FNavigator));
end;

//Reaktion auf die Freigabe von abhngigen Objekten
procedure TCustomScript.FreeNotify(Sender: TObject);
begin
  //registrierte externe Methoden aus dem Pool entfernen:
  if assigned(FExternalMethods) and assigned(Sender) and (Sender is TJScriptFunction)
    then FExternalMethods.UnregisterMethod(TJScriptFunction(Sender).Disp);
end;

function TCustomScript.NewJScriptArray(Length: Integer = 0): IDispatch;
begin
  if Length=0
    then Result:=NewJScriptArray([])
    else Result:=NewJScriptArray([Length]);
end;

function TCustomScript.NewJScriptArray(const Items: array of OleVariant): IDispatch;
begin
  if Length(Items)=0
    then Result:=BrowserTools.Create(Disp,ScriptMethod_Array,true)
    else Result:=BrowserTools.{$IFDEF CAN_OVERLOAD}Create{$ELSE}Create_{$ENDIF}
                              (Disp,ScriptMethod_Array,Items,true);
end;

procedure TCustomScript.MakeFunctionNameUnique(var FunctionName: String);
var
  n: Integer;
begin
  if assigned(FExternalMethods) then
  begin
    //Funktionsname eindeutig machen:
    if FExternalMethods.IndexOf(FunctionName)>=0 then
    begin
      n:=1;
      while FExternalMethods.IndexOf(FunctionName+IntToStr(n))>=0 do
        Inc(n);
      FunctionName:=FunctionName+IntToStr(n);
    end;
  end;
end;

function TCustomScript.NewJScriptFunction(Method: TExternalProc;
                                          ExternalName: String): IDispatch;
begin
  if not assigned(Method)
    then raise Exception.Create(ExceptionNoFunction);
  if ExternalName=''
    then ExternalName:='func';
  MakeFunctionNameUnique(ExternalName);
  //"external"-Funktionsaufruf erstellen und registrieren:
  Result:=NewJScriptFunction([],'external.'+ExternalName+'()');
  if assigned(Result) and assigned(FExternalMethods)
    then FExternalMethods.RegisterMethod(Result,Method,ExternalName);
end;

function TCustomScript.NewJScriptFunction(Method: TExternalFunc;
                                          ExternalName: String): IDispatch;
begin
  if not assigned(Method)
    then raise Exception.Create(ExceptionNoFunction);
  if ExternalName=''
    then ExternalName:='func';
  MakeFunctionNameUnique(ExternalName);
  //"external"-Funktionsaufruf erstellen und registrieren,
  //wobei die Anweisung "return" ganz wichtig ist - sonst knnen keine Werte
  //zurckgegeben werden:
  Result:=NewJScriptFunction([],'return external.'+ExternalName+'()');
  if assigned(Result) and assigned(FExternalMethods)
    then FExternalMethods.RegisterMethod(Result,Method,ExternalName);
end;

function TCustomScript.NewJScriptFunction(const Arguments: array of String;
                                          Body: String): IDispatch;
begin
  Result:=BrowserTools.{$IFDEF CAN_OVERLOAD}Create{$ELSE}Create_{$ENDIF}
                       (Disp,ScriptMethod_Function,Append(Arguments,Body),true);
end;

function TCustomScript.NewJScriptObject: IDispatch;
begin
  Result:=BrowserTools.Create(Disp,ScriptMethod_Object,true);
end;

function TCustomScript.NewJScriptRegExp(const Pattern: String;
                                        const Flags: String): IDispatch;
begin
  if Flags=''
    then Result:=BrowserTools.Create(Disp,ScriptMethod_RegExp,Pattern,true)
    else Result:=BrowserTools.Create(Disp,ScriptMethod_RegExp,
                 {$IFDEF CAN_OVERLOAD}[Pattern,Flags]{$ELSE}Pattern,Flags{$ENDIF},true);
end;

function TCustomScript.NewJScriptString(const Value: String): IDispatch;
begin
  if Length(Value)=0
    then Result:=BrowserTools.Create(Disp,ScriptMethod_String,true)
    else Result:=BrowserTools.Create(Disp,ScriptMethod_String,Value,true);
end;

function TCustomScript.NewArray(Length: Integer): TJScriptArray;
begin
  Result:=TJScriptArray.Create(Self,Length);
end;

function TCustomScript.NewArray(const Arguments: array of OleVariant): TJScriptArray;
begin
  Result:=TJScriptArray.Create(Self,Arguments);
end;

function TCustomScript.NewRegExp(const Pattern: String;
                                 const Flags: String): TJScriptRegularExpression;
begin
  Result:=TJScriptRegularExpression.Create(Self,Pattern,Flags);
end;

function TCustomScript.NewFunction(Method: TExternalProc;
                                   const ExternalName: String): TJScriptFunction;
begin
  Result:=TJScriptFunction.Create(Self,Method,ExternalName);
end;

function TCustomScript.NewFunction(Method: TExternalFunc;
                                   const ExternalName: String): TJScriptFunction;
begin
  Result:=TJScriptFunction.Create(Self,Method,ExternalName);
end;

function TCustomScript.NewFunction(const Arguments: array of String;
                                   const Body: String): TJScriptFunction;
begin
  Result:=TJScriptFunction.Create(Self,Arguments,Body);
end;

{$IFDEF USE_ANONYMOUS_METHODS}
function TCustomScript.NewFunction(Method: TProc;
                                   ExternalName: String): TJScriptFunction;
begin
  Result:=TJScriptFunction.Create(Self,Method,ExternalName);
end;

function TCustomScript.NewFunction(Method: TFunc<Variant>;
                                   ExternalName: String): TJScriptFunction;
begin
  Result:=TJScriptFunction.Create(Self,Method,ExternalName);
end;

function TCustomScript.NewFunction(Method: TProcEx;
                                   ExternalName: String): TJScriptFunction;
begin
  Result:=TJScriptFunction.Create(Self,Method,ExternalName);
end;

function TCustomScript.NewFunction(Method: TFuncEx;
                                   ExternalName: String): TJScriptFunction;
begin
  Result:=TJScriptFunction.Create(Self,Method,ExternalName);
end;
{$ENDIF}

function TCustomScript.NewObject: TCustomJScriptObject;
begin
  Result:=TCustomJScriptObject.Create(Self,NewJScriptObject);
end;

function TCustomScript.NewString(const Value: String): TJScriptString;
begin
  Result:=TJScriptString.Create(Self,Value);
end;

procedure TCustomScript.RegisterObject(ScriptObject: TScriptObject);
begin
end;

const
  ScriptMethod_TypeOfEx = 'TypeOf';

function TCustomScript.TypeOf(const AObject: OleVariant): String;
begin
  //Result:=Exec(ScriptMethod_TypeOf,AObject);
  //Funktioniert leider nicht, da das Script-Objekt keinen direkten Zugriff auf
  //die JavaScript-Funktion "typeof" besitzt. Alternativ knnte "eval" benutzt
  //werden - allerdings besteht hier das Problem, Zeiger zu bergeben, da "eval"
  //nur einen einzigen String als Parameter akzeptiert.
  //Was ebenfalls nicht funktioniert, ist die Ermittlung des Methodenzeigers von
  //"typeof" ber typeof.valueOf(), was bei anderen Skript-Funktionen mglich ist.
  //siehe: http://msdn.microsoft.com/en-us/library/ftx8swz5(v=VS.85).aspx
  //JavaScript-Beispiel:
  //  var Test=eval.valueOf();
  //  alert(Test("2+3"));
  //  -> Anzeige: "5"
  //Die Methode "typeof" scheint eine besondere Stellung innerhalb der Skript-Engine
  //einzunehmen - oder ist einfach nur schlecht implementiert... ;-)
  //Deshalb wird hier eine Hilfsfunktion benutzt:
  if not PropertyExists(ScriptMethod_TypeOfEx) then
  begin
    if not assigned(FTypeOfFunc) then
    begin
      FTypeOfFunc:=NewFunction(['val'],'return typeof(val)');
      AddVar(ScriptMethod_TypeOfEx,FTypeOfFunc);           //Hilfsfunktion dem Script-Objekt hinzufgen
    end;
  end;
  Result:=FTypeOfFunc.Call(Disp,[AObject]);
end;

function TCustomScript.ShowContextMenu(Sender: TObject; Point: TPoint): Boolean;
begin
  if assigned(FOnShowContextMenu)
    then FOnShowContextMenu(Self,Point);
  Result:=true;
end;

function TCustomScript.TranslateAccelerator(Sender: TObject; Accelerator: Integer): Boolean;
begin
  case Accelerator of
    VK_F5:
    begin
      if assigned(FOnReloadAccelerator)
        then FOnReloadAccelerator(Self);
      Result:=true;
    end
    else Result:=false;
  end;
end;

function TCustomScript.TranslateURL(Sender: TObject; var URL: String): Boolean;
begin
  if assigned(FOnTranslateURL)
    then Result:=FOnTranslateURL(Sender,URL)
    else Result:=true;
end;

function TCustomScript.TypeOf(AObject: TDispObject): String;
begin
  Result:='unknown';
  if assigned(AObject)
    then Result:=TypeOf(AObject.Disp);
end;

initialization
  Scripts:=TCustomScriptCollection.Create;
finalization
  FreeAndNil(Scripts);

end.


